@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Hartmut Kaiser
+# Copyright (c) 2020-2023 Hartmut Kaiser
 #
 # SPDX-License-Identifier: BSL-1.0
 # Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -238,6 +238,7 @@ with section("parse"):
                                        'HEADER_ROOT': 1,
                                        'INI': 1,
                                        'INSTALL_SUFFIX': 1,
+                                       'INSTALL_COMPONENT': 1,
                                        'LANGUAGE': 1,
                                        'LINK_FLAGS': '+',
                                        'OUTPUT_SUFFIX': 1,
@@ -252,7 +253,8 @@ with section("parse"):
                                                  'STATIC',
                                                  'PLUGIN',
                                                  'PREPEND_SOURCE_ROOT',
-                                                 'PREPEND_HEADER_ROOT'],
+                                                 'PREPEND_HEADER_ROOT',
+                                                 'UNITY_BUILD'],
                                       'nargs': '1+'}},
     'add_hpx_config_test': { 'kwargs': { 'ARGS': '+',
                                          'CMAKECXXFEATURE': 1,
@@ -324,13 +326,17 @@ with section("parse"):
                                      'SOURCE_GLOB': 1,
                                      'SOURCE_ROOT': 1},
                          'pargs': { 'flags': ['EXCLUDE_FROM_ALL',
+                                               'INSTALL_HEADERS',
                                                'INTERNAL_FLAGS',
                                                'NOLIBS',
                                                'NOEXPORT',
                                                'AUTOGLOB',
                                                'STATIC',
                                                'PLUGIN',
-                                               'NONAMEPREFIX'],
+                                               'NONAMEPREFIX',
+                                               'PREPEND_SOURCE_ROOT',
+                                               'PREPEND_HEADER_ROOT',
+                                               'UNITY_BUILD'],
                                     'nargs': '1+'}},
     'add_hpx_library_headers': { 'kwargs': {'EXCLUDE': '+', 'GLOBS': '+'},
                                  'pargs': {'flags': ['APPEND'], 'nargs': '2+'}},
@@ -346,16 +352,18 @@ with section("parse"):
                                                    'nargs': '1+'}},
     'add_hpx_module': { 'kwargs': { 'CMAKE_SUBDIRS': '+',
                                     'COMPAT_HEADERS': '+',
+                                    'GENERATED_HEADERS': '+',
                                     'DEPENDENCIES': '+',
                                     'EXCLUDE_FROM_GLOBAL_HEADER': '+',
+                                    'ADD_TO_GLOBAL_HEADER': '+',
                                     'GLOBAL_HEADER_GEN': 1,
                                     'HEADERS': '+',
                                     'OBJECTS': '+',
                                     'MODULE_DEPENDENCIES': '+',
                                     'SOURCES': '+'},
-                        'pargs': { 'flags': ['FORCE_LINKING_GEN',
-                                              'CUDA',
-                                              'CONFIG_FILES'],
+                        'pargs': { 'flags': ['CUDA',
+                                             'CONFIG_FILES',
+                                             'NO_CONFIG_IN_GENERATED_HEADERS'],
                                    'nargs': '1+'}},
     'add_hpx_performance_test': {'pargs': {'nargs': 2}},
     'add_hpx_pseudo_dependencies': {'pargs': {'nargs': 0}},
@@ -369,7 +377,27 @@ with section("parse"):
                                           'SOURCE_ROOT': 1},
                               'pargs': { 'flags': ['FAILURE_EXPECTED', 'NOLIBS'],
                                          'nargs': '2+'}},
-    'add_hpx_regression_test': {'pargs': {'nargs': 2}},
+    'add_hpx_regression_test': { 'kwargs': { 'COMPONENT_DEPENDENCIES': '+',
+                                             'DEPENDENCIES': '+',
+                                             'EXECUTABLE': 1,
+                                             'FOLDER': 1,
+                                             'SOURCES': '+',
+                                             'SOURCE_ROOT': 1,
+                                             'ARGS': '+',
+                                             'EXECUTABLE': 1,
+                                             'PSEUDO_DEPS_NAME': 1,
+                                             'LOCALITIES': 1,
+                                             'PARCELPORTS': '+',
+                                             'THREADS_PER_LOCALITY': 1,
+                                             'TIMEOUT': 1},
+                              'pargs': { 'flags': ['FAILURE_EXPECTED',
+                                                   'NOLIBS',
+                                                   'RUN_SERIAL',
+                                                   'NO_PARCELPORT_TCP',
+                                                   'NO_PARCELPORT_LCI',
+                                                   'NO_PARCELPORT_MPI',
+                                                   'NO_PARCELPORT_GASNET'],
+                                         'nargs': '2+'}},
     'add_hpx_source_group': { 'kwargs': { 'CLASS': 1,
                                           'NAME': 1,
                                           'ROOT': 1,
@@ -380,7 +408,12 @@ with section("parse"):
                                   'LOCALITIES': 1,
                                   'PARCELPORTS': '+',
                                   'THREADS_PER_LOCALITY': 1},
-                      'pargs': {'flags': ['FAILURE_EXPECTED'], 'nargs': '2+'}},
+                      'pargs': {'flags': ['FAILURE_EXPECTED',
+                                          'RUN_SERIAL',
+                                          'NO_PARCELPORT_TCP',
+                                          'NO_PARCELPORT_LCI',
+                                          'NO_PARCELPORT_MPI'],
+                                'nargs': '2+'}},
     'add_hpx_test_target_dependencies': { 'kwargs': {'PSEUDO_DEPS_NAME': 1},
                                           'pargs': {'flags': [], 'nargs': '2+'}},
     'add_hpx_unit_compile_test': {
@@ -395,8 +428,19 @@ with section("parse"):
                                           'DEPENDENCIES': '+',
                                           'FOLDER': 1,
                                           'SOURCES': '+',
-                                          'SOURCE_ROOT': 1},
-                           'pargs': { 'flags': ['FAILURE_EXPECTED', 'NOLIBS'],
+                                          'SOURCE_ROOT': 1,
+                                          'ARGS': '+',
+                                          'EXECUTABLE': 1,
+                                          'PSEUDO_DEPS_NAME': 1,
+                                          'LOCALITIES': 1,
+                                          'PARCELPORTS': '+',
+                                          'THREADS_PER_LOCALITY': 1},
+                           'pargs': { 'flags': ['FAILURE_EXPECTED',
+                                                'NOLIBS',
+                                                'RUN_SERIAL',
+                                                'NO_PARCELPORT_TCP',
+                                                'NO_PARCELPORT_LCI',
+                                                'NO_PARCELPORT_MPI'],
                                       'nargs': '2+'}},
     'add_parcelport': { 'kwargs': { 'COMPILE_FLAGS': '+',
                                     'DEPENDENCIES': '+',
@@ -454,19 +498,8 @@ with section("parse"):
     'hpx_check_for_cxx11_std_quick_exit': {'pargs': {'nargs': 0}},
     'hpx_check_for_cxx11_std_shared_ptr_lwg3018': {'pargs': {'nargs': 0}},
     'hpx_check_for_cxx17_aligned_new': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_deduction_guides': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_fallthrough_attribute': {'pargs': {'nargs': 0}},
     'hpx_check_for_cxx17_filesystem': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_fold_expressions': {'pargs': {'nargs': 0}},
     'hpx_check_for_cxx17_hardware_destructive_interference_size': { 'pargs': { 'nargs': 0}},
-    'hpx_check_for_cxx17_if_constexpr': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_inline_constexpr_variable': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_maybe_unused': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_nodiscard_attribute': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_noexcept_functions_as_nontype_template_arguments': { 'pargs': { 'nargs': 0}},
-    'hpx_check_for_cxx17_std_in_place_type_t': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_std_variant': {'pargs': {'nargs': 0}},
-    'hpx_check_for_cxx17_structured_bindings': {'pargs': {'nargs': 0}},
     'hpx_check_for_libfun_std_experimental_optional': {'pargs': {'nargs': 0}},
     'hpx_check_for_mm_prefetch': {'pargs': {'nargs': 0}},
     'hpx_check_for_stable_inplace_merge': {'pargs': {'nargs': 0}},
@@ -529,7 +562,7 @@ with section("parse"):
                                            'DOXYGEN_ARGS': '+'},
                                'pargs': {'flags': [], 'nargs': '1+'}},
     'hpx_warn': {'pargs': {'nargs': 0}},
-    'setup_mpi': {'pargs': {'nargs': 0}},
+    'hpx_setup_mpi': {'pargs': {'nargs': 0}},
     'shorten_hpx_pseudo_target': {'pargs': {'nargs': 2}},
     'write_config_defines_file': { 'kwargs': { 'FILENAME': 1,
                                                'NAMESPACE': 1,
