@@ -8,11 +8,14 @@
 
 #include <atomic>
 #include <cstdint>
+#include <cstdlib>
 
 template <typename T>
 void test_atomic()
 {
     std::atomic<T> a;
+    if (!a.is_lock_free())
+        std::exit(-1);
     a.store(T{});
     T i = a.load();
     (void)i;
