@@ -5,7 +5,24 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 // This tests whether C++20 coroutines are supported
+
+#if defined(__has_include)
+#if __has_include(<coroutine>)
 #include <coroutine>
+namespace hpx {
+    using std::coroutine_handle;
+    using std::suspend_always;
+    using std::suspend_never;
+}    // namespace hpx
+#else
+#include <experimental/coroutine>
+namespace hpx {
+    using std::experimental::coroutine_handle;
+    using std::experimental::suspend_always;
+    using std::experimental::suspend_never;
+}    // namespace hpx
+#endif
+#endif
 
 struct resumable
 {
@@ -19,17 +36,18 @@ struct resumable
             return {};
         }
 
-        std::suspend_never initial_suspend() const noexcept
+        hpx::suspend_never initial_suspend() const noexcept
         {
             return {};
         }
 
-        std::suspend_never final_suspend() const noexcept
+        hpx::suspend_never final_suspend() const noexcept
         {
             return {};
         }
 
         void return_value(int value) const noexcept {}
+        void unhandled_exception() const noexcept {}
     };
 
     bool await_ready() const noexcept
@@ -40,7 +58,9 @@ struct resumable
     {
         return 42;
     }
-    void await_suspend(std::coroutine_handle<promise_type>) const noexcept {}
+    void await_suspend(hpx::coroutine_handle<promise_type>) const noexcept
+    {
+    }
 };
 
 resumable test()
