@@ -1,5 +1,5 @@
 //  Copyright (c) 2014 Anuj R. Sharma
-//  Copyright (c) 2014-2017 Hartmut Kaiser
+//  Copyright (c) 2014-2025 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -12,21 +12,18 @@
 #include <hpx/assert.hpp>
 #include <hpx/async_base/launch_policy.hpp>
 #include <hpx/async_combinators/wait_all.hpp>
-#include <hpx/async_combinators/when_all.hpp>
-#include <hpx/components/client_base.hpp>
 #include <hpx/components/get_ptr.hpp>
 #include <hpx/distribution_policies/container_distribution_policy.hpp>
-#include <hpx/functional/bind_back.hpp>
+#include <hpx/distribution_policies/explicit_container_distribution_policy.hpp>
 #include <hpx/modules/async_distributed.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/modules/execution.hpp>
 #include <hpx/runtime_components/distributed_metadata_base.hpp>
 #include <hpx/runtime_components/new.hpp>
 #include <hpx/runtime_distributed/copy_component.hpp>
-#include <hpx/type_support/unused.hpp>
 
-#include <hpx/components/containers/partitioned_vector/partitioned_vector_decl.hpp>
 #include <hpx/components/containers/partitioned_vector/partitioned_vector_component_impl.hpp>
+#include <hpx/components/containers/partitioned_vector/partitioned_vector_decl.hpp>
 #include <hpx/components/containers/partitioned_vector/partitioned_vector_segmented_iterator.hpp>
 
 #include <algorithm>
@@ -41,29 +38,38 @@
 #include <vector>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx
-{
+namespace hpx {
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::size_t
-    partitioned_vector<T, Data>::get_partition_size() const
+    partitioned_vector<T, Data>::get_partition_size(std::size_t partnum) const
     {
-        std::size_t num_parts = partitions_.size();
-        return num_parts ? ((size_ + num_parts - 1) / num_parts) : 0;
+        std::size_t size = partitions_.size();
+        if (partnum == size)
+        {
+            return 0;
+        }
+        return size == 0 ? 0 : partitions_[partnum].size_;
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::size_t
-    partitioned_vector<T, Data>::get_global_index(
-        std::size_t segment, std::size_t part_size, size_type local_index) const
+    partitioned_vector<T, Data>::get_global_index_part(
+        std::size_t partnum, size_type local_index) const
     {
-        return segment * part_size + local_index;
+        std::size_t size = partitions_.size();
+        if (partnum == size)
+        {
+            return size_;
+        }
+        return size == 0 ? local_index :
+                           partitions_[partnum].first_ + local_index;
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
     partitioned_vector<T, Data>::get_data_helper(
-        id_type id, server::partitioned_vector_config_data data)
+        id_type id, server::partitioned_vector_config_data&& data)
     {
         partitions_.clear();
         partitions_.reserve(data.partitions_.size());
@@ -73,34 +79,26 @@ namespace hpx
             std::back_inserter(partitions_));
 
         std::uint32_t this_locality = get_locality_id();
-        std::vector<future<void>> ptrs;
-
-        typedef typename partitions_vector_type::const_iterator const_iterator;
-
-        std::size_t l = 0;
-        const_iterator end = partitions_.cend();
-        for (const_iterator it = partitions_.cbegin(); it != end; ++it, ++l)
+        for (auto& p : partitions_)
         {
-            if (it->locality_id_ == this_locality)
+            if (p.locality_id_ == this_locality && !p.local_data_)
             {
-                ptrs.push_back(
-                    get_ptr<partitioned_vector_partition_server>(it->partition_)
-                        .then(get_ptr_helper{l, partitions_}));
+                p.local_data_ =
+                    hpx::get_ptr<partitioned_vector_partition_server>(
+                        hpx::launch::sync, p.partition_);
             }
         }
-        wait_all(ptrs);
 
-        partition_size_ = get_partition_size();
-        this->base_type::reset(std::move(id));
+        this->base_type::reset(HPX_MOVE(id));
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT hpx::future<void>
-    partitioned_vector<T, Data>::connect_to_helper(id_type id)
+    partitioned_vector<T, Data>::connect_to_helper([[maybe_unused]] id_type id)
     {
 #if !defined(HPX_COMPUTE_DEVICE_CODE)
-        typedef typename components::server::distributed_metadata_base<
-            server::partitioned_vector_config_data>::get_action act;
+        using act = components::server::distributed_metadata_base<
+            server::partitioned_vector_config_data>::get_action;
 
         return async(act(), id).then(
             [HPX_CXX20_CAPTURE_THIS(=)](
@@ -109,16 +107,15 @@ namespace hpx
             });
 #else
         HPX_ASSERT(false);
-        HPX_UNUSED(id);
         return hpx::make_ready_future();
 #endif
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT hpx::future<void>
-    partitioned_vector<T, Data>::connect_to(std::string const& symbolic_name)
+    partitioned_vector<T, Data>::connect_to(std::string symbolic_name)
     {
-        this->base_type::connect_to(symbolic_name);
+        this->base_type::connect_to(HPX_MOVE(symbolic_name));
         return this->base_type::share().then(
             [HPX_CXX20_CAPTURE_THIS(=)](
                 shared_future<id_type>&& f) -> hpx::future<void> {
@@ -129,37 +126,39 @@ namespace hpx
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
     partitioned_vector<T, Data>::connect_to(
-        launch::sync_policy, std::string const& symbolic_name)
+        launch::sync_policy, std::string symbolic_name)
     {
-        connect_to(symbolic_name).get();
+        connect_to(HPX_MOVE(symbolic_name)).get();
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT hpx::future<void>
-    partitioned_vector<T, Data>::register_as(std::string const& symbolic_name)
+    partitioned_vector<T, Data>::register_as(std::string symbolic_name)
     {
         std::vector<server::partitioned_vector_config_data::partition_data>
             partitions;
         partitions.reserve(partitions_.size());
 
-        std::copy(
-            partitions_.begin(), partitions_.end(), std::back_inserter(partitions));
+        std::copy(partitions_.begin(), partitions_.end(),
+            std::back_inserter(partitions));
 
-        server::partitioned_vector_config_data data(size_, std::move(partitions));
+        server::partitioned_vector_config_data data(
+            size_, HPX_MOVE(partitions));
         this->base_type::reset(
             hpx::new_<components::server::distributed_metadata_base<
                 server::partitioned_vector_config_data>>(
-                hpx::find_here(), std::move(data)));
+                hpx::find_here(), HPX_MOVE(data)));
 
-        return this->base_type::register_as(symbolic_name);
+        return this->base_type::register_as(HPX_MOVE(symbolic_name));
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
     partitioned_vector<T, Data>::register_as(
-        launch::sync_policy, std::string const& symbolic_name)
+        launch::sync_policy, std::string symbolic_name)
     {
-        register_as(symbolic_name).get();
+        register_as(HPX_MOVE(symbolic_name)).get();
+        this->base_type::get();
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -167,11 +166,10 @@ namespace hpx
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
     partitioned_vector<T, Data>::partitioned_vector(future<id_type>&& f)
     {
-        f.share().then(
-            [HPX_CXX20_CAPTURE_THIS(=)](
-                shared_future<id_type>&& f) -> hpx::future<void> {
-                return connect_to_helper(f.get());
-            });
+        f.share().then([HPX_CXX20_CAPTURE_THIS(=)](
+                           shared_future<id_type>&& fut) -> hpx::future<void> {
+            return connect_to_helper(fut.get());
+        });
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -179,28 +177,52 @@ namespace hpx
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::size_t
     partitioned_vector<T, Data>::get_partition(size_type global_index) const
     {
+        if (global_index == 0)
+        {
+            return 0;
+        }
         if (global_index == size_)
+        {
             return partitions_.size();
+        }
 
-        std::size_t part_size = partition_size_;
-        if (part_size != 0)
-            return (part_size != size_) ? (global_index / part_size) : 0;
+        // find approximate partition
+        std::size_t num_partitions = partitions_.size();
+        std::size_t part = global_index / (size_ / num_partitions + 1);
 
-        return partitions_.size();
+        HPX_ASSERT(part < num_partitions);
+
+        // if approximation is too high, go backwards
+        if (global_index < partitions_[part].first_)
+        {
+            while (part != 0)
+            {
+                if (global_index >= partitions_[--part].first_)
+                {
+                    return part;
+                }
+            }
+            return 0;
+        }
+
+        // otherwise find partition that holds the global index by going forward
+        for (/**/; part != num_partitions; ++part)
+        {
+            auto const& partition = partitions_[part];
+            if (global_index < partition.first_ + partition.size_)
+            {
+                return part;
+            }
+        }
+        return num_partitions;
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::size_t
-    partitioned_vector<T, Data>::get_local_index(size_type global_index) const
+    partitioned_vector<T, Data>::get_local_index(
+        size_type partnum, size_type global_index) const
     {
-        if (global_index == size_ || partition_size_ == std::size_t(-1) ||
-            partition_size_ == 0)
-        {
-            return std::size_t(-1);
-        }
-
-        return (partition_size_ != size_) ? (global_index % partition_size_) :
-                                            global_index;
+        return global_index - partitions_[partnum].first_;
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
@@ -210,28 +232,30 @@ namespace hpx
             std::vector<size_type> indices) const
     {
         for (size_type& index : indices)
-            index = get_local_index(index);
+        {
+            index = get_local_index(get_partition(index), index);
+        }
         return indices;
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
         typename partitioned_vector<T, Data>::local_iterator
-        partitioned_vector<T, Data>::get_local_iterator(
-            size_type global_index) const
+        partitioned_vector<T, Data>::get_local_iterator(size_type global_index)
     {
-        HPX_ASSERT(global_index != std::size_t(-1));
+        HPX_ASSERT(global_index != static_cast<std::size_t>(-1));
 
         std::size_t part = get_partition(global_index);
         if (part == partitions_.size())
         {
             // return an iterator to the end of the last partition
             auto const& back = partitions_.back();
-            return local_iterator(back.partition_, back.size_, back.local_data_);
+            return local_iterator(
+                back.partition_, back.size_, back.local_data_);
         }
 
-        std::size_t local_index = get_local_index(global_index);
-        HPX_ASSERT(local_index != std::size_t(-1));
+        std::size_t local_index = get_local_index(part, global_index);
+        HPX_ASSERT(local_index != static_cast<std::size_t>(-1));
 
         return local_iterator(partitions_[part].partition_, local_index,
             partitions_[part].local_data_);
@@ -240,21 +264,22 @@ namespace hpx
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
         typename partitioned_vector<T, Data>::const_local_iterator
-        partitioned_vector<T, Data>::get_const_local_iterator(
+        partitioned_vector<T, Data>::get_local_iterator(
             size_type global_index) const
     {
-        HPX_ASSERT(global_index != std::size_t(-1));
+        HPX_ASSERT(global_index != static_cast<std::size_t>(-1));
 
         std::size_t part = get_partition(global_index);
         if (part == partitions_.size())
         {
             // return an iterator to the end of the last partition
             auto const& back = partitions_.back();
-            return local_iterator(back.partition_, back.size_, back.local_data_);
+            return local_iterator(
+                back.partition_, back.size_, back.local_data_);
         }
 
-        std::size_t local_index = get_local_index(global_index);
-        HPX_ASSERT(local_index != std::size_t(-1));
+        std::size_t local_index = get_local_index(part, global_index);
+        HPX_ASSERT(local_index != static_cast<std::size_t>(-1));
 
         return const_local_iterator(partitions_[part].partition_, local_index,
             partitions_[part].local_data_);
@@ -263,7 +288,8 @@ namespace hpx
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
         typename partitioned_vector<T, Data>::segment_iterator
-        partitioned_vector<T, Data>::get_segment_iterator(size_type global_index)
+        partitioned_vector<T, Data>::get_segment_iterator(
+            size_type global_index)
     {
         std::size_t part = get_partition(global_index);
         if (part == partitions_.size())
@@ -275,7 +301,7 @@ namespace hpx
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
         typename partitioned_vector<T, Data>::const_segment_iterator
-        partitioned_vector<T, Data>::get_const_segment_iterator(
+        partitioned_vector<T, Data>::get_segment_iterator(
             size_type global_index) const
     {
         std::size_t part = get_partition(global_index);
@@ -290,41 +316,29 @@ namespace hpx
     template <typename DistPolicy>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT hpx::future<
         std::vector<typename partitioned_vector<T, Data>::bulk_locality_result>>
-    partitioned_vector<T, Data>::create_helper1(
-        DistPolicy const& policy, std::size_t count, std::size_t size)
+    partitioned_vector<T, Data>::create_helper1(DistPolicy const& policy,
+        std::size_t count, std::vector<std::size_t> const& sizes)
     {
-        typedef typename partitioned_vector_partition_client::server_component_type
-            component_type;
+        using component_type =
+            typename partitioned_vector_partition_client::server_component_type;
 
-        return policy.template bulk_create<component_type>(count, size);
+        return policy.template bulk_create<true, component_type>(count, sizes);
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     template <typename DistPolicy>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT hpx::future<
         std::vector<typename partitioned_vector<T, Data>::bulk_locality_result>>
-    partitioned_vector<T, Data>::create_helper2(
-        DistPolicy const& policy, std::size_t count, std::size_t size, T const& val)
+    partitioned_vector<T, Data>::create_helper2(DistPolicy const& policy,
+        std::size_t count, std::vector<std::size_t> const& sizes, T const& val)
     {
-        typedef typename partitioned_vector_partition_client::server_component_type
-            component_type;
+        using component_type =
+            typename partitioned_vector_partition_client::server_component_type;
 
-        return policy.template bulk_create<component_type>(count, size, val);
+        return policy.template bulk_create<true, component_type>(
+            count, sizes, val);
     }
 
-    template <typename T, typename Data /*= std::vector<T> */>
-    struct partitioned_vector<T, Data>::get_ptr_helper
-    {
-        std::size_t loc;
-        partitions_vector_type& partitions;
-
-        void operator()(
-            future<std::shared_ptr<partitioned_vector_partition_server>>&& f) const
-        {
-            partitions[loc].local_data_ = f.get();
-        }
-    };
-
     template <typename T, typename Data /*= std::vector<T> */>
     template <typename DistPolicy, typename Create>
     void partitioned_vector<T, Data>::create(
@@ -332,15 +346,15 @@ namespace hpx
     {
         std::size_t num_parts =
             traits::num_container_partitions<DistPolicy>::call(policy);
-        std::size_t part_size = (size_ + num_parts - 1) / num_parts;
+        std::vector<std::size_t> part_sizes =
+            traits::container_partition_sizes<DistPolicy>::call(policy, size_);
 
         // create as many partitions as required
         hpx::future<std::vector<bulk_locality_result>> f =
-            creator(policy, num_parts, part_size);
+            HPX_FORWARD(Create, creator)(policy, num_parts, part_sizes);
 
         // now initialize our data structures
-        std::uint32_t this_locality = get_locality_id();
-        std::vector<future<void>> ptrs;
+        std::uint32_t const this_locality = get_locality_id();
 
         std::size_t num_part = 0;
         std::size_t allocated_size = 0;
@@ -357,14 +371,16 @@ namespace hpx
             std::uint32_t locality = get_locality_id_from_id(r.first);
             for (hpx::id_type const& id : r.second)
             {
-                std::size_t size = (std::min)(part_size, size_ - allocated_size);
-                partitions_[l] = partition_data(id, size, locality);
+                std::size_t size =
+                    (std::min)(part_sizes[l], size_ - allocated_size);
+                partitions_[l] =
+                    partition_data(id, allocated_size, size, locality);
 
                 if (locality == this_locality)
                 {
-                    ptrs.push_back(
-                        get_ptr<partitioned_vector_partition_server>(id).then(
-                            get_ptr_helper{l, partitions_}));
+                    partitions_[l].local_data_ =
+                        hpx::get_ptr<partitioned_vector_partition_server>(
+                            hpx::launch::sync, id);
                 }
                 ++l;
 
@@ -374,27 +390,20 @@ namespace hpx
                     HPX_ASSERT(allocated_size == size_);
 
                     // shrink last partition, if appropriate
-                    if (size != part_size)
+                    if (size != part_sizes.back())
                     {
-                        partitioned_vector_partition_client(
-                            partitions_[l - 1].partition_)
-                            .resize(size);
+                        partitioned_vector_partition_client partition(
+                            partitions_[l - 1].partition_, true);
+                        partition.resize(size);
                     }
                     break;
                 }
-                else
-                {
-                    HPX_ASSERT(size == part_size);
-                }
+
+                HPX_ASSERT(size == part_sizes[l - 1]);
                 HPX_ASSERT(l < num_parts);
             }
         }
         HPX_ASSERT(l == num_parts);
-
-        when_all(ptrs).get();
-
-        // cache our partition size
-        partition_size_ = get_partition_size();
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
@@ -410,64 +419,99 @@ namespace hpx
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
     partitioned_vector<T, Data>::create(T const& val, DistPolicy const& policy)
     {
-        create(policy, util::bind_back(
-            &partitioned_vector::create_helper2<DistPolicy>, std::ref(val)));
+        create(policy,
+            [&val](DistPolicy const& policy, std::size_t num_parts,
+                std::vector<std::size_t> const& part_sizes) {
+                return create_helper2(policy, num_parts, part_sizes, val);
+            });
     }
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
     partitioned_vector<T, Data>::copy_from(partitioned_vector const& rhs)
     {
-        typedef typename partitions_vector_type::const_iterator const_iterator;
-
         std::vector<future<id_type>> objs;
-        const_iterator end = rhs.partitions_.end();
-        for (const_iterator it = rhs.partitions_.begin(); it != end; ++it)
+        objs.reserve(rhs.partitions_.size());
+
+        auto end = rhs.partitions_.end();
+        for (auto it = rhs.partitions_.begin(); it != end; ++it)
         {
-            typedef
-                typename partitioned_vector_partition_client::server_component_type
-                    component_type;
-            objs.push_back(hpx::components::copy<component_type>(it->partition_));
+            using component_type =
+                typename partitioned_vector_partition_client::
+                    server_component_type;
+            objs.push_back(
+                hpx::components::copy<component_type>(it->partition_));
         }
-        wait_all(objs);
+        hpx::wait_all(objs);
 
         std::uint32_t this_locality = get_locality_id();
-        std::vector<future<void>> ptrs;
 
         partitions_vector_type partitions;
-        // Fixing the size of partitions to avoid race conditions between
-        // possible reallocations during push back and the continuation
-        // to set the local partition data
-        partitions.resize(rhs.partitions_.size());
+        partitions.reserve(rhs.partitions_.size());
+
         for (std::size_t i = 0; i != rhs.partitions_.size(); ++i)
         {
             std::uint32_t locality = rhs.partitions_[i].locality_id_;
 
-            partitions[i] =
-                partition_data(objs[i].get(), rhs.partitions_[i].size_, locality);
+            partitions.push_back(partition_data(objs[i].get(),
+                rhs.partitions_[i].first_, rhs.partitions_[i].size_, locality));
 
             if (locality == this_locality)
             {
-                ptrs.push_back(
-                    get_ptr<partitioned_vector_partition_server>(
-                        partitions[i].partition_)
-                        .then(get_ptr_helper{i, partitions}));
+                partitions.back().local_data_ =
+                    hpx::get_ptr<partitioned_vector_partition_server>(
+                        hpx::launch::sync, partitions.back().partition_);
             }
         }
 
-        when_all(ptrs).get();
-
         size_ = rhs.size_;
-        partition_size_ = rhs.partition_size_;
         std::swap(partitions_, partitions);
     }
 
+    template <typename T, typename Data /*= std::vector<T> */>
+    HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT void
+    partitioned_vector<T, Data>::copy_data_from(partitioned_vector const& rhs)
+    {
+        std::vector<std::size_t> const empty;
+        for (std::size_t i = 0; i != rhs.partitions_.size(); ++i)
+        {
+            auto values = rhs.get_values(hpx::launch::sync, i, empty);
+            set_values(hpx::launch::sync, i, empty, HPX_MOVE(values));
+        }
+    }
+
+    template <typename T, typename Data /*= std::vector<T> */>
+    HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::vector<std::size_t>
+    partitioned_vector<T, Data>::get_partition_sizes() const
+    {
+        std::vector<std::size_t> result;
+        result.reserve(partitions_.size());
+        for (auto const& part : partitions_)
+        {
+            result.push_back(part.size_);
+        }
+        return result;
+    }
+
+    template <typename T, typename Data /*= std::vector<T> */>
+    HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT std::vector<hpx::id_type>
+    partitioned_vector<T, Data>::get_partition_localities() const
+    {
+        std::vector<hpx::id_type> result;
+        result.reserve(partitions_.size());
+        for (auto const& part : partitions_)
+        {
+            result.push_back(
+                hpx::naming::get_locality_from_id(part.partition_));
+        }
+        return result;
+    }
+
     ///////////////////////////////////////////////////////////////////////////
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
     partitioned_vector<T, Data>::partitioned_vector()
       : size_(0)
-      , partition_size_(std::size_t(-1))
     {
     }
 
@@ -475,7 +519,6 @@ namespace hpx
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
     partitioned_vector<T, Data>::partitioned_vector(size_type size)
       : size_(size)
-      , partition_size_(std::size_t(-1))
     {
         if (size != 0)
             create(hpx::container_layout);
@@ -483,23 +526,49 @@ namespace hpx
 
     template <typename T, typename Data /*= std::vector<T> */>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
-    partitioned_vector<T, Data>::partitioned_vector(size_type size, T const& val)
+    partitioned_vector<T, Data>::partitioned_vector(
+        size_type size, T const& val)
       : size_(size)
-      , partition_size_(std::size_t(-1))
     {
         if (size != 0)
             create(val, hpx::container_layout);
     }
 
+    template <typename T, typename Data /*= std::vector<T> */>
+    HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
+    partitioned_vector<T, Data>::partitioned_vector(
+        typename Data::const_iterator begin, typename Data::const_iterator end)
+      : size_(std::distance(begin, end))
+    {
+        if (size_ != 0)
+        {
+            // create all partitions
+            create(hpx::container_layout);
+
+            // fill partitions with their part of the data
+            std::vector<std::size_t> const empty;
+            std::vector<hpx::future<void>> futures;
+            futures.reserve(partitions_.size());
+            for (std::size_t i = 0; i != partitions_.size(); ++i)
+            {
+                HPX_ASSERT(static_cast<std::size_t>(std::distance(
+                               begin, end)) >= partitions_[i].size_);
+                auto const end_part = std::next(begin, partitions_[i].size_);
+                futures.push_back(set_values(i, empty, Data(begin, end_part)));
+                begin = end_part;
+            }
+            HPX_ASSERT(begin == end);
+            hpx::wait_all(HPX_MOVE(futures));
+        }
+    }
+
     template <typename T, typename Data /*= std::vector<T> */>
     template <typename DistPolicy>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
     partitioned_vector<T, Data>::partitioned_vector(size_type size,
         DistPolicy const& policy,
-        typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value>::type*)
+        std::enable_if_t<traits::is_distribution_policy_v<DistPolicy>>*)
       : size_(size)
-      , partition_size_(std::size_t(-1))
     {
         if (size != 0)
             create(policy);
@@ -508,15 +577,53 @@ namespace hpx
     template <typename T, typename Data /*= std::vector<T> */>
     template <typename DistPolicy>
     HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
-    partitioned_vector<T, Data>::partitioned_vector(size_type size, T const& val,
-        DistPolicy const& policy,
-        typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value>::type* /*= nullptr*/)
+    partitioned_vector<T, Data>::partitioned_vector(DistPolicy const&,
+        std::enable_if_t<traits::is_distribution_policy_v<DistPolicy>>*)
+      : size_(0)
+    {
+    }
+
+    template <typename T, typename Data /*= std::vector<T> */>
+    template <typename DistPolicy>
+    HPX_PARTITIONED_VECTOR_SPECIALIZATION_EXPORT
+    partitioned_vector<T, Data>::partitioned_vector(size_type size,
+        T const& val, DistPolicy const& policy,
+        std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>>* /*= nullptr*/)
       : size_(size)
-      , partition_size_(std::size_t(-1))
     {
         if (size != 0)
             create(val, policy);
     }
-}
 
+    template <typename T, typename Data /*= std::vector<T> */>
+    template <typename DistPolicy>
+    partitioned_vector<T, Data>::partitioned_vector(
+        typename Data::const_iterator begin, typename Data::const_iterator end,
+        DistPolicy const& policy,
+        std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>>* /*= nullptr*/)
+      : size_(std::distance(begin, end))
+    {
+        if (size_ != 0)
+        {
+            // create all partitions
+            create(policy);
+
+            // fill partitions with their part of the data
+            std::vector<std::size_t> const empty;
+            std::vector<hpx::future<void>> futures;
+            futures.reserve(partitions_.size());
+            for (std::size_t i = 0; i != partitions_.size(); ++i)
+            {
+                HPX_ASSERT(static_cast<std::size_t>(std::distance(
+                               begin, end)) >= partitions_[i].size_);
+                auto const end_part = std::next(begin, partitions_[i].size_);
+                futures.push_back(set_values(i, empty, Data(begin, end_part)));
+                begin = end_part;
+            }
+            HPX_ASSERT(begin == end);
+            hpx::wait_all(HPX_MOVE(futures));
+        }
+    }
+}    // namespace hpx
