@@ -21,12 +21,12 @@
 
 namespace hpx { namespace components { namespace process { namespace posix {
 
-template <typename ... Ts>
-child execute(Ts && ... ts)
-{
-    return executor()(std::forward<Ts>(ts)...);
-}
+    template <typename... Ts>
+    child execute(Ts&&... ts)
+    {
+        return executor()(HPX_FORWARD(Ts, ts)...);
+    }
 
-}}}}
+}}}}    // namespace hpx::components::process::posix
 
 #endif
