@@ -23,7 +23,7 @@ namespace hpx { namespace components { namespace process { namespace windows
         HANDLE handles[2];
         if (!::CreatePipe(&handles[0], &handles[1], nullptr, 0))
         {
-            HPX_THROW_EXCEPTION(invalid_status,
+            HPX_THROW_EXCEPTION(hpx::error::invalid_status,
                 "posix::create_pipe", "CreatePipe() failed");
         }
         return make_pipe(handles[0], handles[1]);
@@ -34,7 +34,7 @@ namespace hpx { namespace components { namespace process { namespace windows
         HANDLE handles[2] = { nullptr, nullptr };
         if (!::CreatePipe(&handles[0], &handles[1], nullptr, 0))
         {
-            HPX_THROWS_IF(ec, invalid_status,
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
                 "posix::create_pipe", "CreatePipe() failed");
         }
         else
