@@ -1,4 +1,4 @@
-//  Copyright (c) 2014-2016 Hartmut Kaiser
+//  Copyright (c) 2014-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -24,6 +24,7 @@
 #include <cstddef>
 #include <cstdint>
 #include <iostream>
+#include <memory>
 #include <mutex>
 #include <stack>
 #include <string>
@@ -48,7 +49,7 @@ template <typename T>
 struct partition_allocator
 {
 private:
-    typedef hpx::lcos::local::spinlock mutex_type;
+    typedef hpx::spinlock mutex_type;
 
 public:
     explicit partition_allocator(std::size_t max_size = std::size_t(-1))
@@ -104,17 +105,17 @@ private:
 
     struct hold_reference
     {
-        hold_reference(buffer_type const& data)
+        explicit hold_reference(buffer_type const& data)
           : data_(data)
         {
         }
 
-        void operator()(double*) {}    // no deletion necessary
+        void operator()(double const*) const {}    // no deletion necessary
 
         buffer_type data_;
     };
 
-    static void deallocate(double* p)
+    static void deallocate(double* p) noexcept
     {
         alloc_.deallocate(p);
     }
@@ -286,7 +287,7 @@ struct partition_server : hpx::components::component_base<partition_server>
     // 'get_data_action' which represents the (possibly remote) member function
     // partition::get_data().
     HPX_DEFINE_COMPONENT_DIRECT_ACTION(
-        partition_server, get_data, get_data_action);
+        partition_server, get_data, get_data_action)
 
 private:
     partition_data data_;
@@ -298,12 +299,12 @@ private:
 // HPX_REGISTER_COMPONENT() exposes the component creation
 // through hpx::new_<>().
 typedef hpx::components::component<partition_server> partition_server_type;
-HPX_REGISTER_COMPONENT(partition_server_type, partition_server);
+HPX_REGISTER_COMPONENT(partition_server_type, partition_server)
 
 // HPX_REGISTER_ACTION() exposes the component member function for remote
 // invocation.
 typedef partition_server::get_data_action get_data_action;
-HPX_REGISTER_ACTION(get_data_action);
+HPX_REGISTER_ACTION(get_data_action)
 
 ///////////////////////////////////////////////////////////////////////////////
 // This is a client side helper class allowing to hide some of the tedious
@@ -374,7 +375,7 @@ struct stepper_server : hpx::components::component_base<stepper_server>
     space do_work(
         std::size_t local_np, std::size_t nx, std::size_t nt, std::uint64_t nd);
 
-    HPX_DEFINE_COMPONENT_ACTION(stepper_server, do_work, do_work_action);
+    HPX_DEFINE_COMPONENT_ACTION(stepper_server, do_work, do_work_action)
 
     // receive the left-most partition from the right
     void from_right(std::size_t t, partition p)
@@ -388,8 +389,8 @@ struct stepper_server : hpx::components::component_base<stepper_server>
         left_receive_buffer_.store_received(t, std::move(p));
     }
 
-    HPX_DEFINE_COMPONENT_ACTION(stepper_server, from_right, from_right_action);
-    HPX_DEFINE_COMPONENT_ACTION(stepper_server, from_left, from_left_action);
+    HPX_DEFINE_COMPONENT_ACTION(stepper_server, from_right, from_right_action)
+    HPX_DEFINE_COMPONENT_ACTION(stepper_server, from_left, from_left_action)
 
     // release dependencies
     void release_dependencies()
@@ -399,7 +400,7 @@ struct stepper_server : hpx::components::component_base<stepper_server>
     }
 
     HPX_DEFINE_COMPONENT_ACTION(
-        stepper_server, release_dependencies, release_dependencies_action);
+        stepper_server, release_dependencies, release_dependencies_action)
 
 protected:
     // Our operator
@@ -442,29 +443,29 @@ private:
 // HPX_REGISTER_COMPONENT() exposes the component creation
 // through hpx::new_<>().
 typedef hpx::components::component<stepper_server> stepper_server_type;
-HPX_REGISTER_COMPONENT(stepper_server_type, stepper_server);
+HPX_REGISTER_COMPONENT(stepper_server_type, stepper_server)
 
 // HPX_REGISTER_ACTION() exposes the component member function for remote
 // invocation.
 typedef stepper_server::from_right_action from_right_action;
-HPX_REGISTER_ACTION(from_right_action);
+HPX_REGISTER_ACTION(from_right_action)
 
 typedef stepper_server::from_left_action from_left_action;
-HPX_REGISTER_ACTION(from_left_action);
+HPX_REGISTER_ACTION(from_left_action)
 
 typedef stepper_server::do_work_action do_work_action;
-HPX_REGISTER_ACTION(do_work_action);
+HPX_REGISTER_ACTION(do_work_action)
 
 typedef stepper_server::release_dependencies_action release_dependencies_action;
-HPX_REGISTER_ACTION(release_dependencies_action);
+HPX_REGISTER_ACTION(release_dependencies_action)
 
 void stepper_server::send_left(std::size_t t, partition p) const
 {
-    hpx::apply(from_right_action(), left_.get(), t, std::move(p));
+    hpx::post(from_right_action(), left_.get(), t, std::move(p));
 }
 void stepper_server::send_right(std::size_t t, partition p) const
 {
-    hpx::apply(from_left_action(), right_.get(), t, std::move(p));
+    hpx::post(from_left_action(), right_.get(), t, std::move(p));
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -516,19 +517,18 @@ partition stepper_server::heat_part(
     hpx::shared_future<partition_data> middle_data =
         middle.get_data(partition_server::middle_partition);
 
-    hpx::future<partition_data> next_middle =
-        middle_data.then(hpx::unwrapping(
-            [middle](partition_data const& m) -> partition_data {
-                HPX_UNUSED(middle);
+    hpx::future<partition_data> next_middle = middle_data.then(
+        hpx::unwrapping([middle](partition_data const& m) -> partition_data {
+            HPX_UNUSED(middle);
 
-                // All local operations are performed once the middle data of
-                // the previous time step becomes available.
-                std::size_t size = m.size();
-                partition_data next(size);
-                for (std::size_t i = 1; i != size - 1; ++i)
-                    next[i] = heat(m[i - 1], m[i], m[i + 1]);
-                return next;
-            }));
+            // All local operations are performed once the middle data of
+            // the previous time step becomes available.
+            std::size_t size = m.size();
+            partition_data next(size);
+            for (std::size_t i = 1; i != size - 1; ++i)
+                next[i] = heat(m[i - 1], m[i], m[i + 1]);
+            return next;
+        }));
 
     return hpx::dataflow(hpx::launch::async,
         hpx::unwrapping(
@@ -576,7 +576,7 @@ stepper_server::space stepper_server::do_work(
     }
 
     // limit depth of dependency tree
-    hpx::lcos::local::sliding_semaphore sem(nd);
+    auto sem = std::make_shared<hpx::sliding_semaphore>(nd);
 
     for (std::size_t t = 0; t != nt; ++t)
     {
@@ -627,15 +627,15 @@ stepper_server::space stepper_server::do_work(
         // trigger the semaphore once computation has reached this point
         if ((t % nd) == 0)
         {
-            next[0].then([&sem, t](partition&&) {
+            next[0].then([sem, t](partition&&) {
                 // inform semaphore about new lower limit
-                sem.signal(t);
+                sem->signal(t);
             });
         }
 
         // suspend if the tree has become too deep, the continuation above
         // will resume this thread once the computation has caught up
-        sem.wait(t);
+        sem->wait(t);
     }
 
     return U_[nt % 2];
@@ -747,19 +747,17 @@ int main(int argc, char* argv[])
     desc_commandline.add_options()(
         "results", "print generated results (default: false)")("nx",
         value<std::uint64_t>()->default_value(10),
-        "Local x dimension (of each partition)") ("nt",
+        "Local x dimension (of each partition)")("nt",
         value<std::uint64_t>()->default_value(45),
-        "Number of time steps") ("nd",
+        "Number of time steps")("nd", value<std::uint64_t>()->default_value(10),
+        "Number of time steps to allow the dependency tree to grow to")("np",
         value<std::uint64_t>()->default_value(10),
-        "Number of time steps to allow the dependency tree to grow to") ("np",
-        value<std::uint64_t>()->default_value(10),
-        "Number of partitions") ("k", value<double>(&k)->default_value(0.5),
-        "Heat transfer coefficient (default: 0.5)") ("dt",
+        "Number of partitions")("k", value<double>(&k)->default_value(0.5),
+        "Heat transfer coefficient (default: 0.5)")("dt",
         value<double>(&dt)->default_value(1.0),
-        "Timestep unit (default: 1.0[s])") ("dx",
-        value<double>(&dx)->default_value(1.0),
-        "Local x dimension") ("no-header",
-        "do not print out the csv header row");
+        "Timestep unit (default: 1.0[s])")(
+        "dx", value<double>(&dx)->default_value(1.0), "Local x dimension")(
+        "no-header", "do not print out the csv header row");
 
     // Initialize and run HPX, this example requires to run hpx_main on all
     // localities
