@@ -14,26 +14,22 @@
 //[accumulator_registration_definitions
 ///////////////////////////////////////////////////////////////////////////////
 // Add factory registration functionality.
-HPX_REGISTER_COMPONENT_MODULE();
+HPX_REGISTER_COMPONENT_MODULE()
 
 ///////////////////////////////////////////////////////////////////////////////
-typedef hpx::components::component<
-    examples::server::accumulator
-> accumulator_type;
+typedef hpx::components::component<examples::server::accumulator>
+    accumulator_type;
 
-HPX_REGISTER_COMPONENT(accumulator_type, accumulator);
+HPX_REGISTER_COMPONENT(accumulator_type, accumulator)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Serialization support for accumulator actions.
 HPX_REGISTER_ACTION(
-    accumulator_type::wrapped_type::reset_action,
-    accumulator_reset_action);
+    accumulator_type::wrapped_type::reset_action, accumulator_reset_action)
 HPX_REGISTER_ACTION(
-    accumulator_type::wrapped_type::add_action,
-    accumulator_add_action);
+    accumulator_type::wrapped_type::add_action, accumulator_add_action)
 HPX_REGISTER_ACTION(
-    accumulator_type::wrapped_type::query_action,
-    accumulator_query_action);
+    accumulator_type::wrapped_type::query_action, accumulator_query_action)
 //]
 
 #endif
