@@ -11,5 +11,5 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 // Add factory registration functionality.
-HPX_REGISTER_COMPONENT_MODULE();
+HPX_REGISTER_COMPONENT_MODULE()
 #endif
