@@ -13,24 +13,24 @@
 
 #include "server/template_function_accumulator.hpp"
 
+//[template_func_accumulator_registration_definitions
 ///////////////////////////////////////////////////////////////////////////////
 // Add factory registration functionality.
-HPX_REGISTER_COMPONENT_MODULE();
+HPX_REGISTER_COMPONENT_MODULE()
 
 ///////////////////////////////////////////////////////////////////////////////
 typedef hpx::components::component<
-    examples::server::template_function_accumulator
-> accumulator_type;
+    examples::server::template_function_accumulator>
+    accumulator_type;
 
-HPX_REGISTER_COMPONENT(accumulator_type, template_function_accumulator);
+HPX_REGISTER_COMPONENT(accumulator_type, template_function_accumulator)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Serialization support for managed_accumulator actions.
-HPX_REGISTER_ACTION(
-    accumulator_type::wrapped_type::reset_action,
-    managed_accumulator_reset_action);
-HPX_REGISTER_ACTION(
-    accumulator_type::wrapped_type::query_action,
-    managed_accumulator_query_action);
+HPX_REGISTER_ACTION(accumulator_type::wrapped_type::reset_action,
+    managed_accumulator_reset_action)
+HPX_REGISTER_ACTION(accumulator_type::wrapped_type::query_action,
+    managed_accumulator_query_action)
+//]
 
 #endif
