@@ -11,47 +11,48 @@
 #if !defined(HPX_COMPUTE_DEVICE_CODE)
 #include <hpx/hpx.hpp>
 #include <hpx/include/actions.hpp>
-#include <hpx/include/lcos.hpp>
 #include <hpx/include/components.hpp>
+#include <hpx/include/lcos.hpp>
 #include <hpx/serialization.hpp>
 
 #include <utility>
 
-namespace examples { namespace server
-{
+namespace examples { namespace server {
     struct HPX_COMPONENT_EXPORT hello_world
-        : hpx::components::component_base<hello_world>
+      : hpx::components::component_base<hello_world>
     {
         void invoke();
-        HPX_DEFINE_COMPONENT_ACTION(hello_world, invoke);
+        HPX_DEFINE_COMPONENT_ACTION(hello_world, invoke)
     };
-}}
+}}    // namespace examples::server
 
 HPX_REGISTER_ACTION_DECLARATION(
-    examples::server::hello_world::invoke_action, hello_world_invoke_action);
+    examples::server::hello_world::invoke_action, hello_world_invoke_action)
 
-namespace examples
-{
+namespace examples {
     struct hello_world
       : hpx::components::client_base<hello_world, server::hello_world>
     {
         typedef hpx::components::client_base<hello_world, server::hello_world>
             base_type;
 
-        hello_world(hpx::future<hpx::naming::id_type> && f)
+        hello_world(hpx::future<hpx::id_type>&& f)
           : base_type(std::move(f))
-        {}
+        {
+        }
 
-        hello_world(hpx::naming::id_type && f)
+        hello_world(hpx::id_type&& f)
           : base_type(std::move(f))
-        {}
+        {
+        }
 
         void invoke()
         {
-            hpx::async<server::hello_world::invoke_action>(this->get_id()).get();
+            hpx::async<server::hello_world::invoke_action>(this->get_id())
+                .get();
         }
     };
-}
+}    // namespace examples
 
 #endif
 //]
