@@ -18,25 +18,26 @@
 #include "grid.hpp"
 #include "row.hpp"
 
-namespace jacobi
-{
+namespace jacobi {
     grid::grid(std::size_t nx, std::size_t ny, double value)
-        //: rows(ny)
+    //: rows(ny)
     {
         std::vector<hpx::id_type> ids = hpx::new_<server::row[]>(
-            hpx::default_layout(hpx::find_all_localities()), ny).get();
+            hpx::default_layout(hpx::find_all_localities()), ny)
+                                            .get();
 
         rows.reserve(ny);
-        std::vector<hpx::lcos::future<void> > init_futures;
+        std::vector<hpx::future<void>> init_futures;
         init_futures.reserve(ny);
-        for (hpx::naming::id_type const& id : ids)
+        for (hpx::id_type const& id : ids)
         {
-            row r; r.id = id;
+            row r;
+            r.id = id;
             init_futures.push_back(r.init(nx, value));
             rows.push_back(r);
         }
 
         hpx::wait_all(init_futures);
     }
-}
+}    // namespace jacobi
 #endif
