@@ -12,35 +12,34 @@
 #include "row_range.hpp"
 #include "server/row.hpp"
 
-#include <hpx/include/naming.hpp>
 #include <hpx/include/lcos.hpp>
+#include <hpx/include/naming.hpp>
 
 #include <cstddef>
 
-namespace jacobi
-{
+namespace jacobi {
     struct row
     {
         row() {}
 
-        hpx::lcos::future<void> init(std::size_t nx, double value = 0.0)
+        hpx::future<void> init(std::size_t nx, double value = 0.0)
         {
             return hpx::async<server::row::init_action>(id, nx, value);
         }
 
-        hpx::naming::id_type id;
+        hpx::id_type id;
 
-        hpx::lcos::future<row_range> get(std::size_t begin, std::size_t end)
+        hpx::future<row_range> get(std::size_t begin, std::size_t end)
         {
             return hpx::async<server::row::get_action>(id, begin, end);
         }
 
         template <typename Archive>
-        void serialize(Archive & ar, unsigned)
+        void serialize(Archive& ar, unsigned)
         {
-            ar & id;
+            ar& id;
         }
     };
-}
+}    // namespace jacobi
 
 #endif
