@@ -6,7 +6,7 @@
 
 #pragma once
 
-#include <hpx/include/util.hpp>
+#include <hpx/chrono.hpp>
 
 #include <cstddef>
 #include <cstdint>
@@ -18,10 +18,8 @@
 #include "jacobi.hpp"
 
 namespace jacobi_smp {
-    void jacobi(
-        std::size_t n
-      , std::size_t iterations, std::size_t block_size
-      , std::string const& output_filename)
+    void jacobi(std::size_t n, std::size_t iterations, std::size_t,
+        std::string const& output_filename)
     {
         typedef std::vector<double> vector;
 
@@ -29,24 +27,20 @@ namespace jacobi_smp {
         std::shared_ptr<vector> grid_old(new vector(n * n, 1));
 
         hpx::chrono::high_resolution_timer t;
-        for(std::size_t i = 0; i < iterations; ++i)
+        for (std::size_t i = 0; i < iterations; ++i)
         {
             // MSVC is unhappy if the OMP loop variable is unsigned
 #pragma omp parallel for schedule(JACOBI_SMP_OMP_SCHEDULE)
-            for(std::int64_t y = 1; y < std::int64_t(n-1); ++y)
+            for (std::int64_t y = 1; y < std::int64_t(n - 1); ++y)
             {
-                      double * dst = &(*grid_new)[y * n];
-                const double * src = &(*grid_new)[y * n];
-                jacobi_kernel(
-                    dst
-                  , src
-                  , n
-                );
+                double* dst = &(*grid_new)[y * n];
+                const double* src = &(*grid_new)[y * n];
+                jacobi_kernel(dst, src, n);
             }
             std::swap(grid_new, grid_old);
         }
 
         report_timing(n, iterations, t.elapsed());
         output_grid(output_filename, *grid_old, n);
-   }
-}
+    }
+}    // namespace jacobi_smp
