@@ -11,17 +11,16 @@
 
 #include <iostream>
 
-HPX_REGISTER_CHANNEL(int)
+HPX_REGISTER_CHANNEL(int, pipeline)
 
 void f1(hpx::lcos::channel<int>& c1)
 {
     for (std::size_t i = 0; true; ++i)
     {
-        std::cout << "First Stage: " << i
-            << ". Executed on locality " << hpx::get_locality_id()
-            << " " << hpx::get_locality_name()
-            << '\n';
-        c1.set(i);
+        std::cout << "First Stage: " << i << ". Executed on locality "
+                  << hpx::get_locality_id() << " " << hpx::get_locality_name()
+                  << '\n';
+        c1.set(int(i));
         hpx::this_thread::sleep_for(std::chrono::seconds(5));
     }
     c1.close();
