@@ -11,32 +11,28 @@
 
 #include <iostream>
 
-HPX_REGISTER_CHANNEL(int)
+HPX_REGISTER_CHANNEL(int, pipeline)
 
 hpx::future<void> f2(hpx::lcos::channel<int>& c1, hpx::lcos::channel<int>& c2)
 {
     hpx::future<int> f = c1.get();
-    return f.then(
-        [&c1, &c2](hpx::future<int> f)
+    return f.then([&c1, &c2](hpx::future<int> f) {
+        try
         {
-            try
-            {
-                int i = f.get();
-                std::cout << "Second stage: " << i
-                    << ". Executed on locality " << hpx::get_locality_id()
-                    << " " << hpx::get_locality_name()
-                    << '\n';
-                c2.set(i + 1);
-                hpx::this_thread::sleep_for(std::chrono::microseconds(10));
-                return f2(c1, c2);
-            }
-            catch(...)
-            {
-                c2.close();
-                return hpx::make_ready_future();
-            }
+            int i = f.get();
+            std::cout << "Second stage: " << i << ". Executed on locality "
+                      << hpx::get_locality_id() << " "
+                      << hpx::get_locality_name() << '\n';
+            c2.set(i + 1);
+            hpx::this_thread::sleep_for(std::chrono::microseconds(10));
+            return f2(c1, c2);
         }
-    );
+        catch (...)
+        {
+            c2.close();
+            return hpx::make_ready_future();
+        }
+    });
 }
 
 int main()
