@@ -7,10 +7,10 @@
 // This is a purely local version demonstrating different versions of making
 // the calculation of a fibonacci asynchronous.
 
+#include <hpx/chrono.hpp>
+#include <hpx/format.hpp>
+#include <hpx/future.hpp>
 #include <hpx/init.hpp>
-#include <hpx/local/chrono.hpp>
-#include <hpx/local/future.hpp>
-#include <hpx/modules/format.hpp>
 
 #include <cstddef>
 #include <cstdint>
@@ -238,7 +238,7 @@ int hpx_main(hpx::program_options::variables_map& vm)
         std::cerr << "fibonacci_futures: wrong command line argument value for "
                      "option 'n-runs', should not be zero"
                   << std::endl;
-        return hpx::finalize();    // Handles HPX shutdown
+        return hpx::local::finalize();    // Handles HPX shutdown
     }
 
     threshold = vm["threshold"].as<unsigned int>();
@@ -248,7 +248,7 @@ int hpx_main(hpx::program_options::variables_map& vm)
                      "option 'threshold', should be in between 2 and n-value"
                      ", value specified: "
                   << threshold << std::endl;
-        return hpx::finalize();    // Handles HPX shutdown
+        return hpx::local::finalize();    // Handles HPX shutdown
     }
 
     bool executed_one = false;
@@ -461,7 +461,7 @@ int hpx_main(hpx::program_options::variables_map& vm)
                   << test << std::endl;
     }
 
-    return hpx::finalize();    // Handles HPX shutdown
+    return hpx::local::finalize();    // Handles HPX shutdown
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -485,8 +485,8 @@ int main(int argc, char* argv[])
     // clang-format on
 
     // Initialize and run HPX
-    hpx::init_params init_args;
+    hpx::local::init_params init_args;
     init_args.desc_cmdline = desc_commandline;
 
-    return hpx::init(argc, argv, init_args);
+    return hpx::local::init(hpx_main, argc, argv, init_args);
 }
