@@ -8,18 +8,18 @@
 
 #include "dimension.hpp"
 
-namespace hpx { namespace serialization
-{
+namespace hpx { namespace serialization {
     ///////////////////////////////////////////////////////////////////////////
     // Implementation of the serialization function.
-    void serialize(input_archive& ar, sheneos::dimension& dim, unsigned int const)
+    void serialize(
+        input_archive& ar, sheneos::dimension& dim, unsigned int const)
     {
-        ar & dim.offset_ & dim.count_ & dim.size_;
+        ar& dim.offset_& dim.count_& dim.size_;
     }
 
-    void serialize(output_archive& ar, sheneos::dimension& dim, unsigned int const)
+    void serialize(
+        output_archive& ar, sheneos::dimension& dim, unsigned int const)
     {
-        ar & dim.offset_ & dim.count_ & dim.size_;
+        ar& dim.offset_& dim.count_& dim.size_;
     }
-}}
-
+}}    // namespace hpx::serialization
