@@ -23,21 +23,20 @@
 #include <utility>
 #include <vector>
 
-#include "read_values.hpp"
-#include "partition3d.hpp"
 #include "interpolator.hpp"
+#include "partition3d.hpp"
+#include "read_values.hpp"
 
 #include <H5public.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 // Add factory registration functionality.
-HPX_REGISTER_COMPONENT_MODULE(); // Create entry point for component factory.
+HPX_REGISTER_COMPONENT_MODULE()    // Create entry point for component factory.
 
 ///////////////////////////////////////////////////////////////////////////////
 // Register a shutdown function which will be called as a px-thread during
 // runtime shutdown.
-namespace sheneos
-{
+namespace sheneos {
     ///////////////////////////////////////////////////////////////////////////
     // This function will be registered as a shutdown function for HPX below.
     void shutdown()
@@ -48,23 +47,22 @@ namespace sheneos
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    bool get_shutdown(hpx::shutdown_function_type& shutdown_func,
-        bool& pre_shutdown)
+    bool get_shutdown(
+        hpx::shutdown_function_type& shutdown_func, bool& pre_shutdown)
     {
         shutdown_func = shutdown;
-        pre_shutdown = false;       // run this as late as possible
+        pre_shutdown = false;    // run this as late as possible
         return true;
     }
-}
+}    // namespace sheneos
 HPX_REGISTER_SHUTDOWN_MODULE(&::sheneos::get_shutdown);
 
 ///////////////////////////////////////////////////////////////////////////////
 // Interpolation client.
-namespace sheneos
-{
+namespace sheneos {
     interpolator::interpolator()
-      : num_partitions_per_dim_(0),
-        was_created_(false)
+      : num_partitions_per_dim_(0)
+      , was_created_(false)
     {
         std::memset(minval_, 0, sizeof(minval_));
         std::memset(maxval_, 0, sizeof(maxval_));
@@ -85,7 +83,8 @@ namespace sheneos
 
     interpolator::~interpolator()
     {
-        if (was_created_) {
+        if (was_created_)
+        {
             // Unregister the config data.
             typedef config_data_type::get_action act;
             config_data data = act()(get_id());
@@ -103,11 +102,11 @@ namespace sheneos
 
     ///////////////////////////////////////////////////////////////////////////
     interpolator::interpolator(std::string const& datafilename,
-            std::string const& symbolic_name_base, std::size_t num_instances)
+        std::string const& symbolic_name_base, std::size_t num_instances)
       : base_type(hpx::new_<config_data_type>(hpx::find_here(),
-            config_data(datafilename, symbolic_name_base, num_instances))),
-        num_partitions_per_dim_(0),
-        was_created_(false)
+            config_data(datafilename, symbolic_name_base, num_instances)))
+      , num_partitions_per_dim_(0)
+      , was_created_(false)
     {
         hpx::future<void> f = register_as(symbolic_name_base);
 
@@ -135,7 +134,7 @@ namespace sheneos
         num_partitions_per_dim_ = static_cast<std::size_t>(
             std::exp(std::log(double(data.num_instances_)) / 3));
 
-        if (data.symbolic_name_[data.symbolic_name_.size()-1] != '/')
+        if (data.symbolic_name_[data.symbolic_name_.size() - 1] != '/')
             data.symbolic_name_ += "/";
 
         // Connect to the partitions.
@@ -164,15 +163,15 @@ namespace sheneos
         std::string symbolic_name_base, std::size_t num_instances)
     {
         auto locs = hpx::find_all_localities();
-        hpx::future<std::vector<partition3d> > result =
+        hpx::future<std::vector<partition3d>> result =
             hpx::new_<partition3d[]>(hpx::default_layout(locs), num_instances);
 
         // Read required data from file.
         {
             std::lock_guard<mutex_type> l(mtx_);
-            num_values_[dimension::ye] = extract_data_range(datafilename,
-                "ye", minval_[dimension::ye], maxval_[dimension::ye],
-                delta_[dimension::ye]);
+            num_values_[dimension::ye] =
+                extract_data_range(datafilename, "ye", minval_[dimension::ye],
+                    maxval_[dimension::ye], delta_[dimension::ye]);
             num_values_[dimension::temp] = extract_data_range(datafilename,
                 "logtemp", minval_[dimension::temp], maxval_[dimension::temp],
                 delta_[dimension::temp]);
@@ -182,35 +181,35 @@ namespace sheneos
         }
 
         // Initialize all attached partition objects.
-        num_partitions_per_dim_ = static_cast<std::size_t>(
-            std::cbrt(double(locs.size())));
+        num_partitions_per_dim_ =
+            static_cast<std::size_t>(std::cbrt(double(locs.size())));
 
         std::size_t partition_size_x =
             num_values_[dimension::ye] / num_partitions_per_dim_;
         std::size_t last_partition_size_x = num_values_[dimension::ye] -
-            partition_size_x * (num_partitions_per_dim_-1);
+            partition_size_x * (num_partitions_per_dim_ - 1);
 
         std::size_t partition_size_y =
             num_values_[dimension::temp] / num_partitions_per_dim_;
         std::size_t last_partition_size_y = num_values_[dimension::temp] -
-            partition_size_y * (num_partitions_per_dim_-1);
+            partition_size_y * (num_partitions_per_dim_ - 1);
 
         std::size_t partition_size_z =
             num_values_[dimension::rho] / num_partitions_per_dim_;
         std::size_t last_partition_size_z = num_values_[dimension::rho] -
-            partition_size_z * (num_partitions_per_dim_-1);
+            partition_size_z * (num_partitions_per_dim_ - 1);
 
         dimension dim_x(num_values_[dimension::ye]);
         dimension dim_y(num_values_[dimension::temp]);
         dimension dim_z(num_values_[dimension::rho]);
 
-        std::vector<hpx::future<void> > lazy_sync;
+        std::vector<hpx::future<void>> lazy_sync;
         lazy_sync.reserve(2 * num_instances);
 
         for (std::size_t x = 0; x != num_partitions_per_dim_; ++x)
         {
             dim_x.offset_ = partition_size_x * x;
-            if (x == num_partitions_per_dim_-1)
+            if (x == num_partitions_per_dim_ - 1)
                 dim_x.count_ = last_partition_size_x;
             else
                 dim_x.count_ = partition_size_x;
@@ -218,7 +217,7 @@ namespace sheneos
             for (std::size_t y = 0; y != num_partitions_per_dim_; ++y)
             {
                 dim_y.offset_ = partition_size_y * y;
-                if (y == num_partitions_per_dim_-1)
+                if (y == num_partitions_per_dim_ - 1)
                     dim_y.count_ = last_partition_size_y;
                 else
                     dim_y.count_ = partition_size_y;
@@ -226,13 +225,14 @@ namespace sheneos
                 for (std::size_t z = 0; z != num_partitions_per_dim_; ++z)
                 {
                     dim_z.offset_ = partition_size_z * z;
-                    if (z == num_partitions_per_dim_-1)
+                    if (z == num_partitions_per_dim_ - 1)
                         dim_z.count_ = last_partition_size_z;
                     else
                         dim_z.count_ = partition_size_z;
 
-                    std::size_t index = x + (y + z * num_partitions_per_dim_) *
-                        num_partitions_per_dim_;
+                    std::size_t index = x +
+                        (y + z * num_partitions_per_dim_) *
+                            num_partitions_per_dim_;
                     HPX_ASSERT(index < partitions_.size());
 
                     lazy_sync.push_back(partitions_[index].init_async(
@@ -248,10 +248,10 @@ namespace sheneos
         partitions_ = result.get();
 
         std::size_t i = 0;
-        for (partition3d & p : partitions_)
+        for (partition3d& p : partitions_)
         {
-            lazy_sync.push_back(p.register_as(
-                symbolic_name_base + std::to_string(i++)));
+            lazy_sync.push_back(
+                p.register_as(symbolic_name_base + std::to_string(i++)));
         }
 
         // Wait for initialization to finish.
@@ -259,8 +259,8 @@ namespace sheneos
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    partition3d const&
-    interpolator::get_partition(double ye, double temp, double rho)  const
+    partition3d const& interpolator::get_partition(
+        double ye, double temp, double rho) const
     {
         std::size_t x = get_partition_index(dimension::ye, ye);
         std::size_t y = get_partition_index(dimension::temp, std::log10(temp));
@@ -273,8 +273,8 @@ namespace sheneos
         return partitions_[index];
     }
 
-    std::size_t
-    interpolator::get_partition_index(std::size_t d, double value) const
+    std::size_t interpolator::get_partition_index(
+        std::size_t d, double value) const
     {
         std::size_t partition_size = num_values_[d] / num_partitions_per_dim_;
         std::size_t partition_index = static_cast<std::size_t>(
@@ -286,9 +286,10 @@ namespace sheneos
     }
 
     void interpolator::get_dimension(
-        dimension::type what, double& min, double& max)  const
+        dimension::type what, double& min, double& max) const
     {
-        switch (what) {
+        switch (what)
+        {
         case dimension::ye:
             min = minval_[dimension::ye];
             max = maxval_[dimension::ye];
@@ -301,7 +302,7 @@ namespace sheneos
             break;
 
         default:
-            HPX_THROW_EXCEPTION(hpx::bad_parameter,
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                 "sheneos::interpolator::get_dimension",
                 "value of parameter 'what' is not valid");
             break;
@@ -324,18 +325,21 @@ namespace sheneos
         typedef std::map<hpx::id_type, context_data> partitions_type;
 
         on_completed_bulk_one(std::shared_ptr<partitions_type> parts,
-                context_data const& data,
-                std::vector<double>& overall_result)
-          : data_(data), overall_result_(overall_result), partitions_(parts)
-        {}
+            context_data const& data, std::vector<double>& overall_result)
+          : data_(data)
+          , overall_result_(overall_result)
+          , partitions_(parts)
+        {
+        }
 
-        void operator()(hpx::future<std::vector<double> > f)
+        void operator()(hpx::future<std::vector<double>> f)
         {
             std::vector<double> result = f.get();
             std::vector<std::size_t> const& indices = data_.get().indices_;
 
-            if (result.size() != indices.size()) {
-                HPX_THROW_EXCEPTION(hpx::bad_parameter,
+            if (result.size() != indices.size())
+            {
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "interpolator::on_completed_bulk_one",
                     "inconsistent sizes of result and index arrays");
             }
@@ -346,7 +350,7 @@ namespace sheneos
         }
 
         std::reference_wrapper<context_data const> data_;
-        std::reference_wrapper<std::vector<double> > overall_result_;
+        std::reference_wrapper<std::vector<double>> overall_result_;
         std::shared_ptr<partitions_type> partitions_;
     };
 
@@ -355,9 +359,12 @@ namespace sheneos
         typedef std::map<hpx::id_type, context_data> partitions_type;
 
         bulk_one_context(std::shared_ptr<partitions_type> parts, std::size_t s,
-                std::uint32_t eos)
-          : partitions(parts), size(s), eosvalue(eos)
-        {}
+            std::uint32_t eos)
+          : partitions(parts)
+          , size(s)
+          , eosvalue(eos)
+        {
+        }
 
         std::vector<double> operator()() const
         {
@@ -366,27 +373,24 @@ namespace sheneos
             overall_result.resize(size);
 
             // asynchronously invoke the interpolation on the different partitions
-            std::vector<hpx::future<void> > lazy_results;
+            std::vector<hpx::future<void>> lazy_results;
             lazy_results.reserve(partitions->size());
 
             typedef partitions_type::value_type value_type;
             for (value_type& p : *partitions)
             {
-                typedef sheneos::server::partition3d::interpolate_one_bulk_action
-                    action_type;
+                typedef sheneos::server::partition3d::
+                    interpolate_one_bulk_action action_type;
 
                 context_data& d = p.second;
-                lazy_results.push_back(
-                    hpx::async<action_type>(
-                        p.first, std::move(d.coords_), eosvalue
-                    ).then(
-                        on_completed_bulk_one(partitions, d, overall_result)
-                    )
-                );
+                lazy_results.push_back(hpx::async<action_type>(
+                    p.first, std::move(d.coords_), eosvalue)
+                                           .then(on_completed_bulk_one(
+                                               partitions, d, overall_result)));
             }
 
             // wait for all asynchronous operations to complete
-            wait_all(lazy_results);
+            hpx::wait_all(lazy_results);
 
             return overall_result;
         }
@@ -396,10 +400,8 @@ namespace sheneos
         std::uint32_t eosvalue;
     };
 
-    hpx::future<std::vector<double> >
-    interpolator::interpolate_one_bulk_async(
-        std::vector<sheneos_coord> const& coords,
-        std::uint32_t eosvalue) const
+    hpx::future<std::vector<double>> interpolator::interpolate_one_bulk_async(
+        std::vector<sheneos_coord> const& coords, std::uint32_t eosvalue) const
     {
         typedef std::map<hpx::id_type, context_data> partitions_type;
 
@@ -417,7 +419,8 @@ namespace sheneos
             d.coords_.push_back(c);
         }
 
-        return hpx::async(bulk_one_context(partitions, coords.size(), eosvalue));
+        return hpx::async(
+            bulk_one_context(partitions, coords.size(), eosvalue));
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -428,30 +431,35 @@ namespace sheneos
         typedef std::map<hpx::id_type, context_data> partitions_type;
 
         on_completed_bulk(std::shared_ptr<partitions_type> parts,
-                context_data const& data,
-                std::vector<std::vector<double> >& overall_results)
-          : data_(data), overall_results_(overall_results), partitions_(parts)
-        {}
-
-        void operator()(hpx::future<std::vector<std::vector<double> > > f)
+            context_data const& data,
+            std::vector<std::vector<double>>& overall_results)
+          : data_(data)
+          , overall_results_(overall_results)
+          , partitions_(parts)
         {
-            std::vector<std::vector<double> > result = f.get();
+        }
+
+        void operator()(hpx::future<std::vector<std::vector<double>>> f)
+        {
+            std::vector<std::vector<double>> result = f.get();
             std::vector<std::size_t> const& indices = data_.get().indices_;
 
-            if (result.size() != indices.size()) {
-                HPX_THROW_EXCEPTION(hpx::bad_parameter,
+            if (result.size() != indices.size())
+            {
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "interpolator::on_completed_bulk",
                     "inconsistent sizes of result and index arrays");
             }
 
-            std::vector<std::vector<double> >& overall_results =
+            std::vector<std::vector<double>>& overall_results =
                 overall_results_.get();
             for (std::size_t i = 0; i < indices.size(); ++i)
                 overall_results[indices[i]] = result[i];
         }
 
         std::reference_wrapper<context_data const> data_;
-        std::reference_wrapper<std::vector<std::vector<double> > > overall_results_;
+        std::reference_wrapper<std::vector<std::vector<double>>>
+            overall_results_;
         std::shared_ptr<partitions_type> partitions_;
     };
 
@@ -461,21 +469,24 @@ namespace sheneos
         typedef std::map<hpx::id_type, context_data> partitions_type;
 
         bulk_context(std::shared_ptr<partitions_type> parts, std::size_t s,
-                std::uint32_t eos)
-          : partitions(parts), size(s), eosvalues(eos)
-        {}
+            std::uint32_t eos)
+          : partitions(parts)
+          , size(s)
+          , eosvalues(eos)
+        {
+        }
 
-        std::vector<std::vector<double> > operator()() const
+        std::vector<std::vector<double>> operator()() const
         {
             namespace naming = hpx::naming;
             namespace lcos = hpx::lcos;
 
             // create the overall result vector
-            std::vector<std::vector<double> > overall_results;
+            std::vector<std::vector<double>> overall_results;
             overall_results.resize(size);
 
             // asynchronously invoke the interpolation on the different partitions
-            std::vector<hpx::future<void> > lazy_results;
+            std::vector<hpx::future<void>> lazy_results;
             lazy_results.reserve(partitions->size());
 
             typedef std::map<hpx::id_type, context_data>::value_type value_type;
@@ -485,17 +496,14 @@ namespace sheneos
                     action_type;
 
                 context_data& d = p.second;
-                lazy_results.push_back(
-                    hpx::async<action_type>(
-                        p.first, std::move(d.coords_), eosvalues
-                    ).then(
-                        on_completed_bulk(partitions, d, overall_results)
-                    )
-                 );
+                lazy_results.push_back(hpx::async<action_type>(
+                    p.first, std::move(d.coords_), eosvalues)
+                                           .then(on_completed_bulk(partitions,
+                                               d, overall_results)));
             }
 
             // wait for all asynchronous operations to complete
-            wait_all(lazy_results);
+            hpx::wait_all(lazy_results);
 
             return overall_results;
         }
@@ -505,7 +513,7 @@ namespace sheneos
         std::uint32_t eosvalues;
     };
 
-    hpx::future<std::vector<std::vector<double> > >
+    hpx::future<std::vector<std::vector<double>>>
     interpolator::interpolate_bulk_async(
         std::vector<sheneos_coord> const& coords, std::uint32_t eosvalues) const
     {
@@ -526,5 +534,4 @@ namespace sheneos
 
         return hpx::async(bulk_context(partitions, coords.size(), eosvalues));
     }
-}
-
+}    // namespace sheneos
