@@ -15,12 +15,12 @@
 HPX_REGISTER_COMPONENT(
     hpx::components::component<
         startup_shutdown::server::startup_shutdown_component>,
-    startup_shutdown_component);
+    startup_shutdown_component)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Serialization support for the simple_accumulator actions
 HPX_REGISTER_ACTION(
     startup_shutdown::server::startup_shutdown_component::init_action,
-    startup_shutdown_component_init_action);
+    startup_shutdown_component_init_action)
 
 #endif
