@@ -29,9 +29,7 @@ int HPX_CDECL main(int argc, char** argv)
 {
     // allow for unknown options and inhibit aliasing of short options
     std::vector<std::string> const cfg = {
-        "hpx.commandline.allow_unknown=1",
-        "hpx.commandline.aliasing=0"
-    };
+        "hpx.commandline.allow_unknown=1", "hpx.commandline.aliasing=0"};
 
     hpx::init_params init_args;
     init_args.cfg = cfg;
@@ -41,4 +39,3 @@ int HPX_CDECL main(int argc, char** argv)
 
 // Make sure header testing code does not redefine main()
 #define HPX_MAIN_DEFINED 1
-
