@@ -12,13 +12,14 @@
 
 #include <memory>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     ///////////////////////////////////////////////////////////////////////////
     template <typename Allocator>
     struct allocator_deleter
     {
         template <typename SharedState>
-        void operator()(SharedState* state)
+        void operator()(SharedState* state) noexcept
         {
             using traits = std::allocator_traits<Allocator>;
             traits::deallocate(alloc_, state, 1);
@@ -26,4 +27,4 @@ namespace hpx { namespace util {
 
         Allocator alloc_;
     };
-}}    // namespace hpx::util
+}    // namespace hpx::util
