@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -14,10 +14,12 @@
 #include <deque>
 #include <functional>
 #include <map>
+#include <type_traits>
 #include <utility>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace util { namespace cache {
+namespace hpx::util::cache {
+
     ///////////////////////////////////////////////////////////////////////////
     /// \class local_cache local_cache.hpp hpx/cache/local_cache.hpp
     ///
@@ -72,10 +74,14 @@ namespace hpx { namespace util { namespace cache {
         template <typename Func, typename Iterator>
         struct adapt
         {
-            adapt(Func f)
+            explicit adapt(Func const& f)
               : f_(f)
             {
             }
+            explicit adapt(Func&& f) noexcept
+              : f_(HPX_MOVE(f))
+            {
+            }
 
             bool operator()(Iterator const& lhs, Iterator const& rhs) const
             {
@@ -86,27 +92,27 @@ namespace hpx { namespace util { namespace cache {
         };
 
     public:
-        typedef Key key_type;
-        typedef Entry entry_type;
-        typedef UpdatePolicy update_policy_type;
-        typedef InsertPolicy insert_policy_type;
-        typedef CacheStorage storage_type;
-        typedef Statistics statistics_type;
+        using key_type = Key;
+        using entry_type = Entry;
+        using update_policy_type = UpdatePolicy;
+        using insert_policy_type = InsertPolicy;
+        using storage_type = CacheStorage;
+        using statistics_type = Statistics;
 
-        typedef typename entry_type::value_type value_type;
-        typedef typename storage_type::size_type size_type;
-        typedef typename storage_type::value_type storage_value_type;
+        using value_type = typename entry_type::value_type;
+        using size_type = typename storage_type::size_type;
+        using storage_value_type = typename storage_type::value_type;
 
     private:
-        typedef typename storage_type::iterator iterator;
-        typedef typename storage_type::const_iterator const_iterator;
+        using iterator = typename storage_type::iterator;
+        using const_iterator = typename storage_type::const_iterator;
 
-        typedef std::deque<iterator> heap_type;
-        typedef typename heap_type::iterator heap_iterator;
+        using heap_type = std::deque<iterator>;
+        using heap_iterator = typename heap_type::iterator;
 
-        typedef adapt<UpdatePolicy, iterator> adapted_update_policy_type;
+        using adapted_update_policy_type = adapt<UpdatePolicy, iterator>;
 
-        typedef typename statistics_type::update_on_exit update_on_exit;
+        using update_on_exit = typename statistics_type::update_on_exit;
 
     public:
         ///////////////////////////////////////////////////////////////////////
@@ -126,7 +132,7 @@ namespace hpx { namespace util { namespace cache {
         ///                   constructed instance of the type as defined by
         ///                   the \a InsertPolicy template parameter.
         ///
-        local_cache(size_type max_size = 0,
+        explicit local_cache(size_type max_size = 0,
             update_policy_type const& up = update_policy_type(),
             insert_policy_type const& ip = insert_policy_type())
           : max_size_(max_size)
@@ -136,22 +142,18 @@ namespace hpx { namespace util { namespace cache {
         {
         }
 
-        local_cache(local_cache&& other)
-          : max_size_(other.max_size_)
-          , current_size_(other.current_size_)
-          , store_(std::move(other.store_))
-          , entry_heap_(std::move(other.entry_heap_))
-          , update_policy_(std::move(other.update_policy_.f_))
-          , insert_policy_(std::move(other.insert_policy_))
-          , statistics_(std::move(other.statistics_))
-        {
-        }
+        local_cache(local_cache const& other) = default;
+        local_cache(local_cache&& other) = default;
+        local_cache& operator=(local_cache const& other) = default;
+        local_cache& operator=(local_cache&& other) = default;
+
+        ~local_cache() = default;
 
         ///////////////////////////////////////////////////////////////////////
         /// \brief Return current size of the cache.
         ///
         /// \returns The current size of this cache instance.
-        size_type size() const
+        [[nodiscard]] constexpr size_type size() const noexcept
         {
             return current_size_;
         }
@@ -166,7 +168,7 @@ namespace hpx { namespace util { namespace cache {
         /// \returns    The maximum size this cache instance is currently
         ///             allowed to reach. If this number is zero the cache has
         ///             no limitation with regard to a maximum size.
-        size_type capacity() const
+        [[nodiscard]] constexpr size_type capacity() const noexcept
         {
             return max_size_;
         }
@@ -187,7 +189,7 @@ namespace hpx { namespace util { namespace cache {
             // the old one
             bool retval = true;
             if (max_size && max_size < max_size_ &&
-                !free_space(long(max_size_ - max_size)))
+                !free_space(static_cast<long>(max_size_ - max_size)))
             {
                 retval = false;    // not able to shrink cache
             }
@@ -209,7 +211,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \returns      This function returns \a true if the cache holds the
         ///               referenced entry, otherwise it returns \a false.
-        bool holds_key(key_type const& k) const
+        [[nodiscard]] bool holds_key(key_type const& k) const
         {
             return store_.find(k) != store_.end();
         }
@@ -219,6 +221,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \param k      [in] The key for the entry which should be retrieved
         ///               from the cache.
+        /// \param realkey[out] Return the full real key found in the cache
         /// \param val    [out] If the entry indexed by the key is found in the
         ///               cache this value on successful return will be a copy
         ///               of the corresponding entry.
@@ -231,7 +234,7 @@ namespace hpx { namespace util { namespace cache {
         ///               referenced entry, otherwise it returns \a false.
         bool get_entry(key_type const& k, key_type& realkey, entry_type& val)
         {
-            update_on_exit update(statistics_, statistics::method_get_entry);
+            update_on_exit update(statistics_, statistics::method::get_entry);
 
             // locate the requested entry
             iterator it = store_.find(k);
@@ -255,6 +258,7 @@ namespace hpx { namespace util { namespace cache {
             // return the value
             realkey = (*it).first;
             val = (*it).second;
+
             return true;
         }
 
@@ -274,7 +278,7 @@ namespace hpx { namespace util { namespace cache {
         ///               referenced entry, otherwise it returns \a false.
         bool get_entry(key_type const& k, entry_type& val)
         {
-            update_on_exit update(statistics_, statistics::method_get_entry);
+            update_on_exit update(statistics_, statistics::method::get_entry);
 
             // locate the requested entry
             iterator it = store_.find(k);
@@ -297,6 +301,7 @@ namespace hpx { namespace util { namespace cache {
 
             // return the value
             val = (*it).second;
+
             return true;
         }
 
@@ -316,7 +321,7 @@ namespace hpx { namespace util { namespace cache {
         ///               referenced entry, otherwise it returns \a false.
         bool get_entry(key_type const& k, value_type& val)
         {
-            update_on_exit update(statistics_, statistics::method_get_entry);
+            update_on_exit update(statistics_, statistics::method::get_entry);
 
             // locate the requested entry
             iterator it = store_.find(k);
@@ -339,6 +344,7 @@ namespace hpx { namespace util { namespace cache {
 
             // return the value
             val = (*it).second.get();
+
             return true;
         }
 
@@ -347,7 +353,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \param k      [in] The key for the entry which should be added to
         ///               the cache.
-        /// \param value  [in] The value which should be added to the cache.
+        /// \param val    [in] The value which should be added to the cache.
         ///
         /// \note         This function invokes both, the insert policy as
         ///               provided to the constructor and the function
@@ -371,11 +377,16 @@ namespace hpx { namespace util { namespace cache {
             return insert(k, e);
         }
 
+        bool insert(key_type const& k, value_type&& val)
+        {
+            return insert(k, entry_type(HPX_MOVE(val)));
+        }
+
         /// \brief Insert a new entry into this cache
         ///
         /// \param k      [in] The key for the entry which should be added to
         ///               the cache.
-        /// \param value  [in] The entry which should be added to the cache.
+        /// \param e      [in] The entry which should be added to the cache.
         ///
         /// \note         This function invokes both, the insert policy as
         ///               provided to the constructor and the function
@@ -393,9 +404,14 @@ namespace hpx { namespace util { namespace cache {
         /// \returns      This function returns \a true if the entry has been
         ///               successfully added to the cache, otherwise it returns
         ///               \a false.
-        bool insert(key_type const& k, entry_type& e)
+        template <typename Entry_,
+            std::enable_if_t<
+                std::is_convertible_v<std::decay_t<Entry_>, entry_type>, int> =
+                0>
+        bool insert(key_type const& k, Entry_&& e)
         {
-            update_on_exit update(statistics_, statistics::method_insert_entry);
+            update_on_exit update(
+                statistics_, statistics::method::insert_entry);
 
             // ask entry if it really wants to be inserted
             if (!insert_policy_(e) || !e.insert())
@@ -404,15 +420,15 @@ namespace hpx { namespace util { namespace cache {
             // make sure cache doesn't get too large
             size_type entry_size = e.get_size();
             if (0 != max_size_ && current_size_ + entry_size > max_size_ &&
-                !free_space(long(current_size_ - max_size_ + entry_size)))
+                !free_space(
+                    static_cast<long>(current_size_ - max_size_ + entry_size)))
             {
                 return false;
             }
 
             // insert new entry to cache
-            typedef typename storage_type::value_type storage_value_type;
             std::pair<iterator, bool> p =
-                store_.insert(storage_value_type(k, e));
+                store_.insert(storage_value_type(k, HPX_FORWARD(Entry_, e)));
             if (!p.second)
                 return false;
 
@@ -434,7 +450,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \param k      [in] The key for the value which should be updated in
         ///               the cache.
-        /// \param value  [in] The value which should be used as a replacement
+        /// \param val    [in] The value which should be used as a replacement
         ///               for the existing value in the cache. Any existing
         ///               cache entry is not changed except for its value.
         ///
@@ -451,20 +467,26 @@ namespace hpx { namespace util { namespace cache {
         ///               If the entry currently is not held by the cache it is
         ///               added and the return value reflects the outcome of
         ///               the corresponding insert operation.
-        bool update(key_type const& k, value_type const& val)
+        template <typename Value,
+            std::enable_if_t<
+                std::is_convertible_v<std::decay_t<Value>, value_type>, int> =
+                0>
+        bool update(key_type const& k, Value&& val)
         {
-            update_on_exit update(statistics_, statistics::method_update_entry);
+            update_on_exit update(
+                statistics_, statistics::method::update_entry);
 
             iterator it = store_.find(k);
             if (it == store_.end())
             {
                 // doesn't exist in this cache
                 statistics_.got_miss();    // update statistics
-                return insert(k, val);     // insert into cache
+                return insert(
+                    k, HPX_FORWARD(Value, val));    // insert into cache
             }
 
             // update cache entry
-            (*it).second.get() = val;
+            (*it).second.get() = HPX_FORWARD(Value, val);
 
             // touch the entry
             if ((*it).second.touch())
@@ -485,7 +507,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \param k      [in] The key for the value which should be updated in
         ///               the cache.
-        /// \param value  [in] The value which should be used as a replacement
+        /// \param val    [in] The value which should be used as a replacement
         ///               for the existing value in the cache. Any existing
         ///               cache entry is not changed except for its value.
         /// \param f      [in] A callable taking two arguments, \a k and the
@@ -506,24 +528,28 @@ namespace hpx { namespace util { namespace cache {
         ///               If the entry currently is not held by the cache it is
         ///               added and the return value reflects the outcome of
         ///               the corresponding insert operation.
-        template <typename F>
-        bool update_if(key_type const& k, value_type const& val, F f)
+        template <typename F, typename Value,
+            typename = std::enable_if_t<
+                std::is_convertible_v<std::decay_t<Value>, value_type>>>
+        bool update_if(key_type const& k, Value&& val, F&& f)
         {
-            update_on_exit update(statistics_, statistics::method_update_entry);
+            update_on_exit update(
+                statistics_, statistics::method::update_entry);
 
             iterator it = store_.find(k);
             if (it == store_.end())
             {
                 // doesn't exist in this cache
                 statistics_.got_miss();    // update statistics
-                return insert(k, val);     // insert into cache
+                return insert(
+                    k, HPX_FORWARD(Value, val));    // insert into cache
             }
 
             if (!f(k, (*it).first))
                 return false;
 
             // update cache entry
-            (*it).second.get() = val;
+            (*it).second.get() = HPX_FORWARD(Value, val);
 
             // touch the entry
             if ((*it).second.touch())
@@ -543,7 +569,7 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \param k      [in] The key for the entry which should be updated in
         ///               the cache.
-        /// \param value  [in] The entry which should be used as a replacement
+        /// \param e      [in] The entry which should be used as a replacement
         ///               for the existing entry in the cache. Any existing
         ///               entry is first removed and then this entry is added.
         ///
@@ -560,9 +586,14 @@ namespace hpx { namespace util { namespace cache {
         ///               If the entry currently is not held by the cache it is
         ///               added and the return value reflects the outcome of
         ///               the corresponding insert operation.
-        bool update(key_type const& k, entry_type& e)
+        template <typename Entry_,
+            std::enable_if_t<
+                std::is_convertible_v<std::decay_t<Entry_>, entry_type>, int> =
+                0>
+        bool update(key_type const& k, Entry_&& e)
         {
-            update_on_exit update(statistics_, statistics::method_update_entry);
+            update_on_exit update(
+                statistics_, statistics::method::update_entry);
 
             iterator it = store_.find(k);
             if (it == store_.end())
@@ -581,7 +612,7 @@ namespace hpx { namespace util { namespace cache {
                 return false;    // entry doesn't want to be inserted
 
             // update cache entry
-            (*it).second = e;
+            (*it).second = HPX_FORWARD(Entry_, e);
 
             // touch the entry
             if ((*it).second.touch())
@@ -613,10 +644,10 @@ namespace hpx { namespace util { namespace cache {
         ///               entries (which is the sum of the values returned by
         ///               the \a entry#get_size functions of the removed
         ///               entries).
-        template <typename Func>
-        size_type erase(Func const& ep = policies::always<storage_value_type>())
+        template <typename Func = policies::always<storage_value_type>>
+        size_type erase(Func&& ep = Func())
         {
-            update_on_exit update(statistics_, statistics::method_erase_entry);
+            update_on_exit update(statistics_, statistics::method::erase_entry);
 
             size_type erased = 0;
             for (heap_iterator it = entry_heap_.begin();
@@ -696,12 +727,13 @@ namespace hpx { namespace util { namespace cache {
         ///
         /// \returns      This function returns a reference to the statistics
         ///               instance embedded inside this cache
-        statistics_type const& get_statistics() const
+        [[nodiscard]] constexpr statistics_type const& get_statistics()
+            const noexcept
         {
             return statistics_;
         }
 
-        statistics_type& get_statistics()
+        [[nodiscard]] statistics_type& get_statistics() noexcept
         {
             return statistics_;
         }
@@ -779,4 +811,4 @@ namespace hpx { namespace util { namespace cache {
 
         statistics_type statistics_;    // embedded statistics instance
     };
-}}}    // namespace hpx::util::cache
+}    // namespace hpx::util::cache
