@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -9,14 +9,15 @@
 #include <hpx/config.hpp>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace util { namespace cache { namespace policies {
+namespace hpx::util::cache::policies {
+
     ///////////////////////////////////////////////////////////////////////////
     template <typename Entry>
     struct always
     {
-        bool operator()(Entry const&)
+        constexpr bool operator()(Entry const&) noexcept
         {
             return true;    // always true
         }
     };
-}}}}    // namespace hpx::util::cache::policies
+}    // namespace hpx::util::cache::policies
