@@ -1,4 +1,4 @@
-//  Copyright (c) 2017 Hartmut Kaiser
+//  Copyright (c) 2017-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -17,17 +17,24 @@
 
 #include <hpx/config/warnings_prefix.hpp>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     class HPX_CORE_EXPORT barrier
     {
     private:
-        typedef std::mutex mutex_type;
+        using mutex_type = std::mutex;
 
-        HPX_STATIC_CONSTEXPR std::size_t barrier_flag =
-            static_cast<std::size_t>(1) << (CHAR_BIT * sizeof(std::size_t) - 1);
+        static constexpr std::size_t barrier_flag = static_cast<std::size_t>(1)
+            << (CHAR_BIT * sizeof(std::size_t) - 1);
 
     public:
-        barrier(std::size_t number_of_threads);
+        explicit barrier(std::size_t number_of_threads);
+
+        barrier(barrier const&) = delete;
+        barrier(barrier&&) = delete;
+        barrier& operator=(barrier const&) = delete;
+        barrier& operator=(barrier&&) = delete;
+
         ~barrier();
 
         void wait();
@@ -39,6 +46,6 @@ namespace hpx { namespace util {
         mutable mutex_type mtx_;
         std::condition_variable cond_;
     };
-}}    // namespace hpx::util
+}    // namespace hpx::util
 
 #include <hpx/config/warnings_suffix.hpp>
