@@ -7,26 +7,15 @@
 
 #pragma once
 
-// We need to detect if user code include boost/config.hpp before
-// including hpx/config.hpp
-// Everything else might lead to hard compile errors and possible very subtle bugs.
-#if defined(BOOST_CONFIG_HPP)
-#error Boost.Config was included before the hpx config header. This might lead to subtle failures and compile errors. Please include <hpx/config.hpp> before any other boost header
-#endif
-
 #include <hpx/config/defines.hpp>
 
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
-// On Windows, make sure winsock.h is not included even if windows.h is
-// included before winsock2.h
-#define _WINSOCKAPI_
-#include <winsock2.h>
-#endif
+#include <hpx/config/asio.hpp>
 
 #include <hpx/config/attributes.hpp>
+#include <hpx/config/auto_vectorization.hpp>
 #include <hpx/config/branch_hints.hpp>
+#include <hpx/config/cache_line_size.hpp>
 #include <hpx/config/compiler_fence.hpp>
-#include <hpx/config/compiler_native_tls.hpp>
 #include <hpx/config/compiler_specific.hpp>
 #include <hpx/config/constexpr.hpp>
 #include <hpx/config/debug.hpp>
@@ -34,10 +23,11 @@
 #include <hpx/config/emulate_deleted.hpp>
 #include <hpx/config/export_definitions.hpp>
 #include <hpx/config/forceinline.hpp>
-#include <hpx/config/lambda_capture.hpp>
+#include <hpx/config/forward.hpp>
 #include <hpx/config/lambda_capture_this.hpp>
 #include <hpx/config/manual_profiling.hpp>
 #include <hpx/config/modules_enabled.hpp>
+#include <hpx/config/move.hpp>
 #include <hpx/config/threads_stack.hpp>
 #include <hpx/config/version.hpp>
 #include <hpx/config/weak_symbol.hpp>
@@ -52,11 +42,7 @@
 #include <hpx/preprocessor/cat.hpp>
 #include <hpx/preprocessor/stringize.hpp>
 
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
-// On Windows, make sure winsock.h is not included even if windows.h is
-// included before winsock2.h
-#define _WINSOCKAPI_
-#endif
+#include <cstddef>
 
 // clang-format off
 
@@ -121,6 +107,18 @@
 #  define HPX_PARCEL_MPI_MAX_REQUESTS 2147483647
 #endif
 
+/// This defines the number of cores that perform background work for the MPI
+/// parcelport
+/// This value can be changed at runtime by setting the configuration parameter:
+///
+///   hpx.parcel.mpi.background_threads = ...
+///
+/// (or by setting the corresponding environment variable
+/// HPX_HAVE_PARCELPORT_MPI_BACKGROUND_THREADS).
+#if !defined(HPX_HAVE_PARCELPORT_MPI_BACKGROUND_THREADS)
+#  define HPX_HAVE_PARCELPORT_MPI_BACKGROUND_THREADS std::size_t(-1)
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 /// This defines the number of outgoing (parcel-) connections kept alive (to
 /// each of the other localities). This value can be changed at runtime by
@@ -208,60 +206,10 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 /// By default, enable minimal thread deadlock detection in debug builds only.
-#if !defined(HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION
-#  endif
-#endif
-#if !defined(HPX_HAVE_SPINLOCK_DEADLOCK_DETECTION)
-#  if defined(HPX_DEBUG)
-//#    define HPX_HAVE_SPINLOCK_DEADLOCK_DETECTION
-#  endif
-#endif
 #if !defined(HPX_SPINLOCK_DEADLOCK_DETECTION_LIMIT)
 #  define HPX_SPINLOCK_DEADLOCK_DETECTION_LIMIT 1073741823
 #endif
 
-///////////////////////////////////////////////////////////////////////////////
-/// This defines the default number of coroutine heaps.
-#if !defined(HPX_COROUTINE_NUM_HEAPS)
-#  define HPX_COROUTINE_NUM_HEAPS 7
-#endif
-
-///////////////////////////////////////////////////////////////////////////////
-/// By default, enable storing the parent thread information in debug builds
-/// only.
-#if !defined(HPX_HAVE_THREAD_PARENT_REFERENCE)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_THREAD_PARENT_REFERENCE
-#  endif
-#endif
-
-///////////////////////////////////////////////////////////////////////////////
-/// By default, enable storing the thread phase in debug builds only.
-#if !defined(HPX_HAVE_THREAD_PHASE_INFORMATION)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_THREAD_PHASE_INFORMATION
-#  endif
-#endif
-
-///////////////////////////////////////////////////////////////////////////////
-/// By default, enable storing the thread description in debug builds only.
-#if !defined(HPX_HAVE_THREAD_DESCRIPTION)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_THREAD_DESCRIPTION
-#  endif
-#endif
-
-///////////////////////////////////////////////////////////////////////////////
-/// By default, enable storing the target address of the data the thread is
-/// accessing in debug builds only.
-#if !defined(HPX_HAVE_THREAD_TARGET_ADDRESS)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_THREAD_TARGET_ADDRESS
-#  endif
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 /// By default we do not maintain stack back-traces on suspension. This is a
 /// pure debugging aid to be able to see in the debugger where a suspended
@@ -313,19 +261,19 @@
 ///////////////////////////////////////////////////////////////////////////////
 #if !defined(HPX_WINDOWS)
 #  if defined(HPX_DEBUG)
-#    define HPX_MAKE_DLL_STRING(n)  "lib" + n + "d" + HPX_SHARED_LIB_EXTENSION
+#    define HPX_MAKE_DLL_STRING(n)  "lib" + (n) + "d" + HPX_SHARED_LIB_EXTENSION
 #  else
-#    define HPX_MAKE_DLL_STRING(n)  "lib" + n + HPX_SHARED_LIB_EXTENSION
+#    define HPX_MAKE_DLL_STRING(n)  "lib" + (n) + HPX_SHARED_LIB_EXTENSION
 #  endif
 #elif defined(HPX_DEBUG)
-#  define HPX_MAKE_DLL_STRING(n)   (n) + "d" + HPX_SHARED_LIB_EXTENSION
+#  define HPX_MAKE_DLL_STRING(n)   ((n) + "d" + HPX_SHARED_LIB_EXTENSION)
 #else
-#  define HPX_MAKE_DLL_STRING(n)   (n) + HPX_SHARED_LIB_EXTENSION
+#  define HPX_MAKE_DLL_STRING(n)   ((n) + HPX_SHARED_LIB_EXTENSION)
 #endif
 
 #if defined(HPX_DEBUG)
 #  define HPX_MANGLE_NAME(n)     HPX_PP_CAT(n, d)
-#  define HPX_MANGLE_STRING(n)   (n) + "d"
+#  define HPX_MANGLE_STRING(n)   ((n) + "d")
 #else
 #  define HPX_MANGLE_NAME(n)     n
 #  define HPX_MANGLE_STRING(n)   n
@@ -375,18 +323,6 @@
 #  define HPX_PREFIX HPX_PREFIX_DEFAULT
 #endif
 
-#if defined(HPX_APPLICATION_NAME_DEFAULT) && !defined(HPX_APPLICATION_NAME)
-#  define HPX_APPLICATION_NAME HPX_APPLICATION_NAME_DEFAULT
-#endif
-
-#if !defined(HPX_APPLICATION_STRING)
-#  if defined(HPX_APPLICATION_NAME)
-#    define HPX_APPLICATION_STRING HPX_PP_STRINGIZE(HPX_APPLICATION_NAME)
-#  else
-#    define HPX_APPLICATION_STRING "unknown HPX application"
-#  endif
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 // Count number of empty (no HPX thread available) thread manager loop executions
 #if !defined(HPX_IDLE_LOOP_COUNT_MAX)
@@ -450,6 +386,13 @@
 #  define HPX_THREAD_QUEUE_MAX_TERMINATED_THREADS 100
 #endif
 
+///////////////////////////////////////////////////////////////////////////////
+// Number of threads (of the default stack size) to pre-allocate when
+// initializing a thread queue.
+#if !defined(HPX_THREAD_QUEUE_INIT_THREADS_COUNT)
+#  define HPX_THREAD_QUEUE_INIT_THREADS_COUNT 10
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 // Maximum sleep time for idle backoff in milliseconds (used only if
 // HPX_HAVE_THREAD_MANAGER_IDLE_BACKOFF is defined).
@@ -466,21 +409,6 @@
 #  define HPX_INITIAL_GID_RANGE 0xFFFFU
 #endif
 
-///////////////////////////////////////////////////////////////////////////////
-// Enable lock verification code which allows to check whether there are locks
-// held while HPX-threads are suspended and/or interrupted.
-#if !defined(HPX_HAVE_VERIFY_LOCKS)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_VERIFY_LOCKS
-#  endif
-#endif
-
-#if !defined(HPX_HAVE_VERIFY_LOCKS_GLOBALLY)
-#  if defined(HPX_DEBUG)
-#    define HPX_HAVE_VERIFY_LOCKS_GLOBALLY
-#  endif
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 // This limits how deep the internal recursion of future continuations will go
 // before a new operation is re-spawned.
@@ -511,4 +439,8 @@
 #  define HPX_HAVE_MAX_CPU_COUNT 256
 #endif
 
+#if !defined(HPX_HAVE_MAX_CPU_COUNT) && !defined(HPX_HAVE_MORE_THAN_64_THREADS)
+#define HPX_HAVE_MAX_CPU_COUNT 64
+#endif
+
 // clang-format on
