@@ -7,5 +7,13 @@
 #pragma once
 
 #if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
+// On Windows, include winsock2.h as early as possible to make sure its seen by
+// the compiler before #including windows.h
 #include <winsock2.h>
 #endif
+
+namespace asio {
+
+    // forward declaration
+    class io_context;
+}    // namespace asio
