@@ -14,7 +14,6 @@
 #include <boost/system/error_code.hpp>
 
 #include <system_error>
-#include <type_traits>
 
 ///////////////////////////////////////////////////////////////////////////////
 /// \cond NODETAIL
@@ -25,7 +24,6 @@ namespace hpx {
     public:
         explicit compat_error_code(std::error_code& ec)
           : _ec(ec)
-          , _compat()
         {
         }
 
@@ -34,14 +32,17 @@ namespace hpx {
             _ec = _compat;
         }
 
+        // different versions of clang-format disagree
+        // clang-format off
         operator boost::system::error_code&() noexcept
         {
             return _compat;
         }
+        // clang-format on
 
         template <typename E,
-            typename Enable = typename std::enable_if<
-                boost::system::is_error_code_enum<E>::value>::type>
+            typename Enable =
+                std::enable_if_t<boost::system::is_error_code_enum<E>::value>>
         static bool equal(std::error_code const& lhs, E rhs) noexcept
         {
             return lhs == std::error_code(make_error_code(rhs));
