@@ -1,4 +1,4 @@
-//  Copyright (c) 2021 Hartmut Kaiser
+//  Copyright (c) 2021-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,6 +7,10 @@
 #include <hpx/config.hpp>
 #include <hpx/coroutines/detail/context_impl.hpp>
 
+#if defined(HPX_HAVE_UNISTD_H)
+#include <unistd.h>
+#endif
+
 // The preprocessor conditions below are kept in sync with those used in
 // context_impl.hpp
 
@@ -24,12 +28,10 @@
 
 #include <cstddef>
 
-namespace hpx { namespace threads { namespace coroutines { namespace detail {
-    namespace posix {
+namespace hpx::threads::coroutines::detail::posix {
 
-        std::ptrdiff_t ucontext_context_impl_base::default_stack_size =
-            SIGSTKSZ;
-}}}}}    // namespace hpx::threads::coroutines::detail::posix
+    std::ptrdiff_t ucontext_context_impl_base::default_stack_size = SIGSTKSZ;
+}    // namespace hpx::threads::coroutines::detail::posix
 
 #elif defined(HPX_HAVE_FIBER_BASED_COROUTINES)
 
