@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2013 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,11 +6,12 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
+#include <hpx/serialization/serialize_buffer_fwd.hpp>
 
 #include <type_traits>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
+
     ///////////////////////////////////////////////////////////////////////////
     // Customization point for streaming with util::any
     template <typename T, typename Enable = void>
@@ -18,4 +19,13 @@ namespace hpx { namespace traits {
       : std::true_type    // the default is to support streaming
     {
     };
-}}    // namespace hpx::traits
+
+    ///////////////////////////////////////////////////////////////////////////
+    // Customization point for streaming with util::any, we don't want
+    // serialization::serialize_buffer to be streamable
+    template <typename T, typename Allocator>
+    struct supports_streaming_with_any<
+        serialization::serialize_buffer<T, Allocator>> : std::false_type
+    {
+    };
+}    // namespace hpx::traits
