@@ -1,13 +1,101 @@
 //  Copyright (c) 2011-2013 Thomas Heller
-//  Copyright (c) 2011-2013 Hartmut Kaiser
+//  Copyright (c) 2011-2022 Hartmut Kaiser
 //  Copyright (c) 2013-2015 Agustin Berge
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
+/// \file tuple.hpp
+/// \page hpx::ignore, hpx::tuple, hpx::tuple_size, hpx::tuple_element, hpx::make_tuple, hpx::tie, hpx::forward_as_tuple, hpx::tuple_cat, hpx::get
+/// \headerfile hpx/tuple.hpp
+
 #pragma once
 
+#if defined(DOXYGEN)
+namespace hpx {
+    // clang-format off
+
+    /// \brief Class template hpx::tuple is a fixed-size collection of
+    ///        heterogeneous values. It is a generalization of \a hpx::pair.
+    ///        If \a std::is_trivially_destructible<Ti>::value is true for
+    ///        every \a Ti in \a Ts, the destructor of tuple is trivial.
+    /// \param Ts... the types of the elements that the tuple stores.
+    ////       Empty list is supported.
+    template <typename... Ts>
+    class tuple;
+
+    /// \brief Provides access to the number of elements in a tuple-like type
+    ///        as a compile-time constant expression.
+    /// \details  The primary template is not defined. An explicit (full) or
+    ///           partial specialization is required to make a type tuple-like.
+    template <typename T>
+    struct tuple_size;
+
+    /// \brief Provides compile-time indexed access to the types of the elements
+    ///        of a tuple-like type.
+    /// \details  The primary template is not defined. An explicit (full) or
+    ///           partial specialization is required to make a type tuple-like.
+    template <std::size_t I, typename T, typename Enable = void>
+    struct tuple_element;
+
+    /// \brief An object of unspecified type such that any value can be assigned to
+    ///        it with no effect. Intended for use with hpx::tie when unpacking a
+    ///        hpx::tuple, as a placeholder for the arguments that are not used.
+    ///        While the behavior of hpx::ignore outside of hpx::tie is not formally
+    ///        specified, some code guides recommend using hpx::ignore to avoid warnings
+    ///        from unused return values of [[nodiscard]] functions.
+    constexpr hpx::detail::ignore_type ignore = {};
+
+    /// \brief Provides compile-time indexed access to the types of the elements of the tuple.
+    template <typename... Ts>
+    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE tuple<util::decay_unwrap_t<Ts>...>
+    make_tuple(Ts&&... ts);
+
+    /// \brief Constructs a tuple of references to the arguments in args suitable
+    ///        for forwarding as an argument to a function. The tuple has rvalue
+    ///        reference data members when rvalues are used as arguments, and
+    ///        otherwise has lvalue reference data members.
+    /// \param ts zero or more arguments to construct the tuple from
+    /// \returns hpx::tuple object created as if by
+    ///          \code hpx::tuple<Ts&&...>(HPX_FORWARD(Ts, ts)...) \endcode
+    template <typename... Ts>
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr tuple<Ts&&...> forward_as_tuple(Ts&&... ts);
+
+    /// \brief Creates a tuple of lvalue references to its arguments or instances
+    ///        of hpx::ignore.
+    /// \param ts zero or more lvalue arguments to construct the tuple from.
+    /// \returns hpx::tuple object containing lvalue references.
+    template <typename... Ts>
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr tuple<Ts&...> tie(Ts&... ts);
+
+    /// \brief Constructs a tuple that is a concatenation of all tuples in \c tuples.
+    ///        The behavior is undefined if any type in \c std::decay_t<Tuples>...
+    ///        is not a specialization of \c hpx::tuple. However, an implementation
+    ///        may choose to support types (such as \c std::array and \c std::pair)
+    ///        that follow the tuple-like protocol.
+    ///
+    /// \param tuples - zero or more tuples to concatenate
+    ///
+    /// \returns hpx::tuple object composed of all elements of all argument tuples
+    ///          constructed from \c hpx::get<Is>(HPX_FORWARD(UTuple,t) for each
+    ///          individual element.
+    template <typename... Tuples>
+    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto tuple_cat(Tuples&&... tuples);
+
+    /// \brief Extracts the Ith element from the tuple. I must be an integer
+    ///        value in [0, sizeof...(Ts)).
+    template <std::size_t I> HPX_HOST_DEVICE
+    typename util::at_index<I, Ts...>::type& get() noexcept;
+
+    /// \copybrief hpx::get()
+    template <std::size_t I> HPX_HOST_DEVICE
+    typename util::at_index<I, Ts...>::type const& get() const noexcept;
+    // clang-format on
+}    // namespace hpx
+
+#else
+
 #include <hpx/config.hpp>
 #include <hpx/datastructures/config/defines.hpp>
 #include <hpx/datastructures/member_pack.hpp>
@@ -31,6 +119,7 @@
 #endif
 
 namespace hpx {
+
     template <typename... Ts>
     class tuple;
 
@@ -46,30 +135,29 @@ namespace hpx {
     namespace adl_barrier {
 
         template <std::size_t I, typename Tuple,
-            typename Enable = typename util::always_void<
-                typename hpx::tuple_element<I, Tuple>::type>::type>
+            typename Enable =
+                std::void_t<typename hpx::tuple_element<I, Tuple>::type>>
         constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
             typename hpx::tuple_element<I, Tuple>::type&
             get(Tuple& t) noexcept;
 
         template <std::size_t I, typename Tuple,
-            typename Enable = typename util::always_void<
-                typename hpx::tuple_element<I, Tuple>::type>::type>
+            typename Enable =
+                std::void_t<typename hpx::tuple_element<I, Tuple>::type>>
         constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
             typename hpx::tuple_element<I, Tuple>::type const&
             get(Tuple const& t) noexcept;
 
         template <std::size_t I, typename Tuple,
-            typename Enable =
-                typename util::always_void<typename hpx::tuple_element<I,
-                    typename std::decay<Tuple>::type>::type>::type>
+            typename Enable = std::void_t<
+                typename hpx::tuple_element<I, std::decay_t<Tuple>>::type>>
         constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
             typename hpx::tuple_element<I, Tuple>::type&&
             get(Tuple&& t) noexcept;
 
         template <std::size_t I, typename Tuple,
-            typename Enable = typename util::always_void<
-                typename hpx::tuple_element<I, Tuple>::type>::type>
+            typename Enable =
+                std::void_t<typename hpx::tuple_element<I, Tuple>::type>>
         constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
             typename hpx::tuple_element<I, Tuple>::type const&&
             get(Tuple const&& t) noexcept;
@@ -138,9 +226,8 @@ namespace hpx {
         template <std::size_t... Is, typename... Ts, typename UTuple>
         struct are_tuples_compatible_impl<util::index_pack<Is...>, tuple<Ts...>,
             UTuple,
-            typename std::enable_if<tuple_size<typename std::remove_reference<
-                                        UTuple>::type>::value ==
-                util::pack<Ts...>::size>::type>
+            std::enable_if_t<tuple_size<std::remove_reference_t<UTuple>>::
+                                 value == util::pack<Ts...>::size>>
           : util::all_of<std::is_convertible<
                 decltype(hpx::get<Is>(std::declval<UTuple>())), Ts>...>
         {
@@ -151,9 +238,8 @@ namespace hpx {
 
         template <typename... Ts, typename UTuple>
         struct are_tuples_compatible<tuple<Ts...>, UTuple>
-          : are_tuples_compatible_impl<
-                typename util::make_index_pack<sizeof...(Ts)>::type,
-                tuple<Ts...>, UTuple>
+          : are_tuples_compatible_impl<util::make_index_pack_t<sizeof...(Ts)>,
+                hpx::tuple<Ts...>, UTuple>
         {
         };
 
@@ -161,7 +247,7 @@ namespace hpx {
         struct ignore_type
         {
             template <typename T>
-            void operator=(T&&) const
+            constexpr void operator=(T&&) const noexcept
             {
             }
         };
@@ -176,7 +262,7 @@ namespace hpx {
 
         // constexpr tuple();
         // Value initializes each element.
-        constexpr HPX_HOST_DEVICE tuple() {}
+        constexpr HPX_HOST_DEVICE tuple() noexcept = default;
 
         // tuple(const tuple& u) = default;
         // Initializes each element of *this with the corresponding element
@@ -185,34 +271,29 @@ namespace hpx {
 
         // tuple(tuple&& u) = default;
         // For all i, initializes the ith element of *this with
-        // std::forward<Ti>(get<i>(u)).
+        // HPX_FORWARD(Ti, get<i>(u)).
         constexpr tuple(tuple&& /*other*/) = default;
 
         // 20.4.2.2, tuple assignment
 
         // tuple& operator=(const tuple& u);
         // Assigns each element of u to the corresponding element of *this.
-        HPX_HOST_DEVICE tuple& operator=(tuple const& /*other*/) noexcept
-        {
-            return *this;
-        }
+        HPX_HOST_DEVICE tuple& operator=(
+            tuple const& /*other*/) noexcept = default;
 
         // tuple& operator=(tuple&& u) noexcept(see below );
-        // For all i, assigns std::forward<Ti>(get<i>(u)) to get<i>(*this).
-        HPX_HOST_DEVICE tuple& operator=(tuple&& /*other*/) noexcept
-        {
-            return *this;
-        }
+        // For all i, assigns HPX_FORWARD(Ti, get<i>(u)) to get<i>(*this).
+        HPX_HOST_DEVICE tuple& operator=(tuple&& /*other*/) noexcept = default;
 
         // 20.4.2.3, tuple swap
 
         // void swap(tuple& rhs) noexcept(see below);
         // Calls swap for each element in *this and its corresponding element
         // in rhs.
-        HPX_HOST_DEVICE void swap(tuple& /*other*/) noexcept {}
+        HPX_HOST_DEVICE constexpr void swap(tuple& /*other*/) noexcept {}
 
 #if defined(HPX_DATASTRUCTURES_HAVE_ADAPT_STD_TUPLE)
-        operator std::tuple<>() const
+        constexpr operator std::tuple<>() const noexcept
         {
             return std::tuple<>();
         }
@@ -220,9 +301,9 @@ namespace hpx {
     };
 
     template <typename... Ts>
-    class tuple
+    class tuple    //-V690
     {
-        using index_pack = typename util::make_index_pack<sizeof...(Ts)>::type;
+        using index_pack = util::make_index_pack_t<sizeof...(Ts)>;
 
     public:
         // 20.4.2.1, tuple construction
@@ -230,9 +311,8 @@ namespace hpx {
         // constexpr tuple();
         // Value initializes each element.
         template <typename Dependent = void,
-            typename Enable = typename std::enable_if<
-                util::all_of<std::is_constructible<Ts>...>::value,
-                Dependent>::type>
+            typename Enable = std::enable_if_t<
+                util::all_of_v<std::is_constructible<Ts>...>, Dependent>>
         constexpr HPX_HOST_DEVICE tuple()
           : _members()
         {
@@ -249,19 +329,19 @@ namespace hpx {
         // template <class... UTypes>
         // explicit constexpr tuple(UTypes&&... u);
         // Initializes the elements in the tuple with the corresponding value
-        // in std::forward<UTypes>(u).
+        // in HPX_FORWARD(UTypes, u).
         // This constructor shall not participate in overload resolution
         // unless each type in UTypes is implicitly convertible to its
         // corresponding type in Types.
         template <typename U, typename... Us,
-            typename Enable = typename std::enable_if<
-                !std::is_same<tuple, typename std::decay<U>::type>::value ||
-                util::pack<Us...>::size != 0>::type,
-            typename EnableCompatible = typename std::enable_if<hpx::detail::
-                    are_tuples_compatible<tuple, tuple<U, Us...>>::value>::type>
+            typename Enable =
+                std::enable_if_t<!std::is_same_v<tuple, std::decay_t<U>> ||
+                    util::pack<Us...>::size != 0>,
+            typename EnableCompatible = std::enable_if_t<hpx::detail::
+                    are_tuples_compatible<tuple, tuple<U, Us...>>::value>>
         explicit constexpr HPX_HOST_DEVICE tuple(U&& v, Us&&... vs)
-          : _members(std::piecewise_construct, std::forward<U>(v),
-                std::forward<Us>(vs)...)
+          : _members(std::piecewise_construct, HPX_FORWARD(U, v),
+                HPX_FORWARD(Us, vs)...)
         {
         }
 
@@ -273,7 +353,7 @@ namespace hpx {
 
         // tuple(tuple&& u) = default;
         // For all i, initializes the ith element of *this with
-        // std::forward<Ti>(get<i>(u)).
+        // HPX_FORWARD(Ti, get<i>(u)).
         tuple(tuple&&) = default;
 #else
         // tuple(const tuple& u) = default;
@@ -286,71 +366,90 @@ namespace hpx {
 
         // tuple(tuple&& u) = default;
         // For all i, initializes the ith element of *this with
-        // std::forward<Ti>(get<i>(u)).
+        // HPX_FORWARD(Ti, get<i>(u)).
         constexpr HPX_HOST_DEVICE tuple(tuple&& other)
-          : _members(std::move(other._members))
+          : _members(HPX_MOVE(other._members))
         {
         }
 #endif
 
     private:
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26800)    //  Use of a moved from object: '(*other)'
+#endif
+
         template <std::size_t... Is, typename UTuple>
         constexpr HPX_HOST_DEVICE tuple(util::index_pack<Is...>, UTuple&& other)
           : _members(std::piecewise_construct,
-                hpx::get<Is>(std::forward<UTuple>(other))...)
+                hpx::get<Is>(HPX_FORWARD(UTuple, other))...)
         {
         }
 
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
+
     public:
         // template <class... UTypes> constexpr tuple(const tuple<UTypes...>& u);
         // template <class... UTypes> constexpr tuple(tuple<UTypes...>&& u);
         // For all i, initializes the ith element of *this with
-        // get<i>(std::forward<U>(u)).
+        // get<i>(HPX_FORWARD(U, u)).
         // This constructor shall not participate in overload resolution
         // unless each type in UTypes is implicitly convertible to its
         // corresponding type in Types
         template <typename UTuple,
-            typename Enable = typename std::enable_if<!std::is_same<tuple,
-                typename std::decay<UTuple>::type>::value>::type,
-            typename EnableCompatible = typename std::enable_if<
-                hpx::detail::are_tuples_compatible<tuple, UTuple>::value>::type>
+            typename Enable =
+                std::enable_if_t<!std::is_same_v<tuple, std::decay_t<UTuple>>>,
+            typename EnableCompatible = std::enable_if_t<
+                hpx::detail::are_tuples_compatible<tuple, UTuple>::value>>
+        // NOLINTNEXTLINE(bugprone-forwarding-reference-overload)
         constexpr HPX_HOST_DEVICE tuple(UTuple&& other)
-          : tuple(index_pack{}, std::forward<UTuple>(other))
+          : tuple(index_pack{}, HPX_FORWARD(UTuple, other))
         {
         }
 
         // 20.4.2.2, tuple assignment
     private:
+        template <std::size_t I>
+        HPX_HOST_DEVICE void assign_one_(tuple const& other)
+        {
+            _members.template get<I>() = other._members.template get<I>();
+        }
+
         template <std::size_t... Is>
         HPX_HOST_DEVICE void assign_(
             util::index_pack<Is...>, tuple const& other)
         {
-            int const _sequencer[] = {
-                ((_members.template get<Is>() =
-                         other._members.template get<Is>()),
-                    0)...};
-            (void) _sequencer;
+            (assign_one_<Is>(other), ...);
+        }
+
+        template <std::size_t I>
+        HPX_HOST_DEVICE void assign_one_(tuple&& other) noexcept
+        {
+            _members.template get<I>() =
+                HPX_MOVE(other)._members.template get<I>();
         }
 
         template <std::size_t... Is>
-        HPX_HOST_DEVICE void assign_(util::index_pack<Is...>, tuple&& other)
+        HPX_HOST_DEVICE void assign_(
+            util::index_pack<Is...>, tuple&& other) noexcept
         {
-            int const _sequencer[] = {
-                ((_members.template get<Is>() =
-                         std::move(other._members).template get<Is>()),
-                    0)...};
-            (void) _sequencer;
+            (assign_one_<Is>(HPX_MOVE(other)), ...);
+        }
+
+        template <std::size_t I, typename UTuple>
+        HPX_HOST_DEVICE void assign_one_other_(UTuple&& other)
+        {
+            _members.template get<I>() =
+                // NOLINTNEXTLINE(bugprone-signed-char-misuse)
+                hpx::get<I>(HPX_FORWARD(UTuple, other));
         }
 
         template <std::size_t... Is, typename UTuple>
         HPX_HOST_DEVICE void assign_(util::index_pack<Is...>, UTuple&& other)
         {
-            int const _sequencer[] = {
-                ((_members.template get<Is>() =
-                         // NOLINTNEXTLINE(bugprone-signed-char-misuse)
-                     hpx::get<Is>(std::forward<UTuple>(other))),
-                    0)...};
-            (void) _sequencer;
+            (assign_one_other_<Is>(HPX_FORWARD(UTuple, other)), ...);
         }
 
     public:
@@ -363,20 +462,20 @@ namespace hpx {
         }
 
         // tuple& operator=(tuple&& u) noexcept(see below);
-        // For all i, assigns std::forward<Ti>(get<i>(u)) to get<i>(*this).
-        HPX_HOST_DEVICE tuple& operator=(tuple&& other)
+        // For all i, assigns HPX_FORWARD(Ti, get<i>(u)) to get<i>(*this).
+        HPX_HOST_DEVICE tuple& operator=(tuple&& other) noexcept
         {
-            assign_(index_pack{}, std::move(other));
+            assign_(index_pack{}, HPX_MOVE(other));
             return *this;
         }
 
         // template <class... UTypes> tuple& operator=(const tuple<UTypes...>& u);
         // template <class... UTypes> tuple& operator=(tuple<UTypes...>&& u);
-        // For all i, assigns get<i>(std::forward<U>(u)) to get<i>(*this).
+        // For all i, assigns get<i>(HPX_FORWARD(U, u)) to get<i>(*this).
         template <typename UTuple>
         HPX_HOST_DEVICE tuple& operator=(UTuple&& other)
         {
-            assign_(index_pack{}, std::forward<UTuple>(other));
+            assign_(index_pack{}, HPX_FORWARD(UTuple, other));
             return *this;
         }
 
@@ -386,17 +485,16 @@ namespace hpx {
         HPX_HOST_DEVICE void swap_(util::index_pack<Is...>, tuple& other)
         {
             using std::swap;
-            int const _sequencer[] = {((swap(_members.template get<Is>(),
-                                           other._members.template get<Is>())),
-                0)...};
-            (void) _sequencer;
+            (swap(_members.template get<Is>(),
+                 other._members.template get<Is>()),
+                ...);
         }
 
     public:
         // void swap(tuple& rhs) noexcept(see below );
         // Calls swap for each element in *this and its corresponding element
         // in rhs.
-        HPX_HOST_DEVICE void swap(tuple& other)
+        HPX_HOST_DEVICE void swap(tuple& other) noexcept
         {
             swap_(index_pack{}, other);
         }
@@ -419,7 +517,7 @@ namespace hpx {
         template <std::size_t... Is, typename UTuple>
         std::tuple<Ts...> make_tuple_(util::index_pack<Is...>, UTuple&& t)
         {
-            return std::make_tuple(hpx::get<Is>(std::forward<UTuple>(t))...);
+            return std::make_tuple(hpx::get<Is>(HPX_FORWARD(UTuple, t))...);
         }
 
     public:
@@ -436,15 +534,15 @@ namespace hpx {
         }
 
         HPX_HOST_DEVICE
-        operator std::tuple<Ts...>() &&
+        operator std::tuple<Ts...>() && noexcept
         {
-            return make_tuple_(index_pack{}, std::move(*this));
+            return make_tuple_(index_pack{}, HPX_MOVE(*this));
         }
 
         HPX_HOST_DEVICE
-        operator std::tuple<Ts...>() const&&
+        operator std::tuple<Ts...>() const&& noexcept
         {
-            return make_tuple_(index_pack{}, std::move(*this));
+            return make_tuple_(index_pack{}, HPX_MOVE(*this));
         }
 #endif
 
@@ -456,22 +554,22 @@ namespace hpx {
 
     // template <class Tuple>
     // class tuple_size
-    template <class T>
+    template <typename T>
     struct tuple_size
     {
     };
 
-    template <class T>
-    struct tuple_size<const T> : tuple_size<T>
+    template <typename T>
+    struct tuple_size<T const> : tuple_size<T>
     {
     };
 
-    template <class T>
-    struct tuple_size<volatile T> : tuple_size<T>
+    template <typename T>
+    struct tuple_size<T volatile> : tuple_size<T>
     {
     };
 
-    template <class T>
+    template <typename T>
     struct tuple_size<const volatile T> : tuple_size<T>
     {
     };
@@ -501,6 +599,9 @@ namespace hpx {
     };
 #endif
 
+    template <typename T>
+    inline constexpr std::size_t tuple_size_v = tuple_size<T>::value;
+
     // template <size_t I, class Tuple>
     // class tuple_element
     template <std::size_t I, typename T, typename Enable>
@@ -510,21 +611,21 @@ namespace hpx {
 
     template <std::size_t I, typename T>
     struct tuple_element<I, const T,
-        typename util::always_void<typename tuple_element<I, T>::type>::type>
+        std::void_t<typename tuple_element<I, T>::type>>
       : std::add_const<typename tuple_element<I, T>::type>
     {
     };
 
     template <std::size_t I, typename T>
     struct tuple_element<I, volatile T,
-        typename util::always_void<typename tuple_element<I, T>::type>::type>
+        std::void_t<typename tuple_element<I, T>::type>>
       : std::add_volatile<typename tuple_element<I, T>::type>
     {
     };
 
     template <std::size_t I, typename T>
     struct tuple_element<I, const volatile T,
-        typename util::always_void<typename tuple_element<I, T>::type>::type>
+        std::void_t<typename tuple_element<I, T>::type>>
       : std::add_cv<typename tuple_element<I, T>::type>
     {
     };
@@ -612,7 +713,7 @@ namespace hpx {
     template <std::size_t I, typename... Ts>
     struct tuple_element<I, std::tuple<Ts...>>
     {
-        using type = typename std::tuple_element<I, std::tuple<Ts...>>::type;
+        using type = std::tuple_element_t<I, std::tuple<Ts...>>;
 
         static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE type& get(
             std::tuple<Ts...>& tuple) noexcept
@@ -629,17 +730,20 @@ namespace hpx {
         static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE type&& get(
             std::tuple<Ts...>&& tuple) noexcept
         {
-            return std::get<I>(std::move(tuple));
+            return std::get<I>(HPX_MOVE(tuple));
         }
 
         static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE type const&& get(
             std::tuple<Ts...> const&& tuple) noexcept
         {
-            return std::get<I>(std::move(tuple));
+            return std::get<I>(HPX_MOVE(tuple));
         }
     };
 #endif
 
+    template <std::size_t I, typename T>
+    using tuple_element_t = typename tuple_element<I, T>::type;
+
     // 20.4.2.6, element access
     namespace adl_barrier {
 
@@ -728,17 +832,15 @@ namespace hpx {
     }    // namespace std_adl_barrier
 
     // 20.4.2.4, tuple creation functions
-    hpx::detail::ignore_type const ignore = {};
+    inline constexpr hpx::detail::ignore_type ignore = {};
 
     // template<class... Types>
     // constexpr tuple<VTypes...> make_tuple(Types&&... t);
     template <typename... Ts>
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        tuple<typename util::decay_unwrap<Ts>::type...>
-        make_tuple(Ts&&... vs)
+    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE tuple<util::decay_unwrap_t<Ts>...>
+    make_tuple(Ts&&... ts)
     {
-        return tuple<typename util::decay_unwrap<Ts>::type...>(
-            std::forward<Ts>(vs)...);
+        return tuple<util::decay_unwrap_t<Ts>...>(HPX_FORWARD(Ts, ts)...);
     }
 
     // template<class... Types>
@@ -748,18 +850,19 @@ namespace hpx {
     // references to temporary variables, a program shall ensure that the
     // return value of this function does not outlive any of its arguments.
     template <typename... Ts>
-    HPX_HOST_DEVICE HPX_FORCEINLINE tuple<Ts&&...> forward_as_tuple(
-        Ts&&... vs) noexcept
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr tuple<Ts&&...> forward_as_tuple(
+        Ts&&... ts) noexcept
     {
-        return tuple<Ts&&...>(std::forward<Ts>(vs)...);
+        return tuple<Ts&&...>(HPX_FORWARD(Ts, ts)...);
     }
 
     // template<class... Types>
     // tuple<Types&...> tie(Types&... t) noexcept;
     template <typename... Ts>
-    HPX_HOST_DEVICE HPX_FORCEINLINE tuple<Ts&...> tie(Ts&... vs) noexcept
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr tuple<Ts&...> tie(
+        Ts&... ts) noexcept
     {
-        return tuple<Ts&...>(vs...);
+        return tuple<Ts&...>(ts...);
     }
 
     //template <class... Tuples>
@@ -794,21 +897,21 @@ namespace hpx {
 
         template <std::size_t I, typename Head, typename... Tail>
         struct tuple_cat_element<I, util::pack<Head, Tail...>,
-            typename std::enable_if<(I < tuple_size<Head>::value)>::type>
+            std::enable_if_t<(I < tuple_size<Head>::value)>>
           : tuple_element<I, Head>
         {
             template <typename THead, typename... TTail>
             static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto get(
                 THead&& head, TTail&&... /*tail*/) noexcept
-                -> decltype(hpx::get<I>(std::forward<THead>(head)))
+                -> decltype(hpx::get<I>(HPX_FORWARD(THead, head)))
             {
-                return hpx::get<I>(std::forward<THead>(head));
+                return hpx::get<I>(HPX_FORWARD(THead, head));
             }
         };
 
         template <std::size_t I, typename Head, typename... Tail>
         struct tuple_cat_element<I, util::pack<Head, Tail...>,
-            typename std::enable_if<(I >= tuple_size<Head>::value)>::type>
+            std::enable_if_t<(I >= tuple_size<Head>::value)>>
           : tuple_cat_element<I - tuple_size<Head>::value, util::pack<Tail...>>
         {
             using _members = tuple_cat_element<I - tuple_size<Head>::value,
@@ -817,9 +920,9 @@ namespace hpx {
             template <typename THead, typename... TTail>
             static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto get(
                 THead&& /*head*/, TTail&&... tail) noexcept
-                -> decltype(_members::get(std::forward<TTail>(tail)...))
+                -> decltype(_members::get(HPX_FORWARD(TTail, tail)...))
             {
-                return _members::get(std::forward<TTail>(tail)...);
+                return _members::get(HPX_FORWARD(TTail, tail)...);
             }
         };
 
@@ -845,26 +948,36 @@ namespace hpx {
             Tuples_&&... tuples)
             -> tuple_cat_result_of_t<decltype(is_pack), decltype(tuple_pack)>
         {
+            //  Use of a moved from object: '(*<tuples_0>)'
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26800)
+#endif
+
             return tuple_cat_result_of_t<decltype(is_pack),
                 decltype(tuple_pack)>{
                 tuple_cat_element<Is, util::pack<Tuples...>>::get(
-                    std::forward<Tuples_>(tuples)...)...};
+                    HPX_FORWARD(Tuples_, tuples)...)...};
+
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
         }
     }    // namespace detail
 
     template <typename... Tuples>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto tuple_cat(Tuples&&... tuples)
         -> decltype(hpx::detail::tuple_cat_impl(
-            typename util::make_index_pack<hpx::detail::tuple_cat_size<
-                typename std::decay<Tuples>::type...>::value>::type{},
-            util::pack<typename std::decay<Tuples>::type...>{},
-            std::forward<Tuples>(tuples)...))
+            util::make_index_pack_t<
+                hpx::detail::tuple_cat_size<std::decay_t<Tuples>...>::value>{},
+            util::pack<std::decay_t<Tuples>...>{},
+            HPX_FORWARD(Tuples, tuples)...))
     {
         return hpx::detail::tuple_cat_impl(
-            typename util::make_index_pack<hpx::detail::tuple_cat_size<
-                typename std::decay<Tuples>::type...>::value>::type{},
-            util::pack<typename std::decay<Tuples>::type...>{},
-            std::forward<Tuples>(tuples)...);
+            util::make_index_pack_t<
+                hpx::detail::tuple_cat_size<std::decay_t<Tuples>...>::value>{},
+            util::pack<std::decay_t<Tuples>...>{},
+            HPX_FORWARD(Tuples, tuples)...);
     }
 
     // 20.4.2.7, relational operators
@@ -876,6 +989,7 @@ namespace hpx {
     // upwards. No comparisons or element accesses are performed after the
     // first equality comparison that evaluates to false.
     namespace detail {
+
         template <std::size_t I, std::size_t Size>
         struct tuple_equal_to
         {
@@ -893,7 +1007,7 @@ namespace hpx {
         {
             template <typename TTuple, typename UTuple>
             static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE bool call(
-                TTuple const&, UTuple const&)
+                TTuple const&, UTuple const&) noexcept
             {
                 return true;
             }
@@ -902,7 +1016,7 @@ namespace hpx {
 
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator==(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return hpx::detail::tuple_equal_to<0, sizeof...(Ts)>::call(t, u);
@@ -913,7 +1027,7 @@ namespace hpx {
     //     (const tuple<TTypes...>& t, const tuple<UTypes...>& u);
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator!=(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return !(t == u);
@@ -927,6 +1041,7 @@ namespace hpx {
     // tuple r is a tuple containing all but the first element of r. For any
     // two zero-length tuples e and f, e < f returns false.
     namespace detail {
+
         template <std::size_t I, std::size_t Size>
         struct tuple_less_than
         {
@@ -945,7 +1060,7 @@ namespace hpx {
         {
             template <typename TTuple, typename UTuple>
             static constexpr HPX_HOST_DEVICE HPX_FORCEINLINE bool call(
-                TTuple const&, UTuple const&)
+                TTuple const&, UTuple const&) noexcept
             {
                 return false;
             }
@@ -954,7 +1069,7 @@ namespace hpx {
 
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator<(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return hpx::detail::tuple_less_than<0, sizeof...(Ts)>::call(t, u);
@@ -965,7 +1080,7 @@ namespace hpx {
     //     (const tuple<TTypes...>& t, const tuple<UTypes...>& u);
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator>(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return u < t;
@@ -976,7 +1091,7 @@ namespace hpx {
     //     (const tuple<TTypes...>& t, const tuple<UTypes...>& u);
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator<=(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return !(u < t);
@@ -987,7 +1102,7 @@ namespace hpx {
     //     (const tuple<TTypes...>& t, const tuple<UTypes...>& u);
     template <typename... Ts, typename... Us>
     constexpr HPX_HOST_DEVICE HPX_FORCEINLINE
-        typename std::enable_if<sizeof...(Ts) == sizeof...(Us), bool>::type
+        std::enable_if_t<sizeof...(Ts) == sizeof...(Us), bool>
         operator>=(tuple<Ts...> const& t, tuple<Us...> const& u)
     {
         return !(t < u);
@@ -1023,68 +1138,6 @@ namespace hpx {
 #endif
 }    // namespace hpx
 
-namespace hpx { namespace util {
-    HPX_DEPRECATED_V(
-        1, 6, "hpx::util::ignore is deprecated. Use hpx::ignore instead.")
-    hpx::detail::ignore_type const ignore = {};
-
-    template <typename... Ts>
-    using tuple HPX_DEPRECATED_V(
-        1, 6, "hpx::util::tuple is deprecated. Use hpx::tuple instead.") =
-        hpx::tuple<Ts...>;
-    template <typename T>
-    using tuple_size HPX_DEPRECATED_V(1, 6,
-        "hpx::util::tuple_size is deprecated. Use hpx::tuple_size instead.") =
-        hpx::tuple_size<T>;
-    template <std::size_t I, typename T>
-    using tuple_element HPX_DEPRECATED_V(1, 6,
-        "hpx::util::tuple_element is deprecated. Use hpx::tuple_size "
-        "instead.") = hpx::tuple_element<I, T>;
-
-    template <typename... Ts>
-    HPX_DEPRECATED_V(1, 6,
-        "hpx::util::make_tuple is deprecated. Use hpx::make_tuple instead.")
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto make_tuple(Ts&&... vs)
-    {
-        return hpx::make_tuple(std::forward<Ts>(vs)...);
-    }
-
-    template <typename... Ts>
-    HPX_DEPRECATED_V(
-        1, 6, "hpx::util::tie is deprecated. Use hpx::tie instead.")
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto tie(Ts&&... vs)
-    {
-        return hpx::tie(std::forward<Ts>(vs)...);
-    }
-
-    template <typename... Ts>
-    HPX_DEPRECATED_V(1, 6,
-        "hpx::util::forward_as_tuple is deprecated. Use hpx::forward_as_tuple "
-        "instead.")
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto forward_as_tuple(Ts&&... vs)
-    {
-        return hpx::forward_as_tuple(std::forward<Ts>(vs)...);
-    }
-
-    template <typename... Ts>
-    HPX_DEPRECATED_V(1, 6,
-        "hpx::util::tuple_cat is deprecated. Use hpx::tuple_cat "
-        "instead.")
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto tuple_cat(Ts&&... vs)
-    {
-        return hpx::tuple_cat(std::forward<Ts>(vs)...);
-    }
-
-    template <std::size_t I, typename Tuple>
-    HPX_DEPRECATED_V(1, 6,
-        "hpx::util::get is deprecated. Use hpx::get "
-        "instead.")
-    constexpr HPX_HOST_DEVICE HPX_FORCEINLINE auto get(Tuple&& tuple)
-    {
-        return hpx::get<I>(std::forward<Tuple>(tuple));
-    }
-}}    // namespace hpx::util
-
 #if defined(HPX_MSVC_WARNING_PRAGMA)
 #pragma warning(pop)
 #endif
@@ -1092,3 +1145,5 @@ namespace hpx { namespace util {
 #if defined(__clang__)
 #pragma clang diagnostic pop
 #endif
+
+#endif    //DOXYGEN
