@@ -8,11 +8,14 @@
 
 #include <hpx/config.hpp>
 
-#if defined(HPX_HAVE_CXX17_STD_VARIANT) && defined(HPX_HAVE_CXX17_COPY_ELISION)
+#if defined(HPX_HAVE_CXX17_COPY_ELISION)
+#include <hpx/datastructures/variant_helper.hpp>
 #include <variant>
 
 namespace hpx {
 
+    using std::get;
+    using std::holds_alternative;
     using std::monostate;
     using std::variant;
     using std::visit;
@@ -24,6 +27,8 @@ namespace hpx {
 
 namespace hpx {
 
+    using hpx::variant_ns::get;
+    using hpx::variant_ns::holds_alternative;
     using hpx::variant_ns::monostate;
     using hpx::variant_ns::variant;
     using hpx::variant_ns::visit;
