@@ -1,4 +1,4 @@
-//  Copyright (c) 2018 Hartmut Kaiser
+//  Copyright (c) 2018-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,11 +7,13 @@
 #include <hpx/datastructures/optional.hpp>
 #include <hpx/modules/testing.hpp>
 
+#include <utility>
+
 int main()
 {
-    hpx::util::optional<int> x;
+    hpx::optional<int> x;
     int y = 42;
-    x = y;
+    x = std::move(y);
 
     HPX_TEST(x.has_value());
     HPX_TEST_EQ(*x, 42);
