@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2012 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //  Copyright (c) 2017      Denis Blank
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -8,10 +8,10 @@
 #pragma once
 
 #include <hpx/config.hpp>
-#include <string>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     /// Tries to break an attached debugger, if not supported a loop is
     /// invoked which gives enough time to attach a debugger manually.
     HPX_CORE_EXPORT void attach_debugger();
-}}    // namespace hpx::util
+}    // namespace hpx::util
