@@ -59,7 +59,7 @@
 
 // ------------------------------------------------------------
 /// \cond NODETAIL
-namespace hpx { namespace debug {
+namespace hpx::debug {
 
     // ------------------------------------------------------------------
     // format as zero padded int
@@ -72,7 +72,7 @@ namespace hpx { namespace debug {
         template <int N, typename T>
         struct dec
         {
-            constexpr dec(T const& v)
+            explicit constexpr dec(T const& v) noexcept
               : data_(v)
             {
             }
@@ -89,7 +89,7 @@ namespace hpx { namespace debug {
     }    // namespace detail
 
     template <int N = 2, typename T>
-    constexpr detail::dec<N, T> dec(T const& v)
+    [[nodiscard]] constexpr detail::dec<N, T> dec(T const& v) noexcept
     {
         return detail::dec<N, T>(v);
     }
@@ -99,8 +99,8 @@ namespace hpx { namespace debug {
     // ------------------------------------------------------------------
     struct ptr
     {
-        HPX_CORE_EXPORT ptr(void const* v);
-        HPX_CORE_EXPORT ptr(std::uintptr_t v);
+        HPX_CORE_EXPORT explicit ptr(void const* v) noexcept;
+        HPX_CORE_EXPORT explicit ptr(std::uintptr_t v) noexcept;
 
         void const* data_;
 
@@ -120,10 +120,9 @@ namespace hpx { namespace debug {
         struct hex;
 
         template <int N, typename T>
-        struct hex<N, T,
-            typename std::enable_if<!std::is_pointer<T>::value>::type>
+        struct hex<N, T, std::enable_if_t<!std::is_pointer_v<T>>>
         {
-            constexpr hex(T const& v)
+            explicit constexpr hex(T const& v) noexcept
               : data_(v)
             {
             }
@@ -138,14 +137,12 @@ namespace hpx { namespace debug {
             }
         };
 
-        template <typename Int>
-        HPX_CORE_EXPORT void print_ptr(std::ostream& os, Int v, int n);
+        HPX_CORE_EXPORT void print_ptr(std::ostream& os, void const* v, int n);
 
         template <int N, typename T>
-        struct hex<N, T,
-            typename std::enable_if<std::is_pointer<T>::value>::type>
+        struct hex<N, T, std::enable_if_t<std::is_pointer_v<T>>>
         {
-            constexpr hex(T const& v)
+            explicit constexpr hex(T const& v) noexcept
               : data_(v)
             {
             }
@@ -162,7 +159,7 @@ namespace hpx { namespace debug {
     }    // namespace detail
 
     template <int N = 4, typename T>
-    constexpr detail::hex<N, T> hex(T const& v)
+    [[nodiscard]] constexpr detail::hex<N, T> hex(T const& v) noexcept
     {
         return detail::hex<N, T>(v);
     }
@@ -178,7 +175,7 @@ namespace hpx { namespace debug {
         template <int N = 8, typename T = int>
         struct bin
         {
-            constexpr bin(T const& v)
+            explicit constexpr bin(T const& v) noexcept
               : data_(v)
             {
             }
@@ -195,7 +192,7 @@ namespace hpx { namespace debug {
     }    // namespace detail
 
     template <int N = 8, typename T>
-    constexpr detail::bin<N, T> bin(T const& v)
+    [[nodiscard]] constexpr detail::bin<N, T> bin(T const& v) noexcept
     {
         return detail::bin<N, T>(v);
     }
@@ -211,7 +208,7 @@ namespace hpx { namespace debug {
     template <int N = 20>
     struct str
     {
-        constexpr str(char const* v)
+        explicit constexpr str(char const* v) noexcept
           : data_(v)
         {
         }
@@ -230,8 +227,8 @@ namespace hpx { namespace debug {
     // ------------------------------------------------------------------
     struct ipaddr
     {
-        HPX_CORE_EXPORT ipaddr(void const* a);
-        HPX_CORE_EXPORT ipaddr(std::uint32_t a);
+        HPX_CORE_EXPORT explicit ipaddr(void const* a) noexcept;
+        HPX_CORE_EXPORT explicit ipaddr(std::uint32_t a) noexcept;
 
         std::uint8_t const* data_;
         std::uint32_t const ipdata_;
@@ -244,17 +241,19 @@ namespace hpx { namespace debug {
     // helper class for printing time since start
     // ------------------------------------------------------------------
     namespace detail {
+
         struct current_time_print_helper
         {
             HPX_CORE_EXPORT friend std::ostream& operator<<(
-                std::ostream& os, current_time_print_helper const&);
+                std::ostream& os, current_time_print_helper);
         };
     }    // namespace detail
 
     // ------------------------------------------------------------------
     // helper function for printing CRC32
     // ------------------------------------------------------------------
-    constexpr inline std::uint32_t crc32(void const*, std::size_t)
+    [[nodiscard]] constexpr std::uint32_t crc32(
+        void const*, std::size_t) noexcept
     {
         return 0;
     }
@@ -267,7 +266,7 @@ namespace hpx { namespace debug {
     struct mem_crc32
     {
         HPX_CORE_EXPORT mem_crc32(
-            void const* a, std::size_t len, char const* txt);
+            void const* a, std::size_t len, char const* txt) noexcept;
 
         std::uint64_t const* addr_;
         std::size_t const len_;
@@ -279,7 +278,6 @@ namespace hpx { namespace debug {
 
     namespace detail {
 
-#if defined(HPX_HAVE_CXX17_FOLD_EXPRESSIONS)
         template <typename TupleType, std::size_t... I>
         void tuple_print(
             std::ostream& os, TupleType const& t, std::index_sequence<I...>)
@@ -288,40 +286,10 @@ namespace hpx { namespace debug {
         }
 
         template <typename... Args>
-        void tuple_print(std::ostream& os, const std::tuple<Args...>& t)
+        void tuple_print(std::ostream& os, std::tuple<Args...> const& t)
         {
             tuple_print(os, t, std::make_index_sequence<sizeof...(Args)>());
         }
-#else
-        // C++14 version
-        // helper function to print a tuple of any size
-        template <typename TupleType, std::size_t... I>
-        void tuple_print(
-            std::ostream& os, TupleType const& t, std::index_sequence<I...>)
-        {
-            using expander = int[];
-            (void) expander{
-                0, (void(os << (I == 0 ? "" : " ") << std::get<I>(t)), 0)...};
-        }
-
-        // print a tuple of any size - forwards to helper with index
-        template <typename... Args>
-        void tuple_print(std::ostream& os, std::tuple<Args...> const& t)
-        {
-            detail::tuple_print(
-                os, t, std::make_index_sequence<sizeof...(Args)>{});
-        }
-
-        // print variadic list of args
-        template <typename Arg, typename... Args>
-        void variadic_print(
-            std::ostream& os, Arg const& arg, Args const&... args)
-        {
-            os << arg;
-            using expander = int[];
-            (void) expander{0, (void(os << ' ' << args), 0)...};
-        }
-#endif
     }    // namespace detail
 
     namespace detail {
@@ -331,11 +299,11 @@ namespace hpx { namespace debug {
         // ------------------------------------------------------------------
         struct hostname_print_helper
         {
-            HPX_CORE_EXPORT char const* get_hostname() const;
-            HPX_CORE_EXPORT int guess_rank() const;
+            [[nodiscard]] HPX_CORE_EXPORT char const* get_hostname() const;
+            [[nodiscard]] HPX_CORE_EXPORT int guess_rank() const;
 
             HPX_CORE_EXPORT friend std::ostream& operator<<(
-                std::ostream& os, hostname_print_helper const& h);
+                std::ostream& os, hostname_print_helper h);
         };
 
         ///////////////////////////////////////////////////////////////////////
@@ -343,7 +311,6 @@ namespace hpx { namespace debug {
         HPX_CORE_EXPORT void generate_prefix(std::ostream& os);
 
         ///////////////////////////////////////////////////////////////////////
-#if defined(HPX_HAVE_CXX17_FOLD_EXPRESSIONS)
         template <typename... Args>
         void display(char const* prefix, Args const&... args)
         {
@@ -356,20 +323,6 @@ namespace hpx { namespace debug {
             tempstream << std::endl;
             std::cout << tempstream.str();
         }
-#else
-        template <typename... Args>
-        void display(char const* prefix, Args const&... args)
-        {
-            // using a temp stream object with a single copy to cout at the end
-            // prevents multiple threads from injecting overlapping text
-            std::stringstream tempstream;
-            tempstream << prefix;
-            generate_prefix(tempstream);
-            variadic_print(tempstream, args...);
-            tempstream << std::endl;
-            std::cout << tempstream.str();
-        }
-#endif
 
         template <typename... Args>
         void debug(Args const&... args)
@@ -417,7 +370,7 @@ namespace hpx { namespace debug {
         std::string buffered_msg;
 
         //
-        scoped_var(char const* p, Args const&... args)
+        explicit scoped_var(char const* p, Args const&... args)
           : prefix_(p)
           , message_(args...)
         {
@@ -442,7 +395,7 @@ namespace hpx { namespace debug {
         double const delay_;
         std::tuple<Args...> const message_;
         //
-        timed_var(double const& delay, Args const&... args)
+        explicit timed_var(double delay, Args const&... args)
           : time_start_(std::chrono::steady_clock::now())
           , delay_(delay)
           , message_(args...)
@@ -451,7 +404,7 @@ namespace hpx { namespace debug {
 
         bool elapsed(std::chrono::steady_clock::time_point const& now) const
         {
-            double elapsed_ =
+            double const elapsed_ =
                 std::chrono::duration_cast<std::chrono::duration<double>>(
                     now - time_start_)
                     .count();
@@ -480,85 +433,91 @@ namespace hpx { namespace debug {
     template <>
     struct enable_print<false>
     {
-        constexpr enable_print(const char*) {}
+        explicit constexpr enable_print(char const*) noexcept {}
 
-        constexpr bool is_enabled() const
+        [[nodiscard]] static constexpr bool is_enabled() noexcept
         {
             return false;
         }
 
         template <typename... Args>
-        constexpr void debug(Args const&...) const
+        static constexpr void debug(Args const&...) noexcept
         {
         }
 
         template <typename... Args>
-        constexpr void warning(Args const&...) const
+        static constexpr void warning(Args const&...) noexcept
         {
         }
 
         template <typename... Args>
-        constexpr void trace(Args const&...) const
+        static constexpr void trace(Args const&...) noexcept
         {
         }
 
         template <typename... Args>
-        constexpr void error(Args const&...) const
+        static constexpr void error(Args const&...) noexcept
         {
         }
 
         template <typename... Args>
-        constexpr void timed(Args const&...) const
+        static constexpr void timed(Args const&...) noexcept
         {
         }
 
         template <typename T>
-        constexpr void array(std::string const&, std::vector<T> const&) const
+        static constexpr void array(
+            std::string const&, std::vector<T> const&) noexcept
         {
         }
 
         template <typename T, std::size_t N>
-        constexpr void array(std::string const&, std::array<T, N> const&) const
+        static constexpr void array(
+            std::string const&, std::array<T, N> const&) noexcept
         {
         }
 
         template <typename T>
-        constexpr void array(std::string const&, T const*, std::size_t) const
+        static constexpr void array(
+            std::string const&, T const*, std::size_t) noexcept
         {
         }
 
         template <typename... Args>
-        constexpr bool scope(Args const&...)
+        static constexpr bool scope(Args const&...) noexcept
         {
             return true;
         }
 
         template <typename T, typename... Args>
-        constexpr bool declare_variable(Args const&...) const
+        [[nodiscard]] static constexpr bool declare_variable(
+            Args const&...) noexcept
         {
             return true;
         }
 
         template <typename T, typename V>
-        constexpr void set(T&, V const&)
+        static constexpr void set(T&, V const&) noexcept
         {
         }
 
         // @todo, return void so that timers have zero footprint when disabled
         template <typename... Args>
-        constexpr int make_timer(const double, Args const&...) const
+        [[nodiscard]] static constexpr int make_timer(
+            double const, Args const&...) noexcept
         {
             return 0;
         }
 
         template <typename Expr>
-        constexpr bool eval(Expr const&)
+        [[nodiscard]] static constexpr bool eval(Expr const&) noexcept
         {
             return true;
         }
     };
 
     namespace detail {
+
         template <typename T>
         HPX_CORE_EXPORT void print_array(
             std::string const& name, T const* data, std::size_t size);
@@ -572,17 +531,17 @@ namespace hpx { namespace debug {
         char const* prefix_;
 
     public:
-        constexpr enable_print()
+        constexpr enable_print() noexcept
           : prefix_("")
         {
         }
 
-        constexpr enable_print(const char* p)
+        explicit constexpr enable_print(char const* p) noexcept
           : prefix_(p)
         {
         }
 
-        constexpr bool is_enabled() const
+        [[nodiscard]] static constexpr bool is_enabled() noexcept
         {
             return true;
         }
@@ -612,7 +571,7 @@ namespace hpx { namespace debug {
         }
 
         template <typename... Args>
-        scoped_var<Args...> scope(Args const&... args)
+        [[nodiscard]] scoped_var<Args...> scope(Args const&... args)
         {
             return scoped_var<Args...>(prefix_, args...);
         }
@@ -628,49 +587,48 @@ namespace hpx { namespace debug {
         }
 
         template <typename T>
-        void array(std::string const& name, std::vector<T> const& v) const
+        static void array(std::string const& name, std::vector<T> const& v)
         {
             detail::print_array(name, v.data(), v.size());
         }
 
         template <typename T, std::size_t N>
-        void array(std::string const& name, std::array<T, N> const& v) const
+        static void array(std::string const& name, std::array<T, N> const& v)
         {
             detail::print_array(name, v.data(), N);
         }
 
         template <typename T>
-        void array(
-            std::string const& name, T const* data, std::size_t size) const
+        static void array(
+            std::string const& name, T const* data, std::size_t size)
         {
             detail::print_array(name, data, size);
         }
 
         template <typename T, typename... Args>
-        T declare_variable(Args const&... args) const
+        [[nodiscard]] static T declare_variable(Args const&... args)
         {
             return T(args...);
         }
 
         template <typename T, typename V>
-        void set(T& var, V const& val)
+        static void set(T& var, V const& val)
         {
             var = val;
         }
 
         template <typename... Args>
-        timed_var<Args...> make_timer(
-            const double delay, const Args... args) const
+        [[nodiscard]] static timed_var<Args...> make_timer(
+            double const delay, Args const... args)
         {
             return timed_var<Args...>(delay, args...);
         }
 
         template <typename Expr>
-        auto eval(Expr const& e)
+        [[nodiscard]] static auto eval(Expr const& e)
         {
             return e();
         }
     };
-
-}}    // namespace hpx::debug
+}    // namespace hpx::debug
 /// \endcond
