@@ -16,16 +16,19 @@
 
 #if defined(HPX_WINDOWS)
 #include <Windows.h>
-#endif    // HPX_WINDOWS
+#endif
 
 #if defined(_POSIX_VERSION)
 #include <asio/ip/host_name.hpp>
 #endif
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     void attach_debugger()
     {
-#if defined(_POSIX_VERSION)
+#if defined(HPX_WINDOWS)
+        DebugBreak();
+#elif defined(_POSIX_VERSION) && defined(HPX_HAVE_UNISTD_H)
         volatile int i = 0;
         std::cerr << "PID: " << getpid() << " on " << asio::ip::host_name()
                   << " ready for attaching debugger. Once attached set i = 1 "
@@ -35,8 +38,6 @@ namespace hpx { namespace util {
         {
             sleep(1);
         }
-#elif defined(HPX_WINDOWS)
-        DebugBreak();
 #endif
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
