@@ -6,26 +6,28 @@
 
 #pragma once
 
+#include <hpx/coroutines/thread_enums.hpp>
 #include <hpx/execution_base/context_base.hpp>
 #include <hpx/timing/steady_clock.hpp>
 
 #include <cstddef>
 #include <string>
 
-namespace hpx { namespace execution_base {
+namespace hpx::execution_base {
 
     struct agent_base
     {
         virtual ~agent_base() = default;
 
-        virtual std::string description() const = 0;
+        [[nodiscard]] virtual std::string description() const = 0;
 
-        virtual context_base const& context() const = 0;
+        [[nodiscard]] virtual context_base const& context() const noexcept = 0;
 
         virtual void yield(char const* desc) = 0;
         virtual void yield_k(std::size_t k, char const* desc) = 0;
         virtual void suspend(char const* desc) = 0;
-        virtual void resume(char const* desc) = 0;
+        virtual void resume(
+            hpx::threads::thread_priority priority, char const* desc) = 0;
         virtual void abort(char const* desc) = 0;
         virtual void sleep_for(
             hpx::chrono::steady_duration const& sleep_duration,
@@ -34,4 +36,4 @@ namespace hpx { namespace execution_base {
             hpx::chrono::steady_time_point const& sleep_time,
             char const* desc) = 0;
     };
-}}    // namespace hpx::execution_base
+}    // namespace hpx::execution_base
