@@ -7,16 +7,13 @@
 #pragma once
 
 #include <hpx/execution_base/resource_base.hpp>
-#include <hpx/timing/steady_clock.hpp>
 
-#include <cstdint>
-
-namespace hpx { namespace execution_base {
+namespace hpx::execution_base {
 
     struct context_base
     {
         virtual ~context_base() = default;
 
-        virtual resource_base const& resource() const = 0;
+        virtual resource_base const& resource() const noexcept = 0;
     };
-}}    // namespace hpx::execution_base
+}    // namespace hpx::execution_base
