@@ -11,11 +11,16 @@
 #include <cstddef>
 
 #ifdef HPX_HAVE_SPINLOCK_DEADLOCK_DETECTION
-namespace hpx { namespace util { namespace detail {
-    HPX_CORE_EXPORT void set_spinlock_break_on_deadlock_enabled(bool enabled);
-    HPX_CORE_EXPORT bool get_spinlock_break_on_deadlock_enabled();
+
+namespace hpx::util::detail {
+
+    HPX_CORE_EXPORT void set_spinlock_break_on_deadlock_enabled(
+        bool enabled) noexcept;
+    HPX_CORE_EXPORT bool get_spinlock_break_on_deadlock_enabled() noexcept;
     HPX_CORE_EXPORT void set_spinlock_deadlock_detection_limit(
-        std::size_t limit);
-    HPX_CORE_EXPORT std::size_t get_spinlock_deadlock_detection_limit();
-}}}    // namespace hpx::util::detail
+        std::size_t limit) noexcept;
+    HPX_CORE_EXPORT std::size_t
+    get_spinlock_deadlock_detection_limit() noexcept;
+}    // namespace hpx::util::detail
+
 #endif
