@@ -1,4 +1,4 @@
-//  Copyright (c) 2017-2021 Hartmut Kaiser
+//  Copyright (c) 2017-2023 Hartmut Kaiser
 //  Copyright (c) 2017 Google
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -10,14 +10,14 @@
 #include <hpx/config.hpp>
 #include <hpx/concepts/concepts.hpp>
 #include <hpx/execution_base/traits/is_executor.hpp>
-#include <hpx/functional/tag_fallback_dispatch.hpp>
-#include <hpx/iterator_support/counting_iterator.hpp>
-#include <hpx/iterator_support/iterator_range.hpp>
+#include <hpx/functional/detail/tag_fallback_invoke.hpp>
+#include <hpx/iterator_support/counting_shape.hpp>
 
 #include <type_traits>
 #include <utility>
 
-namespace hpx { namespace execution {
+namespace hpx::execution {
+
     ///////////////////////////////////////////////////////////////////////////
     /// Function invocations executed by a group of sequential execution agents
     /// execute in sequential order.
@@ -45,40 +45,14 @@ namespace hpx { namespace execution {
     struct unsequenced_execution_tag
     {
     };
+}    // namespace hpx::execution
 
-    /// \cond NOINTERNAL
-    struct task_policy_tag
-    {
-        constexpr task_policy_tag() = default;
-    };
-}}    // namespace hpx::execution
-
-namespace hpx { namespace parallel { namespace execution {
-
-    using parallel_execution_tag HPX_DEPRECATED_V(1, 6,
-        "hpx::parallel::execution::parallel_execution_tag is deprecated. Use "
-        "hpx::execution::parallel_execution_tag instead.") =
-        hpx::execution::parallel_execution_tag;
-    using sequenced_execution_tag HPX_DEPRECATED_V(1, 6,
-        "hpx::parallel::execution::sequenced_execution_tag is deprecated. Use "
-        "hpx::execution::sequenced_execution_tag instead.") =
-        hpx::execution::sequenced_execution_tag;
-    using task_policy_tag HPX_DEPRECATED_V(1, 6,
-        "hpx::parallel::execution::task_policy_tag is deprecated. Use "
-        "hpx::execution::task_policy_tag instead.") =
-        hpx::execution::task_policy_tag;
-    using unsequenced_execution_tag HPX_DEPRECATED_V(1, 6,
-        "hpx::parallel::execution::unsequenced_execution_tag is deprecated. "
-        "Use hpx::execution::unsequenced_execution_tag instead.") =
-        hpx::execution::unsequenced_execution_tag;
-}}}    // namespace hpx::parallel::execution
-/// \endcond
-
-namespace hpx { namespace parallel { namespace execution {
+namespace hpx::parallel::execution {
 
     ///////////////////////////////////////////////////////////////////////////
     // Executor customization points
     namespace detail {
+
         /// \cond NOINTERNAL
         template <typename Executor, typename Enable = void>
         struct async_execute_fn_helper;
@@ -101,19 +75,11 @@ namespace hpx { namespace parallel { namespace execution {
         template <typename Executor, typename Enable = void>
         struct bulk_then_execute_fn_helper;
 
-        ///////////////////////////////////////////////////////////////////////
-        template <typename Incrementable>
-        using counting_shape_type = hpx::util::iterator_range<
-            hpx::util::counting_iterator<Incrementable>>;
+        template <typename Executor, typename Enable = void>
+        struct async_invoke_fn_helper;
 
-        template <typename Incrementable>
-        HPX_HOST_DEVICE inline counting_shape_type<Incrementable>
-        make_counting_shape(Incrementable n)
-        {
-            return hpx::util::make_iterator_range(
-                hpx::util::make_counting_iterator(Incrementable(0)),
-                hpx::util::make_counting_iterator(n));
-        }
+        template <typename Executor, typename Enable = void>
+        struct sync_invoke_fn_helper;
         /// \endcond
     }    // namespace detail
 
@@ -136,25 +102,26 @@ namespace hpx { namespace parallel { namespace execution {
     ///
     /// \returns f(ts...)'s result
     ///
-    /// \note This is valid for one way executors only, it will call
-    ///       exec.sync_execute(f, ts...) if it exists.
+    /// \note It will call tag_invoke(sync_execute_t, exec, f, ts...) if it
+    ///       exists. For two-way executors it will invoke asynch_execute_t
+    ///       and wait for the task's completion before returning.
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct sync_execute_t final
-      : hpx::functional::tag_fallback<sync_execute_t>
+    inline constexpr struct sync_execute_t final
+      : hpx::functional::detail::tag_fallback<sync_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value
+                hpx::traits::is_executor_any_v<Executor>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             sync_execute_t, Executor&& exec, F&& f, Ts&&... ts)
         {
             return detail::sync_execute_fn_helper<std::decay_t<Executor>>::call(
-                std::forward<Executor>(exec), std::forward<F>(f),
-                std::forward<Ts>(ts)...);
+                HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                HPX_FORWARD(Ts, ts)...);
         }
     } sync_execute{};
 
@@ -186,22 +153,22 @@ namespace hpx { namespace parallel { namespace execution {
     ///
     /// \returns f(ts...)'s result through a future
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct async_execute_t final
-      : hpx::functional::tag_fallback<async_execute_t>
+    inline constexpr struct async_execute_t final
+      : hpx::functional::detail::tag_fallback<async_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value
+                hpx::traits::is_executor_any_v<Executor>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             async_execute_t, Executor&& exec, F&& f, Ts&&... ts)
         {
             return detail::async_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), std::forward<Ts>(ts)...);
+                std::decay_t<Executor>>::call(HPX_FORWARD(Executor, exec),
+                HPX_FORWARD(F, f), HPX_FORWARD(Ts, ts)...);
         }
     } async_execute{};
 
@@ -225,23 +192,23 @@ namespace hpx { namespace parallel { namespace execution {
     ///       exec.then_execute(f, predecessor, ts...) if it exists) and
     ///       for one way executors (calls predecessor.then(bind(f, ts...))).
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct then_execute_t final
-      : hpx::functional::tag_fallback<then_execute_t>
+    inline constexpr struct then_execute_t final
+      : hpx::functional::detail::tag_fallback<then_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename Future, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value
+                hpx::traits::is_executor_any_v<Executor>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             then_execute_t, Executor&& exec, F&& f, Future&& predecessor,
             Ts&&... ts)
         {
             return detail::then_execute_fn_helper<std::decay_t<Executor>>::call(
-                std::forward<Executor>(exec), std::forward<F>(f),
-                std::forward<Future>(predecessor), std::forward<Ts>(ts)...);
+                HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                HPX_FORWARD(Future, predecessor), HPX_FORWARD(Ts, ts)...);
         }
     } then_execute{};
 
@@ -266,22 +233,22 @@ namespace hpx { namespace parallel { namespace execution {
     ///       returned future), and for non-blocking two way executors
     ///       (calls exec.post(f, ts...) if it exists).
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct post_t final
-      : hpx::functional::tag_fallback<post_t>
+    inline constexpr struct post_t final
+      : hpx::functional::detail::tag_fallback<post_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value
+                hpx::traits::is_executor_any_v<Executor>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             post_t, Executor&& exec, F&& f, Ts&&... ts)
         {
             return detail::post_fn_helper<std::decay_t<Executor>>::call(
-                std::forward<Executor>(exec), std::forward<F>(f),
-                std::forward<Ts>(ts)...);
+                HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                HPX_FORWARD(Ts, ts)...);
         }
     } post{};
 
@@ -324,41 +291,39 @@ namespace hpx { namespace parallel { namespace execution {
     ///       exists; otherwise it executes sync_execute(f, shape, ts...)
     ///       as often as needed.
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct bulk_sync_execute_t final
-      : hpx::functional::tag_fallback<bulk_sync_execute_t>
+    inline constexpr struct bulk_sync_execute_t final
+      : hpx::functional::detail::tag_fallback<bulk_sync_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename Shape, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value&&
-                !std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+               !std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             bulk_sync_execute_t, Executor&& exec, F&& f, Shape const& shape,
             Ts&&... ts)
         {
             return detail::bulk_sync_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), shape, std::forward<Ts>(ts)...);
+                std::decay_t<Executor>>::call(HPX_FORWARD(Executor, exec),
+                HPX_FORWARD(F, f), shape, HPX_FORWARD(Ts, ts)...);
         }
 
         // clang-format off
         template <typename Executor, typename F, typename Shape, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value&&
-                std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+                std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
-            bulk_sync_execute_t, Executor&& exec, F&& f, Shape const& shape,
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
+            bulk_sync_execute_t tag, Executor&& exec, F&& f, Shape const& shape,
             Ts&&... ts)
         {
-            return detail::bulk_sync_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), detail::make_counting_shape(shape),
-                std::forward<Ts>(ts)...);
+            return tag(HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                hpx::util::counting_shape(shape), HPX_FORWARD(Ts, ts)...);
         }
     } bulk_sync_execute{};
 
@@ -395,41 +360,39 @@ namespace hpx { namespace parallel { namespace execution {
     ///       exists; otherwise it executes async_execute(f, shape, ts...)
     ///       as often as needed.
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct bulk_async_execute_t final
-      : hpx::functional::tag_fallback<bulk_async_execute_t>
+    inline constexpr struct bulk_async_execute_t final
+      : hpx::functional::detail::tag_fallback<bulk_async_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename Shape, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value &&
-                !std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+               !std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             bulk_async_execute_t, Executor&& exec, F&& f, Shape const& shape,
             Ts&&... ts)
         {
             return detail::bulk_async_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), shape, std::forward<Ts>(ts)...);
+                std::decay_t<Executor>>::call(HPX_FORWARD(Executor, exec),
+                HPX_FORWARD(F, f), shape, HPX_FORWARD(Ts, ts)...);
         }
 
         // clang-format off
         template <typename Executor, typename F, typename Shape, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value &&
-                std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+                std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
-            bulk_async_execute_t, Executor&& exec, F&& f, Shape const& shape,
-            Ts&&... ts)
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
+            bulk_async_execute_t tag, Executor&& exec, F&& f,
+            Shape const& shape, Ts&&... ts)
         {
-            return detail::bulk_async_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), detail::make_counting_shape(shape),
-                std::forward<Ts>(ts)...);
+            return tag(HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                hpx::util::counting_shape(shape), HPX_FORWARD(Ts, ts)...);
         }
     } bulk_async_execute{};
 
@@ -470,44 +433,124 @@ namespace hpx { namespace parallel { namespace execution {
     ///       async_execute(f, shape, pred.share(), ts...) (if this executor
     ///       is also a TwoWayExecutor) - as often as needed.
     ///
-    HPX_INLINE_CONSTEXPR_VARIABLE struct bulk_then_execute_t final
-      : hpx::functional::tag_fallback<bulk_then_execute_t>
+    inline constexpr struct bulk_then_execute_t final
+      : hpx::functional::detail::tag_fallback<bulk_then_execute_t>
     {
     private:
         // clang-format off
         template <typename Executor, typename F, typename Shape,
             typename Future, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value&&
-                !std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+               !std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
             bulk_then_execute_t, Executor&& exec, F&& f, Shape const& shape,
             Future&& predecessor, Ts&&... ts)
         {
             return detail::bulk_then_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), shape, std::forward<Future>(predecessor),
-                std::forward<Ts>(ts)...);
+                std::decay_t<Executor>>::call(HPX_FORWARD(Executor, exec),
+                HPX_FORWARD(F, f), shape, HPX_FORWARD(Future, predecessor),
+                HPX_FORWARD(Ts, ts)...);
         }
 
         // clang-format off
         template <typename Executor, typename F, typename Shape,
             typename Future, typename... Ts,
             HPX_CONCEPT_REQUIRES_(
-                hpx::traits::is_executor_any<Executor>::value&&
-                std::is_integral<Shape>::value
+                hpx::traits::is_executor_any_v<Executor> &&
+                std::is_integral_v<Shape>
             )>
         // clang-format on
-        friend HPX_FORCEINLINE decltype(auto) tag_fallback_dispatch(
-            bulk_then_execute_t, Executor&& exec, F&& f, Shape const& shape,
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
+            bulk_then_execute_t tag, Executor&& exec, F&& f, Shape const& shape,
             Future&& predecessor, Ts&&... ts)
         {
-            return detail::bulk_then_execute_fn_helper<
-                std::decay_t<Executor>>::call(std::forward<Executor>(exec),
-                std::forward<F>(f), detail::make_counting_shape(shape),
-                std::forward<Future>(predecessor), std::forward<Ts>(ts)...);
+            return tag(HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                hpx::util::counting_shape(shape),
+                HPX_FORWARD(Future, predecessor), HPX_FORWARD(Ts, ts)...);
         }
     } bulk_then_execute{};
-}}}    // namespace hpx::parallel::execution
+
+    /// Asynchronously invoke the given set of nullary functions, each on its
+    /// own execution agent
+    ///
+    /// This creates a group of function invocations whose ordering is given by
+    /// the execution_category associated with the executor.
+    ///
+    /// All exceptions thrown by invocations of the functions are reported in a
+    /// manner consistent with parallel algorithm execution through the returned
+    /// future.
+    ///
+    /// \param exec  [in] The executor object to use for scheduling of the
+    ///              functions \a fs.
+    /// \param fs    [in] The functions which will be scheduled using the
+    ///              given executor.
+    ///
+    /// \returns The return type of \a executor_type::async_invoke if defined by
+    ///          \a executor_type. Otherwise a future<void>
+    ///          representing finishing the execution of all functions \a fs.
+    ///
+    /// \note This calls exec.async_invoke(fs...) if it exists; otherwise it
+    ///       executes async_execute(fs) for each fs.
+    ///
+    inline constexpr struct async_invoke_t final
+      : hpx::functional::detail::tag_fallback<async_invoke_t>
+    {
+    private:
+        // clang-format off
+        template <typename Executor, typename F, typename... Fs,
+            HPX_CONCEPT_REQUIRES_(
+                std::is_invocable_v<F> && (std::is_invocable_v<Fs> && ...)
+            )>
+        // clang-format on
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
+            async_invoke_t, Executor&& exec, F&& f, Fs&&... fs)
+        {
+            return detail::async_invoke_fn_helper<std::decay_t<Executor>>::call(
+                HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                HPX_FORWARD(Fs, fs)...);
+        }
+    } async_invoke{};
+
+    /// Synchronously invoke the given set of nullary functions, each on its own
+    /// execution agent
+    ///
+    /// This creates a group of function invocations whose ordering is given by
+    /// the execution_category associated with the executor.
+    ///
+    /// All exceptions thrown by invocations of the functions are reported in a
+    /// manner consistent with parallel algorithm execution through the returned
+    /// future.
+    ///
+    /// \param exec  [in] The executor object to use for scheduling of the
+    ///              functions \a fs.
+    /// \param fs    [in] The functions which will be scheduled using the
+    ///              given executor.
+    ///
+    /// \returns The return type of \a executor_type::async_invoke if defined by
+    ///          \a executor_type.
+    ///
+    /// \note This calls exec.sync_invoke(fs...) if it exists; otherwise it
+    ///       executes sync_execute(fs) for each fs.
+    ///
+    inline constexpr struct sync_invoke_t final
+      : hpx::functional::detail::tag_fallback<sync_invoke_t>
+    {
+    private:
+        // clang-format off
+        template <typename Executor, typename F, typename... Fs,
+            HPX_CONCEPT_REQUIRES_(
+                std::is_invocable_v<F> && (std::is_invocable_v<Fs> && ...)
+            )>
+        // clang-format on
+        friend HPX_FORCEINLINE decltype(auto) tag_fallback_invoke(
+            sync_invoke_t, Executor&& exec, F&& f, Fs&&... fs)
+        {
+            return detail::sync_invoke_fn_helper<std::decay_t<Executor>>::call(
+                HPX_FORWARD(Executor, exec), HPX_FORWARD(F, f),
+                HPX_FORWARD(Fs, fs)...);
+        }
+    } sync_invoke{};
+}    // namespace hpx::parallel::execution
