@@ -6,20 +6,20 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <hpx/modules/format.hpp>
 #include <hpx/util/bad_lexical_cast.hpp>
 
 #include <string>
 #include <type_traits>
 
-namespace hpx { namespace util {
+namespace hpx::util {
 
     namespace detail {
+
         template <typename T, typename Enable = void>
         struct to_string
         {
-            static std::string call(T const& value)
+            [[nodiscard]] static std::string call(T const& value)
             {
                 return util::format("{}", value);
             }
@@ -27,10 +27,10 @@ namespace hpx { namespace util {
 
         template <typename T>
         struct to_string<T,
-            typename std::enable_if<std::is_integral<T>::value ||
-                std::is_floating_point<T>::value>::type>
+            std::enable_if_t<std::is_integral_v<T> ||
+                std::is_floating_point_v<T>>>
         {
-            static std::string call(T const& value)
+            [[nodiscard]] static std::string call(T const& value)
             {
                 return std::to_string(value);
             }
@@ -38,7 +38,7 @@ namespace hpx { namespace util {
     }    // namespace detail
 
     template <typename T>
-    std::string to_string(T const& v)
+    [[nodiscard]] std::string to_string(T const& v)
     {
         try
         {
@@ -49,5 +49,4 @@ namespace hpx { namespace util {
             return detail::throw_bad_lexical_cast<T, std::string>();
         }
     }
-
-}}    // namespace hpx::util
+}    // namespace hpx::util
