@@ -8,22 +8,22 @@
 
 #include <typeinfo>
 
-namespace hpx { namespace util {
+namespace hpx::util {
 
     const char* bad_lexical_cast::what() const noexcept
     {
-        return "bad lexical cast: "
-               "source type value could not be interpreted as target";
+        return "bad lexical cast: source type value could not be interpreted "
+               "as target";
     }
 
     bad_lexical_cast::~bad_lexical_cast() noexcept = default;
 
     namespace detail {
+
         void throw_bad_lexical_cast(std::type_info const& source_type,
             std::type_info const& target_type)
         {
             throw bad_lexical_cast(source_type, target_type);
         }
     }    // namespace detail
-
-}}    // namespace hpx::util
+}    // namespace hpx::util
