@@ -1,5 +1,5 @@
 //  Copyright (c) 2011 Thomas Heller
-//  Copyright (c) 2013 Hartmut Kaiser
+//  Copyright (c) 2013-2023 Hartmut Kaiser
 //  Copyright (c) 2014-2015 Agustin Berge
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -12,9 +12,10 @@
 #include <hpx/functional/detail/vtable/vtable.hpp>
 
 #include <cstddef>
-#include <new>
+#include <memory>
+
+namespace hpx::util::detail {
 
-namespace hpx { namespace util { namespace detail {
     struct copyable_vtable
     {
         template <typename T>
@@ -22,22 +23,21 @@ namespace hpx { namespace util { namespace detail {
             void const* src, bool destroy)
         {
             if (destroy)
-                vtable::get<T>(storage).~T();
+            {
+                std::destroy_at(std::addressof(vtable::get<T>(storage)));
+            }
 
             void* buffer = vtable::allocate<T>(storage, storage_size);
             return ::new (buffer) T(vtable::get<T>(src));
         }
-        void* (*copy)(void*, std::size_t, void const*, bool);
+        void* (*copy)(void*, std::size_t, void const*, bool) = nullptr;
 
-        constexpr copyable_vtable(std::nullptr_t) noexcept
-          : copy(nullptr)
-        {
-        }
+        explicit constexpr copyable_vtable(std::nullptr_t) noexcept {}
 
         template <typename T>
-        constexpr copyable_vtable(construct_vtable<T>) noexcept
-          : copy(&copyable_vtable::template _copy<T>)
+        explicit constexpr copyable_vtable(construct_vtable<T>) noexcept
+          : copy(&copyable_vtable::_copy<T>)
         {
         }
     };
-}}}    // namespace hpx::util::detail
+}    // namespace hpx::util::detail
