@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //  Copyright (c)      2011 Thomas Heller
 //
@@ -13,12 +13,14 @@
 #include <hpx/modules/itt_notify.hpp>
 #endif
 
-namespace hpx { namespace actions { namespace detail {
+namespace hpx::actions::detail {
+
     template <typename Action>
-    char const* get_action_name();
+    [[nodiscard]] char const* get_action_name() noexcept;
 
 #if HPX_HAVE_ITTNOTIFY != 0 && !defined(HPX_HAVE_APEX)
     template <typename Action>
-    util::itt::string_handle const& get_action_name_itt();
+    [[nodiscard]] util::itt::string_handle const&
+    get_action_name_itt() noexcept;
 #endif
-}}}    // namespace hpx::actions::detail
+}    // namespace hpx::actions::detail
