@@ -1,4 +1,4 @@
-//  Copyright (c) 2017-2021 Hartmut Kaiser
+//  Copyright (c) 2017-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -12,7 +12,8 @@
 #include <cstddef>
 #include <memory>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
+
     // By default we don't know anything about the function's name
     template <typename F, typename Enable = void>
     struct get_function_annotation
@@ -35,4 +36,4 @@ namespace hpx { namespace traits {
         }
     };
 #endif
-}}    // namespace hpx::traits
+}    // namespace hpx::traits
