@@ -14,14 +14,14 @@
 #if defined(HPX_MSVC)
 #pragma warning(disable : 4786)    // identifier truncated in debug info
 #pragma warning(disable : 4710)    // function not inlined
-#pragma warning(                                                               \
-    disable : 4711)    // function selected for automatic inline expansion
+// function selected for automatic inline expansion
+#pragma warning(disable : 4711)
 #pragma warning(disable : 4514)    // unreferenced inline removed
 #endif
 
 #include <hpx/functional/bind.hpp>
 
-namespace placeholders = hpx::util::placeholders;
+namespace placeholders = hpx::placeholders;
 
 #include <iostream>
 
@@ -30,13 +30,16 @@ namespace placeholders = hpx::util::placeholders;
 
 int main()
 {
+    // gcc V13 over-eagerly reports a dangling reference here
+#if !defined(HPX_GCC_VERSION) || HPX_GCC_VERSION < 130000
     typedef std::pair<int, int> pair_type;
 
     pair_type pair(10, 20);
 
-    int const& x = hpx::util::bind(&pair_type::first, placeholders::_1)(pair);
+    int const& x = hpx::bind(&pair_type::first, placeholders::_1)(pair);
 
     HPX_TEST_EQ(&pair.first, &x);
+#endif
 
     return hpx::util::report_errors();
 }
