@@ -24,8 +24,7 @@ void do_sum_avg(int values[], int n, int& sum, double& avg)
 
 int main()
 {
-    hpx::util::function_nonser<void(int values[], int n, int& sum, double& avg)>
-        sum_avg;
+    hpx::function<void(int values[], int n, int& sum, double& avg)> sum_avg;
     sum_avg = &do_sum_avg;
 
     return hpx::util::report_errors();
