@@ -23,9 +23,9 @@
 #include <hpx/hardware/timestamp/cuda.hpp>
 #endif
 
-namespace hpx { namespace util { namespace hardware {
+namespace hpx::util::hardware {
 
-    HPX_HOST_DEVICE inline std::uint64_t timestamp()
+    [[nodiscard]] HPX_HOST_DEVICE inline std::uint64_t timestamp()
     {
 #if defined(HPX_HAVE_CUDA) && defined(HPX_COMPUTE_DEVICE_CODE)
         return timestamp_cuda();
@@ -33,7 +33,6 @@ namespace hpx { namespace util { namespace hardware {
         return GetTimeBase();
 #endif
     }
-
-}}}    // namespace hpx::util::hardware
+}    // namespace hpx::util::hardware
 
 #endif
