@@ -16,37 +16,33 @@
 #include <hpx/hardware/timestamp/cuda.hpp>
 #endif
 
-namespace hpx { namespace util { namespace hardware {
+namespace hpx::util::hardware {
 
     // clang-format off
-    HPX_HOST_DEVICE inline std::uint64_t timestamp()
+    [[nodiscard]] HPX_HOST_DEVICE inline std::uint64_t timestamp()
     {
 #if defined(HPX_HAVE_CUDA) && defined(HPX_COMPUTE_DEVICE_CODE)
         return timestamp_cuda();
 #else
-        std::uint64_t r = 0;
+        std::uint32_t lo = 0, hi = 0;
 
-        #if defined(HPX_HAVE_RDTSCP)
-            __asm__ __volatile__(
-                "rdtscp ;\n"
-                : "=A"(r)
-                :
-                : "%ecx");
-        #elif defined(HPX_HAVE_RDTSC)
-            __asm__ __volatile__(
-                "movl %%ebx, %%edi ;\n"
-                "xorl %%eax, %%eax ;\n"
-                "cpuid ;\n"
-                "rdtsc ;\n"
-                "movl %%edi, %%ebx ;\n"
-                : "=A"(r)
-                :
-                : "%edi", "%ecx");
-        #endif
+#if defined(HPX_HAVE_RDTSCP)
+        __asm__ __volatile__(
+            "rdtscp ;\n"
+            : "=a"(lo), "=d"(hi)
+            :
+            : "rcx");
+#elif defined(HPX_HAVE_RDTSC)
+        __asm__ __volatile__(
+            "cpuid ;\n"
+            "rdtsc ;\n"
+            : "=a"(lo), "=d"(hi)
+            :
+            : "rbx", "rcx");
+#endif
 
-        return r;
+        return ((static_cast<std::uint64_t>(hi)) << 32) | lo;
 #endif
     }
     // clang-format on
-
-}}}    // namespace hpx::util::hardware
+}    // namespace hpx::util::hardware
