@@ -1,5 +1,5 @@
 //  Copyright (c) 2005-2007 Andre Merzky
-//  Copyright (c) 2005-2018 Hartmut Kaiser
+//  Copyright (c) 2005-2022 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -10,6 +10,8 @@
 
 // System Header Files
 #include <cerrno>
+#include <cstddef>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
@@ -40,30 +42,48 @@ extern char** environ;
 #endif
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace util {
+namespace hpx::util {
 
     ///////////////////////////////////////////////////////////////////////////////
     // example ini line: line # comment
-    const char pattern_comment[] = "^([^#]*)(#.*)$";
+    inline constexpr char const pattern_comment[] = "^([^#]*)(#.*)$";
     ///////////////////////////////////////////////////////////////////////////////
 
     namespace detail {
+
         ///////////////////////////////////////////////////////////////////////////
         inline std::string trim_whitespace(std::string const& s)
         {
             typedef std::string::size_type size_type;
 
-            size_type first = s.find_first_not_of(" \t\r\n");
+            size_type const first = s.find_first_not_of(" \t\r\n");
             if (std::string::npos == first)
                 return (std::string());
 
-            size_type last = s.find_last_not_of(" \t\r\n");
+            size_type const last = s.find_last_not_of(" \t\r\n");
             return s.substr(first, last - first + 1);
         }
+
+        // MSVC: using std::string::replace triggers ASAN reports
+        std::string replace_substr(std::string const& str,
+            std::size_t start_pos, std::size_t oldlen, char const* newstr)
+        {
+            std::string result(str.substr(0, start_pos));
+            result += newstr;
+            result += str.substr(start_pos + oldlen);
+            return result;
+        }
+
+        std::string replace_substr(std::string const& str,
+            std::size_t start_pos, std::size_t oldlen,
+            std::string const& newstr)
+        {
+            return replace_substr(str, start_pos, oldlen, newstr.c_str());
+        }
     }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////////
-    section::section()
+    section::section() noexcept
       : root_(this_())
     {
     }
@@ -75,19 +95,19 @@ namespace hpx { namespace util {
         read(filename);
     }
 
-    section::section(const section& in)
+    section::section(section const& in)
       : root_(this_())
       , name_(in.get_name())
       , parent_name_(in.get_parent_name())
     {
         entry_map const& e = in.get_entries();
-        entry_map::const_iterator end = e.end();
-        for (entry_map::const_iterator i = e.begin(); i != end; ++i)
+        auto const end = e.end();
+        for (auto i = e.begin(); i != end; ++i)
             add_entry(i->first, i->second);
 
         section_map s = in.get_sections();
-        section_map::iterator send = s.end();
-        for (section_map::iterator si = s.begin(); si != send; ++si)
+        auto const send = s.end();
+        for (auto si = s.begin(); si != send; ++si)
             add_section(si->first, si->second, get_root());
     }
 
@@ -102,13 +122,13 @@ namespace hpx { namespace util {
             name_ = rhs.get_name();
 
             entry_map const& e = rhs.get_entries();
-            entry_map::const_iterator end = e.end();
-            for (entry_map::const_iterator i = e.begin(); i != end; ++i)
+            auto const end = e.end();
+            for (auto i = e.begin(); i != end; ++i)
                 add_entry(l, i->first, i->first, i->second);
 
             section_map s = rhs.get_sections();
-            section_map::iterator send = s.end();
-            for (section_map::iterator si = s.begin(); si != send; ++si)
+            auto const send = s.end();
+            for (auto si = s.begin(); si != send; ++si)
                 add_section(l, si->first, si->second, get_root());
         }
         return *this;
@@ -125,13 +145,13 @@ namespace hpx { namespace util {
             name_ = rhs.get_name();
 
             entry_map const& e = rhs.get_entries();
-            entry_map::const_iterator end = e.end();
-            for (entry_map::const_iterator i = e.begin(); i != end; ++i)
+            auto const end = e.end();
+            for (auto i = e.begin(); i != end; ++i)
                 add_entry(l, i->first, i->first, i->second);
 
             section_map s = rhs.get_sections();
-            section_map::iterator send = s.end();
-            for (section_map::iterator si = s.begin(); si != send; ++si)
+            auto const send = s.end();
+            for (auto si = s.begin(); si != send; ++si)
                 add_section(l, si->first, si->second, get_root());
         }
         return *this;
@@ -167,7 +187,7 @@ namespace hpx { namespace util {
 
     bool force_entry(std::string& str)
     {
-        std::string::size_type p = str.find_last_of('!');
+        std::string::size_type const p = str.find_last_of('!');
         if (p != std::string::npos &&
             str.find_first_not_of(" \t", p + 1) == std::string::npos)
         {
@@ -187,9 +207,8 @@ namespace hpx { namespace util {
 
         std::regex regex_comment(pattern_comment, std::regex_constants::icase);
 
-        std::vector<std::string>::const_iterator end = lines.end();
-        for (std::vector<std::string>::const_iterator it = lines.begin();
-             it != end; ++it)
+        auto const end = lines.end();
+        for (auto it = lines.begin(); it != end; ++it)
         {
             ++linenum;
 
@@ -213,17 +232,17 @@ namespace hpx { namespace util {
                         continue;
                 }
             }
-            // no comments anymore: line is either section, key=val,
-            // or garbage/empty
+            // no comments anymore: line is either section, key=val, or
+            // garbage/empty
 
-            // Check if we have a section.
-            // Example: [sec.ssec]
+            // Check if we have a section. Example: [sec.ssec]
             if (line.front() == '[' && line.back() == ']')
             {
                 current = this;    // start adding sections at the root
 
-                // got the section name. It might be hierarchical, so split it up, and
-                // for each elem, check if we have it.  If not, create it, and add
+                // got the section name. It might be hierarchical, so split it
+                // up, and for each elem, check if we have it. If not, create
+                // it, and add
                 std::string sec_name(line.substr(1, line.size() - 2));
                 std::string::size_type pos = 0;
                 for (std::string::size_type pos1 = sec_name.find_first_of('.');
@@ -244,13 +263,12 @@ namespace hpx { namespace util {
             {
                 std::string sec_key =
                     detail::trim_whitespace(line.substr(0, assign_pos));
-                std::string value = detail::trim_whitespace(
-                    line.substr(assign_pos + 1, line.size() - assign_pos - 1));
 
                 section* s = current;    // save the section we're in
                 current = this;          // start adding sections at the root
 
                 std::string::size_type pos = 0;
+
                 // Check if we have a qualified key name
                 // Example: hpx.commandline.allow_unknown
                 for (std::string::size_type dot_pos =
@@ -261,6 +279,7 @@ namespace hpx { namespace util {
                     current = current->add_section_if_new(
                         sec_key.substr(pos, dot_pos - pos));
                 }
+
                 // if we don't have section qualifiers, restore current...
                 if (current == this)
                 {
@@ -281,6 +300,8 @@ namespace hpx { namespace util {
 
                 if (replace_existing || !current->has_entry(l, key))
                 {
+                    std::string value = detail::trim_whitespace(line.substr(
+                        assign_pos + 1, line.size() - assign_pos - 1));
                     current->add_entry(l, key, key, value);
                 }
 
@@ -326,16 +347,16 @@ namespace hpx { namespace util {
     bool section::has_section(
         std::unique_lock<mutex_type>& l, std::string const& sec_name) const
     {
-        std::string::size_type i = sec_name.find('.');
+        std::string::size_type const i = sec_name.find('.');
         if (i != std::string::npos)
         {
-            std::string cor_sec_name = sec_name.substr(0, i);
+            std::string const cor_sec_name = sec_name.substr(0, i);
 
-            section_map::const_iterator it = sections_.find(cor_sec_name);
+            auto const it = sections_.find(cor_sec_name);
             if (it != sections_.end())
             {
-                std::string sub_sec_name = sec_name.substr(i + 1);
-                hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                std::string const sub_sec_name = sec_name.substr(i + 1);
+                hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
                 return (*it).second.has_section(sub_sec_name);
             }
             return false;
@@ -346,15 +367,15 @@ namespace hpx { namespace util {
     section* section::get_section(
         std::unique_lock<mutex_type>& l, std::string const& sec_name)
     {
-        std::string::size_type i = sec_name.find('.');
+        std::string::size_type const i = sec_name.find('.');
         if (i != std::string::npos)
         {
-            std::string cor_sec_name = sec_name.substr(0, i);
-            section_map::iterator it = sections_.find(cor_sec_name);
+            std::string const cor_sec_name = sec_name.substr(0, i);
+            auto const it = sections_.find(cor_sec_name);
             if (it != sections_.end())
             {
-                std::string sub_sec_name = sec_name.substr(i + 1);
-                hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                std::string const sub_sec_name = sec_name.substr(i + 1);
+                hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
                 return (*it).second.get_section(sub_sec_name);
             }
 
@@ -362,32 +383,31 @@ namespace hpx { namespace util {
             if (name.empty())
                 name = "<root>";
 
-            HPX_THROW_EXCEPTION(bad_parameter, "section::get_section",
-                "No such section ({}) in section: {}", sec_name, name);
-            return nullptr;
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
+                "section::get_section", "No such section ({}) in section: {}",
+                sec_name, name);
         }
 
-        section_map::iterator it = sections_.find(sec_name);
+        auto const it = sections_.find(sec_name);
         if (it != sections_.end())
             return &((*it).second);
 
-        HPX_THROW_EXCEPTION(bad_parameter, "section::get_section",
+        HPX_THROW_EXCEPTION(hpx::error::bad_parameter, "section::get_section",
             "No such section ({}) in section: {}", sec_name, get_name());
-        return nullptr;
     }
 
     section const* section::get_section(
         std::unique_lock<mutex_type>& l, std::string const& sec_name) const
     {
-        std::string::size_type i = sec_name.find('.');
+        std::string::size_type const i = sec_name.find('.');
         if (i != std::string::npos)
         {
-            std::string cor_sec_name = sec_name.substr(0, i);
-            section_map::const_iterator it = sections_.find(cor_sec_name);
+            std::string const cor_sec_name = sec_name.substr(0, i);
+            auto const it = sections_.find(cor_sec_name);
             if (it != sections_.end())
             {
-                std::string sub_sec_name = sec_name.substr(i + 1);
-                hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                std::string const sub_sec_name = sec_name.substr(i + 1);
+                hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
                 return (*it).second.get_section(sub_sec_name);
             }
 
@@ -395,34 +415,33 @@ namespace hpx { namespace util {
             if (name.empty())
                 name = "<root>";
 
-            HPX_THROW_EXCEPTION(bad_parameter, "section::get_section",
-                "No such section ({}) in section: {}", sec_name, name);
-            return nullptr;
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
+                "section::get_section", "No such section ({}) in section: {}",
+                sec_name, name);
         }
 
-        section_map::const_iterator it = sections_.find(sec_name);
+        auto const it = sections_.find(sec_name);
         if (it != sections_.end())
             return &((*it).second);
 
-        HPX_THROW_EXCEPTION(bad_parameter, "section::get_section",
+        HPX_THROW_EXCEPTION(hpx::error::bad_parameter, "section::get_section",
             "No such section ({}) in section: {}", sec_name, get_name());
-        return nullptr;
     }
 
     void section::add_entry(std::unique_lock<mutex_type>& l,
         std::string const& fullkey, std::string const& key, std::string val)
     {
         // first expand the full property name in the value (avoids infinite recursion)
-        expand_only(
-            l, val, std::string::size_type(-1), get_full_name() + "." + key);
+        expand_only(l, val, static_cast<std::string::size_type>(-1),
+            get_full_name() + "." + key);
 
-        std::string::size_type i = key.find_last_of('.');
+        std::string::size_type const i = key.find_last_of('.');
         if (i != std::string::npos)
         {
             section* current = root_;
 
             // make sure all sections in key exist
-            std::string sec_name = key.substr(0, i);
+            std::string const sec_name = key.substr(0, i);
 
             std::string::size_type pos = 0;
             for (std::string::size_type pos1 = sec_name.find_first_of('.');
@@ -440,17 +459,17 @@ namespace hpx { namespace util {
         }
         else
         {
-            entry_map::iterator it = entries_.find(key);
+            auto const it = entries_.find(key);
             if (it != entries_.end())
             {
                 auto& e = it->second;
-                e.first = std::move(val);
+                e.first = HPX_MOVE(val);
                 if (!e.second.empty())
                 {
-                    std::string value = e.first;
-                    entry_changed_func f = e.second;
+                    std::string const value = e.first;
+                    entry_changed_func const f = e.second;
 
-                    hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                    hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
                     f(fullkey, value);
                 }
             }
@@ -466,13 +485,13 @@ namespace hpx { namespace util {
         std::string const& fullkey, std::string const& key,
         entry_type const& val)
     {
-        std::string::size_type i = key.find_last_of('.');
+        std::string::size_type const i = key.find_last_of('.');
         if (i != std::string::npos)
         {
             section* current = root_;
 
             // make sure all sections in key exist
-            std::string sec_name = key.substr(0, i);
+            std::string const sec_name = key.substr(0, i);
 
             std::string::size_type pos = 0;
             for (std::string::size_type pos1 = sec_name.find_first_of('.');
@@ -490,34 +509,36 @@ namespace hpx { namespace util {
         }
         else
         {
-            entry_map::iterator it = entries_.find(key);
+            auto const it = entries_.find(key);
             if (it != entries_.end())
             {
-                it->second = val;
-                if (!it->second.second.empty())
+                auto& second = it->second;
+                second = val;
+                if (!second.second.empty())
                 {
-                    std::string value = it->second.first;
-                    entry_changed_func f = it->second.second;
+                    std::string const value = second.first;
+                    entry_changed_func const f = second.second;
 
-                    hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                    hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
                     f(fullkey, value);
                 }
             }
             else
             {
                 // just add this entry to the section
-                std::pair<entry_map::iterator, bool> p =
-                    entries_.insert(entry_map::value_type(key, val));
+                std::pair<entry_map::iterator, bool> const p =
+                    entries_.emplace(key, val);
                 HPX_ASSERT(p.second);
 
-                if (!p.first->second.second.empty())
+                auto const& second = p.first->second;
+                if (!second.second.empty())
                 {
-                    std::string key = p.first->first;
-                    std::string value = p.first->second.first;
-                    entry_changed_func f = p.first->second.second;
+                    std::string const k = p.first->first;
+                    std::string const value = second.first;
+                    entry_changed_func const f = second.second;
 
-                    hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
-                    f(key, value);
+                    hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                    f(k, value);
                 }
             }
         }
@@ -530,8 +551,8 @@ namespace hpx { namespace util {
     public:
         template <typename A1, typename A2>
         compose_callback_impl(A1&& f1, A2&& f2)
-          : f1_(std::forward<A1>(f1))
-          , f2_(std::forward<A2>(f2))
+          : f1_(HPX_FORWARD(A1, f1))
+          , f2_(HPX_FORWARD(A2, f2))
         {
         }
 
@@ -548,31 +569,30 @@ namespace hpx { namespace util {
 
     template <typename F1, typename F2>
     static HPX_FORCEINLINE
-        util::function_nonser<void(std::string const&, std::string const&)>
+        hpx::function<void(std::string const&, std::string const&)>
         compose_callback(F1&& f1, F2&& f2)
     {
         if (!f1)
-            return std::forward<F2>(f2);
+            return HPX_FORWARD(F2, f2);
         else if (!f2)
-            return std::forward<F1>(f1);
+            return HPX_FORWARD(F1, f1);
 
         // otherwise create a combined callback
-        typedef compose_callback_impl<typename std::decay<F1>::type,
-            typename std::decay<F2>::type>
-            result_type;
-        return result_type(std::forward<F1>(f1), std::forward<F2>(f2));
+        using result_type =
+            compose_callback_impl<std::decay_t<F1>, std::decay_t<F2>>;
+        return result_type(HPX_FORWARD(F1, f1), HPX_FORWARD(F2, f2));
     }
 
     void section::add_notification_callback(std::unique_lock<mutex_type>& l,
         std::string const& key, entry_changed_func const& callback)
     {
-        std::string::size_type i = key.find_last_of('.');
+        std::string::size_type const i = key.find_last_of('.');
         if (i != std::string::npos)
         {
             section* current = root_;
 
             // make sure all sections in key exist
-            std::string sec_name = key.substr(0, i);
+            std::string const sec_name = key.substr(0, i);
 
             std::string::size_type pos = 0;
             for (std::string::size_type pos1 = sec_name.find_first_of('.');
@@ -591,7 +611,7 @@ namespace hpx { namespace util {
         else
         {
             // just add this entry to the section
-            entry_map::iterator it = entries_.find(key);
+            auto const it = entries_.find(key);
             if (it != entries_.end())
             {
                 it->second.second =
@@ -607,17 +627,17 @@ namespace hpx { namespace util {
     bool section::has_entry(
         std::unique_lock<mutex_type>& l, std::string const& key) const
     {
-        std::string::size_type i = key.find('.');
+        std::string::size_type const i = key.find('.');
         if (i != std::string::npos)
         {
-            std::string sub_sec = key.substr(0, i);
-            std::string sub_key = key.substr(i + 1, key.size() - i);
+            std::string const sub_sec = key.substr(0, i);
             if (has_section(l, sub_sec))
             {
-                section_map::const_iterator cit = sections_.find(sub_sec);
+                std::string const sub_key = key.substr(i + 1, key.size() - i);
+                auto const cit = sections_.find(sub_sec);
                 HPX_ASSERT(cit != sections_.end());
-                hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
-                return (*cit).second.has_entry(sub_key);
+                hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                return (*cit).second.has_entry(sub_key);    //-V783
             }
             return false;
         }
@@ -627,34 +647,32 @@ namespace hpx { namespace util {
     std::string section::get_entry(
         std::unique_lock<mutex_type>& l, std::string const& key) const
     {
-        std::string::size_type i = key.find('.');
-        if (i != std::string::npos)
+        if (std::string::size_type const i = key.find('.');
+            i != std::string::npos)
         {
-            std::string sub_sec = key.substr(0, i);
-            std::string sub_key = key.substr(i + 1, key.size() - i);
+            std::string const sub_sec = key.substr(0, i);
             if (has_section(l, sub_sec))
             {
-                section_map::const_iterator cit = sections_.find(sub_sec);
+                std::string const sub_key = key.substr(i + 1, key.size() - i);
+                auto const cit = sections_.find(sub_sec);
                 HPX_ASSERT(cit != sections_.end());
-                hpx::util::unlock_guard<std::unique_lock<mutex_type>> ul(l);
-                return (*cit).second.get_entry(sub_key);
+                hpx::unlock_guard<std::unique_lock<mutex_type>> ul(l);
+                return (*cit).second.get_entry(sub_key);    //-V783
             }
 
-            HPX_THROW_EXCEPTION(bad_parameter, "section::get_entry",
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter, "section::get_entry",
                 "No such key ({}) in section: {}", key, get_name());
-            return "";
         }
 
         if (entries_.find(key) != entries_.end())
         {
-            entry_map::const_iterator cit = entries_.find(key);
+            auto const cit = entries_.find(key);
             HPX_ASSERT(cit != entries_.end());
-            return expand(l, (*cit).second.first);
+            return expand(l, (*cit).second.first);    //-V783
         }
 
-        HPX_THROW_EXCEPTION(bad_parameter, "section::get_entry",
+        HPX_THROW_EXCEPTION(hpx::error::bad_parameter, "section::get_entry",
             "No such section ({}) in section: {}", key, get_name());
-        return "";
     }
 
     std::string section::get_entry(std::unique_lock<mutex_type>& l,
@@ -666,22 +684,19 @@ namespace hpx { namespace util {
         hpx::string_util::split(
             split_key, key, hpx::string_util::is_any_of("."));
 
-        std::string sk = split_key.back();
+        std::string const sk = split_key.back();
         split_key.pop_back();
 
         section const* cur_section = this;
-        for (string_vector::const_iterator iter = split_key.begin(),
-                                           end = split_key.end();
-             iter != end; ++iter)
+        for (const auto& iter : split_key)
         {
-            section_map::const_iterator next =
-                cur_section->sections_.find(*iter);
+            auto next = cur_section->sections_.find(iter);
             if (cur_section->sections_.end() == next)
                 return expand(l, default_val);
             cur_section = &next->second;
         }
 
-        entry_map::const_iterator entry = cur_section->entries_.find(sk);
+        auto const entry = cur_section->entries_.find(sk);
         if (cur_section->entries_.end() == entry)
             return expand(l, default_val);
 
@@ -722,25 +737,30 @@ namespace hpx { namespace util {
             }
         }
 
-        entry_map::const_iterator eend = entries_.end();
-        for (entry_map::const_iterator i = entries_.begin(); i != eend; ++i)
+        auto const eend = entries_.end();
+        for (auto i = entries_.begin(); i != eend; ++i)
         {
             indent(ind, strm);
 
-            const std::string expansion = expand(l, i->second.first);
+            auto& second = i->second;
+            std::string const expansion = expand(l, second.first);
 
             // Check if the expanded entry is different from the actual entry.
-            if (expansion != i->second.first)
+            if (expansion != second.first)
+            {
                 // If the expansion is different from the real entry, then print
                 // it out.
-                strm << "'" << i->first << "' : '" << i->second.first
-                     << "' -> '" << expansion << "'\n";
+                strm << "'" << i->first << "' : '" << second.first << "' -> '"
+                     << expansion << "'\n";
+            }
             else
-                strm << "'" << i->first << "' : '" << i->second.first << "'\n";
+            {
+                strm << "'" << i->first << "' : '" << second.first << "'\n";
+            }
         }
 
-        section_map::const_iterator send = sections_.end();
-        for (section_map::const_iterator i = sections_.begin(); i != send; ++i)
+        auto const send = sections_.end();
+        for (auto i = sections_.begin(); i != send; ++i)
         {
             indent(ind, strm);
             strm << "[" << i->first << "]\n";
@@ -765,13 +785,13 @@ namespace hpx { namespace util {
 
         // merge entries: keep own entries, and add other entries
         entry_map const& s_entries = second.get_entries();
-        entry_map::const_iterator end = s_entries.end();
-        for (entry_map::const_iterator i = s_entries.begin(); i != end; ++i)
+        auto const end = s_entries.end();
+        for (auto i = s_entries.begin(); i != end; ++i)
             entries_[i->first] = i->second;
 
         // merge subsection known in first section
-        section_map::iterator send = sections_.end();
-        for (section_map::iterator i = sections_.begin(); i != send; ++i)
+        auto const send = sections_.end();
+        for (auto i = sections_.begin(); i != send; ++i)
         {
             // is there something to merge with?
             if (second.has_section(l, i->first))
@@ -780,8 +800,8 @@ namespace hpx { namespace util {
 
         // merge subsection known in second section
         section_map s = second.get_sections();
-        section_map::iterator secend = s.end();
-        for (section_map::iterator i = s.begin(); i != secend; ++i)
+        auto const secend = s.end();
+        for (auto i = s.begin(); i != secend; ++i)
         {
             // if THIS knows the section, we already merged it above
             if (!has_section(l, i->first))
@@ -794,7 +814,7 @@ namespace hpx { namespace util {
 
     /////////////////////////////////////////////////////////////////////////////////
     void section::line_msg(std::string msg, std::string const& file, int lnum,
-        std::string const& line)
+        std::string const& line) const
     {
         msg += " " + file;
         if (lnum > 0)
@@ -802,7 +822,7 @@ namespace hpx { namespace util {
         if (!line.empty())
             msg += " (offending entry: " + line + ")";
 
-        HPX_THROW_EXCEPTION(no_success, "section::line_msg", msg);
+        HPX_THROW_EXCEPTION(hpx::error::no_success, "section::line_msg", msg);
     }
 
     ///////////////////////////////////////////////////////////////////////////////
@@ -816,7 +836,7 @@ namespace hpx { namespace util {
         {
             if (end != 0 && value[end - 1] != '\\')
                 break;
-            value.replace(end - 1, 2, ch);
+            value = detail::replace_substr(value, end - 1, 2, ch);
             end = value.find_first_of(ch, end);
         }
         return end;
@@ -844,19 +864,19 @@ namespace hpx { namespace util {
         expand(l, value, begin);
 
         // now expand the key itself
-        std::string::size_type end = find_next("]", value, begin + 1);
+        std::string::size_type const end = find_next("]", value, begin + 1);
         if (end != std::string::npos)
         {
             std::string to_expand = value.substr(begin + 2, end - begin - 2);
-            std::string::size_type colon = find_next(":", to_expand);
+            std::string::size_type const colon = find_next(":", to_expand);
             if (colon == std::string::npos)
             {
-                value.replace(begin, end - begin + 1,
-                    root_->get_entry(l, to_expand, std::string("")));
+                value = detail::replace_substr(value, begin, end - begin + 1,
+                    root_->get_entry(l, to_expand, std::string()));
             }
             else
             {
-                value.replace(begin, end - begin + 1,
+                value = detail::replace_substr(value, begin, end - begin + 1,
                     root_->get_entry(l, to_expand.substr(0, colon),
                         to_expand.substr(colon + 1)));
             }
@@ -870,21 +890,21 @@ namespace hpx { namespace util {
         expand(l, value, begin);
 
         // now expand the key itself
-        std::string::size_type end = find_next("}", value, begin + 1);
+        std::string::size_type const end = find_next("}", value, begin + 1);
         if (end != std::string::npos)
         {
             std::string to_expand = value.substr(begin + 2, end - begin - 2);
-            std::string::size_type colon = find_next(":", to_expand);
+            std::string::size_type const colon = find_next(":", to_expand);
             if (colon == std::string::npos)
             {
-                char* env = getenv(to_expand.c_str());
-                value.replace(
-                    begin, end - begin + 1, nullptr != env ? env : "");
+                char const* env = std::getenv(to_expand.c_str());
+                value = detail::replace_substr(
+                    value, begin, end - begin + 1, nullptr != env ? env : "");
             }
             else
             {
-                char* env = getenv(to_expand.substr(0, colon).c_str());
-                value.replace(begin, end - begin + 1,
+                char* env = std::getenv(to_expand.substr(0, colon).c_str());
+                value = detail::replace_substr(value, begin, end - begin + 1,
                     nullptr != env ? std::string(env) :
                                      to_expand.substr(colon + 1));
             }
@@ -894,7 +914,7 @@ namespace hpx { namespace util {
     std::string section::expand(
         std::unique_lock<mutex_type>& l, std::string value) const
     {
-        expand(l, value, std::string::size_type(-1));
+        expand(l, value, static_cast<std::string::size_type>(-1));
         return value;
     }
 
@@ -922,22 +942,23 @@ namespace hpx { namespace util {
         expand_only(l, value, begin, expand_this);
 
         // now expand the key itself
-        std::string::size_type end = find_next("]", value, begin + 1);
+        std::string::size_type const end = find_next("]", value, begin + 1);
         if (end != std::string::npos)
         {
             std::string to_expand = value.substr(begin + 2, end - begin - 2);
-            std::string::size_type colon = find_next(":", to_expand);
+            std::string::size_type const colon = find_next(":", to_expand);
             if (colon == std::string::npos)
             {
                 if (to_expand == expand_this)
                 {
-                    value.replace(begin, end - begin + 1,
-                        root_->get_entry(l, to_expand, std::string("")));
+                    value =
+                        detail::replace_substr(value, begin, end - begin + 1,
+                            root_->get_entry(l, to_expand, std::string()));
                 }
             }
             else if (to_expand.substr(0, colon) == expand_this)
             {
-                value.replace(begin, end - begin + 1,
+                value = detail::replace_substr(value, begin, end - begin + 1,
                     root_->get_entry(l, to_expand.substr(0, colon),
                         to_expand.substr(colon + 1)));
             }
@@ -952,21 +973,21 @@ namespace hpx { namespace util {
         expand_only(l, value, begin, expand_this);
 
         // now expand the key itself
-        std::string::size_type end = find_next("}", value, begin + 1);
+        std::string::size_type const end = find_next("}", value, begin + 1);
         if (end != std::string::npos)
         {
             std::string to_expand = value.substr(begin + 2, end - begin - 2);
-            std::string::size_type colon = find_next(":", to_expand);
+            std::string::size_type const colon = find_next(":", to_expand);
             if (colon == std::string::npos)
             {
-                char* env = getenv(to_expand.c_str());
-                value.replace(
-                    begin, end - begin + 1, nullptr != env ? env : "");
+                char const* env = std::getenv(to_expand.c_str());
+                value = detail::replace_substr(
+                    value, begin, end - begin + 1, nullptr != env ? env : "");
             }
             else
             {
-                char* env = getenv(to_expand.substr(0, colon).c_str());
-                value.replace(begin, end - begin + 1,
+                char* env = std::getenv(to_expand.substr(0, colon).c_str());
+                value = detail::replace_substr(value, begin, end - begin + 1,
                     nullptr != env ? std::string(env) :
                                      to_expand.substr(colon + 1));
             }
@@ -976,42 +997,47 @@ namespace hpx { namespace util {
     std::string section::expand_only(std::unique_lock<mutex_type>& l,
         std::string value, std::string const& expand_this) const
     {
-        expand_only(l, value, std::string::size_type(-1), expand_this);
+        expand_only(
+            l, value, static_cast<std::string::size_type>(-1), expand_this);
         return value;
     }
 
-    ///////////////////////////////////////////////////////////////////////////////
-    // explicit instantiation for the correct archive types
-    void serialize(serialization::output_archive& ar,
-        section::entry_type const& data, unsigned int /* version */)
-    {
-        ar << data.first;
-        // do not handle callback function
-    }
-
-    void serialize(serialization::input_archive& ar, section::entry_type& data,
-        unsigned int /* version */)
-    {
-        ar >> data.first;
-        // do not handle callback function
-    }
-
     ///////////////////////////////////////////////////////////////////////////////
     template <typename Archive>
-    void section::save(Archive& ar, const unsigned int /* version */) const
+    void section::save(Archive& ar, unsigned int const /* version */) const
     {
         ar << name_;
         ar << parent_name_;
-        ar << entries_;
+
+        std::uint64_t size = entries_.size();
+        ar << size;
+        for (auto const& val : entries_)
+        {
+            ar << val.first;
+        }
+
         ar << sections_;
     }
 
     template <typename Archive>
-    void section::load(Archive& ar, const unsigned int /* version */)
+    void section::load(Archive& ar, unsigned int const /* version */)
     {
         ar >> name_;
         ar >> parent_name_;
-        ar >> entries_;
+
+        std::uint64_t size;
+        ar >> size;    //-V128
+
+        entries_.clear();
+        for (std::size_t i = 0; i < size; ++i)
+        {
+            using value_type = entry_map::value_type;
+
+            value_type v;
+            ar >> const_cast<std::string&>(v.first);
+            entries_.insert(entries_.end(), HPX_MOVE(v));
+        }
+
         ar >> sections_;
 
         set_root(this, true);    // make this the current root
@@ -1019,9 +1045,8 @@ namespace hpx { namespace util {
 
     // explicit instantiation for the correct archive types
     template HPX_CORE_EXPORT void section::save(
-        serialization::output_archive&, const unsigned int version) const;
+        serialization::output_archive&, unsigned int const version) const;
 
     template HPX_CORE_EXPORT void section::load(
-        serialization::input_archive&, const unsigned int version);
-
-}}    // namespace hpx::util
+        serialization::input_archive&, unsigned int const version);
+}    // namespace hpx::util
