@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2025 Hartmut Kaiser
 //  Copyright (c) 2019 Austin McCartney
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -9,19 +9,37 @@
 
 #include <hpx/config.hpp>
 #include <hpx/iterator_support/boost_iterator_categories.hpp>
-#include <hpx/type_support/always_void.hpp>
 #include <hpx/type_support/equality.hpp>
 
 #include <iterator>
 #include <type_traits>
 #include <utility>
-#include <vector>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
+
     ///////////////////////////////////////////////////////////////////////////
+    template <typename Iter>
+    using iter_value_t = typename std::iterator_traits<Iter>::value_type;
+
+    template <typename Iter>
+    using iter_difference_t =
+        typename std::iterator_traits<Iter>::difference_type;
+
+    template <typename Iter>
+    using iter_pointer_t = typename std::iterator_traits<Iter>::pointer;
+
+    template <typename Iter>
+    using iter_reference_t = typename std::iterator_traits<Iter>::reference;
+
+    template <typename Iter>
+    using iter_category_t =
+        typename std::iterator_traits<Iter>::iterator_category;
+
+    ////////////////////////////////////////////////////////////////////////////
     namespace detail {
-        // This implementation of is_iterator seems to work fine even for
-        // VS2013 which has an implementation of std::iterator_traits which is
+
+        // This implementation of is_iterator seems to work fine even for VS2013
+        // which has an implementation of std::iterator_traits that is
         // SFINAE-unfriendly.
         template <typename T>
         struct is_iterator
@@ -33,17 +51,14 @@ namespace hpx { namespace traits {
             template <typename U>
             static void* test(U*);    // pointer
 #else
-            template <typename U,
-                typename = typename std::iterator_traits<U>::pointer>
+            template <typename U, typename = iter_pointer_t<U>>
             static void* test(U&&);
 #endif
 
             static char test(...);
 
-            enum
-            {
-                value = sizeof(test(std::declval<T>())) == sizeof(void*)
-            };
+            static constexpr bool value =
+                sizeof(test(std::declval<T>())) == sizeof(void*);
         };
 
         ///////////////////////////////////////////////////////////////////////
@@ -54,12 +69,15 @@ namespace hpx { namespace traits {
 
         template <typename T, typename U>
         struct addition_result<T, U,
-            typename util::always_void<decltype(
-                std::declval<T>() + std::declval<U>())>::type>
+            std::void_t<decltype(std::declval<T>() + std::declval<U>())>>
         {
             using type = decltype(std::declval<T>() + std::declval<U>());
         };
 
+        template <typename T, typename U>
+        using addition_result_t = typename addition_result<T, U>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename Enable = void>
         struct dereference_result
         {
@@ -67,11 +85,15 @@ namespace hpx { namespace traits {
 
         template <typename T>
         struct dereference_result<T,
-            typename util::always_void<decltype(*(std::declval<T&>()))>::type>
+            std::void_t<decltype(*(std::declval<T&>()))>>
         {
             using type = decltype(*(std::declval<T&>()));
         };
 
+        template <typename T>
+        using dereference_result_t = typename dereference_result<T>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename U, typename Enable = void>
         struct inplace_addition_result
         {
@@ -79,12 +101,16 @@ namespace hpx { namespace traits {
 
         template <typename T, typename U>
         struct inplace_addition_result<T, U,
-            typename util::always_void<decltype(
-                std::declval<T>() += std::declval<U>())>::type>
+            std::void_t<decltype(std::declval<T>() += std::declval<U>())>>
         {
             using type = decltype(std::declval<T>() += std::declval<U>());
         };
 
+        template <typename T, typename U>
+        using inplace_addition_result_t =
+            typename inplace_addition_result<T, U>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename U, typename Enable = void>
         struct inplace_subtraction_result
         {
@@ -92,12 +118,16 @@ namespace hpx { namespace traits {
 
         template <typename T, typename U>
         struct inplace_subtraction_result<T, U,
-            typename util::always_void<decltype(
-                std::declval<T>() -= std::declval<U>())>::type>
+            std::void_t<decltype(std::declval<T>() -= std::declval<U>())>>
         {
             using type = decltype(std::declval<T>() -= std::declval<U>());
         };
 
+        template <typename T, typename U>
+        using inplace_subtraction_result_t =
+            typename inplace_subtraction_result<T, U>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename Enable = void>
         struct predecrement_result
         {
@@ -105,11 +135,15 @@ namespace hpx { namespace traits {
 
         template <typename T>
         struct predecrement_result<T,
-            typename util::always_void<decltype(--std::declval<T&>())>::type>
+            std::void_t<decltype(--std::declval<T&>())>>
         {
             using type = decltype(--std::declval<T&>());
         };
 
+        template <typename T>
+        using predecrement_result_t = typename predecrement_result<T>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename Enable = void>
         struct preincrement_result
         {
@@ -117,11 +151,15 @@ namespace hpx { namespace traits {
 
         template <typename T>
         struct preincrement_result<T,
-            typename util::always_void<decltype(++std::declval<T&>())>::type>
+            std::void_t<decltype(++std::declval<T&>())>>
         {
             using type = decltype(++std::declval<T&>());
         };
 
+        template <typename T>
+        using preincrement_result_t = typename preincrement_result<T>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename Enable = void>
         struct postdecrement_result
         {
@@ -129,11 +167,15 @@ namespace hpx { namespace traits {
 
         template <typename T>
         struct postdecrement_result<T,
-            typename util::always_void<decltype(std::declval<T&>()--)>::type>
+            std::void_t<decltype(std::declval<T&>()--)>>
         {
             using type = decltype(std::declval<T&>()--);
         };
 
+        template <typename T>
+        using postdecrement_result_t = typename postdecrement_result<T>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename Enable = void>
         struct postincrement_result
         {
@@ -141,11 +183,15 @@ namespace hpx { namespace traits {
 
         template <typename T>
         struct postincrement_result<T,
-            typename util::always_void<decltype(std::declval<T&>()++)>::type>
+            std::void_t<decltype(std::declval<T&>()++)>>
         {
             using type = decltype(std::declval<T&>()++);
         };
 
+        template <typename T>
+        using postincrement_result_t = typename postincrement_result<T>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename U, typename Enable = void>
         struct subscript_result
         {
@@ -153,12 +199,15 @@ namespace hpx { namespace traits {
 
         template <typename T, typename U>
         struct subscript_result<T, U,
-            typename util::always_void<decltype(
-                std::declval<T&>()[std::declval<U>()])>::type>
+            std::void_t<decltype(std::declval<T&>()[std::declval<U>()])>>
         {
             using type = decltype(std::declval<T&>()[std::declval<U>()]);
         };
 
+        template <typename T, typename U>
+        using subscript_result_t = typename subscript_result<T, U>::type;
+
+        ///////////////////////////////////////////////////////////////////////
         template <typename T, typename U, typename Enable = void>
         struct subtraction_result
         {
@@ -166,12 +215,14 @@ namespace hpx { namespace traits {
 
         template <typename T, typename U>
         struct subtraction_result<T, U,
-            typename util::always_void<decltype(
-                std::declval<T>() - std::declval<U>())>::type>
+            std::void_t<decltype(std::declval<T>() - std::declval<U>())>>
         {
             using type = decltype(std::declval<T>() - std::declval<U>());
         };
 
+        template <typename T, typename U>
+        using subtraction_result_t = typename subtraction_result<T, U>::type;
+
         ///////////////////////////////////////////////////////////////////////
         template <typename Iter, typename Enable = void>
         struct bidirectional_concept : std::false_type
@@ -180,26 +231,20 @@ namespace hpx { namespace traits {
 
         template <typename Iter>
         struct bidirectional_concept<Iter,
-            typename util::always_void<typename dereference_result<Iter>::type,
-                typename equality_result<Iter, Iter>::type,
-                typename inequality_result<Iter, Iter>::type,
-                typename predecrement_result<Iter>::type,
-                typename preincrement_result<Iter>::type,
-                typename postdecrement_result<Iter>::type,
-                typename postincrement_result<Iter>::type>::type>
+            std::void_t<dereference_result_t<Iter>,
+                equality_result_t<Iter, Iter>, inequality_result_t<Iter, Iter>,
+                predecrement_result_t<Iter>, preincrement_result_t<Iter>,
+                postdecrement_result_t<Iter>, postincrement_result_t<Iter>>>
           : std::integral_constant<bool,
-                std::is_convertible<bool,
-                    typename equality_result<Iter, Iter>::type>::value &&
-                    std::is_convertible<bool,
-                        typename inequality_result<Iter, Iter>::type>::value &&
-                    std::is_same<typename std::add_lvalue_reference<Iter>::type,
-                        typename predecrement_result<Iter>::type>::value &&
-                    std::is_same<typename std::add_lvalue_reference<Iter>::type,
-                        typename preincrement_result<Iter>::type>::value &&
-                    std::is_same<Iter,
-                        typename postdecrement_result<Iter>::type>::value &&
-                    std::is_same<Iter,
-                        typename postincrement_result<Iter>::type>::value>
+                std::is_convertible_v<bool, equality_result_t<Iter, Iter>> &&
+                    std::is_convertible_v<bool,
+                        inequality_result_t<Iter, Iter>> &&
+                    std::is_same_v<std::add_lvalue_reference_t<Iter>,
+                        predecrement_result_t<Iter>> &&
+                    std::is_same_v<std::add_lvalue_reference_t<Iter>,
+                        preincrement_result_t<Iter>> &&
+                    std::is_same_v<Iter, postdecrement_result_t<Iter>> &&
+                    std::is_same_v<Iter, postincrement_result_t<Iter>>>
         {
         };
 
@@ -211,44 +256,29 @@ namespace hpx { namespace traits {
 
         template <typename Iter>
         struct random_access_concept<Iter,
-            typename util::always_void<typename dereference_result<Iter>::type,
-                typename subscript_result<Iter,
-                    typename std::iterator_traits<Iter>::difference_type>::type,
-                typename addition_result<Iter,
-                    typename std::iterator_traits<Iter>::difference_type>::type,
-                typename inplace_addition_result<Iter,
-                    typename std::iterator_traits<Iter>::difference_type>::type,
-                typename subtraction_result<Iter,
-                    typename std::iterator_traits<Iter>::difference_type>::type,
-                typename subtraction_result<Iter, Iter>::type,
-                typename inplace_subtraction_result<Iter,
-                    typename std::iterator_traits<Iter>::difference_type>::
-                    type>::type>
+            std::void_t<dereference_result_t<Iter>,
+                subscript_result_t<Iter, iter_difference_t<Iter>>,
+                addition_result_t<Iter, iter_difference_t<Iter>>,
+                inplace_addition_result_t<Iter, iter_difference_t<Iter>>,
+                subtraction_result_t<Iter, iter_difference_t<Iter>>,
+                subtraction_result_t<Iter, Iter>,
+                inplace_subtraction_result_t<Iter, iter_difference_t<Iter>>>>
           : std::integral_constant<bool,
                 bidirectional_concept<Iter>::value &&
-                    std::is_same<typename dereference_result<Iter>::type,
-                        typename subscript_result<Iter,
-                            typename std::iterator_traits<
-                                Iter>::difference_type>::type>::value &&
-                    std::is_same<Iter,
-                        typename addition_result<Iter,
-                            typename std::iterator_traits<
-                                Iter>::difference_type>::type>::value &&
-                    std::is_same<typename std::add_lvalue_reference<Iter>::type,
-                        typename inplace_addition_result<Iter,
-                            typename std::iterator_traits<
-                                Iter>::difference_type>::type>::value &&
-                    std::is_same<Iter,
-                        typename subtraction_result<Iter,
-                            typename std::iterator_traits<
-                                Iter>::difference_type>::type>::value &&
-                    std::is_same<
-                        typename std::iterator_traits<Iter>::difference_type,
-                        typename subtraction_result<Iter, Iter>::type>::value &&
-                    std::is_same<typename std::add_lvalue_reference<Iter>::type,
-                        typename inplace_subtraction_result<Iter,
-                            typename std::iterator_traits<
-                                Iter>::difference_type>::type>::value>
+                    std::is_same_v<dereference_result_t<Iter>,
+                        subscript_result_t<Iter, iter_difference_t<Iter>>> &&
+                    std::is_same_v<Iter,
+                        addition_result_t<Iter, iter_difference_t<Iter>>> &&
+                    std::is_same_v<std::add_lvalue_reference_t<Iter>,
+                        inplace_addition_result_t<Iter,
+                            iter_difference_t<Iter>>> &&
+                    std::is_same_v<Iter,
+                        subtraction_result_t<Iter, iter_difference_t<Iter>>> &&
+                    std::is_same_v<iter_difference_t<Iter>,
+                        subtraction_result_t<Iter, Iter>> &&
+                    std::is_same_v<std::add_lvalue_reference_t<Iter>,
+                        inplace_subtraction_result_t<Iter,
+                            iter_difference_t<Iter>>>>
         {
         };
 
@@ -258,21 +288,18 @@ namespace hpx { namespace traits {
         {
         };
 
-        /**
-         * The interface guarantees of InputIterator and ForwardIterator
-         * concepts are not sufficient to robustly distinguish whether a given
-         * type models a forward iterator or an input iterator through interface
-         * inspection and concept emulation alone, given the type in question
-         * models an iterator category no stronger than forward.
-         *
-         * That said, a type which models the BidirectionalIterator concept
-         * also models a ForwardIterator concept, by definition (and the
-         * interface guarantees on the BidirectionalIterator concept are
-         * sufficient for robust concept checking). Here we provide a
-         * specialization to capture this case, such that, bidirectional and
-         * random access iterators will be properly recognized as satisfying the
-         * ForwardIterator concept.
-         */
+        // The interface guarantees of InputIterator and ForwardIterator
+        // concepts are not sufficient to robustly distinguish whether a given
+        // type models a forward iterator or an input iterator through interface
+        // inspection and concept emulation alone, given the type in question
+        // models an iterator category no stronger than forward.
+        //
+        // That said, a type which models the BidirectionalIterator concept also
+        // models a ForwardIterator concept, by definition (and the interface
+        // guarantees on the BidirectionalIterator concept are sufficient for
+        // robust concept checking). Here we provide a specialization to capture
+        // this case, such that, bidirectional and random access iterators will
+        // be properly recognized as satisfying the ForwardIterator concept.
         template <typename Iter>
         struct satisfy_traversal_concept<Iter, hpx::forward_traversal_tag>
           : bidirectional_concept<Iter>
@@ -290,10 +317,14 @@ namespace hpx { namespace traits {
           : random_access_concept<Iter>
         {
         };
+
+        template <typename Iter, typename TraversalTag>
+        inline constexpr bool satisfy_traversal_concept_v =
+            satisfy_traversal_concept<Iter, TraversalTag>::value;
     }    // namespace detail
 
     template <typename Iter, typename Enable = void>
-    struct is_iterator : detail::is_iterator<typename std::decay<Iter>::type>
+    struct is_iterator : detail::is_iterator<std::decay_t<Iter>>
     {
     };
 
@@ -301,11 +332,12 @@ namespace hpx { namespace traits {
     using is_iterator_t = typename is_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_iterator_v = is_iterator<Iter>::value;
+    inline constexpr bool is_iterator_v = is_iterator<Iter>::value;
 
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
 
+        ///////////////////////////////////////////////////////////////////////
         template <typename Iter, typename Cat, typename Enable = void>
         struct belongs_to_iterator_category : std::false_type
         {
@@ -313,12 +345,15 @@ namespace hpx { namespace traits {
 
         template <typename Iter, typename Cat>
         struct belongs_to_iterator_category<Iter, Cat,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
-          : std::is_base_of<Cat,
-                typename std::iterator_traits<Iter>::iterator_category>
+            std::enable_if_t<is_iterator_v<Iter>>>
+          : std::is_base_of<Cat, iter_category_t<Iter>>
         {
         };
 
+        template <typename Iter, typename Cat>
+        inline constexpr bool belongs_to_iterator_category_v =
+            belongs_to_iterator_category<Iter, Cat>::value;
+
         ///////////////////////////////////////////////////////////////////////
         template <typename Iter, typename Traversal, typename Enable = void>
         struct belongs_to_iterator_traversal : std::false_type
@@ -327,14 +362,18 @@ namespace hpx { namespace traits {
 
         template <typename Iter, typename Traversal>
         struct belongs_to_iterator_traversal<Iter, Traversal,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
+            std::enable_if_t<is_iterator_v<Iter>>>
           : std::integral_constant<bool,
-                std::is_base_of<Traversal,
-                    hpx::traits::iterator_traversal_t<Iter>>::value ||
-                    satisfy_traversal_concept<Iter, Traversal>::value>
+                std::is_base_of_v<Traversal,
+                    hpx::traits::iterator_traversal_t<Iter>> ||
+                    satisfy_traversal_concept_v<Iter, Traversal>>
         {
         };
 
+        template <typename Iter, typename Traversal>
+        inline constexpr bool belongs_to_iterator_traversal_v =
+            belongs_to_iterator_traversal<Iter, Traversal>::value;
+
         ///////////////////////////////////////////////////////////////////////
         template <typename Iter, typename Cat, typename Enable = void>
         struct has_category : std::false_type
@@ -342,13 +381,23 @@ namespace hpx { namespace traits {
         };
 
         template <typename Iter, typename Cat>
-        struct has_category<Iter, Cat,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
-          : std::is_same<Cat,
-                typename std::iterator_traits<Iter>::iterator_category>
+        struct has_category<Iter, Cat, std::enable_if_t<is_iterator_v<Iter>>>
+          : std::is_same<Cat, iter_category_t<Iter>>
         {
         };
 
+        ///////////////////////////////////////////////////////////////////////
+        std::random_access_iterator_tag coerce_iterator_tag(
+            std::random_access_iterator_tag const&);
+        std::bidirectional_iterator_tag coerce_iterator_tag(
+            std::bidirectional_iterator_tag const&);
+        std::forward_iterator_tag coerce_iterator_tag(
+            std::forward_iterator_tag const&);
+        std::input_iterator_tag coerce_iterator_tag(
+            std::input_iterator_tag const&);
+        std::output_iterator_tag coerce_iterator_tag(
+            std::output_iterator_tag const&);
+
         ///////////////////////////////////////////////////////////////////////
         template <typename Iter, typename Traversal, typename Enable = void>
         struct has_traversal : std::false_type
@@ -357,53 +406,62 @@ namespace hpx { namespace traits {
 
         template <typename Iter, typename Traversal>
         struct has_traversal<Iter, Traversal,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
+            std::enable_if_t<is_iterator_v<Iter>>>
           : std::is_same<Traversal, hpx::traits::iterator_traversal_t<Iter>>
         {
         };
 
         template <typename Iter>
         struct has_traversal<Iter, hpx::bidirectional_traversal_tag,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
+            std::enable_if_t<is_iterator_v<Iter>>>
           : std::integral_constant<bool,
-                std::is_same<hpx::bidirectional_traversal_tag,
-                    hpx::traits::iterator_traversal_t<Iter>>::value ||
-                    (satisfy_traversal_concept<Iter,
-                         hpx::bidirectional_traversal_tag>::value &&
-                        !satisfy_traversal_concept<Iter,
-                            hpx::random_access_traversal_tag>::value)>
+                std::is_same_v<hpx::bidirectional_traversal_tag,
+                    hpx::traits::iterator_traversal_t<Iter>> ||
+                    (satisfy_traversal_concept_v<Iter,
+                         hpx::bidirectional_traversal_tag> &&
+                        !satisfy_traversal_concept_v<Iter,
+                            hpx::random_access_traversal_tag>)>
         {
         };
 
         template <typename Iter>
         struct has_traversal<Iter, hpx::random_access_traversal_tag,
-            typename std::enable_if<is_iterator<Iter>::value>::type>
+            std::enable_if_t<is_iterator_v<Iter>>>
           : std::integral_constant<bool,
-                std::is_same<hpx::random_access_traversal_tag,
-                    hpx::traits::iterator_traversal_t<Iter>>::value ||
-                    satisfy_traversal_concept<Iter,
-                        hpx::random_access_traversal_tag>::value>
+                std::is_same_v<hpx::random_access_traversal_tag,
+                    hpx::traits::iterator_traversal_t<Iter>> ||
+                    satisfy_traversal_concept_v<Iter,
+                        hpx::random_access_traversal_tag>>
         {
         };
     }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Category>
-    struct has_category
-      : detail::has_category<typename std::decay<Iter>::type, Category>
-    {
-    };
+    inline constexpr bool has_category_v =
+        detail::has_category<std::decay_t<Iter>, Category>::value;
+
+    template <typename Iter, typename Traversal>
+    inline constexpr bool has_traversal_v =
+        detail::has_traversal<std::decay_t<Iter>, Traversal>::value;
+
+    template <typename Iter, typename Category>
+    inline constexpr bool belongs_to_iterator_category_v =
+        detail::belongs_to_iterator_category<std::decay_t<Iter>,
+            Category>::value;
+
+    template <typename Iter, typename Traversal>
+    inline constexpr bool belongs_to_iterator_traversal_v =
+        detail::belongs_to_iterator_traversal<std::decay_t<Iter>,
+            Traversal>::value;
 
     ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Enable = void>
     struct is_output_iterator
       : std::integral_constant<bool,
-            detail::belongs_to_iterator_category<
-                typename std::decay<Iter>::type,
-                std::output_iterator_tag>::value ||
-                detail::belongs_to_iterator_traversal<
-                    typename std::decay<Iter>::type,
-                    hpx::incrementable_traversal_tag>::value>
+            belongs_to_iterator_category_v<Iter, std::output_iterator_tag> ||
+                belongs_to_iterator_traversal_v<Iter,
+                    hpx::incrementable_traversal_tag>>
     {
     };
 
@@ -411,18 +469,16 @@ namespace hpx { namespace traits {
     using is_output_iterator_t = typename is_output_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_output_iterator_v =
+    inline constexpr bool is_output_iterator_v =
         is_output_iterator<Iter>::value;
 
+    ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Enable = void>
     struct is_input_iterator
       : std::integral_constant<bool,
-            detail::belongs_to_iterator_category<
-                typename std::decay<Iter>::type,
-                std::input_iterator_tag>::value ||
-                detail::belongs_to_iterator_traversal<
-                    typename std::decay<Iter>::type,
-                    hpx::single_pass_traversal_tag>::value>
+            belongs_to_iterator_category_v<Iter, std::input_iterator_tag> ||
+                belongs_to_iterator_traversal_v<Iter,
+                    hpx::single_pass_traversal_tag>>
     {
     };
 
@@ -430,18 +486,15 @@ namespace hpx { namespace traits {
     using is_input_iterator_t = typename is_input_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_input_iterator_v =
-        is_input_iterator<Iter>::value;
+    inline constexpr bool is_input_iterator_v = is_input_iterator<Iter>::value;
 
+    ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Enable = void>
     struct is_forward_iterator
       : std::integral_constant<bool,
-            detail::belongs_to_iterator_category<
-                typename std::decay<Iter>::type,
-                std::forward_iterator_tag>::value ||
-                detail::belongs_to_iterator_traversal<
-                    typename std::decay<Iter>::type,
-                    hpx::forward_traversal_tag>::value>
+            belongs_to_iterator_category_v<Iter, std::forward_iterator_tag> ||
+                belongs_to_iterator_traversal_v<Iter,
+                    hpx::forward_traversal_tag>>
     {
     };
 
@@ -449,18 +502,17 @@ namespace hpx { namespace traits {
     using is_forward_iterator_t = typename is_forward_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_forward_iterator_v =
+    inline constexpr bool is_forward_iterator_v =
         is_forward_iterator<Iter>::value;
 
+    ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Enable = void>
     struct is_bidirectional_iterator
       : std::integral_constant<bool,
-            detail::belongs_to_iterator_category<
-                typename std::decay<Iter>::type,
-                std::bidirectional_iterator_tag>::value ||
-                detail::belongs_to_iterator_traversal<
-                    typename std::decay<Iter>::type,
-                    hpx::bidirectional_traversal_tag>::value>
+            belongs_to_iterator_category_v<Iter,
+                std::bidirectional_iterator_tag> ||
+                belongs_to_iterator_traversal_v<Iter,
+                    hpx::bidirectional_traversal_tag>>
     {
     };
 
@@ -469,16 +521,15 @@ namespace hpx { namespace traits {
         typename is_bidirectional_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_bidirectional_iterator_v =
+    inline constexpr bool is_bidirectional_iterator_v =
         is_bidirectional_iterator<Iter>::value;
 
+    ///////////////////////////////////////////////////////////////////////////
     template <typename Iter, typename Enable = void>
     struct is_random_access_iterator
       : std::integral_constant<bool,
-            detail::has_category<typename std::decay<Iter>::type,
-                std::random_access_iterator_tag>::value ||
-                detail::has_traversal<typename std::decay<Iter>::type,
-                    hpx::random_access_traversal_tag>::value>
+            has_category_v<Iter, std::random_access_iterator_tag> ||
+                has_traversal_v<Iter, hpx::random_access_traversal_tag>>
     {
     };
 
@@ -487,7 +538,7 @@ namespace hpx { namespace traits {
         typename is_random_access_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_random_access_iterator_v =
+    inline constexpr bool is_random_access_iterator_v =
         is_random_access_iterator<Iter>::value;
 
     ///////////////////////////////////////////////////////////////////////////
@@ -498,7 +549,7 @@ namespace hpx { namespace traits {
     using is_segmented_iterator_t = typename is_segmented_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_segmented_iterator_v =
+    inline constexpr bool is_segmented_iterator_v =
         is_segmented_iterator<Iter>::value;
 
     template <typename Iterator, typename Enable = void>
@@ -509,7 +560,7 @@ namespace hpx { namespace traits {
         typename is_segmented_local_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_segmented_local_iterator_v =
+    inline constexpr bool is_segmented_local_iterator_v =
         is_segmented_local_iterator<Iter>::value;
 
     ///////////////////////////////////////////////////////////////////////////
@@ -522,53 +573,6 @@ namespace hpx { namespace traits {
     using is_zip_iterator_t = typename is_zip_iterator<Iter>::type;
 
     template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_zip_iterator_v =
-        is_zip_iterator<Iter>::value;
+    inline constexpr bool is_zip_iterator_v = is_zip_iterator<Iter>::value;
 
-    ///////////////////////////////////////////////////////////////////////////
-    // Iterators are contiguous if they are pointers (without concepts we have
-    // no generic way of determining whether an iterator is contiguous)
-
-    namespace detail {
-
-        // Iterators returned from std::vector are contiguous (bydefinition)
-        template <typename Iter,
-            typename T = typename std::iterator_traits<Iter>::value_type>
-        struct is_vector_iterator
-          : std::integral_constant<bool,
-                std::is_same<decltype(std::declval<std::vector<T>&>().begin()),
-                    Iter>::value ||
-                    std::is_same<decltype(
-                                     std::declval<std::vector<T>&>().cbegin()),
-                        Iter>::value>
-        {
-        };
-    }    // namespace detail
-
-    template <typename Iter,
-        bool not_vector = !detail::is_vector_iterator<Iter>::value>
-    struct is_contiguous_iterator : std::is_pointer<Iter>::type
-    {
-    };
-
-    template <typename Iter>
-    struct is_contiguous_iterator<Iter, false> : std::true_type
-    {
-    };
-
-    template <typename Iter>
-    using is_contiguous_iterator_t =
-        typename is_contiguous_iterator<Iter>::type;
-
-    template <typename Iter>
-    HPX_INLINE_CONSTEXPR_VARIABLE bool is_contiguous_iterator_v =
-        is_contiguous_iterator<Iter>::value;
-
-    ///////////////////////////////////////////////////////////////////////////
-    template <typename Iter>
-    using iter_value_t = typename std::iterator_traits<Iter>::value_type;
-
-    template <typename Iter>
-    using iter_ref_t = typename std::iterator_traits<Iter>::reference;
-
-}}    // namespace hpx::traits
+}    // namespace hpx::traits
