@@ -4,11 +4,11 @@
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <hpx/local/init.hpp>
-#include <hpx/local/tuple.hpp>
+#include <hpx/chrono.hpp>
+#include <hpx/init.hpp>
 #include <hpx/modules/iterator_support.hpp>
 #include <hpx/modules/testing.hpp>
-#include <hpx/modules/timing.hpp>
+#include <hpx/tuple.hpp>
 
 #include <cstddef>
 #include <cstdint>
@@ -19,14 +19,12 @@
 #include <utility>
 #include <vector>
 
-#include <boost/range/irange.hpp>
-
 ///////////////////////////////////////////////////////////////////////////////
 int test_count = 100;
 int partition_size = 10000;
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace experimental { namespace detail {
+namespace hpx::experimental::detail {
     template <typename Iterator>
     HPX_FORCEINLINE Iterator previous(Iterator it, std::false_type)
     {
@@ -62,11 +60,11 @@ namespace hpx { namespace experimental { namespace detail {
     {
         return next(it, hpx::traits::is_random_access_iterator<Iterator>());
     }
-}}}    // namespace hpx::experimental::detail
+}    // namespace hpx::experimental::detail
 
 ///////////////////////////////////////////////////////////////////////////////
 // Version of stencil3_iterator which handles boundary elements internally
-namespace hpx { namespace experimental {
+namespace hpx::experimental {
     ///////////////////////////////////////////////////////////////////////////
     template <typename Iterator, typename IterBegin = Iterator,
         typename IterValueBegin = Iterator, typename IterEnd = IterBegin,
@@ -156,12 +154,12 @@ namespace hpx { namespace experimental {
                 left_transformer;
             typedef next_transformer<IterEnd, IterValueEnd> right_transformer;
 
-            typedef util::transform_iterator<Iterator, left_transformer>
+            typedef ::hpx::util::transform_iterator<Iterator, left_transformer>
                 left_iterator;
-            typedef util::transform_iterator<Iterator, right_transformer>
+            typedef ::hpx::util::transform_iterator<Iterator, right_transformer>
                 right_iterator;
 
-            typedef util::detail::zip_iterator_base<
+            typedef ::hpx::util::detail::zip_iterator_base<
                 hpx::tuple<left_iterator, Iterator, right_iterator>,
                 stencil3_iterator_full<Iterator, IterBegin, IterValueBegin,
                     IterEnd, IterValueEnd>>
@@ -175,15 +173,15 @@ namespace hpx { namespace experimental {
                 auto next = make_next_transformer(end, end_val);
 
                 return type(
-                    hpx::make_tuple(util::make_transform_iterator(it, prev), it,
-                        util::make_transform_iterator(it, next)));
+                    hpx::make_tuple(::hpx::util::transform_iterator(it, prev),
+                        it, ::hpx::util::transform_iterator(it, next)));
             }
 
             static type create(Iterator const& it)
             {
                 return type(hpx::make_tuple(
-                    util::make_transform_iterator(it, left_transformer()), it,
-                    util::make_transform_iterator(it, right_transformer())));
+                    ::hpx::util::transform_iterator(it, left_transformer()), it,
+                    ::hpx::util::transform_iterator(it, right_transformer())));
             }
         };
     }    // namespace detail
@@ -258,7 +256,7 @@ namespace hpx { namespace experimental {
             begin, begin, begin_val, detail::previous(end), end_val);
         return std::make_pair(b, make_stencil3_full_iterator<decltype(b)>(end));
     }
-}}    // namespace hpx::experimental
+}    // namespace hpx::experimental
 
 std::uint64_t bench_stencil3_iterator_full()
 {
@@ -284,15 +282,15 @@ std::uint64_t bench_stencil3_iterator_full()
 
 ///////////////////////////////////////////////////////////////////////////////
 // compare with unchecked stencil3_iterator (version 1)
-namespace hpx { namespace experimental {
+namespace hpx::experimental {
     template <typename Iterator>
     class stencil3_iterator_v1
-      : public util::detail::zip_iterator_base<
+      : public ::hpx::util::detail::zip_iterator_base<
             hpx::tuple<Iterator, Iterator, Iterator>,
             stencil3_iterator_v1<Iterator>>
     {
     private:
-        typedef util::detail::zip_iterator_base<
+        typedef ::hpx::util::detail::zip_iterator_base<
             hpx::tuple<Iterator, Iterator, Iterator>,
             stencil3_iterator_v1<Iterator>>
             base_type;
@@ -331,7 +329,7 @@ namespace hpx { namespace experimental {
         return std::make_pair(
             make_stencil3_iterator_v1(begin), make_stencil3_iterator_v1(end));
     }
-}}    // namespace hpx::experimental
+}    // namespace hpx::experimental
 
 std::uint64_t bench_stencil3_iterator_v1()
 {
@@ -361,7 +359,7 @@ std::uint64_t bench_stencil3_iterator_v1()
 
 ///////////////////////////////////////////////////////////////////////////////
 // compare with unchecked stencil3_iterator (version 2)
-namespace hpx { namespace experimental {
+namespace hpx::experimental {
     namespace detail {
         struct stencil_transformer_v2
         {
@@ -388,10 +386,11 @@ namespace hpx { namespace experimental {
     template <typename Iterator,
         typename Transformer = detail::stencil_transformer_v2>
     class stencil3_iterator_v2
-      : public hpx::util::transform_iterator<Iterator, Transformer>
+      : public ::hpx::util::transform_iterator<Iterator, Transformer>
     {
     private:
-        typedef hpx::util::transform_iterator<Iterator, Transformer> base_type;
+        typedef ::hpx::util::transform_iterator<Iterator, Transformer>
+            base_type;
 
     public:
         stencil3_iterator_v2() {}
@@ -440,7 +439,7 @@ namespace hpx { namespace experimental {
         return std::make_pair(
             make_stencil3_iterator_v2(begin), make_stencil3_iterator_v2(end));
     }
-}}    // namespace hpx::experimental
+}    // namespace hpx::experimental
 
 std::uint64_t bench_stencil3_iterator_v2()
 {
@@ -479,7 +478,7 @@ std::uint64_t bench_stencil3_iterator_explicit()
     // handle all elements explicitly
     int result = values.back() + values.front() + values[1];
 
-    auto range = boost::irange(1, partition_size - 1);
+    auto range = hpx::util::counting_shape(1, partition_size - 1);
 
     std::for_each(
         std::begin(range), std::end(range), [&result, &values](std::size_t i) {
