@@ -10,8 +10,9 @@
 
 #include <string>
 
-namespace hpx { namespace detail {
+namespace hpx::detail {
+
     /// Helper utility to set and store a name for the current operating system
     /// thread. Returns a reference to the name for the current thread.
-    HPX_CORE_EXPORT std::string& thread_name();
-}}    // namespace hpx::detail
+    [[nodiscard]] HPX_CORE_EXPORT std::string& thread_name();
+}    // namespace hpx::detail
