@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -89,8 +89,8 @@ HPX_CORE_EXPORT extern bool use_ittnotify_api;
 
 ///////////////////////////////////////////////////////////////////////////////
 HPX_CORE_EXPORT void itt_sync_create(
-    void* addr, const char* objtype, const char* objname) noexcept;
-HPX_CORE_EXPORT void itt_sync_rename(void* addr, const char* name) noexcept;
+    void* addr, char const* objtype, char const* objname) noexcept;
+HPX_CORE_EXPORT void itt_sync_rename(void* addr, char const* name) noexcept;
 HPX_CORE_EXPORT void itt_sync_prepare(void* addr) noexcept;
 HPX_CORE_EXPORT void itt_sync_acquired(void* addr) noexcept;
 HPX_CORE_EXPORT void itt_sync_cancel(void* addr) noexcept;
@@ -98,7 +98,7 @@ HPX_CORE_EXPORT void itt_sync_releasing(void* addr) noexcept;
 HPX_CORE_EXPORT void itt_sync_released(void* addr) noexcept;
 HPX_CORE_EXPORT void itt_sync_destroy(void* addr) noexcept;
 
-HPX_CORE_EXPORT ___itt_caller* itt_stack_create() noexcept;
+[[nodiscard]] HPX_CORE_EXPORT ___itt_caller* itt_stack_create() noexcept;
 HPX_CORE_EXPORT void itt_stack_enter(___itt_caller* ctx) noexcept;
 HPX_CORE_EXPORT void itt_stack_leave(___itt_caller* ctx) noexcept;
 HPX_CORE_EXPORT void itt_stack_destroy(___itt_caller* ctx) noexcept;
@@ -108,7 +108,7 @@ HPX_CORE_EXPORT void itt_frame_begin(
 HPX_CORE_EXPORT void itt_frame_end(
     ___itt_domain const* frame, ___itt_id* id) noexcept;
 
-HPX_CORE_EXPORT int itt_mark_create(char const*) noexcept;
+[[nodiscard]] HPX_CORE_EXPORT int itt_mark_create(char const*) noexcept;
 HPX_CORE_EXPORT void itt_mark_off(int mark) noexcept;
 HPX_CORE_EXPORT void itt_mark(int mark, char const*) noexcept;
 
@@ -125,13 +125,13 @@ HPX_CORE_EXPORT ___itt_domain* itt_domain_create(char const*) noexcept;
 HPX_CORE_EXPORT ___itt_string_handle* itt_string_handle_create(
     char const*) noexcept;
 
-HPX_CORE_EXPORT ___itt_id* itt_make_id(void*, std::size_t);
+[[nodiscard]] HPX_CORE_EXPORT ___itt_id* itt_make_id(void*, std::size_t);
 HPX_CORE_EXPORT void itt_id_create(
     ___itt_domain const*, ___itt_id* id) noexcept;
 HPX_CORE_EXPORT void itt_id_destroy(___itt_id* id) noexcept;
 
-HPX_CORE_EXPORT __itt_heap_function itt_heap_function_create(
-    const char*, const char*) noexcept;
+[[nodiscard]] HPX_CORE_EXPORT __itt_heap_function itt_heap_function_create(
+    char const*, char const*) noexcept;
 HPX_CORE_EXPORT void itt_heap_allocate_begin(
     __itt_heap_function, std::size_t, int) noexcept;
 HPX_CORE_EXPORT void itt_heap_allocate_end(
@@ -145,14 +145,15 @@ HPX_CORE_EXPORT void itt_heap_reallocate_end(
 HPX_CORE_EXPORT void itt_heap_internal_access_begin() noexcept;
 HPX_CORE_EXPORT void itt_heap_internal_access_end() noexcept;
 
-HPX_CORE_EXPORT ___itt_counter* itt_counter_create(
+[[nodiscard]] HPX_CORE_EXPORT ___itt_counter* itt_counter_create(
     char const*, char const*) noexcept;
-HPX_CORE_EXPORT ___itt_counter* itt_counter_create_typed(
+[[nodiscard]] HPX_CORE_EXPORT ___itt_counter* itt_counter_create_typed(
     char const*, char const*, int) noexcept;
 HPX_CORE_EXPORT void itt_counter_destroy(___itt_counter*) noexcept;
 HPX_CORE_EXPORT void itt_counter_set_value(___itt_counter*, void*) noexcept;
 
-HPX_CORE_EXPORT int itt_event_create(char const* name, int namelen) noexcept;
+[[nodiscard]] HPX_CORE_EXPORT int itt_event_create(
+    char const* name, int namelen) noexcept;
 HPX_CORE_EXPORT int itt_event_start(int evnt) noexcept;
 HPX_CORE_EXPORT int itt_event_end(int evnt) noexcept;
 
@@ -166,35 +167,27 @@ HPX_CORE_EXPORT void itt_metadata_add(___itt_domain* domain, ___itt_id* id,
     ___itt_string_handle* key, void const* data) noexcept;
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace util {
+namespace hpx::threads {
 
     struct thread_description;
-}}    // namespace hpx::util
+}    // namespace hpx::threads
 
-namespace hpx { namespace util { namespace itt {
+namespace hpx::util::itt {
 
     struct stack_context
     {
-        stack_context()
-          : itt_context_(nullptr)
-        {
-            HPX_ITT_STACK_CREATE(itt_context_);
-        }
-        ~stack_context()
-        {
-            if (itt_context_)
-                HPX_ITT_STACK_DESTROY(itt_context_);
-        }
+        HPX_CORE_EXPORT stack_context();
+        HPX_CORE_EXPORT ~stack_context();
 
         stack_context(stack_context const& rhs) = delete;
-        stack_context(stack_context&& rhs)
+        stack_context(stack_context&& rhs) noexcept
           : itt_context_(rhs.itt_context_)
         {
             rhs.itt_context_ = nullptr;
         }
 
         stack_context& operator=(stack_context const& rhs) = delete;
-        stack_context& operator=(stack_context&& rhs)
+        stack_context& operator=(stack_context&& rhs) noexcept
         {
             if (this != &rhs)
             {
@@ -204,20 +197,13 @@ namespace hpx { namespace util { namespace itt {
             return *this;
         }
 
-        struct ___itt_caller* itt_context_;
+        struct ___itt_caller* itt_context_ = nullptr;
     };
 
     struct caller_context
     {
-        caller_context(stack_context& ctx)
-          : ctx_(ctx)
-        {
-            HPX_ITT_STACK_CALLEE_ENTER(ctx_.itt_context_);
-        }
-        ~caller_context()
-        {
-            HPX_ITT_STACK_CALLEE_LEAVE(ctx_.itt_context_);
-        }
+        HPX_CORE_EXPORT explicit caller_context(stack_context& ctx);
+        HPX_CORE_EXPORT ~caller_context();
 
         stack_context& ctx_;
     };
@@ -227,10 +213,10 @@ namespace hpx { namespace util { namespace itt {
     {
         HPX_NON_COPYABLE(domain);
 
-        HPX_CORE_EXPORT domain(char const*) noexcept;
-        HPX_CORE_EXPORT domain() noexcept;
+        domain() = default;
+        HPX_CORE_EXPORT explicit domain(char const*) noexcept;
 
-        ___itt_domain* domain_;
+        ___itt_domain* domain_ = nullptr;
     };
 
     struct thread_domain : domain
@@ -242,15 +228,9 @@ namespace hpx { namespace util { namespace itt {
 
     struct id
     {
-        id(domain const& domain, void* addr, unsigned long extra = 0) noexcept
-        {
-            id_ = HPX_ITT_MAKE_ID(addr, extra);
-            HPX_ITT_ID_CREATE(domain.domain_, id_);
-        }
-        ~id()
-        {
-            HPX_ITT_ID_DESTROY(id_);
-        }
+        HPX_CORE_EXPORT id(
+            domain const& domain, void* addr, unsigned long extra = 0) noexcept;
+        HPX_CORE_EXPORT ~id();
 
         id(id const& rhs) = delete;
         id(id&& rhs) noexcept
@@ -270,39 +250,25 @@ namespace hpx { namespace util { namespace itt {
             return *this;
         }
 
-        ___itt_id* id_;
+        ___itt_id* id_ = nullptr;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct frame_context
     {
-        frame_context(domain const& domain, id* ident = nullptr) noexcept
-          : domain_(domain)
-          , ident_(ident)
-        {
-            HPX_ITT_FRAME_BEGIN(
-                domain_.domain_, ident_ ? ident_->id_ : nullptr);
-        }
-        ~frame_context()
-        {
-            HPX_ITT_FRAME_END(domain_.domain_, ident_ ? ident_->id_ : nullptr);
-        }
+        HPX_CORE_EXPORT explicit frame_context(
+            domain const& domain, id* ident = nullptr) noexcept;
+        HPX_CORE_EXPORT ~frame_context();
 
         domain const& domain_;
-        id* ident_;
+        id* ident_ = nullptr;
     };
 
     struct undo_frame_context
     {
-        undo_frame_context(frame_context& frame) noexcept
-          : frame_(frame)
-        {
-            HPX_ITT_FRAME_END(frame_.domain_.domain_, nullptr);
-        }
-        ~undo_frame_context()
-        {
-            HPX_ITT_FRAME_BEGIN(frame_.domain_.domain_, nullptr);
-        }
+        HPX_CORE_EXPORT explicit undo_frame_context(
+            frame_context& frame) noexcept;
+        HPX_CORE_EXPORT ~undo_frame_context();
 
         frame_context& frame_;
     };
@@ -310,32 +276,17 @@ namespace hpx { namespace util { namespace itt {
     ///////////////////////////////////////////////////////////////////////////
     struct mark_context
     {
-        mark_context(char const* name) noexcept
-          : itt_mark_(0)
-          , name_(name)
-        {
-            HPX_ITT_MARK_CREATE(itt_mark_, name);
-        }
-        ~mark_context()
-        {
-            HPX_ITT_MARK_OFF(itt_mark_);
-        }
+        HPX_CORE_EXPORT explicit mark_context(char const* name) noexcept;
+        HPX_CORE_EXPORT ~mark_context();
 
         int itt_mark_;
-        char const* name_;
+        char const* name_ = nullptr;
     };
 
     struct undo_mark_context
     {
-        undo_mark_context(mark_context& mark) noexcept
-          : mark_(mark)
-        {
-            HPX_ITT_MARK_OFF(mark_.itt_mark_);
-        }
-        ~undo_mark_context()
-        {
-            HPX_ITT_MARK_CREATE(mark_.itt_mark_, mark_.name_);
-        }
+        HPX_CORE_EXPORT explicit undo_mark_context(mark_context& mark) noexcept;
+        HPX_CORE_EXPORT ~undo_mark_context();
 
         mark_context& mark_;
     };
@@ -343,31 +294,45 @@ namespace hpx { namespace util { namespace itt {
     ///////////////////////////////////////////////////////////////////////////
     struct string_handle
     {
-        string_handle() noexcept
-          : handle_(nullptr)
-        {
-        }
-        string_handle(char const* s) noexcept
-          : handle_(s == nullptr ? nullptr : HPX_ITT_STRING_HANDLE_CREATE(s))
-        {
-        }
-        string_handle(___itt_string_handle* h) noexcept
+        string_handle() noexcept = default;
+
+        HPX_CORE_EXPORT explicit string_handle(char const* s) noexcept;
+
+        explicit string_handle(___itt_string_handle* h) noexcept
           : handle_(h)
         {
         }
 
+        string_handle(string_handle const&) = default;
+        string_handle(string_handle&& rhs)
+          : handle_(rhs.handle_)
+        {
+            rhs.handle_ = nullptr;
+        }
+
+        string_handle& operator=(string_handle const&) = default;
+        string_handle& operator=(string_handle&& rhs) noexcept
+        {
+            if (this != &rhs)
+            {
+                handle_ = rhs.handle_;
+                rhs.handle_ = nullptr;
+            }
+            return *this;
+        }
+
         string_handle& operator=(___itt_string_handle* h) noexcept
         {
             handle_ = h;
             return *this;
         }
 
-        explicit operator bool() const noexcept
+        explicit constexpr operator bool() const noexcept
         {
             return handle_ != nullptr;
         }
 
-        ___itt_string_handle* handle_;
+        ___itt_string_handle* handle_ = nullptr;
     };
 
     ///////////////////////////////////////////////////////////////////////////
@@ -378,52 +343,39 @@ namespace hpx { namespace util { namespace itt {
         HPX_CORE_EXPORT task(domain const&, string_handle const&) noexcept;
         HPX_CORE_EXPORT ~task();
 
-        void add_metadata(string_handle const& name, std::uint64_t val) noexcept
-        {
-            HPX_ITT_METADATA_ADD(domain_.domain_, id_, name.handle_, val);
-        }
-        void add_metadata(string_handle const& name, double val) noexcept
-        {
-            HPX_ITT_METADATA_ADD(domain_.domain_, id_, name.handle_, val);
-        }
-        void add_metadata(string_handle const& name, char const* val) noexcept
-        {
-            HPX_ITT_METADATA_ADD(domain_.domain_, id_, name.handle_, val);
-        }
+        HPX_CORE_EXPORT void add_metadata(
+            string_handle const& name, std::uint64_t val) noexcept;
+        HPX_CORE_EXPORT void add_metadata(
+            string_handle const& name, double val) noexcept;
+        HPX_CORE_EXPORT void add_metadata(
+            string_handle const& name, char const* val) noexcept;
+        HPX_CORE_EXPORT void add_metadata(
+            string_handle const& name, void const* val) noexcept;
+
         template <typename T>
         void add_metadata(string_handle const& name, T const& val) noexcept
         {
-            HPX_ITT_METADATA_ADD(domain_.domain_, id_, name.handle_,
-                static_cast<void const*>(&val));
+            add_metadata(name, static_cast<void const*>(&val));
         }
 
         domain const& domain_;
-        ___itt_id* id_;
+        ___itt_id* id_ = nullptr;
         string_handle sh_;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct heap_function
     {
-        heap_function(char const* name, char const* domain) noexcept
-          : heap_function_(HPX_ITT_HEAP_FUNCTION_CREATE(name, domain))
-        {
-        }
+        HPX_CORE_EXPORT heap_function(
+            char const* name, char const* domain) noexcept;
 
-        __itt_heap_function heap_function_;
+        __itt_heap_function heap_function_ = nullptr;
     };
 
     struct heap_internal_access
     {
-        heap_internal_access() noexcept
-        {
-            HPX_ITT_HEAP_INTERNAL_ACCESS_BEGIN();
-        }
-
-        ~heap_internal_access()
-        {
-            HPX_ITT_HEAP_INTERNAL_ACCESS_END();
-        }
+        HPX_CORE_EXPORT heap_internal_access() noexcept;
+        HPX_CORE_EXPORT ~heap_internal_access();
     };
 
     struct heap_allocate
@@ -436,14 +388,20 @@ namespace hpx { namespace util { namespace itt {
           , size_(size)
           , init_(init)
         {
-            HPX_ITT_HEAP_ALLOCATE_BEGIN(
-                heap_function_.heap_function_, size_, init_);
+            if (use_ittnotify_api)
+            {
+                HPX_ITT_HEAP_ALLOCATE_BEGIN(
+                    heap_function_.heap_function_, size_, init_);
+            }
         }
 
         ~heap_allocate()
         {
-            HPX_ITT_HEAP_ALLOCATE_END(
-                heap_function_.heap_function_, addr_, size_, init_);
+            if (use_ittnotify_api)
+            {
+                HPX_ITT_HEAP_ALLOCATE_END(
+                    heap_function_.heap_function_, addr_, size_, init_);
+            }
         }
 
     private:
@@ -455,17 +413,9 @@ namespace hpx { namespace util { namespace itt {
 
     struct heap_free
     {
-        heap_free(heap_function& heap_function, void* addr) noexcept
-          : heap_function_(heap_function)
-          , addr_(addr)
-        {
-            HPX_ITT_HEAP_FREE_BEGIN(heap_function_.heap_function_, addr_);
-        }
-
-        ~heap_free()
-        {
-            HPX_ITT_HEAP_FREE_END(heap_function_.heap_function_, addr_);
-        }
+        HPX_CORE_EXPORT heap_free(
+            heap_function& heap_function, void* addr) noexcept;
+        HPX_CORE_EXPORT ~heap_free();
 
     private:
         heap_function& heap_function_;
@@ -475,27 +425,18 @@ namespace hpx { namespace util { namespace itt {
     ///////////////////////////////////////////////////////////////////////////
     struct counter
     {
-        counter(char const* name, char const* domain) noexcept
-          : id_(HPX_ITT_COUNTER_CREATE(name, domain))
-        {
-        }
-        counter(char const* name, char const* domain, int type) noexcept
-          : id_(HPX_ITT_COUNTER_CREATE_TYPED(name, domain, type))
-        {
-        }
-        ~counter()
-        {
-            if (id_)
-                HPX_ITT_COUNTER_DESTROY(id_);
-        }
+        HPX_CORE_EXPORT counter(char const* name, char const* domain) noexcept;
+        HPX_CORE_EXPORT counter(
+            char const* name, char const* domain, int type) noexcept;
+        HPX_CORE_EXPORT ~counter();
 
         template <typename T>
         void set_value(T const& value) noexcept
         {
-            if (id_)
+            if (use_ittnotify_api && id_)
             {
                 HPX_ITT_COUNTER_SET_VALUE(
-                    id_, const_cast<void*>(static_cast<const void*>(&value)));
+                    id_, const_cast<void*>(static_cast<void const*>(&value)));
             }
         }
 
@@ -518,13 +459,13 @@ namespace hpx { namespace util { namespace itt {
         }
 
     private:
-        ___itt_counter* id_;
+        ___itt_counter* id_ = nullptr;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct event
     {
-        event(char const* name) noexcept
+        explicit event(char const* name) noexcept
           : event_(itt_event_create(name, (int) strnlen(name, 256)))
         {
         }
@@ -540,12 +481,12 @@ namespace hpx { namespace util { namespace itt {
         }
 
     private:
-        int event_;
+        int event_ = 0;
     };
 
     struct mark_event
     {
-        mark_event(event const& e) noexcept
+        explicit mark_event(event const& e) noexcept
           : e_(e)
         {
             e_.start();
@@ -563,153 +504,143 @@ namespace hpx { namespace util { namespace itt {
     {
         e.start();
     }
-}}}    // namespace hpx::util::itt
+}    // namespace hpx::util::itt
 
 #else
 
-inline constexpr void itt_sync_create(void*, const char*, const char*) noexcept
-{
-}
-inline constexpr void itt_sync_rename(void*, const char*) noexcept {}
-inline constexpr void itt_sync_prepare(void*) noexcept {}
-inline constexpr void itt_sync_acquired(void*) noexcept {}
-inline constexpr void itt_sync_cancel(void*) noexcept {}
-inline constexpr void itt_sync_releasing(void*) noexcept {}
-inline constexpr void itt_sync_released(void*) noexcept {}
-inline constexpr void itt_sync_destroy(void*) noexcept {}
+constexpr void itt_sync_create(void*, char const*, char const*) noexcept {}
+constexpr void itt_sync_rename(void*, char const*) noexcept {}
+constexpr void itt_sync_prepare(void*) noexcept {}
+constexpr void itt_sync_acquired(void*) noexcept {}
+constexpr void itt_sync_cancel(void*) noexcept {}
+constexpr void itt_sync_releasing(void*) noexcept {}
+constexpr void itt_sync_released(void*) noexcept {}
+constexpr void itt_sync_destroy(void*) noexcept {}
 
-inline constexpr ___itt_caller* itt_stack_create() noexcept
+[[nodiscard]] constexpr ___itt_caller* itt_stack_create() noexcept
 {
     return nullptr;
 }
-inline constexpr void itt_stack_enter(___itt_caller*) noexcept {}
-inline constexpr void itt_stack_leave(___itt_caller*) noexcept {}
-inline constexpr void itt_stack_destroy(___itt_caller*) noexcept {}
+constexpr void itt_stack_enter(___itt_caller*) noexcept {}
+constexpr void itt_stack_leave(___itt_caller*) noexcept {}
+constexpr void itt_stack_destroy(___itt_caller*) noexcept {}
 
-inline constexpr void itt_frame_begin(___itt_domain const*, ___itt_id*) noexcept
-{
-}
-inline constexpr void itt_frame_end(___itt_domain const*, ___itt_id*) noexcept
-{
-}
+constexpr void itt_frame_begin(___itt_domain const*, ___itt_id*) noexcept {}
+constexpr void itt_frame_end(___itt_domain const*, ___itt_id*) noexcept {}
 
-inline constexpr int itt_mark_create(char const*) noexcept
+[[nodiscard]] constexpr int itt_mark_create(char const*) noexcept
 {
     return 0;
 }
-inline constexpr void itt_mark_off(int) noexcept {}
-inline constexpr void itt_mark(int, char const*) noexcept {}
+constexpr void itt_mark_off(int) noexcept {}
+constexpr void itt_mark(int, char const*) noexcept {}
 
-inline constexpr void itt_thread_set_name(char const*) noexcept {}
-inline constexpr void itt_thread_ignore() noexcept {}
+constexpr void itt_thread_set_name(char const*) noexcept {}
+constexpr void itt_thread_ignore() noexcept {}
 
-inline constexpr void itt_task_begin(
+constexpr void itt_task_begin(
     ___itt_domain const*, ___itt_string_handle*) noexcept
 {
 }
-inline constexpr void itt_task_begin(
+constexpr void itt_task_begin(
     ___itt_domain const*, ___itt_id*, ___itt_string_handle*) noexcept
 {
 }
-inline constexpr void itt_task_end(___itt_domain const*) noexcept {}
+constexpr void itt_task_end(___itt_domain const*) noexcept {}
 
-inline constexpr ___itt_domain* itt_domain_create(char const*) noexcept
+[[nodiscard]] constexpr ___itt_domain* itt_domain_create(char const*) noexcept
 {
     return nullptr;
 }
-inline constexpr ___itt_string_handle* itt_string_handle_create(
+[[nodiscard]] constexpr ___itt_string_handle* itt_string_handle_create(
     char const*) noexcept
 {
     return nullptr;
 }
 
-inline constexpr ___itt_id* itt_make_id(void*, unsigned long)
+[[nodiscard]] constexpr ___itt_id* itt_make_id(void*, unsigned long)
 {
     return nullptr;
 }
-inline constexpr void itt_id_create(___itt_domain const*, ___itt_id*) noexcept
-{
-}
-inline constexpr void itt_id_destroy(___itt_id*) noexcept {}
+constexpr void itt_id_create(___itt_domain const*, ___itt_id*) noexcept {}
+constexpr void itt_id_destroy(___itt_id*) noexcept {}
 
-inline constexpr __itt_heap_function itt_heap_function_create(
-    const char*, const char*) noexcept
-{
-    return nullptr;
-}
-inline constexpr void itt_heap_allocate_begin(
-    __itt_heap_function, std::size_t, int) noexcept
-{
-}
-inline constexpr void itt_heap_allocate_end(
-    __itt_heap_function, void**, std::size_t, int) noexcept
-{
-}
-inline constexpr void itt_heap_free_begin(__itt_heap_function, void*) noexcept
-{
-}
-inline constexpr void itt_heap_free_end(__itt_heap_function, void*) noexcept {}
-inline constexpr void itt_heap_reallocate_begin(
-    __itt_heap_function, void*, std::size_t, int) noexcept
-{
-}
-inline constexpr void itt_heap_reallocate_end(
-    __itt_heap_function, void*, void**, std::size_t, int) noexcept
-{
-}
-inline constexpr void itt_heap_internal_access_begin() noexcept {}
-inline constexpr void itt_heap_internal_access_end() noexcept {}
-
-inline constexpr ___itt_counter* itt_counter_create(
+[[nodiscard]] constexpr __itt_heap_function itt_heap_function_create(
     char const*, char const*) noexcept
 {
     return nullptr;
 }
-inline constexpr ___itt_counter* itt_counter_create_typed(
+constexpr void itt_heap_allocate_begin(
+    __itt_heap_function, std::size_t, int) noexcept
+{
+}
+constexpr void itt_heap_allocate_end(
+    __itt_heap_function, void**, std::size_t, int) noexcept
+{
+}
+constexpr void itt_heap_free_begin(__itt_heap_function, void*) noexcept {}
+constexpr void itt_heap_free_end(__itt_heap_function, void*) noexcept {}
+constexpr void itt_heap_reallocate_begin(
+    __itt_heap_function, void*, std::size_t, int) noexcept
+{
+}
+constexpr void itt_heap_reallocate_end(
+    __itt_heap_function, void*, void**, std::size_t, int) noexcept
+{
+}
+constexpr void itt_heap_internal_access_begin() noexcept {}
+constexpr void itt_heap_internal_access_end() noexcept {}
+
+[[nodiscard]] constexpr ___itt_counter* itt_counter_create(
+    char const*, char const*) noexcept
+{
+    return nullptr;
+}
+[[nodiscard]] constexpr ___itt_counter* itt_counter_create_typed(
     char const*, char const*, int) noexcept
 {
     return nullptr;
 }
-inline constexpr void itt_counter_destroy(___itt_counter*) noexcept {}
-inline constexpr void itt_counter_set_value(___itt_counter*, void*) noexcept {}
+constexpr void itt_counter_destroy(___itt_counter*) noexcept {}
+constexpr void itt_counter_set_value(___itt_counter*, void*) noexcept {}
 
-inline constexpr int itt_event_create(char const*, int) noexcept
+[[nodiscard]] constexpr int itt_event_create(char const*, int) noexcept
 {
     return 0;
 }
-inline constexpr int itt_event_start(int) noexcept
+constexpr int itt_event_start(int) noexcept
 {
     return 0;
 }
-inline constexpr int itt_event_end(int) noexcept
+constexpr int itt_event_end(int) noexcept
 {
     return 0;
 }
 
-inline constexpr void itt_metadata_add(___itt_domain*, ___itt_id*,
-    ___itt_string_handle*, std::uint64_t const&) noexcept
+constexpr void itt_metadata_add(
+    ___itt_domain*, ___itt_id*, ___itt_string_handle*, std::uint64_t) noexcept
 {
 }
-inline constexpr void itt_metadata_add(
-    ___itt_domain*, ___itt_id*, ___itt_string_handle*, double const&) noexcept
+constexpr void itt_metadata_add(
+    ___itt_domain*, ___itt_id*, ___itt_string_handle*, double) noexcept
 {
 }
-inline constexpr void itt_metadata_add(
+constexpr void itt_metadata_add(
     ___itt_domain*, ___itt_id*, ___itt_string_handle*, char const*) noexcept
 {
 }
-inline constexpr void itt_metadata_add(
+constexpr void itt_metadata_add(
     ___itt_domain*, ___itt_id*, ___itt_string_handle*, void const*) noexcept
 {
 }
 
 //////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace util {
+namespace hpx::threads {
 
     struct thread_description;
-}}    // namespace hpx::util
+}    // namespace hpx::threads
 
-namespace hpx { namespace util { namespace itt {
+namespace hpx::util::itt {
 
     struct stack_context
     {
@@ -719,7 +650,7 @@ namespace hpx { namespace util { namespace itt {
 
     struct caller_context
     {
-        constexpr caller_context(stack_context&) noexcept {}
+        constexpr explicit caller_context(stack_context&) noexcept {}
         ~caller_context() = default;
     };
 
@@ -728,8 +659,9 @@ namespace hpx { namespace util { namespace itt {
     {
         HPX_NON_COPYABLE(domain);
 
-        constexpr domain(char const*) noexcept {}
+        constexpr explicit domain(char const*) noexcept {}
         domain() = default;
+        ~domain() = default;
     };
 
     struct thread_domain : domain
@@ -737,6 +669,7 @@ namespace hpx { namespace util { namespace itt {
         HPX_NON_COPYABLE(thread_domain);
 
         thread_domain() = default;
+        ~thread_domain() = default;
     };
 
     struct id
@@ -748,33 +681,35 @@ namespace hpx { namespace util { namespace itt {
     ///////////////////////////////////////////////////////////////////////////
     struct frame_context
     {
-        constexpr frame_context(domain const&, id* = nullptr) noexcept {}
+        constexpr explicit frame_context(domain const&, id* = nullptr) noexcept
+        {
+        }
         ~frame_context() = default;
     };
 
     struct undo_frame_context
     {
-        constexpr undo_frame_context(frame_context const&) noexcept {}
+        constexpr explicit undo_frame_context(frame_context const&) noexcept {}
         ~undo_frame_context() = default;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct mark_context
     {
-        constexpr mark_context(char const*) noexcept {}
+        constexpr explicit mark_context(char const*) noexcept {}
         ~mark_context() = default;
     };
 
     struct undo_mark_context
     {
-        constexpr undo_mark_context(mark_context const&) noexcept {}
+        constexpr explicit undo_mark_context(mark_context const&) noexcept {}
         ~undo_mark_context() = default;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct string_handle
     {
-        constexpr string_handle(char const* = nullptr) noexcept {}
+        constexpr explicit string_handle(char const* = nullptr) noexcept {}
     };
 
     //////////////////////////////////////////////////////////////////////////
@@ -828,16 +763,16 @@ namespace hpx { namespace util { namespace itt {
 
     struct event
     {
-        constexpr event(char const*) noexcept {}
+        constexpr explicit event(char const*) noexcept {}
     };
 
     struct mark_event
     {
-        constexpr mark_event(event const&) noexcept {}
+        constexpr explicit mark_event(event const&) noexcept {}
         ~mark_event() = default;
     };
 
-    inline constexpr void event_tick(event const&) noexcept {}
-}}}    // namespace hpx::util::itt
+    constexpr void event_tick(event const&) noexcept {}
+}    // namespace hpx::util::itt
 
 #endif    // HPX_HAVE_ITTNOTIFY
