@@ -20,20 +20,19 @@
 #include <hpx/logging/detail/macros.hpp>
 #include <hpx/logging/level.hpp>
 
-namespace hpx { namespace util { namespace logging {
+namespace hpx::util::logging {
 
     /**
-@file hpx/logging/logging.hpp
+    @file hpx/logging/logging.hpp
 
-Include this file when you're using the logging lib, but don't necessarily want to
-use @ref manipulator "formatters and destinations".
-If you want to use @ref manipulator "formatters and destinations",
-then you can include this one instead:
+    Include this file when you're using the logging lib, but don't necessarily want to
+    use @ref manipulator "formatters and destinations".
+    If you want to use @ref manipulator "formatters and destinations",
+    then you can include this one instead:
 
-@code
-#include <hpx/logging/format.hpp>
-@endcode
+    @code
+    #include <hpx/logging/format/formatters.hpp>
+    @endcode
 
-*/
-
-}}}    // namespace hpx::util::logging
+    */
+}    // namespace hpx::util::logging
