@@ -23,11 +23,11 @@
 #include <memory>
 #include <ostream>
 
-namespace hpx { namespace util { namespace logging { namespace formatter {
+namespace hpx::util::logging::formatter {
 
     idx::~idx() = default;
 
-    struct idx_impl : idx
+    struct idx_impl final : idx
     {
         idx_impl()
           : value(0ull)
@@ -45,7 +45,6 @@ namespace hpx { namespace util { namespace logging { namespace formatter {
 
     std::unique_ptr<idx> idx::make()
     {
-        return std::unique_ptr<idx>(new idx_impl());
+        return std::make_unique<idx_impl>();
     }
-
-}}}}    // namespace hpx::util::logging::formatter
+}    // namespace hpx::util::logging::formatter
