@@ -1,4 +1,4 @@
-//  Copyright (c) 2005-2017 Hartmut Kaiser
+//  Copyright (c) 2005-2022 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Adelstein-Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -9,9 +9,11 @@
 
 #include <hpx/config.hpp>
 
-namespace hpx { namespace threads { namespace policies {
 #ifdef HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION
-    HPX_CORE_EXPORT void set_minimal_deadlock_detection_enabled(bool enabled);
-    HPX_CORE_EXPORT bool get_minimal_deadlock_detection_enabled();
+namespace hpx::threads::policies {
+
+    HPX_CORE_EXPORT void set_minimal_deadlock_detection_enabled(
+        bool enabled) noexcept;
+    HPX_CORE_EXPORT bool get_minimal_deadlock_detection_enabled() noexcept;
+}    // namespace hpx::threads::policies
 #endif
-}}}    // namespace hpx::threads::policies
