@@ -8,7 +8,6 @@
 
 #include <hpx/config.hpp>
 #include <hpx/assert.hpp>
-#include <hpx/datastructures/tuple.hpp>
 #include <hpx/debugging/print.hpp>
 #include <hpx/schedulers/lockfree_queue_backends.hpp>
 #include <hpx/threading_base/print.hpp>
@@ -17,7 +16,6 @@
 #include <hpx/threading_base/thread_data_stackful.hpp>
 #include <hpx/threading_base/thread_data_stackless.hpp>
 #include <hpx/threading_base/thread_queue_init_parameters.hpp>
-#include <hpx/type_support/unused.hpp>
 
 #include <atomic>
 #include <cmath>
@@ -25,12 +23,11 @@
 #include <cstdint>
 #include <exception>
 #include <functional>
-#include <iomanip>
 #include <list>
-#include <map>
 #include <memory>
 #include <mutex>
 #include <string>
+#include <tuple>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -44,30 +41,23 @@
 #endif
 
 namespace hpx {
-    static hpx::debug::enable_print<QUEUE_HOLDER_THREAD_DEBUG> tq_deb(
+
+    inline constexpr hpx::debug::enable_print<QUEUE_HOLDER_THREAD_DEBUG> tq_deb(
         "QH_THRD");
 }
 
-// ------------------------------------------------------------
-namespace hpx { namespace threads { namespace policies {
+namespace hpx::threads::policies {
 
-    // apply the modulo operator only when needed
-    // (i.e. when the input is greater than the ceiling)
-    // NB: the numbers must be positive
-    HPX_FORCEINLINE std::size_t fast_mod(
-        std::size_t const input, std::size_t const ceil)
+    // apply the modulo operator only when needed (i.e. when the input is
+    // greater than the ceiling) NB: the numbers must be positive
+    constexpr std::size_t fast_mod(
+        std::size_t const input, std::size_t const ceil) noexcept
     {
         return input >= ceil ? input % ceil : input;
     }
 
-    enum : std::size_t
-    {
-        max_thread_count = 1000
-    };
-    enum : std::size_t
-    {
-        round_robin_rollover = 1
-    };
+    inline constexpr std::size_t max_thread_count = 1000;
+    inline constexpr std::size_t round_robin_rollover = 1;
 
     // ----------------------------------------------------------------
     // Helper class to hold a set of queues.
@@ -86,21 +76,23 @@ namespace hpx { namespace threads { namespace policies {
         QueueType* const lp_queue_;
 
         // these are the domain and local thread queue ids for the container
-        const std::size_t domain_index_;
-        const std::size_t queue_index_;
-        const std::size_t thread_num_;
+        std::size_t const domain_index_;
+        std::size_t const queue_index_;
+        std::size_t const thread_num_;
+
         // a mask that hold a bit per queue to indicate ownership of the queue
-        const std::size_t owner_mask_;
+        std::size_t const owner_mask_;
 
         // we must use OS mutexes here because we cannot suspend an HPX
         // thread whilst processing the Queues for that thread, this code
         // is running at the OS level in effect.
         using mutex_type = std::mutex;
-        typedef std::unique_lock<mutex_type> scoped_lock;
+        using scoped_lock = std::unique_lock<mutex_type>;
 
         // mutex protecting the thread map
         mutable util::cache_line_data<mutex_type> thread_map_mtx_;
 
+#if !defined(HPX_HAVE_ADDRESS_SANITIZER)
         // every thread maintains lists of free thread data objects
         // sorted by their stack sizes
         using thread_heap_type =
@@ -111,17 +103,13 @@ namespace hpx { namespace threads { namespace policies {
         thread_heap_type thread_heap_large_;
         thread_heap_type thread_heap_huge_;
         thread_heap_type thread_heap_nostack_;
+#endif
 
         // these ought to be atomic, but if we get a race and assign a thread
         // to queue N instead of N+1 it doesn't really matter
-
         mutable util::cache_line_data<std::tuple<std::size_t, std::size_t>>
             rollover_counters_;
 
-        // ----------------------------------------------------------------
-        // ----------------------------------------------------------------
-        // ----------------------------------------------------------------
-
         static util::internal_allocator<threads::thread_data> thread_alloc_;
 
         using task_description = thread_init_data;
@@ -137,9 +125,7 @@ namespace hpx { namespace threads { namespace policies {
         mutable util::cache_line_data<std::atomic<std::int32_t>>
             thread_map_count_;
 
-        // -------------------------------------
-        // terminated tasks
-        // completed tasks that can be reused (stack space etc)
+        // terminated tasks completed tasks that can be reused (stack space etc)
         using terminated_items_type = lockfree_fifo::apply<thread_data*>::type;
         terminated_items_type terminated_items_;
         mutable util::cache_line_data<std::atomic<std::int32_t>>
@@ -147,17 +133,17 @@ namespace hpx { namespace threads { namespace policies {
 
         thread_queue_init_parameters parameters_;
 
-        // ------------------------------------------------------------
         struct queue_mc_print
         {
-            const QueueType* const q_;
-            explicit queue_mc_print(const QueueType* const q)
+            QueueType const* const q_;
+
+            explicit constexpr queue_mc_print(QueueType const* const q) noexcept
               : q_(q)
             {
             }
-            //
+
             friend std::ostream& operator<<(
-                std::ostream& os, const queue_mc_print& d)
+                std::ostream& os, queue_mc_print const& d)
             {
                 os << "n " << debug::dec<3>(d.q_->new_tasks_count_.data_)
                    << " w " << debug::dec<3>(d.q_->work_items_count_.data_);
@@ -167,14 +153,16 @@ namespace hpx { namespace threads { namespace policies {
 
         struct queue_data_print
         {
-            const queue_holder_thread* q_;
-            explicit queue_data_print(const queue_holder_thread* q)
+            queue_holder_thread const* q_;
+
+            explicit constexpr queue_data_print(
+                queue_holder_thread const* q) noexcept
               : q_(q)
             {
             }
-            //
+
             friend std::ostream& operator<<(
-                std::ostream& os, const queue_data_print& d)
+                std::ostream& os, queue_data_print const& d)
             {
                 os << "D " << debug::dec<2>(d.q_->domain_index_) << " Q "
                    << debug::dec<3>(d.q_->queue_index_) << " TM "
@@ -187,15 +175,12 @@ namespace hpx { namespace threads { namespace policies {
                 return os;
             }
         };
-        // ------------------------------------------------------------
 
-        // ----------------------------------------------------------------
-        // ----------------------------------------------------------------
         // ----------------------------------------------------------------
         queue_holder_thread(QueueType* bp_queue, QueueType* hp_queue,
             QueueType* np_queue, QueueType* lp_queue, std::size_t domain,
             std::size_t queue, std::size_t thread_num, std::size_t owner,
-            const thread_queue_init_parameters& init)
+            thread_queue_init_parameters const& init)
           : bp_queue_(bp_queue)
           , hp_queue_(hp_queue)
           , np_queue_(np_queue)
@@ -237,7 +222,8 @@ namespace hpx { namespace threads { namespace policies {
                 delete np_queue_;
             if (owns_lp_queue())
                 delete lp_queue_;
-            //
+
+#if !defined(HPX_HAVE_ADDRESS_SANITIZER)
             for (auto t : thread_heap_small_)
                 deallocate(get_thread_id_data(t));
 
@@ -252,42 +238,43 @@ namespace hpx { namespace threads { namespace policies {
 
             for (auto t : thread_heap_nostack_)
                 deallocate(get_thread_id_data(t));
+#endif
         }
 
         // ----------------------------------------------------------------
-        inline bool owns_bp_queue() const
+        inline bool owns_bp_queue() const noexcept
         {
             return bp_queue_ && ((owner_mask_ & 1) != 0);
         }
 
         // ----------------------------------------------------------------
-        inline bool owns_hp_queue() const
+        inline bool owns_hp_queue() const noexcept
         {
             return hp_queue_ && ((owner_mask_ & 2) != 0);
         }
 
         // ----------------------------------------------------------------
-        inline bool owns_np_queue() const
+        inline bool owns_np_queue() const noexcept
         {
             return ((owner_mask_ & 4) != 0);
         }
 
         // ----------------------------------------------------------------
-        inline bool owns_lp_queue() const
+        inline bool owns_lp_queue() const noexcept
         {
             return lp_queue_ && ((owner_mask_ & 8) != 0);
         }
 
         // ------------------------------------------------------------
-        // return the next round robin thread index across all workers
-        // using a batching of N per worker before incrementing
+        // return the next round robin thread index across all workers using a
+        // batching of N per worker before incrementing
         inline std::size_t worker_next(std::size_t const workers) const
         {
             tq_deb.debug(debug::str<>("worker_next"), "Rollover counter ",
                 debug::dec<4>(std::get<0>(rollover_counters_.data_)),
                 debug::dec<4>(std::get<1>(rollover_counters_.data_)), "workers",
                 debug::dec<4>(workers));
-            if (--std::get<1>(rollover_counters_.data_) == 0)
+            if (--std::get<1>(rollover_counters_.data_) == 0)    //-V516
             {
                 std::get<1>(rollover_counters_.data_) = round_robin_rollover;
                 std::get<0>(rollover_counters_.data_) = fast_mod(
@@ -297,16 +284,16 @@ namespace hpx { namespace threads { namespace policies {
         }
 
         // ------------------------------------------------------------
-        void schedule_thread(threads::thread_data* thrd,
+        void schedule_thread(threads::thread_id_ref_type thrd,
             thread_priority priority, bool other_end = false)
         {
             if (bp_queue_ && (priority == thread_priority::bound))
             {
                 tq_deb.debug(debug::str<>("schedule_thread"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "queueing thread_priority::bound");
-                bp_queue_->schedule_work(thrd, other_end);
+                bp_queue_->schedule_work(HPX_MOVE(thrd), other_end);
             }
             else if (hp_queue_ &&
                 (priority == thread_priority::high ||
@@ -315,46 +302,44 @@ namespace hpx { namespace threads { namespace policies {
             {
                 tq_deb.debug(debug::str<>("schedule_thread"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "queueing thread_priority::high");
-                hp_queue_->schedule_work(thrd, other_end);
+                hp_queue_->schedule_work(HPX_MOVE(thrd), other_end);
             }
             else if (lp_queue_ && (priority == thread_priority::low))
             {
                 tq_deb.debug(debug::str<>("schedule_thread"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "queueing thread_priority::low");
-                lp_queue_->schedule_work(thrd, other_end);
+                lp_queue_->schedule_work(HPX_MOVE(thrd), other_end);
             }
             else
             {
                 tq_deb.debug(debug::str<>("schedule_thread"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "queueing thread_priority::normal");
-                np_queue_->schedule_work(thrd, other_end);
+                np_queue_->schedule_work(HPX_MOVE(thrd), other_end);
             }
         }
 
         // ----------------------------------------------------------------
         bool cleanup_terminated(std::size_t thread_num, bool delete_all)
         {
-            // clang-format off
-            if (thread_num!=thread_num_) {
-                tq_deb.error(debug::str<>("assertion fail")
-                             , "thread_num", thread_num
-                             , "thread_num_", thread_num_
-                             , "queue_index_", queue_index_
-                             , queue_data_print(this)
-                             );
+            if (thread_num != thread_num_)
+            {
+                tq_deb.error(debug::str<>("assertion fail"), "thread_num",
+                    thread_num, "thread_num_", thread_num_, "queue_index_",
+                    queue_index_, queue_data_print(this));
             }
-            // clang-format on
             HPX_ASSERT(thread_num == thread_num_);
 
             if (terminated_items_count_.data_.load(std::memory_order_relaxed) ==
                 0)
+            {
                 return true;
+            }
 
             scoped_lock lk(thread_map_mtx_.data_);
 
@@ -368,14 +353,14 @@ namespace hpx { namespace threads { namespace policies {
                     tq_deb.debug(debug::str<>("cleanup"), "delete",
                         queue_data_print(this),
                         debug::threadinfo<thread_data*>(todelete));
-                    thread_id_type tid(todelete);
+                    thread_id_type const tid(todelete);
                     remove_from_thread_map(tid, true);
                 }
             }
             else
             {
                 // delete only this many threads
-                std::int64_t delete_count = static_cast<std::int64_t>(
+                auto delete_count = static_cast<std::int64_t>(
                     terminated_items_count_.data_.load(
                         std::memory_order_relaxed) /
                     2);
@@ -401,10 +386,12 @@ namespace hpx { namespace threads { namespace policies {
         }
 
         // ----------------------------------------------------------------
-        void create_thread(thread_init_data& data, thread_id_type* tid,
+        void create_thread(thread_init_data& data, thread_id_ref_type* tid,
             std::size_t thread_num, error_code& ec)
         {
-            if (thread_num != thread_num_)
+            if (thread_num != thread_num_ &&
+                (data.initial_state == thread_schedule_state::pending ||
+                    data.initial_state == thread_schedule_state::pending_boost))
             {
                 data.run_now = false;
             }
@@ -452,15 +439,13 @@ namespace hpx { namespace threads { namespace policies {
         }
 
         // ----------------------------------------------------------------
-        // Not thread safe. This function must only be called by the
-        // thread that owns the holder object.
-        // Creates a thread_data object using information from
-        // thread_init_data .
-        // If a thread data object is available on one of the heaps
-        // it will use that, otherwise a new one is created.
-        // Heaps store data ordered/sorted by stack size
+        // Not thread safe. This function must only be called by the thread that
+        // owns the holder object. Creates a thread_data object using
+        // information from thread_init_data . If a thread data object is
+        // available on one of the heaps it will use that, otherwise a new one
+        // is created. Heaps store data ordered/sorted by stack size
         void create_thread_object(
-            threads::thread_id_type& tid, threads::thread_init_data& data)
+            threads::thread_id_ref_type& tid, threads::thread_init_data& data)
         {
             HPX_ASSERT(data.stacksize >= thread_stacksize::minimal);
             HPX_ASSERT(data.stacksize <= thread_stacksize::maximal);
@@ -468,6 +453,7 @@ namespace hpx { namespace threads { namespace policies {
             std::ptrdiff_t const stacksize =
                 data.scheduler_base->get_stack_size(data.stacksize);
 
+#if !defined(HPX_HAVE_ADDRESS_SANITIZER)
             thread_heap_type* heap = nullptr;
             if (stacksize == parameters_.small_stacksize_)
             {
@@ -490,6 +476,7 @@ namespace hpx { namespace threads { namespace policies {
                 heap = &thread_heap_nostack_;
             }
             HPX_ASSERT(heap);
+#endif
 
             if (data.initial_state ==
                     thread_schedule_state::pending_do_not_schedule ||
@@ -498,8 +485,10 @@ namespace hpx { namespace threads { namespace policies {
                 data.initial_state = thread_schedule_state::pending;
             }
 
+            // ASAN gets confused by reusing threads/stacks
+#if !defined(HPX_HAVE_ADDRESS_SANITIZER)
             // Check for an unused thread object.
-            if (!heap->empty())
+            if (heap && !heap->empty())    //-V522
             {
                 // Take ownership of the thread object and rebind it.
                 tid = heap->front();
@@ -507,12 +496,13 @@ namespace hpx { namespace threads { namespace policies {
                 get_thread_id_data(tid)->rebind(data);
                 tq_deb.debug(debug::str<>("create_thread_object"), "rebind",
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_id_type*>(&tid));
+                    debug::threadinfo<threads::thread_id_ref_type*>(&tid));
             }
             else
+#endif
             {
                 // Allocate a new thread object.
-                threads::thread_data* p = nullptr;
+                threads::thread_data* p;
                 if (stacksize == parameters_.nostack_stacksize_)
                 {
                     p = threads::thread_data_stackless::create(
@@ -523,17 +513,19 @@ namespace hpx { namespace threads { namespace policies {
                     p = threads::thread_data_stackful::create(
                         data, this, stacksize);
                 }
-                tid = thread_id_type(p);
+                tid = thread_id_ref_type(p, thread_id_addref::no);
+
                 tq_deb.debug(debug::str<>("create_thread_object"), "new",
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_id_type*>(&tid));
+                    debug::threadinfo<threads::thread_data*>(p));
             }
         }
 
         // ----------------------------------------------------------------
         void recycle_thread(thread_id_type tid)
         {
-            std::ptrdiff_t stacksize =
+#if !defined(HPX_HAVE_ADDRESS_SANITIZER)
+            std::ptrdiff_t const stacksize =
                 get_thread_id_data(tid)->get_stack_size();
 
             if (stacksize == parameters_.small_stacksize_)
@@ -561,13 +553,16 @@ namespace hpx { namespace threads { namespace policies {
                 HPX_ASSERT_MSG(
                     false, util::format("Invalid stack size {1}", stacksize));
             }
+#else
+            deallocate(get_thread_id_data(tid));
+#endif
         }
 
         // ----------------------------------------------------------------
-        static void deallocate(threads::thread_data* p)
+        static void deallocate(threads::thread_data* p) noexcept
         {
             using threads::thread_data;
-            p->~thread_data();
+            std::destroy_at(p);
             thread_alloc_.deallocate(p, 1);
         }
 
@@ -577,15 +572,12 @@ namespace hpx { namespace threads { namespace policies {
             scoped_lock lk(thread_map_mtx_.data_);
 
             // add a new entry in the map for this thread
-            std::pair<thread_map_type::iterator, bool> p =
+            std::pair<thread_map_type::iterator, bool> const p =
                 thread_map_.insert(tid);
 
             if (/*HPX_UNLIKELY*/ (!p.second))
             {
-                std::string map_size = std::to_string(thread_map_.size());
-                // threads::thread_id_type tid2 = *(p.first);
-                // threads::thread_data* td = get_thread_id_data(tid2);
-                // std::string prev = hpx::util::format("{}", td);
+                std::string const map_size = std::to_string(thread_map_.size());
 
                 tq_deb.error(debug::str<>("map add"),
                     "Couldn't add new thread to the thread map",
@@ -593,9 +585,7 @@ namespace hpx { namespace threads { namespace policies {
                     debug::threadinfo<thread_id_type*>(&tid));
 
                 lk.unlock();
-                HPX_THROW_EXCEPTION(hpx::out_of_memory,
-                    "queue_holder_thread::add_to_thread_map",
-                    "Couldn't add new thread to the thread map {}", map_size);
+                HPX_THROW_BAD_ALLOC("queue_holder_thread::add_to_thread_map");
             }
 
             ++thread_map_count_.data_;
@@ -612,12 +602,10 @@ namespace hpx { namespace threads { namespace policies {
         {
             // this thread has to be in this map
             HPX_ASSERT(thread_map_.find(tid) != thread_map_.end());
-
             HPX_ASSERT(thread_map_count_.data_ >= 0);
 
-            bool deleted = thread_map_.erase(tid) != 0;
+            [[maybe_unused]] bool const deleted = thread_map_.erase(tid) != 0;
             HPX_ASSERT(deleted);
-            (void) deleted;
 
             tq_deb.debug(debug::str<>("map remove"), queue_data_print(this),
                 debug::threadinfo<thread_id_type*>(&tid));
@@ -630,8 +618,8 @@ namespace hpx { namespace threads { namespace policies {
         }
 
         // ----------------------------------------------------------------
-        bool get_next_thread_HP(
-            threads::thread_data*& thrd, bool stealing, bool check_new) HPX_HOT
+        bool get_next_thread_HP(threads::thread_id_ref_type& thrd,
+            bool stealing, bool check_new) HPX_HOT
         {
             // only take from BP queue if we are not stealing
             if (!stealing && bp_queue_ &&
@@ -639,7 +627,7 @@ namespace hpx { namespace threads { namespace policies {
             {
                 tq_deb.debug(debug::str<>("next_thread_BP"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "thread_priority::bound");
                 return true;
             }
@@ -649,23 +637,25 @@ namespace hpx { namespace threads { namespace policies {
             {
                 tq_deb.debug(debug::str<>("get_next_thread_HP"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "thread_priority::high");
                 return true;
             }
+
             // if we're out of work in the main queues,
             debug_queues("get_next_thread");
             return false;
         }
 
         // ----------------------------------------------------------------
-        bool get_next_thread(threads::thread_data*& thrd, bool stealing) HPX_HOT
+        bool get_next_thread(
+            threads::thread_id_ref_type& thrd, bool stealing) HPX_HOT
         {
             if (np_queue_->get_next_thread(thrd, stealing))
             {
                 tq_deb.debug(debug::str<>("next_thread_NP"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "thread_priority::normal");
                 return true;
             }
@@ -674,18 +664,19 @@ namespace hpx { namespace threads { namespace policies {
             {
                 tq_deb.debug(debug::str<>("next_thread_LP"),
                     queue_data_print(this),
-                    debug::threadinfo<threads::thread_data*>(thrd),
+                    debug::threadinfo<threads::thread_id_ref_type*>(&thrd),
                     "thread_priority::low");
                 return true;
             }
+
             // if we're out of work in the main queues,
             debug_queues("get_next_thread");
             return false;
         }
 
         // ----------------------------------------------------------------
-        std::size_t add_new_HP(
-            std::int64_t add_count, thread_holder_type* addfrom, bool stealing)
+        std::size_t add_new_HP(std::int64_t add_count,
+            thread_holder_type const* addfrom, bool stealing)
         {
             std::size_t added;
             if (owns_bp_queue() && !stealing)
@@ -726,14 +717,13 @@ namespace hpx { namespace threads { namespace policies {
                 if (added > 0)
                     return added;
             }
-            //
+
             static auto an_timed =
                 tq_deb.make_timer(1, debug::str<>("add_new"));
             tq_deb.timed(an_timed, "add", debug::dec<3>(add_count),
                 "owns bp, hp, np, lp", owns_bp_queue(), owns_hp_queue(),
                 owns_np_queue(), owns_lp_queue(), "this",
                 queue_data_print(this), "from", queue_data_print(addfrom));
-            //
             return 0;
         }
 
@@ -794,12 +784,11 @@ namespace hpx { namespace threads { namespace policies {
             default:
             case thread_priority::unknown:
             {
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "queue_holder_thread::get_thread_count_staged",
                     "unknown thread priority value (thread_priority::unknown)");
             }
             }
-            return 0;
         }
 
         // ----------------------------------------------------------------
@@ -845,12 +834,11 @@ namespace hpx { namespace threads { namespace policies {
             default:
             case thread_priority::unknown:
             {
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "queue_holder_thread::get_thread_count_pending",
                     "unknown thread priority value (thread_priority::unknown)");
             }
             }
-            return 0;
         }
 
         // ----------------------------------------------------------------
@@ -878,9 +866,8 @@ namespace hpx { namespace threads { namespace policies {
             scoped_lock lk(thread_map_mtx_.data_);
 
             std::int64_t num_threads = 0;
-            thread_map_type::const_iterator end = thread_map_.end();
-            for (thread_map_type::const_iterator it = thread_map_.begin();
-                 it != end; ++it)
+            auto const end = thread_map_.end();
+            for (auto it = thread_map_.begin(); it != end; ++it)
             {
                 if (get_thread_id_data(*it)->get_state().state() == state)
                     ++num_threads;
@@ -893,19 +880,20 @@ namespace hpx { namespace threads { namespace policies {
         void destroy_thread(
             threads::thread_data* thrd, std::size_t thread_num, bool xthread)
         {
-            // the thread must be destroyed by the same queue holder that created it
+            // the thread must be destroyed by the same queue holder that
+            // created it
             HPX_ASSERT(&thrd->get_queue<queue_holder_thread>() == this);
             //
             tq_deb.debug(debug::str<>("destroy"), "terminated_items push",
                 "xthread", xthread, queue_data_print(this),
                 debug::threadinfo<threads::thread_data*>(thrd));
             terminated_items_.push(thrd);
-            std::int64_t count = ++terminated_items_count_.data_;
 
+            std::int64_t const count = ++terminated_items_count_.data_;
             if (!xthread && (count > parameters_.max_terminated_threads_))
             {
-                cleanup_terminated(
-                    thread_num, false);    // clean up all terminated threads
+                // clean up all terminated threads
+                cleanup_terminated(thread_num, false);
             }
         }
 
@@ -913,26 +901,26 @@ namespace hpx { namespace threads { namespace policies {
         void abort_all_suspended_threads()
         {
             scoped_lock lk(thread_map_mtx_.data_);
-            thread_map_type::iterator end = thread_map_.end();
-            for (thread_map_type::iterator it = thread_map_.begin(); it != end;
-                 ++it)
+            auto const end = thread_map_.end();
+            for (auto it = thread_map_.begin(); it != end; ++it)
             {
                 if (get_thread_id_data(*it)->get_state().state() ==
                     thread_schedule_state::suspended)
                 {
-                    get_thread_id_data(*it)->set_state(
-                        thread_schedule_state::pending,
-                        thread_restart_state::abort);
+                    [[maybe_unused]] auto const s =
+                        get_thread_id_data(*it)->set_state(
+                            thread_schedule_state::pending,
+                            thread_restart_state::abort);
+
                     // np queue always exists so use that as priority doesn't matter
-                    np_queue_->schedule_work(get_thread_id_data(*it), true);
+                    np_queue_->schedule_work(*it, true);
                 }
             }
             throw std::runtime_error("This function needs to be reimplemented");
         }
 
         // ------------------------------------------------------------
-        bool enumerate_threads(
-            util::function_nonser<bool(thread_id_type)> const& f,
+        bool enumerate_threads(hpx::function<bool(thread_id_type)> const& f,
             thread_schedule_state state = thread_schedule_state::unknown) const
         {
             std::uint64_t count = thread_map_count_.data_;
@@ -942,10 +930,9 @@ namespace hpx { namespace threads { namespace policies {
             }
             else if (state == thread_schedule_state::staged)
             {
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "queue_holder_thread::iterate_threads",
                     "can't iterate over thread ids of staged threads");
-                return false;
             }
 
             std::vector<thread_id_type> tids;
@@ -954,9 +941,8 @@ namespace hpx { namespace threads { namespace policies {
             if (state == thread_schedule_state::unknown)
             {
                 scoped_lock lk(thread_map_mtx_.data_);
-                thread_map_type::const_iterator end = thread_map_.end();
-                for (thread_map_type::const_iterator it = thread_map_.begin();
-                     it != end; ++it)
+                auto const end = thread_map_.end();
+                for (auto it = thread_map_.begin(); it != end; ++it)
                 {
                     tids.push_back(*it);
                 }
@@ -964,9 +950,8 @@ namespace hpx { namespace threads { namespace policies {
             else
             {
                 scoped_lock lk(thread_map_mtx_.data_);
-                thread_map_type::const_iterator end = thread_map_.end();
-                for (thread_map_type::const_iterator it = thread_map_.begin();
-                     it != end; ++it)
+                auto const end = thread_map_.end();
+                for (auto it = thread_map_.begin(); it != end; ++it)
                 {
                     if (get_thread_id_data(*it)->get_state().state() == state)
                         tids.push_back(*it);
@@ -1008,7 +993,7 @@ namespace hpx { namespace threads { namespace policies {
         }
 
         // ------------------------------------------------------------
-        void debug_queues(const char* prefix)
+        void debug_queues(char const* prefix)
         {
             static auto deb_queues =
                 tq_deb.make_timer(1, debug::str<>("debug_queues"));
@@ -1020,4 +1005,4 @@ namespace hpx { namespace threads { namespace policies {
     template <typename QueueType>
     util::internal_allocator<threads::thread_data>
         queue_holder_thread<QueueType>::thread_alloc_;
-}}}    // namespace hpx::threads::policies
+}    // namespace hpx::threads::policies
