@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2019 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2014 Thomas Heller
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -13,13 +13,13 @@
 
 #include <cstddef>
 
-namespace hpx { namespace serialization {
+namespace hpx::serialization {
 
     struct erased_output_container
     {
         virtual ~erased_output_container() = default;
 
-        virtual bool is_preprocessing() const
+        [[nodiscard]] virtual bool is_preprocessing() const noexcept
         {
             return false;
         }
@@ -28,20 +28,23 @@ namespace hpx { namespace serialization {
         virtual std::size_t save_binary_chunk(
             void const* address, std::size_t count) = 0;
         virtual void reset() = 0;
-        virtual std::size_t get_num_chunks() const = 0;
+        virtual std::size_t get_num_chunks() const noexcept = 0;
         virtual void flush() = 0;
     };
 
     struct erased_input_container
     {
-        virtual ~erased_input_container() {}
+        virtual ~erased_input_container() = default;
 
-        virtual bool is_preprocessing() const
+        [[nodiscard]] virtual bool is_preprocessing() const noexcept
         {
             return false;
         }
         virtual void set_filter(binary_filter* filter) = 0;
+        virtual void set_zero_copy_serialization_threshold(
+            std::size_t zero_copy_serialization_threshold) = 0;
         virtual void load_binary(void* address, std::size_t count) = 0;
-        virtual void load_binary_chunk(void* address, std::size_t count) = 0;
+        virtual void load_binary_chunk(
+            void* address, std::size_t count, bool allow_zero_copy_receive) = 0;
     };
-}}    // namespace hpx::serialization
+}    // namespace hpx::serialization
