@@ -1,4 +1,4 @@
-//  Copyright (c) 2017 Hartmut Kaiser
+//  Copyright (c) 2017-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -12,10 +12,11 @@
 #include <cstdint>
 #include <deque>
 
-namespace hpx { namespace serialization {
+namespace hpx::serialization {
 
     template <typename T, typename Allocator>
-    void serialize(input_archive& ar, std::deque<T, Allocator>& d, unsigned)
+    void serialize(
+        input_archive& ar, std::deque<T, Allocator>& d, unsigned)    //-V826
     {
         // normal load ...
         std::uint64_t size;
@@ -27,15 +28,16 @@ namespace hpx { namespace serialization {
     }
 
     template <typename T, typename Allocator>
-    void serialize(
-        output_archive& ar, std::deque<T, Allocator> const& d, unsigned)
+    void serialize(output_archive& ar,
+        std::deque<T, Allocator> const& d,    //-V826
+        unsigned)
     {
         // normal save ...
-        std::uint64_t size = d.size();
+        std::uint64_t const size = d.size();
         ar << size;
         if (d.empty())
             return;
 
         detail::save_collection(ar, d);
     }
-}}    // namespace hpx::serialization
+}    // namespace hpx::serialization
