@@ -13,13 +13,12 @@
 #include <set>
 #include <utility>
 
-namespace hpx { namespace serialization {
+namespace hpx::serialization {
 
     template <typename T, typename Compare, typename Allocator>
     void serialize(
         input_archive& ar, std::set<T, Compare, Allocator>& set, unsigned)
     {
-        set.clear();
         std::uint64_t size;
         ar >> size;
 
@@ -28,7 +27,7 @@ namespace hpx { namespace serialization {
         {
             T t;
             ar >> t;
-            set.insert(set.end(), std::move(t));
+            set.insert(set.end(), HPX_MOVE(t));
         }
     }
 
@@ -36,13 +35,14 @@ namespace hpx { namespace serialization {
     void serialize(output_archive& ar,
         std::set<T, Compare, Allocator> const& set, unsigned)
     {
-        std::uint64_t size = set.size();
+        std::uint64_t const size = set.size();
         ar << size;
-        if (set.empty())
+        if (size == 0)
             return;
+
         for (T const& i : set)
         {
             ar << i;
         }
     }
-}}    // namespace hpx::serialization
+}    // namespace hpx::serialization
