@@ -12,7 +12,7 @@
 #include <cstdint>
 #include <string>
 
-namespace hpx { namespace serialization {
+namespace hpx::serialization {
 
     // load string
     template <typename Char, typename CharTraits, typename Allocator>
@@ -34,8 +34,8 @@ namespace hpx { namespace serialization {
     void serialize(output_archive& ar,
         std::basic_string<Char, CharTraits, Allocator> const& s, unsigned)
     {
-        std::uint64_t size = s.size();
+        std::uint64_t const size = s.size();
         ar << size;
         save_binary(ar, s.data(), s.size() * sizeof(Char));
     }
-}}    // namespace hpx::serialization
+}    // namespace hpx::serialization
