@@ -1,5 +1,5 @@
 //  Copyright (c) 2013-2014 Thomas Heller
-//  Copyright (c) 2013 Hartmut Kaiser
+//  Copyright (c) 2013-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -9,7 +9,7 @@
 
 #include <type_traits>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     // This trait is used to decide whether a class (or specialization) is
     // required to automatically register to the action factory
@@ -17,4 +17,8 @@ namespace hpx { namespace traits {
     struct needs_automatic_registration : std::true_type
     {
     };
-}}    // namespace hpx::traits
+
+    template <typename T>
+    inline constexpr bool needs_automatic_registration_v =
+        needs_automatic_registration<T>::value;
+}    // namespace hpx::traits
