@@ -11,18 +11,19 @@
 #include <hpx/serialization/traits/is_bitwise_serializable.hpp>
 
 #include <cstddef>
+#include <cstdint>
 #include <valarray>
 
-namespace hpx { namespace serialization {
+namespace hpx::serialization {
 
     template <typename T>
     void serialize(input_archive& ar, std::valarray<T>& arr, int /* version */)
     {
-        std::size_t sz = 0;
-        ar& sz;
-        arr.resize(sz);
+        std::uint64_t sz = 0;
+        ar >> sz;
+        arr.resize(static_cast<std::size_t>(sz));
 
-        if (sz < 1)
+        if (sz == 0)
             return;
 
         for (std::size_t i = 0; i < sz; ++i)
@@ -33,9 +34,12 @@ namespace hpx { namespace serialization {
     void serialize(
         output_archive& ar, std::valarray<T> const& arr, int /* version */)
     {
-        const std::size_t sz = arr.size();
-        ar& sz;
-        for (auto v : arr)
+        auto const sz = static_cast<std::uint64_t>(arr.size());
+        ar << sz;
+        if (sz == 0)
+            return;
+
+        for (auto const& v : arr)
             ar << v;
     }
-}}    // namespace hpx::serialization
+}    // namespace hpx::serialization
