@@ -10,11 +10,12 @@
 #include <cctype>
 #include <string>
 
-namespace hpx { namespace string_util {
+namespace hpx::string_util {
+
     template <typename CharT, class Traits, class Alloc>
     void to_lower(std::basic_string<CharT, Traits, Alloc>& s)
     {
         std::transform(std::begin(s), std::end(s), std::begin(s),
             [](int c) { return std::tolower(c); });
     }
-}}    // namespace hpx::string_util
+}    // namespace hpx::string_util
