@@ -1,4 +1,5 @@
 //  Copyright (c) 2021 ETH Zurich
+//  Copyright (c) 2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -8,21 +9,25 @@
 
 #include <hpx/allocator_support/internal_allocator.hpp>
 #include <hpx/assert.hpp>
+#include <hpx/datastructures/detail/small_vector.hpp>
 #include <hpx/datastructures/optional.hpp>
+#include <hpx/execution_base/completion_signatures.hpp>
+#include <hpx/execution_base/operation_state.hpp>
 #include <hpx/execution_base/receiver.hpp>
-#include <hpx/functional/unique_function.hpp>
+#include <hpx/execution_base/sender.hpp>
+#include <hpx/functional/move_only_function.hpp>
 #include <hpx/synchronization/mutex.hpp>
 
-#include <boost/container/small_vector.hpp>
-
 #include <exception>
 #include <memory>
 #include <mutex>
 #include <type_traits>
 #include <utility>
 
-namespace hpx { namespace experimental {
+namespace hpx::experimental {
+
     namespace detail {
+
         enum class async_rw_mutex_access_type
         {
             read,
@@ -34,18 +39,20 @@ namespace hpx { namespace experimental {
         {
             using shared_state_ptr_type =
                 std::shared_ptr<async_rw_mutex_shared_state>;
-            hpx::util::optional<T> value;
+
+            hpx::optional<T> value;
             shared_state_ptr_type next_state;
-            hpx::lcos::local::mutex mtx;
-            boost::container::small_vector<
-                hpx::util::unique_function_nonser<void(shared_state_ptr_type)>,
-                1>
+            hpx::mutex mtx;
+            hpx::detail::small_vector<
+                hpx::move_only_function<void(shared_state_ptr_type)>, 1>
                 continuations;
 
             async_rw_mutex_shared_state() = default;
+
             async_rw_mutex_shared_state(async_rw_mutex_shared_state&&) = delete;
             async_rw_mutex_shared_state& operator=(
                 async_rw_mutex_shared_state&&) = delete;
+
             async_rw_mutex_shared_state(
                 async_rw_mutex_shared_state const&) = delete;
             async_rw_mutex_shared_state& operator=(
@@ -67,7 +74,7 @@ namespace hpx { namespace experimental {
                 {
                     // The current state has now finished all accesses to the
                     // wrapped value, so we move the value to the next state.
-                    next_state->set_value(std::move(value.value()));
+                    next_state->set_value(HPX_MOVE(value.value()));
 
                     for (auto& continuation : continuations)
                     {
@@ -80,22 +87,22 @@ namespace hpx { namespace experimental {
             void set_value(U&& u)
             {
                 HPX_ASSERT(!value);
-                value.emplace(std::forward<U>(u));
+                value.emplace(HPX_FORWARD(U, u));
             }
 
             void set_next_state(
-                std::shared_ptr<async_rw_mutex_shared_state> state)
+                std::shared_ptr<async_rw_mutex_shared_state> state) noexcept
             {
                 // The next state should only be set once
                 HPX_ASSERT(!next_state);
-                next_state = std::move(state);
+                next_state = HPX_MOVE(state);
             }
 
             template <typename F>
             void add_continuation(F&& continuation)
             {
-                std::lock_guard<hpx::lcos::local::mutex> l(mtx);
-                continuations.emplace_back(std::forward<F>(continuation));
+                std::lock_guard<hpx::mutex> l(mtx);
+                continuations.emplace_back(HPX_FORWARD(F, continuation));
             }
         };
 
@@ -104,17 +111,19 @@ namespace hpx { namespace experimental {
         {
             using shared_state_ptr_type =
                 std::shared_ptr<async_rw_mutex_shared_state>;
+
             shared_state_ptr_type next_state;
-            hpx::lcos::local::mutex mtx;
-            boost::container::small_vector<
-                hpx::util::unique_function_nonser<void(shared_state_ptr_type)>,
-                1>
+            hpx::mutex mtx;
+            hpx::detail::small_vector<
+                hpx::move_only_function<void(shared_state_ptr_type)>, 1>
                 continuations;
 
             async_rw_mutex_shared_state() = default;
+
             async_rw_mutex_shared_state(async_rw_mutex_shared_state&&) = delete;
             async_rw_mutex_shared_state& operator=(
                 async_rw_mutex_shared_state&&) = delete;
+
             async_rw_mutex_shared_state(
                 async_rw_mutex_shared_state const&) = delete;
             async_rw_mutex_shared_state& operator=(
@@ -134,18 +143,18 @@ namespace hpx { namespace experimental {
             }
 
             void set_next_state(
-                std::shared_ptr<async_rw_mutex_shared_state> state)
+                std::shared_ptr<async_rw_mutex_shared_state> state) noexcept
             {
                 // The next state should only be set once
                 HPX_ASSERT(!next_state);
-                next_state = std::move(state);
+                next_state = HPX_MOVE(state);
             }
 
             template <typename F>
             void add_continuation(F&& continuation)
             {
-                std::lock_guard<hpx::lcos::local::mutex> l(mtx);
-                continuations.emplace_back(std::forward<F>(continuation));
+                std::lock_guard<hpx::mutex> l(mtx);
+                continuations.emplace_back(HPX_FORWARD(F, continuation));
             }
         };
 
@@ -164,8 +173,9 @@ namespace hpx { namespace experimental {
 
         public:
             async_rw_mutex_access_wrapper() = delete;
-            async_rw_mutex_access_wrapper(shared_state_type state)
-              : state(std::move(state))
+            explicit async_rw_mutex_access_wrapper(
+                shared_state_type state) noexcept
+              : state(HPX_MOVE(state))
             {
             }
             async_rw_mutex_access_wrapper(
@@ -195,11 +205,11 @@ namespace hpx { namespace experimental {
             async_rw_mutex_access_type::readwrite>
         {
         private:
-            static_assert(!std::is_void<ReadWriteT>::value,
+            static_assert(!std::is_void_v<ReadWriteT>,
                 "Cannot mix void and non-void type in "
                 "async_rw_mutex_access_wrapper wrapper (ReadWriteT is void, "
                 "ReadT is non-void)");
-            static_assert(!std::is_void<ReadT>::value,
+            static_assert(!std::is_void_v<ReadT>,
                 "Cannot mix void and non-void type in "
                 "async_rw_mutex_access_wrapper wrapper (ReadT is void, "
                 "ReadWriteT is non-void)");
@@ -210,8 +220,9 @@ namespace hpx { namespace experimental {
 
         public:
             async_rw_mutex_access_wrapper() = delete;
-            async_rw_mutex_access_wrapper(shared_state_type state)
-              : state(std::move(state))
+            explicit async_rw_mutex_access_wrapper(
+                shared_state_type state) noexcept
+              : state(HPX_MOVE(state))
             {
             }
             async_rw_mutex_access_wrapper(
@@ -250,8 +261,9 @@ namespace hpx { namespace experimental {
 
         public:
             async_rw_mutex_access_wrapper() = delete;
-            explicit async_rw_mutex_access_wrapper(shared_state_type state)
-              : state(std::move(state))
+            explicit async_rw_mutex_access_wrapper(
+                shared_state_type state) noexcept
+              : state(HPX_MOVE(state))
             {
             }
             async_rw_mutex_access_wrapper(
@@ -275,8 +287,9 @@ namespace hpx { namespace experimental {
 
         public:
             async_rw_mutex_access_wrapper() = delete;
-            explicit async_rw_mutex_access_wrapper(shared_state_type state)
-              : state(std::move(state))
+            explicit async_rw_mutex_access_wrapper(
+                shared_state_type state) noexcept
+              : state(HPX_MOVE(state))
             {
             }
             async_rw_mutex_access_wrapper(
@@ -381,7 +394,7 @@ namespace hpx { namespace experimental {
         {
             if (prev_access == detail::async_rw_mutex_access_type::readwrite)
             {
-                prev_state = std::move(state);
+                prev_state = HPX_MOVE(state);
                 state = std::allocate_shared<shared_state_type, allocator_type>(
                     alloc);
                 prev_access = detail::async_rw_mutex_access_type::read;
@@ -400,7 +413,7 @@ namespace hpx { namespace experimental {
 
         sender<detail::async_rw_mutex_access_type::readwrite> readwrite()
         {
-            prev_state = std::move(state);
+            prev_state = HPX_MOVE(state);
             state =
                 std::allocate_shared<shared_state_type, allocator_type>(alloc);
             prev_access = detail::async_rw_mutex_access_type::readwrite;
@@ -412,7 +425,7 @@ namespace hpx { namespace experimental {
             {
                 prev_state->set_next_state(state);
             }
-            return {std::move(prev_state), state};
+            return {HPX_MOVE(prev_state), state};
         }
 
     private:
@@ -425,14 +438,36 @@ namespace hpx { namespace experimental {
             using access_type =
                 detail::async_rw_mutex_access_wrapper<readwrite_type, read_type,
                     AccessType>;
-            template <template <typename...> class Tuple,
-                template <typename...> class Variant>
-            using value_types = Variant<Tuple<access_type>>;
+#if defined(HPX_HAVE_STDEXEC)
+            using sender_concept = hpx::execution::experimental::sender_t;
 
-            template <template <typename...> class Variant>
-            using error_types = Variant<std::exception_ptr>;
+            template <typename Env>
+            friend auto tag_invoke(
+                hpx::execution::experimental::get_completion_signatures_t,
+                sender const&, Env const&)
+                -> hpx::execution::experimental::completion_signatures<
+                    hpx::execution::experimental::set_value_t(access_type),
+                    hpx::execution::experimental::set_error_t(
+                        std::exception_ptr)>;
+#else
+            template <typename Env>
+            struct generate_completion_signatures
+            {
+                template <template <typename...> typename Tuple,
+                    template <typename...> typename Variant>
+                using value_types = Variant<Tuple<access_type>>;
 
-            static constexpr bool sends_done = false;
+                template <template <typename...> typename Variant>
+                using error_types = Variant<std::exception_ptr>;
+
+                static constexpr bool sends_stopped = false;
+            };
+
+            template <typename Env>
+            friend auto tag_invoke(
+                hpx::execution::experimental::get_completion_signatures_t,
+                sender const&, Env) -> generate_completion_signatures<Env>;
+#endif
 
             template <typename R>
             struct operation_state
@@ -444,9 +479,9 @@ namespace hpx { namespace experimental {
                 template <typename R_>
                 operation_state(R_&& r, shared_state_ptr_type prev_state,
                     shared_state_ptr_type state)
-                  : r(std::forward<R_>(r))
-                  , prev_state(std::move(prev_state))
-                  , state(std::move(state))
+                  : r(HPX_FORWARD(R_, r))
+                  , prev_state(HPX_MOVE(prev_state))
+                  , state(HPX_MOVE(state))
                 {
                 }
 
@@ -455,51 +490,52 @@ namespace hpx { namespace experimental {
                 operation_state(operation_state const&) = delete;
                 operation_state& operator=(operation_state const&) = delete;
 
-                void start() noexcept
+                friend void tag_invoke(hpx::execution::experimental::start_t,
+                    operation_state& os) noexcept
                 {
-                    HPX_ASSERT_MSG(state,
+                    HPX_ASSERT_MSG(os.state,
                         "async_rw_lock::sender::operation_state state is "
                         "empty, was the sender already started?");
 
                     auto continuation =
-                        [r = std::move(r)](
+                        [r = HPX_MOVE(os.r)](
                             shared_state_ptr_type state) mutable {
                             try
                             {
                                 hpx::execution::experimental::set_value(
-                                    std::move(r),
-                                    access_type{std::move(state)});
+                                    HPX_MOVE(r), access_type{HPX_MOVE(state)});
                             }
                             catch (...)
                             {
                                 hpx::execution::experimental::set_error(
-                                    std::move(r), std::current_exception());
+                                    HPX_MOVE(r), std::current_exception());
                             }
                         };
 
-                    if (prev_state)
+                    if (os.prev_state)
                     {
-                        prev_state->add_continuation(std::move(continuation));
+                        os.prev_state->add_continuation(HPX_MOVE(continuation));
 
                         // We release prev_state here to allow continuations to
                         // run. The operation state may otherwise keep it alive
                         // longer than needed.
-                        prev_state.reset();
+                        os.prev_state.reset();
                     }
                     else
                     {
                         // There is no previous state on the first access. We
                         // can immediately trigger the continuation.
-                        continuation(std::move(state));
+                        continuation(HPX_MOVE(os.state));
                     }
                 }
             };
 
             template <typename R>
-            auto connect(R&& r) &&
+            friend auto tag_invoke(
+                hpx::execution::experimental::connect_t, sender&& s, R&& r)
             {
-                return operation_state<R>{std::forward<R>(r),
-                    std::move(prev_state), std::move(state)};
+                return operation_state<R>{HPX_FORWARD(R, r),
+                    HPX_MOVE(s.prev_state), HPX_MOVE(s.state)};
             }
         };
 
@@ -516,10 +552,10 @@ namespace hpx { namespace experimental {
     class async_rw_mutex
     {
     private:
-        static_assert(!std::is_void<ReadWriteT>::value,
+        static_assert(!std::is_void_v<ReadWriteT>,
             "Cannot mix void and non-void type in async_rw_mutex (ReadWriteT "
             "is void, ReadT is non-void)");
-        static_assert(!std::is_void<ReadT>::value,
+        static_assert(!std::is_void_v<ReadT>,
             "Cannot mix void and non-void type in async_rw_mutex (ReadT is "
             "void, ReadWriteT is non-void)");
 
@@ -543,9 +579,9 @@ namespace hpx { namespace experimental {
         async_rw_mutex() = delete;
         template <typename U,
             typename = std::enable_if_t<
-                !std::is_same<std::decay_t<U>, async_rw_mutex>::value>>
+                !std::is_same_v<std::decay_t<U>, async_rw_mutex>>>
         explicit async_rw_mutex(U&& u, allocator_type const& alloc = {})
-          : value(std::forward<U>(u))
+          : value(HPX_FORWARD(U, u))
           , alloc(alloc)
         {
         }
@@ -558,7 +594,7 @@ namespace hpx { namespace experimental {
         {
             if (prev_access == detail::async_rw_mutex_access_type::readwrite)
             {
-                prev_state = std::move(state);
+                prev_state = HPX_MOVE(state);
                 state = std::allocate_shared<shared_state_type, allocator_type>(
                     alloc);
                 prev_access = detail::async_rw_mutex_access_type::read;
@@ -574,7 +610,7 @@ namespace hpx { namespace experimental {
                 }
                 else
                 {
-                    state->set_value(std::move(value));
+                    state->set_value(HPX_MOVE(value));
                 }
             }
             return {prev_state, state};
@@ -582,7 +618,7 @@ namespace hpx { namespace experimental {
 
         sender<detail::async_rw_mutex_access_type::readwrite> readwrite()
         {
-            prev_state = std::move(state);
+            prev_state = HPX_MOVE(state);
             state =
                 std::allocate_shared<shared_state_type, allocator_type>(alloc);
 
@@ -590,16 +626,16 @@ namespace hpx { namespace experimental {
             // a previous state we set the next state so that the value can be
             // passed from the previous state to the next state. When there is
             // no previous state we need to move the value to the first state.
-            if (HPX_LIKELY(prev_state))
+            if (HPX_LIKELY(prev_state))    //-V1051
             {
                 prev_state->set_next_state(state);
             }
             else
             {
-                state->set_value(std::move(value));
+                state->set_value(HPX_MOVE(value));
             }
             prev_access = detail::async_rw_mutex_access_type::readwrite;
-            return {std::move(prev_state), state};
+            return {HPX_MOVE(prev_state), state};
         }
 
     private:
@@ -616,14 +652,37 @@ namespace hpx { namespace experimental {
             using access_type =
                 detail::async_rw_mutex_access_wrapper<readwrite_type, read_type,
                     AccessType>;
-            template <template <typename...> class Tuple,
-                template <typename...> class Variant>
-            using value_types = Variant<Tuple<access_type>>;
 
-            template <template <typename...> class Variant>
-            using error_types = Variant<std::exception_ptr>;
+#if defined(HPX_HAVE_STDEXEC)
+            using sender_concept = hpx::execution::experimental::sender_t;
 
-            static constexpr bool sends_done = false;
+            template <typename Env>
+            friend auto tag_invoke(
+                hpx::execution::experimental::get_completion_signatures_t,
+                sender const&, Env)
+                -> hpx::execution::experimental::completion_signatures<
+                    hpx::execution::experimental::set_value_t(access_type),
+                    hpx::execution::experimental::set_error_t(
+                        std::exception_ptr)>;
+#else
+            template <typename Env>
+            struct generate_completion_signatures
+            {
+                template <template <typename...> typename Tuple,
+                    template <typename...> typename Variant>
+                using value_types = Variant<Tuple<access_type>>;
+
+                template <template <typename...> typename Variant>
+                using error_types = Variant<std::exception_ptr>;
+
+                static constexpr bool sends_stopped = false;
+            };
+
+            template <typename Env>
+            friend auto tag_invoke(
+                hpx::execution::experimental::get_completion_signatures_t,
+                sender const&, Env) -> generate_completion_signatures<Env>;
+#endif
 
             template <typename R>
             struct operation_state
@@ -635,9 +694,9 @@ namespace hpx { namespace experimental {
                 template <typename R_>
                 operation_state(R_&& r, shared_state_ptr_type prev_state,
                     shared_state_ptr_type state)
-                  : r(std::forward<R_>(r))
-                  , prev_state(std::move(prev_state))
-                  , state(std::move(state))
+                  : r(HPX_FORWARD(R_, r))
+                  , prev_state(HPX_MOVE(prev_state))
+                  , state(HPX_MOVE(state))
                 {
                 }
 
@@ -646,50 +705,51 @@ namespace hpx { namespace experimental {
                 operation_state(operation_state const&) = delete;
                 operation_state& operator=(operation_state const&) = delete;
 
-                void start() noexcept
+                friend void tag_invoke(hpx::execution::experimental::start_t,
+                    operation_state& os) noexcept
                 {
-                    HPX_ASSERT_MSG(state,
+                    HPX_ASSERT_MSG(os.state,
                         "async_rw_lock::sender::operation_state state is "
                         "empty, was the sender already started?");
 
                     auto continuation =
-                        [r = std::move(r)](
+                        [r = HPX_MOVE(os.r)](
                             shared_state_ptr_type state) mutable {
                             try
                             {
                                 hpx::execution::experimental::set_value(
-                                    std::move(r),
-                                    access_type{std::move(state)});
+                                    HPX_MOVE(r), access_type{HPX_MOVE(state)});
                             }
                             catch (...)
                             {
                                 hpx::execution::experimental::set_error(
-                                    std::move(r), std::current_exception());
+                                    HPX_MOVE(r), std::current_exception());
                             }
                         };
 
-                    if (prev_state)
+                    if (os.prev_state)
                     {
-                        prev_state->add_continuation(std::move(continuation));
+                        os.prev_state->add_continuation(HPX_MOVE(continuation));
                         // We release prev_state here to allow continuations to
                         // run. The operation state may otherwise keep it alive
                         // longer than needed.
-                        prev_state.reset();
+                        os.prev_state.reset();
                     }
                     else
                     {
                         // There is no previous state on the first access. We
                         // can immediately trigger the continuation.
-                        continuation(std::move(state));
+                        continuation(HPX_MOVE(os.state));
                     }
                 }
             };
 
             template <typename R>
-            auto connect(R&& r) &&
+            friend auto tag_invoke(
+                hpx::execution::experimental::connect_t, sender&& s, R&& r)
             {
-                return operation_state<R>{std::forward<R>(r),
-                    std::move(prev_state), std::move(state)};
+                return operation_state<R>{HPX_FORWARD(R, r),
+                    HPX_MOVE(s.prev_state), HPX_MOVE(s.state)};
             }
         };
 
@@ -702,4 +762,4 @@ namespace hpx { namespace experimental {
         shared_state_ptr_type prev_state;
         shared_state_ptr_type state;
     };
-}}    // namespace hpx::experimental
+}    // namespace hpx::experimental
