@@ -1,4 +1,4 @@
-//  Copyright (c) 2015 Hartmut Kaiser
+//  Copyright (c) 2015-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,16 +7,16 @@
 // This test case demonstrates the issue described in #1702: Shared_mutex does
 // not compile with no_mutex cond_var
 
-#include <hpx/local/init.hpp>
+#include <hpx/init.hpp>
 #include <hpx/modules/testing.hpp>
-#include <hpx/synchronization/shared_mutex.hpp>
+#include <hpx/shared_mutex.hpp>
 
 #include <mutex>
 #include <shared_mutex>
 
 int hpx_main()
 {
-    typedef hpx::lcos::local::shared_mutex shared_mutex_type;
+    typedef hpx::shared_mutex shared_mutex_type;
 
     int data = 0;
     shared_mutex_type mtx;
