@@ -19,24 +19,25 @@
 #include <hpx/thread_support/spinlock.hpp>
 #include <hpx/util/ios_flags_saver.hpp>
 
-#include <atomic>
 #include <cstddef>
 #include <cstdint>
 #include <mutex>
 #include <ostream>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
+    using test_failure_handler_type = hpx::function<void()>;
 
-    using test_failure_handler_type = function_nonser<void()>;
     HPX_CORE_EXPORT void set_test_failure_handler(test_failure_handler_type f);
 
-    enum counter_type
+    enum class counter_type
     {
-        counter_sanity,
-        counter_test
+        sanity,
+        test
     };
 
     namespace detail {
+
         struct fixture
         {
         public:
@@ -44,19 +45,17 @@ namespace hpx { namespace util {
 
         private:
             std::ostream& stream_;
-            static std::atomic<std::size_t> sanity_failures_;
-            static std::atomic<std::size_t> test_failures_;
             mutex_type mutex_;
 
         public:
-            explicit fixture(std::ostream& stream)
+            explicit fixture(std::ostream& stream) noexcept
               : stream_(stream)
             {
             }
 
-            HPX_CORE_EXPORT void increment(counter_type c);
+            HPX_CORE_EXPORT static void increment(counter_type c) noexcept;
 
-            HPX_CORE_EXPORT std::size_t get(counter_type c) const;
+            HPX_CORE_EXPORT static std::size_t get(counter_type c) noexcept;
 
             template <typename T>
             bool check_(char const* file, int line, char const* function,
@@ -173,18 +172,17 @@ namespace hpx { namespace util {
             }
         };
 
-        HPX_CORE_EXPORT extern fixture global_fixture;
+        HPX_CORE_EXPORT fixture& global_fixture() noexcept;
 
     }    // namespace detail
 
     ////////////////////////////////////////////////////////////////////////////
     HPX_CORE_EXPORT int report_errors();
     HPX_CORE_EXPORT int report_errors(std::ostream& stream);
-    HPX_CORE_EXPORT void print_cdash_timing(const char* name, double time);
+    HPX_CORE_EXPORT void print_cdash_timing(char const* name, double time);
     HPX_CORE_EXPORT void print_cdash_timing(
-        const char* name, std::uint64_t time);
-
-}}    // namespace hpx::util
+        char const* name, std::uint64_t time);
+}    // namespace hpx::util
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST(...)                                                          \
@@ -196,13 +194,14 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))         \
     /**/
 #define HPX_TEST_1(expr)                                                       \
-    HPX_TEST_IMPL(::hpx::util::detail::global_fixture, expr)
+    HPX_TEST_IMPL(::hpx::util::detail::global_fixture(), expr)
 #define HPX_TEST_2(strm, expr)                                                 \
     HPX_TEST_IMPL(::hpx::util::detail::fixture{strm}, expr)
 
 #define HPX_TEST_IMPL(fixture, expr)                                           \
     fixture.check_(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,            \
-        ::hpx::util::counter_test, expr, "test '" HPX_PP_STRINGIZE(expr) "'")
+        ::hpx::util::counter_type::test, expr,                                 \
+        "test '" HPX_PP_STRINGIZE(expr) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_MSG(...)                                                      \
@@ -214,13 +213,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_MSG_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))     \
     /**/
 #define HPX_TEST_MSG_2(expr, msg)                                              \
-    HPX_TEST_MSG_IMPL(::hpx::util::detail::global_fixture, expr, msg)
+    HPX_TEST_MSG_IMPL(::hpx::util::detail::global_fixture(), expr, msg)
 #define HPX_TEST_MSG_3(strm, expr, msg)                                        \
     HPX_TEST_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr, msg)
 
 #define HPX_TEST_MSG_IMPL(fixture, expr, msg)                                  \
     fixture.check_(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,            \
-        ::hpx::util::counter_test, expr, msg)
+        ::hpx::util::counter_type::test, expr, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_EQ(...)                                                       \
@@ -232,13 +231,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_EQ_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))      \
     /**/
 #define HPX_TEST_EQ_2(expr1, expr2)                                            \
-    HPX_TEST_EQ_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_TEST_EQ_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_TEST_EQ_3(strm, expr1, expr2)                                      \
     HPX_TEST_EQ_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_TEST_EQ_IMPL(fixture, expr1, expr2)                                \
     fixture.check_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_test, expr1, expr2,                               \
+        ::hpx::util::counter_type::test, expr1, expr2,                         \
         "test '" HPX_PP_STRINGIZE(expr1) " == " HPX_PP_STRINGIZE(expr2) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -251,13 +250,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_NEQ_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))     \
     /**/
 #define HPX_TEST_NEQ_2(expr1, expr2)                                           \
-    HPX_TEST_NEQ_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_TEST_NEQ_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_TEST_NEQ_3(strm, expr1, expr2)                                     \
     HPX_TEST_NEQ_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_TEST_NEQ_IMPL(fixture, expr1, expr2)                               \
     fixture.check_not_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,   \
-        ::hpx::util::counter_test, expr1, expr2,                               \
+        ::hpx::util::counter_type::test, expr1, expr2,                         \
         "test '" HPX_PP_STRINGIZE(expr1) " != " HPX_PP_STRINGIZE(expr2) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -270,13 +269,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_LT_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))      \
     /**/
 #define HPX_TEST_LT_2(expr1, expr2)                                            \
-    HPX_TEST_LT_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_TEST_LT_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_TEST_LT_3(strm, expr1, expr2)                                      \
     HPX_TEST_LT_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_TEST_LT_IMPL(fixture, expr1, expr2)                                \
     fixture.check_less(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,        \
-        ::hpx::util::counter_test, expr1, expr2,                               \
+        ::hpx::util::counter_type::test, expr1, expr2,                         \
         "test '" HPX_PP_STRINGIZE(expr1) " < " HPX_PP_STRINGIZE(expr2) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -289,13 +288,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_LTE_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))     \
     /**/
 #define HPX_TEST_LTE_2(expr1, expr2)                                           \
-    HPX_TEST_LTE_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_TEST_LTE_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_TEST_LTE_3(strm, expr1, expr2)                                     \
     HPX_TEST_LTE_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_TEST_LTE_IMPL(fixture, expr1, expr2)                               \
     fixture.check_less_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,  \
-        ::hpx::util::counter_test, expr1, expr2,                               \
+        ::hpx::util::counter_type::test, expr1, expr2,                         \
         "test '" HPX_PP_STRINGIZE(expr1) " <= " HPX_PP_STRINGIZE(expr2) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -309,13 +308,13 @@ namespace hpx { namespace util {
     /**/
 #define HPX_TEST_RANGE_3(expr1, expr2, expr3)                                  \
     HPX_TEST_RANGE_IMPL(                                                       \
-        ::hpx::util::detail::global_fixture, expr1, expr2, expr3)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, expr3)
 #define HPX_TEST_RANGE_4(strm, expr1, expr2, expr3)                            \
     HPX_TEST_RANGE_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2, expr3)
 
 #define HPX_TEST_RANGE_IMPL(fixture, expr1, expr2, expr3)                      \
     fixture.check_range(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_test, expr1, expr2, expr3,                        \
+        ::hpx::util::counter_type::test, expr1, expr2, expr3,                  \
         "test '" HPX_PP_STRINGIZE(expr2) " <= " HPX_PP_STRINGIZE(              \
             expr1) " <= " HPX_PP_STRINGIZE(expr3) "'")
 
@@ -329,13 +328,14 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_EQ_MSG_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))  \
     /**/
 #define HPX_TEST_EQ_MSG_3(expr1, expr2, msg)                                   \
-    HPX_TEST_EQ_MSG_IMPL(::hpx::util::detail::global_fixture, expr1, expr2, msg)
+    HPX_TEST_EQ_MSG_IMPL(                                                      \
+        ::hpx::util::detail::global_fixture(), expr1, expr2, msg)
 #define HPX_TEST_EQ_MSG_4(strm, expr1, expr2, msg)                             \
     HPX_TEST_EQ_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2, msg)
 
 #define HPX_TEST_EQ_MSG_IMPL(fixture, expr1, expr2, msg)                       \
     fixture.check_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_test, expr1, expr2, msg)
+        ::hpx::util::counter_type::test, expr1, expr2, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_NEQ_MSG(...)                                                  \
@@ -348,13 +348,13 @@ namespace hpx { namespace util {
     /**/
 #define HPX_TEST_NEQ_MSG_3(expr1, expr2, msg)                                  \
     HPX_TEST_NEQ_MSG_IMPL(                                                     \
-        ::hpx::util::detail::global_fixture, expr1, expr2, msg)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, msg)
 #define HPX_TEST_NEQ_MSG_4(strm, expr1, expr2, msg)                            \
     HPX_TEST_NEQ_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2, msg)
 
 #define HPX_TEST_NEQ_MSG_IMPL(fixture, expr1, expr2, msg)                      \
     fixture.check_not_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,   \
-        ::hpx::util::counter_test, expr1, expr2, msg)
+        ::hpx::util::counter_type::test, expr1, expr2, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_LT_MSG(...)                                                   \
@@ -366,13 +366,14 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_TEST_LT_MSG_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))  \
     /**/
 #define HPX_TEST_LT_MSG_3(expr1, expr2, msg)                                   \
-    HPX_TEST_LT_MSG_IMPL(::hpx::util::detail::global_fixture, expr1, expr2, msg)
+    HPX_TEST_LT_MSG_IMPL(                                                      \
+        ::hpx::util::detail::global_fixture(), expr1, expr2, msg)
 #define HPX_TEST_LT_MSG_4(strm, expr1, expr2, msg)                             \
     HPX_TEST_LT_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2, msg)
 
 #define HPX_TEST_LT_MSG_IMPL(fixture, expr1, expr2, msg)                       \
     fixture.check_less(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,        \
-        ::hpx::util::counter_test, expr1, expr2, msg)
+        ::hpx::util::counter_type::test, expr1, expr2, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_LTE_MSG(...)                                                  \
@@ -385,13 +386,13 @@ namespace hpx { namespace util {
     /**/
 #define HPX_TEST_LTE_MSG_3(expr1, expr2, msg)                                  \
     HPX_TEST_LTE_MSG_IMPL(                                                     \
-        ::hpx::util::detail::global_fixture, expr1, expr2, msg)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, msg)
 #define HPX_TEST_LTE_MSG_4(strm, expr1, expr2, msg)                            \
     HPX_TEST_LTE_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2, msg)
 
 #define HPX_TEST_LTE_MSG_IMPL(fixture, expr1, expr2, msg)                      \
     fixture.check_less_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,  \
-        ::hpx::util::counter_test, expr1, expr2, msg)
+        ::hpx::util::counter_type::test, expr1, expr2, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_RANGE_MSG(...)                                                \
@@ -404,14 +405,14 @@ namespace hpx { namespace util {
     /**/
 #define HPX_TEST_RANGE_MSG_4(expr1, expr2, expr3, msg)                         \
     HPX_TEST_RANGE_MSG_IMPL(                                                   \
-        ::hpx::util::detail::global_fixture, expr1, expr2, expr3, msg)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, expr3, msg)
 #define HPX_TEST_RANGE_MSG_5(strm, expr1, expr2, expr3, msg)                   \
     HPX_TEST_RANGE_MSG_IMPL(                                                   \
         ::hpx::util::detail::fixture{strm}, expr1, expr2, expr3, msg)
 
 #define HPX_TEST_RANGE_MSG_IMPL(fixture, expr1, expr2, expr3, msg)             \
     fixture.check_range(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_test, expr1, expr2, expr3, msg)
+        ::hpx::util::counter_type::test, expr1, expr2, expr3, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_SANITY(...)                                                        \
@@ -423,13 +424,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))       \
     /**/
 #define HPX_SANITY_1(expr)                                                     \
-    HPX_TEST_IMPL(::hpx::util::detail::global_fixture, expr)
+    HPX_TEST_IMPL(::hpx::util::detail::global_fixture(), expr)
 #define HPX_SANITY_2(strm, expr)                                               \
     HPX_SANITY_IMPL(::hpx::util::detail::fixture{strm}, expr)
 
 #define HPX_SANITY_IMPL(fixture, expr)                                         \
     fixture.check_(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,            \
-        ::hpx::util::counter_sanity, expr,                                     \
+        ::hpx::util::counter_type::sanity, expr,                               \
         "sanity check '" HPX_PP_STRINGIZE(expr) "'")
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -442,13 +443,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_MSG_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))   \
     /**/
 #define HPX_SANITY_MSG_2(expr, msg)                                            \
-    HPX_SANITY_MSG_IMPL(::hpx::util::detail::global_fixture, expr, msg)
+    HPX_SANITY_MSG_IMPL(::hpx::util::detail::global_fixture(), expr, msg)
 #define HPX_SANITY_MSG_3(strm, expr, msg)                                      \
     HPX_SANITY_MSG_IMPL(::hpx::util::detail::fixture{strm}, expr, msg)
 
 #define HPX_SANITY_MSG_IMPL(fixture, expr, msg)                                \
     fixture.check_(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,            \
-        ::hpx::util::counter_sanity, expr, msg)
+        ::hpx::util::counter_type::sanity, expr, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_SANITY_EQ(...)                                                     \
@@ -460,13 +461,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_EQ_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))    \
     /**/
 #define HPX_SANITY_EQ_2(expr1, expr2)                                          \
-    HPX_SANITY_EQ_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_SANITY_EQ_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_SANITY_EQ_3(strm, expr1, expr2)                                    \
     HPX_SANITY_EQ_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_SANITY_EQ_IMPL(fixture, expr1, expr2)                              \
     fixture.check_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_sanity, expr1, expr2,                             \
+        ::hpx::util::counter_type::sanity, expr1, expr2,                       \
         "sanity check '" HPX_PP_STRINGIZE(expr1) " == " HPX_PP_STRINGIZE(      \
             expr2) "'")
 
@@ -480,13 +481,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_NEQ_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))   \
     /**/
 #define HPX_SANITY_NEQ_2(expr1, expr2)                                         \
-    HPX_SANITY_NEQ_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_SANITY_NEQ_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_SANITY_NEQ_3(strm, expr1, expr2)                                   \
     HPX_SANITY_NEQ_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_SANITY_NEQ_IMPL(fixture, expr1, expr2)                             \
     fixture.check_not_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,   \
-        ::hpx::util::counter_sanity, expr1, expr2,                             \
+        ::hpx::util::counter_type::sanity, expr1, expr2,                       \
         "sanity check '" HPX_PP_STRINGIZE(expr1) " != " HPX_PP_STRINGIZE(      \
             expr2) "'")
 
@@ -500,13 +501,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_LT_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))    \
     /**/
 #define HPX_SANITY_LT_2(expr1, expr2)                                          \
-    HPX_SANITY_LT_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_SANITY_LT_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_SANITY_LT_3(strm, expr1, expr2)                                    \
     HPX_SANITY_LT_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_SANITY_LT_IMPL(fixture, expr1, expr2)                              \
     fixture.check_less(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,        \
-        ::hpx::util::counter_sanity, expr1, expr2,                             \
+        ::hpx::util::counter_type::sanity, expr1, expr2,                       \
         "sanity check '" HPX_PP_STRINGIZE(expr1) " < " HPX_PP_STRINGIZE(       \
             expr2) "'")
 
@@ -520,13 +521,13 @@ namespace hpx { namespace util {
         HPX_PP_CAT(HPX_SANITY_LTE_, HPX_PP_NARGS(__VA_ARGS__))(__VA_ARGS__))   \
     /**/
 #define HPX_SANITY_LTE_2(expr1, expr2)                                         \
-    HPX_SANITY_LTE_IMPL(::hpx::util::detail::global_fixture, expr1, expr2)
+    HPX_SANITY_LTE_IMPL(::hpx::util::detail::global_fixture(), expr1, expr2)
 #define HPX_SANITY_LTE_3(strm, expr1, expr2)                                   \
     HPX_SANITY_LTE_IMPL(::hpx::util::detail::fixture{strm}, expr1, expr2)
 
 #define HPX_SANITY_LTE_IMPL(fixture, expr1, expr2)                             \
     fixture.check_less_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,  \
-        ::hpx::util::counter_sanity, expr1, expr2,                             \
+        ::hpx::util::counter_type::sanity, expr1, expr2,                       \
         "sanity check '" HPX_PP_STRINGIZE(expr1) " <= " HPX_PP_STRINGIZE(      \
             expr2) "'")
 
@@ -541,14 +542,14 @@ namespace hpx { namespace util {
     /**/
 #define HPX_SANITY_RANGE_3(expr1, expr2, expr3)                                \
     HPX_SANITY_RANGE_IMPL(                                                     \
-        ::hpx::util::detail::global_fixture, expr1, expr2, expr3)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, expr3)
 #define HPX_SANITY_RANGE_4(strm, expr1, expr2, expr3)                          \
     HPX_SANITY_RANGE_IMPL(                                                     \
         ::hpx::util::detail::fixture{strm}, expr1, expr2, expr3)
 
 #define HPX_SANITY_RANGE_IMPL(fixture, expr1, expr2, expr3)                    \
     fixture.check_range(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_sanity, expr1, expr2, expr3,                      \
+        ::hpx::util::counter_type::sanity, expr1, expr2, expr3,                \
         "sanity check '" HPX_PP_STRINGIZE(expr2) " <= " HPX_PP_STRINGIZE(      \
             expr1) " <= " HPX_PP_STRINGIZE(expr3) "'")
 
@@ -563,14 +564,14 @@ namespace hpx { namespace util {
     /**/
 #define HPX_SANITY_EQ_MSG_3(expr1, expr2, msg)                                 \
     HPX_SANITY_EQ_MSG_IMPL(                                                    \
-        ::hpx::util::detail::global_fixture, expr1, expr2, msg)
+        ::hpx::util::detail::global_fixture(), expr1, expr2, msg)
 #define HPX_SANITY_EQ_MSG_4(strm, expr1, expr2, msg)                           \
     HPX_SANITY_EQ_MSG_IMPL(                                                    \
         ::hpx::util::detail::fixture{strm}, expr1, expr2, msg)
 
 #define HPX_SANITY_EQ_MSG_IMPL(fixture, expr1, expr2, msg)                     \
     fixture.check_equal(__FILE__, __LINE__, HPX_ASSERT_CURRENT_FUNCTION,       \
-        ::hpx::util::counter_sanity, expr1, expr2, msg)
+        ::hpx::util::counter_type::sanity, expr1, expr2, msg)
 
 ////////////////////////////////////////////////////////////////////////////////
 #define HPX_TEST_THROW(...)                                                    \
@@ -583,7 +584,7 @@ namespace hpx { namespace util {
     /**/
 #define HPX_TEST_THROW_2(expression, exception)                                \
     HPX_TEST_THROW_IMPL(                                                       \
-        ::hpx::util::detail::global_fixture, expression, exception)
+        ::hpx::util::detail::global_fixture(), expression, exception)
 #define HPX_TEST_THROW_3(strm, expression, exception)                          \
     HPX_TEST_THROW_IMPL(                                                       \
         ::hpx::util::detail::fixture{strm}, expression, exception)
