@@ -12,13 +12,13 @@
 #include <cstddef>
 #include <cstdint>
 
-namespace hpx { namespace threads { namespace detail {
+namespace hpx::threads::detail {
+
 #if defined(HPX_HAVE_BACKGROUND_THREAD_COUNTERS) &&                            \
     defined(HPX_HAVE_THREAD_IDLE_RATES)
     using network_background_callback_type =
-        util::function_nonser<bool(std::size_t, std::int64_t&, std::int64_t&)>;
+        hpx::function<bool(std::size_t, std::int64_t&, std::int64_t&)>;
 #else
-    using network_background_callback_type =
-        util::function_nonser<bool(std::size_t)>;
+    using network_background_callback_type = hpx::function<bool(std::size_t)>;
 #endif
-}}}    // namespace hpx::threads::detail
+}    // namespace hpx::threads::detail
