@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2025 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //  Copyright (c) 2008-2009 Chirag Dekate, Anshul Tandon
 //
@@ -10,6 +10,7 @@
 
 #include <hpx/config.hpp>
 #include <hpx/assert.hpp>
+
 #include <hpx/concurrency/spinlock_pool.hpp>
 #include <hpx/coroutines/coroutine.hpp>
 #include <hpx/coroutines/detail/combined_tagged_state.hpp>
@@ -19,10 +20,9 @@
 #include <hpx/functional/function.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/modules/logging.hpp>
-#include <hpx/modules/memory.hpp>
-#include <hpx/thread_support/atomic_count.hpp>
 #include <hpx/threading_base/thread_description.hpp>
 #include <hpx/threading_base/thread_init_data.hpp>
+#include <hpx/threading_base/threading_base_fwd.hpp>
 #if defined(HPX_HAVE_APEX)
 #include <hpx/threading_base/external_timer.hpp>
 #endif
@@ -31,40 +31,44 @@
 #include <cstddef>
 #include <cstdint>
 #include <forward_list>
+#include <limits>
 #include <memory>
 #include <mutex>
-#include <stack>
-#include <string>
 #include <utility>
 
+#if defined(HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION)
+#include <string>
+#endif
+
 #include <hpx/config/warnings_prefix.hpp>
 
 ////////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace threads {
+namespace hpx::threads {
+
     namespace detail {
+
         using get_locality_id_type = std::uint32_t(hpx::error_code&);
         HPX_CORE_EXPORT void set_get_locality_id(get_locality_id_type* f);
         HPX_CORE_EXPORT std::uint32_t get_locality_id(hpx::error_code&);
     }    // namespace detail
 
     ////////////////////////////////////////////////////////////////////////////
-    /// A \a thread is the representation of a ParalleX thread. It's a first
-    /// class object in ParalleX. In our implementation this is a user level
-    /// thread running on top of one of the OS threads spawned by the \a
-    /// thread-manager.
+    /// A \a thread is the representation of a HPX thread. It's a first class
+    /// object in HPX. In our implementation this is a user level thread running
+    /// on top of one of the OS threads spawned by the \a thread-manager.
     ///
     /// A \a thread encapsulates:
     ///  - A thread status word (see the functions \a thread#get_state and
     ///    \a thread#set_state)
     ///  - A function to execute (the thread function)
-    ///  - A frame (in this implementation this is a block of memory used as
-    ///    the threads stack)
+    ///  - A frame (in this implementation this is a block of memory used as the
+    ///    threads stack)
     ///  - A block of registers (not implemented yet)
     ///
     /// Generally, \a threads are not created or executed directly. All
-    /// functionality related to the management of \a threads is
-    /// implemented by the thread-manager.
-    class HPX_CORE_EXPORT thread_data
+    /// functionality related to the management of \a threads is implemented by
+    /// the thread-manager.
+    class thread_data : public detail::thread_data_reference_counting
     {
     public:
         thread_data(thread_data const&) = delete;
@@ -84,29 +88,32 @@ namespace hpx { namespace threads {
         /// \note           This function will be seldom used directly. Most of
         ///                 the time the state of a thread will be retrieved
         ///                 by using the function \a threadmanager#get_state.
-        thread_state get_state(
-            std::memory_order order = std::memory_order_acquire) const noexcept
+        thread_state get_state(std::memory_order const order =
+                                   std::memory_order_acquire) const noexcept
         {
             return current_state_.load(order);
         }
 
         /// The set_state function changes the state of this thread instance.
         ///
-        /// \param newstate [in] The new state to be set for the thread.
+        /// \param state    [in] The new state to be set for the thread.
+        /// \param state_ex       [in]
+        /// \param load_order     [in]
+        /// \param exchange_order [in]
         ///
         /// \note           This function will be seldom used directly. Most of
         ///                 the time the state of a thread will have to be
-        ///                 changed using the threadmanager. Moreover,
+        ///                 changed using the thread-manager. Moreover,
         ///                 changing the thread state using this function does
         ///                 not change its scheduling status. It only sets the
         ///                 thread's status word. To change the thread's
         ///                 scheduling status \a threadmanager#set_state should
         ///                 be used.
-        thread_state set_state(thread_schedule_state state,
+        thread_state set_state(thread_schedule_state const state,
             thread_restart_state state_ex = thread_restart_state::unknown,
-            std::memory_order load_order = std::memory_order_acquire,
-            std::memory_order exchange_order =
-                std::memory_order_seq_cst) noexcept
+            std::memory_order const load_order = std::memory_order_acquire,
+            std::memory_order const exchange_order =
+                std::memory_order_acq_rel) const noexcept
         {
             thread_state prev_state = current_state_.load(load_order);
 
@@ -132,25 +139,17 @@ namespace hpx { namespace threads {
             }
         }
 
-        bool set_state_tagged(thread_schedule_state newstate,
-            thread_state& prev_state, thread_state& new_tagged_state,
+        bool set_state_tagged(thread_schedule_state const newstate,
+            thread_state const& prev_state, thread_state& new_tagged_state,
             std::memory_order exchange_order =
-                std::memory_order_seq_cst) noexcept
+                std::memory_order_acq_rel) const noexcept
         {
+            new_tagged_state = thread_state(
+                newstate, prev_state.state_ex(), prev_state.tag() + 1);
+
             thread_state tmp = prev_state;
-            thread_restart_state state_ex = tmp.state_ex();
-
-            new_tagged_state =
-                thread_state(newstate, state_ex, prev_state.tag() + 1);
-
-            if (!current_state_.compare_exchange_strong(
-                    tmp, new_tagged_state, exchange_order))
-            {
-                return false;
-            }
-
-            prev_state = tmp;
-            return true;
+            return current_state_.compare_exchange_strong(
+                tmp, new_tagged_state, exchange_order);
         }
 
         /// The restore_state function changes the state of this thread
@@ -159,9 +158,11 @@ namespace hpx { namespace threads {
         /// as the second parameter. Otherwise it won't touch the thread state
         /// of this instance.
         ///
-        /// \param newstate [in] The new state to be set for the thread.
-        /// \param oldstate [in] The old state of the thread which still has to
+        /// \param new_state [in] The new state to be set for the thread.
+        /// \param old_state [in] The old state of the thread which still has to
         ///                 be the current state.
+        /// \param load_order    [in]
+        /// \param load_exchange [in]
         ///
         /// \note           This function will be seldom used directly. Most of
         ///                 the time the state of a thread will have to be
@@ -174,31 +175,32 @@ namespace hpx { namespace threads {
         ///
         /// \returns This function returns \a true if the state has been
         ///          changed successfully
-        bool restore_state(thread_state new_state, thread_state old_state,
-            std::memory_order load_order = std::memory_order_relaxed,
-            std::memory_order load_exchange =
-                std::memory_order_seq_cst) noexcept
+        bool restore_state(thread_state const new_state,
+            thread_state const old_state,
+            std::memory_order const load_order = std::memory_order_relaxed,
+            std::memory_order const load_exchange =
+                std::memory_order_acq_rel) const noexcept
         {
+            // ignore the state_ex while compare-exchanging
+            thread_state const current_state = current_state_.load(load_order);
+            thread_restart_state const state_ex = current_state.state_ex();
+
             // ABA prevention for state only (not for state_ex)
-            std::int64_t tag = old_state.tag();
+            std::int64_t tag = current_state.tag();
             if (new_state.state() != old_state.state())
                 ++tag;
 
-            // ignore the state_ex while compare-exchanging
-            thread_restart_state state_ex =
-                current_state_.load(load_order).state_ex();
-
             thread_state old_tmp(old_state.state(), state_ex, old_state.tag());
-            thread_state new_tmp(new_state.state(), state_ex, tag);
+            thread_state const new_tmp(new_state.state(), state_ex, tag);
 
             return current_state_.compare_exchange_strong(
                 old_tmp, new_tmp, load_exchange);
         }
 
         bool restore_state(thread_schedule_state new_state,
-            thread_restart_state state_ex, thread_state old_state,
-            std::memory_order load_exchange =
-                std::memory_order_seq_cst) noexcept
+            thread_restart_state const state_ex, thread_state old_state,
+            std::memory_order const load_exchange =
+                std::memory_order_acq_rel) const noexcept
         {
             // ABA prevention for state only (not for state_ex)
             std::int64_t tag = old_state.tag();
@@ -213,24 +215,28 @@ namespace hpx { namespace threads {
         /// The set_state function changes the extended state of this
         /// thread instance.
         ///
-        /// \param newstate [in] The new extended state to be set for the
+        /// \param new_state [in] The new extended state to be set for the
         ///                 thread.
+        /// \param load_order [in]
+        /// \param load_exchange [in]
         ///
         /// \note           This function will be seldom used directly. Most of
         ///                 the time the state of a thread will have to be
         ///                 changed using the threadmanager.
-        thread_restart_state set_state_ex(
-            thread_restart_state new_state) noexcept
+        thread_restart_state set_state_ex(thread_restart_state const new_state,
+            std::memory_order const load_order = std::memory_order_acquire,
+            std::memory_order const load_exchange =
+                std::memory_order_acq_rel) const noexcept
         {
-            thread_state prev_state =
-                current_state_.load(std::memory_order_acquire);
+            thread_state prev_state = current_state_.load(load_order);
 
             for (;;)
             {
                 thread_state tmp = prev_state;
 
-                if (HPX_LIKELY(current_state_.compare_exchange_strong(
-                        tmp, thread_state(tmp.state(), new_state, tmp.tag()))))
+                if (HPX_LIKELY(current_state_.compare_exchange_strong(tmp,
+                        thread_state(tmp.state(), new_state, tmp.tag()),
+                        load_exchange)))
                 {
                     return prev_state.state_ex();
                 }
@@ -241,43 +247,42 @@ namespace hpx { namespace threads {
 
     public:
         /// Return the id of the component this thread is running in
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            std::uint64_t    // same as naming::address_type
-            get_component_id() const noexcept
+        static constexpr std::uint64_t    // same as naming::address_type
+        get_component_id() noexcept
         {
             return 0;
         }
 
 #if !defined(HPX_HAVE_THREAD_DESCRIPTION)
-        util::thread_description get_description() const
+        static constexpr threads::thread_description get_description() noexcept
         {
-            return util::thread_description("<unknown>");
+            return {"<unknown>"};
         }
-        util::thread_description set_description(
-            util::thread_description /*value*/)
+        static constexpr threads::thread_description set_description(
+            threads::thread_description /*value*/) noexcept
         {
-            return util::thread_description("<unknown>");
+            return {"<unknown>"};
         }
 
-        util::thread_description get_lco_description() const
+        static constexpr threads::thread_description
+        get_lco_description() noexcept    //-V524
         {
-            return util::thread_description("<unknown>");
+            return {"<unknown>"};
         }
-        util::thread_description set_lco_description(
-            util::thread_description /*value*/)
+        static constexpr threads::thread_description set_lco_description(
+            threads::thread_description /*value*/) noexcept    //-V524
         {
-            return util::thread_description("<unknown>");
+            return {"<unknown>"};
         }
 #else
-        util::thread_description get_description() const
+        threads::thread_description get_description() const
         {
             std::lock_guard<hpx::util::detail::spinlock> l(
                 spinlock_pool::spinlock_for(this));
             return description_;
         }
-        util::thread_description set_description(util::thread_description value)
+        threads::thread_description set_description(
+            threads::thread_description value)
         {
             std::lock_guard<hpx::util::detail::spinlock> l(
                 spinlock_pool::spinlock_for(this));
@@ -285,14 +290,14 @@ namespace hpx { namespace threads {
             return value;
         }
 
-        util::thread_description get_lco_description() const
+        threads::thread_description get_lco_description() const
         {
             std::lock_guard<hpx::util::detail::spinlock> l(
                 spinlock_pool::spinlock_for(this));
             return lco_description_;
         }
-        util::thread_description set_lco_description(
-            util::thread_description value)
+        threads::thread_description set_lco_description(
+            threads::thread_description value)
         {
             std::lock_guard<hpx::util::detail::spinlock> l(
                 spinlock_pool::spinlock_for(this));
@@ -303,32 +308,20 @@ namespace hpx { namespace threads {
 
 #if !defined(HPX_HAVE_THREAD_PARENT_REFERENCE)
         /// Return the locality of the parent thread
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            std::uint32_t
-            get_parent_locality_id() const noexcept
+        static constexpr std::uint32_t get_parent_locality_id() noexcept
         {
             // this is the same as naming::invalid_locality_id
             return ~static_cast<std::uint32_t>(0);
         }
 
         /// Return the thread id of the parent thread
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            thread_id_type
-            get_parent_thread_id() const noexcept
+        static constexpr thread_id_type get_parent_thread_id() noexcept
         {
             return threads::invalid_thread_id;
         }
 
         /// Return the phase of the parent thread
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            std::size_t
-            get_parent_thread_phase() const noexcept
+        static constexpr std::size_t get_parent_thread_phase() noexcept
         {
             return 0;
         }
@@ -366,28 +359,21 @@ namespace hpx { namespace threads {
 #if !defined(HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION)
 
 #ifdef HPX_HAVE_THREAD_FULLBACKTRACE_ON_SUSPENSION
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            char const*
-            get_backtrace() const noexcept
+        static constexpr char const* get_backtrace() noexcept
         {
             return nullptr;
         }
-        char const* set_backtrace(char const*) noexcept
+        static constexpr char const* set_backtrace(char const*) noexcept
         {
             return nullptr;
         }
 #else
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            util::backtrace const*
-            get_backtrace() const noexcept
+        static constexpr util::backtrace const* get_backtrace() noexcept
         {
             return nullptr;
         }
-        util::backtrace const* set_backtrace(util::backtrace const*) noexcept
+        static constexpr util::backtrace const* set_backtrace(
+            util::backtrace const*) noexcept
         {
             return nullptr;
         }
@@ -449,11 +435,7 @@ namespace hpx { namespace threads {
         }
 #endif
 
-#if !defined(HPX_GCC_VERSION) || (HPX_GCC_VERSION >= 70300)
-        constexpr
-#endif
-            thread_priority
-            get_priority() const noexcept
+        constexpr thread_priority get_priority() const noexcept
         {
             return priority_;
         }
@@ -492,39 +474,55 @@ namespace hpx { namespace threads {
             if (flag && !enabled_interrupt_)
             {
                 l.unlock();
-                HPX_THROW_EXCEPTION(thread_not_interruptable,
+                HPX_THROW_EXCEPTION(hpx::error::thread_not_interruptable,
                     "thread_data::interrupt",
                     "interrupts are disabled for this thread");
-                return;
             }
             requested_interrupt_ = flag;
         }
 
         bool interruption_point(bool throw_on_interrupt = true);
 
-        bool add_thread_exit_callback(util::function_nonser<void()> const& f);
+        bool add_thread_exit_callback(function<void()> const& f);
         void run_thread_exit_callbacks();
         void free_thread_exit_callbacks();
 
-        policies::scheduler_base* get_scheduler_base() const noexcept
+        // no need to protect the variables related to scoped children as those
+        // are supposed to be accessed by ourselves only
+        bool runs_as_child(
+            std::memory_order mo = std::memory_order_acquire) const noexcept
+        {
+            return runs_as_child_.load(mo);
+        }
+
+        HPX_FORCEINLINE constexpr bool is_stackless() const noexcept
+        {
+            return is_stackless_;
+        }
+
+        void destroy_thread() override;
+
+        constexpr policies::scheduler_base* get_scheduler_base() const noexcept
         {
             return scheduler_base_;
         }
 
-        std::size_t get_last_worker_thread_num() const noexcept
+        constexpr std::uint16_t get_last_worker_thread_num() const noexcept
         {
             return last_worker_thread_num_;
         }
 
         void set_last_worker_thread_num(
-            std::size_t last_worker_thread_num) noexcept
+            std::uint16_t last_worker_thread_num) noexcept
         {
             last_worker_thread_num_ = last_worker_thread_num;
         }
 
-        std::ptrdiff_t get_stack_size() const noexcept
+        constexpr std::ptrdiff_t get_stack_size() const noexcept
         {
-            return stacksize_;
+            return stacksize_enum_ == thread_stacksize::nostack ?
+                (std::numeric_limits<std::ptrdiff_t>::max)() :
+                stacksize_;
         }
 
         thread_stacksize get_stack_size_enum() const noexcept
@@ -533,7 +531,7 @@ namespace hpx { namespace threads {
         }
 
         template <typename ThreadQueue>
-        ThreadQueue& get_queue() noexcept
+        constexpr ThreadQueue& get_queue() noexcept
         {
             return *static_cast<ThreadQueue*>(queue_);
         }
@@ -548,6 +546,14 @@ namespace hpx { namespace threads {
             hpx::execution_base::this_thread::detail::agent_storage*
                 agent_storage);
 
+        /// \brief Directly execute the thread function (inline)
+        ///
+        /// \returns        This function returns the thread state the thread
+        ///                 should be scheduled from this point on. The thread
+        ///                 manager will use the returned value to set the
+        ///                 thread's scheduling status.
+        inline coroutine_type::result_type invoke_directly();
+
         virtual thread_id_type get_thread_id() const
         {
             return thread_id_type{const_cast<thread_data*>(this)};
@@ -575,6 +581,7 @@ namespace hpx { namespace threads {
             std::size_t data) = 0;
 #endif
 
+        virtual void init() = 0;
         virtual void rebind(thread_init_data& init_data) = 0;
 
 #if defined(HPX_HAVE_APEX)
@@ -592,22 +599,50 @@ namespace hpx { namespace threads {
 
         // Construct a new \a thread
         thread_data(thread_init_data& init_data, void* queue,
-            std::ptrdiff_t stacksize, bool is_stackless = false);
+            std::ptrdiff_t stacksize, bool is_stackless = false,
+            thread_id_addref addref = thread_id_addref::yes);
 
-        virtual ~thread_data();
-        virtual void destroy() = 0;
+        virtual ~thread_data() override;
+        virtual void destroy() noexcept = 0;
 
     protected:
         void rebind_base(thread_init_data& init_data);
 
     private:
+        thread_priority priority_;
+
+        bool requested_interrupt_;
+        bool enabled_interrupt_;
+        bool ran_exit_funcs_;
+        bool const is_stackless_;
+
+        // support scoped child execution
+        std::atomic<bool> runs_as_child_;
+
+        std::uint16_t last_worker_thread_num_;
+
+        thread_stacksize stacksize_enum_;
+        std::int32_t stacksize_;
+
         mutable std::atomic<thread_state> current_state_;
 
+        // Singly linked list (heap-allocated)
+        std::forward_list<hpx::function<void()>> exit_funcs_;
+
+        // reference to scheduler which created/manages this thread
+        policies::scheduler_base* scheduler_base_;
+
+        void* queue_;
+
         ///////////////////////////////////////////////////////////////////////
         // Debugging/logging information
 #ifdef HPX_HAVE_THREAD_DESCRIPTION
-        util::thread_description description_;
-        util::thread_description lco_description_;
+        threads::thread_description description_;
+        threads::thread_description lco_description_;
+#endif
+
+#ifdef HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION
+        mutable thread_schedule_state marked_state_;
 #endif
 
 #ifdef HPX_HAVE_THREAD_PARENT_REFERENCE
@@ -616,137 +651,61 @@ namespace hpx { namespace threads {
         std::size_t parent_thread_phase_;
 #endif
 
-#ifdef HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION
-        mutable thread_schedule_state marked_state_;
-#endif
-
 #ifdef HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION
 #ifdef HPX_HAVE_THREAD_FULLBACKTRACE_ON_SUSPENSION
-        char const* backtrace_;
+        char const* backtrace_ = nullptr;
 #else
-        util::backtrace const* backtrace_;
+        util::backtrace const* backtrace_ = nullptr;
 #endif
 #endif
-        ///////////////////////////////////////////////////////////////////////
-        thread_priority priority_;
-
-        bool requested_interrupt_;
-        bool enabled_interrupt_;
-        bool ran_exit_funcs_;
-
-        // Singly linked list (heap-allocated)
-        std::forward_list<util::function_nonser<void()>> exit_funcs_;
-
-        // reference to scheduler which created/manages this thread
-        policies::scheduler_base* scheduler_base_;
-        std::size_t last_worker_thread_num_;
-
-        std::ptrdiff_t stacksize_;
-        thread_stacksize stacksize_enum_;
-
-        void* queue_;
 
     public:
 #if defined(HPX_HAVE_APEX)
         std::shared_ptr<util::external_timer::task_wrapper> timer_data_;
 #endif
-        bool is_stackless_;
     };
 
-    constexpr inline thread_data* get_thread_id_data(thread_id_type const& tid)
+    HPX_FORCEINLINE constexpr thread_data* get_thread_id_data(
+        thread_id_ref_type const& tid) noexcept
+    {
+        return static_cast<thread_data*>(tid.get().get());
+    }
+
+    HPX_FORCEINLINE constexpr thread_data* get_thread_id_data(
+        thread_id_type const& tid) noexcept
     {
         return static_cast<thread_data*>(tid.get());
     }
-
-    namespace detail {
-        HPX_CORE_EXPORT void set_self_ptr(thread_self*);
-    }
-
-    ///////////////////////////////////////////////////////////////////////
-    /// The function \a get_self returns a reference to the (OS thread
-    /// specific) self reference to the current HPX thread.
-    HPX_CORE_EXPORT thread_self& get_self();
-
-    /// The function \a get_self_ptr returns a pointer to the (OS thread
-    /// specific) self reference to the current HPX thread.
-    HPX_CORE_EXPORT thread_self* get_self_ptr();
-
-    /// The function \a get_ctx_ptr returns a pointer to the internal data
-    /// associated with each coroutine.
-    HPX_CORE_EXPORT thread_self_impl_type* get_ctx_ptr();
-
-    /// The function \a get_self_ptr_checked returns a pointer to the (OS
-    /// thread specific) self reference to the current HPX thread.
-    HPX_CORE_EXPORT thread_self* get_self_ptr_checked(error_code& ec = throws);
-
-    /// The function \a get_self_id returns the HPX thread id of the current
-    /// thread (or zero if the current thread is not a HPX thread).
-    HPX_CORE_EXPORT thread_id_type get_self_id();
-
-    /// The function \a get_self_id_data returns the data of the HPX thread id
-    /// associated with the current thread (or nullptr if the current thread is
-    /// not a HPX thread).
-    HPX_CORE_EXPORT thread_data* get_self_id_data();
-
-    /// The function \a get_parent_id returns the HPX thread id of the
-    /// current thread's parent (or zero if the current thread is not a
-    /// HPX thread).
-    ///
-    /// \note This function will return a meaningful value only if the
-    ///       code was compiled with HPX_HAVE_THREAD_PARENT_REFERENCE
-    ///       being defined.
-    HPX_CORE_EXPORT thread_id_type get_parent_id();
-
-    /// The function \a get_parent_phase returns the HPX phase of the
-    /// current thread's parent (or zero if the current thread is not a
-    /// HPX thread).
-    ///
-    /// \note This function will return a meaningful value only if the
-    ///       code was compiled with HPX_HAVE_THREAD_PARENT_REFERENCE
-    ///       being defined.
-    HPX_CORE_EXPORT std::size_t get_parent_phase();
-
-    /// The function \a get_self_stacksize returns the stack size of the
-    /// current thread (or zero if the current thread is not a HPX thread).
-    HPX_CORE_EXPORT std::ptrdiff_t get_self_stacksize();
-
-    /// The function \a get_self_stacksize_enum returns the stack size of the /
-    //current thread (or thread_stacksize::default if the current thread is not
-    //a HPX thread).
-    HPX_CORE_EXPORT thread_stacksize get_self_stacksize_enum();
-
-    /// The function \a get_parent_locality_id returns the id of the locality of
-    /// the current thread's parent (or zero if the current thread is not a
-    /// HPX thread).
-    ///
-    /// \note This function will return a meaningful value only if the
-    ///       code was compiled with HPX_HAVE_THREAD_PARENT_REFERENCE
-    ///       being defined.
-    HPX_CORE_EXPORT std::uint32_t get_parent_locality_id();
-
-    /// The function \a get_self_component_id returns the lva of the
-    /// component the current thread is acting on
-    ///
-    /// \note This function will return a meaningful value only if the
-    ///       code was compiled with HPX_HAVE_THREAD_TARGET_ADDRESS
-    ///       being defined.
-    HPX_CORE_EXPORT std::uint64_t get_self_component_id();
-}}    // namespace hpx::threads
+}    // namespace hpx::threads
 
 #include <hpx/config/warnings_suffix.hpp>
 
 #include <hpx/threading_base/thread_data_stackful.hpp>
 #include <hpx/threading_base/thread_data_stackless.hpp>
 
-namespace hpx { namespace threads {
+namespace hpx::threads {
 
-    inline coroutine_type::result_type thread_data::operator()(
+    HPX_FORCEINLINE coroutine_type::result_type thread_data::operator()(
         hpx::execution_base::this_thread::detail::agent_storage* agent_storage)
     {
-        if (is_stackless_)
+        // once a thread has started it can't be run directly anymore
+        runs_as_child_.store(false, std::memory_order_release);
+
+        if (is_stackless())
         {
             return static_cast<thread_data_stackless*>(this)->call();
         }
         return static_cast<thread_data_stackful*>(this)->call(agent_storage);
     }
-}}    // namespace hpx::threads
+
+    HPX_FORCEINLINE coroutine_type::result_type thread_data::invoke_directly()
+    {
+        HPX_ASSERT(runs_as_child(std::memory_order_relaxed));
+
+        if (is_stackless())
+        {
+            return static_cast<thread_data_stackless*>(this)->call();
+        }
+        return static_cast<thread_data_stackful*>(this)->invoke_directly();
+    }
+}    // namespace hpx::threads
