@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2015 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2018 Thomas Heller
 //  Copyright (c)      2011 Bryce Lelbach
 //  Copyright (c) 2008-2009 Chirag Dekate, Anshul Tandon
@@ -11,25 +11,24 @@
 
 #include <hpx/config.hpp>
 #include <hpx/coroutines/thread_enums.hpp>
-#include <hpx/execution_base/register_locks.hpp>
-#include <hpx/functional/unique_function.hpp>
+#include <hpx/lock_registration/detail/register_locks.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/threading_base/register_thread.hpp>
-#include <hpx/threading_base/scheduler_mode.hpp>
 #include <hpx/threading_base/thread_description.hpp>
 #include <hpx/threading_base/thread_pool_base.hpp>
 #include <hpx/timing/steady_clock.hpp>
+#if !defined(HPX_HAVE_THREAD_FULLBACKTRACE_ON_SUSPENSION)
+#include <hpx/debugging/backtrace.hpp>
+#endif
 
 #include <atomic>
 #include <chrono>
 #include <cstddef>
 #include <cstdint>
-#include <functional>
-#include <type_traits>
-#include <utility>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace threads {
+namespace hpx::threads {
+
     ///////////////////////////////////////////////////////////////////////////
     /// \brief  Set the thread state of the \a thread referenced by the
     ///         thread_id \a id.
@@ -40,7 +39,9 @@ namespace hpx { namespace threads {
     ///                   referenced by the \a id parameter.
     /// \param stateex    [in] The new extended state to be set for the
     ///                   thread referenced by the \a id parameter.
-    /// \param priority
+    /// \param priority   [in]
+    /// \param retry_on_active [in]
+    ///
     /// \param ec         [in,out] this represents the error status on exit,
     ///                   if this is pre-initialized to \a hpx#throws
     ///                   the function will throw on error instead.
@@ -86,7 +87,8 @@ namespace hpx { namespace threads {
     ///                   referenced by the \a id parameter.
     /// \param stateex    [in] The new extended state to be set for the
     ///                   thread referenced by the \a id parameter.
-    /// \param priority
+    /// \param priority        [in]
+    /// \param retry_on_active [in]
     /// \param ec         [in,out] this represents the error status on exit,
     ///                   if this is pre-initialized to \a hpx#throws
     ///                   the function will throw on error instead.
@@ -98,7 +100,8 @@ namespace hpx { namespace threads {
     ///                   throw but returns the result code using the
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
-    HPX_CORE_EXPORT thread_id_type set_thread_state(thread_id_type const& id,
+    HPX_CORE_EXPORT thread_id_ref_type set_thread_state(
+        thread_id_type const& id,
         hpx::chrono::steady_time_point const& abs_time,
         std::atomic<bool>* started,
         thread_schedule_state state = thread_schedule_state::pending,
@@ -106,7 +109,7 @@ namespace hpx { namespace threads {
         thread_priority priority = thread_priority::normal,
         bool retry_on_active = true, error_code& ec = throws);
 
-    inline thread_id_type set_thread_state(thread_id_type const& id,
+    inline thread_id_ref_type set_thread_state(thread_id_type const& id,
         hpx::chrono::steady_time_point const& abs_time,
         thread_schedule_state state = thread_schedule_state::pending,
         thread_restart_state stateex = thread_restart_state::timeout,
@@ -132,7 +135,8 @@ namespace hpx { namespace threads {
     ///                   referenced by the \a id parameter.
     /// \param stateex    [in] The new extended state to be set for the
     ///                   thread referenced by the \a id parameter.
-    /// \param priority
+    /// \param priority        [in]
+    /// \param retry_on_active [in]
     /// \param ec         [in,out] this represents the error status on exit,
     ///                   if this is pre-initialized to \a hpx#throws
     ///                   the function will throw on error instead.
@@ -144,7 +148,7 @@ namespace hpx { namespace threads {
     ///                   throw but returns the result code using the
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
-    inline thread_id_type set_thread_state(thread_id_type const& id,
+    inline thread_id_ref_type set_thread_state(thread_id_type const& id,
         hpx::chrono::steady_duration const& rel_time,
         thread_schedule_state state = thread_schedule_state::pending,
         thread_restart_state stateex = thread_restart_state::timeout,
@@ -211,7 +215,7 @@ namespace hpx { namespace threads {
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
     HPX_CORE_EXPORT thread_state get_thread_state(
-        thread_id_type const& id, error_code& ec = throws);
+        thread_id_type const& id, error_code& ec = throws) noexcept;
 
     ///////////////////////////////////////////////////////////////////////////
     /// The function get_thread_phase is part of the thread related API.
@@ -234,7 +238,7 @@ namespace hpx { namespace threads {
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
     HPX_CORE_EXPORT std::size_t get_thread_phase(
-        thread_id_type const& id, error_code& ec = throws);
+        thread_id_type const& id, error_code& ec = throws) noexcept;
 
     ///////////////////////////////////////////////////////////////////////////
     /// Returns whether the given thread can be interrupted at this point.
@@ -356,7 +360,7 @@ namespace hpx { namespace threads {
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
     HPX_CORE_EXPORT threads::thread_priority get_thread_priority(
-        thread_id_type const& id, error_code& ec = throws);
+        thread_id_type const& id, error_code& ec = throws) noexcept;
 
     ///////////////////////////////////////////////////////////////////////////
     /// Return stack size of the given thread
@@ -373,7 +377,7 @@ namespace hpx { namespace threads {
     ///                   parameter \a ec. Otherwise it throws an instance
     ///                   of hpx#exception.
     HPX_CORE_EXPORT std::ptrdiff_t get_stack_size(
-        thread_id_type const& id, error_code& ec = throws);
+        thread_id_type const& id, error_code& ec = throws) noexcept;
 
     ///////////////////////////////////////////////////////////////////////////
     /// \cond NOINTERNAL
@@ -381,7 +385,7 @@ namespace hpx { namespace threads {
         thread_id_type const& id, error_code& ec = throws);
 
     HPX_CORE_EXPORT bool add_thread_exit_callback(thread_id_type const& id,
-        util::function_nonser<void()> const& f, error_code& ec = throws);
+        hpx::function<void()> const& f, error_code& ec = throws);
 
     HPX_CORE_EXPORT void free_thread_exit_callbacks(
         thread_id_type const& id, error_code& ec = throws);
@@ -413,8 +417,8 @@ namespace hpx { namespace threads {
         thread_id_type const& id, std::size_t data, error_code& ec = throws);
 #endif
 
-    HPX_CORE_EXPORT std::size_t& get_continuation_recursion_count();
-    HPX_CORE_EXPORT void reset_continuation_recursion_count();
+    HPX_CORE_EXPORT std::size_t& get_continuation_recursion_count() noexcept;
+    HPX_CORE_EXPORT void reset_continuation_recursion_count() noexcept;
     /// \endcond
 
     /// Returns a pointer to the pool that was used to run the current thread
@@ -422,69 +426,76 @@ namespace hpx { namespace threads {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     HPX_CORE_EXPORT threads::thread_pool_base* get_pool(
         thread_id_type const& id, error_code& ec = throws);
-}}    // namespace hpx::threads
+}    // namespace hpx::threads
+
+namespace hpx::this_thread {
 
-namespace hpx { namespace this_thread {
     ///////////////////////////////////////////////////////////////////////////
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to the thread state passed as the parameter.
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// the thread state passed as the parameter.
     ///
     /// \note Must be called from within a HPX-thread.
     ///
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     HPX_CORE_EXPORT threads::thread_restart_state suspend(
-        threads::thread_schedule_state state, threads::thread_id_type const& id,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_schedule_state state, threads::thread_id_type nextid,
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws);
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to the thread state passed as the parameter.
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// the thread state passed as the parameter.
     ///
     /// \note Must be called from within a HPX-thread.
     ///
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     inline threads::thread_restart_state suspend(
         threads::thread_schedule_state state =
             threads::thread_schedule_state::pending,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws)
     {
         return suspend(state, threads::invalid_thread_id, description, ec);
     }
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to \a suspended and schedules a wakeup for this threads at the given
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// \a suspended and schedules a wakeup for this threads at the given
     /// time.
     ///
     /// \note Must be called from within a HPX-thread.
@@ -492,23 +503,25 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     HPX_CORE_EXPORT threads::thread_restart_state suspend(
         hpx::chrono::steady_time_point const& abs_time,
-        threads::thread_id_type const& id,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_id_type id,
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws);
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to \a suspended and schedules a wakeup for this threads at the given
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// \a suspended and schedules a wakeup for this threads at the given
     /// time.
     ///
     /// \note Must be called from within a HPX-thread.
@@ -516,25 +529,27 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     inline threads::thread_restart_state suspend(
         hpx::chrono::steady_time_point const& abs_time,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws)
     {
         return suspend(abs_time, threads::invalid_thread_id, description, ec);
     }
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to \a suspended and schedules a wakeup for this threads after the given
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// \a suspended and schedules a wakeup for this threads after the given
     /// duration.
     ///
     /// \note Must be called from within a HPX-thread.
@@ -542,17 +557,19 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     inline threads::thread_restart_state suspend(
         hpx::chrono::steady_duration const& rel_time,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws)
     {
         return suspend(
@@ -560,8 +577,8 @@ namespace hpx { namespace this_thread {
     }
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to \a suspended and schedules a wakeup for this threads after the given
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// \a suspended and schedules a wakeup for this threads after the given
     /// duration.
     ///
     /// \note Must be called from within a HPX-thread.
@@ -569,26 +586,28 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     inline threads::thread_restart_state suspend(
         hpx::chrono::steady_duration const& rel_time,
         threads::thread_id_type const& id,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws)
     {
         return suspend(rel_time.from_now(), id, description, ec);
     }
 
     /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to \a suspended and schedules a wakeup for this threads after the given
+    /// (suspends the current thread). It sets the new state of this thread to
+    /// \a suspended and schedules a wakeup for this threads after the given
     /// time (specified in milliseconds).
     ///
     /// \note Must be called from within a HPX-thread.
@@ -596,16 +615,18 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     ///
     inline threads::thread_restart_state suspend(std::uint64_t ms,
-        util::thread_description const& description = util::thread_description(
-            "this_thread::suspend"),
+        threads::thread_description const& description =
+            threads::thread_description("this_thread::suspend"),
         error_code& ec = throws)
     {
         return suspend(std::chrono::milliseconds(ms),
@@ -617,22 +638,24 @@ namespace hpx { namespace this_thread {
     /// \throws If <code>&ec != &throws</code>, never throws, but will set \a ec
     ///         to an appropriate value when an error occurs. Otherwise, this
     ///         function will throw an \a hpx#exception with an error code of
-    ///         \a hpx#yield_aborted if it is signaled with \a wait_aborted.
-    ///         If called outside of a HPX-thread, this function will throw
-    ///         an \a hpx#exception with an error code of \a hpx::null_thread_id.
+    ///         \a hpx#error#yield_aborted if it is signaled with \a
+    ///            wait_aborted.
+    ///         If called outside of a HPX-thread, this function will throw an
+    ///         \a hpx#exception with an error code of \a
+    ///            hpx#error#null_thread_id.
     ///         If this function is called while the thread-manager is not
     ///         running, it will throw an \a hpx#exception with an error code of
-    ///         \a hpx#invalid_status.
+    ///         \a hpx#error#invalid_status.
     HPX_CORE_EXPORT threads::thread_pool_base* get_pool(
         error_code& ec = throws);
 
     /// \cond NOINTERNAL
     // returns the remaining available stack space
-    HPX_CORE_EXPORT std::ptrdiff_t get_available_stack_space();
+    HPX_CORE_EXPORT std::ptrdiff_t get_available_stack_space() noexcept;
 
     // returns whether the remaining stack-space is at least as large as
     // requested
     HPX_CORE_EXPORT bool has_sufficient_stack_space(
-        std::size_t space_needed = 8 * HPX_THREADS_STACK_OVERHEAD);
+        std::size_t space_needed = 8 * HPX_THREADS_STACK_OVERHEAD) noexcept;
     /// \endcond
-}}    // namespace hpx::this_thread
+}    // namespace hpx::this_thread
