@@ -1,5 +1,5 @@
 //  Copyright (c)      2018 Mikael Simberg
-//  Copyright (c) 2007-2017 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -9,10 +9,10 @@
 
 #include <hpx/config.hpp>
 #include <hpx/affinity/affinity_data.hpp>
-#include <hpx/concurrency/barrier.hpp>
 #include <hpx/functional/function.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/threading_base/callback_notifier.hpp>
+#include <hpx/threading_base/detail/get_default_pool.hpp>
 #include <hpx/threading_base/network_background_callback.hpp>
 #include <hpx/threading_base/scheduler_mode.hpp>
 #include <hpx/threading_base/scheduler_state.hpp>
@@ -24,85 +24,32 @@
 #include <cstddef>
 #include <cstdint>
 #include <exception>
-#include <functional>
 #include <iosfwd>
 #include <memory>
 #include <mutex>
 #include <string>
 #include <thread>
-#include <vector>
 
 #include <hpx/config/warnings_prefix.hpp>
 
-namespace hpx { namespace threads {
-    /// \brief Data structure which stores statistics collected by an
-    ///        executor instance.
-    struct executor_statistics
-    {
-        executor_statistics()
-          : tasks_scheduled_(0)
-          , tasks_completed_(0)
-          , queue_length_(0)
-        {
-        }
-
-        std::uint64_t tasks_scheduled_;
-        std::uint64_t tasks_completed_;
-        std::uint64_t queue_length_;
-    };
-
-    namespace detail {
-        ///////////////////////////////////////////////////////////////////////
-        enum executor_parameter
-        {
-            min_concurrency = 1,
-            max_concurrency = 2,
-            current_concurrency = 3
-        };
-
-        ///////////////////////////////////////////////////////////////////////
-        // The interface below is used by the resource manager to
-        // interact with the executor.
-        struct HPX_CORE_EXPORT manage_executor
-        {
-            virtual ~manage_executor() {}
-
-            // Return the requested policy element
-            virtual std::size_t get_policy_element(
-                executor_parameter p, error_code& ec) const = 0;
-
-            // Return statistics collected by this scheduler
-            virtual void get_statistics(
-                executor_statistics& stats, error_code& ec) const = 0;
-
-            // Provide the given processing unit to the scheduler.
-            virtual void add_processing_unit(std::size_t virt_core,
-                std::size_t thread_num, error_code& ec) = 0;
-
-            // Remove the given processing unit from the scheduler.
-            virtual void remove_processing_unit(
-                std::size_t thread_num, error_code& ec) = 0;
-
-            // return the description string of the underlying scheduler
-            virtual char const* get_description() const = 0;
-        };
-    }    // namespace detail
+namespace hpx::threads {
 
     ///////////////////////////////////////////////////////////////////////////
     /// \cond NOINTERNAL
     struct pool_id_type
     {
-        pool_id_type(std::size_t index, std::string const& name)
+        pool_id_type(std::size_t index, std::string name) noexcept
           : index_(index)
-          , name_(name)
+          , name_(HPX_MOVE(name))
         {
         }
 
-        std::size_t index() const
+        std::size_t index() const noexcept
         {
             return index_;
-        };
-        std::string const& name() const
+        }
+
+        std::string const& name() const noexcept
         {
             return name_;
         }
@@ -137,7 +84,7 @@ namespace hpx { namespace threads {
             hpx::threads::detail::network_background_callback_type const&
                 network_background_callback =
                     hpx::threads::detail::network_background_callback_type(),
-            std::size_t max_background_threads = std::size_t(-1),
+            std::size_t max_background_threads = static_cast<std::size_t>(-1),
             std::size_t max_idle_loop_count = HPX_IDLE_LOOP_COUNT_MAX,
             std::size_t max_busy_loop_count = HPX_BUSY_LOOP_COUNT_MAX,
             std::size_t shutdown_check_count = 10)
@@ -161,11 +108,16 @@ namespace hpx { namespace threads {
     // note: this data structure has to be protected from races from the outside
 
     /// \brief The base class used to manage a pool of OS threads.
-    class HPX_CORE_EXPORT thread_pool_base
+    class thread_pool_base
     {
     public:
         /// \cond NOINTERNAL
-        thread_pool_base(thread_pool_init_parameters const& init);
+        explicit thread_pool_base(thread_pool_init_parameters const& init);
+
+        thread_pool_base(thread_pool_base const&) = delete;
+        thread_pool_base(thread_pool_base&&) = delete;
+        thread_pool_base& operator=(thread_pool_base const&) = delete;
+        thread_pool_base& operator=(thread_pool_base&&) = delete;
 
         virtual ~thread_pool_base() = default;
 
@@ -181,7 +133,7 @@ namespace hpx { namespace threads {
         virtual bool is_busy() = 0;
         virtual bool is_idle() = 0;
 
-        virtual void print_pool(std::ostream&) = 0;
+        virtual void print_pool(std::ostream&) const = 0;
 
         pool_id_type get_pool_id() const
         {
@@ -192,17 +144,23 @@ namespace hpx { namespace threads {
         /// Suspends the given processing unit. Blocks until the processing unit
         /// has been suspended.
         ///
-        /// \param virt_core [in] The processing unit on the the pool to be
-        ///                  suspended. The processing units are indexed
-        ///                  starting from 0.
+        /// \param virt_core   [in] The processing unit on the the pool to be
+        ///                    suspended. The processing units are indexed
+        ///                    starting from 0.
+        /// \param ec [in,out] this represents the error status on exit, if this
+        ///           is pre-initialized to \a hpx#throws the function will
+        ///           throw on error instead.
         virtual void suspend_processing_unit_direct(
             std::size_t virt_core, error_code& ec = throws) = 0;
 
         /// Resumes the given processing unit. Blocks until the processing unit
         /// has been resumed.
         ///
-        /// \param virt_core [in] The processing unit on the the pool to be resumed.
-        ///                  The processing units are indexed starting from 0.
+        /// \param virt_core   [in] The processing unit on the the pool to be resumed.
+        ///                    The processing units are indexed starting from 0.
+        /// \param ec [in,out] this represents the error status on exit, if this
+        ///           is pre-initialized to \a hpx#throws the function will
+        ///           throw on error instead.
         virtual void resume_processing_unit_direct(
             std::size_t virt_core, error_code& ec = throws) = 0;
 
@@ -237,28 +195,31 @@ namespace hpx { namespace threads {
         virtual std::size_t get_active_os_thread_count() const;
 
         virtual void create_thread(
-            thread_init_data& data, thread_id_type& id, error_code& ec) = 0;
-        virtual void create_work(thread_init_data& data, error_code& ec) = 0;
+            thread_init_data& data, thread_id_ref_type& id, error_code& ec) = 0;
+        virtual thread_id_ref_type create_work(
+            thread_init_data& data, error_code& ec) = 0;
 
         virtual thread_state set_state(thread_id_type const& id,
             thread_schedule_state new_state, thread_restart_state new_state_ex,
             thread_priority priority, error_code& ec) = 0;
 
-        virtual thread_id_type set_state(
+        virtual thread_id_ref_type set_state(
             hpx::chrono::steady_time_point const& abs_time,
             thread_id_type const& id, thread_schedule_state newstate,
             thread_restart_state newstate_ex, thread_priority priority,
             error_code& ec) = 0;
 
-        std::size_t get_pool_index() const
+        [[nodiscard]] std::size_t get_pool_index() const noexcept
         {
             return id_.index();
         }
-        std::string const& get_pool_name() const
+
+        [[nodiscard]] std::string const& get_pool_name() const noexcept
         {
             return id_.name();
         }
-        std::size_t get_thread_offset() const
+
+        [[nodiscard]] std::size_t get_thread_offset() const noexcept
         {
             return thread_offset_;
         }
@@ -268,7 +229,12 @@ namespace hpx { namespace threads {
             return nullptr;
         }
 
-        mask_type get_used_processing_units() const;
+        mask_type get_used_processing_units(bool full_cores = false) const;
+        mask_type get_used_processing_units(
+            std::size_t num_cores, bool full_cores = false) const;
+        mask_type get_used_processing_unit(
+            std::size_t thread_num, bool full_cores = false) const;
+
         hwloc_bitmap_ptr get_numa_domain_bitmap() const;
 
         // performance counters
@@ -316,7 +282,6 @@ namespace hpx { namespace threads {
         }
 #endif
 #endif
-
         virtual std::int64_t get_cumulative_duration(
             std::size_t /*thread_num*/, bool /*reset*/)
         {
@@ -360,29 +325,28 @@ namespace hpx { namespace threads {
 #endif    // HPX_HAVE_BACKGROUND_THREAD_COUNTERS
 
 #if defined(HPX_HAVE_THREAD_IDLE_RATES)
-        virtual std::int64_t avg_idle_rate_all(bool /*reset*/)
+        virtual std::int64_t avg_idle_rate_all(bool /*reset*/) noexcept
         {
             return 0;
         }
-        virtual std::int64_t avg_idle_rate(std::size_t, bool)
+        virtual std::int64_t avg_idle_rate(std::size_t, bool) noexcept
         {
             return 0;
         }
 
 #if defined(HPX_HAVE_THREAD_CREATION_AND_CLEANUP_RATES)
         virtual std::int64_t avg_creation_idle_rate(
-            std::size_t /*thread_num*/, bool /*reset*/)
+            std::size_t /*thread_num*/, bool /*reset*/) noexcept
         {
             return 0;
         }
         virtual std::int64_t avg_cleanup_idle_rate(
-            std::size_t /*thread_num*/, bool /*reset*/)
+            std::size_t /*thread_num*/, bool /*reset*/) noexcept
         {
             return 0;
         }
 #endif
 #endif
-
         virtual std::int64_t get_queue_length(std::size_t, bool)
         {
             return 0;
@@ -434,7 +398,6 @@ namespace hpx { namespace threads {
             return 0;
         }
 #endif
-
         virtual std::int64_t get_thread_count(thread_schedule_state /*state*/,
             thread_priority /*priority*/, std::size_t /*num_thread*/,
             bool /*reset*/)
@@ -449,45 +412,23 @@ namespace hpx { namespace threads {
 
         virtual void get_idle_core_mask(mask_type&) const {}
 
-        virtual std::int64_t get_background_thread_count()
+        virtual std::int64_t get_background_thread_count() const
         {
             return 0;
         }
 
         std::int64_t get_thread_count_unknown(
-            std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::unknown,
-                thread_priority::default_, num_thread, reset);
-        }
-        std::int64_t get_thread_count_active(std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::active,
-                thread_priority::default_, num_thread, reset);
-        }
+            std::size_t num_thread, bool reset);
+        std::int64_t get_thread_count_active(
+            std::size_t num_thread, bool reset);
         std::int64_t get_thread_count_pending(
-            std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::pending,
-                thread_priority::default_, num_thread, reset);
-        }
+            std::size_t num_thread, bool reset);
         std::int64_t get_thread_count_suspended(
-            std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::suspended,
-                thread_priority::default_, num_thread, reset);
-        }
+            std::size_t num_thread, bool reset);
         std::int64_t get_thread_count_terminated(
-            std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::terminated,
-                thread_priority::default_, num_thread, reset);
-        }
-        std::int64_t get_thread_count_staged(std::size_t num_thread, bool reset)
-        {
-            return get_thread_count(thread_schedule_state::staged,
-                thread_priority::default_, num_thread, reset);
-        }
+            std::size_t num_thread, bool reset);
+        std::int64_t get_thread_count_staged(
+            std::size_t num_thread, bool reset);
 
         virtual std::int64_t get_scheduler_utilization() const = 0;
 
@@ -498,14 +439,14 @@ namespace hpx { namespace threads {
 
         ///////////////////////////////////////////////////////////////////////
         virtual bool enumerate_threads(
-            util::function_nonser<bool(thread_id_type)> const& /*f*/,
+            hpx::function<bool(thread_id_type)> const& /*f*/,
             thread_schedule_state /*state*/ =
                 thread_schedule_state::unknown) const
         {
             return false;
         }
 
-        virtual void reset_thread_distribution() {}
+        virtual void reset_thread_distribution() noexcept {}
 
         virtual void abort_all_suspended_threads() {}
         virtual bool cleanup_terminated(bool /*delete_all*/)
@@ -520,12 +461,16 @@ namespace hpx { namespace threads {
 
         virtual void do_some_work(std::size_t /*num_thread*/) {}
 
-        virtual void report_error(
+        virtual bool report_error(
             std::size_t global_thread_num, std::exception_ptr const& e)
         {
-            notifier_.on_error(global_thread_num, e);
+            return notifier_.on_error(global_thread_num, e);
         }
 
+        [[nodiscard]] double timestamp_scale() const noexcept
+        {
+            return timestamp_scale_;
+        }
         /// \endcond
 
     protected:
@@ -556,6 +501,6 @@ namespace hpx { namespace threads {
 
     HPX_CORE_EXPORT std::ostream& operator<<(
         std::ostream& os, thread_pool_base const& thread_pool);
-}}    // namespace hpx::threads
+}    // namespace hpx::threads
 
 #include <hpx/config/warnings_suffix.hpp>
