@@ -1,4 +1,4 @@
-//  Copyright (c) 2019 Hartmut Kaiser
+//  Copyright (c) 2019-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -10,16 +10,20 @@
 #include <hpx/threading_base/thread_data.hpp>
 
 ////////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace threads {
+namespace hpx::threads {
 
     util::internal_allocator<thread_data_stackful>
         thread_data_stackful::thread_alloc_;
 
+#if !defined(HPX_HAVE_LOGGING)
+    thread_data_stackful::~thread_data_stackful() = default;
+#else
     thread_data_stackful::~thread_data_stackful()
     {
         LTM_(debug).format(
             "~thread_data_stackful({}), description({}), phase({})", this,
-            this->get_description(), this->get_thread_phase());
+            this->get_description(),
+            this->thread_data_stackful::get_thread_phase());
     }
-
-}}    // namespace hpx::threads
+#endif
+}    // namespace hpx::threads
