@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2025 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -9,20 +9,22 @@
 
 #include <hpx/assert.hpp>
 #include <hpx/coroutines/thread_enums.hpp>
-#include <hpx/modules/errors.hpp>
-#ifdef HPX_HAVE_VERIFY_LOCKS
-#include <hpx/execution_base/register_locks.hpp>
-#endif
 #include <hpx/execution_base/this_thread.hpp>
-#include <hpx/threading_base/detail/reset_lco_description.hpp>
+#include <hpx/modules/errors.hpp>
 #include <hpx/threading_base/scheduler_base.hpp>
 #include <hpx/threading_base/scheduler_state.hpp>
 #include <hpx/threading_base/set_thread_state.hpp>
+#include <hpx/threading_base/set_thread_state_timed.hpp>
 #include <hpx/threading_base/thread_description.hpp>
 #include <hpx/threading_base/thread_pool_base.hpp>
 #include <hpx/timing/steady_clock.hpp>
-#include <hpx/type_support/unused.hpp>
 
+#ifdef HPX_HAVE_VERIFY_LOCKS
+#include <hpx/lock_registration/detail/register_locks.hpp>
+#endif
+#ifdef HPX_HAVE_THREAD_DESCRIPTION
+#include <hpx/threading_base/detail/reset_lco_description.hpp>
+#endif
 #ifdef HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION
 #include <hpx/debugging/backtrace.hpp>
 #include <hpx/threading_base/detail/reset_backtrace.hpp>
@@ -32,11 +34,11 @@
 #include <cstddef>
 #include <limits>
 #include <memory>
-#include <string>
 #include <utility>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace threads {
+namespace hpx::threads {
+
     ///////////////////////////////////////////////////////////////////////////
     thread_state set_thread_state(thread_id_type const& id,
         thread_schedule_state state, thread_restart_state stateex,
@@ -50,21 +52,21 @@ namespace hpx { namespace threads {
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    thread_id_type set_thread_state(thread_id_type const& id,
+    thread_id_ref_type set_thread_state(thread_id_type const& id,
         hpx::chrono::steady_time_point const& abs_time,
         std::atomic<bool>* timer_started, thread_schedule_state state,
         thread_restart_state stateex, thread_priority priority,
         bool retry_on_active, error_code& ec)
     {
         return detail::set_thread_state_timed(
-            *(get_thread_id_data(id)->get_scheduler_base()), abs_time, id,
-            state, stateex, priority, thread_schedule_hint(), timer_started,
+            get_thread_id_data(id)->get_scheduler_base(), abs_time, id, state,
+            stateex, priority, thread_schedule_hint(), timer_started,
             retry_on_active, ec);
     }
 
     ///////////////////////////////////////////////////////////////////////////
     thread_state get_thread_state(
-        thread_id_type const& id, error_code& /* ec */)
+        thread_id_type const& id, error_code& /* ec */) noexcept
     {
         return id ? get_thread_id_data(id)->get_state() :
                     thread_state(thread_schedule_state::terminated,
@@ -72,22 +74,23 @@ namespace hpx { namespace threads {
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    std::size_t get_thread_phase(thread_id_type const& id, error_code& /* ec */)
+    std::size_t get_thread_phase(
+        thread_id_type const& id, error_code& /* ec */) noexcept
     {
         return id ? get_thread_id_data(id)->get_thread_phase() :
-                    std::size_t(~0);
+                    static_cast<std::size_t>(~0);
     }
 
     ///////////////////////////////////////////////////////////////////////////
     threads::thread_priority get_thread_priority(
-        thread_id_type const& id, error_code& /* ec */)
+        thread_id_type const& id, error_code& /* ec */) noexcept
     {
         return id ? get_thread_id_data(id)->get_priority() :
                     thread_priority::unknown;
     }
 
     std::ptrdiff_t get_stack_size(
-        thread_id_type const& id, error_code& /* ec */)
+        thread_id_type const& id, error_code& /* ec */) noexcept
     {
         return id ? get_thread_id_data(id)->get_stack_size() :
                     static_cast<std::ptrdiff_t>(thread_stacksize::unknown);
@@ -97,8 +100,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::interrupt_thread",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::interrupt_thread", "null thread id encountered");
             return;
         }
 
@@ -117,7 +120,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::interruption_point",
                 "null thread id encountered");
             return;
@@ -135,10 +138,9 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROW_EXCEPTION(null_thread_id,
+            HPX_THROW_EXCEPTION(hpx::error::null_thread_id,
                 "hpx::threads::get_thread_interruption_enabled",
                 "null thread id encountered");
-            return false;
         }
 
         if (&ec != &throws)
@@ -152,10 +154,9 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROW_EXCEPTION(null_thread_id,
+            HPX_THROW_EXCEPTION(hpx::error::null_thread_id,
                 "hpx::threads::get_thread_interruption_enabled",
                 "null thread id encountered");
-            return false;
         }
 
         if (&ec != &throws)
@@ -169,7 +170,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::get_thread_interruption_requested",
                 "null thread id encountered");
             return false;
@@ -186,8 +187,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::get_thread_data",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::get_thread_data", "null thread id encountered");
             return 0;
         }
 
@@ -199,8 +200,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::set_thread_data",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::set_thread_data", "null thread id encountered");
             return 0;
         }
 
@@ -212,8 +213,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::get_libcds_data",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::get_libcds_data", "null thread id encountered");
             return 0;
         }
 
@@ -225,8 +226,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::set_libcds_data",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::set_libcds_data", "null thread id encountered");
             return 0;
         }
 
@@ -238,7 +239,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::get_libcds_hazard_pointer_data",
                 "null thread id encountered");
             return 0;
@@ -252,7 +253,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::set_libcds_hazard_pointer_data",
                 "null thread id encountered");
             return 0;
@@ -266,7 +267,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::get_libcds_dynamic_hazard_pointer_data",
                 "null thread id encountered");
             return 0;
@@ -280,7 +281,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::set_libcds_dynamic_hazard_pointer_data",
                 "null thread id encountered");
             return 0;
@@ -293,18 +294,21 @@ namespace hpx { namespace threads {
 #endif
 
     ////////////////////////////////////////////////////////////////////////////
-    static thread_local std::size_t continuation_recursion_count(0);
+    namespace {
 
-    std::size_t& get_continuation_recursion_count()
+        thread_local std::size_t continuation_recursion_count(0);
+    }
+
+    std::size_t& get_continuation_recursion_count() noexcept
     {
-        thread_self* self_ptr = get_self_ptr();
-        if (self_ptr)
+        if (thread_self* self_ptr = get_self_ptr())
+        {
             return self_ptr->get_continuation_recursion_count();
-
+        }
         return continuation_recursion_count;
     }
 
-    void reset_continuation_recursion_count()
+    void reset_continuation_recursion_count() noexcept
     {
         continuation_recursion_count = 0;
     }
@@ -314,7 +318,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::run_thread_exit_callbacks",
                 "null thread id encountered");
             return;
@@ -327,11 +331,11 @@ namespace hpx { namespace threads {
     }
 
     bool add_thread_exit_callback(thread_id_type const& id,
-        util::function_nonser<void()> const& f, error_code& ec)
+        hpx::function<void()> const& f, error_code& ec)
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::add_thread_exit_callback",
                 "null thread id encountered");
             return false;
@@ -347,7 +351,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::add_thread_exit_callback",
                 "null thread id encountered");
             return;
@@ -369,7 +373,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::get_thread_backtrace",
                 "null thread id encountered");
             return nullptr;
@@ -391,7 +395,7 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id,
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
                 "hpx::threads::set_thread_backtrace",
                 "null thread id encountered");
             return nullptr;
@@ -408,8 +412,8 @@ namespace hpx { namespace threads {
     {
         if (HPX_UNLIKELY(!id))
         {
-            HPX_THROWS_IF(ec, null_thread_id, "hpx::threads::get_pool",
-                "null thread id encountered");
+            HPX_THROWS_IF(ec, hpx::error::null_thread_id,
+                "hpx::threads::get_pool", "null thread id encountered");
             return nullptr;
         }
 
@@ -418,75 +422,84 @@ namespace hpx { namespace threads {
 
         return get_thread_id_data(id)->get_scheduler_base()->get_parent_pool();
     }
-}}    // namespace hpx::threads
+}    // namespace hpx::threads
 
-namespace hpx { namespace this_thread {
+namespace hpx::this_thread {
 
-    /// The function \a suspend will return control to the thread manager
-    /// (suspends the current thread). It sets the new state of this thread
-    /// to the thread state passed as the parameter.
-    ///
-    /// If the suspension was aborted, this function will throw a
-    /// \a yield_aborted exception.
+    // The function 'suspend' will return control to the thread manager
+    // (suspends the current thread). It sets the new state of this thread to
+    // the thread state passed as the parameter.
+    //
+    // If the suspension was aborted, this function will throw a
+    // \a yield_aborted exception.
     threads::thread_restart_state suspend(threads::thread_schedule_state state,
-        threads::thread_id_type const& nextid,
-        util::thread_description const& description, error_code& ec)
+        threads::thread_id_type nextid,
+        [[maybe_unused]] threads::thread_description const& description,
+        error_code& ec)
     {
         // let the thread manager do other things while waiting
         threads::thread_self& self = threads::get_self();
-        threads::thread_id_type id = self.get_thread_id();
+
+        // keep alive
+        threads::thread_id_ref_type id = self.get_outer_thread_id();
 
         // handle interruption, if needed
-        threads::interruption_point(id, ec);
+        threads::interruption_point(id.noref(), ec);
         if (ec)
             return threads::thread_restart_state::unknown;
 
-        threads::thread_restart_state statex =
-            threads::thread_restart_state::unknown;
+        threads::thread_restart_state statex;
 
         {
-            // verify that there are no more registered locks for this OS-thread
 #ifdef HPX_HAVE_VERIFY_LOCKS
+            // verify that there are no more registered locks for this OS-thread
             util::verify_no_locks();
+
+            [[maybe_unused]] auto held_locks = hpx::experimental::scope_exit(
+                [data = hpx::util::get_held_locks_data()]() mutable {
+                    hpx::util::set_held_locks_data(HPX_MOVE(data));
+                });
 #endif
 #ifdef HPX_HAVE_THREAD_DESCRIPTION
-            threads::detail::reset_lco_description desc(id, description, ec);
-#else
-            HPX_UNUSED(description);
+            threads::detail::reset_lco_description desc(
+                id.noref(), description, ec);
 #endif
 #ifdef HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION
             threads::detail::reset_backtrace bt(id, ec);
 #endif
-            // We might need to dispatch 'nextid' to it's correct scheduler
-            // only if our current scheduler is the same, we should yield the id
+
+            // We might need to dispatch 'nextid' to it's correct scheduler only
+            // if our current scheduler is the same, we should yield to the id
             if (nextid &&
                 get_thread_id_data(nextid)->get_scheduler_base() !=
                     get_thread_id_data(id)->get_scheduler_base())
             {
-                get_thread_id_data(nextid)
-                    ->get_scheduler_base()
-                    ->schedule_thread(get_thread_id_data(nextid),
-                        threads::thread_schedule_hint());
+                auto* scheduler =
+                    get_thread_id_data(nextid)->get_scheduler_base();
+                scheduler->schedule_thread(
+                    HPX_MOVE(nextid), threads::thread_schedule_hint());
+
                 statex = self.yield(threads::thread_result_type(
                     state, threads::invalid_thread_id));
             }
             else
             {
-                statex = self.yield(threads::thread_result_type(state, nextid));
+                statex = self.yield(
+                    threads::thread_result_type(state, HPX_MOVE(nextid)));
             }
         }
 
         // handle interruption, if needed
-        threads::interruption_point(id, ec);
+        threads::interruption_point(id.noref(), ec);
         if (ec)
             return threads::thread_restart_state::unknown;
 
         // handle interrupt and abort
         if (statex == threads::thread_restart_state::abort)
         {
-            HPX_THROWS_IF(ec, yield_aborted, "suspend",
+            HPX_THROWS_IF(ec, hpx::error::yield_aborted, "suspend",
                 "thread({}, {}) aborted (yield returned wait_abort)",
-                threads::get_self_id(), threads::get_thread_description(id));
+                id.noref(), threads::get_thread_description(id.noref()));
         }
 
         if (&ec != &throws)
@@ -497,54 +510,62 @@ namespace hpx { namespace this_thread {
 
     threads::thread_restart_state suspend(
         hpx::chrono::steady_time_point const& abs_time,
-        threads::thread_id_type const& nextid,
-        util::thread_description const& description, error_code& ec)
+        threads::thread_id_type nextid,
+        [[maybe_unused]] threads::thread_description const& description,
+        error_code& ec)
     {
         // schedule a thread waking us up at_time
         threads::thread_self& self = threads::get_self();
-        threads::thread_id_type id = self.get_thread_id();
+
+        // keep alive
+        threads::thread_id_ref_type id = self.get_outer_thread_id();
 
         // handle interruption, if needed
-        threads::interruption_point(id, ec);
+        threads::interruption_point(id.noref(), ec);
         if (ec)
             return threads::thread_restart_state::unknown;
 
         // let the thread manager do other things while waiting
-        threads::thread_restart_state statex =
-            threads::thread_restart_state::unknown;
+        threads::thread_restart_state statex;
 
         {
 #ifdef HPX_HAVE_VERIFY_LOCKS
             // verify that there are no more registered locks for this OS-thread
             util::verify_no_locks();
+
+            [[maybe_unused]] auto held_locks = hpx::experimental::scope_exit(
+                [data = hpx::util::get_held_locks_data()]() mutable {
+                    hpx::util::set_held_locks_data(HPX_MOVE(data));
+                });
 #endif
 #ifdef HPX_HAVE_THREAD_DESCRIPTION
-            threads::detail::reset_lco_description desc(id, description, ec);
-#else
-            HPX_UNUSED(description);
+            threads::detail::reset_lco_description desc(
+                id.noref(), description, ec);
 #endif
 #ifdef HPX_HAVE_THREAD_BACKTRACE_ON_SUSPENSION
             threads::detail::reset_backtrace bt(id, ec);
 #endif
+
             std::atomic<bool> timer_started(false);
-            threads::thread_id_type timer_id =
-                threads::set_thread_state(id, abs_time, &timer_started,
+            threads::thread_id_ref_type const timer_id =
+                threads::set_thread_state(id.noref(), abs_time, &timer_started,
                     threads::thread_schedule_state::pending,
                     threads::thread_restart_state::timeout,
                     threads::thread_priority::boost, true, ec);
             if (ec)
                 return threads::thread_restart_state::unknown;
 
-            // We might need to dispatch 'nextid' to it's correct scheduler
-            // only if our current scheduler is the same, we should yield the id
+            // We might need to dispatch 'nextid' to it's correct scheduler only
+            // if our current scheduler is the same, we should yield to the id
             if (nextid &&
                 get_thread_id_data(nextid)->get_scheduler_base() !=
                     get_thread_id_data(id)->get_scheduler_base())
             {
-                get_thread_id_data(nextid)
-                    ->get_scheduler_base()
-                    ->schedule_thread(get_thread_id_data(nextid),
-                        threads::thread_schedule_hint());
+                auto* scheduler =
+                    get_thread_id_data(nextid)->get_scheduler_base();
+                scheduler->schedule_thread(
+                    HPX_MOVE(nextid), threads::thread_schedule_hint());
+
                 statex = self.yield(threads::thread_result_type(
                     threads::thread_schedule_state::suspended,
                     threads::invalid_thread_id));
@@ -552,18 +573,20 @@ namespace hpx { namespace this_thread {
             else
             {
                 statex = self.yield(threads::thread_result_type(
-                    threads::thread_schedule_state::suspended, nextid));
+                    threads::thread_schedule_state::suspended,
+                    HPX_MOVE(nextid)));
             }
 
             if (statex != threads::thread_restart_state::timeout)
             {
                 HPX_ASSERT(statex == threads::thread_restart_state::abort ||
                     statex == threads::thread_restart_state::signaled);
-                error_code ec1(lightweight);    // do not throw
-                hpx::util::yield_while(
+
+                error_code ec1(throwmode::lightweight);    // do not throw
+                hpx::util::yield_while<true>(
                     [&timer_started]() { return !timer_started.load(); },
                     "set_thread_state_timed");
-                threads::set_thread_state(timer_id,
+                threads::set_thread_state(timer_id.noref(),
                     threads::thread_schedule_state::pending,
                     threads::thread_restart_state::abort,
                     threads::thread_priority::boost, true, ec1);
@@ -571,16 +594,16 @@ namespace hpx { namespace this_thread {
         }
 
         // handle interruption, if needed
-        threads::interruption_point(id, ec);
+        threads::interruption_point(id.noref(), ec);
         if (ec)
             return threads::thread_restart_state::unknown;
 
         // handle interrupt and abort
         if (statex == threads::thread_restart_state::abort)
         {
-            HPX_THROWS_IF(ec, yield_aborted, "suspend_at",
+            HPX_THROWS_IF(ec, hpx::error::yield_aborted, "suspend_at",
                 "thread({}, {}) aborted (yield returned wait_abort)",
-                threads::get_self_id(), threads::get_thread_description(id));
+                id.noref(), threads::get_thread_description(id.noref()));
         }
 
         if (&ec != &throws)
@@ -595,35 +618,33 @@ namespace hpx { namespace this_thread {
         return threads::get_pool(threads::get_self_id(), ec);
     }
 
-    std::ptrdiff_t get_available_stack_space()
+    std::ptrdiff_t get_available_stack_space() noexcept
     {
-        threads::thread_self* self = threads::get_self_ptr();
-        if (self)
+        if (threads::thread_self const* self = threads::get_self_ptr())
         {
             return self->get_available_stack_space();
         }
-
         return (std::numeric_limits<std::ptrdiff_t>::max)();
     }
 
-    bool has_sufficient_stack_space(std::size_t space_needed)
+    bool has_sufficient_stack_space(
+        [[maybe_unused]] std::size_t space_needed) noexcept
     {
         if (nullptr == hpx::threads::get_self_ptr())
             return false;
 
 #if defined(HPX_HAVE_THREADS_GET_STACK_POINTER)
-        std::ptrdiff_t remaining_stack = get_available_stack_space();
+        std::ptrdiff_t const remaining_stack = get_available_stack_space();
         if (remaining_stack < 0)
         {
-            HPX_THROW_EXCEPTION(
-                out_of_memory, "has_sufficient_stack_space", "Stack overflow");
+            HPX_THROW_BAD_ALLOC("has_sufficient_stack_space");
         }
-        bool sufficient_stack_space =
-            std::size_t(remaining_stack) >= space_needed;
+        bool const sufficient_stack_space =
+            static_cast<std::size_t>(remaining_stack) >= space_needed;
 
         return sufficient_stack_space;
 #else
         return true;
 #endif
     }
-}}    // namespace hpx::this_thread
+}    // namespace hpx::this_thread
