@@ -13,27 +13,33 @@
 #include <type_traits>
 #include <utility>
 
-namespace hpx { namespace util {
-
-    template <typename T>
-    struct HPX_DEPRECATED_V(
-        1, 6, "hpx::util::decay is deprecated, use std::decay instead.") decay
-      : std::decay<T>
-    {
-    };
+namespace hpx::util {
 
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
-        template <typename TD>
+
+        template <typename TD, typename Enable = void>
         struct decay_unwrap_impl
         {
             using type = TD;
+
+            template <typename T>
+            constexpr static T call(T&& t) noexcept
+            {
+                return std::forward<T>(t);
+            }
         };
 
         template <typename X>
         struct decay_unwrap_impl<::std::reference_wrapper<X>>
         {
             using type = X&;
+
+            constexpr static decltype(auto) call(
+                ::std::reference_wrapper<X> ref) noexcept
+            {
+                return ref.get();
+            }
         };
     }    // namespace detail
 
@@ -44,4 +50,4 @@ namespace hpx { namespace util {
 
     template <typename T>
     using decay_unwrap_t = typename decay_unwrap<T>::type;
-}}    // namespace hpx::util
+}    // namespace hpx::util
