@@ -1,4 +1,4 @@
-//  Copyright (c) 2016 Hartmut Kaiser
+//  Copyright (c) 2016-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,12 +6,11 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
+namespace hpx::traits::detail {
 
-namespace hpx { namespace traits { namespace detail {
     // wraps int so that int argument is favored over wrap_int
     struct wrap_int
     {
-        constexpr wrap_int(int) {}
+        /*implicit*/ constexpr wrap_int(int) noexcept {}
     };
-}}}    // namespace hpx::traits::detail
+}    // namespace hpx::traits::detail
