@@ -10,28 +10,33 @@
 
 #include <functional>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     template <typename T>
     struct unwrap_reference
     {
-        typedef T type;
+        using type = T;
     };
 
     template <typename T>
-    struct unwrap_reference<std::reference_wrapper<T>>
+    struct unwrap_reference<::std::reference_wrapper<T>>
     {
-        typedef T type;
+        using type = T;
     };
 
     template <typename T>
-    struct unwrap_reference<std::reference_wrapper<T> const>
+    struct unwrap_reference<::std::reference_wrapper<T> const>
     {
-        typedef T type;
+        using type = T;
     };
 
     template <typename T>
-    HPX_FORCEINLINE typename unwrap_reference<T>::type& unwrap_ref(T& t)
+    using unwrap_reference_t = typename unwrap_reference<T>::type;
+
+    template <typename T>
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr unwrap_reference_t<T>& unwrap_ref(
+        T& t) noexcept
     {
         return t;
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
