@@ -8,7 +8,8 @@
 
 #include <hpx/config.hpp>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     ///////////////////////////////////////////////////////////////////////////
     // This utility simplifies templates returning compatible types
     //
@@ -24,8 +25,9 @@ namespace hpx { namespace util {
     struct void_guard<void>
     {
         template <typename T>
-        HPX_HOST_DEVICE HPX_FORCEINLINE void operator,(T const&) const noexcept
+        HPX_HOST_DEVICE HPX_FORCEINLINE constexpr void operator,(
+            T const&) const noexcept
         {
         }
     };
-}}    // namespace hpx::util
+}    // namespace hpx::util
