@@ -8,9 +8,10 @@
 
 #include <cstddef>
 
-namespace hpx { namespace util {
-    inline std::size_t calculate_fanout(
-        std::size_t size, std::size_t local_fanout)
+namespace hpx::util {
+
+    constexpr std::size_t calculate_fanout(
+        std::size_t size, std::size_t local_fanout) noexcept
     {
         if (size == 0 || local_fanout == 0)
             return 1;
@@ -25,4 +26,4 @@ namespace hpx { namespace util {
         }
         return fanout;
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
