@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2013 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -10,35 +10,37 @@
 #include <cstdint>
 #include <vector>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     // helper function for counter evaluation
-    inline std::uint64_t get_and_reset_value(
+    [[nodiscard]] constexpr std::uint64_t get_and_reset_value(
         std::uint64_t& value, bool reset) noexcept
     {
-        std::uint64_t result = value;
+        std::uint64_t const result = value;
         if (reset)
             value = 0;
         return result;
     }
 
-    inline std::int64_t get_and_reset_value(
+    [[nodiscard]] constexpr std::int64_t get_and_reset_value(
         std::int64_t& value, bool reset) noexcept
     {
-        std::int64_t result = value;
+        std::int64_t const result = value;
         if (reset)
             value = 0;
         return result;
     }
 
     template <typename T>
-    inline T get_and_reset_value(std::atomic<T>& value, bool reset) noexcept
+    [[nodiscard]] T get_and_reset_value(
+        std::atomic<T>& value, bool reset) noexcept
     {
         if (reset)
             return value.exchange(0, std::memory_order_acq_rel);
         return value.load(std::memory_order_relaxed);
     }
 
-    inline std::vector<std::int64_t> get_and_reset_value(
+    [[nodiscard]] inline std::vector<std::int64_t> get_and_reset_value(
         std::vector<std::int64_t>& value, bool reset) noexcept
     {
         std::vector<std::int64_t> result = value;
@@ -47,4 +49,4 @@ namespace hpx { namespace util {
 
         return result;
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
