@@ -1,4 +1,4 @@
-//  Copyright (c) 2020 Hartmut Kaiser
+//  Copyright (c) 2020-2022 Hartmut Kaiser
 //  Copyright 2002, 2005 Daryle Walker
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -9,21 +9,21 @@
 
 #include <ios>
 
-namespace hpx { namespace util {
+namespace hpx::util {
 
     // this is taken from the Boost.Io library
     class ios_flags_saver
     {
     public:
-        typedef ::std::ios_base state_type;
-        typedef ::std::ios_base::fmtflags aspect_type;
+        using state_type = ::std::ios_base;
+        using aspect_type = ::std::ios_base::fmtflags;
 
         explicit ios_flags_saver(state_type& s)
           : s_save_(s)
           , a_save_(s.flags())
         {
         }
-        ios_flags_saver(state_type& s, aspect_type const& a)
+        ios_flags_saver(state_type& s, aspect_type a)
           : s_save_(s)
           , a_save_(s.flags(a))
         {
@@ -37,7 +37,7 @@ namespace hpx { namespace util {
         ios_flags_saver(ios_flags_saver const&) = delete;
         ios_flags_saver& operator=(ios_flags_saver const&) = delete;
 
-        void restore()
+        void restore() const
         {
             s_save_.flags(a_save_);
         }
@@ -46,4 +46,4 @@ namespace hpx { namespace util {
         state_type& s_save_;
         aspect_type const a_save_;
     };
-}}    // namespace hpx::util
+}    // namespace hpx::util
