@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2012 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -10,17 +10,18 @@
 #include <string>
 #include <vector>
 
-namespace hpx { namespace util {
+namespace hpx::util {
+
     ///////////////////////////////////////////////////////////////////////////
     inline std::string trim_whitespace(std::string const& s)
     {
         typedef std::string::size_type size_type;
 
-        size_type first = s.find_first_not_of(" \t");
+        size_type const first = s.find_first_not_of(" \t");
         if (std::string::npos == first)
-            return std::string();
+            return {};
 
-        size_type last = s.find_last_not_of(" \t");
+        size_type const last = s.find_last_not_of(" \t");
         return s.substr(first, last - first + 1);
     }
 
@@ -34,13 +35,13 @@ namespace hpx { namespace util {
     {
         for (std::string const& s : cfg)
         {
-            std::string::size_type p = s.find_first_of('=');
+            std::string::size_type const p = s.find_first_of('=');
             std::string key(trim_whitespace(s.substr(0, p)));
             if (key[key.size() - 1] == '!')
                 key.erase(key.size() - 1);
 
             std::string value(trim_whitespace(s.substr(p + 1)));
-            config_.insert(map_type::value_type(key, value));
+            config_.emplace(key, HPX_MOVE(value));
         }
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
