@@ -1,4 +1,5 @@
 //  Copyright (c)      2016 Thomas Heller
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,23 +8,21 @@
 #pragma once
 
 #include <hpx/config.hpp>
+#include <hpx/actions_base/basic_action_fwd.hpp>
 
 #include <cstdint>
 
-namespace hpx { namespace actions {
+namespace hpx::actions {
 
     /// \cond NOINTERNAL
 
     ///////////////////////////////////////////////////////////////////////////
-    template <typename Component, typename Signature, typename Derived>
-    struct basic_action;
-
-    /// The type of an action defines whether this action will be executed
+    /// The type of the action defines whether this action will be executed
     /// directly or by an HPX-threads
     enum class action_flavor
     {
         plain_action = 0,    ///< The action will be executed by a newly created
-        ///< thread
+                             ///< thread
         direct_action = 1    ///< The action needs to be executed directly
     };
 
@@ -39,4 +38,4 @@ namespace hpx { namespace actions {
     }    // namespace detail
 
     /// \endcond
-}}    // namespace hpx::actions
+}    // namespace hpx::actions
