@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //  Copyright (c)      2011 Thomas Heller
 //
@@ -7,6 +7,8 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 /// \file hpx/actions_base/basic_action.hpp
+/// \page HPX_REGISTER_ACTION_DECLARATION, HPX_REGISTER_ACTION
+/// \headerfile hpx/components.hpp
 
 #pragma once
 
@@ -42,25 +44,24 @@
 #include <hpx/preprocessor/cat.hpp>
 #include <hpx/preprocessor/expand.hpp>
 #include <hpx/preprocessor/nargs.hpp>
-#include <hpx/preprocessor/stringize.hpp>
 #include <hpx/runtime_local/report_error.hpp>
 #include <hpx/type_support/pack.hpp>
 #include <hpx/util/get_and_reset_value.hpp>
-#if HPX_HAVE_ITTNOTIFY != 0 && !defined(HPX_HAVE_APEX)
+#if defined(HPX_HAVE_ITTNOTIFY) && HPX_HAVE_ITTNOTIFY != 0 &&                  \
+    !defined(HPX_HAVE_APEX)
 #include <hpx/modules/itt_notify.hpp>
 #endif
 
-#include <boost/utility/string_ref.hpp>
-
 #include <atomic>
 #include <cstddef>
 #include <cstdint>
 #include <exception>
 #include <string>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 
-namespace hpx { namespace actions {
+namespace hpx::actions {
 
     /// \cond NOINTERNAL
 
@@ -77,7 +78,7 @@ namespace hpx { namespace actions {
             HPX_FORCEINLINE typename Action::internal_result_type operator()(
                 Ts&&... vs) const
             {
-                return Action::invoke(lva, comptype, std::forward<Ts>(vs)...);
+                return Action::invoke(lva, comptype, HPX_FORWARD(Ts, vs)...);
             }
         };
 
@@ -90,13 +91,13 @@ namespace hpx { namespace actions {
         {
         public:
             template <typename... Ts>
-            explicit thread_function(naming::id_type&& target,
+            explicit thread_function(hpx::id_type&& target,
                 naming::address_type lva, naming::component_type comptype,
                 Ts&&... vs)
-              : target_(std::move(target))
+              : target_(HPX_MOVE(target))
               , lva_(lva)
               , comptype_(comptype)
-              , args_(std::forward<Ts>(vs)...)
+              , args_(HPX_FORWARD(Ts, vs)...)
             {
             }
 
@@ -109,8 +110,8 @@ namespace hpx { namespace actions {
                         "Executing {}.", Action::get_action_name(lva_));
 
                     // invoke the action, ignoring the return value
-                    util::invoke_fused(action_invoke<Action>{lva_, comptype_},
-                        std::move(args_));
+                    hpx::invoke_fused(action_invoke<Action>{lva_, comptype_},
+                        HPX_MOVE(args_));
                 }
                 catch (hpx::thread_interrupted const&)
                 {    //-V565
@@ -146,7 +147,7 @@ namespace hpx { namespace actions {
 
         private:
             // This holds the target alive, if necessary.
-            naming::id_type target_;
+            hpx::id_type target_;
             naming::address_type lva_;
             naming::component_type comptype_;
             typename Action::arguments_type args_;
@@ -158,39 +159,38 @@ namespace hpx { namespace actions {
         {
         public:
             template <typename... Ts>
-            explicit continuation_thread_function(naming::id_type&& target,
+            explicit continuation_thread_function(hpx::id_type&& target,
                 typename Action::continuation_type&& cont,
                 naming::address_type lva, naming::component_type comptype,
                 Ts&&... vs)
-              : target_(std::move(target))
-              , cont_(std::move(cont))
+              : target_(HPX_MOVE(target))
+              , cont_(HPX_MOVE(cont))
               , lva_(lva)
               , comptype_(comptype)
-              , args_(std::forward<Ts>(vs)...)
+              , args_(HPX_FORWARD(Ts, vs)...)
             {
             }
 
             template <typename State,
-                typename Enable = typename std::enable_if<std::is_same<State,
-                    threads::thread_restart_state>::value>::type>
+                typename Enable = std::enable_if_t<
+                    std::is_same_v<State, threads::thread_restart_state>>>
             threads::thread_result_type operator()(State)
             {
                 LTM_(debug).format("Executing {} with continuation({})",
                     Action::get_action_name(lva_), cont_.get_id());
 
                 traits::action_trigger_continuation<
-                    typename Action::continuation_type>::call(std::move(cont_),
-                    util::functional::invoke_fused{},
-                    action_invoke<Action>{lva_, comptype_}, std::move(args_));
+                    typename Action::continuation_type>::call(HPX_MOVE(cont_),
+                    hpx::functional::invoke_fused{},
+                    action_invoke<Action>{lva_, comptype_}, HPX_MOVE(args_));
 
-                return threads::thread_result_type(
-                    threads::thread_schedule_state::terminated,
-                    threads::invalid_thread_id);
+                return {threads::thread_schedule_state::terminated,
+                    threads::invalid_thread_id};
             }
 
         private:
             // This holds the target alive, if necessary.
-            naming::id_type target_;
+            hpx::id_type target_;
             typename Action::continuation_type cont_;
             naming::address_type lva_;
             naming::component_type comptype_;
@@ -201,14 +201,13 @@ namespace hpx { namespace actions {
         template <typename T>
         struct is_non_const_reference
           : std::integral_constant<bool,
-                std::is_lvalue_reference<T>::value &&
-                    !std::is_const<
-                        typename std::remove_reference<T>::type>::value>
+                std::is_lvalue_reference_v<T> &&
+                    !std::is_const_v<std::remove_reference_t<T>>>
         {
         };
 
         ///////////////////////////////////////////////////////////////////////
-        inline std::string make_action_name(boost::string_ref action_name)
+        inline std::string make_action_name(std::string_view action_name)
         {
             return hpx::util::format("action({})", action_name);
         }
@@ -218,19 +217,19 @@ namespace hpx { namespace actions {
         typename Derived>
     struct basic_action<Component, R(Args...), Derived>
     {
+#if !defined(HPX_SERIALIZATION_HAVE_ALLOW_RAW_POINTER_SERIALIZATION)
         // Flag the use of raw pointer types as action arguments
-        static_assert(!util::any_of<std::is_pointer<Args>...>::value,
+        static_assert(!util::any_of_v<std::is_pointer<Args>...>,
             "Using raw pointers as arguments for actions is not supported.");
+#endif
 
         // Flag the use of array types as action arguments
         static_assert(
-            !util::any_of<std::is_array<
-                typename std::remove_reference<Args>::type>...>::value,
+            !util::any_of_v<std::is_array<std::remove_reference_t<Args>>...>,
             "Using arrays as arguments for actions is not supported.");
 
         // Flag the use of non-const reference types as action arguments
-        static_assert(
-            !util::any_of<detail::is_non_const_reference<Args>...>::value,
+        static_assert(!util::any_of_v<detail::is_non_const_reference<Args>...>,
             "Using non-const references as arguments for actions is not "
             "supported.");
 
@@ -254,7 +253,7 @@ namespace hpx { namespace actions {
         static constexpr std::size_t arity = sizeof...(Args);
 
         using internal_result_type = R;
-        using arguments_type = hpx::tuple<typename std::decay<Args>::type...>;
+        using arguments_type = hpx::tuple<std::decay_t<Args>...>;
 
         using action_tag = void;
 
@@ -272,27 +271,15 @@ namespace hpx { namespace actions {
         static remote_result_type invoker(naming::address_type lva,
             naming::component_type comptype, Ts&&... vs)
         {
-            using is_void = typename std::is_void<R>::type;
-            return invoker_impl(
-                is_void{}, lva, comptype, std::forward<Ts>(vs)...);
-        }
-
-    protected:
-        template <typename... Ts>
-        HPX_FORCEINLINE static remote_result_type invoker_impl(std::true_type,
-            naming::address_type lva, naming::component_type comptype,
-            Ts&&... vs)
-        {
-            Derived::invoke(lva, comptype, std::forward<Ts>(vs)...);
-            return util::unused;
-        }
-
-        template <typename... Ts>
-        HPX_FORCEINLINE static remote_result_type invoker_impl(std::false_type,
-            naming::address_type lva, naming::component_type comptype,
-            Ts&&... vs)
-        {
-            return Derived::invoke(lva, comptype, std::forward<Ts>(vs)...);
+            if constexpr (std::is_void_v<R>)
+            {
+                Derived::invoke(lva, comptype, std::forward<Ts>(vs)...);
+                return util::unused;
+            }
+            else
+            {
+                return Derived::invoke(lva, comptype, std::forward<Ts>(vs)...);
+            }
         }
 
     public:
@@ -302,17 +289,13 @@ namespace hpx { namespace actions {
         // case no continuation has been supplied.
         template <typename... Ts>
         static threads::thread_function_type construct_thread_function(
-            naming::id_type target, naming::address_type lva,
+            hpx::id_type target, naming::address_type lva,
             naming::component_type comptype, Ts&&... vs)
         {
-            if (target &&
-                target.get_management_type() == naming::id_type::unmanaged)
-                target = {};
-
             using thread_function = detail::thread_function<Derived>;
             return traits::action_decorate_function<Derived>::call(lva,
                 thread_function(
-                    std::move(target), lva, comptype, std::forward<Ts>(vs)...));
+                    HPX_MOVE(target), lva, comptype, HPX_FORWARD(Ts, vs)...));
         }
 
         // This static construct_thread_function allows to construct
@@ -321,19 +304,15 @@ namespace hpx { namespace actions {
         // case a continuation has been supplied
         template <typename... Ts>
         static threads::thread_function_type construct_thread_function(
-            naming::id_type target, continuation_type&& cont,
+            hpx::id_type target, continuation_type&& cont,
             naming::address_type lva, naming::component_type comptype,
             Ts&&... vs)
         {
-            if (target &&
-                target.get_management_type() == naming::id_type::unmanaged)
-                target = {};
-
             using thread_function =
                 detail::continuation_thread_function<Derived>;
             return traits::action_decorate_function<Derived>::call(lva,
-                thread_function(std::move(target), std::move(cont), lva,
-                    comptype, std::forward<Ts>(vs)...));
+                thread_function(HPX_MOVE(target), HPX_MOVE(cont), lva, comptype,
+                    HPX_FORWARD(Ts, vs)...));
         }
 
         // direct execution
@@ -345,7 +324,7 @@ namespace hpx { namespace actions {
             LTM_(debug).format("basic_action::execute_function {}",
                 Derived::get_action_name(lva));
 
-            return invoker(lva, comptype, std::forward<Ts>(vs)...);
+            return invoker(lva, comptype, HPX_FORWARD(Ts, vs)...);
         }
 
     private:
@@ -355,87 +334,82 @@ namespace hpx { namespace actions {
             IdOrPolicy const& id_or_policy, error_code&, Ts&&... vs)
         {
             return hpx::sync<basic_action>(
-                policy, id_or_policy, std::forward<Ts>(vs)...);
+                policy, id_or_policy, HPX_FORWARD(Ts, vs)...);
         }
 
     public:
         ///////////////////////////////////////////////////////////////////////
         template <typename... Ts>
         HPX_FORCEINLINE result_type operator()(launch policy,
-            naming::id_type const& id, error_code& ec, Ts&&... vs) const
+            hpx::id_type const& id, error_code& ec, Ts&&... vs) const
         {
-            return sync_invoke(policy, id, ec, std::forward<Ts>(vs)...);
+            return sync_invoke(policy, id, ec, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
         HPX_FORCEINLINE result_type operator()(
-            naming::id_type const& id, error_code& ec, Ts&&... vs) const
+            hpx::id_type const& id, error_code& ec, Ts&&... vs) const
         {
-            return sync_invoke(launch::sync, id, ec, std::forward<Ts>(vs)...);
+            return sync_invoke(launch::sync, id, ec, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
         HPX_FORCEINLINE result_type operator()(
-            launch policy, naming::id_type const& id, Ts&&... vs) const
+            launch policy, hpx::id_type const& id, Ts&&... vs) const
         {
-            return sync_invoke(policy, id, throws, std::forward<Ts>(vs)...);
+            return sync_invoke(policy, id, throws, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
         HPX_FORCEINLINE result_type operator()(
-            naming::id_type const& id, Ts&&... vs) const
+            hpx::id_type const& id, Ts&&... vs) const
         {
             return sync_invoke(
-                launch::sync, id, throws, std::forward<Ts>(vs)...);
+                launch::sync, id, throws, HPX_FORWARD(Ts, vs)...);
         }
 
         ///////////////////////////////////////////////////////////////////////
         template <typename DistPolicy, typename... Ts>
-        HPX_FORCEINLINE typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value,
-            result_type>::type
+        HPX_FORCEINLINE std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>, result_type>
         operator()(launch policy, DistPolicy const& dist_policy, error_code& ec,
             Ts&&... vs) const
         {
-            return sync_invoke(
-                policy, dist_policy, ec, std::forward<Ts>(vs)...);
+            return sync_invoke(policy, dist_policy, ec, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename DistPolicy, typename... Ts>
-        HPX_FORCEINLINE typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value,
-            result_type>::type
+        HPX_FORCEINLINE std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>, result_type>
         operator()(
             DistPolicy const& dist_policy, error_code& ec, Ts&&... vs) const
         {
             return sync_invoke(
-                launch::sync, dist_policy, ec, std::forward<Ts>(vs)...);
+                launch::sync, dist_policy, ec, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename DistPolicy, typename... Ts>
-        HPX_FORCEINLINE typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value,
-            result_type>::type
+        HPX_FORCEINLINE std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>, result_type>
         operator()(
             launch policy, DistPolicy const& dist_policy, Ts&&... vs) const
         {
             return sync_invoke(
-                policy, dist_policy, throws, std::forward<Ts>(vs)...);
+                policy, dist_policy, throws, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename DistPolicy, typename... Ts>
-        HPX_FORCEINLINE typename std::enable_if<
-            traits::is_distribution_policy<DistPolicy>::value,
-            result_type>::type
+        HPX_FORCEINLINE std::enable_if_t<
+            traits::is_distribution_policy_v<DistPolicy>, result_type>
         operator()(DistPolicy const& dist_policy, Ts&&... vs) const
         {
             return sync_invoke(
-                launch::sync, dist_policy, throws, std::forward<Ts>(vs)...);
+                launch::sync, dist_policy, throws, HPX_FORWARD(Ts, vs)...);
         }
 
         ///////////////////////////////////////////////////////////////////////
         /// retrieve component type
-        static int get_component_type()
+        static int get_component_type() noexcept
         {
             return static_cast<int>(
                 components::get_component_type<Component>());
@@ -443,9 +417,9 @@ namespace hpx { namespace actions {
 
         using direct_execution = std::false_type;
 
-        /// The function \a get_action_type returns whether this action needs
-        /// to be executed in a new thread or directly.
-        static constexpr actions::action_flavor get_action_type()
+        // The function \a get_action_type returns whether this action needs
+        // to be executed in a new thread or directly.
+        static constexpr actions::action_flavor get_action_type() noexcept
         {
             return actions::action_flavor::plain_action;
         }
@@ -472,6 +446,7 @@ namespace hpx { namespace actions {
         basic_action<Component, R(Args...), Derived>::invocation_count_(0);
 
     namespace detail {
+
         template <typename Action>
         void register_local_action_invocation_count(
             invocation_count_registry& registry)
@@ -484,44 +459,40 @@ namespace hpx { namespace actions {
 
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
-        // simple type allowing to distinguish whether an action is the most
-        // derived one
-        struct this_type
-        {
-        };
 
-        template <typename Action, typename Derived>
+        template <typename Action, typename Derived = void>
         struct action_type
         {
             using type = Derived;
         };
 
         template <typename Action>
-        struct action_type<Action, this_type>
+        struct action_type<Action, void>
         {
             using type = Action;
         };
+
+        template <typename Action, typename Derived>
+        using action_type_t = typename action_type<Action, Derived>::type;
+
     }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
-    template <typename TF, TF F, typename Derived = detail::this_type>
+    template <typename TF, TF F, typename Derived = void>
     struct action;
 
     ///////////////////////////////////////////////////////////////////////////
-    template <typename TF, TF F, typename Derived = detail::this_type>
+    template <typename TF, TF F, typename Derived = void>
     struct direct_action
       : action<TF, F,
-            typename detail::action_type<direct_action<TF, F, Derived>,
-                Derived>::type>
+            detail::action_type_t<direct_action<TF, F, Derived>, Derived>>
     {
-        using derived_type =
-            typename detail::action_type<direct_action, Derived>::type;
-
+        using derived_type = detail::action_type_t<direct_action, Derived>;
         using direct_execution = std::true_type;
 
         /// The function \a get_action_type returns whether this action needs
         /// to be executed in a new thread or directly.
-        static constexpr actions::action_flavor action_flavor()
+        static constexpr actions::action_flavor get_action_type() noexcept
         {
             return actions::action_flavor::direct_action;
         }
@@ -531,7 +502,7 @@ namespace hpx { namespace actions {
     // Base template allowing to generate a concrete action type from a function
     // pointer. It is instantiated only if the supplied pointer is not a
     // supported function pointer.
-    template <typename TF, TF F, typename Derived = detail::this_type,
+    template <typename TF, TF F, typename Derived = void,
         typename Direct = std::false_type>
     struct make_action
     {
@@ -544,22 +515,31 @@ namespace hpx { namespace actions {
         using type = direct_action<TF, F, Derived>;
     };
 
-    template <typename TF, TF F, typename Derived = detail::this_type>
+    template <typename TF, TF F, typename Derived = void,
+        typename Direct = std::false_type>
+    using make_action_t = typename make_action<TF, F, Derived, Direct>::type;
+
+    template <typename TF, TF F, typename Derived = void>
     struct make_direct_action : make_action<TF, F, Derived, std::true_type>
     {
     };
 
+    template <typename TF, TF F, typename Derived>
+    using make_direct_action_t =
+        typename make_direct_action<TF, F, Derived>::type;
+
+    /// \endcond
+}    // namespace hpx::actions
+// namespace hpx::actions
+
+/// \cond NOINTERNAL
+
 // Macros usable to refer to an action given the function to expose
 #define HPX_MAKE_ACTION(func)                                                  \
     hpx::actions::make_action<decltype(&func), &func> /**/ /**/
 #define HPX_MAKE_DIRECT_ACTION(func)                                           \
     hpx::actions::make_direct_action<decltype(&func), &func> /**/ /**/
 
-        /// \endcond
-}}    // namespace hpx::actions
-
-/// \cond NOINTERNAL
-
 ///////////////////////////////////////////////////////////////////////////////
 /// \def HPX_DECLARE_ACTION(func, name)
 /// \brief Declares an action type
@@ -596,29 +576,11 @@ namespace hpx { namespace actions {
     HPX_REGISTER_ACTION_2(action, action)                                      \
     /**/
 
-#if HPX_HAVE_ITTNOTIFY != 0 && !defined(HPX_HAVE_APEX)
-#define HPX_DEFINE_GET_ACTION_NAME_ITT(action, actionname)                     \
-    namespace hpx { namespace actions { namespace detail {                     \
-                template <>                                                    \
-                HPX_ALWAYS_EXPORT util::itt::string_handle const&              \
-                get_action_name_itt<action>()                                  \
-                {                                                              \
-                    static util::itt::string_handle sh(                        \
-                        HPX_PP_STRINGIZE(actionname));                         \
-                    return sh;                                                 \
-                }                                                              \
-            }                                                                  \
-        }                                                                      \
-    }                                                                          \
-/**/
-#else
-#define HPX_DEFINE_GET_ACTION_NAME_ITT(action, actionname)
-#endif
-
 #if !defined(HPX_HAVE_NETWORKING)
 
 #define HPX_DEFINE_GET_ACTION_NAME(action) /**/
-#if HPX_HAVE_ITTNOTIFY != 0 && !defined(HPX_HAVE_APEX)
+#if defined(HPX_HAVE_ITTNOTIFY) && HPX_HAVE_ITTNOTIFY != 0 &&                  \
+    !defined(HPX_HAVE_APEX)
 #define HPX_DEFINE_GET_ACTION_NAME_ITT(action, actionname) /**/
 #endif
 #define HPX_REGISTER_ACTION_EXTERN_DECLARATION(action) /**/
@@ -641,13 +603,12 @@ namespace hpx { namespace actions {
 #define HPX_ACTION_USES_HUGE_STACK(action)   /**/
 #else
 #define HPX_ACTION_USES_STACK(action, size)                                    \
-    namespace hpx { namespace traits {                                         \
-            template <>                                                        \
-            struct action_stacksize<action>                                    \
-            {                                                                  \
-                HPX_STATIC_CONSTEXPR threads::thread_stacksize value = size;   \
-            };                                                                 \
-        }                                                                      \
+    namespace hpx::traits {                                                    \
+        template <>                                                            \
+        struct action_stacksize<action>                                        \
+        {                                                                      \
+            static constexpr threads::thread_stacksize value = size;           \
+        };                                                                     \
     }                                                                          \
     /**/
 
@@ -665,23 +626,6 @@ namespace hpx { namespace actions {
 /**/
 #endif
 
-#if defined(HPX_COMPUTE_DEVICE_CODE)
-#define HPX_ACTION_DOES_NOT_SUSPEND(action) /**/
-#else
-// This macro is deprecated. It expands to an inline function which will emit a
-// warning.
-#define HPX_ACTION_DOES_NOT_SUSPEND(action)                                    \
-    HPX_DEPRECATED_V(1, 4,                                                     \
-        "HPX_ACTION_DOES_NOT_SUSPEND is deprecated and will be "               \
-        "removed in the future")                                               \
-    static inline void HPX_PP_CAT(HPX_ACTION_DOES_NOT_SUSPEND_, action)();     \
-    void HPX_PP_CAT(HPX_ACTION_DOES_NOT_SUSPEND_, action)()                    \
-    {                                                                          \
-        HPX_PP_CAT(HPX_ACTION_DOES_NOT_SUSPEND_, action)();                    \
-    }                                                                          \
-/**/
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 #if defined(HPX_COMPUTE_DEVICE_CODE)
 #define HPX_ACTION_HAS_PRIORITY(action, priority)      /**/
@@ -694,19 +638,17 @@ namespace hpx { namespace actions {
 #else
 ///////////////////////////////////////////////////////////////////////////////
 #define HPX_ACTION_HAS_PRIORITY(action, priority)                              \
-    namespace hpx { namespace traits {                                         \
-            template <>                                                        \
-            struct action_priority<action>                                     \
-            {                                                                  \
-                HPX_STATIC_CONSTEXPR threads::thread_priority value =          \
-                    priority;                                                  \
-            };                                                                 \
-            /* make sure the action is not executed directly */                \
-            template <>                                                        \
-            struct has_decorates_action<action> : std::true_type               \
-            {                                                                  \
-            };                                                                 \
-        }                                                                      \
+    namespace hpx::traits {                                                    \
+        template <>                                                            \
+        struct action_priority<action>                                         \
+        {                                                                      \
+            static constexpr threads::thread_priority value = priority;        \
+        };                                                                     \
+        /* make sure the action is not executed directly */                    \
+        template <>                                                            \
+        struct has_decorates_action<action> : std::true_type                   \
+        {                                                                      \
+        };                                                                     \
     }                                                                          \
     /**/
 
@@ -760,7 +702,7 @@ namespace hpx { namespace actions {
 ///          {
 ///              void print_greeting ()
 ///              {
-///                  hpx::cout << "Hey, how are you?\n" << hpx::flush;
+///                  hpx::cout << "Hey, how are you?\n" << std::flush;
 ///              }
 ///
 ///              // Component actions need to be declared, this also defines the
@@ -771,7 +713,7 @@ namespace hpx { namespace actions {
 ///      }
 ///
 ///      // Declare boilerplate code required for each of the component actions.
-///      HPX_REGISTER_ACTION_DECLARATION(app::server::print_greeting_action);
+///      HPX_REGISTER_ACTION_DECLARATION(app::server::print_greeting_action)
 /// \endcode
 ///
 /// \note This macro has to be used once for each of the component actions
