@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //  Copyright (c)      2011 Thomas Heller
 //
@@ -8,16 +8,7 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
-#include <hpx/actions_base/preassigned_action_id.hpp>
-#include <hpx/functional/traits/get_action_name.hpp>
-
-#if defined(HPX_HAVE_NETWORKING) &&                                            \
-    (HPX_HAVE_ITTNOTIFY != 0 && !defined(HPX_HAVE_APEX))
-#include <hpx/modules/itt_notify.hpp>
-#endif
-
-namespace hpx { namespace actions {
+namespace hpx::actions {
 
     ///////////////////////////////////////////////////////////////////////////
     /// \tparam Component         component type
@@ -25,4 +16,4 @@ namespace hpx { namespace actions {
     /// \tparam Derived           derived action class
     template <typename Component, typename Signature, typename Derived>
     struct basic_action;
-}}    // namespace hpx::actions
+}    // namespace hpx::actions
