@@ -6,13 +6,13 @@
 
 #pragma once
 
-namespace hpx { namespace actions {
+namespace hpx::actions {
 
     template <typename Result, typename RemoteResult = Result>
     struct typed_continuation;
-}}    // namespace hpx::actions
+}    // namespace hpx::actions
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     ///////////////////////////////////////////////////////////////////////////
     // Trait to determine the continuation type for an action
@@ -23,4 +23,7 @@ namespace hpx { namespace traits {
             hpx::actions::typed_continuation<typename Action::local_result_type,
                 typename Action::remote_result_type>;
     };
-}}    // namespace hpx::traits
+
+    template <typename Action>
+    using action_continuation_t = typename action_continuation<Action>::type;
+}    // namespace hpx::traits
