@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -8,7 +8,7 @@
 
 #include <hpx/coroutines/thread_enums.hpp>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     ///////////////////////////////////////////////////////////////////////////
     // Customization point for action stack size
@@ -18,4 +18,8 @@ namespace hpx { namespace traits {
         static constexpr threads::thread_priority value =
             threads::thread_priority::default_;
     };
-}}    // namespace hpx::traits
+
+    template <typename Action>
+    inline constexpr threads::thread_priority action_priority_v =
+        action_priority<Action>::value;
+}    // namespace hpx::traits
