@@ -1,4 +1,5 @@
 //  Copyright (c) 2015 Thomas Heller
+//  Copyright (c) 2021-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,11 +8,10 @@
 #pragma once
 
 #include <hpx/config.hpp>
-#include <hpx/type_support/always_void.hpp>
 
 #include <type_traits>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     namespace detail {
 
@@ -22,15 +22,19 @@ namespace hpx { namespace traits {
 
         template <typename Continuation>
         struct is_continuation_impl<Continuation,
-            typename util::always_void<
-                typename Continuation::continuation_tag>::type> : std::true_type
+            std::void_t<typename Continuation::continuation_tag>>
+          : std::true_type
         {
         };
     }    // namespace detail
 
     template <typename Continuation, typename Enable = void>
     struct is_continuation
-      : detail::is_continuation_impl<typename std::decay<Continuation>::type>
+      : detail::is_continuation_impl<std::decay_t<Continuation>>
     {
     };
-}}    // namespace hpx::traits
+
+    template <typename Continuation>
+    inline constexpr bool is_continuation_v =
+        is_continuation<Continuation>::value;
+}    // namespace hpx::traits
