@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,16 +6,19 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <type_traits>
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     // verify that given Action is actually supported by the given Component
     template <typename Action, typename Component, typename Enable = void>
     struct is_valid_action
-      : std::is_same<typename std::decay<typename Action::component_type>::type,
-            typename std::decay<Component>::type>
+      : std::is_same<std::decay_t<typename Action::component_type>,
+            std::decay_t<Component>>
     {
     };
-}}    // namespace hpx::traits
+
+    template <typename Action, typename Component>
+    inline constexpr bool is_valid_action_v =
+        is_valid_action<Action, Component>::value;
+}    // namespace hpx::traits
