@@ -1,5 +1,5 @@
 //  Copyright (c) 2011 Bryce Lelbach
-//  Copyright (c) 2011-2021 Hartmut Kaiser
+//  Copyright (c) 2011-2023 Hartmut Kaiser
 //  Copyright (c) 2016 Parsa Amini
 //  Copyright (c) 2016 Thomas Heller
 //
@@ -14,17 +14,18 @@
 #include <hpx/cache/lru_cache.hpp>
 #include <hpx/cache/statistics/local_full_statistics.hpp>
 #include <hpx/components_base/pinned_ptr.hpp>
+#include <hpx/datastructures/detail/dynamic_bitset.hpp>
 #include <hpx/functional/function.hpp>
 #include <hpx/modules/agas_base.hpp>
 #include <hpx/modules/errors.hpp>
+#include <hpx/modules/futures.hpp>
 #include <hpx/modules/runtime_configuration.hpp>
 #include <hpx/naming_base/address.hpp>
 #include <hpx/naming_base/id_type.hpp>
-#include <hpx/runtime/parcelset_fwd.hpp>
+#include <hpx/parcelset/parcelset_fwd.hpp>
+#include <hpx/synchronization/shared_mutex.hpp>
 #include <hpx/synchronization/spinlock.hpp>
 
-#include <boost/dynamic_bitset.hpp>
-
 #include <atomic>
 #include <cstddef>
 #include <cstdint>
@@ -53,13 +54,13 @@ namespace hpx { namespace agas {
     public:
         using component_id_type = components::component_type;
 
-        using iterate_names_return_type =
-            std::map<std::string, naming::id_type>;
+        using iterate_names_return_type = std::map<std::string, hpx::id_type>;
 
-        using iterate_types_function_type = hpx::util::function<void(
-            std::string const&, components::component_type)>;
+        using iterate_types_function_type =
+            hpx::function<void(std::string const&, components::component_type),
+                true>;
 
-        using mutex_type = hpx::lcos::local::spinlock;
+        using mutex_type = hpx::spinlock;
 
         // gva cache
         struct gva_cache_key;
@@ -70,7 +71,7 @@ namespace hpx { namespace agas {
         using migrated_objects_table_type = std::set<naming::gid_type>;
         using refcnt_requests_type = std::map<naming::gid_type, std::int64_t>;
 
-        mutable mutex_type gva_cache_mtx_;
+        mutable hpx::shared_mutex gva_cache_mtx_;
         std::shared_ptr<gva_cache_type> gva_cache_;
 
         mutable mutex_type migrated_objects_mtx_;
@@ -104,7 +105,7 @@ namespace hpx { namespace agas {
         std::atomic<hpx::state> state_;
         naming::gid_type locality_;
 
-        mutable mutex_type resolved_localities_mtx_;
+        mutable hpx::shared_mutex resolved_localities_mtx_;
         using resolved_localities_type =
             std::map<naming::gid_type, parcelset::endpoints_type>;
         resolved_localities_type resolved_localities_;
@@ -126,7 +127,7 @@ namespace hpx { namespace agas {
         void initialize(std::uint64_t rts_lva);
 
         /// \brief Adjust the size of the local AGAS Address resolution cache
-        void adjust_local_cache_size(std::size_t);
+        void adjust_local_cache_size(std::size_t) const;
 
         state get_status() const
         {
@@ -149,7 +150,7 @@ namespace hpx { namespace agas {
 
         bool is_bootstrap() const
         {
-            return service_type == service_mode_bootstrap;
+            return service_type == service_mode::bootstrap;
         }
 
         /// \brief Returns whether this addressing_service represents the console
@@ -167,7 +168,7 @@ namespace hpx { namespace agas {
         }
 
         bool resolve_locally_known_addresses(
-            naming::gid_type const& id, naming::address& addr);
+            naming::gid_type const& id, naming::address& addr) const;
 
         void register_server_instances();
 
@@ -177,8 +178,8 @@ namespace hpx { namespace agas {
         // FIXME: document (add comments)
         void garbage_collect(error_code& ec = throws);
 
-        std::int64_t synchronize_with_async_incref(
-            hpx::future<std::int64_t> fut, naming::id_type const& id,
+        static std::int64_t synchronize_with_async_incref(
+            std::int64_t old_credit, hpx::id_type const& id,
             std::int64_t compensated_credit);
 
         server::primary_namespace& get_local_primary_namespace_service()
@@ -197,26 +198,32 @@ namespace hpx { namespace agas {
         }
 
         server::component_namespace* get_local_component_namespace_service()
+            const
         {
             return component_ns_->get_service();
         }
 
-        server::locality_namespace* get_local_locality_namespace_service()
+        server::locality_namespace* get_local_locality_namespace_service() const
         {
             return locality_ns_->get_service();
         }
 
-        server::symbol_namespace& get_local_symbol_namespace_service()
+        server::symbol_namespace& get_local_symbol_namespace_service() const
         {
             return symbol_ns_.get_service();
         }
 
+        naming::address::address_type get_runtime_support_lva() const
+        {
+            return reinterpret_cast<naming::address::address_type>(rts_lva_);
+        }
+
     protected:
         void launch_bootstrap(parcelset::endpoints_type const& endpoints,
             util::runtime_configuration& rtcfg);
 
         naming::address resolve_full_postproc(naming::gid_type const& id,
-            future<primary_namespace::resolved_type> f);
+            primary_namespace::resolved_type const&);
         bool bind_postproc(
             naming::gid_type const& id, gva const& g, future<bool> f);
 
@@ -242,21 +249,21 @@ namespace hpx { namespace agas {
 
     public:
         // Helper functions to access the current cache statistics
-        std::uint64_t get_cache_entries(bool);
-        std::uint64_t get_cache_hits(bool);
-        std::uint64_t get_cache_misses(bool);
-        std::uint64_t get_cache_evictions(bool);
-        std::uint64_t get_cache_insertions(bool);
+        std::uint64_t get_cache_entries(bool) const;
+        std::uint64_t get_cache_hits(bool) const;
+        std::uint64_t get_cache_misses(bool) const;
+        std::uint64_t get_cache_evictions(bool) const;
+        std::uint64_t get_cache_insertions(bool) const;
 
-        std::uint64_t get_cache_get_entry_count(bool reset);
-        std::uint64_t get_cache_insertion_entry_count(bool reset);
-        std::uint64_t get_cache_update_entry_count(bool reset);
-        std::uint64_t get_cache_erase_entry_count(bool reset);
+        std::uint64_t get_cache_get_entry_count(bool reset) const;
+        std::uint64_t get_cache_insertion_entry_count(bool reset) const;
+        std::uint64_t get_cache_update_entry_count(bool reset) const;
+        std::uint64_t get_cache_erase_entry_count(bool reset) const;
 
-        std::uint64_t get_cache_get_entry_time(bool reset);
-        std::uint64_t get_cache_insertion_entry_time(bool reset);
-        std::uint64_t get_cache_update_entry_time(bool reset);
-        std::uint64_t get_cache_erase_entry_time(bool reset);
+        std::uint64_t get_cache_get_entry_time(bool reset) const;
+        std::uint64_t get_cache_insertion_entry_time(bool reset) const;
+        std::uint64_t get_cache_update_entry_time(bool reset) const;
+        std::uint64_t get_cache_erase_entry_time(bool reset) const;
 
     public:
         /// \brief Add a locality to the runtime.
@@ -281,7 +288,7 @@ namespace hpx { namespace agas {
 
         /// \brief Get locality locality_id of the console locality.
         ///
-        /// \param locality_id     [out] The locality_id value uniquely identifying the
+        /// \param locality   [out] The locality_id value uniquely identifying the
         ///                   console locality. This is valid only, if the
         ///                   return value of this function is true.
         /// \param try_cache  [in] If this is set to true the console is first
@@ -317,7 +324,7 @@ namespace hpx { namespace agas {
         /// \param type       [in] The component type will be used to determine
         ///                   the set of prefixes having a registered factory
         ///                   for this component. The default value for this
-        ///                   parameter is \a components#component_invalid,
+        ///                   parameter is \a components#component_enum_type#invalid,
         ///                   which will return prefixes of all localities.
         /// \param ec         [in,out] this represents the error status on exit,
         ///                   if this is pre-initialized to \a hpx#throws
@@ -326,16 +333,16 @@ namespace hpx { namespace agas {
         /// \note             As long as \a ec is not pre-initialized to
         ///                   \a hpx#throws this function doesn't
         ///                   throw but returns the result code using the
-        ///                   parameter \a ec. Otherwise it throws an instance
+        ///                   parameter \a ec. Otherwise, it throws an instance
         ///                   of hpx#exception.
         bool get_localities(std::vector<naming::gid_type>& locality_ids,
-            components::component_type type, error_code& ec = throws);
+            components::component_type type, error_code& ec = throws) const;
 
         bool get_localities(std::vector<naming::gid_type>& locality_ids,
-            error_code& ec = throws)
+            error_code& ec = throws) const
         {
-            return get_localities(
-                locality_ids, components::component_invalid, ec);
+            return get_localities(locality_ids,
+                to_int(components::component_enum_type::invalid), ec);
         }
 
         /// \brief Query for the number of all known localities.
@@ -347,7 +354,7 @@ namespace hpx { namespace agas {
         /// \param type       [in] The component type will be used to determine
         ///                   the set of prefixes having a registered factory
         ///                   for this component. The default value for this
-        ///                   parameter is \a components#component_invalid,
+        ///                   parameter is \a components#component_type#invalid,
         ///                   which will return prefixes of all localities.
         /// \param ec         [in,out] this represents the error status on exit,
         ///                   if this is pre-initialized to \a hpx#throws
@@ -356,25 +363,26 @@ namespace hpx { namespace agas {
         /// \note             As long as \a ec is not pre-initialized to
         ///                   \a hpx#throws this function doesn't
         ///                   throw but returns the result code using the
-        ///                   parameter \a ec. Otherwise it throws an instance
+        ///                   parameter \a ec. Otherwise, it throws an instance
         ///                   of hpx#exception.
-        lcos::future<std::uint32_t> get_num_localities_async(
-            components::component_type type =
-                components::component_invalid) const;
+        hpx::future<std::uint32_t> get_num_localities_async(
+            components::component_type type = to_int(
+                hpx::components::component_enum_type::invalid)) const;
 
         std::uint32_t get_num_localities(
             components::component_type type, error_code& ec = throws) const;
 
         std::uint32_t get_num_localities(error_code& ec = throws) const
         {
-            return get_num_localities(components::component_invalid, ec);
+            return get_num_localities(
+                to_int(hpx::components::component_enum_type::invalid), ec);
         }
 
-        lcos::future<std::uint32_t> get_num_overall_threads_async() const;
+        hpx::future<std::uint32_t> get_num_overall_threads_async() const;
 
         std::uint32_t get_num_overall_threads(error_code& ec = throws) const;
 
-        lcos::future<std::vector<std::uint32_t>> get_num_threads_async() const;
+        hpx::future<std::vector<std::uint32_t>> get_num_threads_async() const;
 
         std::vector<std::uint32_t> get_num_threads(
             error_code& ec = throws) const;
@@ -401,13 +409,13 @@ namespace hpx { namespace agas {
         ///                   parameter \a ec. Otherwise it throws an instance
         ///                   of hpx#exception.
         components::component_type get_component_id(
-            std::string const& name, error_code& ec = throws);
+            std::string const& name, error_code& ec = throws) const;
 
-        void iterate_types(
-            iterate_types_function_type const& f, error_code& ec = throws);
+        void iterate_types(iterate_types_function_type const& f,
+            error_code& ec = throws) const;
 
         std::string get_component_type_name(
-            components::component_type id, error_code& ec = throws);
+            components::component_type id, error_code& ec = throws) const;
 
         /// \brief Register a factory for a specific component type
         ///
@@ -437,14 +445,14 @@ namespace hpx { namespace agas {
         ///                   of hpx#exception.
         components::component_type register_factory(
             naming::gid_type const& locality_id, std::string const& name,
-            error_code& ec = throws)
+            error_code& ec = throws) const
         {
             return register_factory(
                 naming::get_locality_id_from_gid(locality_id), name, ec);
         }
 
         components::component_type register_factory(std::uint32_t locality_id,
-            std::string const& name, error_code& ec = throws);
+            std::string const& name, error_code& ec = throws) const;
 
         /// \brief Get unique range of freely assignable global ids.
         ///
@@ -617,7 +625,6 @@ namespace hpx { namespace agas {
         ///
         /// \note             This function will raise an error if the global
         ///                   reference count of the given gid is not zero!
-        ///                   TODO: confirm that this happens.
         bool unbind_local(naming::gid_type const& id, error_code& ec = throws)
         {
             return unbind_range_local(id, 1, ec);
@@ -657,7 +664,6 @@ namespace hpx { namespace agas {
         ///
         /// \note             This function will raise an error if the global
         ///                   reference count of the given gid is not zero!
-        ///                   TODO: confirm that this happens.
         bool unbind_local(naming::gid_type const& id, naming::address& addr,
             error_code& ec = throws)
         {
@@ -698,7 +704,6 @@ namespace hpx { namespace agas {
         ///
         /// \note             This function will raise an error if the global
         ///                   reference count of the given gid is not zero!
-        ///                   TODO: confirm that this happens.
         bool unbind_range_local(naming::gid_type const& lower_id,
             std::uint64_t count, error_code& ec = throws)
         {
@@ -755,7 +760,7 @@ namespace hpx { namespace agas {
         /// This function will test whether the given address refers to an object
         /// living on the locality of the caller.
         ///
-        /// \param addr       [in] The address to test.
+        /// \param id         [in] The address to test.
         /// \param ec         [in,out] this represents the error status on exit,
         ///                   if this is pre-initialized to \a hpx#throws
         ///                   the function will throw on error instead.
@@ -778,7 +783,13 @@ namespace hpx { namespace agas {
         bool is_local_address_cached(naming::gid_type const& id,
             naming::address& addr, error_code& ec = throws);
 
-        bool is_local_lva_encoded_address(std::uint64_t msb);
+        bool is_local_address_cached(naming::gid_type const& id,
+            naming::address& addr, std::pair<bool, components::pinned_ptr>& r,
+            hpx::move_only_function<std::pair<bool, components::pinned_ptr>(
+                naming::address const&)>&& f,
+            error_code& ec = throws);
+
+        bool is_local_lva_encoded_address(std::uint64_t msb) const;
 
         /// \brief Resolve a given global address (\a id) to its associated local
         ///        address.
@@ -825,7 +836,7 @@ namespace hpx { namespace agas {
             return resolve_full_local(id, addr, ec);
         }
 
-        bool resolve_local(naming::id_type const& id, naming::address& addr,
+        bool resolve_local(hpx::id_type const& id, naming::address& addr,
             error_code& ec = throws)
         {
             return resolve_local(id.get_gid(), addr, ec);
@@ -840,7 +851,7 @@ namespace hpx { namespace agas {
         }
 
         naming::address resolve_local(
-            naming::id_type const& id, error_code& ec = throws)
+            hpx::id_type const& id, error_code& ec = throws)
         {
             naming::address addr;
             resolve_local(id.get_gid(), addr, ec);
@@ -848,23 +859,25 @@ namespace hpx { namespace agas {
         }
 
         ///////////////////////////////////////////////////////////////////////////
-        hpx::future<naming::address> resolve_async(naming::gid_type const& id);
+        hpx::future_or_value<naming::address> resolve_async(
+            naming::gid_type const& id);
 
-        hpx::future<naming::address> resolve_async(naming::id_type const& id)
+        hpx::future_or_value<naming::address> resolve_async(
+            hpx::id_type const& id)
         {
             return resolve_async(id.get_gid());
         }
 
         ///////////////////////////////////////////////////////////////////////////
-        hpx::future<naming::id_type> get_colocation_id_async(
-            naming::id_type const& id);
+        hpx::future_or_value<id_type> get_colocation_id_async(
+            hpx::id_type const& id);
 
         ///////////////////////////////////////////////////////////////////////////
         bool resolve_full_local(naming::gid_type const& id,
             naming::address& addr, error_code& ec = throws);
 
-        bool resolve_full_local(naming::id_type const& id,
-            naming::address& addr, error_code& ec = throws)
+        bool resolve_full_local(hpx::id_type const& id, naming::address& addr,
+            error_code& ec = throws)
         {
             return resolve_full_local(id.get_gid(), addr, ec);
         }
@@ -878,18 +891,18 @@ namespace hpx { namespace agas {
         }
 
         naming::address resolve_full_local(
-            naming::id_type const& id, error_code& ec = throws)
+            hpx::id_type const& id, error_code& ec = throws)
         {
             naming::address addr;
             resolve_full_local(id.get_gid(), addr, ec);
             return addr;
         }
 
-        hpx::future<naming::address> resolve_full_async(
+        hpx::future_or_value<naming::address> resolve_full_async(
             naming::gid_type const& id);
 
-        hpx::future<naming::address> resolve_full_async(
-            naming::id_type const& id)
+        hpx::future_or_value<naming::address> resolve_full_async(
+            hpx::id_type const& id)
         {
             return resolve_full_async(id.get_gid());
         }
@@ -897,7 +910,7 @@ namespace hpx { namespace agas {
         bool resolve_cached(naming::gid_type const& id, naming::address& addr,
             error_code& ec = throws);
 
-        bool resolve_cached(naming::id_type const& id, naming::address& addr,
+        bool resolve_cached(hpx::id_type const& id, naming::address& addr,
             error_code& ec = throws)
         {
             return resolve_cached(id.get_gid(), addr, ec);
@@ -905,15 +918,14 @@ namespace hpx { namespace agas {
 
         ///////////////////////////////////////////////////////////////////////////
         // Bulk version.
-        // TODO: Add versions that take std::vector<id_type> for convenience.
         bool resolve_local(naming::gid_type const* gids, naming::address* addrs,
-            std::size_t size, boost::dynamic_bitset<>& locals,
+            std::size_t size, hpx::detail::dynamic_bitset<>& locals,
             error_code& ec = throws)
         {
             // Try the cache.
             if (caching_)
             {
-                bool all_resolved =
+                bool const all_resolved =
                     resolve_cached(gids, addrs, size, locals, ec);
                 if (ec)
                     return false;
@@ -926,11 +938,11 @@ namespace hpx { namespace agas {
 
         bool resolve_full_local(naming::gid_type const* gids,
             naming::address* addrs, std::size_t size,
-            boost::dynamic_bitset<>& locals, error_code& ec = throws);
+            hpx::detail::dynamic_bitset<>& locals, error_code& ec = throws);
 
         bool resolve_cached(naming::gid_type const* gids,
             naming::address* addrs, std::size_t size,
-            boost::dynamic_bitset<>& locals, error_code& ec = throws);
+            hpx::detail::dynamic_bitset<>& locals, error_code& ec = throws);
 
 #if defined(HPX_HAVE_NETWORKING)
         /// \brief Route the given parcel to the appropriate AGAS service instance
@@ -942,12 +954,13 @@ namespace hpx { namespace agas {
         ///
         /// \param p          [in] this is the parcel which has to be routed to the
         ///                   AGAS service instance managing the destination GID.
+        /// \param local_priority [in]
         ///
         /// \note             The route operation is asynchronous, thus it returns
         ///                   before the parcel has been delivered to its
         ///                   destination.
         void route(parcelset::parcel p,
-            util::function_nonser<void(
+            hpx::function<void(
                 std::error_code const&, parcelset::parcel const&)>&&,
             threads::thread_priority local_priority =
                 threads::thread_priority::default_);
@@ -955,10 +968,11 @@ namespace hpx { namespace agas {
 
         /// \brief Increment the global reference count for the given id
         ///
-        /// \param id         [in] The global address (id) for which the
+        /// \param gid        [in] The global address (id) for which the
         ///                   global reference count has to be incremented.
         /// \param credits    [in] The number of reference counts to add for
         ///                   the given id.
+        /// \param keep_alive [in] Id to keep alive (if valid)
         /// \param ec         [in,out] this represents the error status on exit,
         ///                   if this is pre-initialized to \a hpx#throws
         ///                   the function will throw on error instead.
@@ -970,14 +984,25 @@ namespace hpx { namespace agas {
         ///                   throw but returns the result code using the
         ///                   parameter \a ec. Otherwise it throws an instance
         ///                   of hpx#exception.
-        lcos::future<std::int64_t> incref_async(naming::gid_type const& gid,
+        hpx::future_or_value<std::int64_t> incref_async(
+            naming::gid_type const& gid, std::int64_t credits = 1,
+            hpx::id_type const& keep_alive = hpx::invalid_id);
+
+        ///  \cond NOINTERN
+        std::int64_t incref_async_helper(naming::gid_type const& gid,
             std::int64_t credits = 1,
-            naming::id_type const& keep_alive = naming::invalid_id);
+            hpx::id_type const& keep_alive = hpx::invalid_id);
+        /// \endcond
 
         std::int64_t incref(naming::gid_type const& gid,
             std::int64_t credits = 1, error_code& ec = throws)
         {
-            return incref_async(gid, credits).get(ec);
+            auto result = incref_async(gid, credits);
+            if (result.has_value())
+            {
+                return HPX_MOVE(result).get_value();
+            }
+            return result.get_future().get(ec);
         }
 
         /// \brief Decrement the global reference count for the given id
@@ -1013,11 +1038,11 @@ namespace hpx { namespace agas {
         /// Any error results in an exception thrown (or reported) from this
         /// function.
         ///
-        /// \param pattern    [in] pattern (poosibly using wildcards) to match
+        /// \param pattern    [in] pattern (possibly using wildcards) to match
         ///                   all existing entries against
         ///
         hpx::future<iterate_names_return_type> iterate_ids(
-            std::string const& pattern);
+            std::string const& pattern) const;
 
         /// \brief Register a global name with a global address (id)
         ///
@@ -1043,13 +1068,13 @@ namespace hpx { namespace agas {
         ///                   parameter \a ec. Otherwise it throws an instance
         ///                   of hpx#exception.
         bool register_name(std::string const& name, naming::gid_type const& id,
-            error_code& ec = throws);
+            error_code& ec = throws) const;
 
-        lcos::future<bool> register_name_async(
-            std::string const& name, naming::id_type const& id);
+        hpx::future<bool> register_name_async(
+            std::string const& name, hpx::id_type const& id) const;
 
-        bool register_name(std::string const& name, naming::id_type const& id,
-            error_code& ec = throws);
+        bool register_name(std::string const& name, hpx::id_type const& id,
+            error_code& ec = throws) const;
 
         /// \brief Unregister a global name (release any existing association)
         ///
@@ -1073,11 +1098,11 @@ namespace hpx { namespace agas {
         ///                   throw but returns the result code using the
         ///                   parameter \a ec. Otherwise it throws an instance
         ///                   of hpx#exception.
-        lcos::future<naming::id_type> unregister_name_async(
-            std::string const& name);
+        hpx::future<hpx::id_type> unregister_name_async(
+            std::string const& name) const;
 
-        naming::id_type unregister_name(
-            std::string const& name, error_code& ec = throws);
+        hpx::id_type unregister_name(
+            std::string const& name, error_code& ec = throws) const;
 
         /// \brief Query for the global address associated with a given global name.
         ///
@@ -1104,11 +1129,11 @@ namespace hpx { namespace agas {
         ///                   throw but returns the result code using the
         ///                   parameter \a ec. Otherwise it throws an instance
         ///                   of hpx#exception.
-        lcos::future<naming::id_type> resolve_name_async(
-            std::string const& name);
+        hpx::future<hpx::id_type> resolve_name_async(
+            std::string const& name) const;
 
-        naming::id_type resolve_name(
-            std::string const& name, error_code& ec = throws);
+        hpx::id_type resolve_name(
+            std::string const& name, error_code& ec = throws) const;
 
         /// \brief Install a listener for a given symbol namespace event.
         ///
@@ -1132,7 +1157,7 @@ namespace hpx { namespace agas {
         ///          global id is registered with the given name.
         ///
         future<hpx::id_type> on_symbol_namespace_event(
-            std::string const& name, bool call_for_past_events = false);
+            std::string const& name, bool call_for_past_events = false) const;
 
         /// \warning This function is for internal use only. It is dangerous and
         ///          may break your code if you use it.
@@ -1153,16 +1178,16 @@ namespace hpx { namespace agas {
         /// \warning This function is for internal use only. It is dangerous and
         ///          may break your code if you use it.
         bool get_cache_entry(naming::gid_type const& gid, gva& gva,
-            naming::gid_type& idbase, error_code& ec = throws);
+            naming::gid_type& idbase, error_code& ec = throws) const;
 
         /// \warning This function is for internal use only. It is dangerous and
         ///          may break your code if you use it.
         void remove_cache_entry(
-            naming::gid_type const& id, error_code& ec = throws);
+            naming::gid_type const& id, error_code& ec = throws) const;
 
         /// \warning This function is for internal use only. It is dangerous and
         ///          may break your code if you use it.
-        void clear_cache(error_code& ec = throws);
+        void clear_cache(error_code& ec = throws) const;
 
         // Disable refcnt caching during shutdown
         void start_shutdown(error_code& ec = throws);
@@ -1170,24 +1195,24 @@ namespace hpx { namespace agas {
         /// start/stop migration of an object
         ///
         /// \returns Current locality and address of the object to migrate
-        hpx::future<std::pair<naming::id_type, naming::address>>
-        begin_migration(naming::id_type const& id);
-        bool end_migration(naming::id_type const& id);
+        hpx::future<std::pair<hpx::id_type, naming::address>> begin_migration(
+            hpx::id_type const& id);
+        bool end_migration(hpx::id_type const& id);
 
         /// Maintain list of migrated objects
         std::pair<bool, components::pinned_ptr> was_object_migrated(
             naming::gid_type const& gid,
-            util::unique_function_nonser<components::pinned_ptr()>&& f);
+            hpx::move_only_function<components::pinned_ptr()>&& f);
 
         /// Mark the given object as being migrated (if the object is unpinned).
         /// Delay migration until the object is unpinned otherwise.
         hpx::future<void> mark_as_migrated(naming::gid_type const& gid,
-            util::unique_function_nonser<std::pair<bool, hpx::future<void>>()>&&
-                f,
+            hpx::move_only_function<std::pair<bool, hpx::future<void>>()>&& f,
             bool expect_to_be_marked_as_migrating);
 
         /// Remove the given object from the table of migrated objects
-        void unmark_as_migrated(naming::gid_type const& gid);
+        void unmark_as_migrated(
+            naming::gid_type const& gid_, hpx::move_only_function<void()>&& f);
 
         // Pre-cache locality endpoints in hosted locality namespace
         void pre_cache_endpoints(std::vector<parcelset::endpoints_type> const&);
