@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //  Copyright (c) 2011 Bryce Adelstein-Lelbach
-//  Copyright (c) 2012-2020 Hartmut Kaiser
+//  Copyright (c) 2012-2023 Hartmut Kaiser
 //  Copyright (c) 2016 Thomas Heller
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -21,9 +21,8 @@
 #include <hpx/components_base/server/fixed_component_base.hpp>
 #include <hpx/datastructures/tuple.hpp>
 #include <hpx/naming_base/id_type.hpp>
+#include <hpx/parcelset_base/traits/action_get_embedded_parcel.hpp>
 #include <hpx/synchronization/condition_variable.hpp>
-#include <hpx/traits/action_message_handler.hpp>
-#include <hpx/traits/action_serialization_filter.hpp>
 
 #include <atomic>
 #include <cstddef>
@@ -38,11 +37,11 @@
 
 #include <hpx/config/warnings_prefix.hpp>
 
-namespace hpx { namespace agas {
+namespace hpx::agas {
 
     HPX_EXPORT naming::gid_type bootstrap_primary_namespace_gid();
-    HPX_EXPORT naming::id_type bootstrap_primary_namespace_id();
-}}    // namespace hpx::agas
+    HPX_EXPORT hpx::id_type bootstrap_primary_namespace_id();
+}    // namespace hpx::agas
 
 /// \brief AGAS's primary namespace maps 128-bit global identifiers (GIDs) to
 /// resolved addresses.
@@ -78,9 +77,9 @@ namespace hpx { namespace agas {
 ///                  the underlying object. For all user-defined components,
 ///                  these bits contain a unique 88-bit number which is
 ///                  assigned sequentially for each locality. For
-///                  \a hpx#components#component_runtime_support the high 24
-///                  bits are zeroed and the low 64 bits hold the LVA of the
-///                  component.
+///                  \a hpx#components#component_enum_type#runtime_support
+///                  the high 24 bits are zeroed and the low 64 bits hold the
+///                  LVA of the component.
 ///
 /// The following address ranges are reserved. Some are either explicitly or
 /// implicitly protected by AGAS. The letter x represents a single-byte
@@ -106,7 +105,7 @@ namespace hpx { namespace agas {
 ///         locality.
 ///
 
-namespace hpx { namespace agas { namespace server {
+namespace hpx::agas::server {
 
     // Base name used to register the component
     static constexpr char const* const primary_namespace_service_name =
@@ -115,7 +114,7 @@ namespace hpx { namespace agas { namespace server {
     struct HPX_EXPORT primary_namespace
       : components::fixed_component_base<primary_namespace>
     {
-        using mutex_type = lcos::local::spinlock;
+        using mutex_type = hpx::spinlock;
         using base_type = components::fixed_component_base<primary_namespace>;
 
         using component_type = std::int32_t;
@@ -127,6 +126,11 @@ namespace hpx { namespace agas { namespace server {
         using resolved_type =
             hpx::tuple<naming::gid_type, gva, naming::gid_type>;
 
+        mutex_type& mutex()
+        {
+            return mutex_;
+        }
+
     private:
         // REVIEW: Separate mutexes might reduce contention here. This has to be
         // investigated carefully.
@@ -144,10 +148,9 @@ namespace hpx { namespace agas { namespace server {
         naming::gid_type locality_;    // our locality id
         migration_table_type migrating_objects_;
 
-        struct update_time_on_exit;
-
     public:
-        // data structure holding all counters for the omponent_namespace component
+        // data structure holding all counters for the component_namespace
+        // component
         struct counter_data
         {
         public:
@@ -211,16 +214,22 @@ namespace hpx { namespace agas { namespace server {
             api_counter_data route_;    // primary_ns_
 #endif
 
-            api_counter_data bind_gid_;       // primary_ns_bind_gid
-            api_counter_data resolve_gid_;    // primary_ns_resolve_gid
-            api_counter_data unbind_gid_;     // primary_ns_unbind_gid
-            api_counter_data
-                increment_credit_;    // primary_ns_increment_credit
-            api_counter_data
-                decrement_credit_;                // primary_ns_decrement_credit
-            api_counter_data allocate_;           // primary_ns_allocate
-            api_counter_data begin_migration_;    // primary_ns_begin_migration
-            api_counter_data end_migration_;      // primary_ns_end_migration
+            // primary_ns_bind_gid
+            api_counter_data bind_gid_;
+            // primary_ns_resolve_gid
+            api_counter_data resolve_gid_;
+            // primary_ns_unbind_gid
+            api_counter_data unbind_gid_;
+            // primary_ns_increment_credit
+            api_counter_data increment_credit_;
+            // primary_ns_decrement_credit
+            api_counter_data decrement_credit_;
+            // primary_ns_allocate
+            api_counter_data allocate_;
+            // primary_ns_begin_migration
+            api_counter_data begin_migration_;
+            // primary_ns_end_migration
+            api_counter_data end_migration_;
         };
 
         counter_data counter_data_;
@@ -232,9 +241,10 @@ namespace hpx { namespace agas { namespace server {
         void dump_refcnt_matches(refcnt_table_type::iterator lower_it,
             refcnt_table_type::iterator upper_it, naming::gid_type const& lower,
             naming::gid_type const& upper, std::unique_lock<mutex_type>& l,
-            const char* func_name);
+            char const* func_name);
 #endif
 
+    public:
         // helper function
         void wait_for_migration_locked(std::unique_lock<mutex_type>& l,
             naming::gid_type const& id, error_code& ec);
@@ -242,14 +252,12 @@ namespace hpx { namespace agas { namespace server {
     public:
         primary_namespace()
           : base_type(agas::primary_ns_msb, agas::primary_ns_lsb)
-          , mutex_()
-          , instance_name_()
           , next_id_(naming::invalid_gid)
           , locality_(naming::invalid_gid)
         {
         }
 
-        void finalize();
+        void finalize() const;
 
         void set_local_locality(naming::gid_type const& g)
         {
@@ -261,7 +269,7 @@ namespace hpx { namespace agas { namespace server {
             std::uint32_t locality_id = naming::invalid_locality_id,
             error_code& ec = throws);
 
-        void unregister_server_instance(error_code& ec = throws);
+        void unregister_server_instance(error_code& ec = throws) const;
 
 #if defined(HPX_HAVE_NETWORKING)
         void route(parcelset::parcel&& p);
@@ -271,13 +279,13 @@ namespace hpx { namespace agas { namespace server {
             naming::gid_type const& locality);
 
         // API
-        std::pair<naming::id_type, naming::address> begin_migration(
+        std::pair<hpx::id_type, naming::address> begin_migration(
             naming::gid_type id);
         bool end_migration(naming::gid_type const& id);
 
         resolved_type resolve_gid(naming::gid_type const& id);
 
-        naming::id_type colocate(naming::gid_type const& id);
+        hpx::id_type colocate(naming::gid_type const& id);
 
         naming::address unbind_gid(std::uint64_t count, naming::gid_type id);
 
@@ -291,18 +299,22 @@ namespace hpx { namespace agas { namespace server {
         std::pair<naming::gid_type, naming::gid_type> allocate(
             std::uint64_t count);
 
-    private:
         resolved_type resolve_gid_locked(std::unique_lock<mutex_type>& l,
             naming::gid_type const& gid, error_code& ec);
 
+    private:
+        resolved_type resolve_gid_locked_non_local(
+            std::unique_lock<mutex_type>& l, naming::gid_type const& gid,
+            error_code& ec);
+
         void increment(naming::gid_type const& lower,
-            naming::gid_type const& upper, std::int64_t& credits,
+            naming::gid_type const& upper, std::int64_t const& credits,
             error_code& ec);
 
         ///////////////////////////////////////////////////////////////////////////
         struct free_entry
         {
-            free_entry(agas::gva gva, naming::gid_type const& gid,
+            free_entry(agas::gva const& gva, naming::gid_type const& gid,
                 naming::gid_type const& loc)
               : gva_(gva)
               , gid_(gid)
@@ -329,35 +341,25 @@ namespace hpx { namespace agas { namespace server {
             naming::gid_type const& lower, naming::gid_type const& upper,
             std::int64_t credits, error_code& ec);
 
-        void free_components_sync(free_entry_list_type& free_list,
+        void free_components_sync(free_entry_list_type const& free_list,
             naming::gid_type const& lower, naming::gid_type const& upper,
-            error_code& ec);
+            error_code& ec) const;
 
     public:
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, allocate);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, bind_gid);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, colocate);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, begin_migration);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, end_migration);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, decrement_credit);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, increment_credit);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, resolve_gid);
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, unbind_gid);
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, allocate)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, bind_gid)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, colocate)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, begin_migration)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, end_migration)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, decrement_credit)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, increment_credit)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, resolve_gid)
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, unbind_gid)
 #if defined(HPX_HAVE_NETWORKING)
-        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, route);
-#endif
-
-#if defined(HPX_HAVE_NETWORKING)
-        static parcelset::policies::message_handler* get_message_handler(
-            parcelset::parcelhandler* ph, parcelset::locality const& loc,
-            parcelset::parcel const& p);
-
-        static serialization::binary_filter* get_serialization_filter(
-            parcelset::parcel const& p);
+        HPX_DEFINE_COMPONENT_ACTION(primary_namespace, route)
 #endif
     };
-
-}}}    // namespace hpx::agas::server
+}    // namespace hpx::agas::server
 
 HPX_ACTION_USES_MEDIUM_STACK(
     hpx::agas::server::primary_namespace::allocate_action)
@@ -435,8 +437,7 @@ HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
 typedef hpx::tuple<hpx::naming::gid_type, hpx::agas::gva, hpx::naming::gid_type>
     gva_tuple_type;
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(gva_tuple_type, gva_tuple)
-typedef std::pair<hpx::naming::id_type, hpx::naming::address>
-    std_pair_address_id_type;
+typedef std::pair<hpx::id_type, hpx::naming::address> std_pair_address_id_type;
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
     std_pair_address_id_type, std_pair_address_id_type)
 typedef std::pair<hpx::naming::gid_type, hpx::naming::gid_type>
@@ -447,32 +448,19 @@ HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
     std::vector<std::int64_t>, vector_std_int64_type)
 
 #if !defined(HPX_COMPUTE_DEVICE_CODE) && defined(HPX_HAVE_NETWORKING)
-namespace hpx { namespace traits {
-
-    // Parcel routing forwards the message handler request to the routed action
-    template <>
-    struct action_message_handler<agas::server::primary_namespace::route_action>
+// Parcel routing forwards the binary filter request to the routed action
+template <>
+struct hpx::traits::action_get_embedded_parcel<
+    hpx::agas::server::primary_namespace::route_action>
+{
+    static hpx::optional<hpx::parcelset::parcel> call(
+        hpx::actions::transfer_base_action<
+            hpx::agas::server::primary_namespace::route_action> const& act)
     {
-        static parcelset::policies::message_handler* call(
-            parcelset::parcelhandler* ph, parcelset::locality const& loc,
-            parcelset::parcel const& p)
-        {
-            return agas::server::primary_namespace::get_message_handler(
-                ph, loc, p);
-        }
-    };
-
-    // Parcel routing forwards the binary filter request to the routed action
-    template <>
-    struct action_serialization_filter<
-        agas::server::primary_namespace::route_action>
-    {
-        static serialization::binary_filter* call(parcelset::parcel const& p)
-        {
-            return agas::server::primary_namespace::get_serialization_filter(p);
-        }
-    };
-}}    // namespace hpx::traits
+        auto p = hpx::actions::get<0>(act);
+        return hpx::optional<hpx::parcelset::parcel>(HPX_MOVE(p));
+    }
+};    // namespace hpx::traits
 #endif
 
 #include <hpx/config/warnings_suffix.hpp>
