@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //  Copyright (c) 2011 Bryce Adelstein-Lelbach
-//  Copyright (c) 2012-2017 Hartmut Kaiser
+//  Copyright (c) 2012-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -28,21 +28,21 @@
 
 // TODO: Remove the use of the name "prefix"
 
-namespace hpx { namespace agas {
+namespace hpx::agas {
 
     naming::gid_type bootstrap_component_namespace_gid()
     {
         return naming::gid_type(agas::component_ns_msb, agas::component_ns_lsb);
     }
 
-    naming::id_type bootstrap_component_namespace_id()
+    hpx::id_type bootstrap_component_namespace_id()
     {
-        return naming::id_type(agas::component_ns_msb, agas::component_ns_lsb,
-            naming::id_type::unmanaged);
+        return hpx::id_type(agas::component_ns_msb, agas::component_ns_lsb,
+            hpx::id_type::management_type::unmanaged);
     }
-}}    // namespace hpx::agas
+}    // namespace hpx::agas
 
-namespace hpx { namespace agas { namespace server {
+namespace hpx::agas::server {
 
     void component_namespace::register_server_instance(
         char const* servicename, error_code& ec)
@@ -68,7 +68,7 @@ namespace hpx { namespace agas { namespace server {
     {
         if (!instance_name_.empty())
         {
-            error_code ec(lightweight);
+            error_code ec(throwmode::lightweight);
             agas::unregister_name(launch::sync, instance_name_, ec);
         }
     }
@@ -91,16 +91,14 @@ namespace hpx { namespace agas { namespace server {
         if (component_ids_.find(key) == cend)
         {
             if (HPX_UNLIKELY(!util::insert_checked(
-                    component_ids_.insert(std::make_pair(key, type_counter)),
-                    cit)))
+                    component_ids_.emplace(key, type_counter), cit)))
             {
                 l.unlock();
 
-                HPX_THROW_EXCEPTION(lock_error,
+                HPX_THROW_EXCEPTION(hpx::error::lock_error,
                     "component_namespace::bind_prefix",
                     "component id table insertion failed due to a locking "
                     "error or memory corruption");
-                return components::component_invalid;
             }
 
             // If the insertion succeeded, we need to increment the type
@@ -121,12 +119,11 @@ namespace hpx { namespace agas { namespace server {
                 // Duplicate type registration for this locality.
                 l.unlock();
 
-                HPX_THROW_EXCEPTION(duplicate_component_id,
+                HPX_THROW_EXCEPTION(hpx::error::duplicate_component_id,
                     "component_namespace::bind_prefix",
                     "component id is already registered for the given "
                     "locality, key({1}), prefix({2}), ctype({3})",
                     key, prefix, cit->second);
-                return components::component_invalid;
             }
 
             fit->second.insert(prefix);
@@ -147,15 +144,14 @@ namespace hpx { namespace agas { namespace server {
         // prevent a copy, though most compilers should be able to optimize
         // this without our help.
         if (HPX_UNLIKELY(!util::insert_checked(
-                factories_.insert(std::make_pair(cit->second, prefixes_type())),
-                fit)))
+                factories_.emplace(cit->second, prefixes_type()), fit)))
         {
             l.unlock();
 
-            HPX_THROW_EXCEPTION(lock_error, "component_namespace::bind_prefix",
+            HPX_THROW_EXCEPTION(hpx::error::lock_error,
+                "component_namespace::bind_prefix",
                 "factory table insertion failed due to a locking "
                 "error or memory corruption");
-            return components::component_invalid;
         }
 
         fit->second.insert(prefix);
@@ -185,16 +181,14 @@ namespace hpx { namespace agas { namespace server {
         if (it == end)
         {
             if (HPX_UNLIKELY(!util::insert_checked(
-                    component_ids_.insert(std::make_pair(key, type_counter)),
-                    it)))
+                    component_ids_.emplace(key, type_counter), it)))
             {
                 l.unlock();
 
-                HPX_THROW_EXCEPTION(lock_error,
+                HPX_THROW_EXCEPTION(hpx::error::lock_error,
                     "component_namespace::bind_name",
                     "component id table insertion failed due to a locking "
                     "error or memory corruption");
-                return components::component_invalid;
             }
 
             // If the insertion succeeded, we need to increment the type
@@ -245,8 +239,8 @@ namespace hpx { namespace agas { namespace server {
             std::vector<std::uint32_t> p;
             p.assign(prefixes.cbegin(), prefixes.cend());
 
-            LAGAS_(info).format(
-                "component_namespace::resolve_id, key({1}), localities({2})",
+            LAGAS_(info).format("component_namespace::resolve_id, "
+                                "key({1}), localities({2})",
                 key, prefixes.size());
 
             return p;
@@ -267,8 +261,8 @@ namespace hpx { namespace agas { namespace server {
         // REVIEW: Should this be an error?
         if (it == component_ids_.end())
         {
-            LAGAS_(info).format(
-                "component_namespace::unbind, key({1}), response(no_success)",
+            LAGAS_(info).format("component_namespace::unbind, key({1}), "
+                                "response(no_success)",
                 key);
 
             return false;
@@ -314,7 +308,7 @@ namespace hpx { namespace agas { namespace server {
         component_namespace::component_id_table_type const& m,
         components::component_type t)
     {
-        if (t < components::component_last)
+        if (t < to_int(components::component_enum_type::last))
             return components::get_component_type_name(t);
 
         for (auto const& c : m)
@@ -337,7 +331,7 @@ namespace hpx { namespace agas { namespace server {
 
         std::string result;
 
-        if (t == components::component_invalid)
+        if (t == to_int(hpx::components::component_enum_type::invalid))
         {
             result = "component_invalid";
         }
@@ -356,9 +350,9 @@ namespace hpx { namespace agas { namespace server {
 
         if (result.empty())
         {
-            LAGAS_(info).format(
-                "component_namespace::get_component_typename, key({1}/{2}), "
-                "response(no_success)",
+            LAGAS_(info).format("component_namespace::get_component_"
+                                "typename, key({1}/{2}), "
+                                "response(no_success)",
                 int(components::get_derived_type(t)),
                 int(components::get_base_type(t)));
 
@@ -591,5 +585,4 @@ namespace hpx { namespace agas { namespace server {
             ++num_localities_.count_;
         }
     }
-
-}}}    // namespace hpx::agas::server
+}    // namespace hpx::agas::server
