@@ -1,5 +1,5 @@
 //  Copyright (c) 2011 Bryce Adelstein-Lelbach
-//  Copyright (c) 2012-2021 Hartmut Kaiser
+//  Copyright (c) 2012-2024 Hartmut Kaiser
 //  Copyright (c) 2016 Thomas Heller
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -7,24 +7,21 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <hpx/config.hpp>
+#include <hpx/agas_base/route.hpp>
 #include <hpx/agas_base/server/primary_namespace.hpp>
 #include <hpx/assert.hpp>
-#include <hpx/async_combinators/wait_all.hpp>
-#include <hpx/async_distributed/applier/apply.hpp>
-#include <hpx/async_distributed/continuation.hpp>
 #include <hpx/components_base/agas_interface.hpp>
-#include <hpx/execution_base/register_locks.hpp>
 #include <hpx/format.hpp>
+#include <hpx/lock_registration/detail/register_locks.hpp>
 #include <hpx/modules/async_distributed.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/modules/logging.hpp>
-#include <hpx/thread_support/assert_owns_lock.hpp>
 #include <hpx/timing/scoped_timer.hpp>
+#include <hpx/type_support/assert_owns_lock.hpp>
 #include <hpx/util/get_and_reset_value.hpp>
 #include <hpx/util/insert_checked.hpp>
 
 #include <atomic>
-#include <cstddef>
 #include <cstdint>
 #include <list>
 #include <memory>
@@ -34,21 +31,21 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace agas {
+namespace hpx::agas {
 
     naming::gid_type bootstrap_primary_namespace_gid()
     {
         return naming::gid_type(agas::primary_ns_msb, agas::primary_ns_lsb);
     }
 
-    naming::id_type bootstrap_primary_namespace_id()
+    hpx::id_type bootstrap_primary_namespace_id()
     {
-        return naming::id_type(agas::primary_ns_msb, agas::primary_ns_lsb,
-            naming::id_type::unmanaged);
+        return {agas::primary_ns_msb, agas::primary_ns_lsb,
+            hpx::id_type::management_type::unmanaged};
     }
-}}    // namespace hpx::agas
+}    // namespace hpx::agas
 
-namespace hpx { namespace agas { namespace server {
+namespace hpx::agas::server {
 
     void primary_namespace::register_server_instance(
         char const* servicename, std::uint32_t locality_id, error_code& ec)
@@ -70,54 +67,24 @@ namespace hpx { namespace agas { namespace server {
             launch::sync, instance_name_, get_unmanaged_id().get_gid(), ec);
     }
 
-    void primary_namespace::unregister_server_instance(error_code& ec)
+    void primary_namespace::unregister_server_instance(error_code& ec) const
     {
         agas::unregister_name(launch::sync, instance_name_, ec);
         this->base_type::finalize();
     }
 
-    void primary_namespace::finalize()
+    void primary_namespace::finalize() const
     {
         if (!instance_name_.empty())
         {
-            error_code ec(lightweight);
+            error_code ec(throwmode::lightweight);
             agas::unregister_name(launch::sync, instance_name_, ec);
         }
     }
 
-#if defined(HPX_HAVE_NETWORKING) && !defined(HPX_COMPUTE_DEVICE_CODE)
-    // Parcel routing forwards the message handler request to the routed action
-    parcelset::policies::message_handler*
-    primary_namespace::get_message_handler(parcelset::parcelhandler* ph,
-        parcelset::locality const& loc, parcelset::parcel const& p)
-    {
-        typedef hpx::actions::transfer_action<
-            server::primary_namespace::route_action>
-            action_type;
-
-        action_type* act = static_cast<action_type*>(p.get_action());
-
-        parcelset::parcel const& routed_p = hpx::actions::get<0>(*act);
-        return routed_p.get_message_handler(ph, loc);
-    }
-
-    serialization::binary_filter* primary_namespace::get_serialization_filter(
-        parcelset::parcel const& p)
-    {
-        typedef hpx::actions::transfer_action<
-            server::primary_namespace::route_action>
-            action_type;
-
-        action_type* act = static_cast<action_type*>(p.get_action());
-
-        parcelset::parcel const& routed_p = hpx::actions::get<0>(*act);
-        return routed_p.get_serialization_filter();
-    }
-#endif
-
     // start migration of the given object
-    std::pair<naming::id_type, naming::address>
-    primary_namespace::begin_migration(naming::gid_type id)
+    std::pair<hpx::id_type, naming::address> primary_namespace::begin_migration(
+        naming::gid_type id)
     {
         util::scoped_timer<std::atomic<std::int64_t>> update(
             counter_data_.begin_migration_.time_,
@@ -128,7 +95,7 @@ namespace hpx { namespace agas { namespace server {
         std::unique_lock<mutex_type> l(mutex_);
 
         wait_for_migration_locked(l, id, hpx::throws);
-        resolved_type r = resolve_gid_locked(l, id, hpx::throws);
+        resolved_type r = resolve_gid_locked_non_local(l, id, hpx::throws);
         if (get<0>(r) == naming::invalid_gid)
         {
             l.unlock();
@@ -137,13 +104,13 @@ namespace hpx { namespace agas { namespace server {
                                 "response(no_success)",
                 id);
 
-            return std::make_pair(naming::invalid_id, naming::address());
+            return std::make_pair(hpx::invalid_id, naming::address());
         }
 
-        migration_table_type::iterator it = migrating_objects_.find(id);
+        auto it = migrating_objects_.find(id);
         if (it == migrating_objects_.end())
         {
-            std::pair<migration_table_type::iterator, bool> p =
+            std::pair<migration_table_type::iterator, bool> const p =
                 migrating_objects_.emplace(std::piecewise_construct,
                     std::forward_as_tuple(id), std::forward_as_tuple());
             HPX_ASSERT(p.second);
@@ -151,7 +118,7 @@ namespace hpx { namespace agas { namespace server {
         }
         else
         {
-            HPX_ASSERT(hpx::get<0>(it->second) == false);
+            HPX_ASSERT(!hpx::get<0>(it->second));
         }
 
         // flag this id as being migrated
@@ -159,11 +126,17 @@ namespace hpx { namespace agas { namespace server {
 
         gva const& g(hpx::get<1>(r));
         naming::address addr(g.prefix, g.type, g.lva());
-        naming::id_type loc(hpx::get<2>(r), id_type::unmanaged);
+        hpx::id_type loc(
+            hpx::get<2>(r), hpx::id_type::management_type::unmanaged);
         return std::make_pair(loc, addr);
     }
 
     // migration of the given object is complete
+    // 26115: Failing to release lock 'this->mutex_' in function
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26115)
+#endif
     bool primary_namespace::end_migration(naming::gid_type const& id)
     {
         util::scoped_timer<std::atomic<std::int64_t>> update(
@@ -175,14 +148,14 @@ namespace hpx { namespace agas { namespace server {
 
         using hpx::get;
 
-        migration_table_type::iterator it = migrating_objects_.find(id);
-        if (it != migrating_objects_.end())
+        if (auto const it = migrating_objects_.find(id);
+            it != migrating_objects_.end())
         {
             // flag this id as not being migrated anymore
             get<0>(it->second) = false;
             if (get<1>(it->second) != 0)
             {
-                get<2>(it->second).notify_all(std::move(l), hpx::throws);
+                get<2>(it->second).notify_all(HPX_MOVE(l), hpx::throws);
             }
             else
             {
@@ -192,6 +165,9 @@ namespace hpx { namespace agas { namespace server {
 
         return true;
     }
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
 
     // wait if given object is currently being migrated
     void primary_namespace::wait_for_migration_locked(
@@ -202,8 +178,8 @@ namespace hpx { namespace agas { namespace server {
 
         using hpx::get;
 
-        migration_table_type::iterator it = migrating_objects_.find(id);
-        if (it != migrating_objects_.end())
+        if (auto const it = migrating_objects_.find(id);
+            it != migrating_objects_.end())
         {
             if (get<0>(it->second))
             {
@@ -211,8 +187,10 @@ namespace hpx { namespace agas { namespace server {
 
                 get<2>(it->second).wait(l, ec);
 
-                if (--get<1>(it->second) == 0)
+                if (--get<1>(it->second) == 0)    //-V516
+                {
                     migrating_objects_.erase(it);
+                }
             }
             else
             {
@@ -232,15 +210,15 @@ namespace hpx { namespace agas { namespace server {
         counter_data_.increment_bind_gid_count();
         using hpx::get;
 
-        naming::gid_type gid = id;
+        naming::gid_type const gid = id;
         naming::detail::strip_internal_bits_from_gid(id);
 
         std::unique_lock<mutex_type> l(mutex_);
 
-        gva_table_type::iterator it = gvas_.lower_bound(id),
-                                 begin = gvas_.begin(), end = gvas_.end();
+        auto const begin = gvas_.begin();
+        auto const end = gvas_.end();
 
-        if (it != end)
+        if (auto it = gvas_.lower_bound(id); it != end)
         {
             // If we got an exact match, this is a request to update an existing
             // binding (e.g. move semantics).
@@ -252,11 +230,9 @@ namespace hpx { namespace agas { namespace server {
                 {
                     l.unlock();
 
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "primary_namespace::bind_gid",
                         "cannot rebind gids for non-migratable objects");
-
-                    return false;
                 }
 
                 gva& gaddr = it->second.first;
@@ -269,16 +245,18 @@ namespace hpx { namespace agas { namespace server {
                     // REVIEW: Is this the right error code to use?
                     l.unlock();
 
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "primary_namespace::bind_gid",
                         "cannot change block size of existing binding");
                 }
 
-                if (HPX_UNLIKELY(components::component_invalid == g.type))
+                if (HPX_UNLIKELY(
+                        to_int(hpx::components::component_enum_type::invalid) ==
+                        g.type))
                 {
                     l.unlock();
 
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "primary_namespace::bind_gid",
                         "attempt to update a GVA with an invalid type, "
                         "gid({1}), gva({2}), locality({3})",
@@ -289,7 +267,7 @@ namespace hpx { namespace agas { namespace server {
                 {
                     l.unlock();
 
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "primary_namespace::bind_gid",
                         "attempt to update a GVA with an invalid "
                         "locality id, "
@@ -326,7 +304,7 @@ namespace hpx { namespace agas { namespace server {
                     // REVIEW: Is this the right error code to use?
                     l.unlock();
 
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "primary_namespace::bind_gid",
                         "the new GID is contained in an existing range");
                 }
@@ -343,7 +321,7 @@ namespace hpx { namespace agas { namespace server {
                 // REVIEW: Is this the right error code to use?
                 l.unlock();
 
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "primary_namespace::bind_gid",
                     "the new GID is contained in an existing range");
             }
@@ -361,22 +339,25 @@ namespace hpx { namespace agas { namespace server {
             return true;
         }
 
-        naming::gid_type upper_bound(id + (g.count - 1));
+        naming::gid_type const upper_bound(id + (g.count - 1));
 
         if (HPX_UNLIKELY(id.get_msb() != upper_bound.get_msb()))
         {
             l.unlock();
 
-            HPX_THROW_EXCEPTION(internal_server_error,
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
                 "primary_namespace::bind_gid",
                 "MSBs of lower and upper range bound do not match");
         }
 
-        if (HPX_UNLIKELY(components::component_invalid == g.type))
+        if (HPX_UNLIKELY(
+                to_int(hpx::components::component_enum_type::invalid) ==
+                g.type))
         {
             l.unlock();
 
-            HPX_THROW_EXCEPTION(bad_parameter, "primary_namespace::bind_gid",
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
+                "primary_namespace::bind_gid",
                 "attempt to insert a GVA with an invalid type, "
                 "gid({1}), gva({2}), locality({3})",
                 id, g, locality);
@@ -384,11 +365,12 @@ namespace hpx { namespace agas { namespace server {
 
         // Insert a GID -> GVA entry into the GVA table.
         if (HPX_UNLIKELY(!util::insert_checked(
-                gvas_.insert(std::make_pair(id, std::make_pair(g, locality))))))
+                gvas_.emplace(id, std::make_pair(g, locality)))))
         {
             l.unlock();
 
-            HPX_THROW_EXCEPTION(lock_error, "primary_namespace::bind_gid",
+            HPX_THROW_EXCEPTION(hpx::error::lock_error,
+                "primary_namespace::bind_gid",
                 "GVA table insertion failed due to a locking error or "
                 "memory corruption, gid({1}), gva({2}), locality({3})",
                 id, g, locality);
@@ -403,6 +385,17 @@ namespace hpx { namespace agas { namespace server {
         return true;
     }    // }}}
 
+    inline primary_namespace::resolved_type resolve_local_id(
+        naming::gid_type const& id) noexcept
+    {
+        naming::gid_type locality = naming::get_locality_from_gid(id);
+        gva addr(locality,
+            static_cast<std::uint32_t>(
+                naming::detail::get_component_type_from_gid(id.get_msb())),
+            1, id.get_lsb());
+        return primary_namespace::resolved_type(id, addr, locality);
+    }
+
     primary_namespace::resolved_type primary_namespace::resolve_gid(
         naming::gid_type const& id)
     {    // {{{ resolve_gid implementation
@@ -414,6 +407,13 @@ namespace hpx { namespace agas { namespace server {
 
         resolved_type r;
 
+        // handle (non-migratable) components located on this locality first
+        if (naming::refers_to_local_lva(id) &&
+            !naming::refers_to_virtual_memory(id))
+        {
+            r = resolve_local_id(id);
+        }
+        else
         {
             std::unique_lock<mutex_type> l(mutex_);
 
@@ -424,7 +424,7 @@ namespace hpx { namespace agas { namespace server {
             }
 
             // now, resolve the id
-            r = resolve_gid_locked(l, id, hpx::throws);
+            r = resolve_gid_locked_non_local(l, id, hpx::throws);
         }
 
         if (get<0>(r) == naming::invalid_gid)
@@ -445,10 +445,10 @@ namespace hpx { namespace agas { namespace server {
         return r;
     }    // }}}
 
-    naming::id_type primary_namespace::colocate(naming::gid_type const& id)
+    hpx::id_type primary_namespace::colocate(naming::gid_type const& id)
     {
-        return naming::id_type(
-            hpx::get<2>(resolve_gid(id)), naming::id_type::unmanaged);
+        return {hpx::get<2>(resolve_gid(id)),
+            hpx::id_type::management_type::unmanaged};
     }
 
     naming::address primary_namespace::unbind_gid(
@@ -463,19 +463,18 @@ namespace hpx { namespace agas { namespace server {
 
         std::unique_lock<mutex_type> l(mutex_);
 
-        gva_table_type::iterator it = gvas_.find(id), end = gvas_.end();
-
-        if (it != end)
+        auto const it = gvas_.find(id);
+        if (auto const end = gvas_.end(); it != end)
         {
             if (HPX_UNLIKELY(it->second.first.count != count))
             {
                 l.unlock();
 
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "primary_namespace::unbind_gid", "block sizes must match");
             }
 
-            gva_table_data_type data = it->second;
+            gva_table_data_type const data = it->second;
 
             gvas_.erase(it);
 
@@ -486,24 +485,25 @@ namespace hpx { namespace agas { namespace server {
                 id, count, data.first, data.second);
 
             gva g = data.first;
-            return naming::address(g.prefix, g.type, g.lva());
+            return {g.prefix, g.type, g.lva()};
         }
 
         // non-migratable gids are not bound
         if (naming::refers_to_local_lva(id) &&
             !naming::refers_to_virtual_memory(id))
         {
-            naming::gid_type locality = naming::get_locality_from_gid(id);
+            naming::gid_type const locality = naming::get_locality_from_gid(id);
             gva g(locality,
-                naming::detail::get_component_type_from_gid(id.get_msb()), 0,
-                id.get_lsb());
+                static_cast<std::uint32_t>(
+                    naming::detail::get_component_type_from_gid(id.get_msb())),
+                0, id.get_lsb());
 
             LAGAS_(info).format(
                 "primary_namespace::unbind_gid, gid({1}), count({2}), "
                 "gva({3}), locality({4})",
                 id, count, g, g.prefix);
 
-            return naming::address(g.prefix, g.type, g.lva());
+            return {g.prefix, g.type, g.lva()};
         }
 
         l.unlock();
@@ -513,7 +513,7 @@ namespace hpx { namespace agas { namespace server {
             "response(no_success)",
             id, count);
 
-        return naming::address();
+        return {};
     }    // }}}
 
     std::int64_t primary_namespace::increment_credit(
@@ -534,17 +534,12 @@ namespace hpx { namespace agas { namespace server {
         if (credits > 0)
         {
             increment(lower, upper, credits, hpx::throws);
-            return 0;
-        }
-        else
-        {
-            HPX_THROW_EXCEPTION(bad_parameter,
-                "primary_namespace::increment_credit",
-                "invalid credit count of {1}", credits);
-            return 0;
+            return credits;
         }
 
-        return credits;
+        HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
+            "primary_namespace::increment_credit",
+            "invalid credit count of {1}", credits);
     }
 
     std::vector<std::int64_t> primary_namespace::decrement_credit(
@@ -581,7 +576,7 @@ namespace hpx { namespace agas { namespace server {
             }
             else
             {
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "primary_namespace::decrement_credit",
                     "invalid credit count of {1}", credits);
             }
@@ -598,11 +593,8 @@ namespace hpx { namespace agas { namespace server {
             counter_data_.allocate_.time_, counter_data_.allocate_.enabled_);
         counter_data_.increment_allocate_count();
 
-        std::uint64_t const real_count = (count) ? (count - 1) : (0);
-
         // Just return the prefix
-        // REVIEW: Should this be an error?
-        if (0 == count)
+        if (count == 0)
         {
             LAGAS_(info).format(
                 "primary_namespace::allocate, count({1}), lower({1}), "
@@ -613,6 +605,8 @@ namespace hpx { namespace agas { namespace server {
             return std::make_pair(next_id_, next_id_);
         }
 
+        std::uint64_t const real_count = count - 1;
+
         // Compute the new allocation.
         naming::gid_type lower(next_id_ + 1);
         naming::gid_type upper(lower + real_count);
@@ -626,13 +620,13 @@ namespace hpx { namespace agas { namespace server {
                     (lower.get_msb() & naming::gid_type::virtual_memory_mask) ==
                     naming::gid_type::virtual_memory_mask))
             {
-                HPX_THROW_EXCEPTION(internal_server_error,
+                HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
                     "locality_namespace::allocate",
                     "primary namespace has been exhausted");
             }
 
             // Otherwise, correct
-            lower = naming::gid_type(upper.get_msb(), 0);
+            lower = naming::gid_type(upper.get_msb(), nullptr);
             upper = lower + real_count;
         }
 
@@ -641,9 +635,9 @@ namespace hpx { namespace agas { namespace server {
 
         // Set the initial credit count.
         naming::detail::set_credit_for_gid(
-            lower, std::int64_t(HPX_GLOBALCREDIT_INITIAL));
+            lower, static_cast<std::int64_t>(HPX_GLOBALCREDIT_INITIAL));
         naming::detail::set_credit_for_gid(
-            upper, std::int64_t(HPX_GLOBALCREDIT_INITIAL));
+            upper, static_cast<std::int64_t>(HPX_GLOBALCREDIT_INITIAL));
 
         LAGAS_(info).format(
             "primary_namespace::allocate, count({1}), lower({2}), upper({3}), "
@@ -658,8 +652,9 @@ namespace hpx { namespace agas { namespace server {
         refcnt_table_type::iterator lower_it,
         refcnt_table_type::iterator upper_it, naming::gid_type const& lower,
         naming::gid_type const& upper, std::unique_lock<mutex_type>& l,
-        const char* func_name)
-    {    // dump_refcnt_matches implementation
+        char const* func_name)
+    {
+        // dump_refcnt_matches implementation
         HPX_ASSERT(l.owns_lock());
 
         if (lower_it == refcnts_.end() && upper_it == refcnts_.end())
@@ -686,7 +681,8 @@ namespace hpx { namespace agas { namespace server {
 
     ///////////////////////////////////////////////////////////////////////////////
     void primary_namespace::increment(naming::gid_type const& lower,
-        naming::gid_type const& upper, std::int64_t& credits, error_code& ec)
+        naming::gid_type const& upper, std::int64_t const& credits,
+        error_code& ec)
     {    // {{{ increment implementation
         std::unique_lock<mutex_type> l(mutex_);
 
@@ -725,19 +721,20 @@ namespace hpx { namespace agas { namespace server {
 
         for (naming::gid_type raw = lower; raw != upper; ++raw)
         {
-            refcnt_table_type::iterator it = refcnts_.find(raw);
+            auto it = refcnts_.find(raw);
             if (it == refcnts_.end())
             {
                 std::int64_t count =
-                    std::int64_t(HPX_GLOBALCREDIT_INITIAL) + credits;
+                    static_cast<std::int64_t>(HPX_GLOBALCREDIT_INITIAL) +
+                    credits;
 
-                std::pair<refcnt_table_type::iterator, bool> p =
+                std::pair<refcnt_table_type::iterator, bool> const p =
                     refcnts_.insert(refcnt_table_type::value_type(raw, count));
                 if (!p.second)
                 {
                     l.unlock();
 
-                    HPX_THROWS_IF(ec, invalid_data,
+                    HPX_THROWS_IF(ec, hpx::error::invalid_data,
                         "primary_namespace::increment",
                         "couldn't create entry in reference count table, "
                         "raw({1}), ref-count({2})",
@@ -762,6 +759,11 @@ namespace hpx { namespace agas { namespace server {
     }    // }}}
 
     ///////////////////////////////////////////////////////////////////////////////
+    // 26110: Caller failing to hold lock 'l' before calling function
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26110)
+#endif
     void primary_namespace::resolve_free_list(std::unique_lock<mutex_type>& l,
         std::list<refcnt_table_type::iterator> const& free_list,
         free_entry_list_type& free_entry_list,
@@ -772,11 +774,11 @@ namespace hpx { namespace agas { namespace server {
 
         using hpx::get;
 
-        typedef refcnt_table_type::iterator iterator;
+        using iterator = refcnt_table_type::iterator;
 
         for (iterator const& it : free_list)
         {
-            typedef refcnt_table_type::key_type key_type;
+            using key_type = refcnt_table_type::key_type;
 
             // The mapping's key space.
             key_type gid = it->first;
@@ -797,7 +799,7 @@ namespace hpx { namespace agas { namespace server {
             {
                 l.unlock();
 
-                HPX_THROWS_IF(ec, internal_server_error,
+                HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                     "primary_namespace::resolve_free_list",
                     "primary_namespace::resolve_free_list, failed to resolve "
                     "gid, gid({1})",
@@ -809,11 +811,13 @@ namespace hpx { namespace agas { namespace server {
             gva& g = get<1>(r);
 
             // REVIEW: Should we do more to make sure the GVA is valid?
-            if (HPX_UNLIKELY(components::component_invalid == g.type))
+            if (HPX_UNLIKELY(
+                    to_int(hpx::components::component_enum_type::invalid) ==
+                    g.type))
             {
                 l.unlock();
 
-                HPX_THROWS_IF(ec, internal_server_error,
+                HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                     "primary_namespace::resolve_free_list",
                     "encountered a GVA with an invalid type while performing a "
                     "decrement, gid({1}), gva({2})",
@@ -824,7 +828,7 @@ namespace hpx { namespace agas { namespace server {
             {
                 l.unlock();
 
-                HPX_THROWS_IF(ec, internal_server_error,
+                HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                     "primary_namespace::resolve_free_list",
                     "encountered a GVA with a count of zero while performing a "
                     "decrement, gid({1}), gva({2})",
@@ -842,12 +846,15 @@ namespace hpx { namespace agas { namespace server {
 
             // Add the information needed to destroy these components to the
             // free list.
-            free_entry_list.push_back(free_entry(resolved, gid, get<2>(r)));
+            free_entry_list.emplace_back(resolved, gid, get<2>(r));
 
             // remove this entry from the refcnt table
             refcnts_.erase(it);
         }
     }
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
 
     ///////////////////////////////////////////////////////////////////////////////
     void primary_namespace::decrement_sweep(
@@ -896,36 +903,39 @@ namespace hpx { namespace agas { namespace server {
             // we know that it's global reference count is the initial global
             // reference count.
 
-            std::list<refcnt_table_type::iterator> free_list;
+            std::list<refcnt_table_type::iterator> free_list;    //-V826
             for (naming::gid_type raw = lower; raw != upper; ++raw)
             {
-                refcnt_table_type::iterator it = refcnts_.find(raw);
+                auto it = refcnts_.find(raw);
                 if (it == refcnts_.end())
                 {
-                    if (credits > std::int64_t(HPX_GLOBALCREDIT_INITIAL))
+                    if (credits >
+                        static_cast<std::int64_t>(HPX_GLOBALCREDIT_INITIAL))
                     {
                         l.unlock();
 
-                        HPX_THROWS_IF(ec, invalid_data,
+                        HPX_THROWS_IF(ec, hpx::error::invalid_data,
                             "primary_namespace::decrement_sweep",
                             "negative entry in reference count table, "
                             "raw({1}), refcount({2})",
                             raw,
-                            std::int64_t(HPX_GLOBALCREDIT_INITIAL) - credits);
+                            static_cast<std::int64_t>(
+                                HPX_GLOBALCREDIT_INITIAL) -
+                                credits);
                         return;
                     }
 
                     std::int64_t count =
-                        std::int64_t(HPX_GLOBALCREDIT_INITIAL) - credits;
+                        static_cast<std::int64_t>(HPX_GLOBALCREDIT_INITIAL) -
+                        credits;
 
-                    std::pair<refcnt_table_type::iterator, bool> p =
-                        refcnts_.insert(
-                            refcnt_table_type::value_type(raw, count));
+                    std::pair<refcnt_table_type::iterator, bool> const p =
+                        refcnts_.emplace(raw, count);
                     if (!p.second)
                     {
                         l.unlock();
 
-                        HPX_THROWS_IF(ec, invalid_data,
+                        HPX_THROWS_IF(ec, hpx::error::invalid_data,
                             "primary_namespace::decrement_sweep",
                             "couldn't create entry in reference count table, "
                             "raw({1}), ref-count({2})",
@@ -945,7 +955,7 @@ namespace hpx { namespace agas { namespace server {
                 {
                     l.unlock();
 
-                    HPX_THROWS_IF(ec, invalid_data,
+                    HPX_THROWS_IF(ec, hpx::error::invalid_data,
                         "primary_namespace::decrement_sweep",
                         "negative entry in reference count table, raw({1}), "
                         "refcount({2})",
@@ -969,9 +979,9 @@ namespace hpx { namespace agas { namespace server {
 
     ///////////////////////////////////////////////////////////////////////////////
     void primary_namespace::free_components_sync(
-        free_entry_list_type& free_list, naming::gid_type const& lower,
-        naming::gid_type const& upper, error_code& ec)
-    {    // {{{ free_components_sync implementation
+        free_entry_list_type const& free_list, naming::gid_type const& lower,
+        naming::gid_type const& upper, error_code& ec) const
+    {
         using hpx::get;
 
         ///////////////////////////////////////////////////////////////////////////
@@ -979,7 +989,7 @@ namespace hpx { namespace agas { namespace server {
         for (free_entry const& e : free_list)
         {
             // Bail if we're in late shutdown and non-local.
-            if (HPX_UNLIKELY(!threads::threadmanager_is(state_running)) &&
+            if (HPX_UNLIKELY(!threads::threadmanager_is(hpx::state::running)) &&
                 e.locality_ != locality_)
             {
                 LAGAS_(info).format(
@@ -1000,17 +1010,17 @@ namespace hpx { namespace agas { namespace server {
             naming::address addr(e.locality_, e.gva_.type, e.gva_.lva());
             if (e.locality_ == locality_)
             {
-                auto deleter = components::deleter(e.gva_.type);
+                auto const deleter = components::deleter(e.gva_.type);
                 if (deleter == nullptr)
                 {
-                    HPX_THROWS_IF(ec, internal_server_error,
+                    HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                         "primary_namespace::free_components_sync",
                         "Attempting to delete object of unknown component "
                         "type: " +
                             std::to_string(e.gva_.type));
                     return;
                 }
-                deleter(e.gid_, std::move(addr));
+                deleter(e.gid_, HPX_MOVE(addr));
             }
             else
             {
@@ -1020,33 +1030,50 @@ namespace hpx { namespace agas { namespace server {
 
         if (&ec != &throws)
             ec = make_success_code();
-    }    // }}}
+    }
 
     primary_namespace::resolved_type primary_namespace::resolve_gid_locked(
         std::unique_lock<mutex_type>& l, naming::gid_type const& gid,
         error_code& ec)
-    {    // {{{ resolve_gid_locked implementation
+    {
         HPX_ASSERT_OWNS_LOCK(l);
 
         // handle (non-migratable) components located on this locality first
         if (naming::refers_to_local_lva(gid) &&
             !naming::refers_to_virtual_memory(gid))
         {
-            naming::gid_type locality = naming::get_locality_from_gid(gid);
-            gva addr(locality,
-                naming::detail::get_component_type_from_gid(gid.get_msb()), 1,
-                gid.get_lsb());
-            return resolved_type(gid, addr, locality);
+            if (&ec != &throws)
+                ec = make_success_code();
+
+            return resolve_local_id(gid);
         }
 
+        return resolve_gid_locked_non_local(l, gid, ec);
+    }
+
+    // 26110: Caller failing to hold lock 'l' before calling function
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26110)
+#endif
+    primary_namespace::resolved_type
+    primary_namespace::resolve_gid_locked_non_local(
+        std::unique_lock<mutex_type>& l, naming::gid_type const& gid,
+        error_code& ec)
+    {
+        HPX_ASSERT_OWNS_LOCK(l);
+
+        HPX_ASSERT(!(naming::refers_to_local_lva(gid) &&
+            !naming::refers_to_virtual_memory(gid)));
+
         // parameters
         naming::gid_type id = gid;
         naming::detail::strip_internal_bits_from_gid(id);
 
-        gva_table_type::const_iterator it = gvas_.lower_bound(id),
-                                       begin = gvas_.begin(), end = gvas_.end();
+        gva_table_type::const_iterator it = gvas_.lower_bound(id);
+        gva_table_type::const_iterator const begin = gvas_.begin();
 
-        if (it != end)
+        if (gva_table_type::const_iterator const end = gvas_.end(); it != end)
         {
             // Check for exact match
             if (it->first == id)
@@ -1060,7 +1087,7 @@ namespace hpx { namespace agas { namespace server {
 
             // We need to decrement the iterator, first we check that it's safe
             // to do this.
-            else if (it != begin)
+            if (it != begin)
             {
                 --it;
 
@@ -1072,7 +1099,7 @@ namespace hpx { namespace agas { namespace server {
                     {
                         l.unlock();
 
-                        HPX_THROWS_IF(ec, internal_server_error,
+                        HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                             "primary_namespace::resolve_gid_locked",
                             "MSBs of lower and upper range bound do not "
                             "match");
@@ -1100,7 +1127,7 @@ namespace hpx { namespace agas { namespace server {
                 {
                     l.unlock();
 
-                    HPX_THROWS_IF(ec, internal_server_error,
+                    HPX_THROWS_IF(ec, hpx::error::internal_server_error,
                         "primary_namespace::resolve_gid_locked",
                         "MSBs of lower and upper range bound do not match");
                     return resolved_type(
@@ -1118,7 +1145,23 @@ namespace hpx { namespace agas { namespace server {
             ec = make_success_code();
 
         return resolved_type(naming::invalid_gid, gva(), naming::invalid_gid);
-    }    // }}}
+    }
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
+
+#if defined(HPX_HAVE_NETWORKING)
+    void (*route)(primary_namespace& server, parcelset::parcel&& p) = nullptr;
+
+    void primary_namespace::route(parcelset::parcel&& p)
+    {
+        util::scoped_timer<std::atomic<std::int64_t>> update(
+            counter_data_.route_.time_, counter_data_.route_.enabled_);
+        counter_data_.increment_route_count();
+
+        (*server::route)(*this, HPX_MOVE(p));
+    }
+#endif
 
     // access current counter values
     std::int64_t primary_namespace::counter_data::get_bind_gid_count(bool reset)
@@ -1345,5 +1388,4 @@ namespace hpx { namespace agas { namespace server {
         }
     }
 #endif
-
-}}}    // namespace hpx::agas::server
+}    // namespace hpx::agas::server
