@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //  Copyright (c) 2011      Bryce Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -10,18 +10,26 @@
 #include <hpx/async_colocated/get_colocation_id.hpp>
 #include <hpx/components_base/agas_interface.hpp>
 #include <hpx/modules/errors.hpp>
+#include <hpx/modules/futures.hpp>
 #include <hpx/naming_base/id_type.hpp>
 
 namespace hpx {
 
-    naming::id_type get_colocation_id(
-        launch::sync_policy, naming::id_type const& id, error_code& ec)
+    hpx::id_type get_colocation_id(
+        launch::sync_policy, hpx::id_type const& id, error_code& ec)
     {
         return agas::get_colocation_id(launch::sync, id, ec);
     }
 
-    future<naming::id_type> get_colocation_id(naming::id_type const& id)
+    hpx::future<id_type> get_colocation_id(hpx::id_type const& id)
     {
-        return agas::get_colocation_id(id);
+        auto result = agas::get_colocation_id(id);
+        if (result.has_value())
+        {
+            return hpx::make_ready_future(HPX_MOVE(result).get_value());
+        }
+
+        auto f = HPX_MOVE(result).get_future();
+        return f;
     }
 }    // namespace hpx
