@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //  Copyright (c) 2012-2017 Thomas Heller
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -22,7 +22,7 @@
 #include <hpx/ini/ini.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/naming_base/id_type.hpp>
-#include <hpx/plugins/parcel/coalescing_message_handler_registration.hpp>
+#include <hpx/parcelset/coalescing_message_handler_registration.hpp>
 #include <hpx/preprocessor/cat.hpp>
 #include <hpx/preprocessor/expand.hpp>
 #include <hpx/preprocessor/nargs.hpp>
@@ -36,7 +36,7 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace lcos {
+namespace hpx::lcos {
 
     namespace detail {
 
@@ -74,8 +74,8 @@ namespace hpx { namespace lcos {
       , public ComponentTag
     {
     protected:
-        typedef typename std::conditional<std::is_void<Result>::value,
-            util::unused_type, Result>::type result_type;
+        using result_type = std::conditional_t<std::is_void_v<Result>,
+            util::unused_type, Result>;
 
         /// Destructor, needs to be virtual to allow for clean destruction of
         /// derived objects
@@ -83,22 +83,19 @@ namespace hpx { namespace lcos {
 
         void set_event() override
         {
-            set_event_nonvirt(std::is_default_constructible<RemoteResult>());
-        }
-
-        void set_event_nonvirt(std::false_type)
-        {
-            // this shouldn't ever be called
-            HPX_THROW_EXCEPTION(invalid_status,
-                "base_lco_with_value::set_event_nonvirt",
-                "attempt to use a non-default-constructible return type with "
-                "an action in a context where default-construction would be "
-                "required");
-        }
-
-        void set_event_nonvirt(std::true_type)
-        {
-            set_value(RemoteResult());
+            if constexpr (std::is_default_constructible_v<RemoteResult>)
+            {
+                set_value(RemoteResult());
+            }
+            else
+            {
+                // this shouldn't ever be called
+                HPX_THROW_EXCEPTION(hpx::error::invalid_status,
+                    "base_lco_with_value::set_event_nonvirt",
+                    "attempt to use a non-default-constructible return type "
+                    "with an action in a context where default-construction "
+                    "would be required");
+            }
         }
 
         virtual void set_value(RemoteResult&& result) = 0;
@@ -138,7 +135,7 @@ namespace hpx { namespace lcos {
 
         void set_value_nonvirt(RemoteResult&& result)
         {
-            set_value(std::move(result));
+            set_value(HPX_MOVE(result));
         }
 
         /// The \a function get_result_nonvirt is called whenever a
@@ -148,7 +145,18 @@ namespace hpx { namespace lcos {
 
         Result get_value_nonvirt()
         {
+            // Use of the comma-operator in a tested expression causes the left
+            // argument to be ignored when it has no side-effects.
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 6319)
+#endif
+
             return util::void_guard<Result>(), get_value();
+
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
         }
 
     public:
@@ -164,12 +172,12 @@ namespace hpx { namespace lcos {
         /// \param RemoteResult [in] The type of the result to be transferred
         ///               back to this LCO instance.
         HPX_DEFINE_COMPONENT_DIRECT_ACTION(
-            base_lco_with_value, set_value_nonvirt, set_value_action);
+            base_lco_with_value, set_value_nonvirt, set_value_action)
 
         /// The \a get_value_action may be used to query the value this LCO
         /// instance exposes as its 'result' value.
         HPX_DEFINE_COMPONENT_DIRECT_ACTION(
-            base_lco_with_value, get_value_nonvirt, get_value_action);
+            base_lco_with_value, get_value_nonvirt, get_value_action)
     };
 
     /// The base_lco<void> specialization is used whenever the set_event action
@@ -205,11 +213,11 @@ namespace hpx { namespace lcos {
         void get_value() {}
 
         HPX_DEFINE_COMPONENT_DIRECT_ACTION(
-            base_lco_with_value, get_value, get_value_action);
+            base_lco_with_value, get_value, get_value_action)
     };
-}}    // namespace hpx::lcos
+}    // namespace hpx::lcos
 
-namespace hpx { namespace traits {
+namespace hpx::traits {
 
     // define component type data base entry generator
     template <typename Result, typename RemoteResult, typename Enable>
@@ -220,7 +228,8 @@ namespace hpx { namespace traits {
     {
         static constexpr components::component_type get() noexcept
         {
-            return components::component_base_lco_with_value;
+            return to_int(
+                hpx::components::component_enum_type::base_lco_with_value);
         }
 
         static void set(components::component_type)
@@ -237,7 +246,8 @@ namespace hpx { namespace traits {
     {
         static constexpr components::component_type get() noexcept
         {
-            return components::component_base_lco_with_value_unmanaged;
+            return to_int(hpx::components::component_enum_type::
+                    base_lco_with_value_unmanaged);
         }
 
         static void set(components::component_type)
@@ -245,9 +255,9 @@ namespace hpx { namespace traits {
             HPX_ASSERT(false);
         }
     };
-}}    // namespace hpx::traits
+}    // namespace hpx::traits
 
-namespace hpx { namespace components { namespace detail {
+namespace hpx::components::detail {
 
     template <typename Result, typename RemoteResult>
     struct component_heap_impl<
@@ -284,7 +294,7 @@ namespace hpx { namespace components { namespace detail {
             return heap.get();
         }
     };
-}}}    // namespace hpx::components::detail
+}    // namespace hpx::components::detail
 
 ///////////////////////////////////////////////////////////////////////////////
 #define HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(...)                      \
@@ -419,20 +429,20 @@ namespace hpx { namespace components { namespace detail {
         HPX_PP_CAT(                                                            \
             HPX_PP_CAT(base_lco_with_value_, Name), Tag)::set_value_action,    \
         "lco_set_value_action", std::size_t(-1), std::size_t(-1))              \
-/**/
+    /**/
 
+#if !defined(HPX_HAVE_STATIC_LINKING)
 ///////////////////////////////////////////////////////////////////////////////
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(hpx::naming::gid_type, gid_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
     std::vector<hpx::naming::gid_type>, vector_gid_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
-    hpx::naming::id_type, hpx::naming::gid_type, id_type)
-HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
-    hpx::naming::id_type, naming_id_type)
-HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(std::vector<hpx::naming::id_type>,
+    hpx::id_type, hpx::naming::gid_type, id_type)
+HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(hpx::id_type, naming_id_type)
+HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(std::vector<hpx::id_type>,
     std::vector<hpx::naming::gid_type>, vector_id_gid_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
-    std::vector<hpx::naming::id_type>, vector_id_type)
+    std::vector<hpx::id_type>, vector_id_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
     hpx::util::unused_type, hpx_unused_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(float)
@@ -452,3 +462,4 @@ HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(
     std::vector<std::uint32_t>, vector_std_uint32_type)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(hpx::util::section, hpx_section)
 HPX_REGISTER_BASE_LCO_WITH_VALUE_DECLARATION(std::string, std_string)
+#endif
