@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2018 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,16 +6,15 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <hpx/actions_base/traits/extract_action.hpp>
-#include <hpx/async_base/launch_policy.hpp>
 #include <hpx/async_local/async_fwd.hpp>
-#include <hpx/futures/future.hpp>
+#include <hpx/naming_base/id_type.hpp>
+
+namespace hpx::detail {
 
-namespace hpx { namespace detail {
     ///////////////////////////////////////////////////////////////////////////
     template <typename Action, typename Launch, typename... Ts>
-    typename hpx::traits::extract_action<Action>::type::local_result_type
+    typename hpx::traits::extract_action_t<Action>::local_result_type
     async_unwrap_result_impl(
         Launch&& policy, hpx::id_type const& id, Ts&&... vs);
-}}    // namespace hpx::detail
+}    // namespace hpx::detail
