@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -13,8 +13,9 @@
 #include <hpx/actions_base/traits/extract_action.hpp>
 #include <hpx/allocator_support/internal_allocator.hpp>
 #include <hpx/assert.hpp>
-#include <hpx/async_distributed/applier/apply.hpp>
-#include <hpx/async_distributed/applier/apply_callback.hpp>
+#include <hpx/async_distributed/detail/post.hpp>
+#include <hpx/async_distributed/detail/post_callback.hpp>
+#include <hpx/async_distributed/detail/post_implementations_fwd.hpp>
 #include <hpx/async_distributed/promise.hpp>
 #include <hpx/components_base/component_type.hpp>
 #include <hpx/components_base/traits/component_supports_migration.hpp>
@@ -34,7 +35,7 @@
 #include <utility>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace lcos {
+namespace hpx::lcos {
 
 #if defined(HPX_HAVE_NETWORKING)
     namespace detail {
@@ -128,20 +129,20 @@ namespace hpx { namespace lcos {
     ///////////////////////////////////////////////////////////////////////////
     template <typename Action, typename Result>
     class packaged_action<Action, Result, /*DirectExecute=*/false>
-      : public promise<Result,
+      : public hpx::distributed::promise<Result,
             typename hpx::traits::extract_action<Action>::remote_result_type>
     {
     protected:
         using action_type = typename hpx::traits::extract_action<Action>::type;
         using remote_result_type = typename action_type::remote_result_type;
-        using base_type = promise<Result, remote_result_type>;
+        using base_type = hpx::distributed::promise<Result, remote_result_type>;
 
         ///////////////////////////////////////////////////////////////////////
         template <typename... Ts>
-        void do_apply(naming::address&& addr, naming::id_type const& id,
-            threads::thread_priority priority, Ts&&... vs)
+        void do_post(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Ts&&... vs)
         {
-            LLCO_(info).format("packaged_action::do_apply({}, {}) args({})",
+            LLCO_(info).format("packaged_action::do_post({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
@@ -150,36 +151,25 @@ namespace hpx { namespace lcos {
                 detail::parcel_write_handler<Result>{this->shared_state_};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state)]() {};
+            auto&& f = [shared_state = HPX_MOVE(shared_state)]() {};
 #endif
             naming::address resolved_addr(this->resolve());
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
-            if (addr)
-            {
-                hpx::apply_p_cb<action_type>(
-                    actions::typed_continuation<Result, remote_result_type>(
-                        std::move(cont_id), std::move(resolved_addr)),
-                    std::move(addr), id, priority, std::move(f),
-                    std::forward<Ts>(vs)...);
-            }
-            else
-            {
-                hpx::apply_p_cb<action_type>(
-                    actions::typed_continuation<Result, remote_result_type>(
-                        std::move(cont_id), std::move(resolved_addr)),
-                    id, priority, std::move(f), std::forward<Ts>(vs)...);
-            }
-
             this->shared_state_->mark_as_started();
+
+            hpx::post_p_cb<action_type>(
+                actions::typed_continuation<Result, remote_result_type>(
+                    HPX_MOVE(cont_id), HPX_MOVE(resolved_addr)),
+                HPX_MOVE(addr), id, policy, HPX_MOVE(f),
+                HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
-        void do_apply(naming::id_type const& id,
-            threads::thread_priority priority, Ts&&... vs)
+        void do_post(hpx::id_type const& id, hpx::launch policy, Ts&&... vs)
         {
-            LLCO_(info).format("packaged_action::do_apply({}, {}) args({})",
+            LLCO_(info).format("packaged_action::do_post({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
@@ -188,95 +178,95 @@ namespace hpx { namespace lcos {
                 detail::parcel_write_handler<Result>{this->shared_state_};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state)]() {};
+            auto&& f = [shared_state = HPX_MOVE(shared_state)]() {};
 #endif
 
             naming::address resolved_addr(this->resolve());
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
-            hpx::apply_p_cb<action_type>(
-                actions::typed_continuation<Result, remote_result_type>(
-                    std::move(cont_id), std::move(resolved_addr)),
-                id, priority, std::move(f), std::forward<Ts>(vs)...);
-
             this->shared_state_->mark_as_started();
+
+            hpx::post_p_cb<action_type>(
+                actions::typed_continuation<Result, remote_result_type>(
+                    HPX_MOVE(cont_id), HPX_MOVE(resolved_addr)),
+                id, policy, HPX_MOVE(f), HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void do_apply_cb(naming::address&& addr, naming::id_type const& id,
-            threads::thread_priority priority, Callback&& cb, Ts&&... vs)
+        void do_post_cb(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Callback&& cb, Ts&&... vs)
         {
-            LLCO_(info).format("packaged_action::do_apply_cb({}, {}) args({})",
+            LLCO_(info).format("packaged_action::do_post_cb({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
 #if defined(HPX_HAVE_NETWORKING)
-            using callback_type = typename std::decay<Callback>::type;
+            using callback_type = std::decay_t<Callback>;
             auto&& f = detail::parcel_write_handler_cb<Result, callback_type>{
-                this->shared_state_, std::forward<Callback>(cb)};
+                this->shared_state_, HPX_FORWARD(Callback, cb)};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state),
-                           cb = std::forward<Callback>(cb)]() { cb(); };
+            auto&& f = [shared_state = HPX_MOVE(shared_state),
+                           cb = HPX_FORWARD(Callback, cb)]() { cb(); };
 #endif
 
             naming::address resolved_addr(this->resolve());
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
+            this->shared_state_->mark_as_started();
+
             if (addr)
             {
-                hpx::apply_p_cb<action_type>(
+                hpx::post_p_cb<action_type>(
                     actions::typed_continuation<Result, remote_result_type>(
-                        std::move(cont_id), std::move(resolved_addr)),
-                    std::move(addr), id, priority, std::move(f),
-                    std::forward<Ts>(vs)...);
+                        HPX_MOVE(cont_id), HPX_MOVE(resolved_addr)),
+                    HPX_MOVE(addr), id, policy, HPX_MOVE(f),
+                    HPX_FORWARD(Ts, vs)...);
             }
             else
             {
-                hpx::apply_p_cb<action_type>(
+                hpx::post_p_cb<action_type>(
                     actions::typed_continuation<Result, remote_result_type>(
-                        std::move(cont_id), std::move(resolved_addr)),
-                    id, priority, std::move(f), std::forward<Ts>(vs)...);
+                        HPX_MOVE(cont_id), HPX_MOVE(resolved_addr)),
+                    id, policy, HPX_MOVE(f), HPX_FORWARD(Ts, vs)...);
             }
-
-            this->shared_state_->mark_as_started();
         }
 
         template <typename Callback, typename... Ts>
-        void do_apply_cb(naming::id_type const& id,
-            threads::thread_priority priority, Callback&& cb, Ts&&... vs)
+        void do_post_cb(hpx::id_type const& id, hpx::launch policy,
+            Callback&& cb, Ts&&... vs)
         {
-            LLCO_(info).format("packaged_action::do_apply_cb({}, {}) args({})",
+            LLCO_(info).format("packaged_action::do_post_cb({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
 #if defined(HPX_HAVE_NETWORKING)
-            using callback_type = typename std::decay<Callback>::type;
+            using callback_type = std::decay_t<Callback>;
             auto&& f = detail::parcel_write_handler_cb<Result, callback_type>{
-                this->shared_state_, std::forward<Callback>(cb)};
+                this->shared_state_, HPX_FORWARD(Callback, cb)};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state),
-                           cb = std::forward<Callback>(cb)]() { cb(); };
+            auto&& f = [shared_state = HPX_MOVE(shared_state),
+                           cb = HPX_FORWARD(Callback, cb)]() { cb(); };
 #endif
 
             naming::address resolved_addr(this->resolve());
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
-            hpx::apply_p_cb<action_type>(
-                actions::typed_continuation<Result, remote_result_type>(
-                    std::move(cont_id), std::move(resolved_addr)),
-                id, priority, std::move(f), std::forward<Ts>(vs)...);
-
             this->shared_state_->mark_as_started();
+
+            hpx::post_p_cb<action_type>(
+                actions::typed_continuation<Result, remote_result_type>(
+                    HPX_MOVE(cont_id), HPX_MOVE(resolved_addr)),
+                id, policy, HPX_MOVE(f), HPX_FORWARD(Ts, vs)...);
         }
 
     public:
         // Construct a (non-functional) instance of an \a packaged_action. To
-        // use this instance its member function \a apply needs to be directly
+        // use this instance its member function \a post needs to be directly
         // called.
         packaged_action()
           : base_type(std::allocator_arg, hpx::util::internal_allocator<>{})
@@ -290,74 +280,80 @@ namespace hpx { namespace lcos {
         }
 
         template <typename... Ts>
-        void apply(naming::id_type const& id, Ts&&... vs)
+        void post(hpx::id_type const& id, Ts&&... vs)
         {
-            do_apply(id, actions::action_priority<action_type>(),
-                std::forward<Ts>(vs)...);
+            constexpr launch::async_policy policy(
+                actions::action_priority<action_type>(),
+                actions::action_stacksize<action_type>());
+            do_post(id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
-        void apply(
-            naming::address&& addr, naming::id_type const& id, Ts&&... vs)
+        void post(naming::address&& addr, hpx::id_type const& id, Ts&&... vs)
         {
-            do_apply(std::move(addr), id,
+            constexpr launch::async_policy policy(
                 actions::action_priority<action_type>(),
-                std::forward<Ts>(vs)...);
+                actions::action_stacksize<action_type>());
+            do_post(HPX_MOVE(addr), id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_cb(naming::id_type const& id, Callback&& cb, Ts&&... vs)
+        void post_cb(hpx::id_type const& id, Callback&& cb, Ts&&... vs)
         {
-            do_apply_cb(id, actions::action_priority<action_type>(),
-                std::forward<Callback>(cb), std::forward<Ts>(vs)...);
+            constexpr launch::async_policy policy(
+                actions::action_priority<action_type>(),
+                actions::action_stacksize<action_type>());
+            do_post_cb(
+                id, policy, HPX_FORWARD(Callback, cb), HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_cb(naming::address&& addr, naming::id_type const& id,
+        void post_cb(naming::address&& addr, hpx::id_type const& id,
             Callback&& cb, Ts&&... vs)
         {
-            do_apply_cb(std::move(addr), id,
+            constexpr launch::async_policy policy(
                 actions::action_priority<action_type>(),
-                std::forward<Callback>(cb), std::forward<Ts>(vs)...);
+                actions::action_stacksize<action_type>());
+            do_post_cb(HPX_MOVE(addr), id, policy, HPX_FORWARD(Callback, cb),
+                HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
-        void apply_p(naming::id_type const& id,
-            threads::thread_priority priority, Ts&&... vs)
+        void post_p(hpx::id_type const& id, hpx::launch policy, Ts&&... vs)
         {
-            do_apply(id, priority, std::forward<Ts>(vs)...);
+            do_post(id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
-        void apply_p(naming::address&& addr, naming::id_type const& id,
-            threads::thread_priority priority, Ts&&... vs)
+        void post_p(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Ts&&... vs)
         {
-            do_apply(std::move(addr), id, priority, std::forward<Ts>(vs)...);
+            do_post(HPX_MOVE(addr), id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_p_cb(naming::id_type const& id,
-            threads::thread_priority priority, Callback&& cb, Ts&&... vs)
+        void post_p_cb(hpx::id_type const& id, hpx::launch policy,
+            Callback&& cb, Ts&&... vs)
         {
-            do_apply_cb(id, priority, std::forward<Callback>(cb),
-                std::forward<Ts>(vs)...);
+            do_post_cb(
+                id, policy, HPX_FORWARD(Callback, cb), HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_p_cb(naming::address&& addr, naming::id_type const& id,
-            threads::thread_priority priority, Callback&& cb, Ts&&... vs)
+        void post_p_cb(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Callback&& cb, Ts&&... vs)
         {
-            do_apply_cb(std::move(addr), id, priority,
-                std::forward<Callback>(cb), std::forward<Ts>(vs)...);
+            do_post_cb(HPX_MOVE(addr), id, policy, HPX_FORWARD(Callback, cb),
+                HPX_FORWARD(Ts, vs)...);
         }
 
         ///////////////////////////////////////////////////////////////////////
         template <typename... Ts>
-        void apply_deferred(
-            naming::address&& addr, naming::id_type const& id, Ts&&... vs)
+        void post_deferred(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Ts&&... vs)
         {
             LLCO_(info).format(
-                "packaged_action::apply_deferred({}, {}) args({})",
+                "packaged_action::post_deferred({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
@@ -366,46 +362,46 @@ namespace hpx { namespace lcos {
                 detail::parcel_write_handler<Result>{this->shared_state_};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state)]() {};
+            auto&& f = [shared_state = HPX_MOVE(shared_state)]() {};
 #endif
 
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
-            auto fut = hpx::functional::apply_c_p_cb<action_type>(cont_id,
-                std::move(addr), id, actions::action_priority<action_type>(),
-                std::move(f), std::forward<Ts>(vs)...);
+            auto fut = hpx::functional::post_c_p_cb<action_type>(cont_id,
+                HPX_MOVE(addr), id, policy, HPX_MOVE(f),
+                HPX_FORWARD(Ts, vs)...);
 
-            this->shared_state_->set_task(std::move(fut));
+            this->shared_state_->set_task(HPX_MOVE(fut));
         }
 
         template <typename Callback, typename... Ts>
-        void apply_deferred_cb(naming::address&& addr,
-            naming::id_type const& id, Callback&& cb, Ts&&... vs)
+        void post_deferred_cb(naming::address&& addr, hpx::id_type const& id,
+            hpx::launch policy, Callback&& cb, Ts&&... vs)
         {
             LLCO_(info).format(
-                "packaged_action::apply_deferred({}, {}) args({})",
+                "packaged_action::post_deferred({}, {}) args({})",
                 hpx::actions::detail::get_action_name<action_type>(), id,
                 sizeof...(Ts));
 
 #if defined(HPX_HAVE_NETWORKING)
-            using callback_type = typename std::decay<Callback>::type;
+            using callback_type = std::decay_t<Callback>;
             auto&& f = detail::parcel_write_handler_cb<Result, callback_type>{
-                this->shared_state_, std::forward<Callback>(cb)};
+                this->shared_state_, HPX_FORWARD(Callback, cb)};
 #else
             auto shared_state = this->shared_state_;
-            auto&& f = [shared_state = std::move(shared_state),
-                           cb = std::forward<Callback>(cb)]() { cb(); };
+            auto&& f = [shared_state = HPX_MOVE(shared_state),
+                           cb = HPX_FORWARD(Callback, cb)]() { cb(); };
 #endif
 
-            naming::id_type cont_id(this->get_id(false));
+            hpx::id_type cont_id(this->get_id(false));
             naming::detail::set_dont_store_in_cache(cont_id);
 
-            auto fut = hpx::functional::apply_c_p_cb<action_type>(cont_id,
-                std::move(addr), id, actions::action_priority<action_type>(),
-                std::move(f), std::forward<Ts>(vs)...);
+            auto fut = hpx::functional::post_c_p_cb<action_type>(cont_id,
+                HPX_MOVE(addr), id, policy, HPX_MOVE(f),
+                HPX_FORWARD(Ts, vs)...);
 
-            this->shared_state_->set_task(std::move(fut));
+            this->shared_state_->set_task(HPX_MOVE(fut));
         }
     };
 
@@ -419,7 +415,7 @@ namespace hpx { namespace lcos {
 
     public:
         /// Construct a (non-functional) instance of an \a packaged_action. To
-        /// use this instance its member function \a apply needs to be directly
+        /// use this instance its member function \a post needs to be directly
         /// called.
         packaged_action()
           : packaged_action<Action, Result, false>(
@@ -435,212 +431,205 @@ namespace hpx { namespace lcos {
 
         ///////////////////////////////////////////////////////////////////////
         template <typename... Ts>
-        void apply(naming::id_type const& id, Ts&&... vs)
+        void post(hpx::id_type const& id, Ts&&... vs)
         {
-            std::pair<bool, components::pinned_ptr> r;
+            using action_type = hpx::traits::extract_action_t<Action>;
+            using component_type = typename action_type::component_type;
 
+            [[maybe_unused]] std::pair<bool, components::pinned_ptr> r;
             naming::address addr;
-            if (agas::is_local_address_cached(id, addr))
-            {
-                using component_type = typename Action::component_type;
-                HPX_ASSERT(
-                    traits::component_type_is_compatible<component_type>::call(
-                        addr));
 
-                if (traits::component_supports_migration<
-                        component_type>::call())
-                {
-                    r = traits::action_was_object_migrated<Action>::call(
+            if constexpr (traits::component_supports_migration<
+                              component_type>::call())
+            {
+                auto f = [id](naming::address const& addr) {
+                    return traits::action_was_object_migrated<Action>::call(
                         id, addr.address_);
-                    if (!r.first)
-                    {
-                        // local, direct execution
-                        auto&& result = action_type::execute_function(
-                            addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                        this->shared_state_->mark_as_started();
-                        this->shared_state_->set_remote_data(std::move(result));
-                        return;
-                    }
-                }
-                else
+                };
+
+                if (agas::is_local_address_cached(id, addr, r, HPX_MOVE(f)) &&
+                    !r.first)
                 {
+                    HPX_ASSERT(traits::component_type_is_compatible<
+                        component_type>::call(addr));
+
                     // local, direct execution
-                    auto&& result = action_type::execute_function(
-                        addr.address_, addr.type_, std::forward<Ts>(vs)...);
                     this->shared_state_->mark_as_started();
-                    this->shared_state_->set_remote_data(std::move(result));
+                    auto&& result = action_type::execute_function(
+                        addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                    this->shared_state_->set_remote_data(HPX_MOVE(result));
                     return;
                 }
+
+                // fall through
+            }
+            else
+            {
+                // non-migratable objects
+                if (agas::is_local_address_cached(id, addr))
+                {
+                    HPX_ASSERT(traits::component_type_is_compatible<
+                        component_type>::call(addr));
+
+                    // local, direct execution
+                    this->shared_state_->mark_as_started();
+                    auto&& result = action_type::execute_function(
+                        addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                    this->shared_state_->set_remote_data(HPX_MOVE(result));
+                    return;
+                }
+
+                // fall through
             }
 
             // remote execution
-            this->do_apply(id, actions::action_priority<action_type>(),
-                std::forward<Ts>(vs)...);
+            constexpr launch::sync_policy policy(
+                actions::action_priority<action_type>(),
+                actions::action_stacksize<action_type>());
+            this->do_post(id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename... Ts>
-        void apply(
-            naming::address&& addr, naming::id_type const& id, Ts&&... vs)
+        void post(naming::address&& addr, hpx::id_type const& id, Ts&&... vs)
         {
-            std::pair<bool, components::pinned_ptr> r;
+            using action_type = hpx::traits::extract_action_t<Action>;
+            using component_type = typename action_type::component_type;
 
-            if (naming::get_locality_id_from_gid(addr.locality_) ==
-                agas::get_locality_id())
+            if (addr &&
+                naming::get_locality_id_from_gid(addr.locality_) ==
+                    agas::get_locality_id())
             {
-                using component_type = typename Action::component_type;
                 HPX_ASSERT(
                     traits::component_type_is_compatible<component_type>::call(
                         addr));
 
-                if (traits::component_supports_migration<
-                        component_type>::call())
+                if constexpr (traits::component_supports_migration<
+                                  component_type>::call())
                 {
-                    r = traits::action_was_object_migrated<Action>::call(
-                        id, addr.address_);
-                    if (!r.first)
-                    {
-                        // local, direct execution
-                        auto&& result = action_type::execute_function(
-                            addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                        this->shared_state_->mark_as_started();
-                        this->shared_state_->set_remote_data(std::move(result));
-                        return;
-                    }
-                }
-                else
-                {
-                    // local, direct execution
-                    auto&& result = action_type::execute_function(
-                        addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                    this->shared_state_->mark_as_started();
-                    this->shared_state_->set_remote_data(std::move(result));
-                    return;
+                    HPX_ASSERT(
+                        !traits::action_was_object_migrated<action_type>::call(
+                            id, addr.address_)
+                             .first);
+                    HPX_ASSERT(hpx::detail::pin_count_is_valid<component_type>(
+                        addr.address_));
                 }
+
+                // local, direct execution
+                this->shared_state_->mark_as_started();
+                auto&& result = action_type::execute_function(
+                    addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                this->shared_state_->set_remote_data(HPX_MOVE(result));
+                return;
             }
 
             // remote execution
-            this->do_apply(std::move(addr), id,
+            constexpr launch::sync_policy policy(
                 actions::action_priority<action_type>(),
-                std::forward<Ts>(vs)...);
+                actions::action_stacksize<action_type>());
+            this->do_post(HPX_MOVE(addr), id, policy, HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_cb(naming::id_type const& id, Callback&& cb, Ts&&... vs)
+        void post_cb(hpx::id_type const& id, Callback&& cb, Ts&&... vs)
         {
-            std::pair<bool, components::pinned_ptr> r;
+            using action_type = hpx::traits::extract_action_t<Action>;
+            using component_type = typename action_type::component_type;
 
+            [[maybe_unused]] std::pair<bool, components::pinned_ptr> r;
             naming::address addr;
-            if (agas::is_local_address_cached(id, addr))
+
+            if constexpr (traits::component_supports_migration<
+                              component_type>::call())
             {
-                using component_type = typename Action::component_type;
-                HPX_ASSERT(
-                    traits::component_type_is_compatible<component_type>::call(
-                        addr));
-
-                if (traits::component_supports_migration<
-                        component_type>::call())
-                {
-                    r = traits::action_was_object_migrated<Action>::call(
+                auto f = [id](naming::address const& addr) {
+                    return traits::action_was_object_migrated<Action>::call(
                         id, addr.address_);
-                    if (!r.first)
-                    {
-                        // local, direct execution
-                        auto&& result = action_type::execute_function(
-                            addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                        this->shared_state_->mark_as_started();
-                        this->shared_state_->set_remote_data(std::move(result));
+                };
 
-                        // invoke callback
-#if defined(HPX_HAVE_NETWORKING)
-                        cb(std::error_code(), parcelset::parcel());
-#else
-                        cb();
-#endif
-
-                        return;
-                    }
-                }
-                else
+                if (agas::is_local_address_cached(id, addr, r, HPX_MOVE(f)) &&
+                    !r.first)
                 {
                     // local, direct execution
-                    auto&& result = action_type::execute_function(
-                        addr.address_, addr.type_, std::forward<Ts>(vs)...);
                     this->shared_state_->mark_as_started();
-                    this->shared_state_->set_remote_data(std::move(result));
+                    auto&& result = action_type::execute_function(
+                        addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                    this->shared_state_->set_remote_data(HPX_MOVE(result));
 
-                    // invoke callback
-#if defined(HPX_HAVE_NETWORKING)
-                    cb(std::error_code(), parcelset::parcel());
-#else
-                    cb();
-#endif
+                    hpx::detail::invoke_callback(HPX_FORWARD(Callback, cb));
                     return;
                 }
+
+                // fall through
+            }
+            else
+            {
+                // non-migratable objects
+                if (agas::is_local_address_cached(id, addr))
+                {
+                    HPX_ASSERT(traits::component_type_is_compatible<
+                        component_type>::call(addr));
+
+                    // local, direct execution
+                    this->shared_state_->mark_as_started();
+                    auto&& result = action_type::execute_function(
+                        addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                    this->shared_state_->set_remote_data(HPX_MOVE(result));
+
+                    hpx::detail::invoke_callback(HPX_FORWARD(Callback, cb));
+                    return;
+                }
+
+                // fall through
             }
 
             // remote execution
-            this->do_apply_cb(id, actions::action_priority<action_type>(),
-                std::forward<Callback>(cb), std::forward<Ts>(vs)...);
+            constexpr launch::sync_policy policy(
+                actions::action_priority<action_type>(),
+                actions::action_stacksize<action_type>());
+            this->do_post_cb(
+                id, policy, HPX_FORWARD(Callback, cb), HPX_FORWARD(Ts, vs)...);
         }
 
         template <typename Callback, typename... Ts>
-        void apply_cb(naming::address&& addr, naming::id_type const& id,
+        void post_cb(naming::address&& addr, hpx::id_type const& id,
             Callback&& cb, Ts&&... vs)
         {
-            std::pair<bool, components::pinned_ptr> r;
-
-            if (naming::get_locality_id_from_gid(addr.locality_) ==
-                agas::get_locality_id())
+            if (addr &&
+                naming::get_locality_id_from_gid(addr.locality_) ==
+                    agas::get_locality_id())
             {
                 using component_type = typename Action::component_type;
                 HPX_ASSERT(
                     traits::component_type_is_compatible<component_type>::call(
                         addr));
 
-                if (traits::component_supports_migration<
-                        component_type>::call())
+                if constexpr (traits::component_supports_migration<
+                                  component_type>::call())
                 {
-                    r = traits::action_was_object_migrated<Action>::call(
-                        id, addr.address_);
-                    if (!r.first)
-                    {
-                        // local, direct execution
-                        auto&& result = action_type::execute_function(
-                            addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                        this->shared_state_->mark_as_started();
-                        this->shared_state_->set_remote_data(std::move(result));
-
-                        // invoke callback
-#if defined(HPX_HAVE_NETWORKING)
-                        cb(std::error_code(), parcelset::parcel());
-#else
-                        cb();
-#endif
-                        return;
-                    }
+                    HPX_ASSERT(
+                        !traits::action_was_object_migrated<action_type>::call(
+                            id, addr.address_)
+                             .first);
+                    HPX_ASSERT(hpx::detail::pin_count_is_valid<component_type>(
+                        addr.address_));
                 }
-                else
-                {
-                    // local, direct execution
-                    auto&& result = action_type::execute_function(
-                        addr.address_, addr.type_, std::forward<Ts>(vs)...);
-                    this->shared_state_->mark_as_started();
-                    this->shared_state_->set_remote_data(std::move(result));
 
-                    // invoke callback
-#if defined(HPX_HAVE_NETWORKING)
-                    cb(std::error_code(), parcelset::parcel());
-#else
-                    cb();
-#endif
-                    return;
-                }
+                // local, direct execution
+                this->shared_state_->mark_as_started();
+                auto&& result = action_type::execute_function(
+                    addr.address_, addr.type_, HPX_FORWARD(Ts, vs)...);
+                this->shared_state_->set_remote_data(HPX_MOVE(result));
+
+                hpx::detail::invoke_callback(HPX_FORWARD(Callback, cb));
+                return;
             }
 
             // remote execution
-            this->do_apply_cb(std::move(addr), id,
+            constexpr launch::sync_policy policy(
                 actions::action_priority<action_type>(),
-                std::forward<Callback>(cb), std::forward<Ts>(vs)...);
+                actions::action_stacksize<action_type>());
+            this->do_post_cb(HPX_MOVE(addr), id, policy,
+                HPX_FORWARD(Callback, cb), HPX_FORWARD(Ts, vs)...);
         }
     };
-}}    // namespace hpx::lcos
+}    // namespace hpx::lcos
