@@ -18,13 +18,13 @@ namespace hpx { namespace actions {
     struct set_lco_value_continuation
     {
         template <typename T>
-        HPX_FORCEINLINE T operator()(naming::id_type const& lco, T&& t) const
+        HPX_FORCEINLINE T operator()(hpx::id_type const& lco, T&& t) const
         {
-            hpx::set_lco_value(lco, std::forward<T>(t));
+            hpx::set_lco_value(lco, HPX_FORWARD(T, t));
 
             // Yep, 't' is a zombie, however we don't use the returned value
             // anyways. We need it for result type calculation, though.
-            return std::forward<T>(t);
+            return HPX_FORWARD(T, t);
         }
     };
 
@@ -32,13 +32,13 @@ namespace hpx { namespace actions {
     struct set_lco_value_unmanaged_continuation
     {
         template <typename T>
-        HPX_FORCEINLINE T operator()(naming::id_type const& lco, T&& t) const
+        HPX_FORCEINLINE T operator()(hpx::id_type const& lco, T&& t) const
         {
-            hpx::set_lco_value_unmanaged(lco, std::forward<T>(t));
+            hpx::set_lco_value_unmanaged(lco, HPX_FORWARD(T, t));
 
             // Yep, 't' is a zombie, however we don't use the returned value
             // anyways. We need it for result type calculation, though.
-            return std::forward<T>(t);
+            return HPX_FORWARD(T, t);
         }
     };
 }}    // namespace hpx::actions
