@@ -14,7 +14,7 @@
 #include <hpx/hpx_main.hpp>
 #include <hpx/pack_traversal/unwrap.hpp>
 
-typedef hpx::lcos::shared_future<double> future_type;
+typedef hpx::shared_future<double> future_type;
 
 template <typename Value>
 struct mul
