@@ -19,7 +19,7 @@
 // the following line causes compile errors
 using hpx::dataflow;
 
-typedef hpx::lcos::shared_future<double> future_type;
+typedef hpx::shared_future<double> future_type;
 
 template <typename Value>
 struct mul
