@@ -13,7 +13,7 @@
 #include <hpx/include/runtime.hpp>
 
 void test(int (&ptr)[20]) {}
-HPX_PLAIN_ACTION(test);
+HPX_PLAIN_ACTION(test)
 
 ///////////////////////////////////////////////////////////////////////////////
 int main()
@@ -21,7 +21,7 @@ int main()
     int arr[20] = {0};
 
     test_action act;
-    hpx::apply(act, hpx::find_here(), arr);
+    hpx::post(act, hpx::find_here(), arr);
 
     return 0;
 }
