@@ -13,7 +13,7 @@
 #include <hpx/include/runtime.hpp>
 
 void test(int& ref) {}
-HPX_PLAIN_ACTION(test);
+HPX_PLAIN_ACTION(test)
 
 ///////////////////////////////////////////////////////////////////////////////
 int main()
@@ -21,7 +21,7 @@ int main()
     int val = 0;
 
     test_action act;
-    hpx::apply(act, hpx::find_here(), val);
+    hpx::post(act, hpx::find_here(), val);
 
     return 0;
 }
