@@ -10,11 +10,11 @@
 #if !defined(HPX_COMPUTE_DEVICE_CODE)
 #include <hpx/hpx_main.hpp>
 #include <hpx/include/actions.hpp>
-#include <hpx/include/apply.hpp>
+#include <hpx/include/post.hpp>
 #include <hpx/include/runtime.hpp>
 
 void test(int const& ref) {}
-HPX_PLAIN_ACTION(test);
+HPX_PLAIN_ACTION(test)
 
 ///////////////////////////////////////////////////////////////////////////////
 int main()
@@ -22,7 +22,7 @@ int main()
     int val = 0;
 
     test_action act;
-    hpx::apply(act, hpx::find_here(), val);
+    hpx::post(act, hpx::find_here(), val);
 
     return 0;
 }
