@@ -1,5 +1,5 @@
 // Copyright (c) 2018 Adrian Serio
-// Copyright (c) 2018-2021 Hartmut Kaiser
+// Copyright (c) 2018-2023 Hartmut Kaiser
 //
 // SPDX-License-Identifier: BSL-1.0
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -99,7 +99,7 @@ namespace hpx { namespace util {
         {
         }
         checkpoint(std::vector<char>&& vec) noexcept
-          : data_(std::move(vec))
+          : data_(HPX_MOVE(vec))
         {
         }
 
@@ -220,18 +220,19 @@ namespace hpx { namespace util {
                 typename std::decay<T>::type>::value>::type>
         T&& prepare_client(T&& t) noexcept
         {
-            return std::forward<T>(t);
+            return HPX_FORWARD(T, t);
         }
 
         // Properly handle Clients to components
-        template <typename Client, typename Server>
+        template <typename Client, typename Server, typename Data>
         hpx::future<std::shared_ptr<typename hpx::components::client_base<
-            Client, Server>::server_component_type>>
-        prepare_client(hpx::components::client_base<Client, Server> const& c)
+            Client, Server, Data>::server_component_type>>
+        prepare_client(
+            hpx::components::client_base<Client, Server, Data> const& c)
         {
             // Use shared pointer to serialize server
             using server_type = typename hpx::components::client_base<Client,
-                Server>::server_component_type;
+                Server, Data>::server_component_type;
             return hpx::get_ptr<server_type>(c.get_id());
         }
 
@@ -241,8 +242,8 @@ namespace hpx { namespace util {
             checkpoint operator()(checkpoint&& c, Ts&&... ts) const
             {
                 hpx::util::save_checkpoint_data(
-                    c.data_, std::forward<Ts>(ts)...);
-                return std::move(c);
+                    c.data_, HPX_FORWARD(Ts, ts)...);
+                return HPX_MOVE(c);
             }
         };
     }    // namespace detail
@@ -289,8 +290,8 @@ namespace hpx { namespace util {
     hpx::future<checkpoint> save_checkpoint(T&& t, Ts&&... ts)
     {
         return hpx::dataflow(detail::save_funct_obj{}, checkpoint{},
-            detail::prepare_client(std::forward<T>(t)),
-            detail::prepare_client(std::forward<Ts>(ts))...);
+            detail::prepare_client(HPX_FORWARD(T, t)),
+            detail::prepare_client(HPX_FORWARD(Ts, ts))...);
     }
 
     /// \cond NOINTERNAL
@@ -338,16 +339,16 @@ namespace hpx { namespace util {
     template <typename T, typename... Ts>
     hpx::future<checkpoint> save_checkpoint(checkpoint&& c, T&& t, Ts&&... ts)
     {
-        return hpx::dataflow(detail::save_funct_obj{}, std::move(c),
-            detail::prepare_client(std::forward<T>(t)),
-            detail::prepare_client(std::forward<Ts>(ts))...);
+        return hpx::dataflow(detail::save_funct_obj{}, HPX_MOVE(c),
+            detail::prepare_client(HPX_FORWARD(T, t)),
+            detail::prepare_client(HPX_FORWARD(Ts, ts))...);
     }
 
     /// \cond NOINTERNAL
     // Same as above, just nullary
     inline hpx::future<checkpoint> save_checkpoint(checkpoint&& c)
     {
-        return hpx::make_ready_future(std::move(c));
+        return hpx::make_ready_future(HPX_MOVE(c));
     }
     /// \endcond
 
@@ -392,8 +393,8 @@ namespace hpx { namespace util {
     hpx::future<checkpoint> save_checkpoint(hpx::launch p, T&& t, Ts&&... ts)
     {
         return hpx::dataflow(p, detail::save_funct_obj{}, checkpoint{},
-            detail::prepare_client(std::forward<T>(t)),
-            detail::prepare_client(std::forward<Ts>(ts))...);
+            detail::prepare_client(HPX_FORWARD(T, t)),
+            detail::prepare_client(HPX_FORWARD(Ts, ts))...);
     }
 
     /// \cond NOINTERNAL
@@ -446,16 +447,16 @@ namespace hpx { namespace util {
     hpx::future<checkpoint> save_checkpoint(
         hpx::launch p, checkpoint&& c, T&& t, Ts&&... ts)
     {
-        return hpx::dataflow(p, detail::save_funct_obj{}, std::move(c),
-            detail::prepare_client(std::forward<T>(t)),
-            detail::prepare_client(std::forward<Ts>(ts))...);
+        return hpx::dataflow(p, detail::save_funct_obj{}, HPX_MOVE(c),
+            detail::prepare_client(HPX_FORWARD(T, t)),
+            detail::prepare_client(HPX_FORWARD(Ts, ts))...);
     }
 
     /// \cond NOINTERNAL
     // Same as above, just nullary
     inline hpx::future<checkpoint> save_checkpoint(hpx::launch, checkpoint&& c)
     {
-        return hpx::make_ready_future(std::move(c));
+        return hpx::make_ready_future(HPX_MOVE(c));
     }
     /// \endcond
 
@@ -503,8 +504,8 @@ namespace hpx { namespace util {
     {
         hpx::future<checkpoint> f_chk =
             hpx::dataflow(sync_p, detail::save_funct_obj{}, checkpoint{},
-                detail::prepare_client(std::forward<T>(t)),
-                detail::prepare_client(std::forward<Ts>(ts))...);
+                detail::prepare_client(HPX_FORWARD(T, t)),
+                detail::prepare_client(HPX_FORWARD(Ts, ts))...);
         return f_chk.get();
     }
 
@@ -556,9 +557,9 @@ namespace hpx { namespace util {
         hpx::launch::sync_policy sync_p, checkpoint&& c, T&& t, Ts&&... ts)
     {
         hpx::future<checkpoint> f_chk =
-            hpx::dataflow(sync_p, detail::save_funct_obj{}, std::move(c),
-                detail::prepare_client(std::forward<T>(t)),
-                detail::prepare_client(std::forward<Ts>(ts))...);
+            hpx::dataflow(sync_p, detail::save_funct_obj{}, HPX_MOVE(c),
+                detail::prepare_client(HPX_FORWARD(T, t)),
+                detail::prepare_client(HPX_FORWARD(Ts, ts))...);
         return f_chk.get();
     }
 
@@ -566,7 +567,7 @@ namespace hpx { namespace util {
     // Same as above, just nullary
     inline checkpoint save_checkpoint(hpx::launch::sync_policy, checkpoint&& c)
     {
-        return std::move(c);
+        return HPX_MOVE(c);
     }
     /// \endcond
 
@@ -580,7 +581,7 @@ namespace hpx { namespace util {
             {
                 std::size_t size = hpx::util::prepare_checkpoint_data(ts...);
                 c.data_.resize(size);
-                return std::move(c);
+                return HPX_MOVE(c);
             }
         };
     }    // namespace detail
@@ -650,7 +651,7 @@ namespace hpx { namespace util {
     hpx::future<checkpoint> prepare_checkpoint(
         checkpoint&& c, T const& t, Ts const&... ts)
     {
-        return hpx::dataflow(detail::prepare_checkpoint{}, std::move(c),
+        return hpx::dataflow(detail::prepare_checkpoint{}, HPX_MOVE(c),
             detail::prepare_client(t), detail::prepare_client(ts)...);
     }
 
@@ -658,7 +659,7 @@ namespace hpx { namespace util {
     // Same as above, just nullary
     inline hpx::future<checkpoint> prepare_checkpoint(checkpoint&& c)
     {
-        return hpx::make_ready_future(std::move(c));
+        return hpx::make_ready_future(HPX_MOVE(c));
     }
     /// \endcond
 
@@ -750,7 +751,7 @@ namespace hpx { namespace util {
     hpx::future<checkpoint> prepare_checkpoint(
         hpx::launch p, checkpoint&& c, T const& t, Ts const&... ts)
     {
-        return hpx::dataflow(p, detail::prepare_checkpoint{}, std::move(c),
+        return hpx::dataflow(p, detail::prepare_checkpoint{}, HPX_MOVE(c),
             detail::prepare_client(t), detail::prepare_client(ts)...);
     }
 
@@ -759,7 +760,7 @@ namespace hpx { namespace util {
     checkpoint prepare_checkpoint(hpx::launch::sync_policy sync_p,
         checkpoint&& c, T const& t, Ts const&... ts)
     {
-        return hpx::dataflow(sync_p, detail::prepare_checkpoint{}, std::move(c),
+        return hpx::dataflow(sync_p, detail::prepare_checkpoint{}, HPX_MOVE(c),
             detail::prepare_client(t), detail::prepare_client(ts)...)
             .get();
     }
@@ -768,13 +769,13 @@ namespace hpx { namespace util {
     inline hpx::future<checkpoint> prepare_checkpoint(
         hpx::launch, checkpoint&& c)
     {
-        return hpx::make_ready_future(std::move(c));
+        return hpx::make_ready_future(HPX_MOVE(c));
     }
 
     inline checkpoint prepare_checkpoint(
         hpx::launch::sync_policy, checkpoint&& c)
     {
-        return std::move(c);
+        return HPX_MOVE(c);
     }
     /// \endcond
 
@@ -793,21 +794,21 @@ namespace hpx { namespace util {
             }
 
             // Properly handle client/server restoration
-            template <typename Client, typename Server>
+            template <typename Client, typename Server, typename Data>
             void operator()(hpx::serialization::input_archive& ar,
-                hpx::components::client_base<Client, Server>& c) const
+                hpx::components::client_base<Client, Server, Data>& c) const
             {
                 // Revive server
                 using server_component_type =
-                    typename hpx::components::client_base<Client,
-                        Server>::server_component_type;
+                    typename hpx::components::client_base<Client, Server,
+                        Data>::server_component_type;
 
                 hpx::future<std::shared_ptr<server_component_type>>
                     f_server_ptr;
                 ar >> f_server_ptr;
                 c = hpx::new_<Client>(
                     naming::get_id_from_locality_id(agas::get_locality_id()),
-                    std::move(*(f_server_ptr.get())));
+                    HPX_MOVE(*(f_server_ptr.get())));
             }
         };
     }    // namespace detail
