@@ -133,8 +133,8 @@ namespace hpx { namespace lcos {
 
         ///////////////////////////////////////////////////////////////////////
         template <typename Action, typename Futures, typename... Ts>
-        void reduce_invoke(Action, Futures& futures, hpx::id_type const& id,
-            std::size_t, Ts const&... vs)
+        void reduce_invoke(Action, Futures& futures,    //-V524
+            hpx::id_type const& id, std::size_t, Ts const&... vs)
         {
             futures.push_back(hpx::async<Action>(id, vs...));
         }
@@ -209,7 +209,7 @@ namespace hpx { namespace lcos {
             Result operator()(
                 hpx::future<std::vector<hpx::future<Result>>> r) const
             {
-                std::vector<hpx::future<Result>> fres = std::move(r.get());
+                std::vector<hpx::future<Result>> fres = r.get();
 
                 HPX_ASSERT(!fres.empty());
 
@@ -271,7 +271,7 @@ namespace hpx { namespace lcos {
                     hpx::id_type id(ids_next[0]);
                     reduce_futures.push_back(
                         hpx::detail::async_colocated<reduce_impl_action>(id,
-                            act, std::move(ids_next), reduce_op,
+                            act, HPX_MOVE(ids_next), reduce_op,
                             global_idx + applied, vs...));
 
                     applied += next_fan;
@@ -299,12 +299,13 @@ namespace hpx { namespace lcos {
         if (ids.empty())
         {
             return hpx::make_exceptional_future<action_result>(
-                HPX_GET_EXCEPTION(bad_parameter, "hpx::lcos::reduce",
+                HPX_GET_EXCEPTION(hpx::error::bad_parameter,
+                    "hpx::lcos::reduce",
                     "empty list of targets for reduce operation"));
         }
 
         return hpx::detail::async_colocated<reduce_impl_action>(
-            ids[0], Action(), ids, std::forward<ReduceOp>(reduce_op), 0, vs...);
+            ids[0], Action(), ids, HPX_FORWARD(ReduceOp, reduce_op), 0, vs...);
     }
 
     template <typename Component, typename Signature, typename Derived,
@@ -315,7 +316,7 @@ namespace hpx { namespace lcos {
         std::vector<hpx::id_type> const& ids, ReduceOp&& reduce_op,
         Ts const&... vs)
     {
-        return reduce<Derived>(ids, std::forward<ReduceOp>(reduce_op), vs...);
+        return reduce<Derived>(ids, HPX_FORWARD(ReduceOp, reduce_op), vs...);
     }
 
     template <typename Action, typename ReduceOp, typename... Ts>
@@ -324,7 +325,7 @@ namespace hpx { namespace lcos {
         Ts const&... vs)
     {
         return reduce<detail::reduce_with_index<Action>>(
-            ids, std::forward<ReduceOp>(reduce_op), vs...);
+            ids, HPX_FORWARD(ReduceOp, reduce_op), vs...);
     }
 
     template <typename Component, typename Signature, typename Derived,
@@ -337,7 +338,7 @@ namespace hpx { namespace lcos {
         Ts const&... vs)
     {
         return reduce<detail::reduce_with_index<Derived>>(
-            ids, std::forward<ReduceOp>(reduce_op), vs...);
+            ids, HPX_FORWARD(ReduceOp, reduce_op), vs...);
     }
 }}    // namespace hpx::lcos
 
