@@ -4,10 +4,7 @@
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <hpx/config.hpp>
-#if !defined(HPX_COMPUTE_DEVICE_CODE)
-#include <hpx/hpx.hpp>
-#include <hpx/hpx_init.hpp>
+#include <hpx/init.hpp>
 #include <hpx/modules/testing.hpp>
 #include <hpx/util/from_string.hpp>
 
@@ -27,12 +24,11 @@ int hpx_main()
     HPX_TEST_EQ(
         hpx::get_config_entry("hpx.runtime_mode", ""), std::string("console"));
 
-    return hpx::finalize();
+    return hpx::local::finalize();
 }
 
 int main(int argc, char* argv[])
 {
-    HPX_TEST_EQ(hpx::init(argc, argv), 0);
+    HPX_TEST_EQ(hpx::local::init(hpx_main, argc, argv), 0);
     return hpx::util::report_errors();
 }
-#endif
