@@ -16,15 +16,15 @@ std::atomic<bool> invoked(false);
 int hpx_main(int, char*[])
 {
     invoked = true;
-    return hpx::finalize();
+    return hpx::local::finalize();
 }
 
 int main(int argc, char* argv[])
 {
-    hpx::init_params params;
+    hpx::local::init_params params;
     params.cfg = {"--hpx:exit"};
 
-    HPX_TEST_EQ(hpx::init(argc, argv, params), 1);
+    HPX_TEST_EQ(hpx::local::init(hpx_main, argc, argv, params), 1);
     HPX_TEST(!invoked);
 
     return hpx::util::report_errors();
