@@ -5,7 +5,6 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <hpx/init.hpp>
-#include <hpx/local/runtime.hpp>
 #include <hpx/modules/testing.hpp>
 
 #include <string>
@@ -14,17 +13,17 @@
 int hpx_main()
 {
     HPX_TEST_EQ(hpx::get_config_entry("hpx.cores", "1"), std::string("3"));
-    return hpx::finalize();
+    return hpx::local::finalize();
 }
 
 int main(int argc, char* argv[])
 {
     std::vector<std::string> const cfg = {"--hpx:cores=3"};
 
-    hpx::init_params init_args;
+    hpx::local::init_params init_args;
     init_args.cfg = cfg;
 
-    HPX_TEST_EQ(hpx::init(argc, argv, init_args), 0);
+    HPX_TEST_EQ(hpx::local::init(hpx_main, argc, argv, init_args), 0);
 
     return hpx::util::report_errors();
 }
