@@ -10,8 +10,8 @@
 #include <hpx/config.hpp>
 #if !defined(HPX_COMPUTE_DEVICE_CODE)
 #include <hpx/init.hpp>
-#include <hpx/local/thread.hpp>
 #include <hpx/modules/testing.hpp>
+#include <hpx/thread.hpp>
 
 #include <string>
 #include <vector>
@@ -21,7 +21,7 @@ int hpx_main()
     HPX_TEST_EQ(
         hpx::threads::hardware_concurrency(), hpx::get_os_thread_count());
 
-    return hpx::finalize();
+    return hpx::local::finalize();
 }
 
 // Ignore all command line options to avoid any interference with the test
@@ -32,10 +32,10 @@ int main()
 {
     std::vector<std::string> cfg = {"hpx.os_threads=all"};
 
-    hpx::init_params init_args;
+    hpx::local::init_params init_args;
     init_args.cfg = cfg;
 
-    HPX_TEST_EQ(hpx::init(1, argv, init_args), 0);
+    HPX_TEST_EQ(hpx::local::init(hpx_main, 1, argv, init_args), 0);
     return hpx::util::report_errors();
 }
 #endif
