@@ -1,4 +1,4 @@
-//  Copyright (c) 2017-2021 Hartmut Kaiser
+//  Copyright (c) 2017-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,15 +6,10 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <hpx/components_base/components_base_fwd.hpp>
 
-namespace hpx {
+namespace hpx::components {
 
-    /// \namespace components
-    namespace components {
-
-        template <typename Derived, typename Stub>
-        class client_base;
-    }
-}    // namespace hpx
+    template <typename Derived, typename Stub, typename ClientData = void>
+    class client_base;
+}    // namespace hpx::components
