@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //  Copyright (c) 2011-2017 Thomas Heller
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -18,18 +18,15 @@
 #include <hpx/components_base/server/wrapper_heap_list.hpp>
 #include <hpx/components_base/traits/is_component.hpp>
 #include <hpx/components_base/traits/managed_component_policies.hpp>
-#include <hpx/functional/unique_function.hpp>
 #include <hpx/modules/errors.hpp>
 
 #include <cstddef>
 #include <cstdint>
-#include <stdexcept>
 #include <type_traits>
 #include <utility>
-#include <vector>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace components {
+namespace hpx::components {
 
     ///////////////////////////////////////////////////////////////////////////
     namespace detail_adl_barrier {
@@ -51,7 +48,7 @@ namespace hpx { namespace components {
                 Component*& component, Managed* this_, Ts&&... vs)
             {
                 using wrapped_type = typename Managed::wrapped_type;
-                component = new wrapped_type(this_, std::forward<Ts>(vs)...);
+                component = new wrapped_type(this_, HPX_FORWARD(Ts, vs)...);
             }
         };
 
@@ -69,7 +66,7 @@ namespace hpx { namespace components {
                 Component*& component, Managed* this_, Ts&&... vs)
             {
                 using wrapped_type = typename Managed::wrapped_type;
-                component = new wrapped_type(std::forward<Ts>(vs)...);
+                component = new wrapped_type(HPX_FORWARD(Ts, vs)...);
                 component->set_back_ptr(this_);
             }
         };
@@ -89,7 +86,7 @@ namespace hpx { namespace components {
                 // The managed_component's controls the lifetime of the
                 // component implementation.
                 back_ptr->finalize();
-                back_ptr->~BackPtr();
+                std::destroy_at(back_ptr);
                 component_heap<typename BackPtr::wrapped_type>().free(back_ptr);
             }
         };
@@ -138,7 +135,8 @@ namespace hpx { namespace components {
         struct manage_lifetime<traits::managed_object_controls_lifetime>
         {
             template <typename Component>
-            static void call(Component* component)
+            static void call(Component* component) noexcept(
+                noexcept(component->finalize()))
             {
                 // The managed_component controls the lifetime of the
                 // component implementation.
@@ -161,25 +159,24 @@ namespace hpx { namespace components {
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
 
-        struct base_managed_component
-          : public traits::detail::managed_component_tag
+        struct base_managed_component : traits::detail::managed_component_tag
         {
             // finalize() will be called just before the instance gets destructed
             static constexpr void finalize() noexcept {}
 
-            static void mark_as_migrated()
+            static void mark_as_migrated() noexcept
             {
-                // If this assertion is triggered then this component instance is
-                // being migrated even if the component type has not been enabled
-                // to support migration.
+                // If this assertion is triggered then this component instance
+                // is being migrated even if the component type has not been
+                // enabled to support migration.
                 HPX_ASSERT(false);
             }
 
-            static void on_migrated()
+            static void on_migrated() noexcept
             {
-                // If this assertion is triggered then this component instance is being
-                // migrated even if the component type has not been enabled to support
-                // migration.
+                // If this assertion is triggered then this component instance
+                // is being migrated even if the component type has not been
+                // enabled to support migration.
                 HPX_ASSERT(false);
             }
         };
@@ -190,12 +187,15 @@ namespace hpx { namespace components {
     class managed_component_base : public detail::base_managed_component
     {
     public:
-        HPX_NON_COPYABLE(managed_component_base);
+        managed_component_base(managed_component_base const&) = delete;
+        managed_component_base(managed_component_base&&) = delete;
+        managed_component_base& operator=(
+            managed_component_base const&) = delete;
+        managed_component_base& operator=(managed_component_base&&) = delete;
 
-    public:
-        using this_component_type = typename std::conditional<
-            std::is_same<Component, detail::this_type>::value,
-            managed_component_base, Component>::type;
+        using this_component_type =
+            std::conditional_t<std::is_void_v<Component>,
+                managed_component_base, Component>;
 
         using wrapped_type = this_component_type;
 
@@ -205,22 +205,17 @@ namespace hpx { namespace components {
 
         // make sure that we have a back_ptr whenever we need to control the
         // lifetime of the managed_component
-        static_assert((std::is_same<ctor_policy,
-                           traits::construct_without_back_ptr>::value ||
-                          std::is_same<dtor_policy,
-                              traits::managed_object_controls_lifetime>::value),
-            "std::is_same<ctor_policy, "
-            "traits::construct_without_back_ptr>::value || "
-            "std::is_same<dtor_policy, "
-            "traits::managed_object_controls_lifetime>::value");
+        static_assert(
+            (std::is_same_v<ctor_policy, traits::construct_without_back_ptr> ||
+                std::is_same_v<dtor_policy,
+                    traits::managed_object_controls_lifetime>),
+            "std::is_same_v<ctor_policy, construct_without_back_ptr> || "
+            "std::is_same_v<dtor_policy, managed_object_controls_lifetime>");
 
-        constexpr managed_component_base() noexcept
-          : back_ptr_(nullptr)
-        {
-        }
+        constexpr managed_component_base() noexcept = default;
 
         explicit managed_component_base(
-            managed_component<Component, Wrapper>* back_ptr)
+            managed_component<Component, Wrapper>* back_ptr) noexcept
           : back_ptr_(back_ptr)
         {
             HPX_ASSERT(back_ptr);
@@ -239,17 +234,17 @@ namespace hpx { namespace components {
         using wrapping_type = managed_component<Component, Wrapper>;
         using base_type_holder = Component;
 
-        naming::id_type get_unmanaged_id() const;
-        naming::id_type get_id() const;
+        hpx::id_type get_unmanaged_id() const;
+        hpx::id_type get_id() const;
 
     protected:
         naming::gid_type get_base_gid() const;
 
-    protected:
         template <typename>
         friend struct detail_adl_barrier::init;
 
-        void set_back_ptr(components::managed_component<Component, Wrapper>* bp)
+        void set_back_ptr(
+            components::managed_component<Component, Wrapper>* bp) noexcept
         {
             HPX_ASSERT(nullptr == back_ptr_);
             HPX_ASSERT(bp);
@@ -257,22 +252,24 @@ namespace hpx { namespace components {
         }
 
     private:
-        managed_component<Component, Wrapper>* back_ptr_;
+        managed_component<Component, Wrapper>* back_ptr_ = nullptr;
     };
 
     // reference counting
     template <typename Component, typename Derived>
-    void intrusive_ptr_add_ref(managed_component<Component, Derived>* p)
+    void intrusive_ptr_add_ref(
+        managed_component<Component, Derived>* p) noexcept
     {
         detail_adl_barrier::manage_lifetime<
-            typename traits::managed_component_dtor_policy<Component>::type>::
+            traits::managed_component_dtor_policy_t<Component>>::
             addref(p->component_);
     }
     template <typename Component, typename Derived>
-    void intrusive_ptr_release(managed_component<Component, Derived>* p)
+    void intrusive_ptr_release(
+        managed_component<Component, Derived>* p) noexcept
     {
         detail_adl_barrier::manage_lifetime<
-            typename traits::managed_component_dtor_policy<Component>::type>::
+            traits::managed_component_dtor_policy_t<Component>>::
             release(p->component_);
     }
 
@@ -283,11 +280,11 @@ namespace hpx { namespace components {
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    /// The managed_component template is used as a indirection layer
+    /// The managed_component template is used as an indirection layer
     /// for components allowing to gracefully handle the access to non-existing
     /// components.
     ///
-    /// Additionally it provides memory management capabilities for the
+    /// Additionally, it provides memory management capabilities for the
     /// wrapping instances, and it integrates the memory management with the
     /// AGAS service. Every instance of a managed_component gets assigned
     /// a global id.
@@ -296,19 +293,20 @@ namespace hpx { namespace components {
     /// full network round trip to the AGAS service for each of the allocated
     /// instances.
     ///
-    /// \tparam Component
-    /// \tparam Derived
+    /// \tparam Component Component type
+    /// \tparam Derived Most derived component type
     ///
     template <typename Component, typename Derived>
     class managed_component
     {
     public:
-        HPX_NON_COPYABLE(managed_component);
+        managed_component(managed_component const&) = delete;
+        managed_component(managed_component&&) = delete;
+        managed_component& operator=(managed_component const&) = delete;
+        managed_component& operator=(managed_component&&) = delete;
 
-    public:
-        using derived_type = typename std::conditional<
-            std::is_same<Derived, detail::this_type>::value, managed_component,
-            Derived>::type;
+        using derived_type = std::conditional_t<std::is_void_v<Derived>,
+            managed_component, Derived>;
 
         using wrapped_type = Component;
         using type_holder = Component;
@@ -322,38 +320,43 @@ namespace hpx { namespace components {
         /// wrapped instance. This constructor takes ownership of the
         /// passed pointer.
         ///
-        /// \param c    [in] The pointer to the wrapped instance. The
+        /// \param comp [in] The pointer to the wrapped instance. The
         ///             managed_component takes ownership of this pointer.
         explicit managed_component(Component* comp)
           : component_(comp)
         {
-            detail_adl_barrier::init<
-                typename traits::managed_component_ctor_policy<
-                    Component>::type>::call(component_, this);
+            detail_adl_barrier::
+                init<traits::managed_component_ctor_policy_t<Component>>::call(
+                    component_, this);
             intrusive_ptr_add_ref(this);
         }
 
-    public:
         // Construct a managed_component instance holding a new wrapped instance
-        template <typename... Ts>
-        managed_component(Ts&&... vs)
-          : component_(nullptr)
+        managed_component()
         {
-            detail_adl_barrier::init<
-                typename traits::managed_component_ctor_policy<
-                    Component>::type>::call_new(component_, this,
-                std::forward<Ts>(vs)...);
+            detail_adl_barrier::init<traits::managed_component_ctor_policy_t<
+                Component>>::call_new(component_, this);
+            intrusive_ptr_add_ref(this);
+        }
+
+        template <typename T, typename... Ts,
+            typename Enable = std::enable_if_t<
+                !std::is_same_v<std::decay_t<T>, managed_component>>>
+        explicit managed_component(T&& t, Ts&&... ts)
+        {
+            detail_adl_barrier::init<traits::managed_component_ctor_policy_t<
+                Component>>::call_new(component_, this, HPX_FORWARD(T, t),
+                HPX_FORWARD(Ts, ts)...);
             intrusive_ptr_add_ref(this);
         }
 
-    public:
         // The destructor releases any wrapped instances
         ~managed_component()
         {
             intrusive_ptr_release(this);
             detail_adl_barrier::manage_lifetime<
-                typename traits::managed_component_dtor_policy<
-                    Component>::type>::call(component_);
+                traits::managed_component_dtor_policy_t<Component>>::
+                call(component_);
         }
 
         //finalize() will be called just before the instance gets destructed
@@ -374,7 +377,7 @@ namespace hpx { namespace components {
         {
             if (!component_)
             {
-                HPX_THROW_EXCEPTION(invalid_status,
+                HPX_THROW_EXCEPTION(hpx::error::invalid_status,
                     "managed_component<Component, Derived>::get_checked",
                     "component pointer ({}) is invalid (gid: {})",
                     components::get_component_type_name(
@@ -388,7 +391,7 @@ namespace hpx { namespace components {
         {
             if (!component_)
             {
-                HPX_THROW_EXCEPTION(invalid_status,
+                HPX_THROW_EXCEPTION(hpx::error::invalid_status,
                     "managed_component<Component, Derived>::get_checked",
                     "component pointer ({}) is invalid (gid: {})",
                     components::get_component_type_name(
@@ -398,7 +401,6 @@ namespace hpx { namespace components {
             return get();
         }
 
-    public:
         ///////////////////////////////////////////////////////////////////////
         // The managed_component behaves just like the wrapped object
         Component* operator->()
@@ -424,44 +426,42 @@ namespace hpx { namespace components {
 
         ///////////////////////////////////////////////////////////////////////
         /// \brief Return the global id of this \a future instance
-        naming::id_type get_unmanaged_id() const
+        hpx::id_type get_unmanaged_id() const
         {
-            return naming::id_type(get_base_gid(), naming::id_type::unmanaged);
+            return hpx::id_type(
+                get_base_gid(), hpx::id_type::management_type::unmanaged);
         }
 
-    public:
         naming::gid_type get_base_gid(
             naming::gid_type const& assign_gid = naming::invalid_gid) const
         {
             if (assign_gid)
             {
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "managed_component::get_base_gid",
                     "managed_components must be assigned new gids on creation");
-                return naming::invalid_gid;
             }
 
             return component_heap<managed_component>().get_gid(
                 const_cast<managed_component*>(this));
         }
 
-    public:
         // reference counting
         template <typename C, typename D>
-        friend void intrusive_ptr_add_ref(managed_component<C, D>* p);
+        friend void intrusive_ptr_add_ref(managed_component<C, D>* p) noexcept;
 
         template <typename C, typename D>
-        friend void intrusive_ptr_release(managed_component<C, D>* p);
+        friend void intrusive_ptr_release(managed_component<C, D>* p) noexcept;
 
     protected:
-        Component* component_;
+        Component* component_ = nullptr;
     };
 
     ///////////////////////////////////////////////////////////////////////////
     template <typename Component, typename Wrapper, typename CtorPolicy,
         typename DtorPolicy>
-    inline naming::id_type managed_component_base<Component, Wrapper,
-        CtorPolicy, DtorPolicy>::get_unmanaged_id() const
+    hpx::id_type managed_component_base<Component, Wrapper, CtorPolicy,
+        DtorPolicy>::get_unmanaged_id() const
     {
         HPX_ASSERT(back_ptr_);
         return back_ptr_->get_unmanaged_id();
@@ -469,29 +469,29 @@ namespace hpx { namespace components {
 
     template <typename Component, typename Wrapper, typename CtorPolicy,
         typename DtorPolicy>
-    inline naming::id_type managed_component_base<Component, Wrapper,
-        CtorPolicy, DtorPolicy>::get_id() const
+    hpx::id_type managed_component_base<Component, Wrapper, CtorPolicy,
+        DtorPolicy>::get_id() const
     {
         // all credits should have been taken already
         naming::gid_type gid =
             static_cast<Component const&>(*this).get_base_gid();
 
         // The underlying heap will always give us a full set of credits, but
-        // those are valid for the first invocation of get_base_gid() only.
-        // We have to get rid of those credits and properly replenish those.
+        // those are valid for the first invocation of get_base_gid() only. We
+        // have to get rid of those credits and properly replenish those.
         naming::detail::strip_credits_from_gid(gid);
 
         // any invocation causes the credits to be replenished
         agas::replenish_credits(gid);
-        return naming::id_type(gid, naming::id_type::managed);
+        return hpx::id_type(gid, hpx::id_type::management_type::managed);
     }
 
     template <typename Component, typename Wrapper, typename CtorPolicy,
         typename DtorPolicy>
-    inline naming::gid_type managed_component_base<Component, Wrapper,
-        CtorPolicy, DtorPolicy>::get_base_gid() const
+    naming::gid_type managed_component_base<Component, Wrapper, CtorPolicy,
+        DtorPolicy>::get_base_gid() const
     {
         HPX_ASSERT(back_ptr_);
         return back_ptr_->get_base_gid();
     }
-}}    // namespace hpx::components
+}    // namespace hpx::components
