@@ -1,4 +1,4 @@
-//  Copyright (c) 1998-2021 Hartmut Kaiser
+//  Copyright (c) 1998-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,21 +6,19 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <hpx/components_base/component_type.hpp>
-#include <hpx/components_base/generate_unique_ids.hpp>
 
 #include <cstddef>
 
-namespace hpx { namespace util {
+namespace hpx::util {
 
     struct wrapper_heap_base
     {
         struct heap_parameters
         {
-            std::size_t capacity;
-            std::size_t element_alignment;
-            std::size_t element_size;
+            std::size_t capacity = 0;
+            std::size_t element_alignment = 0;
+            std::size_t element_size = 0;
         };
 
         virtual ~wrapper_heap_base() = default;
@@ -29,11 +27,11 @@ namespace hpx { namespace util {
         virtual bool did_alloc(void* p) const = 0;
         virtual void free(void* p, std::size_t count = 1) = 0;
 
-        virtual naming::gid_type get_gid(util::unique_id_ranges& ids, void* p,
-            components::component_type type) = 0;
+        virtual naming::gid_type get_gid(
+            void* p, components::component_type type) = 0;
 
         virtual std::size_t heap_count() const = 0;
         virtual std::size_t size() const = 0;
         virtual std::size_t free_size() const = 0;
     };
-}}    // namespace hpx::util
+}    // namespace hpx::util
