@@ -1,4 +1,4 @@
-//  Copyright (c) 2020 Hartmut Kaiser
+//  Copyright (c) 2020-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,11 +7,10 @@
 #include <hpx/components_base/component_type.hpp>
 #include <hpx/modules/format.hpp>
 #include <hpx/naming_base/address.hpp>
-#include <hpx/util/ios_flags_saver.hpp>
 
 #include <ostream>
 
-namespace hpx { namespace naming {
+namespace hpx::naming {
 
     // this is defined in this module as its implementation relies on
     // components::get_component_type_name()
@@ -20,4 +19,4 @@ namespace hpx { namespace naming {
         return hpx::util::format_to(os, "({}:{}:{:#x})", addr.locality_,
             components::get_component_type_name(addr.type_), addr.address_);
     }
-}}    // namespace hpx::naming
+}    // namespace hpx::naming
