@@ -1,4 +1,4 @@
-//  Copyright (c) 2016-2017 Hartmut Kaiser
+//  Copyright (c) 2016-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,19 +7,27 @@
 #pragma once
 
 #include <hpx/config.hpp>
-#include <hpx/futures/future_fwd.hpp>
+#include <hpx/compute_local/host/get_targets.hpp>
+
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
+#include <hpx/futures/future_fwd.hpp>
+#include <hpx/modules/compute_local.hpp>
 #include <hpx/modules/naming.hpp>
-#endif
 
 #include <vector>
 
-namespace hpx { namespace compute { namespace host {
+namespace hpx::compute::host::distributed {
+
     struct HPX_EXPORT target;
 
-    HPX_EXPORT std::vector<target> get_local_targets();
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
     HPX_EXPORT hpx::future<std::vector<target>> get_targets(
         hpx::id_type const& locality);
+
+    namespace detail {
+
+        HPX_EXPORT std::vector<host::distributed::target> get_remote_targets(
+            std::vector<host::target> const& targets);
+    }
+}    // namespace hpx::compute::host::distributed
+
 #endif
-}}}    // namespace hpx::compute::host
