@@ -8,5 +8,4 @@
 
 #pragma once
 
-#include <hpx/compute/traits/access_target.hpp>
-#include <hpx/compute/traits/allocator_traits.hpp>
+#include <hpx/compute_local/traits.hpp>
