@@ -6,5 +6,11 @@
 
 #pragma once
 
-#include <hpx/local/algorithm.hpp>
+#include <hpx/config.hpp>
+#include <hpx/parallel/algorithm.hpp>
+#include <hpx/parallel/container_algorithms.hpp>
+#include <hpx/parallel/unseq.hpp>
+#if defined(HPX_HAVE_DATAPAR)
+#include <hpx/parallel/datapar.hpp>
+#endif
 #include <hpx/parallel/segmented_algorithm.hpp>
