@@ -7,8 +7,4 @@
 #pragma once
 
 #include <hpx/collectives/barrier.hpp>
-#include <hpx/local/barrier.hpp>
-
-namespace hpx { namespace distributed {
-    using hpx::lcos::barrier;
-}}    // namespace hpx::distributed
+#include <hpx/synchronization/barrier.hpp>
