@@ -7,8 +7,4 @@
 #pragma once
 
 #include <hpx/lcos_distributed/channel.hpp>
-#include <hpx/local/channel.hpp>
-
-namespace hpx { namespace distributed {
-    using hpx::lcos::channel;
-}}    // namespace hpx::distributed
+#include <hpx/lcos_local/channel.hpp>
