@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2012 Hartmut Kaiser
+//  Copyright (c) 2007-2025 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -6,10 +6,10 @@
 
 #pragma once
 
-#include <hpx/config.hpp>
 #include <hpx/algorithm.hpp>
 #include <hpx/any.hpp>
 #include <hpx/chrono.hpp>
+#include <hpx/collectives.hpp>
 #include <hpx/execution.hpp>
 #include <hpx/functional.hpp>
 #include <hpx/future.hpp>
@@ -17,6 +17,7 @@
 #include <hpx/numeric.hpp>
 #include <hpx/optional.hpp>
 #include <hpx/runtime.hpp>
+#include <hpx/serialization.hpp>
 #include <hpx/task_block.hpp>
 #include <hpx/tuple.hpp>
 #include <hpx/type_traits.hpp>
@@ -27,8 +28,5 @@
 #include <hpx/modules/async_local.hpp>
 #include <hpx/modules/errors.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/include/components.hpp>
 #include <hpx/include/performance_counters.hpp>
-#include <hpx/modules/async_distributed.hpp>
-#endif
