@@ -9,6 +9,4 @@
 #include <hpx/config.hpp>
 #include <hpx/parallel/algorithms/adjacent_difference.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/parallel/segmented_algorithms/adjacent_difference.hpp>
-#endif
