@@ -10,6 +10,4 @@
 #include <hpx/config.hpp>
 #include <hpx/parallel/algorithms/adjacent_find.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/parallel/segmented_algorithms/adjacent_find.hpp>
-#endif
