@@ -11,6 +11,4 @@
 #include <hpx/parallel/algorithms/minmax.hpp>
 #include <hpx/parallel/container_algorithms/minmax.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/parallel/segmented_algorithms/minmax.hpp>
-#endif
