@@ -12,6 +12,4 @@
 #include <hpx/parallel/algorithms/reduce_by_key.hpp>
 #include <hpx/parallel/container_algorithms/reduce.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/parallel/segmented_algorithms/reduce.hpp>
-#endif
