@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2012 Hartmut Kaiser
+//  Copyright (c) 2007-2021 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -7,6 +7,6 @@
 #pragma once
 
 #include <hpx/config.hpp>
-#include <hpx/runtime/parcelset/parcel.hpp>
-#include <hpx/runtime/parcelset/parcelhandler.hpp>
-#include <hpx/runtime/parcelset/parcelport.hpp>
+#include <hpx/parcelset/parcel.hpp>
+#include <hpx/parcelset/parcelhandler.hpp>
+#include <hpx/parcelset_base/parcelport.hpp>
