@@ -12,7 +12,6 @@
 #include <hpx/include/threadmanager.hpp>
 #include <hpx/runtime_local/runtime_local.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/include/actions.hpp>
 #include <hpx/include/applier.hpp>
 #include <hpx/include/components.hpp>
@@ -20,4 +19,3 @@
 #include <hpx/include/parcelset.hpp>
 #include <hpx/modules/runtime_distributed.hpp>
 #include <hpx/runtime_distributed.hpp>
-#endif
