@@ -9,6 +9,4 @@
 #include <hpx/config.hpp>
 #include <hpx/async_local/sync.hpp>
 
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
 #include <hpx/async_distributed/sync.hpp>
-#endif
