@@ -6,5 +6,6 @@
 
 #pragma once
 
-#include <hpx/local/runtime.hpp>
 #include <hpx/runtime_distributed/runtime_fwd.hpp>
+#include <hpx/runtime_local/runtime_local_fwd.hpp>
+#include <hpx/runtime_local/service_executors.hpp>
