@@ -13,13 +13,21 @@
 namespace hpx {
     /// \cond NOINTERNAL
     namespace detail {
+
         HPX_EXPORT int run_or_start(
-            util::function_nonser<int(
-                hpx::program_options::variables_map& vm)> const& f,
+            hpx::function<int(hpx::program_options::variables_map& vm)> const&
+                f,
             int argc, char** argv, init_params const& params, bool blocking);
 
-        HPX_EXPORT int init_helper(hpx::program_options::variables_map&,
-            util::function_nonser<int(int, char**)> const&);
+        HPX_EXPORT int init_impl(
+            hpx::function<int(hpx::program_options::variables_map&)> const& f,
+            int argc, char** argv, init_params const& params,
+            char const* hpx_prefix, char** env);
+
+        HPX_EXPORT bool start_impl(
+            hpx::function<int(hpx::program_options::variables_map&)> const& f,
+            int argc, char** argv, init_params const& params,
+            char const* hpx_prefix, char** env);
     }    // namespace detail
     /// \endcond
 }    // namespace hpx
