@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2017 Shoshana Jakobovits
 //  Copyright (c) 2010-2011 Phillip LeBlanc, Dylan Stark
 //  Copyright (c)      2011 Bryce Lelbach
@@ -8,22 +8,24 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <hpx/hpx_init.hpp>
+#include <algorithm>
 
 #include <hpx/assert.hpp>
 #include <hpx/command_line_handling/command_line_handling.hpp>
 #include <hpx/coroutines/detail/context_impl.hpp>
 #include <hpx/execution/detail/execution_parameter_callbacks.hpp>
-#include <hpx/execution_base/register_locks.hpp>
 #include <hpx/executors/exception_list.hpp>
 #include <hpx/functional/bind_front.hpp>
 #include <hpx/functional/function.hpp>
 #include <hpx/futures/detail/future_data.hpp>
 #include <hpx/hpx_finalize.hpp>
+#include <hpx/hpx_main_winsocket.hpp>
 #include <hpx/hpx_suspend.hpp>
 #include <hpx/hpx_user_main_config.hpp>
 #include <hpx/init_runtime/detail/init_logging.hpp>
 #include <hpx/init_runtime/detail/run_or_start.hpp>
 #include <hpx/init_runtime_local/init_runtime_local.hpp>
+#include <hpx/lock_registration/detail/register_locks.hpp>
 #include <hpx/modules/errors.hpp>
 #include <hpx/modules/filesystem.hpp>
 #include <hpx/modules/format.hpp>
@@ -32,6 +34,7 @@
 #include <hpx/modules/testing.hpp>
 #include <hpx/modules/timing.hpp>
 #include <hpx/parallel/util/detail/handle_exception_termination_handler.hpp>
+#include <hpx/prefix/find_prefix.hpp>
 #include <hpx/program_options/parsers.hpp>
 #include <hpx/program_options/variables_map.hpp>
 #include <hpx/resource_partitioner/partitioner.hpp>
@@ -49,6 +52,7 @@
 #include <hpx/string_util/classification.hpp>
 #include <hpx/string_util/split.hpp>
 #include <hpx/threading/thread.hpp>
+#include <hpx/threading_base/detail/get_default_timer_service.hpp>
 #include <hpx/type_support/pack.hpp>
 #include <hpx/type_support/unused.hpp>
 #include <hpx/util/from_string.hpp>
@@ -63,27 +67,26 @@
 #include <hpx/init_runtime/pre_main.hpp>
 #include <hpx/modules/async_distributed.hpp>
 #include <hpx/modules/naming.hpp>
+#if defined(HPX_HAVE_NETWORKING)
+#include <hpx/parcelports/init_all_parcelports.hpp>
+#include <hpx/parcelset/parcelhandler.hpp>
+#include <hpx/parcelset_base/locality_interface.hpp>
+#endif
 #include <hpx/performance_counters/counters.hpp>
 #include <hpx/performance_counters/query_counters.hpp>
-#include <hpx/runtime_configuration/register_locks_globally.hpp>
 #include <hpx/runtime_distributed.hpp>
-#include <hpx/runtime_distributed/find_localities.hpp>
 #include <hpx/runtime_distributed/runtime_fwd.hpp>
 #include <hpx/runtime_distributed/runtime_support.hpp>
 #endif
 
-#if defined(HPX_NATIVE_MIC) || defined(__bgq__)
-#include <cstdlib>
-#endif
-
 #include <cmath>
 #include <cstddef>
+#include <cstdlib>
 #include <exception>
 #include <functional>
 #include <iostream>
 #include <map>
 #include <memory>
-#include <new>
 #include <sstream>
 #include <string>
 #include <utility>
@@ -95,32 +98,117 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace hpx_startup {
+
     std::vector<std::string> (*user_main_config_function)(
         std::vector<std::string> const&) = nullptr;
-}
+}    // namespace hpx_startup
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace detail {
+namespace hpx::detail {
+
     // forward declarations only
     void console_print(std::string const&);
-}}    // namespace hpx::detail
+
+    int init_impl(
+        hpx::function<int(hpx::program_options::variables_map&)> const& f,
+        int argc, char** argv, init_params const& params,
+        char const* hpx_prefix, [[maybe_unused]] char** env)
+    {
+        if (argc == 0 || argv == nullptr)
+        {
+            argc = hpx::local::detail::dummy_argc;
+            argv = hpx::local::detail::dummy_argv;
+        }
+
+#if defined(HPX_WINDOWS)
+        detail::init_winsocket();
+#if defined(HPX_HAVE_APEX)
+        // artificially force the apex shared library to be loaded by the
+        // application
+        apex::version();
+#endif
+#endif
+#if defined(HPX_HAVE_NETWORKING)
+        // force linking parcelports
+        hpx::parcelset::init_all_parcelports();
+#endif
+        util::set_hpx_prefix(hpx_prefix);
+#if defined(__FreeBSD__)
+        freebsd_environ = env;
+#endif
+        // set a handler for std::at_quick_exit, and std::atexit
+        [[maybe_unused]] auto const ret_at_exit = std::atexit(detail::on_exit);
+        HPX_ASSERT(ret_at_exit == 0);
+
+#if defined(HPX_HAVE_CXX11_STD_QUICK_EXIT)
+        [[maybe_unused]] auto const ret_at_quick_exit =
+            std::at_quick_exit(detail::on_exit);
+        HPX_ASSERT(ret_at_quick_exit == 0);
+#endif
+        return detail::run_or_start(f, argc, argv, params, true);
+    }
+
+    bool start_impl(
+        hpx::function<int(hpx::program_options::variables_map&)> const& f,
+        int argc, char** argv, init_params const& params,
+        char const* hpx_prefix, [[maybe_unused]] char** env)
+    {
+        if (argc == 0 || argv == nullptr)
+        {
+            argc = local::detail::dummy_argc;
+            argv = local::detail::dummy_argv;
+        }
+
+#if defined(HPX_WINDOWS)
+        detail::init_winsocket();
+#if defined(HPX_HAVE_APEX)
+        // artificially force the apex shared library to be loaded by the
+        // application
+        apex::version();
+#endif
+#endif
+#if defined(HPX_HAVE_NETWORKING)
+        // force linking parcelports
+        hpx::parcelset::init_all_parcelports();
+#endif
+        util::set_hpx_prefix(hpx_prefix);
+#if defined(__FreeBSD__)
+        freebsd_environ = env;
+#endif
+        // set a handler for std::abort, std::at_quick_exit, and std::atexit
+        [[maybe_unused]] auto const prev_signal =
+            std::signal(SIGABRT, detail::on_abort);
+        HPX_ASSERT(prev_signal != SIG_ERR);
+
+        [[maybe_unused]] auto const ret_atexit = std::atexit(detail::on_exit);
+        HPX_ASSERT(ret_atexit == 0);
+
+#if defined(HPX_HAVE_CXX11_STD_QUICK_EXIT)
+        [[maybe_unused]] auto const ret_at_quick_exit =
+            std::at_quick_exit(detail::on_exit);
+        HPX_ASSERT(ret_at_quick_exit == 0);
+#endif
+        return 0 == detail::run_or_start(f, argc, argv, params, false);
+    }
+}    // namespace hpx::detail
 
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
-namespace hpx { namespace detail {
+namespace hpx::detail {
+
     // forward declarations only
     void list_symbolic_name(std::string const&, hpx::id_type const&);
     void list_component_type(std::string const&, components::component_type);
-}}    // namespace hpx::detail
+}    // namespace hpx::detail
 
 HPX_PLAIN_ACTION_ID(hpx::detail::console_print, console_print_action,
     hpx::actions::console_print_action_id)
 HPX_PLAIN_ACTION_ID(hpx::detail::list_component_type,
     list_component_type_action, hpx::actions::list_component_type_action_id)
 
-typedef hpx::util::detail::bound_action<list_component_type_action,
-    hpx::util::index_pack<0, 1, 2>, hpx::naming::id_type,
-    hpx::util::detail::placeholder<1>, hpx::util::detail::placeholder<2>>
-    bound_list_component_type_action;
+using bound_list_component_type_action =
+    hpx::detail::bound_action<list_component_type_action,
+        hpx::util::index_pack<0, 1, 2>, hpx::id_type,
+        hpx::detail::placeholder<1>, hpx::detail::placeholder<2>>;
 
 HPX_UTIL_REGISTER_FUNCTION_DECLARATION(
     void(std::string const&, hpx::components::component_type),
@@ -131,7 +219,8 @@ HPX_UTIL_REGISTER_FUNCTION(
     bound_list_component_type_action, list_component_type_function)
 #endif
 
-namespace hpx { namespace detail {
+namespace hpx::detail {
+
     ///////////////////////////////////////////////////////////////////////////
     // print string on the console
     void console_print(std::string const& name)
@@ -142,7 +231,7 @@ namespace hpx { namespace detail {
     inline void print(std::string const& name, error_code& ec = throws)
     {
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
-        naming::id_type console(agas::get_console_locality(ec));
+        hpx::id_type console(agas::get_console_locality(ec));
         if (ec)
             return;
 
@@ -180,8 +269,8 @@ namespace hpx { namespace detail {
     {
         // list all counter names
         list_counter_names_header(true);
-        performance_counters::discover_counter_types(
-            &list_counter, performance_counters::discover_counters_minimal);
+        performance_counters::discover_counter_types(&list_counter,
+            performance_counters::discover_counters_mode::minimal);
     }
 
     void list_counter_names_full()
@@ -189,7 +278,7 @@ namespace hpx { namespace detail {
         // list all counter names
         list_counter_names_header(false);
         performance_counters::discover_counter_types(
-            &list_counter, performance_counters::discover_counters_full);
+            &list_counter, performance_counters::discover_counters_mode::full);
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -233,23 +322,24 @@ namespace hpx { namespace detail {
         // list all counter information
         list_counter_infos_header(true);
         performance_counters::discover_counter_types(&list_counter_info,
-            performance_counters::discover_counters_minimal);
+            performance_counters::discover_counters_mode::minimal);
     }
 
     void list_counter_infos_full()
     {
         // list all counter information
         list_counter_infos_header(false);
-        performance_counters::discover_counter_types(
-            &list_counter_info, performance_counters::discover_counters_full);
+        performance_counters::discover_counter_types(&list_counter_info,
+            performance_counters::discover_counters_mode::full);
     }
 
     ///////////////////////////////////////////////////////////////////////////
     void list_symbolic_name(std::string const& name, hpx::id_type const& id)
     {
-        std::string str = hpx::util::format("{}, {}, {}", name, id,
-            (id.get_management_type() == id_type::managed ? "managed" :
-                                                            "unmanaged"));
+        std::string const str = hpx::util::format("{}, {}, {}", name, id,
+            (id.get_management_type() == id_type::management_type::managed ?
+                    "management_type::managed" :
+                    "management_type::unmanaged"));
         print(str);
     }
 
@@ -258,7 +348,7 @@ namespace hpx { namespace detail {
         print(std::string("List of all registered symbolic names:"));
         print(std::string("--------------------------------------"));
 
-        std::map<std::string, hpx::id_type> entries =
+        std::map<std::string, hpx::id_type> const entries =
             agas::find_symbols(hpx::launch::sync);
 
         for (auto const& e : entries)
@@ -280,12 +370,12 @@ namespace hpx { namespace detail {
         print(std::string("List of all registered component types:"));
         print(std::string("---------------------------------------"));
 
-        using hpx::util::placeholders::_1;
-        using hpx::util::placeholders::_2;
+        using hpx::placeholders::_1;
+        using hpx::placeholders::_2;
 
-        naming::id_type console(agas::get_console_locality());
+        hpx::id_type console(agas::get_console_locality());
         naming::get_agas_client().iterate_types(
-            hpx::util::bind<list_component_type_action>(console, _1, _2));
+            hpx::bind<list_component_type_action>(console, _1, _2));
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -304,7 +394,7 @@ namespace hpx { namespace detail {
         }
     }
 #endif
-}}    // namespace hpx::detail
+}    // namespace hpx::detail
 
 ///////////////////////////////////////////////////////////////////////////////
 #if defined(HPX_HAVE_DYNAMIC_HPX_MAIN) &&                                      \
@@ -320,6 +410,7 @@ namespace hpx_start {
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace hpx {
+
     // Print stack trace and exit.
 #if defined(HPX_WINDOWS)
     extern BOOL WINAPI termination_handler(DWORD ctrl_type);
@@ -350,17 +441,6 @@ namespace hpx {
                 util::disable_lock_detection();
             }
 #endif
-#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME) &&                                   \
-    defined(HPX_HAVE_VERIFY_LOCKS_GLOBALLY)
-            if (cmdline.rtcfg_.enable_global_lock_detection())
-            {
-                util::enable_global_lock_detection();
-            }
-            else
-            {
-                util::disable_global_lock_detection();
-            }
-#endif
 #ifdef HPX_HAVE_THREAD_MINIMAL_DEADLOCK_DETECTION
             threads::policies::set_minimal_deadlock_detection_enabled(
                 cmdline.rtcfg_.enable_minimal_deadlock_detection());
@@ -395,9 +475,13 @@ namespace hpx {
             hpx::program_options::variables_map& vm,
             bool print_counters_locally)
         {
-            runtime_distributed* rtd =
-                dynamic_cast<hpx::runtime_distributed*>(&rt);
-            HPX_ASSERT(rtd != nullptr);
+            auto* rtd = dynamic_cast<hpx::runtime_distributed*>(&rt);
+            if (rtd == nullptr)
+            {
+                throw detail::command_line_error(
+                    "Unexpected: runtime system was not initialized.");
+            }
+
             if (vm.count("hpx:list-counters"))
             {
                 // Print the names of all registered performance counters.
@@ -476,10 +560,10 @@ namespace hpx {
                         vm["hpx:print-counter-format"].as<std::string>();
                     if (counter_format == "csv-short")
                     {
-                        for (std::size_t i = 0; i != counters.size(); ++i)
+                        for (auto& counter : counters)
                         {
                             std::vector<std::string> entry;
-                            hpx::string_util::split(entry, counters[i],
+                            hpx::string_util::split(entry, counter,
                                 hpx::string_util::is_any_of(","),
                                 hpx::string_util::token_compress_mode::on);
 
@@ -492,7 +576,7 @@ namespace hpx {
                             }
 
                             counter_shortnames.push_back(entry[0]);
-                            counters[i] = entry[1];
+                            counter = entry[1];
                         }
                     }
                 }
@@ -522,13 +606,13 @@ namespace hpx {
                 if (get_config_entry("hpx.print_counter.shutdown", "0") == "1")
                 {
                     // schedule to run at shutdown
-                    rt.add_pre_shutdown_function(util::bind_front(
+                    rt.add_pre_shutdown_function(hpx::bind_front(
                         &util::query_counters::evaluate, qc, true));
                 }
 
                 // schedule to start all counters
 
-                rt.add_startup_function(util::bind_front(&start_counters, qc));
+                rt.add_startup_function(hpx::bind_front(&start_counters, qc));
 
                 // register the query_counters object with the runtime system
                 rtd->register_query_counters(qc);
@@ -538,16 +622,14 @@ namespace hpx {
                 throw detail::command_line_error(
                     "Invalid command line option "
                     "--hpx:print-counter-interval, valid in conjunction "
-                    "with "
-                    "--hpx:print-counter only");
+                    "with --hpx:print-counter only");
             }
             else if (vm.count("hpx:print-counter-destination"))
             {
                 throw detail::command_line_error(
                     "Invalid command line option "
                     "--hpx:print-counter-destination, valid in conjunction "
-                    "with "
-                    "--hpx:print-counter only");
+                    "with --hpx:print-counter only");
             }
             else if (vm.count("hpx:print-counter-format"))
             {
@@ -579,20 +661,21 @@ namespace hpx {
         {
             if (vm.count("hpx:app-config"))
             {
-                std::string config(vm["hpx:app-config"].as<std::string>());
+                std::string const config(
+                    vm["hpx:app-config"].as<std::string>());
                 rt.get_config().load_application_configuration(config.c_str());
             }
 
             if (!!startup)
-                rt.add_startup_function(std::move(startup));
+                rt.add_startup_function(HPX_MOVE(startup));
 
             if (!!shutdown)
-                rt.add_shutdown_function(std::move(shutdown));
+                rt.add_shutdown_function(HPX_MOVE(shutdown));
 
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
             // Add startup function related to listing counter names or counter
             // infos (on console only).
-            bool print_counters_locally =
+            bool const print_counters_locally =
                 vm.count("hpx:print-counters-locally") != 0;
             if (mode == runtime_mode::console || print_counters_locally)
                 handle_list_and_print_options(rt, vm, print_counters_locally);
@@ -616,42 +699,42 @@ namespace hpx {
 
         ///////////////////////////////////////////////////////////////////////
         int run(hpx::runtime& rt,
-            util::function_nonser<int(
-                hpx::program_options::variables_map& vm)> const& f,
+            hpx::function<int(hpx::program_options::variables_map& vm)> const&
+                f,
             hpx::program_options::variables_map& vm, runtime_mode mode,
             startup_function_type startup, shutdown_function_type shutdown)
         {
             LPROGRESS_;
 
             add_startup_functions(
-                rt, vm, mode, std::move(startup), std::move(shutdown));
+                rt, vm, mode, HPX_MOVE(startup), HPX_MOVE(shutdown));
 
             // Run this runtime instance using the given function f.
             if (!f.empty())
-                return rt.run(util::bind_front(f, vm));
+                return rt.run(hpx::bind_front(f, vm));
 
-            // Run this runtime instance without an hpx_main
+            // Run this runtime instance without a hpx_main
             return rt.run();
         }
 
         int start(hpx::runtime& rt,
-            util::function_nonser<int(
-                hpx::program_options::variables_map& vm)> const& f,
+            hpx::function<int(hpx::program_options::variables_map& vm)> const&
+                f,
             hpx::program_options::variables_map& vm, runtime_mode mode,
             startup_function_type startup, shutdown_function_type shutdown)
         {
             LPROGRESS_;
 
             add_startup_functions(
-                rt, vm, mode, std::move(startup), std::move(shutdown));
+                rt, vm, mode, HPX_MOVE(startup), HPX_MOVE(shutdown));
 
             if (!f.empty())
             {
                 // Run this runtime instance using the given function f.
-                return rt.start(util::bind_front(f, vm));
+                return rt.start(hpx::bind_front(f, vm));
             }
 
-            // Run this runtime instance without an hpx_main
+            // Run this runtime instance without a hpx_main
             return rt.start();
         }
 
@@ -662,22 +745,23 @@ namespace hpx {
             if (blocking)
             {
                 return run(*rt, cfg.hpx_main_f_, cfg.vm_, cfg.rtcfg_.mode_,
-                    std::move(startup), std::move(shutdown));
+                    HPX_MOVE(startup), HPX_MOVE(shutdown));
             }
 
             // non-blocking version
             start(*rt, cfg.hpx_main_f_, cfg.vm_, cfg.rtcfg_.mode_,
-                std::move(startup), std::move(shutdown));
+                HPX_MOVE(startup), HPX_MOVE(shutdown));
 
             // pointer to runtime is stored in TLS
-            hpx::runtime* p = rt.release();
-            (void) p;
+            [[maybe_unused]] hpx::runtime const* p = rt.release();
+            HPX_ASSERT(p != nullptr);
 
             return 0;
         }
 
         ////////////////////////////////////////////////////////////////////////
-        void init_environment()
+        void init_environment(
+            [[maybe_unused]] hpx::util::runtime_configuration const& cfg)
         {
             HPX_UNUSED(hpx::filesystem::initial_path());
 
@@ -707,7 +791,7 @@ namespace hpx {
                 &detail::register_locks_predicate);
 #endif
 #if !defined(HPX_HAVE_DISABLED_SIGNAL_EXCEPTION_HANDLERS)
-            set_error_handlers();
+            set_error_handlers(cfg);
 #endif
             hpx::threads::detail::set_get_default_pool(
                 &detail::get_default_pool);
@@ -718,11 +802,21 @@ namespace hpx {
                 &hpx::detail::get_pu_mask);
             hpx::parallel::execution::detail::set_get_os_thread_count(
                 []() { return hpx::get_os_thread_count(); });
-            hpx::parallel::v1::detail::set_exception_list_termination_handler(
+            hpx::parallel::detail::set_exception_list_termination_handler(
                 &hpx::terminate);
             hpx::parallel::util::detail::
                 set_parallel_exception_termination_handler(&hpx::terminate);
 
+            // instantiate the interface function initialization objects
+#if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
+#if defined(HPX_HAVE_NETWORKING)
+            parcelset::locality_init();
+#endif
+            agas::agas_init();
+            agas::runtime_components_init();
+            components::counter_init();
+#endif
+
 #if defined(HPX_NATIVE_MIC) || defined(__bgq__) || defined(__bgqion__)
             unsetenv("LANG");
             unsetenv("LC_CTYPE");
@@ -769,13 +863,12 @@ namespace hpx {
         }
 
         ///////////////////////////////////////////////////////////////////////
-        int run_or_start(util::function_nonser<int(
-                             hpx::program_options::variables_map& vm)> const& f,
+        int run_or_start(
+            hpx::function<int(hpx::program_options::variables_map& vm)> const&
+                f,
             int argc, char** argv, init_params const& params, bool blocking)
         {
-            init_environment();
-
-            int result = 0;
+            int result;
             try
             {
                 // make sure the runtime system is not active yet
@@ -784,22 +877,31 @@ namespace hpx {
                     return result;
                 }
 
+#if defined(HPX_HAVE_NETWORKING)
                 hpx::util::command_line_handling cmdline{
-                    hpx::util::runtime_configuration(argv[0], params.mode),
+                    hpx::util::runtime_configuration(argv[0], params.mode,
+                        hpx::parcelset::load_runtime_configuration()),
                     hpx_startup::user_main_config(params.cfg), f};
+#else
+                hpx::util::command_line_handling cmdline{
+                    hpx::util::runtime_configuration(argv[0], params.mode, {}),
+                    hpx_startup::user_main_config(params.cfg), f};
+#endif
+
+                std::vector<
+                    std::shared_ptr<components::component_registry_base>>
+                    component_registries;
 
                 // scope exception handling to resource partitioner initialization
                 // any exception thrown during run_or_start below are handled
                 // separately
                 try
                 {
-                    std::vector<
-                        std::shared_ptr<components::component_registry_base>>
-                        component_registries;
-
                     result = cmdline.call(
                         params.desc_cmdline, argc, argv, component_registries);
 
+                    init_environment(cmdline.rtcfg_);
+
                     hpx::threads::policies::detail::affinity_data
                         affinity_data{};
                     affinity_data.init(hpx::util::get_entry_as<std::size_t>(
@@ -815,19 +917,12 @@ namespace hpx {
                         cmdline.rtcfg_.get_entry("hpx.affinity", ""),
                         cmdline.rtcfg_.get_entry("hpx.bind", ""),
                         hpx::util::get_entry_as<bool>(
-                            cmdline.rtcfg_, "hpx.use_process_mask", 0));
+                            cmdline.rtcfg_, "hpx.use_process_mask", false));
 
                     hpx::resource::partitioner rp =
                         hpx::resource::detail::make_partitioner(
                             params.rp_mode, cmdline.rtcfg_, affinity_data);
 
-                    for (auto& registry : component_registries)
-                    {
-                        hpx::register_startup_function([registry]() {
-                            registry->register_component_type();
-                        });
-                    }
-
                     activate_global_options(cmdline, argc, argv);
 
                     // check whether HPX should be exited at this point
@@ -835,8 +930,7 @@ namespace hpx {
                     // contain --hpx:help or --hpx:version, on error result is < 0)
                     if (result != 0)
                     {
-                        if (result > 0)
-                            result = 0;
+                        result = (std::min)(result, 0);
                         return result;
                     }
 
@@ -846,6 +940,13 @@ namespace hpx {
                         params.rp_callback(rp, cmdline.vm_);
                     }
 
+#if defined(HPX_HAVE_NETWORKING)
+                    if (cmdline.num_localities_ != 1 || cmdline.node_ != 0 ||
+                        cmdline.rtcfg_.enable_networking())
+                    {
+                        parcelset::parcelhandler::init(rp);
+                    }
+#endif
                     // Setup all internal parameters of the resource_partitioner
                     rp.configure_pools();
                 }
@@ -864,23 +965,27 @@ namespace hpx {
 
                 // Command line handling should have updated this by now.
                 HPX_ASSERT(cmdline.rtcfg_.mode_ != runtime_mode::default_);
-                switch (cmdline.rtcfg_.mode_)
-                {
-                case runtime_mode::local:
+                if (cmdline.rtcfg_.mode_ == runtime_mode::local)
                 {
                     LPROGRESS_ << "creating local runtime";
-                    rt.reset(new hpx::runtime(cmdline.rtcfg_));
-                    break;
+                    rt.reset(new hpx::runtime(cmdline.rtcfg_, true));
                 }
-                default:
+                else
                 {
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
+                    for (auto const& registry : component_registries)
+                    {
+                        hpx::register_startup_function([registry]() {
+                            registry->register_component_type();
+                        });
+                    }
+
                     LPROGRESS_ << "creating distributed runtime";
-                    rt.reset(new hpx::runtime_distributed(
-                        cmdline.rtcfg_, &hpx::detail::pre_main));
-                    break;
+                    rt.reset(new hpx::runtime_distributed(cmdline.rtcfg_,
+                        &hpx::detail::pre_main, &hpx::detail::post_main));
 #else
-                    HPX_THROW_EXCEPTION(invalid_status, "run_or_start",
+                    HPX_THROW_EXCEPTION(hpx::error::invalid_status,
+                        "run_or_start",
                         "Attempted to start the runtime in the mode \"{1}\", "
                         "but HPX was compiled with "
                         "HPX_WITH_DISTRIBUTED_RUNTIME=OFF, and \"{1}\" "
@@ -888,13 +993,14 @@ namespace hpx {
                         "Recompile HPX with HPX_WITH_DISTRIBUTED_RUNTIME=ON or "
                         "change the runtime mode.",
                         get_runtime_mode_name(cmdline.rtcfg_.mode_));
-                    break;
 #endif
                 }
-                }
 
-                result = run_or_start(blocking, std::move(rt), cmdline,
-                    std::move(params.startup), std::move(params.shutdown));
+                // Store application defined command line options
+                rt->set_app_options(params.desc_cmdline);
+
+                result = run_or_start(blocking, HPX_MOVE(rt), cmdline,
+                    params.startup, params.shutdown);
             }
             catch (detail::command_line_error const& e)
             {
@@ -918,7 +1024,7 @@ namespace hpx {
                 }
                 catch (hpx::util::bad_lexical_cast const&)
                 {
-                    ;    // do nothing
+                    // do nothing
                 }
             }
             return default_;
@@ -930,14 +1036,14 @@ namespace hpx {
     {
         if (!threads::get_self_ptr())
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::finalize",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::finalize",
                 "this function can be called from an HPX thread only");
             return -1;
         }
 
         if (!is_running())
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::finalize",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::finalize",
                 "the runtime system is not active (did you already "
                 "call finalize?)");
             return -1;
@@ -950,9 +1056,9 @@ namespace hpx {
             localwait = detail::get_option("hpx.finalize_wait_time", -1.0);
 
         {
-            hpx::chrono::high_resolution_timer t;
-            double start_time = t.elapsed();
-            double current = 0.0;
+            hpx::chrono::high_resolution_timer const t;
+            double const start_time = t.elapsed();
+            double current;
             do
             {
                 current = t.elapsed();
@@ -965,7 +1071,7 @@ namespace hpx {
         runtime* rt = get_runtime_ptr();
         if (nullptr == rt)
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::finalize",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::finalize",
                 "the runtime system is not active (did you already "
                 "call hpx::stop?)");
             return -1;
@@ -973,6 +1079,11 @@ namespace hpx {
 
         rt->finalize(shutdown_timeout);
 
+        // invoke user supplied finalizer
+        if (hpx::on_finalize != nullptr)
+        {
+            (*hpx::on_finalize)();
+        }
         return 0;
     }
 
@@ -982,14 +1093,14 @@ namespace hpx {
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
         if (!threads::get_self_ptr())
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::disconnect",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::disconnect",
                 "this function can be called from an HPX thread only");
             return -1;
         }
 
         if (!is_running())
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::disconnect",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::disconnect",
                 "the runtime system is not active (did you already "
                 "call finalize?)");
             return -1;
@@ -1002,9 +1113,9 @@ namespace hpx {
             localwait = detail::get_option("hpx.finalize_wait_time", -1.0);
 
         {
-            hpx::chrono::high_resolution_timer t;
-            double start_time = t.elapsed();
-            double current = 0.0;
+            hpx::chrono::high_resolution_timer const t;
+            double const start_time = t.elapsed();
+            double current;
             do
             {
                 current = t.elapsed();
@@ -1014,13 +1125,12 @@ namespace hpx {
         if (std::abs(shutdown_timeout + 1.0) < 1e-16)
             shutdown_timeout = detail::get_option("hpx.shutdown_timeout", -1.0);
 
-        components::server::runtime_support* p =
-            reinterpret_cast<components::server::runtime_support*>(
-                get_runtime_distributed().get_runtime_support_lva());
+        auto* p = static_cast<components::server::runtime_support*>(
+            get_runtime_distributed().get_runtime_support_lva());
 
         if (nullptr == p)
         {
-            HPX_THROWS_IF(ec, invalid_status, "hpx::disconnect",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status, "hpx::disconnect",
                 "the runtime system is not active (did you already "
                 "call finalize?)");
             return -1;
@@ -1029,7 +1139,7 @@ namespace hpx {
         p->call_shutdown_functions(true);
         p->call_shutdown_functions(false);
 
-        p->stop(shutdown_timeout, naming::invalid_id, true);
+        p->stop(shutdown_timeout, hpx::invalid_id, true);
 #else
         HPX_UNUSED(shutdown_timeout);
         HPX_UNUSED(localwait);
@@ -1049,9 +1159,8 @@ namespace hpx {
         }
 
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME)
-        components::server::runtime_support* p =
-            reinterpret_cast<components::server::runtime_support*>(
-                get_runtime_distributed().get_runtime_support_lva());
+        auto* p = static_cast<components::server::runtime_support*>(
+            get_runtime_distributed().get_runtime_support_lva());
 
         if (nullptr == p)
         {
