@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2014 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //  Copyright (c) 2011 Bryce Lelbach
 //  Copyright (c) 2007 Richard D. Guidry Jr.
 //
@@ -9,17 +9,19 @@
 #pragma once
 
 #include <hpx/config.hpp>
-#include <hpx/datastructures/any.hpp>
-#include <hpx/futures/future.hpp>
+#include <hpx/modules/async_base.hpp>
+#include <hpx/modules/futures.hpp>
 #include <hpx/naming_base/gid_type.hpp>
-#include <hpx/synchronization/spinlock.hpp>
 
-#include <map>
 #include <mutex>
 
-namespace hpx { namespace naming { namespace detail {
+namespace hpx::naming::detail {
 
-    HPX_EXPORT hpx::future<gid_type> split_gid_if_needed(gid_type& id);
-    HPX_EXPORT hpx::future<gid_type> split_gid_if_needed_locked(
+    HPX_EXPORT hpx::future_or_value<gid_type> split_gid_if_needed(gid_type& id);
+
+    HPX_EXPORT gid_type split_gid_if_needed(
+        hpx::launch::sync_policy, gid_type& id);
+
+    HPX_EXPORT hpx::future_or_value<gid_type> split_gid_if_needed_locked(
         std::unique_lock<gid_type::mutex_type>& l, gid_type& gid);
-}}}    // namespace hpx::naming::detail
+}    // namespace hpx::naming::detail
