@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //  Copyright (c) 2011 Bryce Lelbach
 //  Copyright (c) 2007 Richard D. Guidry Jr.
 //
@@ -11,8 +11,8 @@
 #include <hpx/config.hpp>
 #include <hpx/assert.hpp>
 #include <hpx/concurrency/spinlock_pool.hpp>
-#include <hpx/execution_base/register_locks.hpp>
 #include <hpx/execution_base/this_thread.hpp>
+#include <hpx/lock_registration/detail/register_locks.hpp>
 #include <hpx/modules/itt_notify.hpp>
 #include <hpx/naming_base/naming_base.hpp>
 #include <hpx/serialization/serialization_fwd.hpp>
@@ -32,12 +32,16 @@
 #define HPX_GIDTYPE_VERSION 0x10
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace naming {
+namespace hpx::naming {
 
     namespace detail {
 
         constexpr bool is_locked(gid_type const& gid) noexcept;
-    }
+
+        ///////////////////////////////////////////////////////////////////////
+        constexpr std::uint64_t strip_internal_bits_from_gid(
+            std::uint64_t msb) noexcept;
+    }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
     HPX_EXPORT gid_type operator+(
@@ -58,7 +62,8 @@ namespace hpx { namespace naming {
 
         static constexpr std::uint64_t credit_mask = credit_base_mask
             << credit_shift;
-        static constexpr std::uint64_t was_split_mask = 0x80000000ull;
+        static constexpr std::uint64_t was_split_mask =
+            0x80000000ull;    //-V112
         static constexpr std::uint64_t has_credits_mask = 0x40000000ull;
         static constexpr std::uint64_t is_locked_mask = 0x20000000ull;
 
@@ -99,8 +104,16 @@ namespace hpx { namespace naming {
         {
         }
 
-        explicit inline gid_type(
+        explicit gid_type(void* lsb_id) noexcept
+          : id_msb_(0)
+          , id_lsb_(reinterpret_cast<std::uint64_t>(lsb_id))
+        {
+        }
+
+        explicit constexpr gid_type(
             std::uint64_t msb_id, std::uint64_t lsb_id) noexcept;
+        explicit inline gid_type(std::uint64_t msb_id, void* lsb_id) noexcept;
+
         inline constexpr gid_type(gid_type const& rhs) noexcept;
         inline constexpr gid_type(gid_type&& rhs) noexcept;
 
@@ -192,9 +205,18 @@ namespace hpx { namespace naming {
         }
 
         // comparison is required as well
-        friend HPX_EXPORT bool operator==(
-            gid_type const& lhs, gid_type const& rhs) noexcept;
-        friend bool operator!=(
+        friend constexpr bool operator==(
+            gid_type const& lhs, gid_type const& rhs) noexcept
+        {
+            std::int64_t lhs_msb =
+                detail::strip_internal_bits_from_gid(lhs.id_msb_);
+            std::int64_t rhs_msb =
+                detail::strip_internal_bits_from_gid(rhs.id_msb_);
+
+            return (lhs_msb == rhs_msb) && (lhs.id_lsb_ == rhs.id_lsb_);
+        }
+
+        friend constexpr bool operator!=(
             gid_type const& lhs, gid_type const& rhs) noexcept
         {
             return !(lhs == rhs);
@@ -247,7 +269,7 @@ namespace hpx { namespace naming {
 
             while (!acquire_lock())
             {
-                util::yield_while([this] { return is_locked(); },
+                util::yield_while<true>([this] { return is_locked(); },
                     "hpx::naming::gid_type::lock");
             }
 
@@ -281,7 +303,7 @@ namespace hpx { namespace naming {
             HPX_ITT_SYNC_RELEASED(this);
         }
 
-        mutex_type& get_mutex() const noexcept
+        constexpr mutex_type& get_mutex() const noexcept
         {
             return const_cast<mutex_type&>(*this);
         }
@@ -340,66 +362,69 @@ namespace hpx { namespace naming {
     HPX_EXPORT void load(
         serialization::input_archive& ar, gid_type&, unsigned int version);
 
-    HPX_SERIALIZATION_SPLIT_FREE(gid_type);
-}}    // namespace hpx::naming
+    HPX_SERIALIZATION_SPLIT_FREE(gid_type)
+}    // namespace hpx::naming
 
 ///////////////////////////////////////////////////////////////////////////////
 // we know that we can serialize a gid as a byte sequence
 HPX_IS_BITWISE_SERIALIZABLE(hpx::naming::gid_type)
 
-namespace hpx { namespace naming {
+namespace hpx::naming {
+
+    namespace detail {
+
+        ///////////////////////////////////////////////////////////////////////
+        constexpr bool store_in_cache(gid_type const& id) noexcept
+        {
+            return (id.get_msb() & gid_type::dont_cache_mask) ? false : true;
+        }
+
+        constexpr void set_dont_store_in_cache(gid_type& gid) noexcept
+        {
+            gid.set_msb(gid.get_msb() | gid_type::dont_cache_mask);
+        }
+    }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
     //  Handle conversion to/from locality_id
-    inline gid_type get_gid_from_locality_id(
-        std::uint32_t locality_id) noexcept HPX_SUPER_PURE;
-
-    inline gid_type get_gid_from_locality_id(std::uint32_t locality_id) noexcept
+    constexpr gid_type get_gid_from_locality_id(
+        std::uint32_t locality_id) noexcept
     {
-        return gid_type(
-            (std::uint64_t(locality_id) + 1) << gid_type::locality_id_shift, 0);
+        return gid_type((static_cast<std::uint64_t>(locality_id) + 1)
+                << gid_type::locality_id_shift,
+            static_cast<std::uint64_t>(0));
     }
 
-    inline std::uint32_t get_locality_id_from_gid(
-        std::uint64_t msb) noexcept HPX_PURE;
-
-    inline std::uint32_t get_locality_id_from_gid(std::uint64_t msb) noexcept
+    constexpr std::uint32_t get_locality_id_from_gid(std::uint64_t msb) noexcept
     {
-        return std::uint32_t(msb >> gid_type::locality_id_shift) - 1;
+        return static_cast<std::uint32_t>(msb >> gid_type::locality_id_shift) -
+            1;
     }
 
-    inline std::uint32_t get_locality_id_from_gid(
-        gid_type const& id) noexcept HPX_PURE;
-
-    inline std::uint32_t get_locality_id_from_gid(gid_type const& id) noexcept
+    constexpr std::uint32_t get_locality_id_from_gid(
+        gid_type const& id) noexcept
     {
         return get_locality_id_from_gid(id.get_msb());
     }
 
-    inline gid_type get_locality_from_gid(gid_type const& id) noexcept
+    constexpr gid_type get_locality_from_gid(gid_type const& id) noexcept
     {
         return get_gid_from_locality_id(get_locality_id_from_gid(id));
     }
 
-    inline bool is_locality(gid_type const& gid) noexcept
+    constexpr bool is_locality(gid_type const& gid) noexcept
     {
         return get_locality_from_gid(gid) == gid;
     }
 
-    inline std::uint64_t replace_locality_id(
-        std::uint64_t msb, std::uint32_t locality_id) noexcept HPX_PURE;
-
-    inline std::uint64_t replace_locality_id(
+    constexpr std::uint64_t replace_locality_id(
         std::uint64_t msb, std::uint32_t locality_id) noexcept
     {
         msb &= ~gid_type::locality_id_mask;
         return msb | get_gid_from_locality_id(locality_id).get_msb();
     }
 
-    inline gid_type replace_locality_id(
-        gid_type const& gid, std::uint32_t locality_id) noexcept HPX_PURE;
-
-    inline gid_type replace_locality_id(
+    constexpr gid_type replace_locality_id(
         gid_type const& gid, std::uint32_t locality_id) noexcept
     {
         std::uint64_t msb = gid.get_msb() & ~gid_type::locality_id_mask;
@@ -430,7 +455,8 @@ namespace hpx { namespace naming {
         std::uint64_t msb = gid.get_msb() & ~gid_type::component_type_mask;
 
         HPX_ASSERT(!(msb & gid_type::dynamically_assigned));
-        msb |= ((std::uint64_t(type) << gid_type::component_type_shift) &
+        msb |= ((static_cast<std::uint64_t>(type)
+                    << gid_type::component_type_shift) &
             gid_type::component_type_mask);
         return gid_type(msb, gid.get_lsb());
     }
@@ -495,7 +521,7 @@ namespace hpx { namespace naming {
         }
 
         ///////////////////////////////////////////////////////////////////////
-        inline std::uint32_t get_component_type_from_gid(
+        constexpr std::uint32_t get_component_type_from_gid(
             std::uint64_t msb) noexcept
         {
             HPX_ASSERT(!(msb & gid_type::dynamically_assigned));
@@ -503,12 +529,13 @@ namespace hpx { namespace naming {
                 gid_type::component_type_base_mask;
         }
 
-        inline std::uint64_t add_component_type_to_gid(
+        constexpr std::uint64_t add_component_type_to_gid(
             std::uint64_t msb, std::uint32_t type) noexcept
         {
             HPX_ASSERT(!(msb & gid_type::dynamically_assigned));
             return (msb & ~gid_type::component_type_mask) |
-                ((std::uint64_t(type) << gid_type::component_type_shift) &
+                ((static_cast<std::uint64_t>(type)
+                     << gid_type::component_type_shift) &
                     gid_type::component_type_mask);
         }
 
@@ -530,7 +557,7 @@ namespace hpx { namespace naming {
         }
 
         ///////////////////////////////////////////////////////////////////////
-        inline gid_type get_stripped_gid(gid_type const& id) noexcept
+        constexpr gid_type get_stripped_gid(gid_type const& id) noexcept
         {
             std::uint64_t const msb =
                 strip_internal_bits_from_gid(id.get_msb());
@@ -572,35 +599,24 @@ namespace hpx { namespace naming {
             return static_cast<std::int64_t>(1) << log2credits;
         }
 
-        ///////////////////////////////////////////////////////////////////////
-        constexpr bool store_in_cache(gid_type const& id) noexcept
-        {
-            return (id.get_msb() & gid_type::dont_cache_mask) ? false : true;
-        }
-
-        inline void set_dont_store_in_cache(gid_type& gid) noexcept
-        {
-            gid.set_msb(gid.get_msb() | gid_type::dont_cache_mask);
-        }
-
         ///////////////////////////////////////////////////////////////////////
         constexpr bool is_migratable(gid_type const& id) noexcept
         {
             return (id.get_msb() & gid_type::is_migratable) ? true : false;
         }
 
-        inline void set_is_migratable(gid_type& gid) noexcept
+        constexpr void set_is_migratable(gid_type& gid) noexcept
         {
             gid.set_msb(gid.get_msb() | gid_type::is_migratable);
         }
 
         ///////////////////////////////////////////////////////////////////////
-        inline gid_type get_stripped_gid_except_dont_cache(
-            gid_type const& id) noexcept
+        constexpr gid_type get_stripped_gid_except_dont_cache(
+            gid_type const& gid) noexcept
         {
             std::uint64_t const msb =
-                strip_internal_bits_except_dont_cache_from_gid(id.get_msb());
-            std::uint64_t const lsb = id.get_lsb();
+                strip_internal_bits_except_dont_cache_from_gid(gid.get_msb());
+            std::uint64_t const lsb = gid.get_lsb();
             return gid_type(msb, lsb);
         }
 
@@ -610,25 +626,26 @@ namespace hpx { namespace naming {
             return msb & ~gid_type::credit_bits_mask;
         }
 
-        constexpr gid_type& strip_credits_from_gid(gid_type& id) noexcept
+        constexpr gid_type& strip_credits_from_gid(gid_type& gid) noexcept
         {
-            id.set_msb(strip_credits_from_gid(id.get_msb()));
-            return id;
+            gid.set_msb(strip_credits_from_gid(gid.get_msb()));
+            return gid;
         }
 
         ///////////////////////////////////////////////////////////////////////
         constexpr std::int16_t get_log2credit_from_gid(
-            gid_type const& id) noexcept
+            gid_type const& gid) noexcept
         {
-            HPX_ASSERT(has_credits(id));
-            return std::int16_t((id.get_msb() >> gid_type::credit_shift) &
+            HPX_ASSERT(has_credits(gid));
+            return static_cast<std::int16_t>(
+                (gid.get_msb() >> gid_type::credit_shift) &
                 gid_type::credit_base_mask);
         }
 
-        constexpr std::int64_t get_credit_from_gid(gid_type const& id) noexcept
+        constexpr std::int64_t get_credit_from_gid(gid_type const& gid) noexcept
         {
-            return has_credits(id) ?
-                detail::power2(get_log2credit_from_gid(id)) :
+            return has_credits(gid) ?
+                detail::power2(get_log2credit_from_gid(gid)) :
                 0;
         }
 
@@ -641,7 +658,8 @@ namespace hpx { namespace naming {
             HPX_ASSERT(0 == (log2credits & ~gid_type::credit_base_mask));
 
             id.set_msb((id.get_msb() & ~gid_type::credit_mask) |
-                ((std::int32_t(log2credits) << gid_type::credit_shift) &
+                ((static_cast<std::int64_t>(log2credits)
+                     << gid_type::credit_shift) &
                     gid_type::credit_mask) |
                 gid_type::has_credits_mask);
         }
@@ -664,19 +682,25 @@ namespace hpx { namespace naming {
     }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
-    HPX_INLINE_CONSTEXPR_VARIABLE gid_type const invalid_gid{};
+    inline constexpr gid_type const invalid_gid{};
 
     ///////////////////////////////////////////////////////////////////////////
     HPX_EXPORT std::ostream& operator<<(std::ostream& os, gid_type const& id);
 
     ///////////////////////////////////////////////////////////////////////////
-    inline gid_type::gid_type(
+    inline constexpr gid_type::gid_type(
         std::uint64_t msb_id, std::uint64_t lsb_id) noexcept
       : id_msb_(naming::detail::strip_lock_from_gid(msb_id))
       , id_lsb_(lsb_id)
     {
     }
 
+    inline gid_type::gid_type(std::uint64_t msb_id, void* lsb_id) noexcept
+      : id_msb_(naming::detail::strip_lock_from_gid(msb_id))
+      , id_lsb_(reinterpret_cast<std::uint64_t>(lsb_id))
+    {
+    }
+
     inline constexpr gid_type::gid_type(gid_type const& rhs) noexcept
       : id_msb_(naming::detail::strip_lock_from_gid(rhs.get_msb()))
       , id_lsb_(rhs.get_lsb())
@@ -712,7 +736,7 @@ namespace hpx { namespace naming {
         }
         return *this;
     }
-}}    // namespace hpx::naming
+}    // namespace hpx::naming
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace std {
