@@ -1,21 +1,24 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2021 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <hpx/modules/errors.hpp>
+#include <hpx/modules/format.hpp>
 #include <hpx/naming_base/address.hpp>
 #include <hpx/serialization/serialize.hpp>
 
+#include <cstddef>
+
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace naming {
+namespace hpx::naming {
 
     template <typename Archive>
     void address::save(Archive& ar, unsigned int /* version */) const
     {
         // clang-format off
-        ar & locality_ & type_ & address_;
+        std::size_t address = reinterpret_cast<std::size_t>(address_);
+        ar & locality_ & type_ & address;
         // clang-format on
     }
 
@@ -23,7 +26,9 @@ namespace hpx { namespace naming {
     void address::load(Archive& ar, unsigned int /* version */)
     {
         // clang-format off
-        ar & locality_ & type_ & address_;
+        std::size_t address;
+        ar & locality_ & type_ & address;
+        address_ = reinterpret_cast<address_type>(address);
         // clang-format on
     }
 
@@ -32,4 +37,4 @@ namespace hpx { namespace naming {
 
     template HPX_EXPORT void address::load(
         serialization::input_archive&, unsigned int);
-}}    // namespace hpx::naming
+}    // namespace hpx::naming
