@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -24,8 +24,7 @@
 namespace hpx { namespace performance_counters {
     ///////////////////////////////////////////////////////////////////////////
     constexpr char const counter_prefix[] = "/counters";
-    constexpr std::size_t counter_prefix_len =
-        (sizeof(counter_prefix) / sizeof(counter_prefix[0])) - 1;
+    constexpr std::size_t counter_prefix_len = std::size(counter_prefix) - 1;
 
     ///////////////////////////////////////////////////////////////////////////
     inline std::string& ensure_counter_prefix(std::string& name)
@@ -35,8 +34,8 @@ namespace hpx { namespace performance_counters {
         return name;
     }
 
-    inline std::string ensure_counter_prefix(
-        std::string const& counter)    //-V659
+    inline std::string ensure_counter_prefix(    //-V659
+        std::string const& counter)
     {
         std::string name(counter);
         return ensure_counter_prefix(name);
@@ -49,8 +48,8 @@ namespace hpx { namespace performance_counters {
         return name;
     }
 
-    inline std::string remove_counter_prefix(
-        std::string const& counter)    //-V659
+    inline std::string remove_counter_prefix(    //-V659
+        std::string const& counter)
     {
         std::string name(counter);
         return remove_counter_prefix(name);
@@ -58,44 +57,45 @@ namespace hpx { namespace performance_counters {
 
 #if defined(DOXYGEN)
     ///////////////////////////////////////////////////////////////////////////
-    enum counter_type
+    enum class counter_type
     {
-        /// \a counter_text shows a variable-length text string. It does not
+        /// \a text shows a variable-length text string. It does not
         /// deliver calculated values.
         ///
         /// Formula:  None
         /// Average:  None
         /// Type:     Text
-        counter_text,
+        text,
 
-        /// \a counter_raw shows the last observed value only. It does
+        /// \a raw shows the last observed value only. It does
         /// not deliver an average.
         ///
         /// Formula:  None. Shows raw data as collected.
         /// Average:  None
         /// Type:     Instantaneous
-        counter_raw,
+        raw,
 
-        // \a counter_raw shows the cumulatively accumulated observed value.
-        // It does not deliver an average.
-        //
-        // Formula:  None. Shows cumulatively accumulated data as collected.
-        // Average:  None
-        // Type:     Instantaneous
-        counter_monotonically_increasing,
+        /// \a monotonically_increasing shows the cumulatively accumulated
+        /// observed value. It does not deliver an average.
+        ///
+        /// Formula:  None. Shows cumulatively accumulated data as collected.
+        /// Average:  None
+        /// Type:     Instantaneous
+        monotonically_increasing,
 
-        /// \a counter_average_base is used as the base data (denominator) in the
-        /// computation of time or count averages for the \a counter_average_count
-        /// and \a counter_average_timer counter types. This counter type
-        /// collects the last observed value only.
+        /// \a average_base is used as the base data (denominator) in the
+        /// computation of time or count averages for the
+        /// \a counter_type::average_count and \a counter_type::average_timer
+        /// counter types.
+        /// This counter type collects the last observed value only.
         ///
         /// Formula:  None. This counter uses raw data in factional calculations
-        ///           without delivering an output.
-        /// Average:  SUM (N) / x
-        /// Type:     Instantaneous
-        counter_average_base,
+        /// without delivering an output.
+        /// Average:  SUM (N) / x Type:
+        /// Instantaneous
+        average_base,
 
-        /// \a counter_average_count shows how many items are processed, on
+        /// \a average_count shows how many items are processed, on
         /// average, during an operation. Counters of this type display a ratio
         /// of the items processed (such as bytes sent) to the number of
         /// operations completed. The ratio is calculated by comparing the
@@ -109,16 +109,16 @@ namespace hpx { namespace performance_counters {
         ///           intervals.
         /// Average:  (Nx - N0) / (Dx - D0)
         /// Type:     Average
-        counter_average_count,
+        average_count,
 
-        /// \a counter_aggregating applies a function to an embedded counter
+        /// \a aggregating applies a function to an embedded counter
         /// instance. The embedded counter is usually evaluated repeatedly
         /// after a fixed (but configurable) time interval.
         ///
         /// Formula:  F(Nx)
-        counter_aggregating,
+        aggregating,
 
-        /// \a counter_average_timer measures the average time it takes to
+        /// \a average_timer measures the average time it takes to
         /// complete a process or operation. Counters of this type display a
         /// ratio of the total elapsed time of the sample interval to the
         /// number of processes or operations completed during that time. This
@@ -135,9 +135,9 @@ namespace hpx { namespace performance_counters {
         ///           interval.
         /// Average:  ((Nx - N0) / F) / (Dx - D0)
         /// Type:     Average
-        counter_average_timer,
+        average_timer,
 
-        /// \a counter_elapsed_time shows the total time between when the
+        /// \a elapsed_time shows the total time between when the
         /// component or process started and the time when this value is
         /// calculated. The variable F represents the number of time units that
         /// elapse in one second. The value of F is factored into the equation
@@ -149,9 +149,9 @@ namespace hpx { namespace performance_counters {
         ///           number of time units that elapse in one second.
         /// Average:  (Dx - N0) / F
         /// Type:     Difference
-        counter_elapsed_time,
+        elapsed_time,
 
-        /// \a counter_histogram exposes a histogram of the measured values
+        /// \a histogram exposes a histogram of the measured values
         /// instead of a single value as many of the other counter types.
         /// Counters of this type expose a \a counter_value_array instead of a
         /// \a counter_value. Those will also not implement the
@@ -162,15 +162,15 @@ namespace hpx { namespace performance_counters {
         /// and upper boundaries, and the size of the histogram buckets. All
         /// remaining values in the returned array represent the number of
         /// measurements for each of the buckets in the histogram.
-        counter_histogram,
+        histogram,
 
-        /// \a counter_raw_values exposes an array of measured values
+        /// \a raw_values exposes an array of measured values
         /// instead of a single value as many of the other counter types.
         /// Counters of this type expose a \a counter_value_array instead of a
         /// \a counter_value. Those will also not implement the
         /// \a get_counter_value() functionality. The results are exposed
         /// through a separate \a get_counter_values_array() function.
-        counter_raw_values
+        raw_values
     };
 #endif
 
@@ -182,21 +182,21 @@ namespace hpx { namespace performance_counters {
     ///////////////////////////////////////////////////////////////////////////
     /// \brief Status and error codes used by the functions related to
     ///        performance counters.
-    enum counter_status
+    enum class counter_status
     {
-        status_valid_data,      ///< No error occurred, data is valid
-        status_new_data,        ///< Data is valid and different from last call
-        status_invalid_data,    ///< Some error occurred, data is not value
-        status_already_defined,    ///< The type or instance already has been defined
-        status_counter_unknown,         ///< The counter instance is unknown
-        status_counter_type_unknown,    ///< The counter type is unknown
-        status_generic_error            ///< A unknown error occurred
+        valid_data,         ///< No error occurred, data is valid
+        new_data,           ///< Data is valid and different from last call
+        invalid_data,       ///< Some error occurred, data is not value
+        already_defined,    ///< The type or instance already has been defined
+        counter_unknown,    ///< The counter instance is unknown
+        counter_type_unknown,    ///< The counter type is unknown
+        generic_error            ///< A unknown error occurred
     };
 #endif
 
     inline bool status_is_valid(counter_status s)
     {
-        return s == status_valid_data || s == status_new_data;
+        return s == counter_status::valid_data || s == counter_status::new_data;
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -211,7 +211,7 @@ namespace hpx { namespace performance_counters {
     ///
     struct counter_type_path_elements
     {
-        counter_type_path_elements() {}
+        counter_type_path_elements() = default;
 
         counter_type_path_elements(std::string const& objectname,
             std::string const& countername, std::string const& parameters)
@@ -231,9 +231,9 @@ namespace hpx { namespace performance_counters {
         friend class hpx::serialization::access;
 
         HPX_EXPORT void serialize(
-            serialization::output_archive& ar, const unsigned int);
+            serialization::output_archive& ar, unsigned int) const;
         HPX_EXPORT void serialize(
-            serialization::input_archive& ar, const unsigned int);
+            serialization::input_archive& ar, unsigned int);
     };
 
     ///////////////////////////////////////////////////////////////////////////
@@ -249,7 +249,7 @@ namespace hpx { namespace performance_counters {
     ///
     struct counter_path_elements : counter_type_path_elements
     {
-        typedef counter_type_path_elements base_type;
+        using base_type = counter_type_path_elements;
 
         counter_path_elements()
           : parentinstanceindex_(-1)
@@ -307,25 +307,25 @@ namespace hpx { namespace performance_counters {
         friend class hpx::serialization::access;
 
         HPX_EXPORT void serialize(
-            serialization::output_archive& ar, const unsigned int);
+            serialization::output_archive& ar, unsigned int);
         HPX_EXPORT void serialize(
-            serialization::input_archive& ar, const unsigned int);
+            serialization::input_archive& ar, unsigned int);
     };
 
     ///////////////////////////////////////////////////////////////////////////
     struct counter_info
     {
-        counter_info(counter_type type = counter_raw)
+        explicit counter_info(counter_type type = counter_type::raw)
           : type_(type)
           , version_(HPX_PERFORMANCE_COUNTER_V1)
-          , status_(status_invalid_data)
+          , status_(counter_status::invalid_data)
         {
         }
 
-        counter_info(std::string const& name)
-          : type_(counter_raw)
+        explicit counter_info(std::string const& name)
+          : type_(counter_type::raw)
           , version_(HPX_PERFORMANCE_COUNTER_V1)
-          , status_(status_invalid_data)
+          , status_(counter_status::invalid_data)
           , fullname_(name)
         {
         }
@@ -336,7 +336,7 @@ namespace hpx { namespace performance_counters {
             std::string const& uom = "")
           : type_(type)
           , version_(version)
-          , status_(status_invalid_data)
+          , status_(counter_status::invalid_data)
           , fullname_(name)
           , helptext_(helptext)
           , unit_of_measure_(uom)
@@ -358,32 +358,30 @@ namespace hpx { namespace performance_counters {
         friend class hpx::serialization::access;
 
         HPX_EXPORT void serialize(
-            serialization::output_archive& ar, const unsigned int);
+            serialization::output_archive& ar, unsigned int) const;
         HPX_EXPORT void serialize(
-            serialization::input_archive& ar, const unsigned int);
+            serialization::input_archive& ar, unsigned int);
     };
 
     ///////////////////////////////////////////////////////////////////////////
     /// \brief This declares the type of a function, which will be
     ///        called by HPX whenever a new performance counter instance of a
     ///        particular type needs to be created.
-    typedef hpx::util::function_nonser<naming::gid_type(
-        counter_info const&, error_code&)>
-        create_counter_func;
+    using create_counter_func =
+        hpx::function<naming::gid_type(counter_info const&, error_code&)>;
 
     ///////////////////////////////////////////////////////////////////////////
     /// \brief This declares a type of a function, which will be passed to
     ///        a \a discover_counters_func in order to be called for each
     ///        discovered performance counter instance.
-    typedef hpx::util::function_nonser<bool(counter_info const&, error_code&)>
-        discover_counter_func;
+    using discover_counter_func =
+        hpx::function<bool(counter_info const&, error_code&)>;
 
     /// \brief This declares the type of a function, which will be called by
     ///        HPX whenever it needs to discover all performance counter
     ///        instances of a particular type.
-    typedef hpx::util::function_nonser<bool(counter_info const&,
-        discover_counter_func const&, discover_counters_mode, error_code&)>
-        discover_counters_func;
+    using discover_counters_func = hpx::function<bool(counter_info const&,
+        discover_counter_func const&, discover_counters_mode, error_code&)>;
 
     ///////////////////////////////////////////////////////////////////////
     inline counter_status add_counter_type(
@@ -393,20 +391,20 @@ namespace hpx { namespace performance_counters {
             info, create_counter_func(), discover_counters_func(), ec);
     }
 
-    inline naming::id_type get_counter(std::string const& name, error_code& ec)
+    inline hpx::id_type get_counter(std::string const& name, error_code& ec)
     {
-        lcos::future<naming::id_type> f = get_counter_async(name, ec);
+        hpx::future<hpx::id_type> f = get_counter_async(name, ec);
         if (ec)
-            return naming::invalid_id;
+            return hpx::invalid_id;
 
         return f.get(ec);
     }
 
-    inline naming::id_type get_counter(counter_info const& info, error_code& ec)
+    inline hpx::id_type get_counter(counter_info const& info, error_code& ec)
     {
-        lcos::future<naming::id_type> f = get_counter_async(info, ec);
+        hpx::future<hpx::id_type> f = get_counter_async(info, ec);
         if (ec)
-            return naming::invalid_id;
+            return hpx::invalid_id;
 
         return f.get(ec);
     }
