@@ -1,4 +1,4 @@
-//  Copyright (c) 2017 Hartmut Kaiser
+//  Copyright (c) 2017-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -21,6 +21,7 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace hpx { namespace performance_counters {
+
     ///////////////////////////////////////////////////////////////////////////
     inline std::string& ensure_counter_prefix(std::string& name);
     inline std::string ensure_counter_prefix(std::string const& counter);
@@ -28,44 +29,44 @@ namespace hpx { namespace performance_counters {
     inline std::string remove_counter_prefix(std::string const& counter);
 
     ///////////////////////////////////////////////////////////////////////////
-    enum counter_type
+    enum class counter_type
     {
-        // \a counter_text shows a variable-length text string. It does not
+        // \a text counter shows a variable-length text string. It does not
         // deliver calculated values.
         //
         // Formula:  None
         // Average:  None
         // Type:     Text
-        counter_text,
+        text,
 
-        // \a counter_raw shows the last observed value only. It does
+        // \a raw counter shows the last observed value only. It does
         // not deliver an average.
         //
         // Formula:  None. Shows raw data as collected.
         // Average:  None
         // Type:     Instantaneous
-        counter_raw,
+        raw,
 
-        // \a counter_raw shows the cumulatively accumulated observed value.
-        // It does not deliver an average.
+        // \a monotinacally_increasing counter shows the cumulatively
+        // accumulated observed value. It does not deliver an average.
         //
         // Formula:  None. Shows cumulatively accumulated data as collected.
         // Average:  None
         // Type:     Instantaneous
-        counter_monotonically_increasing,
+        monotonically_increasing,
 
-        // \a counter_average_base is used as the base data (denominator) in the
-        // computation of time or count averages for the \a counter_average_count
-        // and \a counter_average_timer counter types. This counter type
+        // \a average_base counter is used as the base data (denominator) in the
+        // computation of time or count averages for the \a average_count
+        // and \a average_timer counter types. This counter type
         // collects the last observed value only.
         //
         // Formula:  None. This counter uses raw data in factional calculations
         //           without delivering an output.
         // Average:  SUM (N) / x
         // Type:     Instantaneous
-        counter_average_base,
+        average_base,
 
-        // \a counter_average_count shows how many items are processed, on
+        // \a average_count counter shows how many items are processed, on
         // average, during an operation. Counters of this type display a ratio
         // of the items processed (such as bytes sent) to the number of
         // operations completed. The ratio is calculated by comparing the
@@ -79,16 +80,16 @@ namespace hpx { namespace performance_counters {
         //           intervals.
         // Average:  (Nx - N0) / (Dx - D0)
         // Type:     Average
-        counter_average_count,
+        average_count,
 
-        // \a counter_aggregating applies a function to an embedded counter
+        // \a aggregating counter applies a function to an embedded counter
         // instance. The embedded counter is usually evaluated repeatedly
         // after a fixed (but configurable) time interval.
         //
         // Formula:  F(Nx)
-        counter_aggregating,
+        aggregating,
 
-        // \a counter_average_timer measures the average time it takes to
+        // \a average_timer counter measures the average time it takes to
         // complete a process or operation. Counters of this type display a
         // ratio of the total elapsed time of the sample interval to the
         // number of processes or operations completed during that time. This
@@ -105,9 +106,9 @@ namespace hpx { namespace performance_counters {
         //           interval.
         // Average:  ((Nx - N0) / F) / (Dx - D0)
         // Type:     Average
-        counter_average_timer,
+        average_timer,
 
-        // \a counter_elapsed_time shows the total time between when the
+        // \a elapsed_time counter shows the total time between when the
         // component or process started and the time when this value is
         // calculated. The variable F represents the number of time units that
         // elapse in one second. The value of F is factored into the equation
@@ -119,12 +120,12 @@ namespace hpx { namespace performance_counters {
         //           number of time units that elapse in one second.
         // Average:  (Dx - N0) / F
         // Type:     Difference
-        counter_elapsed_time,
+        elapsed_time,
 
-        // \a counter_histogram exposes a histogram of the measured values
+        // \a histogram counter exposes a histogram of the measured values
         // instead of a single value as many of the other counter types.
         // Counters of this type expose a \a counter_value_array instead of a
-        // \a counter_value. Those will also not implement the
+        // \a value. Those will also not implement the
         // \a get_counter_value() functionality. The results are exposed
         // through a separate \a get_counter_values_array() function.
         //
@@ -132,17 +133,60 @@ namespace hpx { namespace performance_counters {
         // and upper boundaries, and the size of the histogram buckets. All
         // remaining values in the returned array represent the number of
         // measurements for each of the buckets in the histogram.
-        counter_histogram,
+        histogram,
 
-        /// \a counter_raw_values exposes an array of measured values
+        /// \a raw_values counter exposes an array of measured values
         /// instead of a single value as many of the other counter types.
         /// Counters of this type expose a \a counter_value_array instead of a
         /// \a counter_value. Those will also not implement the
         /// \a get_counter_value() functionality. The results are exposed
         /// through a separate \a get_counter_values_array() function.
-        counter_raw_values
+        raw_values
     };
 
+    inline constexpr bool operator<(counter_type lhs, counter_type rhs) noexcept
+    {
+        return static_cast<int>(lhs) < static_cast<int>(rhs);
+    }
+
+    inline constexpr bool operator>(counter_type lhs, counter_type rhs) noexcept
+    {
+        return static_cast<int>(lhs) > static_cast<int>(rhs);
+    }
+
+#define HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG                         \
+    "The unscoped counter_type names are deprecated. Please use "              \
+    "counter_type::<type> instead."
+
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_text = counter_type::text;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_raw = counter_type::raw;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_monotonically_increasing =
+        counter_type::monotonically_increasing;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_average_base =
+        counter_type::average_base;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_average_count =
+        counter_type::average_count;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_aggregating =
+        counter_type::aggregating;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_average_timer =
+        counter_type::average_timer;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_elapsed_time =
+        counter_type::elapsed_time;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_raw_values = counter_type::raw_values;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_type counter_histogram = counter_type::histogram;
+
+#undef HPX_COUNTER_TYPE_UNSCOPED_ENUM_DEPRECATION_MSG
+
     ///////////////////////////////////////////////////////////////////////////
     // Return the readable name of a given counter type
     HPX_EXPORT char const* get_counter_type_name(counter_type state);
@@ -150,17 +194,46 @@ namespace hpx { namespace performance_counters {
     ///////////////////////////////////////////////////////////////////////////
     // Status and error codes used by the functions related to
     // performance counters.
-    enum counter_status
+    enum class counter_status
     {
-        status_valid_data,         // No error occurred, data is valid
-        status_new_data,           // Data is valid and different from last call
-        status_invalid_data,       // Some error occurred, data is not value
-        status_already_defined,    // The type or instance already has been defined
-        status_counter_unknown,         // The counter instance is unknown
-        status_counter_type_unknown,    // The counter type is unknown
-        status_generic_error            // A unknown error occurred
+        valid_data,         // No error occurred, data is valid
+        new_data,           // Data is valid and different from last call
+        invalid_data,       // Some error occurred, data is not value
+        already_defined,    // The type or instance already has been defined
+        counter_unknown,    // The counter instance is unknown
+        counter_type_unknown,    // The counter type is unknown
+        generic_error            // A unknown error occurred
     };
 
+    HPX_EXPORT std::ostream& operator<<(std::ostream& os, counter_status rhs);
+
+#define HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG                       \
+    "The unscoped counter_status names are deprecated. Please use "            \
+    "counter_status::<status> instead."
+
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_valid_data =
+        counter_status::valid_data;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_new_data = counter_status::new_data;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_invalid_data =
+        counter_status::invalid_data;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_already_defined =
+        counter_status::already_defined;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_counter_unknown =
+        counter_status::counter_unknown;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_counter_type_unknown =
+        counter_status::counter_type_unknown;
+    HPX_DEPRECATED_V(1, 9, HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr counter_status status_generic_error =
+        counter_status::generic_error;
+
+#undef HPX_COUNTER_STATUS_UNSCOPED_ENUM_DEPRECATION_MSG
+
     inline bool status_is_valid(counter_status s);
 
     ///////////////////////////////////////////////////////////////////////////
@@ -231,29 +304,42 @@ namespace hpx { namespace performance_counters {
     // This declares the type of a function, which will be
     // called by HPX whenever a new performance counter instance of a
     // particular type needs to be created.
-    typedef hpx::util::function_nonser<naming::gid_type(
-        counter_info const&, error_code&)>
-        create_counter_func;
+    using create_counter_func =
+        hpx::function<naming::gid_type(counter_info const&, error_code&)>;
 
     ///////////////////////////////////////////////////////////////////////////
     // This declares a type of a function, which will be passed to
     // a \a discover_counters_func in order to be called for each
     // discovered performance counter instance.
-    typedef hpx::util::function_nonser<bool(counter_info const&, error_code&)>
-        discover_counter_func;
+    using discover_counter_func =
+        hpx::function<bool(counter_info const&, error_code&)>;
 
-    enum discover_counters_mode
+    enum class discover_counters_mode
     {
-        discover_counters_minimal,
-        discover_counters_full    // fully expand all wild cards
+        minimal,
+        full    // fully expand all wild cards
     };
 
+#define HPX_DISCOVER_COUNTERS_MODE_UNSCOPED_ENUM_DEPRECATION_MSG               \
+    "The unscoped discover counters mode names are deprecated. Please use "    \
+    "discover_counters_mode::<mode> instead."
+
+    HPX_DEPRECATED_V(
+        1, 9, HPX_DISCOVER_COUNTERS_MODE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr discover_counters_mode discover_counters_minimal =
+        discover_counters_mode::minimal;
+    HPX_DEPRECATED_V(
+        1, 9, HPX_DISCOVER_COUNTERS_MODE_UNSCOPED_ENUM_DEPRECATION_MSG)
+    inline constexpr discover_counters_mode discover_counters_full =
+        discover_counters_mode::full;
+
+#undef HPX_DISCOVER_COUNTERS_MODE_UNSCOPED_ENUM_DEPRECATION_MSG
+
     // This declares the type of a function, which will be called by
     // HPX whenever it needs to discover all performance counter
     // instances of a particular type.
-    typedef hpx::util::function_nonser<bool(counter_info const&,
-        discover_counter_func const&, discover_counters_mode, error_code&)>
-        discover_counters_func;
+    using discover_counters_func = hpx::function<bool(counter_info const&,
+        discover_counter_func const&, discover_counters_mode, error_code&)>;
 
     ///////////////////////////////////////////////////////////////////////////
     /// \brief Complement the counter info if parent instance name is missing
@@ -272,7 +358,7 @@ namespace hpx { namespace performance_counters {
           , count_(0)
           , value_(value)
           , scaling_(scaling)
-          , status_(status_new_data)
+          , status_(counter_status::new_data)
           , scale_inverse_(scale_inverse)
         {
         }
@@ -293,7 +379,8 @@ namespace hpx { namespace performance_counters {
         {
             if (!status_is_valid(status_))
             {
-                HPX_THROWS_IF(ec, invalid_status, "counter_value::get_value<T>",
+                HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                    "counter_value::get_value<T>",
                     "counter value is in invalid status");
                 return T();
             }
@@ -304,7 +391,7 @@ namespace hpx { namespace performance_counters {
             {
                 if (scaling_ == 0)
                 {
-                    HPX_THROWS_IF(ec, uninitialized_value,
+                    HPX_THROWS_IF(ec, hpx::error::uninitialized_value,
                         "counter_value::get_value<T>",
                         "scaling should not be zero");
                     return T();
@@ -324,7 +411,7 @@ namespace hpx { namespace performance_counters {
         friend class hpx::serialization::access;
 
         HPX_EXPORT void serialize(
-            serialization::output_archive& ar, const unsigned int);
+            serialization::output_archive& ar, const unsigned int) const;
         HPX_EXPORT void serialize(
             serialization::input_archive& ar, const unsigned int);
     };
@@ -338,7 +425,7 @@ namespace hpx { namespace performance_counters {
           , count_(0)
           , values_()
           , scaling_(scaling)
-          , status_(status_new_data)
+          , status_(counter_status::new_data)
           , scale_inverse_(scale_inverse)
         {
         }
@@ -347,9 +434,9 @@ namespace hpx { namespace performance_counters {
             std::int64_t scaling = 1, bool scale_inverse = false)
           : time_()
           , count_(0)
-          , values_(std::move(values))
+          , values_(HPX_MOVE(values))
           , scaling_(scaling)
-          , status_(status_new_data)
+          , status_(counter_status::new_data)
           , scale_inverse_(scale_inverse)
         {
         }
@@ -360,7 +447,7 @@ namespace hpx { namespace performance_counters {
           , count_(0)
           , values_(values)
           , scaling_(scaling)
-          , status_(status_new_data)
+          , status_(counter_status::new_data)
           , scale_inverse_(scale_inverse)
         {
         }
@@ -381,14 +468,14 @@ namespace hpx { namespace performance_counters {
         {
             if (!status_is_valid(status_))
             {
-                HPX_THROWS_IF(ec, invalid_status,
+                HPX_THROWS_IF(ec, hpx::error::invalid_status,
                     "counter_values_array::get_value<T>",
                     "counter value is in invalid status");
                 return T();
             }
             if (index >= values_.size())
             {
-                HPX_THROWS_IF(ec, bad_parameter,
+                HPX_THROWS_IF(ec, hpx::error::bad_parameter,
                     "counter_values_array::get_value<T>",
                     "index out of bounds");
                 return T();
@@ -400,7 +487,7 @@ namespace hpx { namespace performance_counters {
             {
                 if (scaling_ == 0)
                 {
-                    HPX_THROWS_IF(ec, uninitialized_value,
+                    HPX_THROWS_IF(ec, hpx::error::uninitialized_value,
                         "counter_values_array::get_value<T>",
                         "scaling should not be zero");
                     return T();
@@ -420,7 +507,7 @@ namespace hpx { namespace performance_counters {
         friend class hpx::serialization::access;
 
         HPX_EXPORT void serialize(
-            serialization::output_archive& ar, const unsigned int);
+            serialization::output_archive& ar, const unsigned int) const;
         HPX_EXPORT void serialize(
             serialization::input_archive& ar, const unsigned int);
     };
@@ -439,36 +526,36 @@ namespace hpx { namespace performance_counters {
     /// \brief Call the supplied function for each registered counter type
     HPX_EXPORT counter_status discover_counter_types(
         discover_counter_func const& discover_counter,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     /// \brief Return a list of all available counter descriptions.
     HPX_EXPORT counter_status discover_counter_types(
         std::vector<counter_info>& counters,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     /// \brief Call the supplied function for the given registered counter type.
     HPX_EXPORT counter_status discover_counter_type(std::string const& name,
         discover_counter_func const& discover_counter,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     HPX_EXPORT counter_status discover_counter_type(counter_info const& info,
         discover_counter_func const& discover_counter,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     /// \brief Return a list of matching counter descriptions for the given
     ///        registered counter type.
     HPX_EXPORT counter_status discover_counter_type(std::string const& name,
         std::vector<counter_info>& counters,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     HPX_EXPORT counter_status discover_counter_type(counter_info const& info,
         std::vector<counter_info>& counters,
-        discover_counters_mode mode = discover_counters_minimal,
+        discover_counters_mode mode = discover_counters_mode::minimal,
         error_code& ec = throws);
 
     /// \brief call the supplied function will all expanded versions of the
@@ -495,19 +582,19 @@ namespace hpx { namespace performance_counters {
     /// \brief Get the global id of an existing performance counter, if the
     ///        counter does not exist yet, the function attempts to create the
     ///        counter based on the given counter name.
-    HPX_EXPORT lcos::future<naming::id_type> get_counter_async(
+    HPX_EXPORT hpx::future<hpx::id_type> get_counter_async(
         std::string name, error_code& ec = throws);
 
-    inline naming::id_type get_counter(
+    inline hpx::id_type get_counter(
         std::string const& name, error_code& ec = throws);
 
     /// \brief Get the global id of an existing performance counter, if the
     ///        counter does not exist yet, the function attempts to create the
     ///        counter based on the given counter info.
-    HPX_EXPORT lcos::future<naming::id_type> get_counter_async(
+    HPX_EXPORT hpx::future<hpx::id_type> get_counter_async(
         counter_info const& info, error_code& ec = throws);
 
-    inline naming::id_type get_counter(
+    inline hpx::id_type get_counter(
         counter_info const& info, error_code& ec = throws);
 
     ///////////////////////////////////////////////////////////////////////////
@@ -523,36 +610,34 @@ namespace hpx { namespace performance_counters {
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
         /// \brief Add an existing performance counter instance to the registry
-        HPX_EXPORT counter_status add_counter(naming::id_type const& id,
+        HPX_EXPORT counter_status add_counter(hpx::id_type const& id,
             counter_info const& info, error_code& ec = throws);
 
         /// \brief Remove an existing performance counter instance with the
         ///        given id (as returned from \a create_counter)
         HPX_EXPORT counter_status remove_counter(counter_info const& info,
-            naming::id_type const& id, error_code& ec = throws);
+            hpx::id_type const& id, error_code& ec = throws);
 
         ///////////////////////////////////////////////////////////////////////
         // Helper function for creating counters encapsulating a function
         // returning the counter value.
         HPX_EXPORT naming::gid_type create_raw_counter(counter_info const&,
-            hpx::util::function_nonser<std::int64_t()> const&, error_code&);
+            hpx::function<std::int64_t()> const&, error_code&);
 
         // Helper function for creating counters encapsulating a function
         // returning the counter value.
         HPX_EXPORT naming::gid_type create_raw_counter(counter_info const&,
-            hpx::util::function_nonser<std::int64_t(bool)> const&, error_code&);
+            hpx::function<std::int64_t(bool)> const&, error_code&);
 
         // Helper function for creating counters encapsulating a function
         // returning the counter values array.
         HPX_EXPORT naming::gid_type create_raw_counter(counter_info const&,
-            hpx::util::function_nonser<std::vector<std::int64_t>()> const&,
-            error_code&);
+            hpx::function<std::vector<std::int64_t>()> const&, error_code&);
 
         // Helper function for creating counters encapsulating a function
         // returning the counter values array.
         HPX_EXPORT naming::gid_type create_raw_counter(counter_info const&,
-            hpx::util::function_nonser<std::vector<std::int64_t>(bool)> const&,
-            error_code&);
+            hpx::function<std::vector<std::int64_t>(bool)> const&, error_code&);
 
         // Helper function for creating a new performance counter instance
         // based on a given counter value.
