@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -63,7 +63,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty())
         {
             if (p.parentinstancename_.empty())
@@ -112,7 +112,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty())
         {
             if (p.parentinstancename_.empty())
@@ -183,7 +183,7 @@ namespace hpx { namespace performance_counters {
             p.parentinstanceindex_ = 0;
         }
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty())
         {
             if (p.parentinstancename_.empty())
@@ -228,7 +228,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty())
         {
             if (p.parentinstancename_.empty())
@@ -284,7 +284,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty() ||
             p.subinstancename_.empty())
         {
@@ -351,7 +351,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty() ||
             p.subinstancename_.empty())
         {
@@ -401,7 +401,7 @@ namespace hpx { namespace performance_counters {
         if (!status_is_valid(status))
             return false;
 
-        if (mode == discover_counters_minimal ||
+        if (mode == discover_counters_mode::minimal ||
             p.parentinstancename_.empty() || p.instancename_.empty())
         {
             if (p.parentinstancename_.empty())
@@ -446,7 +446,7 @@ namespace hpx { namespace performance_counters {
     ///   /<objectname>{locality#<locality_id>/total}/<instancename>
     ///
     naming::gid_type locality_raw_counter_creator(counter_info const& info,
-        hpx::util::function_nonser<std::int64_t(bool)> const& f, error_code& ec)
+        hpx::function<std::int64_t(bool)> const& f, error_code& ec)
     {
         // verify the validity of the counter instance name
         counter_path_elements paths;
@@ -456,7 +456,8 @@ namespace hpx { namespace performance_counters {
 
         if (paths.parentinstance_is_basename_)
         {
-            HPX_THROWS_IF(ec, bad_parameter, "locality_raw_counter_creator",
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                "locality_raw_counter_creator",
                 "invalid counter instance parent name: " +
                     paths.parentinstancename_);
             return naming::invalid_gid;
@@ -466,15 +467,15 @@ namespace hpx { namespace performance_counters {
             return detail::create_raw_counter(
                 info, f, ec);    // overall counter
 
-        HPX_THROWS_IF(ec, bad_parameter, "locality_raw_counter_creator",
+        HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+            "locality_raw_counter_creator",
             "invalid counter instance name: " + paths.instancename_);
         return naming::invalid_gid;
     }
 
     naming::gid_type locality_raw_values_counter_creator(
         counter_info const& info,
-        hpx::util::function_nonser<std::vector<std::int64_t>(bool)> const& f,
-        error_code& ec)
+        hpx::function<std::vector<std::int64_t>(bool)> const& f, error_code& ec)
     {
         // verify the validity of the counter instance name
         counter_path_elements paths;
@@ -484,7 +485,7 @@ namespace hpx { namespace performance_counters {
 
         if (paths.parentinstance_is_basename_)
         {
-            HPX_THROWS_IF(ec, bad_parameter,
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
                 "locality_raw_values_counter_creator",
                 "invalid counter instance parent name: " +
                     paths.parentinstancename_);
@@ -497,7 +498,8 @@ namespace hpx { namespace performance_counters {
                 info, f, ec);    // overall counter
         }
 
-        HPX_THROWS_IF(ec, bad_parameter, "locality_raw_values_counter_creator",
+        HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+            "locality_raw_values_counter_creator",
             "invalid counter instance name: " + paths.instancename_);
         return naming::invalid_gid;
     }
@@ -505,7 +507,7 @@ namespace hpx { namespace performance_counters {
     namespace detail {
 
         naming::gid_type retrieve_agas_counter(std::string const& name,
-            naming::id_type const& agas_id, error_code& ec)
+            hpx::id_type const& agas_id, error_code& ec)
         {
             naming::gid_type id;
 
@@ -539,7 +541,8 @@ namespace hpx { namespace performance_counters {
                 return action(agas_id, name).get_gid();
             }
             default:
-                HPX_THROWS_IF(ec, bad_parameter, "retrieve_statistics_counter",
+                HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                    "retrieve_statistics_counter",
                     "unknown counter agas counter name: " + name);
                 break;
             }
@@ -571,13 +574,15 @@ namespace hpx { namespace performance_counters {
 
         if (paths.objectname_ != "agas")
         {
-            HPX_THROWS_IF(ec, bad_parameter, "agas_raw_counter_creator",
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                "agas_raw_counter_creator",
                 "unknown performance counter (unrelated to AGAS)");
             return naming::invalid_gid;
         }
         if (paths.parentinstance_is_basename_)
         {
-            HPX_THROWS_IF(ec, bad_parameter, "agas_raw_counter_creator",
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                "agas_raw_counter_creator",
                 "invalid counter instance parent name: " +
                     paths.parentinstancename_);
             return naming::invalid_gid;
@@ -593,7 +598,8 @@ namespace hpx { namespace performance_counters {
 
             if (-1 == paths.parentinstanceindex_)
             {
-                HPX_THROWS_IF(ec, bad_parameter, "agas_raw_counter_creator",
+                HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                    "agas_raw_counter_creator",
                     "invalid parent instance index: -1");
                 return naming::invalid_gid;
             }
@@ -603,10 +609,11 @@ namespace hpx { namespace performance_counters {
             service += "/";
             service += service_name;
 
-            naming::id_type id = agas::resolve_name(launch::sync, service, ec);
-            if (id == naming::invalid_id)
+            hpx::id_type id = agas::resolve_name(launch::sync, service, ec);
+            if (id == hpx::invalid_id)
             {
-                HPX_THROWS_IF(ec, not_implemented, "agas_raw_counter_creator",
+                HPX_THROWS_IF(ec, hpx::error::not_implemented,
+                    "agas_raw_counter_creator",
                     "invalid counter name: " +
                         remove_counter_prefix(info.fullname_));
                 return naming::invalid_gid;
@@ -615,7 +622,8 @@ namespace hpx { namespace performance_counters {
             return detail::retrieve_agas_counter(info.fullname_, id, ec);
         }
 
-        HPX_THROWS_IF(ec, not_implemented, "agas_raw_counter_creator",
+        HPX_THROWS_IF(ec, hpx::error::not_implemented,
+            "agas_raw_counter_creator",
             "invalid counter type name: " + paths.instancename_);
         return naming::invalid_gid;
     }
