@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -17,6 +17,7 @@
 #include <hpx/performance_counters/performance_counter.hpp>
 #include <hpx/performance_counters/query_counters.hpp>
 #include <hpx/runtime_local/config_entry.hpp>
+#include <hpx/runtime_local/get_locality_id.hpp>
 #include <hpx/runtime_local/get_thread_name.hpp>
 #include <hpx/thread_support/unlock_guard.hpp>
 #include <hpx/threading_base/external_timer.hpp>
@@ -39,7 +40,7 @@
 #include <map>
 #endif
 
-namespace hpx { namespace util {
+namespace hpx::util {
 
     query_counters::query_counters(std::vector<std::string> const& names,
         std::vector<std::string> const& reset_names, std::int64_t interval,
@@ -55,8 +56,8 @@ namespace hpx { namespace util {
       , csv_header_(csv_header)
       , print_counters_locally_(print_counters_locally)
       , counter_types_(counter_types)
-      , timer_(util::bind_front(&query_counters::evaluate, this_(), false),
-            util::bind_front(&query_counters::terminate, this_()),
+      , timer_(hpx::bind_front(&query_counters::evaluate, this_(), false),
+            hpx::bind_front(&query_counters::terminate, this_()),
             interval * 1000, "query_counters", true)
     {
         // add counter prefix, if necessary
@@ -103,8 +104,17 @@ namespace hpx { namespace util {
 
     void query_counters::start()
     {
+#if defined(HPX_GCC_VERSION) && HPX_GCC_VERSION >= 110000
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wrestrict"
+#endif
         if (print_counters_locally_ && destination_ != "cout")
+        {
             destination_ += "." + std::to_string(hpx::get_locality_id());
+        }
+#if defined(HPX_GCC_VERSION) && HPX_GCC_VERSION >= 110000
+#pragma GCC diagnostic pop
+#endif
 
         find_counters();
 
@@ -123,29 +133,29 @@ namespace hpx { namespace util {
     ///////////////////////////////////////////////////////////////////////////
     namespace strings {
 
-        char const* const counter_type_short_names[] = {
-            "text",
-            "instant",
-            "monoinc",
-            "avgbase",
-            "avgcount",
-            "aggregated",
-            "avgtime",
-            "elapsed",
-            "histogram",
-            "values",
+        constexpr char const* counter_type_short_names[] = {
+            "counter_type::text",
+            "counter_type::raw",
+            "counter_type::monotonically_increasing",
+            "counter_type::average_base",
+            "counter_type::average_count",
+            "counter_type::aggregated",
+            "counter_type::average_timer",
+            "counter_type::elapsed_time",
+            "counter_type::histogram",
+            "counter_type::raw_values",
         };
     }
 
     char const* get_counter_short_type_name(
         performance_counters::counter_type type)
     {
-        if (type < performance_counters::counter_text ||
-            type > performance_counters::counter_raw_values)
+        if (type < performance_counters::counter_type::text ||
+            type > performance_counters::counter_type::raw_values)
         {
             return "unknown";
         }
-        return strings::counter_type_short_names[type];
+        return strings::counter_type_short_names[static_cast<int>(type)];
     }
 
     template <typename Stream>
@@ -166,7 +176,7 @@ namespace hpx { namespace util {
         std::string const& name = info.fullname_;
         std::string const& uom = info.unit_of_measure_;
 
-        error_code ec(lightweight);    // do not throw
+        error_code ec(throwmode::lightweight);    // do not throw
         double val = value.get_value<double>(ec);
 
         if (!ec)
@@ -190,7 +200,7 @@ namespace hpx { namespace util {
             print_name_csv(*out, name);
             *out << "," << value.count_ << ",";
 
-            double elapsed = static_cast<double>(value.time_) * 1e-9;
+            double const elapsed = static_cast<double>(value.time_) * 1e-9;
             *out << hpx::util::format("{:.6}", elapsed) << ",[s]," << val;
             if (!uom.empty())
                 *out << ",[" << uom << "]";
@@ -221,12 +231,12 @@ namespace hpx { namespace util {
         std::string const& name = info.fullname_;
         std::string const& uom = info.unit_of_measure_;
 
-        error_code ec(lightweight);    // do not throw
+        error_code ec(throwmode::lightweight);    // do not throw
 
         print_name_csv(*out, name);
         *out << "," << value.count_ << ",";
 
-        double elapsed = static_cast<double>(value.time_) * 1e-9;
+        double const elapsed = static_cast<double>(value.time_) * 1e-9;
         *out << hpx::util::format("{:.6}", elapsed) << ",[s],";
 
         bool first = true;
@@ -255,7 +265,7 @@ namespace hpx { namespace util {
         performance_counters::counter_info const& info,
         performance_counters::counter_value const& value)
     {
-        error_code ec(lightweight);
+        error_code ec(throwmode::lightweight);
         double val = value.get_value<double>(ec);
 
         if (!ec)
@@ -324,12 +334,13 @@ namespace hpx { namespace util {
                 for (std::size_t i = 0; i != infos.size(); ++i)
                 {
                     using namespace performance_counters;
-                    if (infos[i].type_ != counter_raw &&
-                        infos[i].type_ != counter_monotonically_increasing &&
-                        infos[i].type_ != counter_aggregating &&
-                        infos[i].type_ != counter_elapsed_time &&
-                        infos[i].type_ != counter_average_count &&
-                        infos[i].type_ != counter_average_timer)
+                    if (infos[i].type_ != counter_type::raw &&
+                        infos[i].type_ !=
+                            counter_type::monotonically_increasing &&
+                        infos[i].type_ != counter_type::aggregating &&
+                        infos[i].type_ != counter_type::elapsed_time &&
+                        infos[i].type_ != counter_type::average_count &&
+                        infos[i].type_ != counter_type::average_timer)
                     {
                         continue;
                     }
@@ -343,9 +354,9 @@ namespace hpx { namespace util {
                 for (std::size_t i = 0; i != infos.size(); ++i)
                 {
                     if (infos[i].type_ !=
-                            performance_counters::counter_histogram &&
+                            performance_counters::counter_type::histogram &&
                         infos[i].type_ !=
-                            performance_counters::counter_raw_values)
+                            performance_counters::counter_type::raw_values)
                     {
                         continue;
                     }
@@ -365,12 +376,13 @@ namespace hpx { namespace util {
                 for (std::size_t i = 0; i != counter_shortnames_.size(); ++i)
                 {
                     using namespace performance_counters;
-                    if (infos[i].type_ != counter_raw &&
-                        infos[i].type_ != counter_monotonically_increasing &&
-                        infos[i].type_ != counter_aggregating &&
-                        infos[i].type_ != counter_elapsed_time &&
-                        infos[i].type_ != counter_average_count &&
-                        infos[i].type_ != counter_average_timer)
+                    if (infos[i].type_ != counter_type::raw &&
+                        infos[i].type_ !=
+                            counter_type::monotonically_increasing &&
+                        infos[i].type_ != counter_type::aggregating &&
+                        infos[i].type_ != counter_type::elapsed_time &&
+                        infos[i].type_ != counter_type::average_count &&
+                        infos[i].type_ != counter_type::average_timer)
                     {
                         continue;
                     }
@@ -384,9 +396,9 @@ namespace hpx { namespace util {
                 for (std::size_t i = 0; i != counter_shortnames_.size(); ++i)
                 {
                     if (infos[i].type_ !=
-                            performance_counters::counter_histogram &&
+                            performance_counters::counter_type::histogram &&
                         infos[i].type_ !=
-                            performance_counters::counter_raw_values)
+                            performance_counters::counter_type::raw_values)
                     {
                         continue;
                     }
@@ -424,7 +436,7 @@ namespace hpx { namespace util {
         else
         {
             std::size_t idx = 0;
-            for (std::size_t i : indices)
+            for (std::size_t const i : indices)
             {
                 print_value(output, infos[i], values[idx]);
                 ++idx;
@@ -450,7 +462,8 @@ namespace hpx { namespace util {
         if (counters_.size() == 0)
         {
             // start has not been called yet
-            HPX_THROWS_IF(ec, invalid_status, "query_counters::start_counters",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                "query_counters::start_counters",
                 "The counters to be evaluated have not been initialized yet");
             return;
         }
@@ -464,7 +477,8 @@ namespace hpx { namespace util {
         if (counters_.size() == 0)
         {
             // start has not been called yet
-            HPX_THROWS_IF(ec, invalid_status, "query_counters::stop_counters",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                "query_counters::stop_counters",
                 "The counters to be evaluated have not been initialized yet");
             return;
         }
@@ -478,7 +492,8 @@ namespace hpx { namespace util {
         if (counters_.size() == 0)
         {
             // start has not been called yet
-            HPX_THROWS_IF(ec, invalid_status, "query_counters::reset_counters",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                "query_counters::reset_counters",
                 "The counters to be evaluated have not been initialized yet");
             return;
         }
@@ -492,7 +507,8 @@ namespace hpx { namespace util {
         if (counters_.size() == 0)
         {
             // start has not been called yet
-            HPX_THROWS_IF(ec, invalid_status, "query_counters::reinit_counters",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                "query_counters::reinit_counters",
                 "The counters to be evaluated have not been initialized yet");
             return;
         }
@@ -513,8 +529,10 @@ namespace hpx { namespace util {
 
         for (std::size_t i = 0; i != infos.size(); ++i)
         {
-            if (infos[i].type_ == performance_counters::counter_histogram ||
-                infos[i].type_ == performance_counters::counter_raw_values)
+            if (infos[i].type_ ==
+                    performance_counters::counter_type::histogram ||
+                infos[i].type_ ==
+                    performance_counters::counter_type::raw_values)
             {
                 continue;
             }
@@ -537,8 +555,8 @@ namespace hpx { namespace util {
         // Output the performance counter value.
         if (!no_output)
             print_headers(output, infos);
-        print_values(no_output ? nullptr : &output, std::move(values),
-            std::move(indices), infos);
+        print_values(no_output ? nullptr : &output, HPX_MOVE(values),
+            HPX_MOVE(indices), infos);
 
         if (!no_output)
         {
@@ -567,8 +585,10 @@ namespace hpx { namespace util {
 
         for (std::size_t i = 0; i != infos.size(); ++i)
         {
-            if (infos[i].type_ != performance_counters::counter_histogram &&
-                infos[i].type_ != performance_counters::counter_raw_values)
+            if (infos[i].type_ !=
+                    performance_counters::counter_type::histogram &&
+                infos[i].type_ !=
+                    performance_counters::counter_type::raw_values)
             {
                 continue;
             }
@@ -591,8 +611,8 @@ namespace hpx { namespace util {
         // Output the performance counter value.
         if (!no_output)
             print_headers(output, infos);
-        print_values(no_output ? nullptr : &output, std::move(values),
-            std::move(indices), infos);
+        print_values(no_output ? nullptr : &output, HPX_MOVE(values),
+            HPX_MOVE(indices), infos);
 
         if (!no_output)
         {
@@ -618,8 +638,8 @@ namespace hpx { namespace util {
             return false;
         }
 
-        bool destination_is_cout = false;
-        bool no_output = false;
+        bool destination_is_cout;
+        bool no_output;
 
         {
             std::lock_guard<mutex_type> l(mtx_);
@@ -636,16 +656,16 @@ namespace hpx { namespace util {
         if (counters_.size() == 0)
         {
             // start has not been called yet
-            HPX_THROWS_IF(ec, invalid_status, "query_counters::evaluate",
+            HPX_THROWS_IF(ec, hpx::error::invalid_status,
+                "query_counters::evaluate",
                 "The counters to be evaluated have not been initialized yet");
             return false;
         }
 
-        bool result = false;
-        std::vector<performance_counters::counter_info> infos =
+        std::vector<performance_counters::counter_info> const infos =
             counters_.get_counter_infos();
 
-        result = print_raw_counters(
+        bool result = print_raw_counters(
             destination_is_cout, reset, no_output, description, infos, ec);
         if (ec)
             return false;
@@ -661,4 +681,4 @@ namespace hpx { namespace util {
 
         return result;
     }
-}}    // namespace hpx::util
+}    // namespace hpx::util
