@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2021 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -16,7 +16,6 @@
 #include <hpx/runtime_components/derived_component_factory.hpp>
 #include <hpx/runtime_local/runtime_local_fwd.hpp>
 #include <hpx/thread_support/unlock_guard.hpp>
-#include <hpx/timing/high_resolution_clock.hpp>
 
 #include <boost/accumulators/accumulators.hpp>
 #include <boost/accumulators/statistics/max.hpp>
@@ -53,7 +52,8 @@
 #include <vector>
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace performance_counters { namespace server {
+namespace hpx::performance_counters::server {
+
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
         template <typename Statistic>
@@ -63,10 +63,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<boost::accumulators::tag::mean>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::mean aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::mean;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t /*parameter2*/) {}
 
@@ -93,10 +93,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<boost::accumulators::tag::variance>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::variance aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::variance;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t /*parameter2*/) {}
 
@@ -123,13 +123,13 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<boost::accumulators::tag::median>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::median aggregating_tag;
-            typedef boost::accumulators::with_p_square_quantile
-                aggregating_type_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag(
-                    aggregating_type_tag)>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::median;
+            using aggregating_type_tag =
+                boost::accumulators::with_p_square_quantile;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag(
+                        aggregating_type_tag)>>;
 
             counter_type_from_statistic(std::size_t /*parameter2*/) {}
 
@@ -157,10 +157,10 @@ namespace hpx { namespace performance_counters { namespace server {
             boost::accumulators::tag::rolling_mean>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::rolling_mean aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::rolling_mean;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t parameter2)
               : accum_(boost::accumulators::tag::rolling_window::window_size =
@@ -168,7 +168,7 @@ namespace hpx { namespace performance_counters { namespace server {
             {
                 if (parameter2 == 0)
                 {
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "counter_type_from_statistic<Statistic>",
                         "base rolling window size is specified to be zero");
                 }
@@ -198,10 +198,10 @@ namespace hpx { namespace performance_counters { namespace server {
             boost::accumulators::tag::rolling_variance>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::rolling_variance aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::rolling_variance;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t parameter2)
               : accum_(boost::accumulators::tag::rolling_window::window_size =
@@ -209,7 +209,7 @@ namespace hpx { namespace performance_counters { namespace server {
             {
                 if (parameter2 == 0)
                 {
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "counter_type_from_statistic<Statistic>",
                         "base rolling window size is specified to be zero");
                 }
@@ -238,10 +238,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<boost::accumulators::tag::max>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::max aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::max;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t /*parameter2*/) {}
 
@@ -268,10 +268,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<boost::accumulators::tag::min>
           : counter_type_from_statistic_base
         {
-            typedef boost::accumulators::tag::min aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = boost::accumulators::tag::min;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t /*parameter2*/) {}
 
@@ -298,10 +298,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<hpx::util::tag::rolling_min>
           : counter_type_from_statistic_base
         {
-            typedef hpx::util::tag::rolling_min aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = hpx::util::tag::rolling_min;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t parameter2)
               : accum_(boost::accumulators::tag::rolling_window::window_size =
@@ -309,7 +309,7 @@ namespace hpx { namespace performance_counters { namespace server {
             {
                 if (parameter2 == 0)
                 {
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "counter_type_from_statistic<Statistic>",
                         "base rolling window size is specified to be zero");
                 }
@@ -338,10 +338,10 @@ namespace hpx { namespace performance_counters { namespace server {
         struct counter_type_from_statistic<hpx::util::tag::rolling_max>
           : counter_type_from_statistic_base
         {
-            typedef hpx::util::tag::rolling_max aggregating_tag;
-            typedef boost::accumulators::accumulator_set<double,
-                boost::accumulators::stats<aggregating_tag>>
-                accumulator_type;
+            using aggregating_tag = hpx::util::tag::rolling_max;
+            using accumulator_type =
+                boost::accumulators::accumulator_set<double,
+                    boost::accumulators::stats<aggregating_tag>>;
 
             counter_type_from_statistic(std::size_t parameter2)
               : accum_(boost::accumulators::tag::rolling_window::window_size =
@@ -349,7 +349,7 @@ namespace hpx { namespace performance_counters { namespace server {
             {
                 if (parameter2 == 0)
                 {
-                    HPX_THROW_EXCEPTION(bad_parameter,
+                    HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                         "counter_type_from_statistic<Statistic>",
                         "base rolling window size is specified to be zero");
                 }
@@ -376,13 +376,22 @@ namespace hpx { namespace performance_counters { namespace server {
     }    // namespace detail
 
     ///////////////////////////////////////////////////////////////////////////
+    template <typename Statistic>
+    statistics_counter<Statistic>::statistics_counter()
+      : has_prev_value_(false)
+      , parameter1_(0)
+      , parameter2_(0)
+      , reset_base_counter_(false)
+    {
+    }
+
     template <typename Statistic>
     statistics_counter<Statistic>::statistics_counter(counter_info const& info,
         std::string const& base_counter_name, std::size_t parameter1,
         std::size_t parameter2, bool reset_base_counter)
       : base_type_holder(info)
-      , timer_(util::bind_front(&statistics_counter::evaluate, this_()),
-            util::bind_front(&statistics_counter::on_terminate, this_()),
+      , timer_(hpx::bind_front(&statistics_counter::evaluate, this_()),
+            hpx::bind_front(&statistics_counter::on_terminate, this_()),
             1000 * parameter1, info.fullname_, true)
       , base_counter_name_(ensure_counter_prefix(base_counter_name))
       , value_(new detail::counter_type_from_statistic<Statistic>(parameter2))
@@ -393,20 +402,20 @@ namespace hpx { namespace performance_counters { namespace server {
     {
         if (parameter1 == 0)
         {
-            HPX_THROW_EXCEPTION(bad_parameter,
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                 "statistics_counter<Statistic>::statistics_counter",
                 "base interval is specified to be zero");
         }
 
-        if (info.type_ != counter_aggregating)
+        if (info.type_ != counter_type::aggregating)
         {
-            HPX_THROW_EXCEPTION(bad_parameter,
+            HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                 "statistics_counter<Statistic>::statistics_counter",
                 "unexpected counter type specified");
         }
 
         // make sure this counter starts collecting data
-        start();
+        statistics_counter<Statistic>::start();
     }
 
     template <typename Statistic>
@@ -418,7 +427,7 @@ namespace hpx { namespace performance_counters { namespace server {
         hpx::performance_counters::counter_value value;
 
         prev_value_.value_ = static_cast<std::int64_t>(value_->get_value());
-        prev_value_.status_ = status_new_data;
+        prev_value_.status_ = counter_status::new_data;
         prev_value_.time_ = static_cast<std::int64_t>(hpx::get_system_uptime());
         prev_value_.count_ = ++invocation_count_;
         has_prev_value_ = true;
@@ -451,7 +460,7 @@ namespace hpx { namespace performance_counters { namespace server {
             base_value.scale_inverse_ != prev_value_.scale_inverse_)
         {
             // not supported right now
-            HPX_THROW_EXCEPTION(not_implemented,
+            HPX_THROW_EXCEPTION(hpx::error::not_implemented,
                 "statistics_counter<Statistic>::evaluate",
                 "base counter should keep scaling constant over time");
             return false;
@@ -475,11 +484,11 @@ namespace hpx { namespace performance_counters { namespace server {
         if (!base_counter_id_)
         {
             // get or create the base counter
-            error_code ec(lightweight);
+            error_code ec(throwmode::lightweight);
             hpx::id_type base_counter_id;
             {
                 // We need to unlock the lock here since get_counter might suspend
-                util::unlock_guard<std::unique_lock<mutex_type>> unlock(l);
+                unlock_guard<std::unique_lock<mutex_type>> unlock(l);
                 base_counter_id = get_counter(base_counter_name_, ec);
             }
 
@@ -498,7 +507,7 @@ namespace hpx { namespace performance_counters { namespace server {
             if (HPX_UNLIKELY(ec || !base_counter_id_))
             {
                 // base counter could not be retrieved
-                HPX_THROW_EXCEPTION(bad_parameter,
+                HPX_THROW_EXCEPTION(hpx::error::bad_parameter,
                     "statistics_counter<Statistic>::evaluate_base_counter",
                     "could not get or create performance counter: '{}'",
                     base_counter_name_);
@@ -596,7 +605,28 @@ namespace hpx { namespace performance_counters { namespace server {
         // start off with last base value
         value_->add_value(static_cast<double>(prev_value_.value_));
     }
-}}}    // namespace hpx::performance_counters::server
+
+    template <typename Statistic>
+    void statistics_counter<Statistic>::on_terminate()
+    {
+    }
+
+    template <typename Statistic>
+    void statistics_counter<Statistic>::finalize()
+    {
+        base_performance_counter::finalize();
+        base_type::finalize();
+    }
+
+    template <typename Statistic>
+    naming::address statistics_counter<Statistic>::get_current_address() const
+    {
+        return naming::address(
+            naming::get_gid_from_locality_id(agas::get_locality_id()),
+            components::get_component_type<statistics_counter>(),
+            const_cast<statistics_counter*>(this));
+    }
+}    // namespace hpx::performance_counters::server
 
 ///////////////////////////////////////////////////////////////////////////////
 template class HPX_EXPORT hpx::performance_counters::server::statistics_counter<
@@ -620,162 +650,157 @@ template class HPX_EXPORT hpx::performance_counters::server::statistics_counter<
 
 ///////////////////////////////////////////////////////////////////////////////
 // Average
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::mean>>
-    average_count_counter_type;
+using average_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::mean>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(average_count_counter_type,
     average_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(average_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Rolling variance
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::rolling_variance>>
-    rolling_variance_count_counter_type;
+using rolling_variance_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::rolling_variance>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(rolling_variance_count_counter_type,
     rolling_variance_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(rolling_variance_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Variance
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::variance>>
-    variance_count_counter_type;
+using variance_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::variance>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(variance_count_counter_type,
     variance_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(variance_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Rolling average
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::rolling_mean>>
-    rolling_mean_count_counter_type;
+using rolling_mean_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::rolling_mean>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(rolling_mean_count_counter_type,
     rolling_mean_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(rolling_mean_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Median
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::median>>
-    median_count_counter_type;
+using median_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::median>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(median_count_counter_type,
     median_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(median_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Max
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::max>>
-    max_count_counter_type;
+using max_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::max>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(max_count_counter_type,
     max_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(max_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Min
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<boost::accumulators::tag::min>>
-    min_count_counter_type;
+using min_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<boost::accumulators::tag::min>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(min_count_counter_type,
     min_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(min_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Rolling min
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<hpx::util::tag::rolling_min>>
-    rolling_min_count_counter_type;
+using rolling_min_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<hpx::util::tag::rolling_min>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(rolling_min_count_counter_type,
     rolling_min_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(rolling_min_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
 // Rolling max
-typedef hpx::components::component<hpx::performance_counters::server::
-        statistics_counter<hpx::util::tag::rolling_max>>
-    rolling_max_count_counter_type;
+using rolling_max_count_counter_type =
+    hpx::components::component<hpx::performance_counters::server::
+            statistics_counter<hpx::util::tag::rolling_max>>;
 
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(rolling_max_count_counter_type,
     rolling_max_count_counter, "base_performance_counter",
-    hpx::components::factory_enabled)
+    hpx::components::factory_state::enabled)
 HPX_DEFINE_GET_COMPONENT_TYPE(rolling_max_count_counter_type::wrapped_type)
 
 ///////////////////////////////////////////////////////////////////////////////
-namespace hpx { namespace performance_counters { namespace detail {
+namespace hpx::performance_counters::detail {
+
     /// Creation function for aggregating performance counters to be registered
     /// with the counter types.
     naming::gid_type statistics_counter_creator(
         counter_info const& info, error_code& ec)
     {
-        switch (info.type_)
+        if (info.type_ != counter_type::aggregating)
         {
-        case counter_aggregating:
-        {
-            counter_path_elements paths;
-            get_counter_path_elements(info.fullname_, paths, ec);
-            if (ec)
-                return naming::invalid_gid;
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                "statistics_counter_creator", "invalid counter type requested");
+            return naming::invalid_gid;
+        }
 
-            if (!paths.parentinstance_is_basename_)
+        counter_path_elements paths;
+        get_counter_path_elements(info.fullname_, paths, ec);
+        if (ec)
+            return naming::invalid_gid;
+
+        if (!paths.parentinstance_is_basename_)
+        {
+            HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                "statistics_counter_creator",
+                "invalid aggregate counter "
+                "name (instance name must be valid base counter name)");
+            return naming::invalid_gid;
+        }
+
+        std::string base_name;
+        get_counter_name(paths.parentinstancename_, base_name, ec);
+        if (ec)
+            return naming::invalid_gid;
+
+        std::vector<std::size_t> parameters;
+        if (!paths.parameters_.empty())
+        {
+            // try to interpret the additional parameters
+            namespace x3 = boost::spirit::x3;
+            if (!x3::parse(paths.parameters_.begin(), paths.parameters_.end(),
+                    x3::uint_ % ',', parameters))
             {
-                HPX_THROWS_IF(ec, bad_parameter, "statistics_counter_creator",
-                    "invalid aggregate counter "
-                    "name (instance name must be valid base counter name)");
+                HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                    "statistics_counter_creator",
+                    "invalid parameter specification format for "
+                    "this counter: {}",
+                    paths.parameters_);
                 return naming::invalid_gid;
             }
-
-            std::string base_name;
-            get_counter_name(paths.parentinstancename_, base_name, ec);
-            if (ec)
-                return naming::invalid_gid;
-
-            std::vector<std::size_t> parameters;
-            if (!paths.parameters_.empty())
+            if (paths.countername_.find("rolling") != std::string::npos)
             {
-                // try to interpret the additional parameters
-                namespace x3 = boost::spirit::x3;
-                if (!x3::parse(paths.parameters_.begin(),
-                        paths.parameters_.end(), x3::uint_ % ',', parameters))
+                if (parameters.size() > 3)
                 {
-                    HPX_THROWS_IF(ec, bad_parameter,
-                        "statistics_counter_creator",
-                        "invalid parameter specification format for "
-                        "this counter: {}",
-                        paths.parameters_);
-                    return naming::invalid_gid;
-                }
-                if (paths.countername_.find("rolling") != std::string::npos)
-                {
-                    if (parameters.size() > 3)
-                    {
-                        HPX_THROWS_IF(ec, bad_parameter,
-                            "statistics_counter_creator",
-                            "too many parameter specifications for "
-                            "this counter: {}",
-                            paths.parameters_);
-                        return naming::invalid_gid;
-                    }
-                }
-                else if (parameters.size() > 2)
-                {
-                    HPX_THROWS_IF(ec, bad_parameter,
+                    HPX_THROWS_IF(ec, hpx::error::bad_parameter,
                         "statistics_counter_creator",
                         "too many parameter specifications for "
                         "this counter: {}",
@@ -783,25 +808,28 @@ namespace hpx { namespace performance_counters { namespace detail {
                     return naming::invalid_gid;
                 }
             }
-            else if (paths.countername_.find("rolling") != std::string::npos)
+            else if (parameters.size() > 2)
             {
-                parameters.push_back(1000);    // sample interval
-                parameters.push_back(10);      // rolling window
-                parameters.push_back(0);       // don't reset underlying counter
+                HPX_THROWS_IF(ec, hpx::error::bad_parameter,
+                    "statistics_counter_creator",
+                    "too many parameter specifications for "
+                    "this counter: {}",
+                    paths.parameters_);
+                return naming::invalid_gid;
             }
-            else
-            {
-                parameters.push_back(1000);    // sample interval
-                parameters.push_back(0);       // don't reset underlying counter
-            }
-            return create_statistics_counter(info, base_name, parameters, ec);
         }
-        break;
+        else if (paths.countername_.find("rolling") != std::string::npos)
+        {
+            parameters.push_back(1000);    // sample interval
+            parameters.push_back(10);      // rolling window
+            parameters.push_back(0);       // don't reset underlying counter
+        }
+        else
+        {
+            parameters.push_back(1000);    // sample interval
+            parameters.push_back(0);       // don't reset underlying counter
+        }
 
-        default:
-            HPX_THROWS_IF(ec, bad_parameter, "statistics_counter_creator",
-                "invalid counter type requested");
-            return naming::invalid_gid;
-        }
+        return create_statistics_counter(info, base_name, parameters, ec);
     }
-}}}    // namespace hpx::performance_counters::detail
+}    // namespace hpx::performance_counters::detail
