@@ -18,7 +18,7 @@ namespace hpx { namespace components {
 
     // Stub function which applies the console_error_sink action.
     HPX_EXPORT void console_error_sink(
-        naming::id_type const& dst, std::exception_ptr const& e);
+        hpx::id_type const& dst, std::exception_ptr const& e);
 
     // Stub function which applies the console_error_sink action.
     HPX_EXPORT void console_error_sink(std::exception_ptr const& e);
