@@ -5,6 +5,8 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 /// \file new.hpp
+/// \page hpx::new_
+/// \headerfile hpx/components.hpp
 
 #pragma once
 
@@ -19,7 +21,6 @@
 #include <hpx/futures/future.hpp>
 #include <hpx/naming_base/id_type.hpp>
 #include <hpx/runtime_components/create_component_helpers.hpp>
-#include <hpx/runtime_components/default_distribution_policy.hpp>
 #include <hpx/type_support/lazy_enable_if.hpp>
 
 #include <algorithm>
@@ -67,7 +68,7 @@ namespace hpx {
     ///          component instance.
     ///
     template <typename Component, typename... Ts>
-    <unspecified> new_(id_type const& locality, Ts&&... vs);
+    auto new_(id_type const& locality, Ts&&... vs);
 
     /// \brief Create one new instance of the given Component type on the
     /// current locality.
@@ -111,7 +112,7 @@ namespace hpx {
     ///          only.
     ///
     template <typename Component, typename... Ts>
-    <unspecified> local_new(Ts&&... vs);
+    auto local_new(Ts&&... vs);
 
     /// \brief Create multiple new instances of the given Component type on the
     /// specified locality.
@@ -156,7 +157,7 @@ namespace hpx {
     ///          components.
     ///
     template <typename Component, typename... Ts>
-    <unspecified> new_(id_type const& locality, std::size_t count, Ts&&... vs);
+    auto new_(id_type const& locality, std::size_t count, Ts&&... vs);
 
     /// \brief Create one or more new instances of the given Component type
     /// based on the given distribution policy.
@@ -195,7 +196,7 @@ namespace hpx {
     ///          component instance.
     ///
     template <typename Component, typename DistPolicy, typename... Ts>
-    <unspecified> new_(DistPolicy const& policy, Ts&&... vs);
+    auto new_(DistPolicy const& policy, Ts&&... vs);
 
     /// \brief Create multiple new instances of the given Component type on the
     /// localities as defined by the given distribution policy.
@@ -240,14 +241,16 @@ namespace hpx {
     ///          components.
     ///
     template <typename Component, typename DistPolicy, typename... Ts>
-    <unspecified> new_(DistPolicy const& policy, std::size_t count, Ts&&... vs);
+    auto new_(DistPolicy const& policy, std::size_t count, Ts&&... vs);
 }    // namespace hpx
 
 #else
 
-namespace hpx { namespace components {
+namespace hpx::components {
+
     ///////////////////////////////////////////////////////////////////////////
     namespace detail {
+
         // create a single instance of a component
         template <typename Component>
         struct new_component
@@ -258,14 +261,14 @@ namespace hpx { namespace components {
             static type call(hpx::id_type const& locality, Ts&&... vs)
             {
                 return components::create_async<Component>(
-                    locality, std::forward<Ts>(vs)...);
+                    locality, HPX_FORWARD(Ts, vs)...);
             }
 
             template <typename DistPolicy, typename... Ts>
             static type call(DistPolicy const& policy, Ts&&... vs)
             {
                 return policy.template create<Component>(
-                    std::forward<Ts>(vs)...);
+                    HPX_FORWARD(Ts, vs)...);
             }
         };
 
@@ -279,8 +282,8 @@ namespace hpx { namespace components {
             static type call(
                 hpx::id_type const& locality, std::size_t count, Ts&&... vs)
             {
-                return components::bulk_create_async<Component>(
-                    locality, count, std::forward<Ts>(vs)...);
+                return components::bulk_create_async<false, Component>(
+                    locality, count, HPX_FORWARD(Ts, vs)...);
             }
 
             template <typename DistPolicy, typename... Ts>
@@ -291,8 +294,8 @@ namespace hpx { namespace components {
                     typename DistPolicy::bulk_locality_result;
 
                 hpx::future<std::vector<bulk_locality_result>> f =
-                    policy.template bulk_create<Component>(
-                        count, std::forward<Ts>(vs)...);
+                    policy.template bulk_create<false, Component>(
+                        count, HPX_FORWARD(Ts, vs)...);
 
                 return f.then(launch::sync,
                     [count](hpx::future<std::vector<bulk_locality_result>>&& f)
@@ -322,9 +325,9 @@ namespace hpx { namespace components {
                 using component_type = typename Component::wrapping_type;
 
                 hpx::id_type id(components::server::create<component_type>(
-                                    std::forward<Ts>(ts)...),
-                    hpx::id_type::managed);
-                return hpx::make_ready_future(std::move(id));
+                                    HPX_FORWARD(Ts, ts)...),
+                    hpx::id_type::management_type::managed);
+                return hpx::make_ready_future(HPX_MOVE(id));
             }
         };
 
@@ -343,9 +346,9 @@ namespace hpx { namespace components {
 
                 for (std::size_t i = 0; i != count; ++i)
                 {
-                    result.push_back(hpx::id_type(
+                    result.emplace_back(
                         components::server::create<component_type>(ts...),
-                        hpx::id_type::managed));
+                        hpx::id_type::management_type::managed);
                 }
 
                 return hpx::make_ready_future(result);
@@ -364,8 +367,8 @@ namespace hpx { namespace components {
                 using component_type = typename Component::wrapping_type;
 
                 hpx::id_type id(components::server::create<component_type>(
-                                    std::forward<Ts>(ts)...),
-                    hpx::id_type::managed);
+                                    HPX_FORWARD(Ts, ts)...),
+                    hpx::id_type::management_type::managed);
 
                 return id;
             }
@@ -386,9 +389,9 @@ namespace hpx { namespace components {
 
                 for (std::size_t i = 0; i != count; ++i)
                 {
-                    result.push_back(hpx::id_type(
+                    result.emplace_back(
                         components::server::create<component_type>(ts...),
-                        hpx::id_type::managed));
+                        hpx::id_type::management_type::managed);
                 }
 
                 return result;
@@ -407,11 +410,11 @@ namespace hpx { namespace components {
             agas::get_locality_id())
         {
             return detail::local_new_component<Component>::call(
-                std::forward<Ts>(vs)...);
+                HPX_FORWARD(Ts, vs)...);
         }
 
         return detail::new_component<Component>::call(
-            locality, std::forward<Ts>(vs)...);
+            locality, HPX_FORWARD(Ts, vs)...);
     }
 
     template <typename Component, typename DistPolicy, typename... Ts>
@@ -422,7 +425,7 @@ namespace hpx { namespace components {
     new_(DistPolicy const& policy, Ts&&... vs)
     {
         return detail::new_component<Component>::call(
-            policy, std::forward<Ts>(vs)...);
+            policy, HPX_FORWARD(Ts, vs)...);
     }
 
     ///////////////////////////////////////////////////////////////////////////
@@ -440,7 +443,7 @@ namespace hpx { namespace components {
             {
                 return make_client<Client>(
                     components::create_async<component_type>(
-                        locality, std::forward<Ts>(vs)...));
+                        locality, HPX_FORWARD(Ts, vs)...));
             }
 
             template <typename DistPolicy, typename... Ts>
@@ -448,7 +451,7 @@ namespace hpx { namespace components {
             {
                 return make_client<Client>(
                     policy.template create<component_type>(
-                        std::forward<Ts>(vs)...));
+                        HPX_FORWARD(Ts, vs)...));
             }
         };
 
@@ -463,7 +466,7 @@ namespace hpx { namespace components {
             static type call(Ts&&... vs)
             {
                 return new_component<component_type[]>::call(
-                    std::forward<Ts>(vs)...)
+                    HPX_FORWARD(Ts, vs)...)
                     .then(hpx::launch::sync,
                         [](hpx::future<std::vector<hpx::id_type>>&& v)
                             -> std::vector<Client> {
@@ -484,9 +487,9 @@ namespace hpx { namespace components {
             static type call(Ts&&... ts)
             {
                 hpx::id_type id(components::server::create<component_type>(
-                                    std::forward<Ts>(ts)...),
-                    hpx::id_type::managed);
-                return make_client<Client>(std::move(id));
+                                    HPX_FORWARD(Ts, ts)...),
+                    hpx::id_type::management_type::managed);
+                return make_client<Client>(HPX_MOVE(id));
             }
         };
 
@@ -501,7 +504,7 @@ namespace hpx { namespace components {
             static type call(Ts&&... ts)
             {
                 return local_new_component<component_type[]>::call(
-                    std::forward<Ts>(ts)...)
+                    HPX_FORWARD(Ts, ts)...)
                     .then(hpx::launch::sync,
                         [](hpx::future<std::vector<hpx::id_type>>&& v)
                             -> std::vector<Client> {
@@ -513,8 +516,7 @@ namespace hpx { namespace components {
 
     ///////////////////////////////////////////////////////////////////////////
     template <typename Client, typename... Ts>
-    inline typename util::lazy_enable_if<
-        traits::is_client_or_client_array<Client>::value,
+    typename util::lazy_enable_if<traits::is_client_or_client_array_v<Client>,
         detail::new_client<Client>>::type
     new_(id_type const& locality, Ts&&... vs)
     {
@@ -522,31 +524,29 @@ namespace hpx { namespace components {
             agas::get_locality_id())
         {
             return detail::local_new_client<Client>::call(
-                std::forward<Ts>(vs)...);
+                HPX_FORWARD(Ts, vs)...);
         }
 
         return detail::new_client<Client>::call(
-            locality, std::forward<Ts>(vs)...);
+            locality, HPX_FORWARD(Ts, vs)...);
     }
 
     template <typename Client, typename DistPolicy, typename... Ts>
-    inline typename util::lazy_enable_if<
-        traits::is_client_or_client_array<Client>::value &&
-            traits::is_distribution_policy<DistPolicy>::value,
+    typename util::lazy_enable_if<traits::is_client_or_client_array_v<Client> &&
+            traits::is_distribution_policy_v<DistPolicy>,
         detail::new_client<Client>>::type
     new_(DistPolicy const& policy, Ts&&... vs)
     {
-        return detail::new_client<Client>::call(
-            policy, std::forward<Ts>(vs)...);
+        return detail::new_client<Client>::call(policy, HPX_FORWARD(Ts, vs)...);
     }
 
     ///////////////////////////////////////////////////////////////////////////
     // Same as above, but just on this locality. This does not go through an
-    // action, that means that the constructor arguments can be non-copyable
-    // and non-movable.
+    // action, that means that the constructor arguments can be non-copyable and
+    // non-movable.
     template <typename Component>
-    inline typename util::lazy_enable_if<
-        traits::is_component_or_component_array<Component>::value,
+    typename util::lazy_enable_if<
+        traits::is_component_or_component_array_v<Component>,
         detail::local_new_component<Component>>::type
     local_new()
     {
@@ -554,37 +554,35 @@ namespace hpx { namespace components {
     }
 
     template <typename Component, typename T1, typename... Ts>
-    inline typename util::lazy_enable_if<
-        traits::is_component_or_component_array<Component>::value &&
-            !std::is_same<typename std::decay<T1>::type,
-                launch::sync_policy>::value,
+    typename util::lazy_enable_if<
+        traits::is_component_or_component_array_v<Component> &&
+            !std::is_same_v<std::decay_t<T1>, launch::sync_policy>,
         detail::local_new_component<Component>>::type
     local_new(T1&& t1, Ts&&... ts)
     {
         return detail::local_new_component<Component>::call(
-            std::forward<T1>(t1), std::forward<Ts>(ts)...);
+            HPX_FORWARD(T1, t1), HPX_FORWARD(Ts, ts)...);
     }
 
     template <typename Component, typename... Ts>
-    inline typename util::lazy_enable_if<
-        traits::is_component_or_component_array<Component>::value,
+    typename util::lazy_enable_if<
+        traits::is_component_or_component_array_v<Component>,
         detail::local_new_component_sync<Component>>::type
     local_new(launch::sync_policy, Ts&&... ts)
     {
         return detail::local_new_component_sync<Component>::call(
-            std::forward<Ts>(ts)...);
+            HPX_FORWARD(Ts, ts)...);
     }
 
     ///////////////////////////////////////////////////////////////////////////
     template <typename Client, typename... Ts>
-    inline typename util::lazy_enable_if<
-        traits::is_client_or_client_array<Client>::value,
+    typename util::lazy_enable_if<traits::is_client_or_client_array_v<Client>,
         detail::local_new_client<Client>>::type
     local_new(Ts&&... ts)
     {
-        return detail::local_new_client<Client>::call(std::forward<Ts>(ts)...);
+        return detail::local_new_client<Client>::call(HPX_FORWARD(Ts, ts)...);
     }
-}}    // namespace hpx::components
+}    // namespace hpx::components
 
 namespace hpx {
 
