@@ -12,7 +12,7 @@
 
 #include "server/simple_refcnt_checker.hpp"
 
-HPX_REGISTER_COMPONENT_MODULE();
+HPX_REGISTER_COMPONENT_MODULE()
 
 using hpx::test::server::simple_refcnt_checker;
 
@@ -27,6 +27,6 @@ HPX_REGISTER_DISABLED_COMPONENT_FACTORY(
 
 ///////////////////////////////////////////////////////////////////////////////
 HPX_REGISTER_ACTION(simple_refcnt_checker::take_reference_action,
-    simple_refcnt_checker_take_reference_action);
+    simple_refcnt_checker_take_reference_action)
 
 #endif
