@@ -1,5 +1,5 @@
 //  Copyright (c) 2019 Maximilian Bremer
-//  Copyright (c) 2019 Hartmut Kaiser
+//  Copyright (c) 2019-2023 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -22,6 +22,8 @@
 #include <utility>
 #include <vector>
 
+constexpr std::size_t N = 50;
+
 ///////////////////////////////////////////////////////////////////////////////
 struct test_server_base
   : hpx::components::abstract_base_migration_support<
@@ -37,7 +39,7 @@ struct test_server_base
     {
         return hpx::find_here();
     }
-    HPX_DEFINE_COMPONENT_ACTION(test_server_base, call, call_action);
+    HPX_DEFINE_COMPONENT_ACTION(test_server_base, call, call_action)
 
     void busy_work() const
     {
@@ -45,7 +47,7 @@ struct test_server_base
         hpx::this_thread::sleep_for(std::chrono::seconds(1));
         HPX_TEST_NEQ(pin_count(), std::uint32_t(0));
     }
-    HPX_DEFINE_COMPONENT_ACTION(test_server_base, busy_work, busy_work_action);
+    HPX_DEFINE_COMPONENT_ACTION(test_server_base, busy_work, busy_work_action)
 
     hpx::future<void> lazy_busy_work() const
     {
@@ -60,7 +62,7 @@ struct test_server_base
         });
     }
     HPX_DEFINE_COMPONENT_ACTION(
-        test_server_base, lazy_busy_work, lazy_busy_work_action);
+        test_server_base, lazy_busy_work, lazy_busy_work_action)
 
     int get_base_data() const
     {
@@ -68,7 +70,7 @@ struct test_server_base
         return base_data_;
     }
     HPX_DEFINE_COMPONENT_ACTION(
-        test_server_base, get_base_data, get_base_data_action);
+        test_server_base, get_base_data, get_base_data_action)
 
     hpx::future<int> lazy_get_base_data() const
     {
@@ -84,7 +86,7 @@ struct test_server_base
         });
     }
     HPX_DEFINE_COMPONENT_ACTION(
-        test_server_base, lazy_get_base_data, lazy_get_base_data_action);
+        test_server_base, lazy_get_base_data, lazy_get_base_data_action)
 
     virtual int get_data() const
     {
@@ -95,7 +97,7 @@ struct test_server_base
         return get_data();
     }
     HPX_DEFINE_COMPONENT_ACTION(
-        test_server_base, get_data_nonvirt, get_data_action);
+        test_server_base, get_data_nonvirt, get_data_action)
 
     virtual hpx::future<int> lazy_get_data() const
     {
@@ -106,7 +108,7 @@ struct test_server_base
         return lazy_get_data();
     }
     HPX_DEFINE_COMPONENT_ACTION(
-        test_server_base, lazy_get_data_nonvirt, lazy_get_data_action);
+        test_server_base, lazy_get_data_nonvirt, lazy_get_data_action)
 
     // Components which should be migrated using hpx::migrate<> need to
     // be Serializable and CopyConstructable. Components can be
@@ -131,39 +133,47 @@ struct test_server_base
         // clang-format on
     }
 
+    hpx::naming::address get_current_address() const override
+    {
+        return hpx::naming::address(
+            hpx::naming::get_gid_from_locality_id(hpx::get_locality_id()),
+            hpx::components::get_component_type<test_server_base>(),
+            const_cast<test_server_base*>(this));
+    }
+
 private:
     int base_data_;
 };
 
-HPX_DEFINE_GET_COMPONENT_TYPE(test_server_base);
+HPX_DEFINE_GET_COMPONENT_TYPE(test_server_base)
 
 typedef test_server_base::call_action call_action;
-HPX_REGISTER_ACTION_DECLARATION(call_action);
-HPX_REGISTER_ACTION(call_action);
+HPX_REGISTER_ACTION_DECLARATION(call_action)
+HPX_REGISTER_ACTION(call_action)
 
 typedef test_server_base::busy_work_action busy_work_action;
-HPX_REGISTER_ACTION_DECLARATION(busy_work_action);
-HPX_REGISTER_ACTION(busy_work_action);
+HPX_REGISTER_ACTION_DECLARATION(busy_work_action)
+HPX_REGISTER_ACTION(busy_work_action)
 
 typedef test_server_base::lazy_busy_work_action lazy_busy_work_action;
-HPX_REGISTER_ACTION_DECLARATION(lazy_busy_work_action);
-HPX_REGISTER_ACTION(lazy_busy_work_action);
+HPX_REGISTER_ACTION_DECLARATION(lazy_busy_work_action)
+HPX_REGISTER_ACTION(lazy_busy_work_action)
 
 typedef test_server_base::get_base_data_action get_base_data_action;
-HPX_REGISTER_ACTION_DECLARATION(get_base_data_action);
-HPX_REGISTER_ACTION(get_base_data_action);
+HPX_REGISTER_ACTION_DECLARATION(get_base_data_action)
+HPX_REGISTER_ACTION(get_base_data_action)
 
 typedef test_server_base::lazy_get_base_data_action lazy_get_base_data_action;
-HPX_REGISTER_ACTION_DECLARATION(lazy_get_base_data_action);
-HPX_REGISTER_ACTION(lazy_get_base_data_action);
+HPX_REGISTER_ACTION_DECLARATION(lazy_get_base_data_action)
+HPX_REGISTER_ACTION(lazy_get_base_data_action)
 
 typedef test_server_base::get_data_action get_data_action;
-HPX_REGISTER_ACTION_DECLARATION(get_data_action);
-HPX_REGISTER_ACTION(get_data_action);
+HPX_REGISTER_ACTION_DECLARATION(get_data_action)
+HPX_REGISTER_ACTION(get_data_action)
 
 typedef test_server_base::lazy_get_data_action lazy_get_data_action;
-HPX_REGISTER_ACTION_DECLARATION(lazy_get_data_action);
-HPX_REGISTER_ACTION(lazy_get_data_action);
+HPX_REGISTER_ACTION_DECLARATION(lazy_get_data_action)
+HPX_REGISTER_ACTION(lazy_get_data_action)
 
 ///////////////////////////////////////////////////////////////////////////////
 struct test_server
@@ -203,13 +213,13 @@ struct test_server
     // be Serializable and CopyConstructable. Components can be
     // MoveConstructable in which case the serialized data is moved into the
     // component's constructor.
-    test_server(test_server&& rhs)
+    test_server(test_server&& rhs) noexcept
       : base_type(std::move(rhs))
       , data_(rhs.data_)
     {
     }
 
-    test_server& operator=(test_server&& rhs)
+    test_server& operator=(test_server&& rhs) noexcept
     {
         this->test_server_base::operator=(
             std::move(static_cast<test_server_base&>(rhs)));
@@ -226,13 +236,21 @@ struct test_server
         // clang-format on
     }
 
+    hpx::naming::address get_current_address() const override
+    {
+        return hpx::naming::address(
+            hpx::naming::get_gid_from_locality_id(hpx::get_locality_id()),
+            hpx::components::get_component_type<test_server>(),
+            const_cast<test_server*>(this));
+    }
+
 private:
     int data_;
 };
 
 typedef hpx::components::component<test_server> server_type;
 HPX_REGISTER_DERIVED_COMPONENT_FACTORY(
-    server_type, test_server, "test_server_base");
+    server_type, test_server, "test_server_base")
 
 ///////////////////////////////////////////////////////////////////////////////
 struct test_client : hpx::components::client_base<test_client, test_server_base>
@@ -292,7 +310,7 @@ bool test_migrate_polymorphic_component(
 {
     // create component on given locality
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
@@ -301,13 +319,13 @@ bool test_migrate_polymorphic_component(
 
     try
     {
-        hpx::cout << "Migrating..." << hpx::endl;
+        hpx::cout << "Migrating..." << std::endl;
         // migrate t1 to the target
         test_client t2(hpx::components::migrate<test_server>(t1, target));
 
         // wait for migration to be done
-        HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
-        hpx::cout << "...completed migrating..." << hpx::endl;
+        HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
+        hpx::cout << "...completed migrating..." << std::endl;
 
         // the migrated object should have the same id as before
         HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -317,7 +335,7 @@ bool test_migrate_polymorphic_component(
         HPX_TEST_EQ(t2.get_data(), 42);
         HPX_TEST_EQ(t2.get_base_data(), 7);
 
-        hpx::cout << "...pass all tests!" << hpx::endl;
+        hpx::cout << "...pass all tests!" << std::endl;
     }
     catch (hpx::exception const& e)
     {
@@ -334,7 +352,7 @@ bool test_migrate_lazy_polymorphic_component(
 {
     // create component on given locality
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
@@ -347,7 +365,7 @@ bool test_migrate_lazy_polymorphic_component(
         test_client t2(hpx::components::migrate<test_server>(t1, target));
 
         // wait for migration to be done
-        HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+        HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
         // the migrated object should have the same id as before
         HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -372,7 +390,7 @@ bool test_migrate_busy_polymorphic_component(
 {
     // create component on given locality
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
@@ -391,7 +409,7 @@ bool test_migrate_busy_polymorphic_component(
         HPX_TEST_EQ(t1.get_base_data(), 7);
 
         // wait for migration to be done
-        HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+        HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
         // the migrated object should have the same id as before
         HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -419,7 +437,7 @@ bool test_migrate_lazy_busy_polymorphic_component(
 {
     // create component on given locality
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
@@ -438,7 +456,7 @@ bool test_migrate_lazy_busy_polymorphic_component(
         HPX_TEST_EQ(t1.lazy_get_base_data(), 7);
 
         // wait for migration to be done
-        HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+        HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
         // the migrated object should have the same id as before
         HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -465,15 +483,13 @@ bool test_migrate_polymorphic_component2(
     hpx::id_type source, hpx::id_type target)
 {
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
     HPX_TEST_EQ(t1.get_data(), 42);
     HPX_TEST_EQ(t1.get_base_data(), 7);
 
-    std::size_t N = 100;
-
     try
     {
         // migrate an object back and forth between 2 localities a couple of
@@ -487,7 +503,7 @@ bool test_migrate_polymorphic_component2(
             HPX_TEST_EQ(t1.get_base_data(), 7);
 
             // wait for migration to be done
-            HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+            HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
             // the migrated object should have the same id as before
             HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -517,15 +533,13 @@ bool test_migrate_lazy_polymorphic_component2(
     hpx::id_type source, hpx::id_type target)
 {
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
     HPX_TEST_EQ(t1.lazy_get_data(), 42);
     HPX_TEST_EQ(t1.lazy_get_base_data(), 7);
 
-    std::size_t N = 100;
-
     try
     {
         // migrate an object back and forth between 2 localities a couple of
@@ -539,7 +553,7 @@ bool test_migrate_lazy_polymorphic_component2(
             HPX_TEST_EQ(t1.lazy_get_base_data(), 7);
 
             // wait for migration to be done
-            HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+            HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
             // the migrated object should have the same id as before
             HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -570,19 +584,17 @@ bool test_migrate_busy_polymorphic_component2(
     hpx::id_type source, hpx::id_type target)
 {
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
     HPX_TEST_EQ(t1.get_data(), 42);
     HPX_TEST_EQ(t1.get_base_data(), 7);
 
-    std::size_t N = 100;
-
     // First, spawn a thread (locally) that will migrate the object between
     // source and target a few times
-    hpx::future<void> migrate_future = hpx::async([source, target, t1,
-                                                      N]() mutable {
+    hpx::future<void> migrate_future = hpx::async([source, target,
+                                                      t1]() mutable {
         for (std::size_t i = 0; i < N; ++i)
         {
             // migrate t1 to the target (loc2)
@@ -592,7 +604,7 @@ bool test_migrate_busy_polymorphic_component2(
             HPX_TEST_EQ(t1.get_base_data(), 7);
 
             // wait for migration to be done
-            HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+            HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
             // the migrated object should have the same id as before
             HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -612,7 +624,7 @@ bool test_migrate_busy_polymorphic_component2(
 
     // Second, we generate tons of work which should automatically follow
     // the migration.
-    hpx::future<void> create_work = hpx::async([t1, N]() {
+    hpx::future<void> create_work = hpx::async([t1]() {
         for (std::size_t i = 0; i < 2 * N; ++i)
         {
             hpx::cout << hpx::naming::get_locality_id_from_id(t1.call())
@@ -643,19 +655,17 @@ bool test_migrate_lazy_busy_polymorphic_component2(
     hpx::id_type source, hpx::id_type target)
 {
     test_client t1(hpx::new_<test_server>(source, 7, 42));
-    HPX_TEST_NEQ(hpx::naming::invalid_id, t1.get_id());
+    HPX_TEST_NEQ(hpx::invalid_id, t1.get_id());
 
     // the new object should live on the source locality
     HPX_TEST_EQ(t1.call(), source);
     HPX_TEST_EQ(t1.get_data(), 42);
     HPX_TEST_EQ(t1.get_base_data(), 7);
 
-    std::size_t N = 100;
-
     // First, spawn a thread (locally) that will migrate the object between
     // source and target a few times
-    hpx::future<void> migrate_future = hpx::async([source, target, t1,
-                                                      N]() mutable {
+    hpx::future<void> migrate_future = hpx::async([source, target,
+                                                      t1]() mutable {
         for (std::size_t i = 0; i < N; ++i)
         {
             // migrate t1 to the target (loc2)
@@ -665,7 +675,7 @@ bool test_migrate_lazy_busy_polymorphic_component2(
             HPX_TEST_EQ(t1.lazy_get_base_data(), 7);
 
             // wait for migration to be done
-            HPX_TEST_NEQ(hpx::naming::invalid_id, t2.get_id());
+            HPX_TEST_NEQ(hpx::invalid_id, t2.get_id());
 
             // the migrated object should have the same id as before
             HPX_TEST_EQ(t1.get_id(), t2.get_id());
@@ -685,7 +695,7 @@ bool test_migrate_lazy_busy_polymorphic_component2(
 
     // Second, we generate tons of work which should automatically follow
     // the migration.
-    hpx::future<void> create_work = hpx::async([t1, N]() {
+    hpx::future<void> create_work = hpx::async([t1]() {
         for (std::size_t i = 0; i < 2 * N; ++i)
         {
             hpx::cout << hpx::naming::get_locality_id_from_id(t1.call())
@@ -715,7 +725,7 @@ bool test_migrate_lazy_busy_polymorphic_component2(
 ///////////////////////////////////////////////////////////////////////////////
 int main()
 {
-    std::vector<hpx::id_type> localities = hpx::find_remote_localities();
+    std::vector<hpx::id_type> const localities = hpx::find_remote_localities();
 
     for (hpx::id_type const& id : localities)
     {
