@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2020 Hartmut Kaiser
+//  Copyright (c) 2007-2023 Hartmut Kaiser
 //  Copyright (c)      2011 Bryce Lelbach
 //
 //  SPDX-License-Identifier: BSL-1.0
@@ -10,15 +10,17 @@
 #include <hpx/config.hpp>
 #include <hpx/agas/addressing_service.hpp>
 #include <hpx/components_base/generate_unique_ids.hpp>
-#include <hpx/io_service/io_service_pool.hpp>
+#include <hpx/io_service/io_service_pool_fwd.hpp>
+#include <hpx/parcelset/message_handler_fwd.hpp>
+#include <hpx/parcelset/parcelhandler.hpp>
+#include <hpx/parcelset_base/locality.hpp>
+#include <hpx/parcelset_base/parcelport.hpp>
 #include <hpx/performance_counters/query_counters.hpp>
 #include <hpx/performance_counters/registry.hpp>
-#include <hpx/runtime/parcelset/locality.hpp>
-#include <hpx/runtime/parcelset/parcelhandler.hpp>
-#include <hpx/runtime/parcelset/parcelport.hpp>
 #include <hpx/runtime_components/server/console_error_sink_singleton.hpp>
 #include <hpx/runtime_distributed/applier.hpp>
 #include <hpx/runtime_distributed/find_localities.hpp>
+#include <hpx/runtime_distributed/runtime_fwd.hpp>
 #include <hpx/runtime_distributed/server/runtime_support.hpp>
 #include <hpx/runtime_local/runtime_local.hpp>
 #include <hpx/threading_base/callback_notifier.hpp>
@@ -37,15 +39,6 @@
 #include <hpx/config/warnings_prefix.hpp>
 
 namespace hpx {
-    // Returns if HPX continues past connection signals caused by crashed nodes
-    HPX_EXPORT bool tolerate_node_faults();
-
-    namespace detail {
-
-        // Get access to the registry of registered message handlers
-        HPX_EXPORT std::vector<hpx::tuple<char const*, char const*>>&
-        get_message_handler_registrations();
-    }    // namespace detail
 
     /// The \a runtime class encapsulates the HPX runtime system in a simple to
     /// use way. It makes sure all required parts of the HPX runtime system are
@@ -55,10 +48,14 @@ namespace hpx {
     public:
         /// Construct a new HPX runtime instance
         ///
-        /// \param locality_mode  [in] This is the mode the given runtime
-        ///                       instance should be executed in.
+        /// \param rtcfg        Runtime configuration for this instance
+        /// \param pre_main     Function to be called before running the
+        ///                     main action of this instance
+        /// \param post_main    Function to be called after running the
+        ///                     main action of this instance
         explicit runtime_distributed(util::runtime_configuration& rtcfg,
-            int (*pre_main)(runtime_mode) = nullptr);
+            int (*pre_main)(runtime_mode) = nullptr,
+            void (*post_main)() = nullptr);
 
         /// \brief The destructor makes sure all HPX runtime services are
         ///        properly shut down before exiting.
@@ -82,7 +79,7 @@ namespace hpx {
         ///                   return the value as returned as the result of the
         ///                   invocation of the function object given by the
         ///                   parameter \p func. Otherwise it will return zero.
-        int start(util::function_nonser<hpx_main_function_type> const& func,
+        int start(hpx::function<hpx_main_function_type> const& func,
             bool blocking = false) override;
 
         ///  \brief Start the runtime system
@@ -128,6 +125,9 @@ namespace hpx {
         ///                   return immediately. Use a second call to stop
         ///                   with this parameter set to \a true to wait for
         ///                   all internal work to be completed.
+        /// \param cond       Condition used to update all thread when done
+        /// \param mtx        Mutex used by this function to sync all threads
+
         void stop_helper(
             bool blocking, std::condition_variable& cond, std::mutex& mtx);
 
@@ -185,8 +185,7 @@ namespace hpx {
         /// \returns          This function will return the value as returned
         ///                   as the result of the invocation of the function
         ///                   object given by the parameter \p func.
-        int run(
-            util::function_nonser<hpx_main_function_type> const& func) override;
+        int run(hpx::function<hpx_main_function_type> const& func) override;
 
         /// \brief Run the HPX runtime system, initially use the given number
         ///        of (OS) threads in the thread-manager and block waiting for
@@ -203,7 +202,7 @@ namespace hpx {
             F&& sink)
         {
             return components::server::get_error_dispatcher().set_error_sink(
-                std::forward<F>(sink));
+                HPX_FORWARD(F, sink));
         }
 
         /// \brief Allow access to the registry counter registry instance used
@@ -216,22 +215,23 @@ namespace hpx {
 
         /// \brief Install all performance counters related to this runtime
         ///        instance
-        void register_counter_types();
+        static void register_counter_types();
 
         ///////////////////////////////////////////////////////////////////////
         // management API for active performance counters
         void register_query_counters(
             std::shared_ptr<util::query_counters> const& active_counters);
 
-        void start_active_counters(error_code& ec = throws);
-        void stop_active_counters(error_code& ec = throws);
-        void reset_active_counters(error_code& ec = throws);
-        void reinit_active_counters(bool reset = true, error_code& ec = throws);
+        void start_active_counters(error_code& ec = throws) const;
+        void stop_active_counters(error_code& ec = throws) const;
+        void reset_active_counters(error_code& ec = throws) const;
+        void reinit_active_counters(
+            bool reset = true, error_code& ec = throws) const;
         void evaluate_active_counters(bool reset = false,
-            char const* description = nullptr, error_code& ec = throws);
+            char const* description = nullptr, error_code& ec = throws) const;
 
         // stop periodic evaluation of counters during shutdown
-        void stop_evaluating_counters(bool terminate = false);
+        void stop_evaluating_counters(bool terminate = false) const;
 
         ///////////////////////////////////////////////////////////////////////
         /// \brief Allow access to the AGAS client instance used by the HPX
@@ -266,7 +266,7 @@ namespace hpx {
         /// \brief Returns a string of the locality endpoints (usable in debug output)
         std::string here() const override;
 
-        std::uint64_t get_runtime_support_lva() const;
+        naming::address_type get_runtime_support_lva() const;
 
         naming::gid_type get_next_id(std::size_t count = 1);
 
@@ -275,14 +275,15 @@ namespace hpx {
 
 #if defined(HPX_HAVE_NETWORKING)
         void register_message_handler(char const* message_handler_type,
-            char const* action, error_code& ec = throws);
+            char const* action, error_code& ec = throws) const;
         parcelset::policies::message_handler* create_message_handler(
             char const* message_handler_type, char const* action,
             parcelset::parcelport* pp, std::size_t num_messages,
-            std::size_t interval, error_code& ec = throws);
+            std::size_t interval, error_code& ec = throws) const;
         serialization::binary_filter* create_binary_filter(
             char const* binary_filter_type, bool compress,
-            serialization::binary_filter* next_filter, error_code& ec = throws);
+            serialization::binary_filter* next_filter,
+            error_code& ec = throws) const;
 #endif
 
         /// Initialize AGAS operation
@@ -357,14 +358,14 @@ namespace hpx {
 
         std::uint32_t get_initial_num_localities() const override;
 
-        lcos::future<std::uint32_t> get_num_localities() const override;
+        hpx::future<std::uint32_t> get_num_localities() const override;
 
         std::string get_locality_name() const override;
 
         std::uint32_t get_num_localities(hpx::launch::sync_policy,
             components::component_type type, error_code& ec) const;
 
-        lcos::future<std::uint32_t> get_num_localities(
+        hpx::future<std::uint32_t> get_num_localities(
             components::component_type type) const;
 
         std::uint32_t assign_cores(std::string const& locality_basename,
@@ -373,17 +374,13 @@ namespace hpx {
         std::uint32_t assign_cores() override;
 
     private:
-        // avoid warnings about usage of this in member initializer list
-        runtime_distributed* This()
-        {
-            return this;
-        }
-
-        //
         threads::thread_result_type run_helper(
-            util::function_nonser<runtime::hpx_main_function_type> const& func,
+            hpx::function<runtime::hpx_main_function_type> const& func,
             int& result);
 
+        void init_global_data();
+        void deinit_global_data();
+
         void wait_helper(
             std::mutex& mtx, std::condition_variable& cond, bool& running);
 
@@ -392,12 +389,12 @@ namespace hpx {
             std::size_t global_thread_num, char const* pool_name,
             char const* postfix, bool service_thread);
 
-        void deinit_tss_helper(char const* context, std::size_t num);
+        void deinit_tss_helper(char const* context, std::size_t num) const;
 
         void init_tss_ex(std::string const& locality, char const* context,
             runtime_local::os_thread_type type, std::size_t local_thread_num,
             std::size_t global_thread_num, char const* pool_name,
-            char const* postfix, bool service_thread, error_code& ec);
+            char const* postfix, bool service_thread, error_code& ec) const;
 
         static void default_errorsink(std::string const&);
 
@@ -418,6 +415,7 @@ namespace hpx {
         std::shared_ptr<util::query_counters> active_counters_;
 
         int (*pre_main_)(runtime_mode);
+        void (*post_main_)();
     };
 }    // namespace hpx
 
