@@ -21,13 +21,13 @@
 #include <hpx/modules/errors.hpp>
 #include <hpx/modules/plugin.hpp>
 #include <hpx/modules/program_options.hpp>
+#include <hpx/parcelset_base/locality.hpp>
 #include <hpx/performance_counters/counters.hpp>
-#include <hpx/plugins/plugin_factory_base.hpp>
-#include <hpx/runtime/parcelset/locality.hpp>
+#include <hpx/plugin_factories/plugin_factory_base.hpp>
 #include <hpx/runtime_components/components_fwd.hpp>
 #include <hpx/runtime_configuration/static_factory_data.hpp>
 #include <hpx/runtime_distributed/find_here.hpp>
-#include <hpx/synchronization/condition_variable.hpp>
+#include <hpx/synchronization/latch.hpp>
 #include <hpx/synchronization/mutex.hpp>
 #include <hpx/synchronization/spinlock.hpp>
 
@@ -48,13 +48,13 @@
 
 #include <hpx/config/warnings_prefix.hpp>
 
-namespace hpx { namespace components { namespace server {
+namespace hpx::components::server {
 
     ///////////////////////////////////////////////////////////////////////////
     class runtime_support
     {
     private:
-        typedef lcos::local::spinlock plugin_map_mutex_type;
+        using plugin_map_mutex_type = hpx::spinlock;
 
         struct plugin_factory
         {
@@ -71,14 +71,14 @@ namespace hpx { namespace components { namespace server {
             hpx::util::plugin::dll const& second;
             bool isenabled;
         };
-        typedef plugin_factory plugin_factory_type;
-        typedef std::map<std::string, plugin_factory_type> plugin_map_type;
+        using plugin_factory_type = plugin_factory;
+        using plugin_map_type = std::map<std::string, plugin_factory_type>;
 
-        typedef std::map<std::string, hpx::util::plugin::dll> modules_map_type;
-        typedef std::vector<static_factory_load_data_type> static_modules_type;
+        using modules_map_type = std::map<std::string, hpx::util::plugin::dll>;
+        using static_modules_type = std::vector<static_factory_load_data_type>;
 
     public:
-        typedef runtime_support type_holder;
+        using type_holder = runtime_support;
 
         static component_type get_component_type()
         {
@@ -99,10 +99,6 @@ namespace hpx { namespace components { namespace server {
 
         /// \brief finalize() will be called just before the instance gets
         ///        destructed
-        ///
-        /// \param self [in] The HPX \a thread used to execute this function.
-        /// \param appl [in] The applier to be used for finalization of the
-        ///             component instance.
         static constexpr void finalize() {}
 
         void delete_function_lists();
@@ -110,7 +106,7 @@ namespace hpx { namespace components { namespace server {
 
         // This component type requires valid locality id for its actions to
         // be invoked
-        static bool is_target_valid(naming::id_type const& id)
+        static bool is_target_valid(hpx::id_type const& id)
         {
             return naming::is_locality(id);
         }
@@ -125,12 +121,13 @@ namespace hpx { namespace components { namespace server {
         template <typename Component, typename T, typename... Ts>
         naming::gid_type create_component(T v, Ts... vs);
 
-        template <typename Component>
-        std::vector<naming::gid_type> bulk_create_component(std::size_t count);
-
-        template <typename Component, typename T, typename... Ts>
+        template <typename Component, typename... Ts>
         std::vector<naming::gid_type> bulk_create_component(
-            std::size_t count, T v, Ts... vs);
+            std::size_t count, Ts... vs);
+
+        template <typename Component, typename... Ts>
+        std::vector<naming::gid_type> bulk_create_component_with_count(
+            std::size_t count, Ts... vs);
 
         template <typename Component>
         naming::gid_type copy_create_component(
@@ -138,24 +135,24 @@ namespace hpx { namespace components { namespace server {
 
         template <typename Component>
         naming::gid_type migrate_component_to_here(
-            std::shared_ptr<Component> const& p, naming::id_type);
+            std::shared_ptr<Component> const& p, hpx::id_type);
 
         /// \brief Gracefully shutdown this runtime system instance
-        void shutdown(double timeout, naming::id_type const& respond_to);
+        void shutdown(double timeout, hpx::id_type const& respond_to);
 
         /// \brief Gracefully shutdown runtime system instances on all localities
         void shutdown_all(double timeout);
 
         /// \brief Shutdown this runtime system instance
-        HPX_NORETURN void terminate(naming::id_type const& respond_to);
+        [[noreturn]] void terminate(hpx::id_type const& respond_to);
 
-        void terminate_act(naming::id_type const& id)
+        void terminate_act(hpx::id_type const& id)
         {
             terminate(id);
         }
 
         /// \brief Shutdown runtime system instances on all localities
-        HPX_NORETURN void terminate_all();
+        [[noreturn]] void terminate_all();
 
         void terminate_all_act()
         {
@@ -189,33 +186,32 @@ namespace hpx { namespace components { namespace server {
 #endif
 
         ///////////////////////////////////////////////////////////////////////
-        // Each of the exposed functions needs to be encapsulated into a action
+        // Each of the exposed functions needs to be encapsulated into an action
         // type, allowing to generate all require boilerplate code for threads,
         // serialization, etc.
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, load_components);
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, call_startup_functions);
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, call_shutdown_functions);
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, shutdown);
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, shutdown_all);
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, load_components)
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, call_startup_functions)
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, call_shutdown_functions)
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, shutdown)
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, shutdown_all)
         HPX_DEFINE_COMPONENT_ACTION(
-            runtime_support, terminate_act, terminate_action);
+            runtime_support, terminate_act, terminate_action)
         HPX_DEFINE_COMPONENT_ACTION(
-            runtime_support, terminate_all_act, terminate_all_action);
+            runtime_support, terminate_all_act, terminate_all_action)
 
         // even if this is not a short/minimal action, we still execute it
         // directly to avoid a deadlock condition inside the thread manager
         // waiting for this thread to finish, which waits for the thread
         // manager to exit
-        HPX_DEFINE_COMPONENT_DIRECT_ACTION(runtime_support, get_config);
+        HPX_DEFINE_COMPONENT_DIRECT_ACTION(runtime_support, get_config)
 
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, garbage_collect);
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, garbage_collect)
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, create_performance_counter)
         HPX_DEFINE_COMPONENT_ACTION(
-            runtime_support, create_performance_counter);
-        HPX_DEFINE_COMPONENT_ACTION(
-            runtime_support, remove_from_connection_cache);
+            runtime_support, remove_from_connection_cache)
 
 #if defined(HPX_HAVE_NETWORKING)
-        HPX_DEFINE_COMPONENT_ACTION(runtime_support, dijkstra_termination);
+        HPX_DEFINE_COMPONENT_ACTION(runtime_support, dijkstra_termination)
 #endif
 
         ///////////////////////////////////////////////////////////////////////
@@ -235,7 +231,7 @@ namespace hpx { namespace components { namespace server {
         ///        be properly stopped.
         ///
         /// \note      This function can be called from any thread.
-        void stop(double timeout, naming::id_type const& respond_to,
+        void stop(double timeout, hpx::id_type const& respond_to,
             bool remove_from_remote_caches);
 
         /// called locally only
@@ -330,7 +326,7 @@ namespace hpx { namespace components { namespace server {
 
         // the name says it all
         std::size_t dijkstra_termination_detection(
-            std::vector<naming::id_type> const& locality_ids);
+            std::vector<hpx::id_type> const& locality_ids);
 
 #if defined(HPX_HAVE_NETWORKING)
         void send_dijkstra_termination_token(std::uint32_t target_locality_id,
@@ -349,10 +345,10 @@ namespace hpx { namespace components { namespace server {
         std::atomic<bool> shutdown_all_invoked_;
 
 #if defined(HPX_HAVE_NETWORKING)
-        typedef hpx::lcos::local::spinlock dijkstra_mtx_type;
+        using dijkstra_mtx_type = hpx::spinlock;
         dijkstra_mtx_type dijkstra_mtx_;
-        lcos::local::condition_variable_any dijkstra_cond_;
-        bool dijkstra_color_;    // false: white, true: black
+        std::unique_ptr<hpx::latch> dijkstra_cond_;
+        std::atomic<bool> dijkstra_color_;    // false: white, true: black
 #endif
 
         plugin_map_mutex_type p_mtx_;
@@ -361,7 +357,7 @@ namespace hpx { namespace components { namespace server {
         modules_map_type& modules_;
         static_modules_type static_modules_;
 
-        lcos::local::spinlock globals_mtx_;
+        hpx::spinlock globals_mtx_;
         std::list<startup_function_type> pre_startup_functions_;
         std::list<startup_function_type> startup_functions_;
         std::list<shutdown_function_type> pre_shutdown_functions_;
@@ -391,7 +387,7 @@ namespace hpx { namespace components { namespace server {
         components::component_type const type =
             components::get_component_type<typename Component::wrapped_type>();
 
-        typedef typename Component::wrapping_type wrapping_type;
+        using wrapping_type = typename Component::wrapping_type;
         naming::gid_type id = create<wrapping_type>();
         LRT_(info).format("successfully created component {} of type: {}", id,
             components::get_component_type_name(type));
@@ -405,12 +401,12 @@ namespace hpx { namespace components { namespace server {
         components::component_type const type =
             components::get_component_type<typename Component::wrapped_type>();
 
-        typedef typename Component::wrapping_type wrapping_type;
+        using wrapping_type = typename Component::wrapping_type;
         // Note, T and Ts can't be (non-const) references, and parameters
         // should be moved to allow for move-only constructor argument
         // types.
         naming::gid_type id =
-            create<wrapping_type>(std::move(v), std::move(vs)...);
+            create<wrapping_type>(HPX_MOVE(v), HPX_MOVE(vs)...);
 
         LRT_(info).format("successfully created component {} of type: {}", id,
             components::get_component_type_name(type));
@@ -419,9 +415,9 @@ namespace hpx { namespace components { namespace server {
     }
 #endif
 
-    template <typename Component>
+    template <typename Component, typename... Ts>
     std::vector<naming::gid_type> runtime_support::bulk_create_component(
-        std::size_t count)
+        std::size_t count, Ts... vs)
     {
         components::component_type const type =
             components::get_component_type<typename Component::wrapped_type>();
@@ -429,10 +425,10 @@ namespace hpx { namespace components { namespace server {
         std::vector<naming::gid_type> ids;
         ids.reserve(count);
 
-        typedef typename Component::wrapping_type wrapping_type;
+        using wrapping_type = typename Component::wrapping_type;
         for (std::size_t i = 0; i != count; ++i)
         {
-            ids.push_back(create<wrapping_type>());
+            ids.push_back(create<wrapping_type>(vs...));
         }
 
         LRT_(info).format("successfully created {} component(s) of type: {}",
@@ -441,26 +437,40 @@ namespace hpx { namespace components { namespace server {
         return ids;
     }
 
-    template <typename Component, typename T, typename... Ts>
-    std::vector<naming::gid_type> runtime_support::bulk_create_component(
-        std::size_t count, T v, Ts... vs)
-    {
-        components::component_type const type =
-            components::get_component_type<typename Component::wrapped_type>();
+    namespace detail {
 
-        std::vector<naming::gid_type> ids;
-        ids.reserve(count);
-
-        typedef typename Component::wrapping_type wrapping_type;
-        for (std::size_t i = 0; i != count; ++i)
+        template <typename Component, typename... Ts>
+        std::vector<naming::gid_type> bulk_create_component_with_count_helper(
+            std::size_t count, std::size_t first, Ts... vs)
         {
-            ids.push_back(create<wrapping_type>(v, vs...));
+            components::component_type const type =
+                components::get_component_type<
+                    typename Component::wrapped_type>();
+
+            std::vector<naming::gid_type> ids;
+            ids.reserve(count);
+
+            using wrapping_type = typename Component::wrapping_type;
+            for (std::size_t i = 0; i != count; ++i)
+            {
+                ids.push_back(create<wrapping_type>(first + i, vs...));
+            }
+
+            LRT_(info).format(
+                "successfully created {} component(s) of type: {}", count,
+                components::get_component_type_name(type));
+
+            return ids;
         }
+    }    // namespace detail
 
-        LRT_(info).format("successfully created {} component(s) of type: {}",
-            count, components::get_component_type_name(type));
-
-        return ids;
+    template <typename Component, typename... Ts>
+    std::vector<naming::gid_type>
+    runtime_support::bulk_create_component_with_count(
+        std::size_t count, Ts... vs)
+    {
+        return detail::bulk_create_component_with_count_helper<Component>(
+            count, HPX_MOVE(vs)...);
     }
 
     template <typename Component>
@@ -470,12 +480,12 @@ namespace hpx { namespace components { namespace server {
         components::component_type const type =
             components::get_component_type<typename Component::wrapped_type>();
 
-        typedef typename Component::wrapping_type wrapping_type;
+        using wrapping_type = typename Component::wrapping_type;
         naming::gid_type id;
 
         if (!local_op)
         {
-            id = create<wrapping_type>(std::move(*p));
+            id = create<wrapping_type>(HPX_MOVE(*p));
         }
         else
         {
@@ -491,7 +501,7 @@ namespace hpx { namespace components { namespace server {
     ///////////////////////////////////////////////////////////////////////////
     template <typename Component>
     naming::gid_type runtime_support::migrate_component_to_here(
-        std::shared_ptr<Component> const& p, naming::id_type to_migrate)
+        std::shared_ptr<Component> const& p, hpx::id_type to_migrate)
     {
         components::component_type const type =
             components::get_component_type<typename Component::wrapped_type>();
@@ -500,49 +510,47 @@ namespace hpx { namespace components { namespace server {
         // AGAS
         naming::gid_type migrated_id = to_migrate.get_gid();
 
-        typedef typename Component::wrapping_type wrapping_type;
+        using wrapping_type = typename Component::wrapping_type;
         typename wrapping_type::derived_type* new_instance = nullptr;
 
-        naming::gid_type id = create_migrated<wrapping_type>(migrated_id,
-            reinterpret_cast<void**>(&new_instance), std::move(*p));
+        naming::gid_type id = create_migrated<wrapping_type>(
+            migrated_id, reinterpret_cast<void**>(&new_instance), HPX_MOVE(*p));
 
         // sanity checks
         if (!id || new_instance == nullptr)
         {
             // we should not get here (id should not be invalid)
-            HPX_THROW_EXCEPTION(hpx::invalid_status,
+            HPX_THROW_EXCEPTION(hpx::error::invalid_status,
                 "runtime_support::migrate_component_to_here",
                 "could not create copy of given component");
-            return naming::invalid_gid;
         }
         if (id != migrated_id)
         {
             // we should not get here either (the ids should be the same)
-            HPX_THROW_EXCEPTION(hpx::invalid_status,
+            HPX_THROW_EXCEPTION(hpx::error::invalid_status,
                 "runtime_support::migrate_component_to_here",
                 "could not create copy of given component (the new id is "
                 "different from the original id)");
-            return naming::invalid_gid;
         }
 
         LRT_(info).format(
             "successfully migrated component {} of type: {} to locality: {}",
             id, components::get_component_type_name(type), find_here());
 
-        // inform the newly created component that it has been migrated
-        new_instance->on_migrated();
-
         // At this point the object has been fully migrated. We now remove
         // the object from the AGAS table of migrated objects. This is
         // necessary as this object might have been migrated off this locality
         // before it was migrated back.
-        agas::unmark_as_migrated(id);
+        agas::unmark_as_migrated(id, [&] {
+            // inform the newly created component that it has been migrated
+            new_instance->on_migrated();
+        });
 
         to_migrate.make_unmanaged();
 
         return id;
     }
-}}}    // namespace hpx::components::server
+}    // namespace hpx::components::server
 
 #include <hpx/config/warnings_suffix.hpp>
 
@@ -601,7 +609,8 @@ HPX_REGISTER_ACTION_DECLARATION(
     dijkstra_termination_action)
 #endif
 
-namespace hpx { namespace components { namespace server {
+namespace hpx::components::server {
+
     template <typename Component, typename... Ts>
     struct create_component_action
       : ::hpx::actions::action<naming::gid_type (runtime_support::*)(Ts...),
@@ -636,40 +645,43 @@ namespace hpx { namespace components { namespace server {
     };
 
     ///////////////////////////////////////////////////////////////////////////
-    template <typename Component, typename... Ts>
+    template <bool WithCount, typename Component, typename... Ts>
     struct bulk_create_component_action
       : ::hpx::actions::action<std::vector<naming::gid_type> (
                                    runtime_support::*)(std::size_t, Ts...),
             &runtime_support::bulk_create_component<Component, Ts...>,
-            bulk_create_component_action<Component, Ts...>>
-    {
-    };
-
-    template <typename Component>
-    struct bulk_create_component_action<Component>
-      : ::hpx::actions::action<std::vector<naming::gid_type> (
-                                   runtime_support::*)(std::size_t),
-            &runtime_support::bulk_create_component<Component>,
-            bulk_create_component_action<Component>>
+            bulk_create_component_action<WithCount, Component, Ts...>>
     {
     };
 
     template <typename Component, typename... Ts>
+    struct bulk_create_component_action<true, Component, Ts...>
+      : ::hpx::actions::action<std::vector<naming::gid_type> (
+                                   runtime_support::*)(std::size_t, Ts...),
+            &runtime_support::bulk_create_component_with_count<Component,
+                Ts...>,
+            bulk_create_component_action<true, Component, Ts...>>
+    {
+    };
+
+    template <bool WithCount, typename Component, typename... Ts>
     struct bulk_create_component_direct_action
       : ::hpx::actions::direct_action<std::vector<naming::gid_type> (
                                           runtime_support::*)(
                                           std::size_t, Ts...),
             &runtime_support::bulk_create_component<Component, Ts...>,
-            bulk_create_component_direct_action<Component, Ts...>>
+            bulk_create_component_direct_action<WithCount, Component, Ts...>>
     {
     };
 
-    template <typename Component>
-    struct bulk_create_component_direct_action<Component>
+    template <typename Component, typename... Ts>
+    struct bulk_create_component_direct_action<true, Component, Ts...>
       : ::hpx::actions::direct_action<std::vector<naming::gid_type> (
-                                          runtime_support::*)(std::size_t),
-            &runtime_support::bulk_create_component<Component>,
-            bulk_create_component_direct_action<Component>>
+                                          runtime_support::*)(
+                                          std::size_t, Ts...),
+            &runtime_support::bulk_create_component_with_count<Component,
+                Ts...>,
+            bulk_create_component_direct_action<true, Component, Ts...>>
     {
     };
 
@@ -686,31 +698,30 @@ namespace hpx { namespace components { namespace server {
     struct migrate_component_here_action
       : ::hpx::actions::action<naming::gid_type (runtime_support::*)(
                                    std::shared_ptr<Component> const&,
-                                   naming::id_type),
+                                   hpx::id_type),
             &runtime_support::migrate_component_to_here<Component>,
             migrate_component_here_action<Component>>
     {
     };
-}}}    // namespace hpx::components::server
+}    // namespace hpx::components::server
 
-namespace hpx { namespace traits {
-    ///////////////////////////////////////////////////////////////////////////
-    // Termination detection does not make this locality black
+///////////////////////////////////////////////////////////////////////////
+// Termination detection does not make this locality black
 #if !defined(HPX_COMPUTE_DEVICE_CODE) && defined(HPX_HAVE_NETWORKING)
-    template <>
-    struct action_does_termination_detection<
-        hpx::components::server::runtime_support::dijkstra_termination_action>
+template <>
+struct hpx::traits::action_does_termination_detection<
+    hpx::components::server::runtime_support::dijkstra_termination_action>
+{
+    static constexpr bool call() noexcept
     {
-        static bool call()
-        {
-            return true;
-        }
-    };
+        return true;
+    }
+};
 #endif
 
-    // runtime_support is a (hand-rolled) component
-    template <>
-    struct is_component<components::server::runtime_support> : std::true_type
-    {
-    };
-}}    // namespace hpx::traits
+// runtime_support is a (hand-rolled) component
+template <>
+struct hpx::traits::is_component<hpx::components::server::runtime_support>
+  : std::true_type
+{
+};
