@@ -15,19 +15,20 @@
 #include <hpx/agas_base/detail/hosted_component_namespace.hpp>
 #include <hpx/agas_base/detail/hosted_locality_namespace.hpp>
 #include <hpx/assert.hpp>
+#include <hpx/async_distributed/put_parcel.hpp>
 #include <hpx/components_base/agas_interface.hpp>
 #include <hpx/components_base/server/managed_component_base.hpp>
 #include <hpx/execution_base/this_thread.hpp>
 #include <hpx/functional/bind_front.hpp>
 #include <hpx/modules/agas_base.hpp>
 #include <hpx/modules/format.hpp>
-#include <hpx/runtime/parcelset/detail/parcel_await.hpp>
-#include <hpx/runtime/parcelset/parcel.hpp>
-#include <hpx/runtime/parcelset/parcelport.hpp>
-#include <hpx/runtime/parcelset/put_parcel.hpp>
+#include <hpx/parcelset/detail/parcel_await.hpp>
+#include <hpx/parcelset_base/parcel_interface.hpp>
+#include <hpx/parcelset_base/parcelport.hpp>
 #include <hpx/runtime_configuration/runtime_configuration.hpp>
 #include <hpx/runtime_distributed.hpp>
 #include <hpx/runtime_distributed/big_boot_barrier.hpp>
+#include <hpx/runtime_distributed/runtime_fwd.hpp>
 #include <hpx/serialization/detail/polymorphic_id_factory.hpp>
 #include <hpx/serialization/vector.hpp>
 #include <hpx/static_reinit/reinitializable_static.hpp>
@@ -47,18 +48,18 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace detail {
+namespace hpx::detail {
 
     std::string get_locality_base_name();
-}}    // namespace hpx::detail
+}
 
-namespace hpx { namespace parcelset {
+namespace hpx::parcelset {
 
     // shortcut for get_runtime().get_parcel_handler()
     parcelhandler& get_parcel_handler();
-}}    // namespace hpx::parcelset
+}    // namespace hpx::parcelset
 
-namespace hpx { namespace agas { namespace detail {
+namespace hpx::agas::detail {
 
     void register_unassigned_typenames()
     {
@@ -228,9 +229,10 @@ namespace hpx { namespace agas { namespace detail {
         std::vector<std::uint32_t> serialization_ids;
         std::vector<std::uint32_t> action_ids;
     };
-}}}    // namespace hpx::agas::detail
+}    // namespace hpx::agas::detail
+
+namespace hpx::agas {
 
-namespace hpx { namespace agas {
     template <typename Action, typename... Args>
     void big_boot_barrier::apply(std::uint32_t source_locality_id,
         std::uint32_t target_locality_id, parcelset::locality dest, Action act,
@@ -241,7 +243,7 @@ namespace hpx { namespace agas {
             naming::get_gid_from_locality_id(target_locality_id));
         parcelset::parcel p(parcelset::detail::create_parcel::call(
             naming::get_gid_from_locality_id(target_locality_id),
-            std::move(addr), act, std::forward<Args>(args)...));
+            HPX_MOVE(addr), act, HPX_FORWARD(Args, args)...));
 #if defined(HPX_HAVE_PARCEL_PROFILING)
         if (!p.parcel_id())
         {
@@ -252,11 +254,11 @@ namespace hpx { namespace agas {
         HPX_UNUSED(source_locality_id);
 #endif
 
-        parcelset::detail::parcel_await_apply(std::move(p),
+        parcelset::detail::parcel_await_apply(HPX_MOVE(p),
             parcelset::write_handler_type(), 0,
             [this, dest](
                 parcelset::parcel&& p, parcelset::write_handler_type&&) {
-                pp->send_early_parcel(dest, std::move(p));
+                pp->send_early_parcel(dest, HPX_MOVE(p));
             });
     }    // }}}
 
@@ -270,10 +272,10 @@ namespace hpx { namespace agas {
         naming::address addr(
             naming::get_gid_from_locality_id(target_locality_id));
 
-        parcelset::put_parcel(naming::id_type(naming::get_gid_from_locality_id(
-                                                  target_locality_id),
-                                  naming::id_type::unmanaged),
-            std::move(addr), act, std::forward<Args>(args)...);
+        parcelset::put_parcel(
+            hpx::id_type(naming::get_gid_from_locality_id(target_locality_id),
+                hpx::id_type::management_type::unmanaged),
+            HPX_MOVE(addr), act, HPX_FORWARD(Args, args)...);
     }    // }}}
 
     //typedef components::detail::heap_factory<
@@ -294,8 +296,8 @@ namespace hpx { namespace agas {
     struct registration_header
     {
         registration_header()
-          : primary_ns_ptr(0)
-          , symbol_ns_ptr(0)
+          : primary_ns_ptr(nullptr)
+          , symbol_ns_ptr(nullptr)
           , cores_needed(0)
           , num_threads(0)
         {
@@ -303,9 +305,9 @@ namespace hpx { namespace agas {
 
         // TODO: pass head address as a GVA
         registration_header(parcelset::endpoints_type const& endpoints_,
-            std::uint64_t primary_ns_ptr_, std::uint64_t symbol_ns_ptr_,
-            std::uint32_t cores_needed_, std::uint32_t num_threads_,
-            std::string const& hostname_,
+            naming::address_type primary_ns_ptr_,
+            naming::address_type symbol_ns_ptr_, std::uint32_t cores_needed_,
+            std::uint32_t num_threads_, std::string const& hostname_,
             detail::unassigned_typename_sequence const& typenames_,
             naming::gid_type prefix_ = naming::gid_type())
           : endpoints(endpoints_)
@@ -320,8 +322,8 @@ namespace hpx { namespace agas {
         }
 
         parcelset::endpoints_type endpoints;
-        std::uint64_t primary_ns_ptr;
-        std::uint64_t symbol_ns_ptr;
+        naming::address_type primary_ns_ptr;
+        naming::address_type symbol_ns_ptr;
         std::uint32_t cores_needed;
         std::uint32_t num_threads;
         std::string hostname;    // hostname of locality
@@ -331,14 +333,23 @@ namespace hpx { namespace agas {
         template <typename Archive>
         void serialize(Archive& ar, const unsigned int)
         {
-            ar& endpoints;
-            ar& primary_ns_ptr;
-            ar& symbol_ns_ptr;
-            ar& cores_needed;
-            ar& num_threads;
-            ar& hostname;
-            ar& typenames;
-            ar& prefix;
+            // clang-format off
+            ar & endpoints;
+
+            std::size_t address = reinterpret_cast<std::size_t>(primary_ns_ptr);
+            ar & address;
+            primary_ns_ptr = reinterpret_cast<naming::address_type>(address);
+
+            address = reinterpret_cast<std::size_t>(symbol_ns_ptr);
+            ar & address;
+            symbol_ns_ptr = reinterpret_cast<naming::address_type>(address);
+
+            ar & cores_needed;
+            ar & num_threads;
+            ar & hostname;
+            ar & typenames;
+            ar & prefix;
+            // clang-format on
         }
     };
 
@@ -389,17 +400,19 @@ namespace hpx { namespace agas {
         template <typename Archive>
         void serialize(Archive& ar, const unsigned int)
         {
-            ar& prefix;
-            ar& agas_locality;
-            ar& locality_ns_address;
-            ar& primary_ns_address;
-            ar& component_ns_address;
-            ar& symbol_ns_address;
-            ar& num_localities;
-            ar& used_cores;
-            ar& agas_endpoints;
-            ar& ids;
-            ar& endpoints;
+            // clang-format off
+            ar & prefix;
+            ar & agas_locality;
+            ar & locality_ns_address;
+            ar & primary_ns_address;
+            ar & component_ns_address;
+            ar & symbol_ns_address;
+            ar & num_localities;
+            ar & used_cores;
+            ar & agas_endpoints;
+            ar & ids;
+            ar & endpoints;
+            // clang-format on
         }
     };
 
@@ -409,29 +422,28 @@ namespace hpx { namespace agas {
     // }}}
 
     // {{{ early action types
-    typedef actions::direct_action<void (*)(registration_header const&),
-        register_worker>
-        register_worker_action;
+    using register_worker_action =
+        actions::direct_action<void (*)(registration_header const&),
+            register_worker>;
 
-    typedef actions::direct_action<void (*)(notification_header const&),
-        notify_worker>
-        notify_worker_action;
+    using notify_worker_action =
+        actions::direct_action<void (*)(notification_header const&),
+            notify_worker>;
     // }}}
-
-}}    // namespace hpx::agas
+}    // namespace hpx::agas
 
 using hpx::agas::notify_worker_action;
 using hpx::agas::register_worker_action;
 
-HPX_ACTION_HAS_CRITICAL_PRIORITY(register_worker_action);
-HPX_ACTION_HAS_CRITICAL_PRIORITY(notify_worker_action);
+HPX_ACTION_HAS_CRITICAL_PRIORITY(register_worker_action)
+HPX_ACTION_HAS_CRITICAL_PRIORITY(notify_worker_action)
 
 HPX_REGISTER_ACTION_ID(register_worker_action, register_worker_action,
     hpx::actions::register_worker_action_id)
 HPX_REGISTER_ACTION_ID(notify_worker_action, notify_worker_action,
     hpx::actions::notify_worker_action_id)
 
-namespace hpx { namespace agas {
+namespace hpx::agas {
 
     // remote call to AGAS
     void register_worker(registration_header const& header)
@@ -444,13 +456,15 @@ namespace hpx { namespace agas {
 
         if (HPX_UNLIKELY(agas_client.is_connecting()))
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "agas::register_worker",
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "agas::register_worker",
                 "a locality in connect mode cannot be an AGAS server.");
         }
 
         if (HPX_UNLIKELY(!agas_client.is_bootstrap()))
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "agas::register_worker",
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "agas::register_worker",
                 "registration parcel received by non-bootstrap locality.");
         }
 
@@ -458,7 +472,8 @@ namespace hpx { namespace agas {
         if (prefix != naming::invalid_gid &&
             naming::get_locality_id_from_gid(prefix) == 0)
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "agas::register_worker",
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "agas::register_worker",
                 "worker node ({}) can't suggest locality_id zero, "
                 "this is reserved for the console",
                 header.endpoints);
@@ -468,23 +483,23 @@ namespace hpx { namespace agas {
         if (!agas_client.register_locality(
                 header.endpoints, prefix, header.num_threads))
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "agas::register_worker",
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "agas::register_worker",
                 "attempt to register locality {} more than once",
                 header.endpoints);
-            return;
         }
 
         naming::address locality_addr(agas::get_locality(),
-            hpx::components::component_agas_locality_namespace,
+            to_int(components::component_enum_type::agas_locality_namespace),
             agas_client.locality_ns_->ptr());
         naming::address primary_addr(agas::get_locality(),
-            hpx::components::component_agas_primary_namespace,
+            to_int(components::component_enum_type::agas_primary_namespace),
             agas_client.primary_ns_.ptr());
         naming::address component_addr(agas::get_locality(),
-            hpx::components::component_agas_component_namespace,
+            to_int(components::component_enum_type::agas_component_namespace),
             agas_client.component_ns_->ptr());
         naming::address symbol_addr(agas::get_locality(),
-            hpx::components::component_agas_symbol_namespace,
+            to_int(components::component_enum_type::agas_symbol_namespace),
             agas_client.symbol_ns_.ptr());
 
         // assign cores to the new locality
@@ -519,13 +534,13 @@ namespace hpx { namespace agas {
 
         // TODO: Handle cases where localities try to connect to AGAS while it's
         // shutting down.
-        if (agas_client.get_status() != state_starting)
+        if (agas_client.get_status() != hpx::state::starting)
         {
             // We can just send the parcel now, the connecting locality isn't a part
             // of startup synchronization.
             get_big_boot_barrier().apply_late(0,
                 naming::get_locality_id_from_gid(prefix), dest,
-                notify_worker_action(), std::move(hdr));
+                notify_worker_action(), HPX_MOVE(hdr));
         }
 
         else
@@ -534,12 +549,12 @@ namespace hpx { namespace agas {
             // synchronization.
 
             // delay the final response until the runtime system is up and running
-            util::unique_function_nonser<void()>* thunk =
-                new util::unique_function_nonser<void()>(util::one_shot(
-                    util::bind_front(&big_boot_barrier::apply_notification,
+            hpx::move_only_function<void()>* thunk =
+                new hpx::move_only_function<void()>(util::one_shot(
+                    hpx::bind_front(&big_boot_barrier::apply_notification,
                         &get_big_boot_barrier(), 0,
                         naming::get_locality_id_from_gid(prefix), dest,
-                        std::move(hdr))));
+                        HPX_MOVE(hdr))));
             get_big_boot_barrier().add_thunk(thunk);
         }
     }
@@ -557,10 +572,11 @@ namespace hpx { namespace agas {
         runtime_distributed& rt = get_runtime_distributed();
         naming::resolver_client& agas_client = naming::get_agas_client();
 
-        if (HPX_UNLIKELY(agas_client.get_status() != state_starting))
+        if (HPX_UNLIKELY(agas_client.get_status() != hpx::state::starting))
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "agas::notify_worker",
-                "locality {} has launched early", rt.here());
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "agas::notify_worker", "locality {} has launched early",
+                rt.here());
         }
 
         util::runtime_configuration& cfg = rt.get_config();
@@ -612,7 +628,7 @@ namespace hpx { namespace agas {
     {
         hdr.endpoints = localities;
         apply(source_locality_id, target_locality_id, dest,
-            notify_worker_action(), std::move(hdr));
+            notify_worker_action(), HPX_MOVE(hdr));
     }
 
     void big_boot_barrier::add_locality_endpoints(std::uint32_t locality_id,
@@ -630,10 +646,11 @@ namespace hpx { namespace agas {
         std::unique_lock<std::mutex> lock(mtx);
         while (connected)
         {
-            cond.wait(lock);
+            cond.wait(lock);    //-V1089
         }
+
         // pre-cache all known locality endpoints in local AGAS on locality 0 as well
-        if (service_mode_bootstrap == service_type)
+        if (service_mode::bootstrap == service_type)
         {
             naming::resolver_client& agas_client = naming::get_agas_client();
             agas_client.pre_cache_endpoints(localities);
@@ -646,7 +663,7 @@ namespace hpx { namespace agas {
         service_mode service_type = ini.get_agas_service_mode();
         std::size_t result = 1;
 
-        if (service_mode_bootstrap == service_type)
+        if (service_mode::bootstrap == service_type)
         {
             std::size_t num_localities =
                 static_cast<std::size_t>(ini.get_num_localities());
@@ -666,10 +683,10 @@ namespace hpx { namespace agas {
       , cond()
       , mtx()
       , connected(get_number_of_bootstrap_connections(ini_))
-      , thunks(32)
+      , thunks(32)    //-V112
     {
         // register all not registered typenames
-        if (service_type == service_mode_bootstrap)
+        if (service_type == service_mode::bootstrap)
         {
             detail::register_unassigned_typenames();
             // store endpoints of root locality for later
@@ -679,7 +696,7 @@ namespace hpx { namespace agas {
 
     void big_boot_barrier::wait_bootstrap()
     {    // {{{
-        HPX_ASSERT(service_mode_bootstrap == service_type);
+        HPX_ASSERT(service_mode::bootstrap == service_type);
 
         // the root just waits until all localities have connected
         spin();
@@ -710,11 +727,11 @@ namespace hpx { namespace agas {
         naming::address::address_type primary_ns_server,
         naming::address::address_type symbol_ns_server)
     {    // {{{
-        HPX_ASSERT(service_mode_bootstrap != service_type);
+        HPX_ASSERT(service_mode::bootstrap != service_type);
 
         // any worker sends a request for registration and waits
-        HPX_ASSERT(0 != primary_ns_server);
-        HPX_ASSERT(0 != symbol_ns_server);
+        HPX_ASSERT(nullptr != primary_ns_server);
+        HPX_ASSERT(nullptr != symbol_ns_server);
 
         runtime& rt = get_runtime_distributed();
 
@@ -744,12 +761,17 @@ namespace hpx { namespace agas {
 
         // random first parcel id
         apply(static_cast<std::uint32_t>(std::random_device{}()), 0,
-            bootstrap_agas, register_worker_action(), std::move(hdr));
+            bootstrap_agas, register_worker_action(), HPX_MOVE(hdr));
 
         // wait for registration to be complete
         spin();
     }    // }}}
 
+    // 26110: Caller failing to hold lock 'this->mtx' before calling function
+#if defined(HPX_MSVC)
+#pragma warning(push)
+#pragma warning(disable : 26110)
+#endif
     void big_boot_barrier::notify()
     {
         naming::resolver_client& agas_client = naming::get_agas_client();
@@ -757,7 +779,7 @@ namespace hpx { namespace agas {
         bool notify = false;
         {
             std::lock_guard<std::mutex> lk(mtx, std::adopt_lock);
-            if (agas_client.get_status() == state_starting)
+            if (agas_client.get_status() == hpx::state::starting)
             {
                 --connected;
                 if (connected == 0)
@@ -767,15 +789,18 @@ namespace hpx { namespace agas {
         if (notify)
             cond.notify_all();
     }
+#if defined(HPX_MSVC)
+#pragma warning(pop)
+#endif
 
     // This is triggered in runtime_impl::start, after the early action handler
     // has been replaced by the parcelhandler. We have to delay the notifications
     // until this point so that the AGAS locality can come up.
     void big_boot_barrier::trigger()
     {
-        if (service_mode_bootstrap == service_type)
+        if (service_mode::bootstrap == service_type)
         {
-            util::unique_function_nonser<void()>* p;
+            hpx::move_only_function<void()>* p;
 
             while (thunks.pop(p))
             {
@@ -793,7 +818,7 @@ namespace hpx { namespace agas {
         }
     }
 
-    void big_boot_barrier::add_thunk(util::unique_function_nonser<void()>* f)
+    void big_boot_barrier::add_thunk(hpx::move_only_function<void()>* f)
     {
         std::size_t k = 0;
         while (!thunks.push(f))
@@ -816,7 +841,7 @@ namespace hpx { namespace agas {
             bbb;
         if (bbb.get())
         {
-            HPX_THROW_EXCEPTION(internal_server_error,
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
                 "create_big_boot_barrier",
                 "create_big_boot_barrier was called more than once");
         }
@@ -829,7 +854,7 @@ namespace hpx { namespace agas {
             bbb;
         if (!bbb.get())
         {
-            HPX_THROW_EXCEPTION(internal_server_error,
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
                 "destroy_big_boot_barrier",
                 "big_boot_barrier has not been created yet");
         }
@@ -842,12 +867,12 @@ namespace hpx { namespace agas {
             bbb;
         if (!bbb.get())
         {
-            HPX_THROW_EXCEPTION(internal_server_error, "get_big_boot_barrier",
+            HPX_THROW_EXCEPTION(hpx::error::internal_server_error,
+                "get_big_boot_barrier",
                 "big_boot_barrier has not been created yet");
         }
         return *(bbb.get());
     }
-
-}}    // namespace hpx::agas
+}    // namespace hpx::agas
 
 #endif
