@@ -1,4 +1,5 @@
 //  Copyright (c) 2017 Ajai V George
+//  Copyright (c) 2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -24,7 +25,8 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace parallel { inline namespace v1 {
+namespace hpx::parallel {
+
     ///////////////////////////////////////////////////////////////////////////
     // segmented_all_any_none
     namespace detail {
@@ -34,14 +36,14 @@ namespace hpx { namespace parallel { inline namespace v1 {
         // sequential remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_none_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::true_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -92,24 +94,23 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 }
             }
 
-            return result::get(std::move(output));
+            return result::get(HPX_MOVE(output));
         }
 
         // parallel remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_none_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::false_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
-            typedef std::integral_constant<bool,
-                !hpx::traits::is_forward_iterator<FwdIter>::value>
-                forced_seq;
+            using forced_seq = std::integral_constant<bool,
+                !hpx::traits::is_forward_iterator<FwdIter>::value>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -167,7 +168,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     std::list<std::exception_ptr> errors;
                     parallel::util::detail::handle_remote_exceptions<
                         ExPolicy>::call(r, errors);
-                    std::vector<bool> res = hpx::unwrap(std::move(r));
+                    std::vector<bool> res = hpx::unwrap(HPX_MOVE(r));
                     auto it = res.begin();
                     while (it != res.end())
                     {
@@ -177,21 +178,21 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     }
                     return true;
                 },
-                std::move(segments)));
+                HPX_MOVE(segments)));
         }
 
         ///////////////////////////////////////////////////////////////////////
         // sequential remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_any_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::true_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -242,24 +243,23 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 }
             }
 
-            return result::get(std::move(output));
+            return result::get(HPX_MOVE(output));
         }
 
         // parallel remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_any_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::false_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
-            typedef std::integral_constant<bool,
-                !hpx::traits::is_forward_iterator<FwdIter>::value>
-                forced_seq;
+            using forced_seq = std::integral_constant<bool,
+                !hpx::traits::is_forward_iterator<FwdIter>::value>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -317,7 +317,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     std::list<std::exception_ptr> errors;
                     parallel::util::detail::handle_remote_exceptions<
                         ExPolicy>::call(r, errors);
-                    std::vector<bool> res = hpx::unwrap(std::move(r));
+                    std::vector<bool> res = hpx::unwrap(HPX_MOVE(r));
                     auto it = res.begin();
                     while (it != res.end())
                     {
@@ -327,21 +327,21 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     }
                     return false;
                 },
-                std::move(segments)));
+                HPX_MOVE(segments)));
         }
 
         ///////////////////////////////////////////////////////////////////////
         // sequential remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_all_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::true_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -392,24 +392,23 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 }
             }
 
-            return result::get(std::move(output));
+            return result::get(HPX_MOVE(output));
         }
 
         // parallel remote implementation
         template <typename Algo, typename ExPolicy, typename FwdIter,
             typename F, typename Proj>
-        static typename util::detail::algorithm_result<ExPolicy, bool>::type
+        static util::detail::algorithm_result_t<ExPolicy, bool>
         segmented_all_of(Algo&& algo, ExPolicy const& policy, FwdIter first,
             FwdIter last, F&& f, Proj&& proj, std::false_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<FwdIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef util::detail::algorithm_result<ExPolicy, bool> result;
+            using traits = hpx::traits::segmented_iterator_traits<FwdIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using result = util::detail::algorithm_result<ExPolicy, bool>;
 
-            typedef std::integral_constant<bool,
-                !hpx::traits::is_forward_iterator<FwdIter>::value>
-                forced_seq;
+            using forced_seq = std::integral_constant<bool,
+                !hpx::traits::is_forward_iterator<FwdIter>::value>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
@@ -467,7 +466,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     std::list<std::exception_ptr> errors;
                     parallel::util::detail::handle_remote_exceptions<
                         ExPolicy>::call(r, errors);
-                    std::vector<bool> res = hpx::unwrap(std::move(r));
+                    std::vector<bool> res = hpx::unwrap(HPX_MOVE(r));
                     auto it = res.begin();
                     while (it != res.end())
                     {
@@ -477,14 +476,14 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     }
                     return true;
                 },
-                std::move(segments)));
+                HPX_MOVE(segments)));
         }
         /// \endcond
     }    // namespace detail
-}}}      // namespace hpx::parallel::v1
+}    // namespace hpx::parallel
 
 // The segmented iterators we support all live in namespace hpx::segmented
-namespace hpx { namespace segmented {
+namespace hpx::segmented {
 
     // clang-format off
     template <typename InIter,
@@ -494,28 +493,26 @@ namespace hpx { namespace segmented {
             hpx::traits::is_segmented_iterator<InIter>::value
         )>
     // clang-format on
-    bool tag_dispatch(hpx::none_of_t, InIter first, InIter last, F&& f)
+    bool tag_invoke(hpx::none_of_t, InIter first, InIter last, F&& f)
     {
         static_assert(hpx::traits::is_input_iterator<InIter>::value,
             "Requires at least input iterator.");
 
-        return hpx::parallel::v1::detail::segmented_none_of(
-            hpx::parallel::v1::detail::none_of(), hpx::execution::seq, first,
-            last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, std::true_type());
+        return hpx::parallel::detail::segmented_none_of(
+            hpx::parallel::detail::none_of(), hpx::execution::seq, first, last,
+            HPX_FORWARD(F, f), hpx::identity_v, std::true_type());
     }
 
     // clang-format off
     template <typename ExPolicy, typename SegIter,
         typename F,
         HPX_CONCEPT_REQUIRES_(
-            hpx::is_execution_policy<ExPolicy>::value &&
-            hpx::traits::is_iterator<SegIter>::value &&
-            hpx::traits::is_segmented_iterator<SegIter>::value
+            hpx::is_execution_policy_v<ExPolicy> &&
+            hpx::traits::is_iterator_v<SegIter> &&
+            hpx::traits::is_segmented_iterator_v<SegIter>
         )>
     // clang-format on
-    typename hpx::parallel::util::detail::algorithm_result<ExPolicy, bool>::type
-    tag_dispatch(
+    hpx::parallel::util::detail::algorithm_result_t<ExPolicy, bool> tag_invoke(
         hpx::none_of_t, ExPolicy&& policy, SegIter first, SegIter last, F&& f)
     {
         static_assert(hpx::traits::is_forward_iterator<SegIter>::value,
@@ -523,10 +520,9 @@ namespace hpx { namespace segmented {
 
         using is_seq = hpx::is_sequenced_execution_policy<ExPolicy>;
 
-        return hpx::parallel::v1::detail::segmented_none_of(
-            hpx::parallel::v1::detail::none_of(),
-            std::forward<ExPolicy>(policy), first, last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, is_seq());
+        return hpx::parallel::detail::segmented_none_of(
+            hpx::parallel::detail::none_of(), HPX_FORWARD(ExPolicy, policy),
+            first, last, HPX_FORWARD(F, f), hpx::identity_v, is_seq());
     }
 
     // clang-format off
@@ -537,28 +533,26 @@ namespace hpx { namespace segmented {
             hpx::traits::is_segmented_iterator<InIter>::value
         )>
     // clang-format on
-    bool tag_dispatch(hpx::any_of_t, InIter first, InIter last, F&& f)
+    bool tag_invoke(hpx::any_of_t, InIter first, InIter last, F&& f)
     {
         static_assert(hpx::traits::is_input_iterator<InIter>::value,
             "Requires at least input iterator.");
 
-        return hpx::parallel::v1::detail::segmented_any_of(
-            hpx::parallel::v1::detail::any_of(), hpx::execution::seq, first,
-            last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, std::true_type());
+        return hpx::parallel::detail::segmented_any_of(
+            hpx::parallel::detail::any_of(), hpx::execution::seq, first, last,
+            HPX_FORWARD(F, f), hpx::identity_v, std::true_type());
     }
 
     // clang-format off
     template <typename ExPolicy, typename SegIter,
         typename F,
         HPX_CONCEPT_REQUIRES_(
-            hpx::is_execution_policy<ExPolicy>::value &&
-            hpx::traits::is_iterator<SegIter>::value &&
-            hpx::traits::is_segmented_iterator<SegIter>::value
+            hpx::is_execution_policy_v<ExPolicy> &&
+            hpx::traits::is_iterator_v<SegIter> &&
+            hpx::traits::is_segmented_iterator_v<SegIter>
         )>
     // clang-format on
-    typename hpx::parallel::util::detail::algorithm_result<ExPolicy, bool>::type
-    tag_dispatch(
+    hpx::parallel::util::detail::algorithm_result_t<ExPolicy, bool> tag_invoke(
         hpx::any_of_t, ExPolicy&& policy, SegIter first, SegIter last, F&& f)
     {
         static_assert(hpx::traits::is_forward_iterator<SegIter>::value,
@@ -566,10 +560,9 @@ namespace hpx { namespace segmented {
 
         using is_seq = hpx::is_sequenced_execution_policy<ExPolicy>;
 
-        return hpx::parallel::v1::detail::segmented_any_of(
-            hpx::parallel::v1::detail::any_of(), std::forward<ExPolicy>(policy),
-            first, last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, is_seq());
+        return hpx::parallel::detail::segmented_any_of(
+            hpx::parallel::detail::any_of(), HPX_FORWARD(ExPolicy, policy),
+            first, last, HPX_FORWARD(F, f), hpx::identity_v, is_seq());
     }
 
     // clang-format off
@@ -580,28 +573,26 @@ namespace hpx { namespace segmented {
             hpx::traits::is_segmented_iterator<InIter>::value
         )>
     // clang-format on
-    bool tag_dispatch(hpx::all_of_t, InIter first, InIter last, F&& f)
+    bool tag_invoke(hpx::all_of_t, InIter first, InIter last, F&& f)
     {
         static_assert(hpx::traits::is_input_iterator<InIter>::value,
             "Requires at least input iterator.");
 
-        return hpx::parallel::v1::detail::segmented_all_of(
-            hpx::parallel::v1::detail::all_of(), hpx::execution::seq, first,
-            last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, std::true_type());
+        return hpx::parallel::detail::segmented_all_of(
+            hpx::parallel::detail::all_of(), hpx::execution::seq, first, last,
+            HPX_FORWARD(F, f), hpx::identity_v, std::true_type());
     }
 
     // clang-format off
     template <typename ExPolicy, typename SegIter,
         typename F,
         HPX_CONCEPT_REQUIRES_(
-            hpx::is_execution_policy<ExPolicy>::value &&
-            hpx::traits::is_iterator<SegIter>::value &&
-            hpx::traits::is_segmented_iterator<SegIter>::value
+            hpx::is_execution_policy_v<ExPolicy> &&
+            hpx::traits::is_iterator_v<SegIter> &&
+            hpx::traits::is_segmented_iterator_v<SegIter>
         )>
     // clang-format on
-    typename hpx::parallel::util::detail::algorithm_result<ExPolicy, bool>::type
-    tag_dispatch(
+    hpx::parallel::util::detail::algorithm_result_t<ExPolicy, bool> tag_invoke(
         hpx::all_of_t, ExPolicy&& policy, SegIter first, SegIter last, F&& f)
     {
         static_assert(hpx::traits::is_forward_iterator<SegIter>::value,
@@ -609,9 +600,8 @@ namespace hpx { namespace segmented {
 
         using is_seq = hpx::is_sequenced_execution_policy<ExPolicy>;
 
-        return hpx::parallel::v1::detail::segmented_all_of(
-            hpx::parallel::v1::detail::all_of(), std::forward<ExPolicy>(policy),
-            first, last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity{}, is_seq());
+        return hpx::parallel::detail::segmented_all_of(
+            hpx::parallel::detail::all_of(), HPX_FORWARD(ExPolicy, policy),
+            first, last, HPX_FORWARD(F, f), hpx::identity_v, is_seq());
     }
-}}    // namespace hpx::segmented
+}    // namespace hpx::segmented
