@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2016 Hartmut Kaiser
+//  Copyright (c) 2007-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -27,7 +27,7 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace parallel { inline namespace v1 {
+namespace hpx { namespace parallel {
     ///////////////////////////////////////////////////////////////////////////
     // segmented_count
     namespace detail {
@@ -98,7 +98,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 }
             }
 
-            return result::get(std::move(overall_result));
+            return result::get(HPX_MOVE(overall_result));
         }
 
         // parallel remote implementation
@@ -214,8 +214,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     overall_result = dispatch(traits::get_id(sit), algo, policy,
-                        std::true_type(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj));
+                        std::true_type(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj));
                 }
             }
             else
@@ -226,8 +226,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     overall_result += dispatch(traits::get_id(sit), algo,
-                        policy, std::true_type(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj));
+                        policy, std::true_type(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj));
                 }
 
                 // handle all of the full partitions
@@ -239,7 +239,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     {
                         overall_result += dispatch(traits::get_id(sit), algo,
                             policy, std::true_type(), beg, end,
-                            std::forward<F>(f), std::forward<Proj>(proj));
+                            HPX_FORWARD(F, f), HPX_FORWARD(Proj, proj));
                     }
                 }
 
@@ -249,12 +249,12 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     overall_result += dispatch(traits::get_id(sit), algo,
-                        policy, std::true_type(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj));
+                        policy, std::true_type(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj));
                 }
             }
 
-            return result::get(std::move(overall_result));
+            return result::get(HPX_MOVE(overall_result));
         }
 
         // parallel remote implementation
@@ -291,8 +291,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     segments.push_back(dispatch_async(traits::get_id(sit), algo,
-                        policy, forced_seq(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj)));
+                        policy, forced_seq(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj)));
                 }
             }
             else
@@ -303,8 +303,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     segments.push_back(dispatch_async(traits::get_id(sit), algo,
-                        policy, forced_seq(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj)));
+                        policy, forced_seq(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj)));
                 }
 
                 // handle all of the full partitions
@@ -316,7 +316,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     {
                         segments.push_back(dispatch_async(traits::get_id(sit),
                             algo, policy, forced_seq(), beg, end,
-                            std::forward<F>(f), std::forward<Proj>(proj)));
+                            HPX_FORWARD(F, f), HPX_FORWARD(Proj, proj)));
                     }
                 }
 
@@ -326,8 +326,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 if (beg != end)
                 {
                     segments.push_back(dispatch_async(traits::get_id(sit), algo,
-                        policy, forced_seq(), beg, end, std::forward<F>(f),
-                        std::forward<Proj>(proj)));
+                        policy, forced_seq(), beg, end, HPX_FORWARD(F, f),
+                        HPX_FORWARD(Proj, proj)));
                 }
             }
 
@@ -344,11 +344,11 @@ namespace hpx { namespace parallel { inline namespace v1 {
                             return val + curr.get();
                         });
                 },
-                std::move(segments)));
+                HPX_MOVE(segments)));
         }
         /// \endcond
     }    // namespace detail
-}}}      // namespace hpx::parallel::v1
+}}       // namespace hpx::parallel
 
 // The segmented iterators we support all live in namespace hpx::segmented
 namespace hpx { namespace segmented {
@@ -361,7 +361,7 @@ namespace hpx { namespace segmented {
             hpx::traits::is_segmented_iterator<InIter>::value
         )>
     // clang-format on
-    typename std::iterator_traits<InIter>::difference_type tag_dispatch(
+    typename std::iterator_traits<InIter>::difference_type tag_invoke(
         hpx::count_t, InIter first, InIter last, T const& value)
     {
         static_assert((hpx::traits::is_input_iterator<InIter>::value),
@@ -375,24 +375,24 @@ namespace hpx { namespace segmented {
             return difference_type();
         }
 
-        return hpx::parallel::v1::detail::segmented_count(
-            hpx::parallel::v1::detail::count<difference_type>(),
-            hpx::execution::seq, first, last, value,
-            hpx::parallel::util::projection_identity(), std::true_type());
+        return hpx::parallel::detail::segmented_count(
+            hpx::parallel::detail::count<difference_type>(),
+            hpx::execution::seq, first, last, value, hpx::identity_v,
+            std::true_type());
     }
 
     // clang-format off
     template <typename ExPolicy, typename SegIter,
         typename T,
         HPX_CONCEPT_REQUIRES_(
-            hpx::is_execution_policy<ExPolicy>::value &&
-            hpx::traits::is_iterator<SegIter>::value &&
-            hpx::traits::is_segmented_iterator<SegIter>::value
+            hpx::is_execution_policy_v<ExPolicy> &&
+            hpx::traits::is_iterator_v<SegIter> &&
+            hpx::traits::is_segmented_iterator_v<SegIter>
         )>
     // clang-format on
     typename hpx::parallel::util::detail::algorithm_result<ExPolicy,
         typename std::iterator_traits<SegIter>::difference_type>::type
-    tag_dispatch(hpx::count_t, ExPolicy&& policy, SegIter first, SegIter last,
+    tag_invoke(hpx::count_t, ExPolicy&& policy, SegIter first, SegIter last,
         T const& value)
     {
         static_assert((hpx::traits::is_forward_iterator<SegIter>::value),
@@ -409,10 +409,10 @@ namespace hpx { namespace segmented {
                 difference_type>::get(difference_type());
         }
 
-        return hpx::parallel::v1::detail::segmented_count(
-            hpx::parallel::v1::detail::count<difference_type>(),
-            std::forward<ExPolicy>(policy), first, last, value,
-            hpx::parallel::util::projection_identity(), is_seq());
+        return hpx::parallel::detail::segmented_count(
+            hpx::parallel::detail::count<difference_type>(),
+            HPX_FORWARD(ExPolicy, policy), first, last, value, hpx::identity_v,
+            is_seq());
     }
 
     // clang-format off
@@ -423,7 +423,7 @@ namespace hpx { namespace segmented {
             hpx::traits::is_segmented_iterator<InIter>::value
         )>
     // clang-format on
-    typename std::iterator_traits<InIter>::difference_type tag_dispatch(
+    typename std::iterator_traits<InIter>::difference_type tag_invoke(
         hpx::count_if_t, InIter first, InIter last, F&& f)
     {
         static_assert((hpx::traits::is_input_iterator<InIter>::value),
@@ -437,24 +437,24 @@ namespace hpx { namespace segmented {
             return difference_type();
         }
 
-        return hpx::parallel::v1::detail::segmented_count_if(
-            hpx::parallel::v1::detail::count_if<difference_type>(),
-            hpx::execution::seq, first, last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity(), std::true_type());
+        return hpx::parallel::detail::segmented_count_if(
+            hpx::parallel::detail::count_if<difference_type>(),
+            hpx::execution::seq, first, last, HPX_FORWARD(F, f),
+            hpx::identity_v, std::true_type());
     }
 
     // clang-format off
     template <typename ExPolicy, typename SegIter,
         typename F,
         HPX_CONCEPT_REQUIRES_(
-            hpx::is_execution_policy<ExPolicy>::value &&
-            hpx::traits::is_iterator<SegIter>::value &&
-            hpx::traits::is_segmented_iterator<SegIter>::value
+            hpx::is_execution_policy_v<ExPolicy> &&
+            hpx::traits::is_iterator_v<SegIter> &&
+            hpx::traits::is_segmented_iterator_v<SegIter>
         )>
     // clang-format on
     typename hpx::parallel::util::detail::algorithm_result<ExPolicy,
         typename std::iterator_traits<SegIter>::difference_type>::type
-    tag_dispatch(
+    tag_invoke(
         hpx::count_if_t, ExPolicy&& policy, SegIter first, SegIter last, F&& f)
     {
         static_assert((hpx::traits::is_forward_iterator<SegIter>::value),
@@ -471,9 +471,9 @@ namespace hpx { namespace segmented {
                 difference_type>::get(difference_type());
         }
 
-        return hpx::parallel::v1::detail::segmented_count_if(
-            hpx::parallel::v1::detail::count_if<difference_type>(),
-            std::forward<ExPolicy>(policy), first, last, std::forward<F>(f),
-            hpx::parallel::util::projection_identity(), is_seq());
+        return hpx::parallel::detail::segmented_count_if(
+            hpx::parallel::detail::count_if<difference_type>(),
+            HPX_FORWARD(ExPolicy, policy), first, last, HPX_FORWARD(F, f),
+            hpx::identity_v, is_seq());
     }
 }}    // namespace hpx::segmented
