@@ -1,5 +1,5 @@
 //  Copyright (c) 2016 Minh-Khanh Do
-//  Copyright (c) 2020 Hartmut Kaiser
+//  Copyright (c) 2020-2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -17,6 +17,7 @@
 #include <hpx/futures/future.hpp>
 #include <hpx/iterator_support/traits/is_iterator.hpp>
 #include <hpx/pack_traversal/unwrap.hpp>
+#include <hpx/parallel/util/partitioner.hpp>
 #include <hpx/type_support/unused.hpp>
 #include <hpx/type_support/void_guard.hpp>
 
@@ -33,7 +34,8 @@
 #include <utility>
 #include <vector>
 
-namespace hpx { namespace parallel { inline namespace v1 {
+namespace hpx::parallel {
+
     ///////////////////////////////////////////////////////////////////////////
     // segmented scan
     namespace detail {
@@ -46,13 +48,12 @@ namespace hpx { namespace parallel { inline namespace v1 {
         T sequential_segmented_scan_T(
             InIter first, InIter last, Op&& op, Conv&& conv)
         {
-            T ret = hpx::util::invoke(conv, *first);
+            T ret = HPX_INVOKE(conv, *first);
             if (first != last)
             {
                 for (++first; first != last; ++first)
                 {
-                    ret = hpx::util::invoke(
-                        op, ret, hpx::util::invoke(conv, *first));
+                    ret = hpx::invoke(op, ret, hpx::invoke(conv, *first));
                 }
             }
             return ret;
@@ -60,13 +61,12 @@ namespace hpx { namespace parallel { inline namespace v1 {
 
         // does a scan and returns last value of the scan
         template <typename Value>
-        struct segmented_scan_T
-          : public detail::algorithm<segmented_scan_T<Value>, Value>
+        struct segmented_scan_T : algorithm<segmented_scan_T<Value>, Value>
         {
-            typedef Value T;
+            using T = Value;
 
             segmented_scan_T()
-              : segmented_scan_T::algorithm("segmented_scan_T")
+              : algorithm<segmented_scan_T<Value>, Value>("segmented_scan_T")
             {
             }
 
@@ -75,8 +75,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
             static T sequential(
                 ExPolicy&&, InIter first, InIter last, Op&& op, Conv&& conv)
             {
-                return sequential_segmented_scan_T<T>(first, last,
-                    std::forward<Op>(op), std::forward<Conv>(conv));
+                return sequential_segmented_scan_T<T>(
+                    first, last, HPX_FORWARD(Op, op), HPX_FORWARD(Conv, conv));
             }
 
             template <typename ExPolicy, typename FwdIter, typename Op,
@@ -87,23 +87,25 @@ namespace hpx { namespace parallel { inline namespace v1 {
             {
                 using execution_policy_type = std::decay_t<ExPolicy>;
                 return util::partitioner<ExPolicy, T>::call(
-                    std::forward<ExPolicy>(policy), first,
+                    HPX_FORWARD(ExPolicy, policy), first,
                     std::distance(first, last),
-                    [op, conv](FwdIter part_begin, std::size_t part_size) -> T {
-                        T ret = hpx::util::invoke(conv, *part_begin);
+                    [op, conv](FwdIter part_begin,
+                        std::size_t part_size) mutable -> T {
+                        T ret = hpx::invoke(conv, *part_begin);
                         if (part_size > 1)
                         {
                             // MSVC complains if 'op' is captured by reference
                             util::loop_n<execution_policy_type>(part_begin + 1,
                                 part_size - 1,
-                                [&ret, op, conv](FwdIter const& curr) {
-                                    ret = hpx::util::invoke(op, ret,
-                                        hpx::util::invoke(conv, *curr));
+                                [&ret, op, conv](
+                                    FwdIter const& curr) mutable -> void {
+                                    ret = hpx::invoke(
+                                        op, ret, hpx::invoke(conv, *curr));
                                 });
                         }
                         return ret;
                     },
-                    hpx::unwrapping([op](std::vector<T>&& results) -> T {
+                    hpx::unwrapping([op](auto&& results) mutable -> T {
                         T ret = *results.begin();
                         if (results.size() > 1)
                         {
@@ -112,8 +114,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                                 results.begin() + 1, results.size() - 1,
                                 [&ret, op](
                                     typename std::vector<T>::iterator const&
-                                        curr) {
-                                    ret = hpx::util::invoke(op, ret, *curr);
+                                        curr) mutable {
+                                    ret = hpx::invoke(op, ret, *curr);
                                 });
                         }
                         return ret;
@@ -124,11 +126,10 @@ namespace hpx { namespace parallel { inline namespace v1 {
         // do the scan (exclusive/inclusive)
         // does not return anything
         template <typename Algo>
-        struct segmented_scan_void
-          : public detail::algorithm<segmented_scan_void<Algo>>
+        struct segmented_scan_void : algorithm<segmented_scan_void<Algo>>
         {
             segmented_scan_void()
-              : segmented_scan_void::algorithm("segmented_scan_void")
+              : algorithm<segmented_scan_void<Algo>>("segmented_scan_void")
             {
             }
 
@@ -138,9 +139,9 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 InIter first, InIter last, OutIter dest, T&& init, Op&& op,
                 Conv&& conv)
             {
-                Algo().sequential(std::forward<ExPolicy>(policy), first, last,
-                    dest, std::forward<T>(init), std::forward<Op>(op),
-                    std::forward<Conv>(conv));
+                Algo().sequential(HPX_FORWARD(ExPolicy, policy), first, last,
+                    dest, HPX_FORWARD(T, init), HPX_FORWARD(Op, op),
+                    HPX_FORWARD(Conv, conv));
 
                 return hpx::util::unused;
             }
@@ -151,16 +152,16 @@ namespace hpx { namespace parallel { inline namespace v1 {
             parallel(ExPolicy&& policy, InIter first, InIter last, OutIter dest,
                 T&& init, Op&& op, Conv&& conv)
             {
-                typedef typename util::detail::algorithm_result<ExPolicy>::type
-                    result_type;
+                using result_type =
+                    typename util::detail::algorithm_result<ExPolicy>::type;
 
                 if (first == last)
                     return util::detail::algorithm_result<ExPolicy>::get();
 
                 return hpx::util::void_guard<result_type>(),
-                       Algo().parallel(std::forward<ExPolicy>(policy), first,
-                           last, dest, std::forward<T>(init),
-                           std::forward<Op>(op), std::forward<Conv>(conv));
+                       Algo().parallel(HPX_FORWARD(ExPolicy, policy), first,
+                           last, dest, HPX_FORWARD(T, init),
+                           HPX_FORWARD(Op, op), HPX_FORWARD(Conv, conv));
             }
         };
 
@@ -177,13 +178,13 @@ namespace hpx { namespace parallel { inline namespace v1 {
         static bool is_segmented_the_same(
             SegIter first, SegIter last, OutIter dest, std::true_type)
         {
-            typedef hpx::traits::segmented_iterator_traits<SegIter> traits_in;
-            typedef typename traits_in::segment_iterator segment_iterator_in;
-            typedef typename traits_in::local_iterator local_iterator_type_in;
+            using traits_in = hpx::traits::segmented_iterator_traits<SegIter>;
+            using segment_iterator_in = typename traits_in::segment_iterator;
+            using local_iterator_type_in = typename traits_in::local_iterator;
 
-            typedef hpx::traits::segmented_iterator_traits<OutIter> traits_out;
-            typedef typename traits_out::segment_iterator segment_iterator_out;
-            typedef typename traits_out::local_iterator local_iterator_type_out;
+            using traits_out = hpx::traits::segmented_iterator_traits<OutIter>;
+            using segment_iterator_out = typename traits_out::segment_iterator;
+            using local_iterator_type_out = typename traits_out::local_iterator;
 
             segment_iterator_in sit_in = traits_in::segment(first);
             segment_iterator_in send_in = traits_in::segment(last);
@@ -203,16 +204,17 @@ namespace hpx { namespace parallel { inline namespace v1 {
 
                 if (beg_in != end_in)
                 {
-                    id_type in_id =
+                    id_type const in_id =
                         get_locality_from_id(traits_in::get_id(sit_in));
-                    id_type out_id =
+                    id_type const out_id =
                         get_locality_from_id(traits_out::get_id(sit_out));
 
                     if (in_id != out_id)
                         return false;
 
-                    std::size_t in_dist = std::distance(beg_in, end_in);
-                    std::size_t out_dist = std::distance(beg_out, end_out);
+                    std::size_t const in_dist = std::distance(beg_in, end_in);
+                    std::size_t const out_dist =
+                        std::distance(beg_out, end_out);
 
                     if (in_dist != out_dist)
                         return false;
@@ -229,16 +231,17 @@ namespace hpx { namespace parallel { inline namespace v1 {
 
                 if (beg_in != end_in)
                 {
-                    id_type in_id =
+                    id_type const in_id =
                         get_locality_from_id(traits_in::get_id(sit_in));
-                    id_type out_id =
+                    id_type const out_id =
                         get_locality_from_id(traits_out::get_id(sit_out));
 
                     if (in_id != out_id)
                         return false;
 
-                    std::size_t in_dist = std::distance(beg_in, end_in);
-                    std::size_t out_dist = std::distance(beg_out, end_out);
+                    std::size_t const in_dist = std::distance(beg_in, end_in);
+                    std::size_t const out_dist =
+                        std::distance(beg_out, end_out);
 
                     if (in_dist != out_dist)
                         return false;
@@ -264,8 +267,10 @@ namespace hpx { namespace parallel { inline namespace v1 {
                         if (in_id != out_id)
                             return false;
 
-                        std::size_t in_dist = std::distance(beg_in, end_in);
-                        std::size_t out_dist = std::distance(beg_out, end_out);
+                        std::size_t const in_dist =
+                            std::distance(beg_in, end_in);
+                        std::size_t const out_dist =
+                            std::distance(beg_out, end_out);
 
                         if (in_dist != out_dist)
                             return false;
@@ -281,16 +286,17 @@ namespace hpx { namespace parallel { inline namespace v1 {
 
                 if (beg_in != end_in)
                 {
-                    id_type in_id =
+                    id_type const in_id =
                         get_locality_from_id(traits_in::get_id(sit_in));
-                    id_type out_id =
+                    id_type const out_id =
                         get_locality_from_id(traits_out::get_id(sit_out));
 
                     if (in_id != out_id)
                         return false;
 
-                    std::size_t in_dist = std::distance(beg_in, end_in);
-                    std::size_t out_dist = std::distance(beg_out, end_out);
+                    std::size_t const in_dist = std::distance(beg_in, end_in);
+                    std::size_t const out_dist =
+                        std::distance(beg_out, end_out);
 
                     if (in_dist != out_dist)
                         return false;
@@ -309,22 +315,21 @@ namespace hpx { namespace parallel { inline namespace v1 {
         segmented_scan_seq(ExPolicy const& policy, SegIter first, SegIter last,
             OutIter dest, Conv&& conv, T const& init, Op&& op, std::true_type)
         {
-            typedef util::detail::algorithm_result<ExPolicy, OutIter> result;
+            using result = util::detail::algorithm_result<ExPolicy, OutIter>;
 
             if (first == last)
-                return result::get(std::move(dest));
+                return result::get(HPX_MOVE(dest));
 
-            typedef hpx::traits::segmented_iterator_traits<SegIter> traits_in;
-            typedef typename traits_in::segment_iterator segment_iterator_in;
-            typedef typename traits_in::local_iterator local_iterator_type_in;
+            using traits_in = hpx::traits::segmented_iterator_traits<SegIter>;
+            using segment_iterator_in = typename traits_in::segment_iterator;
+            using local_iterator_type_in = typename traits_in::local_iterator;
 
-            typedef hpx::traits::segmented_iterator_traits<OutIter> traits_out;
-            typedef typename traits_out::segment_iterator segment_iterator_out;
-            typedef typename traits_out::local_iterator local_iterator_type_out;
+            using traits_out = hpx::traits::segmented_iterator_traits<OutIter>;
+            using segment_iterator_out = typename traits_out::segment_iterator;
+            using local_iterator_type_out = typename traits_out::local_iterator;
 
-            typedef typename hpx::tuple<local_iterator_type_in,
-                local_iterator_type_in>
-                local_iterator_in_tuple;
+            using local_iterator_in_tuple =
+                hpx::tuple<local_iterator_type_in, local_iterator_type_in>;
 
             segment_iterator_in sit_in = traits_in::segment(first);
             segment_iterator_in send_in = traits_in::segment(last);
@@ -406,7 +411,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                 dispatch(traits_out::get_id(out_iters[i]),
                     segmented_scan_void<Algo>(), policy, std::true_type(),
                     get<0>(in_iters[i]), get<1>(in_iters[i]), out,
-                    std::forward<Conv>(conv), last_value, std::forward<Op>(op));
+                    HPX_FORWARD(Conv, conv), last_value, HPX_FORWARD(Op, op));
 
                 // 3. Step: compute new init values for the next segment
                 last_value = op(results[i], last_value);
@@ -415,7 +420,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
             OutIter final_dest = dest;
             std::advance(final_dest, std::distance(first, last));
 
-            return result::get(std::move(final_dest));
+            return result::get(HPX_MOVE(final_dest));
         }
 
         // sequential non segmented OutIter implementation
@@ -426,19 +431,19 @@ namespace hpx { namespace parallel { inline namespace v1 {
             SegIter last, OutIter dest, T const& init, Op&& op, F1&& f1,
             F2&& f2)
         {
-            typedef util::detail::algorithm_result<ExPolicy, OutIter> result;
+            using result = util::detail::algorithm_result<ExPolicy, OutIter>;
 
             if (first == last)
-                return result::get(std::move(dest));
+                return result::get(HPX_MOVE(dest));
 
-            typedef hpx::traits::segmented_iterator_traits<SegIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
+            using traits = hpx::traits::segmented_iterator_traits<SegIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
 
-            typedef typename std::vector<T> vector_type;
+            using vector_type = std::vector<T>;
 
             std::vector<vector_type> results;
 
@@ -494,9 +499,16 @@ namespace hpx { namespace parallel { inline namespace v1 {
             for (auto res : results)
             {
                 dest = f1(res.begin(), res.end(), dest, last_value, op);
+#if defined(HPX_GCC_VERSION) && HPX_GCC_VERSION >= 100000
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Warray-bounds"
+#endif
                 last_value = f2(res, last_value);
+#if defined(HPX_GCC_VERSION) && HPX_GCC_VERSION >= 100000
+#pragma GCC diagnostic pop
+#endif
             }
-            return result::get(std::move(dest));
+            return result::get(HPX_MOVE(dest));
         }
 
         ///////////////////////////////////////////////////////////////////////
@@ -509,29 +521,27 @@ namespace hpx { namespace parallel { inline namespace v1 {
         segmented_scan_par(ExPolicy const& policy, SegIter first, SegIter last,
             OutIter dest, Conv&& conv, T const& init, Op&& op, std::true_type)
         {
-            typedef util::detail::algorithm_result<ExPolicy, OutIter> result;
+            using result = util::detail::algorithm_result<ExPolicy, OutIter>;
 
             if (first == last)
-                return result::get(std::move(dest));
+                return result::get(HPX_MOVE(dest));
 
-            typedef hpx::traits::segmented_iterator_traits<SegIter> traits_in;
-            typedef typename traits_in::segment_iterator segment_iterator_in;
-            typedef typename traits_in::local_iterator local_iterator_type_in;
+            using traits_in = hpx::traits::segmented_iterator_traits<SegIter>;
+            using segment_iterator_in = typename traits_in::segment_iterator;
+            using local_iterator_type_in = typename traits_in::local_iterator;
 
-            typedef hpx::traits::segmented_iterator_traits<OutIter> traits_out;
-            typedef typename traits_out::segment_iterator segment_iterator_out;
-            typedef typename traits_out::local_iterator local_iterator_type_out;
+            using traits_out = hpx::traits::segmented_iterator_traits<OutIter>;
+            using segment_iterator_out = typename traits_out::segment_iterator;
+            using local_iterator_type_out = typename traits_out::local_iterator;
 
-            typedef typename std::iterator_traits<
-                segment_iterator_in>::difference_type difference_type;
+            using difference_type = typename std::iterator_traits<
+                segment_iterator_in>::difference_type;
 
-            typedef std::integral_constant<bool,
-                !hpx::traits::is_forward_iterator<SegIter>::value>
-                forced_seq;
+            using forced_seq = std::integral_constant<bool,
+                !hpx::traits::is_forward_iterator<SegIter>::value>;
 
-            typedef typename hpx::tuple<local_iterator_type_in,
-                local_iterator_type_in>
-                local_iterator_in_tuple;
+            using local_iterator_in_tuple =
+                hpx::tuple<local_iterator_type_in, local_iterator_type_in>;
 
             segment_iterator_in sit_in = traits_in::segment(first);
             segment_iterator_in send_in = traits_in::segment(last);
@@ -655,8 +665,7 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     std::vector<hpx::future<void>>&&) mutable -> OutIter {
                     return final_dest;
                 },
-                std::move(results), std::move(workitems),
-                std::move(finalitems)));
+                HPX_MOVE(results), HPX_MOVE(workitems), HPX_MOVE(finalitems)));
         }
 
         // parallel non-segmented OutIter implementation
@@ -667,28 +676,26 @@ namespace hpx { namespace parallel { inline namespace v1 {
             SegIter last, OutIter dest, T const& init, Op&& op, F1&& f1,
             F2&& f2)
         {
-            typedef util::detail::algorithm_result<ExPolicy, OutIter> result;
+            using result = util::detail::algorithm_result<ExPolicy, OutIter>;
 
             if (first == last)
-                return result::get(std::move(dest));
+                return result::get(HPX_MOVE(dest));
 
-            typedef hpx::traits::segmented_iterator_traits<SegIter> traits;
-            typedef typename traits::segment_iterator segment_iterator;
-            typedef typename traits::local_iterator local_iterator_type;
-            typedef
-                typename std::iterator_traits<segment_iterator>::difference_type
-                    difference_type;
+            using traits = hpx::traits::segmented_iterator_traits<SegIter>;
+            using segment_iterator = typename traits::segment_iterator;
+            using local_iterator_type = typename traits::local_iterator;
+            using difference_type = typename std::iterator_traits<
+                segment_iterator>::difference_type;
 
-            typedef std::integral_constant<bool,
-                !hpx::traits::is_forward_iterator<SegIter>::value>
-                forced_seq;
+            using forced_seq = std::integral_constant<bool,
+                !hpx::traits::is_forward_iterator<SegIter>::value>;
 
             segment_iterator sit = traits::segment(first);
             segment_iterator send = traits::segment(last);
 
             difference_type count = std::distance(sit, send);
 
-            typedef typename std::vector<T> vector_type;
+            using vector_type = std::vector<T>;
             std::vector<hpx::shared_future<vector_type>> results;
             results.reserve(count);
 
@@ -785,9 +792,8 @@ namespace hpx { namespace parallel { inline namespace v1 {
                     std::vector<hpx::future<void>>&&) mutable -> OutIter {
                     return final_dest;
                 },
-                std::move(results), std::move(workitems),
-                std::move(finalitems)));
+                HPX_MOVE(results), HPX_MOVE(workitems), HPX_MOVE(finalitems)));
         }
         /// \endcond
     }    // namespace detail
-}}}      // namespace hpx::parallel::v1
+}    // namespace hpx::parallel
