@@ -4,4 +4,6 @@
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-// This file is intentionally left empty.
+// This file is intended for use in precompiled headers targets.
+
+int main() {}
