@@ -1,6 +1,7 @@
 //  Copyright (c) 2018-2020 Mikael Simberg
 //  Copyright (c) 2018-2019 John Biddiscombe
 //  Copyright (c) 2011 Bryce Adelstein-Lelbach
+//  Copyright (c) 2024 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -9,27 +10,24 @@
 #include <hpx/config.hpp>
 #if defined(HPX_HAVE_DISTRIBUTED_RUNTIME) && !defined(HPX_COMPUTE_DEVICE_CODE)
 #include <hpx/actions_base/plain_action.hpp>
-#include <hpx/async_distributed/continuation.hpp>
-#include <hpx/future.hpp>
-#include <hpx/runtime.hpp>
 #endif
+#include <hpx/algorithm.hpp>
+#include <hpx/execution.hpp>
+#include <hpx/format.hpp>
+#include <hpx/future.hpp>
 #include <hpx/init.hpp>
-#include <hpx/local/algorithm.hpp>
-#include <hpx/local/execution.hpp>
-#include <hpx/local/future.hpp>
-#include <hpx/local/runtime.hpp>
-#include <hpx/local/thread.hpp>
-#include <hpx/modules/format.hpp>
 #include <hpx/modules/synchronization.hpp>
 #include <hpx/modules/testing.hpp>
 #include <hpx/modules/timing.hpp>
-#include <hpx/threading_base/annotated_function.hpp>
+#include <hpx/runtime.hpp>
+#include <hpx/thread.hpp>
 
 #include <array>
 #include <atomic>
 #include <cstddef>
 #include <cstdint>
 #include <iostream>
+#include <memory>
 #include <sstream>
 #include <stdexcept>
 #include <string>
@@ -40,10 +38,9 @@ using hpx::program_options::options_description;
 using hpx::program_options::value;
 using hpx::program_options::variables_map;
 
-using hpx::apply;
 using hpx::async;
 using hpx::future;
-using hpx::lcos::wait_each;
+using hpx::post;
 
 using hpx::chrono::high_resolution_timer;
 
@@ -54,11 +51,11 @@ static std::uint64_t num_threads = 1;
 static std::string info_string = "";
 
 ///////////////////////////////////////////////////////////////////////////////
-void print_stats(const char* title, const char* wait, const char* exec,
+void print_stats(char const* title, char const* wait, char const* exec,
     std::int64_t count, double duration, bool csv)
 {
     std::ostringstream temp;
-    double us = 1e6 * duration / count;
+    double const us = 1e6 * duration / count;
     if (csv)
     {
         hpx::util::format_to(temp,
@@ -80,28 +77,28 @@ void print_stats(const char* title, const char* wait, const char* exec,
     //hpx::util::print_cdash_timing(title, duration);
 }
 
-const char* exec_name(hpx::execution::parallel_executor const&)
+char const* exec_name(hpx::execution::parallel_executor const&)
 {
     return "parallel_executor";
 }
 
-const char* exec_name(
-    hpx::parallel::execution::parallel_executor_aggregated const&)
+char const* exec_name(hpx::execution::experimental::scheduler_executor<
+    hpx::execution::experimental::thread_pool_scheduler> const&)
 {
-    return "parallel_executor_aggregated";
+    return "scheduler_executor<thread_pool_scheduler>";
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // we use globals here to prevent the delay from being optimized away
-double global_scratch = 0;
-std::uint64_t num_iterations = 0;
+double volatile global_scratch = 0;
+std::uint64_t volatile num_iterations = 0;
 
 ///////////////////////////////////////////////////////////////////////////////
 double null_function() noexcept
 {
     if (num_iterations > 0)
     {
-        const int array_size = 4096;
+        constexpr int array_size = 4096;
         std::array<double, array_size> dummy;
         for (std::uint64_t i = 0; i < num_iterations; ++i)
         {
@@ -110,6 +107,7 @@ double null_function() noexcept
                 dummy[j] = 1.0 / (2.0 * i * j + 1.0);
             }
         }
+        global_scratch = dummy[0];
         return dummy[0];
     }
     return 0.0;
@@ -119,7 +117,7 @@ struct scratcher
 {
     void operator()(future<double> r) const
     {
-        global_scratch += r.get();
+        global_scratch = global_scratch + r.get();
     }
 };
 
@@ -129,36 +127,36 @@ HPX_PLAIN_ACTION(null_function, null_action)
 // Time async action execution using wait each on futures vector
 void measure_action_futures_wait_each(std::uint64_t count, bool csv)
 {
-    const hpx::naming::id_type here = hpx::find_here();
+    hpx::id_type const here = hpx::find_here();
     std::vector<future<double>> futures;
     futures.reserve(count);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
         futures.push_back(async<null_action>(here));
-    wait_each(scratcher(), futures);
+    hpx::wait_each(scratcher(), futures);
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("action", "WaitEach", "no-executor", count, duration, csv);
 }
 
 // Time async action execution using wait each on futures vector
 void measure_action_futures_wait_all(std::uint64_t count, bool csv)
 {
-    const hpx::naming::id_type here = hpx::find_here();
+    hpx::id_type const here = hpx::find_here();
     std::vector<future<double>> futures;
     futures.reserve(count);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
         futures.push_back(async<null_action>(here));
-    wait_all(futures);
+    hpx::wait_all(futures);
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("action", "WaitAll", "no-executor", count, duration, csv);
 }
 #endif
@@ -172,13 +170,13 @@ void measure_function_futures_wait_each(
     futures.reserve(count);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
         futures.push_back(async(exec, &null_function));
-    wait_each(scratcher(), futures);
+    hpx::wait_each(scratcher(), futures);
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("async", "WaitEach", exec_name(exec), count, duration, csv);
 }
 
@@ -190,12 +188,12 @@ void measure_function_futures_wait_all(
     futures.reserve(count);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
         futures.push_back(async(exec, &null_function));
-    wait_all(futures);
+    hpx::wait_all(futures);
 
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("async", "WaitAll", exec_name(exec), count, duration, csv);
 }
 
@@ -213,31 +211,32 @@ void measure_function_futures_limiting_executor(
     {
         sched->add_remove_scheduler_mode(
             // add these flags
-            hpx::threads::policies::scheduler_mode(
-                hpx::threads::policies::enable_stealing |
-                hpx::threads::policies::assign_work_round_robin |
-                hpx::threads::policies::steal_after_local),
+            hpx::threads::policies::scheduler_mode::enable_stealing |
+                hpx::threads::policies::scheduler_mode::
+                    assign_work_round_robin |
+                hpx::threads::policies::scheduler_mode::steal_after_local,
             // remove these flags
-            hpx::threads::policies::scheduler_mode(
-                hpx::threads::policies::enable_stealing_numa |
-                hpx::threads::policies::assign_work_thread_parent |
-                hpx::threads::policies::steal_high_priority_first));
+            hpx::threads::policies::scheduler_mode::enable_stealing_numa |
+                hpx::threads::policies::scheduler_mode::
+                    assign_work_thread_parent |
+                hpx::threads::policies::scheduler_mode::
+                    steal_high_priority_first);
     }
 
     // test a parallel algorithm on custom pool with high priority
     auto const chunk_size = count / (num_threads * 2);
-    hpx::execution::static_chunk_size fixed(chunk_size);
+    hpx::execution::experimental::static_chunk_size fixed(chunk_size);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     {
         hpx::execution::experimental::limiting_executor<Executor> signal_exec(
             exec, tasks, tasks + 1000);
-        hpx::for_loop(
+        hpx::experimental::for_loop(
             hpx::execution::par.with(fixed), 0, count, [&](std::uint64_t) {
-                hpx::apply(signal_exec, [&]() {
+                hpx::post(signal_exec, [&]() {
                     null_function();
-                    sanity_check--;
+                    --sanity_check;
                 });
             });
     }
@@ -249,7 +248,7 @@ void measure_function_futures_limiting_executor(
     }
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats(
         "apply", "limiting-Exec", exec_name(exec), count, duration, csv);
 }
@@ -259,64 +258,66 @@ void measure_function_futures_sliding_semaphore(
     std::uint64_t count, bool csv, Executor& exec)
 {
     // start the clock
-    high_resolution_timer walltime;
-    const int sem_count = 5000;
-    hpx::lcos::local::sliding_semaphore sem(sem_count);
+    high_resolution_timer const walltime;
+    constexpr int sem_count = 5000;
+    auto sem = std::make_shared<hpx::sliding_semaphore>(sem_count);
     for (std::uint64_t i = 0; i < count; ++i)
     {
-        hpx::async(exec, [i, &sem]() {
+        hpx::async(exec, [i, sem]() {
             null_function();
-            sem.signal(i);
+            sem->signal(i);
         });
-        sem.wait(i);
+        sem->wait(i);
     }
-    sem.wait(count + sem_count - 1);
+    sem->wait(count + sem_count - 1);
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("apply", "Sliding-Sem", exec_name(exec), count, duration, csv);
 }
 
 struct unlimited_number_of_chunks
 {
     template <typename Executor>
-    std::size_t maximal_number_of_chunks(
-        Executor&& /*executor*/, std::size_t /*cores*/, std::size_t num_tasks)
+    friend std::size_t tag_override_invoke(
+        hpx::execution::experimental::maximal_number_of_chunks_t,
+        unlimited_number_of_chunks, Executor&&, std::size_t,
+        std::size_t num_tasks)
     {
         return num_tasks;
     }
 };
 
-namespace hpx { namespace parallel { namespace execution {
-    template <>
-    struct is_executor_parameters<unlimited_number_of_chunks> : std::true_type
-    {
-    };
-}}}    // namespace hpx::parallel::execution
+template <>
+struct hpx::execution::experimental::is_executor_parameters<
+    unlimited_number_of_chunks> : std::true_type
+{
+};
 
 template <typename Executor>
 void measure_function_futures_for_loop(std::uint64_t count, bool csv,
     Executor& exec, char const* executor_name = nullptr)
 {
     // start the clock
-    high_resolution_timer walltime;
-    hpx::for_loop(
+    high_resolution_timer const walltime;
+    hpx::experimental::for_loop(
         hpx::execution::par.on(exec).with(
-            hpx::execution::static_chunk_size(1), unlimited_number_of_chunks()),
+            hpx::execution::experimental::static_chunk_size(1),
+            unlimited_number_of_chunks()),
         0, count, [](std::uint64_t) { null_function(); });
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("for_loop", "par",
         executor_name ? executor_name : exec_name(exec), count, duration, csv);
 }
 
 void measure_function_futures_register_work(std::uint64_t count, bool csv)
 {
-    hpx::lcos::local::latch l(count);
+    hpx::latch l(count);
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
     {
         hpx::threads::thread_init_data data(
@@ -330,13 +331,13 @@ void measure_function_futures_register_work(std::uint64_t count, bool csv)
     l.wait();
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("register_work", "latch", "none", count, duration, csv);
 }
 
 void measure_function_futures_create_thread(std::uint64_t count, bool csv)
 {
-    hpx::lcos::local::latch l(count);
+    hpx::latch l(count);
 
     auto const sched = hpx::threads::get_self_id_data()->get_scheduler_base();
     auto func = [&l]() {
@@ -345,14 +346,14 @@ void measure_function_futures_create_thread(std::uint64_t count, bool csv)
     };
     auto const thread_func =
         hpx::threads::detail::thread_function_nullary<decltype(func)>{func};
-    auto const desc = hpx::util::thread_description();
-    auto const prio = hpx::threads::thread_priority::normal;
-    auto const hint = hpx::threads::thread_schedule_hint();
-    auto const stack_size = hpx::threads::thread_stacksize::small_;
+    constexpr auto desc = hpx::threads::thread_description();
+    constexpr auto prio = hpx::threads::thread_priority::normal;
+    constexpr auto hint = hpx::threads::thread_schedule_hint();
+    constexpr auto stack_size = hpx::threads::thread_stacksize::small_;
     hpx::error_code ec;
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::uint64_t i = 0; i < count; ++i)
     {
         auto init = hpx::threads::thread_init_data(
@@ -364,14 +365,14 @@ void measure_function_futures_create_thread(std::uint64_t count, bool csv)
     l.wait();
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("create_thread", "latch", "none", count, duration, csv);
 }
 
 void measure_function_futures_create_thread_hierarchical_placement(
     std::uint64_t count, bool csv)
 {
-    hpx::lcos::local::latch l(count);
+    hpx::latch l(count);
 
     auto sched = hpx::threads::get_self_id_data()->get_scheduler_base();
 
@@ -379,14 +380,14 @@ void measure_function_futures_create_thread_hierarchical_placement(
         sched->get_description())
     {
         sched->add_remove_scheduler_mode(
-            hpx::threads::policies::scheduler_mode(
-                hpx::threads::policies::assign_work_thread_parent),
-            hpx::threads::policies::scheduler_mode(
-                hpx::threads::policies::enable_stealing |
-                hpx::threads::policies::enable_stealing_numa |
-                hpx::threads::policies::assign_work_round_robin |
-                hpx::threads::policies::steal_after_local |
-                hpx::threads::policies::steal_high_priority_first));
+            hpx::threads::policies::scheduler_mode::assign_work_thread_parent,
+            hpx::threads::policies::scheduler_mode::enable_stealing |
+                hpx::threads::policies::scheduler_mode::enable_stealing_numa |
+                hpx::threads::policies::scheduler_mode::
+                    assign_work_round_robin |
+                hpx::threads::policies::scheduler_mode::steal_after_local |
+                hpx::threads::policies::scheduler_mode::
+                    steal_high_priority_first);
     }
     auto const func = [&l]() {
         null_function();
@@ -394,20 +395,20 @@ void measure_function_futures_create_thread_hierarchical_placement(
     };
     auto const thread_func =
         hpx::threads::detail::thread_function_nullary<decltype(func)>{func};
-    auto const desc = hpx::util::thread_description();
+    auto const desc = hpx::threads::thread_description();
     auto prio = hpx::threads::thread_priority::normal;
-    auto const stack_size = hpx::threads::thread_stacksize::small_;
-    auto const num_threads = hpx::get_num_worker_threads();
+    auto stack_size = hpx::threads::thread_stacksize::small_;
+    auto num_threads = hpx::get_num_worker_threads();
     hpx::error_code ec;
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::size_t t = 0; t < num_threads; ++t)
     {
         auto const hint =
             hpx::threads::thread_schedule_hint(static_cast<std::int16_t>(t));
         auto spawn_func = [&thread_func, sched, hint, t, count, num_threads,
-                              desc, prio]() {
+                              stack_size, desc, prio]() {
             std::uint64_t const count_start = t * count / num_threads;
             std::uint64_t const count_end = (t + 1) * count / num_threads;
             hpx::error_code ec;
@@ -433,7 +434,7 @@ void measure_function_futures_create_thread_hierarchical_placement(
     l.wait();
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats(
         "create_thread_hierarchical", "latch", "none", count, duration, csv);
 }
@@ -441,7 +442,7 @@ void measure_function_futures_create_thread_hierarchical_placement(
 void measure_function_futures_apply_hierarchical_placement(
     std::uint64_t count, bool csv)
 {
-    hpx::lcos::local::latch l(count);
+    hpx::latch l(count);
 
     auto const func = [&l]() {
         null_function();
@@ -450,7 +451,7 @@ void measure_function_futures_apply_hierarchical_placement(
     auto const num_threads = hpx::get_num_worker_threads();
 
     // start the clock
-    high_resolution_timer walltime;
+    high_resolution_timer const walltime;
     for (std::size_t t = 0; t < num_threads; ++t)
     {
         auto const hint =
@@ -462,17 +463,17 @@ void measure_function_futures_apply_hierarchical_placement(
 
             for (std::uint64_t i = count_start; i < count_end; ++i)
             {
-                hpx::apply(exec, func);
+                hpx::post(exec, func);
             }
         };
 
         auto exec = hpx::execution::parallel_executor(hint);
-        hpx::apply(exec, spawn_func);
+        hpx::post(exec, spawn_func);
     }
     l.wait();
 
     // stop the clock
-    const double duration = walltime.elapsed();
+    double const duration = walltime.elapsed();
     print_stats("apply_hierarchical", "latch", "parallel_executor", count,
         duration, csv);
 }
@@ -489,8 +490,8 @@ int hpx_main(variables_map& vm)
         else
             numa_sensitive = 0;
 
-        bool test_all = (vm.count("test-all") > 0);
-        const int repetitions = vm["repetitions"].as<int>();
+        bool const test_all = (vm.count("test-all") > 0);
+        int const repetitions = vm["repetitions"].as<int>();
 
         if (vm.count("info"))
             info_string = vm["info"].as<std::string>();
@@ -499,16 +500,18 @@ int hpx_main(variables_map& vm)
 
         num_iterations = vm["delay-iterations"].as<std::uint64_t>();
 
-        const std::uint64_t count = vm["futures"].as<std::uint64_t>();
-        bool csv = vm.count("csv") != 0;
+        std::uint64_t const count = vm["futures"].as<std::uint64_t>();
+        bool const csv = vm.count("csv") != 0;
         if (HPX_UNLIKELY(0 == count))
             throw std::logic_error("error: count of 0 futures specified\n");
 
         hpx::execution::parallel_executor par;
-        hpx::parallel::execution::parallel_executor_aggregated par_agg;
         hpx::execution::parallel_executor par_nostack(
             hpx::threads::thread_priority::default_,
             hpx::threads::thread_stacksize::nostack);
+        hpx::execution::experimental::scheduler_executor<
+            hpx::execution::experimental::thread_pool_scheduler>
+            sched_exec_tps;
 
         for (int i = 0; i < repetitions; i++)
         {
@@ -525,7 +528,7 @@ int hpx_main(variables_map& vm)
                 measure_function_futures_wait_all(count, csv, par);
                 measure_function_futures_sliding_semaphore(count, csv, par);
                 measure_function_futures_for_loop(count, csv, par);
-                measure_function_futures_for_loop(count, csv, par_agg);
+                measure_function_futures_for_loop(count, csv, sched_exec_tps);
                 measure_function_futures_for_loop(
                     count, csv, par_nostack, "parallel_executor_nostack");
                 measure_function_futures_register_work(count, csv);
