@@ -8,38 +8,32 @@
 #if !defined(HPX_COMPUTE_DEVICE_CODE)
 #include <hpx/hpx.hpp>
 #include <hpx/hpx_init.hpp>
-#include <hpx/iostream.hpp>
 #include <hpx/include/util.hpp>
-#include <hpx/preprocessor/stringize.hpp>
+#include <hpx/iostream.hpp>
 #include <hpx/modules/format.hpp>
+#include <hpx/preprocessor/stringize.hpp>
 
-using hpx::program_options::variables_map;
 using hpx::program_options::options_description;
 using hpx::program_options::value;
+using hpx::program_options::variables_map;
 
 using hpx::find_here;
 
 using hpx::cout;
-using hpx::flush;
 
 ///////////////////////////////////////////////////////////////////////////////
-int hpx_main(
-    variables_map&
-    )
+int hpx_main(variables_map&)
 {
     {
-#       define HPX_SIZEOF(type)                                               \
-            hpx::util::format("{1:-40} {2}\n",                                \
-                HPX_PP_STRINGIZE(type), sizeof(type))                         \
-            /**/
+#define HPX_SIZEOF(type)                                                       \
+    hpx::util::format(                                                         \
+        "{1:-40} {2}\n", HPX_PP_STRINGIZE(type), sizeof(type)) /**/
 
-        cout << HPX_SIZEOF(hpx::naming::gid_type)
-             << HPX_SIZEOF(hpx::naming::id_type)
+        cout << HPX_SIZEOF(hpx::naming::gid_type) << HPX_SIZEOF(hpx::id_type)
              << HPX_SIZEOF(hpx::naming::address)
-             << HPX_SIZEOF(hpx::threads::thread_data)
-             << flush;
+             << HPX_SIZEOF(hpx::threads::thread_data) << std::flush;
 
-#       undef HPX_SIZEOF
+#undef HPX_SIZEOF
     }
 
     hpx::finalize();
@@ -47,10 +41,7 @@ int hpx_main(
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-int main(
-    int argc
-  , char* argv[]
-    )
+int main(int argc, char* argv[])
 {
     // Initialize and run HPX.
     return hpx::init(argc, argv);
