@@ -1,5 +1,5 @@
 //  Copyright (c) 2015 Thomas Heller
-//  Copyright (c) 2015 Hartmut Kaiser
+//  Copyright (c) 2015-2022 Hartmut Kaiser
 //
 //  SPDX-License-Identifier: BSL-1.0
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
@@ -17,17 +17,15 @@
 #define BOOST_NO_CXX11_ALLOCATOR
 #endif
 
+#include <hpx/algorithm.hpp>
+#include <hpx/execution.hpp>
+#include <hpx/format.hpp>
 #include <hpx/init.hpp>
-#include <hpx/local/algorithm.hpp>
-#include <hpx/local/execution.hpp>
-#include <hpx/local/thread.hpp>
 #include <hpx/modules/compute.hpp>
-#include <hpx/modules/format.hpp>
+#include <hpx/modules/compute_local.hpp>
+#include <hpx/thread.hpp>
 #include <hpx/type_support/unused.hpp>
 #include <hpx/version.hpp>
-#if defined(HPX_HAVE_MODULE_COMPUTE_CUDA)
-#include <hpx/modules/compute_cuda.hpp>
-#endif
 
 #include <cstddef>
 #include <iostream>
@@ -59,25 +57,25 @@ double mysecond()
 int checktick()
 {
     static const std::size_t M = 20;
-    int minDelta, Delta;
-    double t1, t2, timesfound[M];
+    double timesfound[M];
 
     // Collect a sequence of M unique time values from the system.
     for (std::size_t i = 0; i < M; i++)
     {
-        t1 = mysecond();
+        double const t1 = mysecond();
+        double t2;
         while (((t2 = mysecond()) - t1) < 1.0E-6)
             ;
-        timesfound[i] = t1 = t2;
+        timesfound[i] = t2;
     }
 
     // Determine the minimum difference between these M values.
     // This result will be our estimate (in microseconds) for the
     // clock granularity.
-    minDelta = 1000000;
+    int minDelta = 1000000;
     for (std::size_t i = 1; i < M; i++)
     {
-        Delta = (int) (1.0E6 * (timesfound[i] - timesfound[i - 1]));
+        int Delta = (int) (1.0E6 * (timesfound[i] - timesfound[i - 1]));
         minDelta = (std::min)(minDelta, (std::max)(Delta, 0));
     }
 
@@ -106,8 +104,6 @@ void check_results(std::size_t iterations, Vector const& a_res,
     aj = 1.0;
     bj = 2.0;
     cj = 0.0;
-    /* a[] is modified during timing check */
-    aj = 2.0E0 * aj;
     /* now execute timing loop */
     scalar = 3.0;
     for (std::size_t k = 0; k < iterations; k++)
@@ -268,7 +264,7 @@ void check_results(std::size_t iterations, Vector const& a_res,
 template <typename T>
 struct multiply_step
 {
-    multiply_step(T factor)
+    constexpr explicit multiply_step(T factor) noexcept
       : factor_(factor)
     {
     }
@@ -278,7 +274,7 @@ struct multiply_step
     //         (used in invoke()) to get the return type
 
     template <typename U>
-    HPX_HOST_DEVICE HPX_FORCEINLINE T operator()(U val) const
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr T operator()(U val) const noexcept
     {
         return val * factor_;
     }
@@ -294,7 +290,8 @@ struct add_step
     //         (used in invoke()) to get the return type
 
     template <typename U>
-    HPX_HOST_DEVICE HPX_FORCEINLINE T operator()(U val1, U val2) const
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr T operator()(
+        U val1, U val2) const noexcept
     {
         return val1 + val2;
     }
@@ -303,7 +300,7 @@ struct add_step
 template <typename T>
 struct triad_step
 {
-    triad_step(T factor)
+    constexpr explicit triad_step(T factor) noexcept
       : factor_(factor)
     {
     }
@@ -313,7 +310,8 @@ struct triad_step
     //         (used in invoke()) to get the return type
 
     template <typename U>
-    HPX_HOST_DEVICE HPX_FORCEINLINE T operator()(U val1, U val2) const
+    HPX_HOST_DEVICE HPX_FORCEINLINE constexpr T operator()(
+        U val1, U val2) const noexcept
     {
         return val1 + val2 * factor_;
     }
@@ -323,8 +321,9 @@ struct triad_step
 
 ///////////////////////////////////////////////////////////////////////////////
 template <typename Allocator, typename Policy>
-std::vector<std::vector<double>> run_benchmark(std::size_t iterations,
-    std::size_t size, Allocator&& alloc, Policy&& policy)
+std::vector<std::vector<double>> run_benchmark(std::size_t warmup_iterations,
+    std::size_t iterations, std::size_t size, Allocator&& alloc,
+    Policy&& policy)
 {
     // Allocate our data
     using vector_type = hpx::compute::vector<STREAM_TYPE, Allocator>;
@@ -387,10 +386,35 @@ std::vector<std::vector<double>> run_benchmark(std::size_t iterations,
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    // Main Loop
-    std::vector<std::vector<double>> timing(4, std::vector<double>(iterations));
-
+    // Warmup loop
     double scalar = 3.0;
+    for (std::size_t iteration = 0; iteration != warmup_iterations; ++iteration)
+    {
+        // Copy
+        hpx::copy(policy, a.begin(), a.end(), c.begin());
+
+        // Scale
+        hpx::transform(policy, c.begin(), c.end(), b.begin(),
+            multiply_step<STREAM_TYPE>(scalar));
+
+        // Add
+        hpx::ranges::transform(policy, a.begin(), a.end(), b.begin(), b.end(),
+            c.begin(), add_step<STREAM_TYPE>());
+
+        // Triad
+        hpx::ranges::transform(policy, b.begin(), b.end(), c.begin(), c.end(),
+            a.begin(), triad_step<STREAM_TYPE>(scalar));
+    }
+
+    ///////////////////////////////////////////////////////////////////////////
+    // Reinitialize arrays (if needed)
+    hpx::fill(policy, a.begin(), a.end(), 1.0);
+    hpx::fill(policy, b.begin(), b.end(), 2.0);
+    hpx::fill(policy, c.begin(), c.end(), 0.0);
+
+    ///////////////////////////////////////////////////////////////////////////
+    // Main timing loop
+    std::vector<std::vector<double>> timing(4, std::vector<double>(iterations));
     for (std::size_t iteration = 0; iteration != iterations; ++iteration)
     {
         // Copy
@@ -436,6 +460,7 @@ int hpx_main(hpx::program_options::variables_map& vm)
     std::size_t vector_size = vm["vector_size"].as<std::size_t>();
     std::size_t offset = vm["offset"].as<std::size_t>();
     std::size_t iterations = vm["iterations"].as<std::size_t>();
+    std::size_t warmup_iterations = vm["warmup_iterations"].as<std::size_t>();
     std::size_t chunk_size = vm["chunk_size"].as<std::size_t>();
     std::size_t executor = vm["executor"].as<std::size_t>();
     csv = vm.count("csv") > 0;
@@ -445,6 +470,18 @@ int hpx_main(hpx::program_options::variables_map& vm)
 
     std::string chunker = vm["chunker"].as<std::string>();
 
+    if (vector_size < 1)
+    {
+        HPX_THROW_EXCEPTION(hpx::error::commandline_option_error, "hpx_main",
+            "Invalid vector size, must be at least 1");
+    }
+
+    if (iterations < 1)
+    {
+        HPX_THROW_EXCEPTION(hpx::error::commandline_option_error, "hpx_main",
+            "Invalid number of iterations given, must be at least 1");
+    }
+
     // clang-format off
     if (!csv)
     {
@@ -479,38 +516,11 @@ int hpx_main(hpx::program_options::variables_map& vm)
     double time_total = mysecond();
     std::vector<std::vector<double>> timing;
 
-#if defined(HPX_HAVE_MODULE_COMPUTE_CUDA)
-    bool use_accel = false;
-    if (vm.count("use-accelerator"))
-        use_accel = true;
-
-    if (use_accel)
-    {
-        using executor_type = hpx::cuda::experimental::concurrent_executor<>;
-        using allocator_type = hpx::cuda::experimental::allocator<STREAM_TYPE>;
-
-        // Get the cuda targets we want to run on
-        hpx::cuda::experimental::target target;
-
-        // Get the host targets we want to run on
-        auto host_targets = hpx::compute::host::get_local_targets();
-
-        allocator_type alloc(target);
-        executor_type exec(target, host_targets);
-        auto policy = hpx::execution::par.on(exec);
-
-        // perform benchmark
-        timing = run_benchmark<>(
-            iterations, vector_size, std::move(alloc), std::move(policy));
-        //iterations, vector_size, std::move(target));
-    }
-    else
-#endif
     {
         if (executor == 0)
         {
             // Default parallel policy with serial allocator.
-            timing = run_benchmark<>(iterations, vector_size,
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
                 std::allocator<STREAM_TYPE>{}, hpx::execution::par);
         }
         else if (executor == 1)
@@ -525,8 +535,8 @@ int hpx_main(hpx::program_options::variables_map& vm)
             executor_type exec(numa_nodes);
             auto policy = hpx::execution::par.on(exec);
 
-            timing = run_benchmark<>(
-                iterations, vector_size, std::move(alloc), std::move(policy));
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
+                std::move(alloc), std::move(policy));
         }
         else if (executor == 2)
         {
@@ -536,8 +546,8 @@ int hpx_main(hpx::program_options::variables_map& vm)
                 decltype(policy)>
                 alloc(policy);
 
-            timing = run_benchmark<>(
-                iterations, vector_size, std::move(alloc), std::move(policy));
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
+                std::move(alloc), std::move(policy));
         }
         else if (executor == 3)
         {
@@ -551,13 +561,46 @@ int hpx_main(hpx::program_options::variables_map& vm)
                 decltype(policy)>
                 alloc(policy);
 
-            timing = run_benchmark<>(
-                iterations, vector_size, std::move(alloc), std::move(policy));
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
+                std::move(alloc), std::move(policy));
+        }
+        else if (executor == 4)
+        {
+            // thread_pool_scheduler used through a scheduler_executor.
+            using executor_type =
+                hpx::execution::experimental::scheduler_executor<
+                    hpx::execution::experimental::thread_pool_scheduler>;
+
+            executor_type exec;
+            auto policy = hpx::execution::par.on(exec);
+            hpx::compute::host::detail::policy_allocator<STREAM_TYPE,
+                decltype(policy)>
+                alloc(policy);
+
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
+                std::move(alloc), std::move(policy));
+        }
+        else if (executor == 5)
+        {
+            // Block-fork-join executor and allocator with block-fork-join
+            // executor.
+            using executor_type =
+                hpx::execution::experimental::block_fork_join_executor;
+
+            executor_type exec;
+
+            auto policy = hpx::execution::par.on(exec);
+            hpx::compute::host::detail::policy_allocator<STREAM_TYPE,
+                decltype(policy)>
+                alloc(policy);
+
+            timing = run_benchmark<>(warmup_iterations, iterations, vector_size,
+                std::move(alloc), std::move(policy));
         }
         else
         {
-            HPX_THROW_EXCEPTION(hpx::commandline_option_error, "hpx_main",
-                "Invalid executor id given (0-3 allowed");
+            HPX_THROW_EXCEPTION(hpx::error::commandline_option_error,
+                "hpx_main", "Invalid executor id given (0-4 allowed");
         }
     }
     time_total = mysecond() - time_total;
@@ -579,11 +622,18 @@ int hpx_main(hpx::program_options::variables_map& vm)
         3 * sizeof(STREAM_TYPE) * static_cast<double>(vector_size),
         3 * sizeof(STREAM_TYPE) * static_cast<double>(vector_size)};
 
-    // Note: skip first iteration
     std::vector<double> avgtime(num_stream_tests, 0.0);
     std::vector<double> mintime(
         num_stream_tests, (std::numeric_limits<double>::max)());
     std::vector<double> maxtime(num_stream_tests, 0.0);
+
+    for (std::size_t j = 0; j < num_stream_tests; j++)
+    {
+        avgtime[j] = timing[j][0];
+        mintime[j] = timing[j][0];
+        maxtime[j] = timing[j][0];
+    }
+
     for (std::size_t iteration = 1; iteration != iterations; ++iteration)
     {
         for (std::size_t j = 0; j < num_stream_tests; j++)
@@ -604,9 +654,10 @@ int hpx_main(hpx::program_options::variables_map& vm)
                 "max,add_bytes,add_bw,add_avg,add_min,add_max,triad_bytes,"
                 "triad_bw,triad_avg,triad_min,triad_max\n");
         }
-        std::size_t const num_executors = 5;
+        std::size_t const num_executors = 6;
         const char* executors[num_executors] = {"parallel-serial", "block",
-            "parallel-parallel", "fork_join_executor"};
+            "parallel-parallel", "fork_join_executor", "scheduler_executor",
+            "block_fork_join_executor"};
         hpx::util::format_to(std::cout, "{},{},{},", executors[executor],
             hpx::get_os_thread_count(), vector_size);
     }
@@ -618,7 +669,7 @@ int hpx_main(hpx::program_options::variables_map& vm)
 
     for (std::size_t j = 0; j < num_stream_tests; j++)
     {
-        avgtime[j] = avgtime[j] / (double) (iterations - 1);
+        avgtime[j] = avgtime[j] / (double) (iterations);
 
         if (csv)
         {
@@ -670,6 +721,9 @@ int main(int argc, char* argv[])
         (   "iterations",
             hpx::program_options::value<std::size_t>()->default_value(10),
             "number of iterations to repeat each test. (default: 10)")
+        (   "warmup_iterations",
+            hpx::program_options::value<std::size_t>()->default_value(1),
+            "number of warmup iterations to perform before timing. (default: 1)")
         (   "chunker",
             hpx::program_options::value<std::string>()->default_value("default"),
             "Which chunker to use for the parallel algorithms. "
@@ -679,12 +733,7 @@ int main(int argc, char* argv[])
             "size of vector (default: 1024)")
         (   "executor",
             hpx::program_options::value<std::size_t>()->default_value(2),
-            "executor to use (0-3) (default: 2, parallel_executor)")
-
-#if defined(HPX_HAVE_COMPUTE)
-        (   "use-accelerator",
-            "Use this flag to run the stream benchmark on the GPU")
-#endif
+            "executor to use (0-5) (default: 2, parallel_executor)")
         ;
     // clang-format on
 
