@@ -5,8 +5,8 @@
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <hpx/hpx_main.hpp>
-#include <hpx/local/thread.hpp>
 #include <hpx/modules/testing.hpp>
+#include <hpx/thread.hpp>
 
 void stackless_thread()
 {
